#ifndef _FlowField_h
#define _FlowField_h

#include "Image.h"
#include "IntegerImage.h"
#include "MotionPara.h"
#include <QFile>

class FlowField
{
private:
	IntegerImage m_Vx,m_Vy;					// x and y components of the flow field
	OpticalFlowPara m_OpticalFlowPara; // this one contains the information of how the optical flow field is computed
	AlgorithmPara m_AlgorithmPara;	
	int m_Top,m_Left,m_Right,m_Bottom; // the bounding box information
public:
	FlowField();
	FlowField(const FlowField& other);
	~FlowField();
	void copyData(const FlowField& other);
	FlowField& operator=(const FlowField& other);
	
	void fromDoublePrecision(const DImage& vx,const DImage& vy);
	void toDoublePrecision(DImage& vx,DImage& vy) const;
	void toFloatPrecision(FImage& vx,FImage& vy) const;

	void getFlow(int x, int y, float *vx, float *vy);

	// functions to access the member variables
	OpticalFlowPara& opticalFlowPara(){return m_OpticalFlowPara;};
	AlgorithmPara& algorithmPara(){return m_AlgorithmPara;};
	const OpticalFlowPara& opticalFlowPara()const {return m_OpticalFlowPara;};
	const AlgorithmPara& algorithmPara()const {return m_AlgorithmPara;};
	int top() const{return m_Top;};
	int left() const{return m_Left;};
	int bottom() const{return m_Bottom;};
	int right() const{return m_Right;};
	int& top() {return m_Top;};
	int& left() {return m_Left;};
	int& bottom() {return m_Bottom;};
	int& right() {return m_Right;};

	// functions for file IO
	bool writeFlowField(QFile& file) const;
	bool readFlowField(QFile& file);
};
#endif