#include "FlowInterpolation.h"
#include "ImageIO.h"

FlowInterpolation::FlowInterpolation(void)
{
	level=1;
}

FlowInterpolation::~FlowInterpolation(void)
{
}

void FlowInterpolation::Initialize(int width, int height)
{
	//printf("width %d, height %d\n",interpFlow.width(),interpFlow.height());
	if(!interpFlow.matchDimension(width,height,2))
	{
		interpFlow.allocate(width,height,2);
		//printf("width %d, height %d\n",interpFlow.width(),interpFlow.height());
	}
	interpFlow.reset();
	imWidth=width;
	imHeight=height;
}

void FlowInterpolation::Initialize(const DImage &flow)
{
	interpFlow=flow;
	imWidth=interpFlow.width();
	imHeight=interpFlow.height();
}

void FlowInterpolation::Interpolate(const QList<Vector> XList,const QList<Matrix> &InvCovList, const QList<Vector> &MeanList,double lambda,int nIterations)
{
	if(interpFlow.IsEmpty())
	{
		//printf("width %d, height %d\n",interpFlow.width(),interpFlow.height());
		return;
	}
	xList.clear();
	invCovList.clear();
	meanList.clear();

	for(int i=0;i<XList.size();i++)
		if(XList[i][0]>= 0 && XList[i][0]<interpFlow.width() && XList[i][1]>=0 && XList[i][1]<interpFlow.height())
		{
			xList.append(XList[i]);
			invCovList.append(InvCovList[i]);
			meanList.append(MeanList[i]);
		}
	//cout<<"Go multigrid"<<endl;
	// go multigrid
	if(level>0 && imWidth>6 && imHeight>6)
	{
		FlowInterpolation coarserGrid;
		coarserGrid.Initialize(imWidth/2,imHeight/2);
		coarserGrid.setLevel(level-1);
		// generate the correspondence for the coarser level
		QList<Vector> xListC(xList);
		QList<Matrix> invCovListC(invCovList);
		QList<Vector> meanListC(meanList);
		for(int i=0;i<xList.size();i++)
		{
			xListC[i][0]=(int)xList[i][0]/2;
			xListC[i][1]=(int)xList[i][1]/2;
			invCovListC[i]*=4;
			meanListC[i]/=2;
		}
		coarserGrid.Interpolate(xListC,invCovListC,meanListC,lambda/2,max(coarserGrid.imWidth,60));
		coarserGrid.interpFlow.imresize(interpFlow,imWidth,imHeight);
		interpFlow.Multiplywith(2);
	}
	// IRLS
	weightList=QList<double>::fromVector(QVector<double>(xList.size()));
	for(int count=0;count<3;count++)
	{
		// compute weight;
		for(int i=0;i<xList.size();i++)
		{
			int offset=(xList[i][1]*imWidth+xList[i][0])*2;
			Vector vect(2),result;
			vect[0]=interpFlow.data()[offset];
			vect[1]=interpFlow.data()[offset+1];
			vect-=meanList[i];
			result=invCovList[i]*vect;
			weightList[i]=0.5/sqrt(innerproduct(vect,result)+0.00001);			
		}

		// compute b
		DImage b(imWidth,imHeight,2);
		b.reset();
		for(int i=0;i<xList.size();i++)
		{
			int offset=(xList[i][1]*imWidth+xList[i][0])*2;
			Vector result;
			result=invCovList[i]*meanList[i];
			b.data()[offset]+=result[0]*lambda*weightList[i];
			b.data()[offset+1]+=result[1]*lambda*weightList[i];
		}

		DImage r(imWidth,imHeight,2),p(imWidth,imHeight,2),q(imWidth,imHeight,2);
		DImage foo;
		LinearSystem(foo,interpFlow,lambda);
		r.Subtract(b,foo);

		// start conjugate gradient algorithm
		Vector rou(nIterations);
		for(int k=0;k<nIterations;k++)
		{
			rou[k]=r.norm2();
			//cout<<rou[k]<<endl;

			if(rou[k]<1E-20)
				break;
			if(k==0)
				p.copyData(r);
			else
			{
				double ratio=rou[k]/rou[k-1];
				p.Add(r,p,ratio);
			}
			LinearSystem(q,p,lambda);
			double alpha=rou[k]/p.innerproduct(q);
			interpFlow.Add(p,alpha);
			r.Add(q,-alpha);
		}
	}
}

void FlowInterpolation::LinearSystem(DImage &output, const DImage &input,double lambda)
{
	input.laplacian(foobar1);
	foobar1.laplacian(output);
	//input.laplacian(output);
	for(int i=0;i<xList.size();i++)
	{
		int offset=(xList[i][1]*imWidth+xList[i][0])*2;
		Vector vect(2),result;
		vect[0]=input.data()[offset];
		vect[1]=input.data()[offset+1];
		result=invCovList[i]*vect;
		output.data()[offset]+=result[0]*lambda*weightList[i];
		output.data()[offset+1]+=result[1]*lambda*weightList[i];
	}
}

void FlowInterpolation::dispFlow()
{
	DImage vx(imWidth,imHeight),vy(imWidth,imHeight);
	for(int i=0;i<vx.npixels();i++)
	{
		vx.data()[i]=interpFlow.data()[i*2];
		vy.data()[i]=interpFlow.data()[i*2+1];
	}
	vx.imwrite("vx.bmp",ImageIO::normalized);
	vy.imwrite("vy.bmp",ImageIO::normalized);
}

void FlowInterpolation::dispAccuracy()
{
	printf("%d points\n",xList.size());
	for(int i=0;i<xList.size();i++)
	{
		printf("(%d,%d)-->",(int)xList[i][0],(int)xList[i][1]);
		int offset=xList[i][1]*imWidth+xList[i][0];
		printf("(%f,%f)  ",interpFlow.data()[offset*2],interpFlow.data()[offset*2+1]);
		printf(" (%f,%f)\n",meanList[i][0],meanList[i][1]);
	}
}