#include "ImageMatching.h"
#include "Stochastic.h"
#include "math.h"

int ImageMatching::_MRatio=6;
const MaskPyramid* ImageMatching::pMaskPyramid=NULL;
bool ImageMatching::IsMaskSpecified=false;

ImageMatching::ImageMatching(void)
{
}

ImageMatching::~ImageMatching(void)
{
}

void ImageMatching::MatchCoarse2Fine(int x1,int y1,double& x2,double& y2,const ImPyramid& ImPyramid1,const ImPyramid& ImPyramid2,int searchsize,int winsize,double sigma)
{
	SortQueue<Point> sortqueue(1);
	MatchCoarse2Fine(x1,y1,x2,y2,sortqueue,ImPyramid1,ImPyramid2,searchsize,winsize,sigma);
	x2=sortqueue[0].x();
	y2=sortqueue[0].y();

	//cout<<"Original :"<<x1<<" "<<y1<<endl;
	//for(int i=0;i<sortqueue.num();i++)
	//	cout<<sortqueue.score(i)<<" ("<<sortqueue[i].x()<<" "<<sortqueue[i].y()<<")"<<endl;
	//cout<<endl;

	//BinaryImage searchArea(searchsize*2+1,searchsize*2+1);
	//searchArea.setAllZeros();
	//for(int i=0;i<sortqueue.num();i++)
	//{
	//	int x=sortqueue[i].x()-x1+searchsize;
	//	int y=sortqueue[i].y()-y1+searchsize;
	//	searchArea.setPixel(x,y);
	//}
	//searchArea.imwrite("temp.bmp");
}

void ImageMatching::MatchCoarse2Fine(int x1,int y1,int x2,int y2,SortQueue<Point>& sortqueue,const ImPyramid& ImPyramid1,const ImPyramid& ImPyramid2,int searchsize,int winsize,double sigma)
{
	if(sigma==-1)
		sigma=(double)winsize/2.5;
	SortQueue<Point> sort;

        int nLevels=__min((double) ImPyramid1.nlevels()-1,log((double)winsize)/log((double)2)-2);
	nLevels=__max(nLevels,2);
	if(nLevels<0)
		nLevels=0;

	for(int k=nLevels;k>=0;k--)
	{
		int rate=1;
		for(int j=0;j<k;j++)
			rate*=2;
		int winSize=winsize/rate; 
		int searchSize=searchsize/rate;
		double Sigma=sigma/rate;
		int X1=x1/rate;
		int Y1=y1/rate;
		int X2=x2/rate;
		int Y2=y2/rate;

		BinaryImage searchArea(searchSize*2+1,searchSize*2+1);

		if(k==nLevels) // top level
		{
			searchArea.setAllOnes();
			sort.allocate(searchSize*_MRatio+1);
		}
		else
		{
			searchArea.setAllZeros();
			for(int i=0;i<sort.num();i++)
			{
				int x=sort[i].x()*2-X2+searchSize;
				int y=sort[i].y()*2-Y2+searchSize;
				searchArea.setPixel(x,y);
			}
			searchArea.imdilate8();
			//searchArea.imwrite("temp.bmp");
			sort.allocate(searchSize*2+1);
		}
		if(k>0)
			MatchPoint(X1,Y1,X2,Y2,sort,searchArea,ImPyramid1.Image(k),ImPyramid2.Image(k),searchSize,winSize,Sigma,k);
		else
			MatchPoint(X1,Y1,X2,Y2,sortqueue,searchArea,ImPyramid1.Image(k),ImPyramid2.Image(k),searchSize,winSize,Sigma,k);
	}
}

void ImageMatching::LucasKanade(int x1, int y1, double &x2, double &y2, const ImPyramid &ImPyramid1, const ImPyramid &ImPyramid2, int winSize, double sigma)
{
	ImageType im1,im2,imdx,imdy,imdt;
	ImPyramid1.Image(0).getPatch(im1,x1,y1,winSize);
	double* pGaussian=new double[(winSize*2+1)*(winSize*2+1)];
	if(sigma==-1)
		sigma=winSize/2;
	CStochastic::Generate2DGaussian(pGaussian,winSize,sigma);
	// modulate the Gaussian with a mask
	if(IsMaskSpecified && pMaskPyramid!=NULL)
	{
		const DImage& mask=pMaskPyramid->mask(0);
		double nMaskPixels=0;
		for(int i=-winSize;i<=winSize;i++)
			for(int j=-winSize;j<=winSize;j++)
			{
				int x,y;
				x=x1+j;
				y=y1+i;
				if(x<0 || y<0 || x>=mask.width() || y>=mask.height())
					continue;
				nMaskPixels+=mask.data()[y*mask.width()+x];
			}
		if(nMaskPixels>winSize/2) // there has to be certain number of mask pixels
		{
			for(int i=-winSize;i<=winSize;i++)
				for(int j=-winSize;j<=winSize;j++)
				{
					int x,y;
					x=x1+j;
					y=y1+i;
					if(x<0 || y<0 || x>=mask.width() || y>=mask.height())
						continue;
					pGaussian[(i+winSize)*(2*winSize+1)+j+winSize]*=mask.data()[y*mask.width()+x];
				}
			DImage temp(winSize*2+1,winSize*2+1);
			memcpy(temp.data(),pGaussian,sizeof(double)*(winSize*2+1)*(winSize*2+1));
			temp.imwrite("mask_lk.bmp",ImageIO::normalized);
		}
	}


	int nChannels=ImPyramid1.Image(0).nchannels();
	double du,dv,det;
	for(int k=0;k<4;k++)
	{
		ImPyramid2.Image(0).getPatch(im2,x2,y2,winSize);
		ImPyramid2.dX(0).getPatch(imdx,x2,y2,winSize);
		ImPyramid2.dY(0).getPatch(imdy,x2,y2,winSize);
		imdt.Subtract(im2,im1);
		
		double dXX,dYY,dXY,dXT,dYT,totalWeight,weight,dx,dy,dt;
		dXX=dYY=dXY=dXT=dYT=totalWeight=0;

		for(int i=-winSize;i<=winSize;i++)
			for(int j=-winSize;j<=winSize;j++)
			{
				if(!ImPyramid1.Image(0).IsInImage(x1+j,y1+i))
					continue;
				if(!ImPyramid2.Image(0).IsInImage(x2+j,y2+i))
					continue;
				int offset=(i+winSize)*(2*winSize+1)+j+winSize;
				weight=pGaussian[offset];
				offset*=nChannels;
				for(int k=0;k<nChannels;k++)
				{
					dx=imdx.data()[offset+k];
					dy=imdy.data()[offset+k];
					dt=imdt.data()[offset+k];
					dXX+=dx*dx*weight;
					dYY+=dy*dy*weight;
					dXY+=dx*dy*weight;
					dXT+=dx*dt*weight;
					dYT+=dy*dt*weight;
				}
			}
		// solve du, dv
		//cout<<dXX<<" "<<dXY<<" "<<dYY<<" "<<dXT<<" "<<dYT<<endl;
		det=dXX*dYY-dXY*dXY;
		du=-(dYY*dXT-dXY*dYT)/det;
		dv=-(-dXY*dXT+dXX*dYT)/det;
		//cout<<du<<" "<<dv<<endl;

		x2+=du;
		y2+=dv;
		
	}
	delete pGaussian;
}
