
#include <QtGui>
#include "ImageViewer.h"
#include "math.h"
#include "ImageProcessing.h"
#include <iostream>

using namespace std;

ImageViewer::ImageViewer(QWidget* parent): QWidget(parent)
{
	IsDualImage=false;
	showingImage=true;

	// set the default values
	minScale=0.3;
	maxScale=6;
	ImHeight=480;
	ImWidth=640;
	HMargin=VMargin=40;

	min_Vx=-5;max_Vx=5;
	min_Vy=-3;max_Vy=3;

	QImage temp(ImWidth,ImHeight,QImage::Format_RGB32);

	QPainter painter(&temp);
	QBrush brush(QColor(0,0,0));
	painter.setBrush(brush);
	painter.fillRect(0,0,ImWidth,ImHeight,brush);

	m_Image=temp;
	dispImage=m_Image;
	scaleFactor=1;
	setBackgroundRole(QPalette::Dark);

	if (parent!=0)
		parentViewerSize=parent->frameSize()-QSize(10,10);
	else
		parentViewerSize=QSize(ViewerWidth,ViewerHeight);

	computeViewerSize();	
	setMouseTracking(true);

	displayType=Animation;
	timer.start(800,this);
	steps=0;

	_imageTransparency=1;
	

}

void ImageViewer::reset()
{
	ImHeight=480;
	ImWidth=640;
	QImage temp(ImWidth,ImHeight,QImage::Format_ARGB32);

	QPainter painter(&temp);
	QBrush brush(QColor(0,0,0));
	painter.setBrush(brush);
	painter.fillRect(0,0,ImWidth,ImHeight,brush);

	m_Image=temp;
	resizeImage(0);
	IsDualImage=false;
	update();
}

ImageViewer::~ImageViewer(void)
{
}



void ImageViewer::setParentSize(QSize size)
{
	parentViewerSize=size-QSize(10,10);
	computeViewerSize();
}

void ImageViewer::computeViewerSize()
{
	ViewerHeight=ImHeight+2+VMargin*scaleFactor*2;
	ViewerWidth=ImWidth+2+HMargin*scaleFactor*2;

	xoffset=yoffset=0;
	if (parentViewerSize.height()>ViewerHeight)
		yoffset=(parentViewerSize.height()-ViewerHeight)/2;
	if (parentViewerSize.width()>ViewerWidth)
		xoffset=(parentViewerSize.width()-ViewerWidth)/2;
	HScreenOffset=HMargin*scaleFactor+xoffset+1;
	VScreenOffset=VMargin*scaleFactor+yoffset+1;
	setFixedSize(qMax(parentViewerSize.width(),ViewerWidth),qMax(parentViewerSize.height(),ViewerHeight));
	//if(pVideoRotoscoping!=NULL)
	//	pVideoRotoscoping->setDispInfo(QPointF(HScreenOffset,VScreenOffset),scaleFactor);
}


void ImageViewer::paintEvent(QPaintEvent *event)
{
        //printf("paint event\n");
	QPainter mPainter(this);
	mPainter.setPen(QColor(0,0,0));
	mPainter.setRenderHint(QPainter::Antialiasing);
	mPainter.drawRect(HScreenOffset-1,VScreenOffset-1,ImWidth+1,ImHeight+1);
	
	if (_imageTransparency != 1)
		mPainter.setOpacity(_imageTransparency);

	//if (m_SaveSegmentation)
	QImage saveimg(parentViewerSize.width(),parentViewerSize.height(),QImage::Format_ARGB32_Premultiplied);
	//QPainter rotoPainter(&saveimg);

	if (showingImage)
	{

		if(IsDualImage==false)
			mPainter.drawImage(QPoint(HScreenOffset,VScreenOffset),dispImage);
		else
		{
			//cout<<"redrawed!"<<endl;
			switch(displayType){
				case Difference:
					mPainter.drawImage(QPoint(HScreenOffset,VScreenOffset),dispDiffImage);
					break;
				case Animation:
					if(steps%2==0)
						mPainter.drawImage(QPoint(HScreenOffset,VScreenOffset),dispImage);
					else
						mPainter.drawImage(QPoint(HScreenOffset,VScreenOffset),dispWarpImage2);
					break;
				case DisplayVx:
					mPainter.drawImage(QPoint(HScreenOffset,VScreenOffset),dispVx);
					break;
				case DisplayVy:
					mPainter.drawImage(QPoint(HScreenOffset,VScreenOffset),dispVy);
					break;
			}
		}
	}

	//printf ("is %i\n", mPainter.paintEngine()->hasFeature(QPaintEngine::ConstantOpacity));
	if (_imageTransparency != 1)
		mPainter.setOpacity(1);

	//emit paintStrokes(*this,mPainter);
	emit paintingImageViewer(*this,mPainter,saveimg);
	//mPainter.end();

	emit finishedPainting();

	

}





void ImageViewer::timerEvent(QTimerEvent *event)
 {
     if (event->timerId() == timer.timerId()) {
         ++steps;
		 if(displayType==Animation && IsDualImage==true)
			update();
     } else {
         QWidget::timerEvent(event);
	 }
} 



//----------------------------------------------------------------------------------------------------
// function to compute warped image based on the flow field m_Vx and m_Vy
//----------------------------------------------------------------------------------------------------
void ImageViewer::computeWarpImage()
{
	// take care of the memory
	if(m_WarpImage2.width()!=m_Image2.width() || m_WarpImage2.height()!=m_Image2.height())
		m_WarpImage2=m_Image2;

	// get the scanlines of image 2
	const unsigned char **pScanLines;
	pScanLines=new const unsigned char*[m_Image2.height()];
	for(int i=0;i<m_Image2.height();i++)
		pScanLines[i]=m_Image2.scanLine(i);

	int xx,yy,u,v;
	double x,y;
	double dx,dy,s;
	int result[3];
	int width=m_Image2.width();
	int height=m_Image2.height();
	for(int i=0;i<height;i++)
	{
		unsigned char* scanline=m_WarpImage2.scanLine(i);
		memset(scanline,255,width*4);
		for(int j=0;j<width;j++)
		{
			x=j+Vx.data()[i*width+j];
			y=i+Vy.data()[i*width+j];
			xx=x;
			yy=y;
			if(xx<0 || yy<0 || xx>width-1 || yy>height-1) // if the flow is outside of the image boundary
			{
				for(int l=0;l<3;l++)
					scanline[j*4+l]=0;
				continue;
			}
                        dx=__max((double)__min(x-xx,1.0),0.0);
                        dy=__max((double)__min(y-yy,1.0),0.0);
			for(int l=0;l<3;l++)
				result[l]=0;
			for(int m=0;m<=1;m++)
				for(int n=0;n<=1;n++)
				{
					u=ImageProcessing::EnforceRange(xx+m,width);
					v=ImageProcessing::EnforceRange(yy+n,height);
					s=fabs(1-m-dx)*fabs(1-n-dy);
					for(int l=0;l<3;l++)
						result[l]+=pScanLines[v][u*4+l]*s;
				}
			for(int l=0;l<3;l++)
				scanline[j*4+l]=__min(__max(result[l],0),255);
		}
	}
	delete pScanLines;
}

//----------------------------------------------------------------------------------------------------
// function to compute the difference between image1 and warped image2
//----------------------------------------------------------------------------------------------------
void ImageViewer::computeDiffImage()
{
	if(IsDualImage==false)
		return;
	if(m_DiffImage.width()!=m_Image.width() || m_DiffImage.height()!=m_Image.height())
		m_DiffImage=m_Image;
	for(int i=0;i<m_DiffImage.height();i++)
	{
		const unsigned char* linebuffer1=m_Image.scanLine(i);
		const unsigned char* linebuffer2=m_WarpImage2.scanLine(i);
		unsigned char* linebuffer=m_DiffImage.scanLine(i);
		for(int j=0;j<m_DiffImage.width();j++)
			for(int k=0;k<3;k++)
			{
				int temp=linebuffer1[j*4+k]-linebuffer2[j*4+k];
				linebuffer[j*4+k]=__max(__min(temp+128,255),0);
			}
	}
}

//----------------------------------------------------------------------------------------------------
// function to generate the vx and vy images for display
//----------------------------------------------------------------------------------------------------
void ImageViewer::computeFlow()
{
	if(m_Vx.width()!=Vx.width() || m_Vx.height()!=Vx.height())
	{
		QImage temp(Vx.width(),Vx.height(),QImage::Format_RGB32);
		m_Vx=temp;
	}
	if(m_Vy.width()!=Vy.width() || m_Vy.height()!=Vy.height())
	{
		QImage temp(Vy.width(),Vy.height(),QImage::Format_RGB32);
		m_Vy=temp;
	}


	int width=Vx.width();
	int height=Vx.height();

	if ((Vx.width()==0)||(Vx.height()==0)||(Vy.width()==0)||(Vy.height()==0))
		return;

	double Min_Vx=min(min_Vx,Vx.minimum());
	double Max_Vx=max(max_Vx,Vx.maximum());
	double Min_Vy=min(min_Vy,Vy.minimum());
	double Max_Vy=max(max_Vy,Vy.maximum());
	for(int i=0;i<height;i++)
	{
		unsigned char* linebufferVx=m_Vx.scanLine(i);
		unsigned char* linebufferVy=m_Vy.scanLine(i);

		for(int j=0;j<width;j++)
		{
			int value=(Vx.data()[i*width+j]-Min_Vx)/(Max_Vx-Min_Vx)*255;
			value=__min(__max(value,0),255);
			for(int k=0;k<3;k++)
				linebufferVx[j*4+k]=value;

			value=(Vy.data()[i*width+j]-Min_Vy)/(Max_Vy-Min_Vy)*255;
			value=__min(__max(value,0),255);
			for(int k=0;k<3;k++)
				linebufferVy[j*4+k]=value;
		}
	}
}

void ImageViewer::loadImage(const QImage &inputImage)
{
	m_Image=inputImage;
	IsDualImage=false;
	resizeImage(-1);
}

void ImageViewer::loadImage2(const QImage& image1,const QImage& image2)
{
	m_Image=image1;
	m_Image2=image2;
	// check whether the dimensions match
	if(m_Image.width()!=m_Image2.width() || m_Image.height()!=m_Image2.height())
	{
		IsDualImage=false;
		resizeImage(-1);
		return;
	}
	m_WarpImage2=image2;
	if(!Vx.matchDimension(image1.width(),image1.height(),1))
		Vx.allocate(image1.width(),image1.height());
	if(!Vy.matchDimension(image1.width(),image1.height(),1))
		Vy.allocate(image1.width(),image1.height());
	IsDualImage=true;
	// there is no need to compute 
	computeDiffImage();
	computeFlow();
	resizeImage(-1);
}

void ImageViewer::loadImage2(const QImage& image1,const QImage& image2,const DImage& vx,const DImage& vy)
{
	m_Image=image1;
	m_Image2=image2;
	Vx=vx;
	Vy=vy;
	// check whether the dimensions match
	if(m_Image.width()!=m_Image2.width() || m_Image.height()!=m_Image2.height() || m_Image.width()!=Vx.width() || m_Image.height()!=Vx.height() ||
		m_Image.width()!=Vy.width() || m_Image.height()!=Vy.height())
	{
		IsDualImage=false;
		resizeImage(-1);
		return;
	}
	IsDualImage=true;
	computeWarpImage();
	computeDiffImage();
	computeFlow();
	resizeImage(-1);
}

void ImageViewer::resizeImage(double factor)
{
	if (factor==0)
		scaleFactor=1;
	else if (factor!=-1)
		scaleFactor*=factor;

	ImWidth=m_Image.width()*scaleFactor;
	ImHeight=m_Image.height()*scaleFactor;
	if (fabs(scaleFactor-1)<0.01)
	{
		scaleFactor=1;
		dispImage=m_Image;
		dispImage2=m_Image2;
		dispWarpImage2=m_WarpImage2;
		dispDiffImage=m_DiffImage;
		dispVx=m_Vx;
		dispVy=m_Vy;
	}
	else
	{
		dispImage=m_Image.scaled(ImWidth,ImHeight, Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
	
		if(IsDualImage)
		{
			//dispImage2=m_Image2.scaled(ImWidth,ImHeight, Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
			dispWarpImage2=m_WarpImage2.scaled(ImWidth,ImHeight, Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
			dispDiffImage=m_DiffImage.scaled(ImWidth,ImHeight, Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
			dispVx=m_Vx.scaled(ImWidth,ImHeight, Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
			dispVy=m_Vy.scaled(ImWidth,ImHeight, Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
		}
	}
	computeViewerSize();
	update();

	if (scaleFactor<=minScale)
		emit setZoomOut(false);
	else
		emit setZoomOut(true);
	if (scaleFactor>=maxScale)
		emit setZoomIn(false);
	else
		emit setZoomIn(true);
	emit imageResized(scaleFactor);
}

void ImageViewer::zoomOut()
{
	resizeImage(0.9);
	emit adjustScrollBar(0.9);
}

void ImageViewer::zoomIn()
{
	resizeImage(1.2);
	emit adjustScrollBar(1.2);
}

void ImageViewer::normalSize()
{
	double temp=scaleFactor;
	scaleFactor=1;
	resizeImage(0);
	emit adjustScrollBar((double)1/temp);
}

void ImageViewer::mousePressEvent ( QMouseEvent * event )
{
	QPoint pos=event->pos();
	MouseAction mouseAction;
        Qt::KeyboardModifiers keymod =event->modifiers();

	if(event->button()==Qt::LeftButton)
	{
		mouseAction=LeftClick;
                emit viewerMouseEvent(*this,mouseAction,pos,1.0,keymod);
	}
	else if(event->button()==Qt::RightButton)
	{
		mouseAction=RightClick;
                emit viewerMouseEvent(*this,mouseAction,pos,1.0,keymod);
	}
}

void ImageViewer::mouseMoveEvent(QMouseEvent *event)
{
       Qt::KeyboardModifiers keymod =event->modifiers();

	QPoint pos=event->pos();
	MouseAction mouseAction=Move;
        emit viewerMouseEvent(*this,mouseAction,pos,1.0,keymod);
}

void ImageViewer::mouseReleaseEvent(QMouseEvent *event)
{
        Qt::KeyboardModifiers keymod =event->modifiers();
	QPoint pos=event->pos();
	MouseAction mouseAction=Release;
        emit viewerMouseEvent(*this,mouseAction,pos,1.0,keymod);
}

void ImageViewer::mouseDoubleClickEvent(QMouseEvent *event)
{
         Qt::KeyboardModifiers keymod =event->modifiers();
	QPoint pos=event->pos();
	MouseAction mouseAction=DoubleClick;
        emit viewerMouseEvent(*this,mouseAction,pos,1.0,keymod);
}

void ImageViewer::switchDisplay()
{
	if(displayType==Difference) 
		setDisplayType(Animation); 
	else 
		setDisplayType(Difference);
}


/*
 void ImageViewer::tabletEvent(QTabletEvent *event)
 {
	QPoint pos=event->pos();
	MouseAction mouseAction;

	static int count = 0;
	count++;


     switch (event->type()) {
         case QEvent::TabletPress:
				mouseAction=LeftClick;
				emit viewerMouseEvent(*this,mouseAction,pos,event->pressure());
				break;
         case QEvent::TabletRelease:
				mouseAction=Release;
				emit viewerMouseEvent(*this,mouseAction,pos,event->pressure());
				break;
         case QEvent::TabletMove:
				mouseAction=Move;
				if (count%5 != 0)
					return;
				emit viewerMouseEvent(*this,mouseAction,pos,event->pressure());
				break;
         default:
				//printf("other tablet %i\n", event->type());
				break;
     }
    // update	();
 }
 */

