#ifndef INSPECTOR_H
#define INSPECTOR_H

#include <QTabWidget>
#include <QWidget>
#include <QList>
#include <QTableWidget>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QLabel>
#include <QRadioButton>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QCheckBox>
#include <QButtonGroup>
#include <QRadioButton>
#include "MotionPara.h"
#include <QBasicTimer>
#include "Image.h"

class Inspector : public QWidget
{
	Q_OBJECT

public:
	//Inspector(QWidget *parent = 0, Qt::WFlags flags = 0);
	Inspector(QWidget *parent = 0, Qt::WFlags flags = 0);
	~Inspector();
	void addOpticalFlowPara(const OpticalFlowPara& para);
	void setEditing();
protected:
	 virtual void timerEvent(QTimerEvent *event);
signals:
	void selectedRow(int row);
	void paraModified(const MotionPara& mPara);
	void manualMatchingChanged(bool isManualMatching);
	void uncertainMatchingChanged(bool isUncertainMatching);
	void searchSizeChanged(int searchSize);
	void winSizeChanged(int winSize);
	void motionTypeSelected(int type);
	void tabChanged(int index);
	void clearLabeling();
	void redoMatching();
	void lambdaChanged(double);
	void grabFeatures();

public slots:
	void paraLoaded(const MotionPara& mPara);
	void clearPara();
	void setSearchingWindowSize(int searchSize,int winSize);
	void setMotionType(int type);
	void selectRow(int row);
	void selectTab(int index);
	void setLambda(double lambda);
private slots:
	void addOpticalFlowParaSet();
	void removeOpticalFlowParaSet();
	void selectCell(int row,int column);
	void modifyData(int row,int column);
	void setRatio(double);
	void setMinWidth(int);
	void setnIterations(int);
	void setnIRLSIterations(int);
	void setnCGIterations(int);
	void setSearchSizeFromSlider(int searchSize);
	void setWinSizeFromSlider(int winSize);
	void translationChecked(bool);
	void affineChecked(bool);
	void homographyChecked(bool);
	void piecewisesmoothChecked(bool);
	
	//void selectRow(int row);
private:
	MotionPara motionPara;

	QTableWidget *tableWidget;
	QVBoxLayout* vLayout;
	QHBoxLayout* hLayout;
	QHBoxLayout* buttonLayout;
	QPushButton* addParaButton;
	QPushButton* removeParaButton;

	QLabel* ratioLabel;
	QLabel* minWidthLabel;
	QLabel* nIterationsLabel;
	QLabel* nIRLSIterationsLabel;
	QLabel* nCGIterationsLabel;

	QDoubleSpinBox* ratioSpinBox;
	QSpinBox* minWidthSpinBox;
	QSpinBox* nIterationsSpinBox;
	QSpinBox* nIRLSIterationsSpinBox;
	QSpinBox* nCGIterationsSpinBox;

	QHBoxLayout* ratioLayout;
	QHBoxLayout* minWidthLayout;
	QHBoxLayout* nIterationsLayout;
	QHBoxLayout* nIRLSIterationsLayout;
	QHBoxLayout* nCGIterationsLayout;

	QTabWidget* tabGroup;
	QWidget* OpticalFlow;
	QWidget* Parametric;
	QWidget* Manual;

	QVBoxLayout* vGlobalLayout;
	bool IsAddPara;
	bool IsActive; // IsActive is false when no layer is selected, accessed by slot clearPara();

	int m_SearchSize,m_WinSize;
	bool IsUncertaintyMatching;

	QSpinBox* spinBoxSearchSize;
	QSlider* sliderSearchSize;

	QSpinBox* spinBoxWinSize;
	QSlider* sliderWinSize;

	QCheckBox* manualMatching;
	QCheckBox* enableUncertaintyMatching;
	QRadioButton* button1; // these are the button for motion selection
	QRadioButton* button2;
	QRadioButton* button3;
	QRadioButton* button4;

	QDoubleSpinBox* lambdaSpinBox;
	QPushButton* clearLabelingButton;
	QPushButton* redoMatchingButton;
	QPushButton* grabFeaturesButton;
	bool IsSettingValue;

	QBasicTimer timer;
	unsigned int steps;
};

#endif // INSPECTOR_H
