#include "IntegerImage.h"

IntegerImage::IntegerImage(void)
{
	imWidth=0;
	imHeight=0;
	nPixels=0;
	m_Min=m_Max=m_Interval=0;
	pData=NULL;
}

IntegerImage::IntegerImage(const IntegerImage& other)
{
	imWidth=0;
	imHeight=0;
	nPixels=0;
	m_Min=m_Max=m_Interval=0;
	pData=NULL;
	copyData(other);
}

IntegerImage::~IntegerImage(void)
{
	release();
}

void IntegerImage::release()
{
	if(pData!=NULL)
		delete pData;
	pData=NULL;
}

void IntegerImage::copyData(const IntegerImage &other)
{
	m_Min=other.m_Min;
	m_Max=other.m_Max;
	m_Interval=other.m_Interval;
	imWidth=other.imWidth;
	imHeight=other.imHeight;
	if((nPixels!=other.nPixels))
	{
		release();
		pData=new unsigned short int[other.nPixels];
	}
	nPixels=other.nPixels;
	memcpy(pData,other.pData,sizeof(unsigned short int)*nPixels);
}

void IntegerImage::allocate(int width, int height)
{
	if(nPixels!=width*height)
	{
		release();
		pData=new unsigned short int[width*height];
	}
	imWidth=width;
	imHeight=height;
	nPixels=imWidth*imHeight;
}

void IntegerImage::allocate(const IntegerImage &other)
{
	allocate(other.imWidth,other.imHeight);
}

IntegerImage& IntegerImage::operator=(const IntegerImage& other)
{
	copyData(other);
	return *this;
}

//---------------------------------------------------------------------------------
// function to convert from or to double precision images
//---------------------------------------------------------------------------------
void IntegerImage::fromDoublePrecision(const DImage &v)
{
	allocate(v.width(),v.height());
	const double *pImageData=v.data();
	m_Min=m_Max=pImageData[0];
	for(int i=0;i<nPixels;i++)
	{
		m_Min=__min(pImageData[i],m_Min);
		m_Max=__max(pImageData[i],m_Max);
	}
	if(m_Max==m_Min)
	{
		m_Interval=0;
		memset(pData,0,sizeof(unsigned short int)*nPixels);
		return;
	}
	m_Interval=(m_Max-m_Min)/60000;
	//m_Interval=(m_Max-m_Min);
	for(int i=0;i<nPixels;i++)
		pData[i]=(pImageData[i]-m_Min)/m_Interval;
}

void IntegerImage::toDoublePrecision(DImage &v) const
{
	if(!v.matchDimension(imWidth,imHeight,1))
		v.allocate(imWidth,imHeight);
	double *pImageData=v.data();
	for(int i=0;i<nPixels;i++)
		pImageData[i]=pData[i]*m_Interval+m_Min;
}


void IntegerImage::toFloatPrecision(FImage &v) const
{
	if(!v.matchDimension(imWidth,imHeight,1))
		v.allocate(imWidth,imHeight);
	float *pImageData=v.data();
	for(int i=0;i<nPixels;i++)
		pImageData[i]=pData[i]*m_Interval+m_Min;
}

double IntegerImage::val(int x, int y)
{
	return pData[y*imWidth + x]*m_Interval+m_Min;
}


//---------------------------------------------------------------------------------
// functions for file IO
//---------------------------------------------------------------------------------
bool IntegerImage::writeIntegerImage(QFile& file) const
{
	// output the dimension and range information
	file.write((char *)&m_Min,sizeof(double));
	file.write((char *)&m_Max,sizeof(double));
	file.write((char *)&m_Interval,sizeof(double));
	file.write((char *)&imWidth,sizeof(int));
	file.write((char *)&imHeight,sizeof(int));
	if(imWidth*imHeight!=nPixels)
		return false;
	file.write((char *)&nPixels,sizeof(int));

	// output the image
	if(file.write((char *)pData,sizeof(short int)*nPixels)!=sizeof(short int)*nPixels)
		return false;

	return true;
}

bool IntegerImage::readIntegerImage(QFile &file)
{
	// read the range and dimension information of the image
	file.read((char *)&m_Min,sizeof(double));
	file.read((char *)&m_Max,sizeof(double));
	file.read((char *)&m_Interval,sizeof(double));
	file.read((char *)&imWidth,sizeof(int));
	file.read((char *)&imHeight,sizeof(int));

	// it's impossible for the image to be too big
	if(imWidth<0 || imWidth>10000 || imHeight<0 || imHeight>10000)
		return false;

	allocate(imWidth,imHeight);

	file.read((char *)&nPixels,sizeof(int));
	if(imWidth*imHeight!=nPixels)
		return false;

	// read the image
	if(file.read((char *)pData,sizeof(short int)*nPixels)!=sizeof(short int)*nPixels)
		return false;

	return true;
}