#include <QtGui>
#include <iostream>
#include "LayerObject.h"
#include "MyPreference.h"
#include "QVideoProcessing.h"
#include "OpticalFlowBidir.h"
#include "VideoLabeling.h"
#include "dt/dt.h"
#include "dt/image.h"
#include "jl_vectors.h"
#include "tri_logger.hpp"

// static variables
int LayerObject::selectedPointIndex=0;
ContourLabel LayerObject::m_OrgContourLabel=ContourLabel();
QPointF LayerObject::m_OrgCursorPoint=QPointF();


LayerObject::LayerObject(void): QObject()
{
	m_NumFrames=0;
	m_StartFrame=0;
	m_EndFrame=0;
	m_CreatedFrame=0;

	m_CurrentStroke =-1;


	m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.01,0.01,0.5));
	
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.02,0.02,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.03,0.03,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.05,0.04,0.5));

	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.05,0.05,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.045,0.045,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.04,0.04,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.035,0.035,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.03,0.03,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.025,0.025,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.02,0.02,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.015,0.015,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.01,0.01,0.5));
	//m_LayerMotionPara.addFlowPara(OpticalFlowPara(1,0.005,0.005,0.5));

	m_SelectedRow=0;
	m_MaxStrokeId = 0;

	m_StrokeBorderSize = 0;
	m_StrokeBoundaryBlur = 0;
	m_StrokeMedianBlur = 0;
	m_NearestStroke=-1;
	//m_TabIndex=0;
}

LayerObject::LayerObject(const LayerObject& layerObject):QObject()
{
	copyData(layerObject);

}

LayerObject::~LayerObject(void)
{
}

void LayerObject::copyData(const LayerObject &layerObject)
{
	sprintf(_strbuf, "Copying layer: %s", layerObject.m_LayerName.toAscii().data());
	LOG_STR_COUT(_strbuf);

	m_ContourLabelList=layerObject.m_ContourLabelList;
	m_DepthList=layerObject.m_DepthList;
	m_LayerName=layerObject.m_LayerName;

	m_IsGlobalLabelList=layerObject.m_IsGlobalLabelList;
	m_IsLocalLabelList=layerObject.m_IsLocalLabelList;
	m_IsTrackedList=layerObject.m_IsTrackedList;
	m_IsDepthLabeledList=layerObject.m_IsDepthLabeledList;

	m_NumFrames=layerObject.m_NumFrames;
	m_StartFrame=layerObject.m_StartFrame;
	m_EndFrame=layerObject.m_EndFrame;
	m_CreatedFrame=layerObject.m_CreatedFrame;

	m_LayerMaskList=layerObject.m_LayerMaskList;
	m_VisibleLayerMaskList=layerObject.m_VisibleLayerMaskList;
	m_BoundingBoxList=layerObject.m_BoundingBoxList;
	m_FullBoundingBoxList=layerObject.m_FullBoundingBoxList;

	m_LayerMotionPara=layerObject.m_LayerMotionPara;
	m_LayerMotionList=layerObject.m_LayerMotionList;
	m_LayerReverseMotionList=layerObject.m_LayerReverseMotionList;
	m_SelectedRow=layerObject.m_SelectedRow;

	m_CorrespondenceList=layerObject.m_CorrespondenceList;

	//m_TabIndex=layerObject.m_TabIndex;
	m_TabIndexList=layerObject.m_TabIndexList;

	m_StrokeImageList = layerObject.m_StrokeImageList;
	m_StrokeImageTextureList = layerObject.m_StrokeImageTextureList;
	m_StrokeImageStateList = layerObject.m_StrokeImageStateList;

	m_BoundaryDistanceImageList = layerObject.m_BoundaryDistanceImageList;
	m_BoundaryDistanceDxImageList = layerObject.m_BoundaryDistanceDxImageList;
	m_BoundaryDistanceDyImageList = layerObject.m_BoundaryDistanceDyImageList;

	m_IsBoundaryDistanceCreatedList = layerObject.m_IsBoundaryDistanceCreatedList;

	m_CurrentStroke = layerObject.m_CurrentStroke;

	m_StrokeList = layerObject.m_StrokeList;

	m_GeneratedStrokeList = layerObject.m_GeneratedStrokeList;
	m_GeneratedStrokeParameterList = layerObject.m_GeneratedStrokeParameterList;

	for(int i=0;i < m_NumFrames;i++)
	{
		
		m_StrokeList[i] = layerObject.m_StrokeList[i];
		m_StrokeImageList[i] = layerObject.m_StrokeImageList[i];
		m_BoundaryDistanceImageList[i] = layerObject.m_BoundaryDistanceImageList[i];
		m_BoundaryDistanceDxImageList[i] = layerObject.m_BoundaryDistanceDxImageList[i];
		m_BoundaryDistanceDyImageList[i] = layerObject.m_BoundaryDistanceDyImageList[i];

	}


	m_BoundaryFlowList = layerObject.m_BoundaryFlowList;
	m_BoundaryReverseFlowList = layerObject.m_BoundaryReverseFlowList;
	m_IsBoundaryFlowCreatedList = layerObject.m_IsBoundaryFlowCreatedList;

	m_ImageWidth =layerObject.m_ImageWidth;
	m_ImageHeight =layerObject.m_ImageHeight;

	m_MaxStrokeId = layerObject.m_MaxStrokeId;

	m_StrokeBoundaryBlur =0;
	m_StrokeMedianBlur =0;

	m_GuideStrokeKDTreeList = layerObject.m_GuideStrokeKDTreeList;
	m_ColorStrokeKDTreeList = layerObject.m_ColorStrokeKDTreeList;
	m_BoundaryKDTreeList = layerObject.m_BoundaryKDTreeList;


}

LayerObject& LayerObject::operator =(const LayerObject &layerObject)
{

	copyData(layerObject);
	return *this;
}

void LayerObject::create(int numFrames, int frameIndex, const QPointF& point, int layer_index)
{
	m_NumFrames=numFrames;
	m_StartFrame=frameIndex;
	m_EndFrame=frameIndex;
	m_CreatedFrame=frameIndex;

	create(m_NumFrames);

	m_IsLocalLabelList[frameIndex]=true;
	m_IsGlobalLabelList[frameIndex]=true;
	m_IsDepthLabeledList[frameIndex]=true;
	m_IsTrackedList[frameIndex]=true;

	// add point
	m_ContourLabelList[frameIndex].addPoint(point,true);
	m_ContourLabelList[frameIndex].addPoint(point,true);

	// initialize depth
	int depth=layer_index*30;
	if(depth>255)
		depth=255;
	QVector<double> depthVector(m_NumFrames,depth);
	m_DepthList=QList<double>::fromVector(depthVector);

	// initialize name
	m_LayerName=MyPreference::DefaultLayerName;

}

void LayerObject::createBackground(int numFrames, int width, int height)
{
	m_NumFrames=numFrames;
	m_StartFrame=0;
	m_EndFrame=numFrames-1;
	m_CreatedFrame=0;

	create(m_NumFrames);
	ContourLabel bg;
	bg.setIsPolygon(false);
        bg.setBoundingBox(QRectF(0,0,width,height));
	for(int i=0;i<m_NumFrames;i++)
	{

                m_ContourLabelList[i].addPoint(QPointF(0,0),false);
                m_ContourLabelList[i].addPoint(QPointF(width,0),false);
                m_ContourLabelList[i].addPoint(QPointF(width,height),false);
                m_ContourLabelList[i].addPoint(QPointF(0,height),false);

		m_ContourLabelList[i]=bg;
		m_DepthList[i]=255;
	}

	m_LayerName="Background";
}

// create the list based on the number of frames
void LayerObject::create(int numFrames)
{
	m_NumFrames=numFrames;
	QVector<ContourLabel> contourLabelVector(m_NumFrames);
	m_ContourLabelList=QList<ContourLabel>::fromVector(contourLabelVector);
	QVector<bool> labelStatusVector(m_NumFrames,false);
	m_IsGlobalLabelList=QList<bool>::fromVector(labelStatusVector);
	m_IsLocalLabelList=m_IsGlobalLabelList;
	m_IsTrackedList=m_IsGlobalLabelList;
	m_IsDepthLabeledList=m_IsGlobalLabelList;

	QVector<double> depthVector(m_NumFrames,0);
	m_DepthList=QList<double>::fromVector(depthVector);

	QVector<BinaryImage> maskVector(m_NumFrames);
	m_LayerMaskList=QList<BinaryImage>::fromVector(maskVector);
	m_VisibleLayerMaskList=QList<BinaryImage>::fromVector(maskVector);

	QVector<QRect> boundingBoxVector(m_NumFrames);
	m_BoundingBoxList=QList<QRect>::fromVector(boundingBoxVector);

	QVector<QRect> fullBoundingBoxVector(m_NumFrames);
	m_FullBoundingBoxList=QList<QRect>::fromVector(fullBoundingBoxVector);
	

	QVector<Motion> layerMotionVector(m_NumFrames);
	m_LayerMotionList=QList<Motion>::fromVector(layerMotionVector);

	QVector<Motion> layerReverseMotionVector(m_NumFrames);
	m_LayerReverseMotionList=QList<Motion>::fromVector(layerReverseMotionVector);
	

	QVector<Correspondence> correspondenceVector(m_NumFrames);
	m_CorrespondenceList=QList<Correspondence>::fromVector(correspondenceVector);
	m_SelectedRow=0;
	//m_TabIndex=0;

	m_TabIndexList=QList<int>::fromVector(QVector<int>(m_NumFrames,0));

    QVector<QList<ContourLabel> > frameStrokeVector(m_NumFrames);
    m_StrokeList=QList<QList<ContourLabel> >::fromVector(frameStrokeVector);

    QVector<QMap<int,QVector<Stroke *> > > genStrokeVector(m_NumFrames);
    m_GeneratedStrokeList=QList<QMap<int,QVector<Stroke *> > >::fromVector(genStrokeVector);


	QVector<QImage> strokeImageVector(m_NumFrames);
	m_StrokeImageList=QList<QImage>::fromVector(strokeImageVector);

	QVector<int> strokeImageTextureVector(m_NumFrames);
	m_StrokeImageTextureList=QList<int>::fromVector(strokeImageTextureVector);

	QVector<int> strokeImageStateVector(m_NumFrames);
	m_StrokeImageStateList=QList<int>::fromVector(strokeImageStateVector);

	QVector<bool> boundaryDistanceCreatedVector(m_NumFrames);
	m_IsBoundaryDistanceCreatedList=QList<bool>::fromVector(boundaryDistanceCreatedVector);


	QVector<ANNkd_tree*> guideStrokeKDTreeVector(m_NumFrames);
	m_GuideStrokeKDTreeList=QList<ANNkd_tree*>::fromVector(guideStrokeKDTreeVector);

	QVector<ANNkd_tree*> colorStrokeKDTreeVector(m_NumFrames);
	m_ColorStrokeKDTreeList=QList<ANNkd_tree*>::fromVector(colorStrokeKDTreeVector);

	QVector<ANNkd_tree*> boundaryKDTreeVector(m_NumFrames);
	m_BoundaryKDTreeList=QList<ANNkd_tree*>::fromVector(boundaryKDTreeVector);


	m_BoundaryDistanceImageList.clear();
	m_BoundaryDistanceDxImageList.clear();
	m_BoundaryDistanceDyImageList.clear();

	for(int i=0;i < m_NumFrames;i++)
	{
		QList<ContourLabel> contourList;
		m_StrokeList[i] = contourList;
		m_StrokeImageTextureList[i] = 0;
		m_StrokeImageStateList[i] = NO_IMAGE;
		m_GuideStrokeKDTreeList[i] = 0;
		m_ColorStrokeKDTreeList[i] = 0;
		m_BoundaryKDTreeList[i] = 0;
		m_BoundaryDistanceImageList.push_back( IntegerImage());
		m_BoundaryDistanceDxImageList.push_back( IntegerImage());
		m_BoundaryDistanceDyImageList.push_back( IntegerImage());
	}


	QVector<bool> boundaryFlowCreatedVector(m_NumFrames);
	m_IsBoundaryFlowCreatedList=QList<bool>::fromVector(boundaryFlowCreatedVector);

	QVector<FlowField> boundaryFlowVector(m_NumFrames);
	m_BoundaryFlowList=QList<FlowField>::fromVector(boundaryFlowVector);

	QVector<FlowField> boundaryReverseFlowVector(m_NumFrames);
	m_BoundaryReverseFlowList=QList<FlowField>::fromVector(boundaryReverseFlowVector);


}

//-----------------------------------------------------------------------------------------------------------------------
//	 function to stretch the range of the object
//-----------------------------------------------------------------------------------------------------------------------
void LayerObject::StretchToFrame(int frameIndex)
{
	if(InRange(frameIndex))
		return;
	frameIndex=qMin(qMax(frameIndex,0),m_NumFrames-1);
	if(frameIndex>m_EndFrame)
	{
		for(int i=m_EndFrame+1;i<=frameIndex;i++)
		{
			m_ContourLabelList[i]=m_ContourLabelList[m_EndFrame];
			m_ContourLabelList[i].clearLabels();
			m_DepthList[i]=m_DepthList[m_EndFrame];
		}
		m_EndFrame=frameIndex;
	}
	if(frameIndex<m_StartFrame)
	{
		for(int i=frameIndex;i<=m_StartFrame-1;i++)
		{
			m_ContourLabelList[i]=m_ContourLabelList[m_StartFrame];
			m_ContourLabelList[i].clearLabels();
			m_DepthList[i]=m_DepthList[m_StartFrame];
		}
		m_StartFrame=frameIndex;
	}
}


//-----------------------------------------------------------------------------------------------------------------------
//	 function to replicate the strokes of the object over a sequence of frae
//-----------------------------------------------------------------------------------------------------------------------
bool LayerObject::ReplicateStrokesToFrame(int curFrameIndex, int destFrameIndex,int actionId,QList<QImage>& frameList,double colorUpdate, float optFlowFactor, float boundFlowFactor, float userFlowFactor, float fixedFlowFactor, float fixedFlowAngle, float fixedFlowDist, float followStrokeFactor,float followFlowFactor)
{


	if (destFrameIndex  <= m_StartFrame)
		destFrameIndex = m_StartFrame;

	if (destFrameIndex  >= m_EndFrame)
		destFrameIndex= m_EndFrame;

	//destination can't be the same as the current frame
	if(destFrameIndex==curFrameIndex)
		return false;


	bool replicateStrokes = false;



	bool forward=true;
	if(destFrameIndex<curFrameIndex)
		forward=false;
	
	QList<ContourLabel*> contourList;

	
	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].groupSelected()) 
		{
			//if (((forward)&&(m_StrokeList[curFrameIndex][j].getNext() == 0)) || 
			//	((!forward)&&(m_StrokeList[curFrameIndex][j].getPrev() == 0)))
				contourList.push_back(&m_StrokeList[curFrameIndex][j]);
		}
	}


	if (contourList.size()==0)
		return false;


	while(curFrameIndex!=destFrameIndex)
	{

		FlowField* nextLayer=0;

		int lastFrame;

		if(forward)
		{
			lastFrame=curFrameIndex;		
			curFrameIndex++;	
		}
		else
		{
			lastFrame=curFrameIndex;
			curFrameIndex--;
		}

		int leftFlow=0;
		int topFlow=0;



		bool useOpticalFlow=true;
		if (((forward)&&(flowExists(lastFrame)==false))||((!forward)&&(reverseFlowExists(lastFrame)==false)))
		{
			sprintf(_strbuf, "WARNING: No optical flow for frame %i, only using boundary or user flow. Run motion analysis if wanted.", lastFrame);
			LOG_STR_COUT(_strbuf);
			useOpticalFlow=false;
		}
		else
		{

			if(forward)
				nextLayer=&(layerFlow(lastFrame));
			else
				nextLayer=&(layerReverseFlow(lastFrame));

			assert(nextLayer!=0);
			
			leftFlow = nextLayer->left();
			topFlow = nextLayer->top();

		}




		QVector<Vec2f> curPts;
		QVector<Vec2f> disp;


		for (int k=0;k < m_StrokeList[lastFrame].size();k++)
		{

			QList<QPointF> pts = m_StrokeList[lastFrame][k].PointList();

			ContourLabel *nextStroke= m_StrokeList[lastFrame][k].getNext();

			if (forward)
				nextStroke= m_StrokeList[lastFrame][k].getNext();
			else
				nextStroke= m_StrokeList[lastFrame][k].getPrev();

			if (nextStroke == 0)
				continue;

			QList<QPointF> nextPts = nextStroke->PointList();

			if(nextPts.size() != pts.size())
			{
				sprintf(_strbuf, "WARNING: next stroke size %i, doesn't match current %i", nextPts.size(),pts.size());
				LOG_STR_COUT(_strbuf);
				continue;
			}

			for(int n=0;n < pts.size();n++)
			{
				curPts.push_back(Vec2f(pts[n].x(), pts[n].y()));
				disp.push_back(Vec2f(nextPts[n].x()-pts[n].x(), nextPts[n].y()-pts[n].y()));
			}
		}


		for(int j=0;j <contourList.size();j++)
		{

			bool foundThisFrame=false;
			for (int k=0;k < m_StrokeList[curFrameIndex].size();k++)
			{

				//this frame already has the current stroke
				if (m_StrokeList[curFrameIndex][k].getStrokeId()==contourList[j]->getStrokeId())
				{				
					//skip this stroke if the stroke already has a following 
					if ((forward)&&( contourList[j]->getNext()!= 0))
					{
						
						contourList[j]=contourList[j]->getNext();
						foundThisFrame=true;
						k = m_StrokeList[curFrameIndex].size();
					}
					if ((!forward)&&( contourList[j]->getPrev()!= 0))
					{
						
						contourList[j]=contourList[j]->getPrev();
						foundThisFrame=true;
						k = m_StrokeList[curFrameIndex].size();
					}
				}
			}
			if (foundThisFrame)
				continue;
	

			//Get a copy of the contour we're moving
			ContourLabel* oldlabel = contourList[j];
			ContourLabel newlabel = *oldlabel;

			
			newlabel.clearLabels();
			newlabel.clearStroke();
			//newlabel.clearStroke();

	
			QList<QPointF> ptlist = newlabel.PointList();
			
			bool visiblePoint = false;
			float visx=0, visy=0;
			if (useOpticalFlow)
			{
				for (int p=0;p < ptlist.size();p++)
				{
					if (m_VisibleLayerMaskList[lastFrame].getPixel(ptlist[p].x(),ptlist[p].y()))
					{
						visiblePoint = true;
						nextLayer->getFlow(ptlist[p].x()-leftFlow,ptlist[p].y()-topFlow,&visx,&visy);
						break;
					}
				}
			}
			for (int p=0;p < ptlist.size();p++)
			{
				float optflowx = 0, optflowy = 0, userflowx=0, userflowy=0,boundflowx=0, boundflowy=0;

				float fixedflowx = 0;
				float fixedflowy = 0;

				if (forward)
				{
					fixedflowx = cos(fixedFlowAngle)*fixedFlowDist;
					fixedflowy = -sin(fixedFlowAngle)*fixedFlowDist;
				}
				else
				{
					fixedflowx = -cos(fixedFlowAngle)*fixedFlowDist;	
					fixedflowy = sin(fixedFlowAngle)*fixedFlowDist;
				}

				float flowx, flowy;

				if ((getBoundaryFlow(forward, lastFrame, ptlist[p].x(), ptlist[p].y(), &boundflowx, &boundflowy) == false))
				{
					sprintf(_strbuf, "boundary flow prob...ptlist p %i, %f %f", p, ptlist[p].x(), ptlist[p].y());
					LOG_STR_COUT(_strbuf);
				}

				if (userFlowFactor != 0)
				{
					if (getUserStrokeFlow(forward, lastFrame, ptlist[p].x(), ptlist[p].y(),curPts, disp,&userflowx, &userflowy) == false)
					{
									
						userflowx = boundflowx;
						userflowy = boundflowy;
					}
				}	

				if (optFlowFactor != 0)
				{
					if ((useOpticalFlow==true)&&(m_VisibleLayerMaskList[lastFrame].getPixel(ptlist[p].x(),ptlist[p].y())==true))
					{
						nextLayer->getFlow(ptlist[p].x()-leftFlow,ptlist[p].y()-topFlow,&optflowx,&optflowy);
					}
					else if (visiblePoint== true)
					{
						optflowx = visx;
						optflowy = visy;
					}
					else
					{
						optflowx = boundflowx;
						optflowy = boundflowy;
					}
				}
	

				flowx = optflowx*optFlowFactor + boundflowx*boundFlowFactor + userflowx*userFlowFactor + fixedflowx*fixedFlowFactor;
				flowy = optflowy*optFlowFactor + boundflowy*boundFlowFactor + userflowy*userFlowFactor + fixedflowy*fixedFlowFactor;
					
				newlabel.updatePoint(QPointF(ptlist[p].x()+flowx,ptlist[p].y()+flowy),p,false);
				newlabel.addStrokePoint(ptlist[p].x()+flowx,ptlist[p].y()+flowy);
			}

			regularizeContour(*(contourList[j]),newlabel,followStrokeFactor,followFlowFactor);

	

			
			if (newlabel.getStroke()->properties.useImageColor)
			{
				float r=0,g=0,b=0,c=0;

				QList<QPointF> ptlist = newlabel.PointList();
				for (int p=0;p < ptlist.size();p++)
				{

					if ((ptlist[p].x() > 0) && (ptlist[p].y() > 0) && (ptlist[p].x()< frameList[curFrameIndex].width()) && (ptlist[p].y() < frameList[curFrameIndex].height())
						&&(m_VisibleLayerMaskList[curFrameIndex].getPixel(ptlist[p].x(),ptlist[p].y())))
					{

						QRgb pix = frameList[curFrameIndex].pixel(ptlist[p].x(), ptlist[p].y());
						r += qRed(pix);
						g += qGreen(pix);
						b += qBlue(pix);
						c++;
					}
				}
				if (c!=0)
				{
					QColor newcol(r/c, g/c,b/c);
				
				
					int h,s,v;
					newcol.getHsv(&h,&s,&v);
		                  
					int newH = min(max(0,h +  newlabel.getStroke()->properties.hueOffset),359);
					int newS = min(max(0,s +  newlabel.getStroke()->properties.saturationOffset),254);
					int newV = min(max(0,v +  newlabel.getStroke()->properties.valueOffset),254);


					newcol.setHsv(newH,newS,newV);
				
					//printf("before %i %i %i, last \n",newcol.red(),newcol.green(),newcol.blue());
					newcol.setRed(colorUpdate*((float)newcol.red())+ (1-colorUpdate)*((float)newlabel.getStroke()->properties.color.red()));
					newcol.setGreen(colorUpdate*((float)newcol.green())+ (1-colorUpdate)*((float)newlabel.getStroke()->properties.color.green()));
					newcol.setBlue(colorUpdate*((float)newcol.blue())+ (1-colorUpdate)*((float)newlabel.getStroke()->properties.color.blue()));
					//printf("after %i %i %i\n",newcol.red(),newcol.green(),newcol.blue());

					newlabel.getStroke()->properties.color=  newcol;
				}
			}


	
			newlabel.recomputeStroke();

			m_StrokeList[curFrameIndex].push_back(newlabel);
			int latest = m_StrokeList[curFrameIndex].size();
			ContourLabel* newl = &m_StrokeList[curFrameIndex][latest-1];
			contourList[j] =newl;
			if (forward)
			{
				newl->setPrev(oldlabel);
				oldlabel->setNext(newl);
			}
			else
			{
				newl->setNext(oldlabel);
				oldlabel->setPrev(newl);
			}

			//m_StrokeList[i][latest-1].getStroke()->forceRecompute();
			QVector<Point> *pointList = m_StrokeList[curFrameIndex][latest-1].getStroke()->getLimitCurve();
			//sprintf(_strbuf, "pointlist size %i", pointList->size());

			replicateStrokes=true;
		}
	}
		


	return replicateStrokes;

}



void LayerObject::updateSelectedStrokeColor(int curFrameIndex, QList<QImage>& frameList,double colorUpdate, int strokeId)
{

	QVector<int> strokeIds;

	if (strokeId==-1)
	{
		for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
		{
			if (m_StrokeList[curFrameIndex][j].groupSelected())
				strokeIds.push_back(m_StrokeList[curFrameIndex][j].getStrokeId());
		}
	}
	else
		strokeIds.push_back(strokeId);



	for(int k=0;k < strokeIds.size();k++)
	{	
		
		QColor lastcol;
		bool pastFirst=false;

		for(int i=m_StartFrame;i<=m_EndFrame;i++) 
		{

			for(int j=0;j < m_StrokeList[i].size();j++)
			{
				if ((strokeIds[k]==m_StrokeList[i][j].getStrokeId())&&(m_StrokeList[i][j].deleted()==false))
				{

					if (m_StrokeList[i][j].getStroke()->properties.useImageColor)
						{
							float r=0,g=0,b=0,c=0;

							QList<QPointF> ptlist = m_StrokeList[i][j].PointList();
							for (int p=0;p < ptlist.size();p++)
							{

								if ((ptlist[p].x() > 0) && (ptlist[p].y() > 0) && (ptlist[p].x()< frameList[curFrameIndex].width()) && (ptlist[p].y() < frameList[curFrameIndex].height())
									&&(m_VisibleLayerMaskList[curFrameIndex].getPixel(ptlist[p].x(),ptlist[p].y())))
								{

									QRgb pix = frameList[curFrameIndex].pixel(ptlist[p].x(), ptlist[p].y());
									r += qRed(pix);
									g += qGreen(pix);
									b += qBlue(pix);
									c++;
								}
							}
							if (c!=0)
							{
								QColor newcol(r/c, g/c,b/c);
							
							
								int h,s,v;
								newcol.getHsv(&h,&s,&v);
					                  
								int newH = min(max(0,h +  m_StrokeList[i][j].getStroke()->properties.hueOffset),359);
								int newS = min(max(0,s +  m_StrokeList[i][j].getStroke()->properties.saturationOffset),254);
								int newV = min(max(0,v +  m_StrokeList[i][j].getStroke()->properties.valueOffset),254);

								newcol.setHsv(newH,newS,newV);
							
						
								if (pastFirst)
								{
									newcol.setRed(colorUpdate*((float)newcol.red())+ (1-colorUpdate)*((float)lastcol.red()));
									newcol.setGreen(colorUpdate*((float)newcol.green())+ (1-colorUpdate)*((float)lastcol.green()));
									newcol.setBlue(colorUpdate*((float)newcol.blue())+ (1-colorUpdate)*((float)lastcol.blue()));
								}
								else
									pastFirst=true;

								lastcol=newcol;

								m_StrokeList[i][j].getStroke()->properties.color=  newcol;
							}
						}
				}
			}
		}

	}
	
}


//-----------------------------------------------------------------------------------------------------------------------
// display functions
//-----------------------------------------------------------------------------------------------------------------------
void LayerObject::DrawCorrespondence(int frameIndex,QPainter& painter,QPointF& ScreenOffset,double ScaleFactor)
{
	if(InRange(frameIndex)==false)
		return;
	m_CorrespondenceList[frameIndex].DrawCorresponance(painter,QPoint(ScreenOffset.x(),ScreenOffset.y()),ScaleFactor);
}

void LayerObject::DrawContour(int frameIndex, QPainter &painter, QPointF &ScreenOffset, double ScaleFactor, bool IsActive, bool IsCreating, bool IsCursorOver,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return;

	bool active = IsActive;
	bool cursorOver = IsCursorOver;
	if (boundaryContour == false)
	{
		active = false;
		cursorOver = false;
	}

	m_ContourLabelList[frameIndex].DrawContour(painter,ScreenOffset,ScaleFactor,m_DepthList[frameIndex],false,active,IsCreating,cursorOver,m_IsGlobalLabelList[frameIndex]);

}

void LayerObject::DrawStrokes(int frameIndex, QPainter &painter, QPointF &ScreenOffset, double ScaleFactor, bool highlightNearest, bool IsActive, bool IsCreating, bool IsCursorOver,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return;

	bool cursorOverSelected = false;
	bool active = false;

	for(int i=0;i < m_StrokeList[frameIndex].size();i++)
	{
		if ((i == m_CurrentStroke) && (IsCursorOver))
			cursorOverSelected = true;
		else 
			cursorOverSelected = false;

		if ((i == m_CurrentStroke) && (IsActive))
			active = true;
		else 
			active = false;

		bool nearestHighlight = ((highlightNearest==true) && (i==m_NearestStroke));

		m_StrokeList[frameIndex][i].DrawContour(painter,ScreenOffset,ScaleFactor,m_DepthList[frameIndex],nearestHighlight,active,false,cursorOverSelected,m_IsGlobalLabelList[frameIndex]);
	}
}


void LayerObject::ShowText(int frameIndex, QPainter &painter, QPointF &ScreenOffset, double ScaleFactor)
{
	if(InRange(frameIndex)==false)
		return;
	m_ContourLabelList[frameIndex].ShowText(painter,ScreenOffset,ScaleFactor,m_LayerName);
}

double LayerObject::distanceToContour(int frameIndex, const QPointF& point)
{
	if(InRange(frameIndex)==false)
		return MyPreference::DefaultContourDistance;
	return m_ContourLabelList[frameIndex].distanceToContour(point);
}

void LayerObject::addPoint(int frameIndex, const QPointF &point, bool IsLabeled)
{
	if(InRange(frameIndex)==false)
		return;
	m_ContourLabelList[frameIndex].addPoint(point,IsLabeled);
}

void LayerObject::updateLastPoint(int frameIndex, const QPointF &point, bool IsLabeled)
{
	if(InRange(frameIndex)==false)
		return;
	m_ContourLabelList[frameIndex].updateLastPoint(point,IsLabeled);
}

void LayerObject::removeLastPoint(int frameIndex)
{
	if(InRange(frameIndex)==false)
		return;
	m_ContourLabelList[frameIndex].removeLastPoint();
}

double LayerObject::minPointDistance(int frameIndex, const QPointF &point,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return MyPreference::DefaultContourDistance;

	
	if (boundaryContour == true) 
		return m_ContourLabelList[frameIndex].minPointDistance(point,selectedPointIndex);
	else if ((boundaryContour == false) && (m_CurrentStroke>=0)&& (m_CurrentStroke<m_StrokeList[frameIndex].size()))
		return m_StrokeList[frameIndex][m_CurrentStroke].minPointDistance(point,selectedPointIndex);
}

double LayerObject::minPointDistanceSelectedStrokes(int frameIndex, const QPointF &point)
{
	if(InRange(frameIndex)==false)
		return MyPreference::DefaultContourDistance;

	float minDist = MyPreference::DefaultContourDistance;

	m_NearestStroke=-1;

	for(int i=0;i< m_StrokeList[frameIndex].size();i++)
	{
		if (m_StrokeList[frameIndex][i].groupSelected()==false)
			continue;

		float dist = m_StrokeList[frameIndex][i].minPointDistance(point,selectedPointIndex);

		if (dist < minDist)
		{
			minDist=dist;
			m_NearestStroke=i;
		}
	}

	return minDist;
}




double LayerObject::minLineSegDistance(int frameIndex,const QPointF& point,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return MyPreference::DefaultContourDistance;
	
	QPointF projectPoint;

	if (boundaryContour == true) 
		return m_ContourLabelList[frameIndex].minLineSegDistance(point,selectedPointIndex,projectPoint);
	else if ((boundaryContour == false) && (m_CurrentStroke>=0)&& (m_CurrentStroke<m_StrokeList[frameIndex].size()))
		return m_StrokeList[frameIndex][m_CurrentStroke].minLineSegDistance(point,selectedPointIndex,projectPoint);


}

bool LayerObject::IsCentroidSelected(int frameIndex, const QPointF &point, double ScaleFactor)
{
	if(InRange(frameIndex)==false)
		return false;
	if(m_ContourLabelList[frameIndex].IsCentroidSelected(point,ScaleFactor))
	{
		m_OrgContourLabel=m_ContourLabelList[frameIndex];
		m_OrgCursorPoint=point;
		return true;
	}
	else
		return false;
}

/*
	Check whether a stroke is close to the given point and if so, set this to the current stroke.

*/
bool LayerObject::IsStrokeSelected(int frameIndex, const QPointF &point, double ScaleFactor)
{
	if(InRange(frameIndex)==false)
		return false;

	int closest;
	double minDist = 9999.0, dist =0;

        //sprintf(_strbuf, "num strokes %i", m_StrokeList[frameIndex].size());
	for(int i=0;i < m_StrokeList[frameIndex].size();i++)
	{
		dist = m_StrokeList[frameIndex][i].distanceToContour(point);
                //sprintf(_strbuf, "dist %f", dist);
		if((dist < MyPreference::Threshold_ContourDistance) && (dist< minDist ))
		{
			minDist = dist;
			closest = i;
		}
	}
	
	if (minDist < 9999.0)
	{

		m_CurrentStroke=closest;

		return true;
	}
	else
		return false;
}

void LayerObject::strokeClicked(int frameIndex)
{

	for(int k=0;k < m_StrokeList[frameIndex].size();k++)
	{
		m_StrokeList[frameIndex][k].groupDeselectContour();
	}
	if ((m_CurrentStroke>=0)&&(m_CurrentStroke<m_StrokeList[frameIndex].size()))
		m_StrokeList[frameIndex][m_CurrentStroke].groupSelectContour();
}



StrokeProperties& LayerObject::getSelectedStrokeProperties(int frameIndex)
{
	
	if ((m_CurrentStroke<0)||(m_CurrentStroke>=m_StrokeList[frameIndex].size()))
	{
		StrokeProperties prop;
		return prop;
	}

	assert(InRange(frameIndex)==true);

	return 	m_StrokeList[frameIndex][m_CurrentStroke].getStroke()->properties;
}


	

int LayerObject::selectBoundingBox(int frameIndex,const QPointF &point, double ScaleFactor,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return -1;

        //sprintf(_strbuf, "looking for bounding box, selected %i",m_Selected);
	
	ContourLabel* label;

	if ((boundaryContour == false) && (m_CurrentStroke>=0) && (m_StrokeList[frameIndex].size() > m_CurrentStroke) )
	{
		label = &(m_StrokeList[frameIndex][m_CurrentStroke]);
                //sprintf(_strbuf, "\t in stroke");
	}
	else if (boundaryContour == true)
	{
		label = &(m_ContourLabelList[frameIndex]);
                //sprintf(_strbuf, "\t in other");
	}
	else
		return -1;

	int result = label->selectBoundingBox(point,ScaleFactor,MyPreference::Threshold_BoundingBox);
	if( result >=0)
	{
		m_OrgContourLabel=*label;
		m_OrgCursorPoint=point;
	}
	return result;
}

//------------------------------------------------------------------------------------------------------------
// function that need to propagate the operation of the current frame to the rest of the frames
//------------------------------------------------------------------------------------------------------------

// function to find the first labeled frame
int LayerObject::findFirstLocalLabeledFrame(int frameIndex,int pointIndex,bool& IsForwardLabeled)
{
	if(frameIndex==m_StartFrame)
	{
		IsForwardLabeled=m_ContourLabelList[m_StartFrame].labeled(pointIndex);
		return m_StartFrame;
	}
	int i;
	IsForwardLabeled=false;
	bool IsInside=m_ContourLabelList[frameIndex].IsInsideImage(pointIndex);
	for(i=frameIndex-1;i>=m_StartFrame;i--)
	{
		// if the intial point is outside image boundary and the i-th one is inside, then don't change the inside one
		if(IsInside==false && m_ContourLabelList[i].IsInsideImage(pointIndex)==true)
		{
			IsForwardLabeled=true;
			return i;
		}
		if(m_ContourLabelList[i].labeled(pointIndex)==true)
		{
			IsForwardLabeled=true;
			return i;
		}
	}
	return i;
}

// function to find the first labeled frame
int LayerObject::findContourFirstLocalLabeledFrame(ContourLabel& contour, int frameIndex,int pointIndex,bool& IsForwardLabeled)
{

	if(frameIndex==m_StartFrame)
	{
		IsForwardLabeled=contour.labeled(pointIndex);
		return m_StartFrame;
	}
	int i;
	IsForwardLabeled=false;
	bool IsInside=contour.IsInsideImage(pointIndex);
	ContourLabel *currLabel = contour.getPrev();
	for(i=frameIndex-1;(i>=m_StartFrame) && (currLabel != 0);i--)
	{
		// if the intial point is outside image boundary and the i-th one is inside, then don't change the inside one
		//if(IsInside==false && currLabel->IsInsideImage(pointIndex)==true)
		//{
		//	IsForwardLabeled=true;
		//	return i;
		//}
		if (currLabel->numPoints() < pointIndex)
		{
    sprintf(_strbuf, "problem with number of stroke points %i at index %i", currLabel->numPoints() , pointIndex);
			LOG_STR_COUT(_strbuf);
			return -1;
		}

		if(currLabel->labeled(pointIndex)==true)
		{
			IsForwardLabeled=true;
			return i;
		}
		currLabel = currLabel->getPrev();
	}
	return i;
}

// function to find the last labeled frame
int LayerObject::findLastLocalLabeledFrame(int frameIndex, int pointIndex,bool& IsBackwardLabeled)
{
	if(frameIndex==m_EndFrame)
	{
		IsBackwardLabeled=m_ContourLabelList[m_EndFrame].labeled(pointIndex);
		return m_EndFrame;
	}
	int i;
	IsBackwardLabeled=false;
	bool IsInside=m_ContourLabelList[frameIndex].IsInsideImage(pointIndex);
	for(i=frameIndex+1;i<=m_EndFrame;i++)
	{
		// if the intial point is outside image boundary and the i-th one is inside, then don't change the inside one
		if(IsInside==false && m_ContourLabelList[i].IsInsideImage(pointIndex)==true)
		{
			IsBackwardLabeled=true;
			return i;
		}
		if(m_ContourLabelList[i].labeled(pointIndex)==true)
		{
			IsBackwardLabeled=true;
			return i;
		}
	}
	return i;
}

// function to find the last labeled frame
int LayerObject::findContourLastLocalLabeledFrame(ContourLabel& contour,int frameIndex, int pointIndex,bool& IsBackwardLabeled)
{

	if(frameIndex==m_EndFrame)
	{
		IsBackwardLabeled=contour.labeled(pointIndex);
		return m_EndFrame;
	}

	int i;
	IsBackwardLabeled=false;
	//bool IsInside=contour.IsInsideImage(pointIndex);
	ContourLabel *currLabel = contour.getNext();
	for(i=frameIndex+1;((i<=m_EndFrame)&& (currLabel != 0));i++)
	{
		// if the intial point is outside image boundary and the i-th one is inside, then don't change the inside one
		//if(IsInside==false && currLabel->IsInsideImage(pointIndex)==true)
		//{
		//	IsBackwardLabeled=true;
		//	return i;
		//}

		if (currLabel->numPoints() < pointIndex)
		{
    sprintf(_strbuf, "problem with number of stroke points %i at index %i", currLabel->numPoints() , pointIndex);
			LOG_STR_COUT(_strbuf);

			return -1;
		}

		if(currLabel->labeled(pointIndex)==true)
		{
			IsBackwardLabeled=true;
			return i;
		}
		currLabel = currLabel->getNext();
	}

	return i;
}


int LayerObject::findFirstGlobalLabeledFrame(int frameIndex,bool& IsForwardLabeled)
{
	if(frameIndex==m_StartFrame)
	{
		IsForwardLabeled=m_IsGlobalLabelList[m_StartFrame];
		return m_StartFrame;
	}
	int i;
	IsForwardLabeled=false;
	for(i=frameIndex-1;i>=m_StartFrame;i--)
		if(m_IsGlobalLabelList[i]==true || m_IsLocalLabelList[i]==true)
		{
			IsForwardLabeled=true;
			return i;
		}
	return i;
}

int LayerObject::findLastGlobalLabeledFrame(int frameIndex,bool& IsBackwardLabeled)
{
	if(frameIndex==m_EndFrame)
	{
		IsBackwardLabeled=m_IsGlobalLabelList[m_EndFrame];
		return m_EndFrame;
	}
	int i;
	IsBackwardLabeled=false;
	for(i=frameIndex+1;i<=m_EndFrame;i++)
		if(m_IsGlobalLabelList[i]==true || m_IsLocalLabelList[i]==true)
		{
			IsBackwardLabeled=true;
			return i;
		}
	return i;
}

//-------------------------------------------------------------------------------------------------------
// Function to insert a point. Note that the contour has already been selected before this
// function is called. So there is only the need to compute which lin segment has been
// selected. Then the coordinate of the new point needs to be propagated to other frames
//-------------------------------------------------------------------------------------------------------
void LayerObject::insertPoint(int frameIndex, const QPointF &point,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return;

	ContourLabel *contour;

	if ((boundaryContour == false) && (m_CurrentStroke>=0)&&(m_CurrentStroke<m_StrokeList[frameIndex].size()))
		contour = &(m_StrokeList[frameIndex][m_CurrentStroke]);
	else if (boundaryContour == true)
		contour = &(m_ContourLabelList[frameIndex]);
	else
		return;

	QPointF projectPoint;
	contour->minLineSegDistance(point,selectedPointIndex,projectPoint);
	contour->insertPoint(point,++selectedPointIndex,true);
	m_IsLocalLabelList[frameIndex]=true;
	m_IsGlobalLabelList[frameIndex]=true;
	
	// propagate to the rest of the frames
	double projection,extension;
	contour->computeTwoPointsCoeff(selectedPointIndex,projection,extension);

	if (boundaryContour == true)
	{
		for(int i=startFrame();i<=endFrame();i++)
		{
			//m_IsBoundaryFlowCreatedList[i] = false;
			if(i==frameIndex)
				continue;
			m_ContourLabelList[i].insertPointTwoPointsCoeff(selectedPointIndex,projection,extension);
		}
		
	}
	else
	{
		ContourLabel *c = contour;
		while ((c= c->getNext()) != 0)
		{
			c->insertPointTwoPointsCoeff(selectedPointIndex,projection,extension);
		}
		c = contour;
		while ((c= c->getPrev()) != 0)
		{
			c->insertPointTwoPointsCoeff(selectedPointIndex,projection,extension);
		}
	}



}

/*
Set new properties for the current stroke

*/
void LayerObject::setStrokeProperties(int frameIndex, StrokeProperties& sp)
{
	LOG_STR_COUT("setting stroke properties");
	int strokeId = -1;
	if ((m_CurrentStroke>=0) && m_CurrentStroke < m_StrokeList[frameIndex].size())
		strokeId = m_StrokeList[frameIndex][m_CurrentStroke].getStrokeId();
	else
		return;

	for (int i=0;i < m_NumFrames;i++)
	{
		for (int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].getStrokeId() == strokeId)
			{
				m_StrokeList[i][j].getStroke()->properties = sp;
				m_StrokeList[i][j].getStroke()->forceRecompute();
			}
		}
	}
}

void LayerObject::setSelectedStrokeProperties(int curFrameIndex, QString prop, QString val)
{

	QVector<int> strokeIds;
	LOG_STR_COUT("setting stroke properties");

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].groupSelected())
			strokeIds.push_back(m_StrokeList[curFrameIndex][j].getStrokeId());
	}

	for (int i=0;i < m_NumFrames;i++)
	{
		for(int k=0;k < m_StrokeList[i].size();k++)
		{
			for (int n=0;n < strokeIds.size();n++)
			{
				if (m_StrokeList[i][k].getStrokeId() == strokeIds[n])
					m_StrokeList[i][k].getStroke()->properties.updateStrokeProperty(prop,val);
			}
		}

	}
}



void LayerObject::updatePoint(int frameIndex, const QPointF &point,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return;

	if ((boundaryContour == false) && (m_CurrentStroke>=0)&&(m_CurrentStroke<m_StrokeList[frameIndex].size()))
	{

		 m_StrokeList[frameIndex][m_CurrentStroke].updatePoint(point,selectedPointIndex,true);
		 m_StrokeList[frameIndex][m_CurrentStroke].recomputeStroke();
	}
	else if (boundaryContour == true) 
	{
		m_ContourLabelList[frameIndex].updatePoint(point,selectedPointIndex,true);
		m_IsBoundaryDistanceCreatedList[frameIndex]=false;
	}
	else return;

        sprintf(_strbuf, "Reading in layer: %s",m_LayerName.toAscii().data());
	LOG_STR_COUT(_strbuf);



	m_IsLocalLabelList[frameIndex]=true;
	m_IsGlobalLabelList[frameIndex]=true;

}

void LayerObject::propagateUpdatePoint(int frameIndex,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return;

	int firstFrame,lastFrame;
	bool IsForwardLabeled,IsBackwardLabeled;

	if (boundaryContour == true)
	{
		firstFrame=findFirstLocalLabeledFrame(frameIndex,selectedPointIndex,IsBackwardLabeled);
		lastFrame=findLastLocalLabeledFrame(frameIndex,selectedPointIndex,IsForwardLabeled);
		propagateUpdatePoint(frameIndex,lastFrame,IsForwardLabeled,firstFrame,IsBackwardLabeled);
		
	}
	else if ((boundaryContour == false) && (m_CurrentStroke>=0)&& (m_CurrentStroke < m_StrokeList[frameIndex].size()))
	{
		firstFrame=findContourFirstLocalLabeledFrame(m_StrokeList[frameIndex][m_CurrentStroke],frameIndex,selectedPointIndex,IsBackwardLabeled);
		lastFrame=findContourLastLocalLabeledFrame(m_StrokeList[frameIndex][m_CurrentStroke],frameIndex,selectedPointIndex,IsForwardLabeled);
		propagateContourUpdatePoint(m_StrokeList[frameIndex][m_CurrentStroke],frameIndex,selectedPointIndex, lastFrame,IsForwardLabeled,firstFrame,IsBackwardLabeled);
	}
}


//void LayerObject::propagateUpdatePoint(int frameIndex,int lastFrame,int firstFrame)
//{
//	if(InRange(frameIndex)==false)
//		return;
//	// propagate to the rest of the frames
//	double projection0,extension0;
//	m_ContourLabelList[frameIndex].computeTwoPointsCoeff(selectedPointIndex,projection0,extension0);
//
//	double projection1,extension1;
//	double ratio;
//	// backward
//	m_ContourLabelList[qMax(firstFrame,m_StartFrame)].computeTwoPointsCoeff(selectedPointIndex,projection1,extension1);
//	for(int i=firstFrame+1;i<frameIndex;i++)
//	{
//		ratio=(double)(i-firstFrame)/(frameIndex-firstFrame);
//		m_ContourLabelList[i].updatePointTwoPointsCoeff(selectedPointIndex,projection0,extension0,projection1,extension1,ratio);
//		m_ContourLabelList[i].computeCentroid();
//	}
//	// forward
//	m_ContourLabelList[qMin(lastFrame,m_EndFrame)].computeTwoPointsCoeff(selectedPointIndex,projection1,extension1);
//	for(int i=frameIndex+1;i<lastFrame;i++)
//	{
//		ratio=1-(double)(i-frameIndex)/(lastFrame-frameIndex);
//		m_ContourLabelList[i].updatePointTwoPointsCoeff(selectedPointIndex,projection0,extension0,projection1,extension1,ratio);
//		m_ContourLabelList[i].computeCentroid();
//	}
//}

void LayerObject::propagateUpdatePoint(int frameIndex,int lastFrame,bool IsForwardLabeled,int firstFrame,bool IsBackwardLabeled)
{
	if(InRange(frameIndex)==false)
		return;
	// propagate to the rest of the frames
	QPointF curPoint,endPoint;

	// backward
	for(int i=firstFrame+1;i<frameIndex;i++)
	{
		m_ContourLabelList[frameIndex].AffineFit(selectedPointIndex,m_ContourLabelList[i],curPoint);
		if(IsBackwardLabeled)
		{
			double ratio=(double)(i-firstFrame)/(frameIndex-firstFrame);
			m_ContourLabelList[firstFrame].AffineFit(selectedPointIndex,m_ContourLabelList[i],endPoint);
			curPoint=curPoint*ratio+endPoint*(1-ratio);
		}
		m_ContourLabelList[i].updatePoint(curPoint,selectedPointIndex,false);
		m_IsBoundaryDistanceCreatedList[i]=false;
		m_ContourLabelList[i].computeCentroid();
	}
	// forward
	for(int i=frameIndex+1;i<lastFrame;i++)
	{
		m_ContourLabelList[frameIndex].AffineFit(selectedPointIndex,m_ContourLabelList[i],curPoint);
		if(IsForwardLabeled)
		{
			double ratio=1-(double)(i-frameIndex)/(lastFrame-frameIndex);
			m_ContourLabelList[lastFrame].AffineFit(selectedPointIndex,m_ContourLabelList[i],endPoint);
			curPoint=curPoint*ratio+endPoint*(1-ratio);
		}
		m_ContourLabelList[i].updatePoint(curPoint,selectedPointIndex,false);
		m_IsBoundaryDistanceCreatedList[i]=false;
		m_ContourLabelList[i].computeCentroid();
		
	}
}

void LayerObject::propagateContourUpdatePoint(ContourLabel& contour,int frameIndex, int pointIndex, int lastFrame,bool IsForwardLabeled,int firstFrame,bool IsBackwardLabeled)
{


	if((InRange(frameIndex)==false) || (InRange(lastFrame-1)==false) || (InRange(firstFrame)==false))
		return;
	// propagate to the rest of the frames
	QPointF curPoint,endPoint;

     //   sprintf(_strbuf, "propagating point %i", pointIndex);
	//LOG_STR_COUT(_strbuf);


	QPointF shift(0,0);
	bool foundStroke = false;

	for (int i=0;i < m_StrokeListCopy.size();i++)
	{
		if (m_StrokeListCopy[i].getStrokeId() == contour.getStrokeId())
		{

			if ((contour.numPoints() != m_StrokeListCopy[i].numPoints()) || (contour.numPoints() <= pointIndex))
			{
            sprintf(_strbuf, "problem, updating point %i for contour with %i points, copy %i", pointIndex,  contour.numPoints(),m_StrokeListCopy[i].numPoints());
				LOG_STR_COUT(_strbuf);
				return;
			}

			shift = contour.getPoint(pointIndex) -m_StrokeListCopy[i].getPoint(pointIndex) ;
			foundStroke = true;

		}
	}
	if (foundStroke==false)
	{
                sprintf(_strbuf, "ERROR: Couldn't find previous stroke");
		LOG_STR_COUT(_strbuf);
		return;
	}

	ContourLabel *currLabel;
	// backward
	for(int i=firstFrame+1;i<frameIndex;i++)
	{		
		currLabel = contour.getPrev(frameIndex-i);

		double ratio=1;
		if(IsBackwardLabeled)
		{
			ratio=(double)(i-firstFrame)/(frameIndex-firstFrame);
		}
		curPoint=currLabel->getPoint(pointIndex) + shift*ratio;	
		currLabel->updatePoint(curPoint,pointIndex,false);
		//currLabel->recomputeStroke();
	}


	// forward
	for(int i=frameIndex+1;i<lastFrame;i++)
	{
		currLabel = contour.getNext(i-frameIndex);

		double ratio=1;
		if(IsForwardLabeled)
		{
			ratio=1-(double)(i-frameIndex)/(lastFrame-frameIndex);
		}

		curPoint=currLabel->getPoint(pointIndex) + shift*ratio;	
		currLabel->updatePoint(curPoint,pointIndex,false);
		//currLabel->recomputeStroke();

	}

}

bool LayerObject::boundaryImageCreated(int frameIndex)
{
	if ((frameIndex<0) ||(frameIndex>=m_IsBoundaryDistanceCreatedList.size()))
		return false; 
	else 
		return m_IsBoundaryDistanceCreatedList[frameIndex];
}
	


void LayerObject::propagateUpdatePointForward(int frameIndex, int lastFrame)
{
	if(InRange(frameIndex)==false)
		return;
	double projection0,extension0;
	m_ContourLabelList[frameIndex].computeTwoPointsCoeff(selectedPointIndex,projection0,extension0);

	double projection1,extension1;
	double ratio;
	
	m_ContourLabelList[qMin(lastFrame,m_EndFrame)].computeTwoPointsCoeff(selectedPointIndex,projection1,extension1);
	for(int i=frameIndex+1;i<lastFrame;i++)
	{
		ratio=1-(double)(i-frameIndex)/(lastFrame-frameIndex);
		m_ContourLabelList[i].updatePointTwoPointsCoeff(selectedPointIndex,projection0,extension0,projection1,extension1,ratio);
		m_ContourLabelList[i].computeCentroid();
	}
}


void LayerObject::propagateUpdatePointBackward(int frameIndex, int firstFrame)
{
	if(InRange(frameIndex)==false)
		return;
	double projection0,extension0;
	m_ContourLabelList[frameIndex].computeTwoPointsCoeff(selectedPointIndex,projection0,extension0);

	double projection1,extension1;
	double ratio;
	
	m_ContourLabelList[qMax(firstFrame,m_StartFrame)].computeTwoPointsCoeff(selectedPointIndex,projection1,extension1);
	for(int i=firstFrame+1;i<frameIndex;i++)
	{
		ratio=(double)(i-firstFrame)/(frameIndex-firstFrame);
		m_ContourLabelList[i].updatePointTwoPointsCoeff(selectedPointIndex,projection0,extension0,projection1,extension1,ratio);
		m_ContourLabelList[i].computeCentroid();
	}
}


//-------------------------------------------------------------------------------------------------------
// function to move the contour globally
//-------------------------------------------------------------------------------------------------------
void LayerObject::moveContour(int frameIndex, const QPointF &point,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return;

	if (boundaryContour == true)
		m_ContourLabelList[frameIndex].shiftContour(m_OrgContourLabel, point-m_OrgCursorPoint);
	else if ((boundaryContour == false) && (m_CurrentStroke>=0) && (m_CurrentStroke<m_StrokeList[frameIndex].size() ))
		m_StrokeList[frameIndex][m_CurrentStroke].shiftContour(m_OrgContourLabel, point-m_OrgCursorPoint);
	

	m_IsGlobalLabelList[frameIndex]=true;
}

//-------------------------------------------------------------------------------------------------------
// function to propagate the contour moving to the rest of the frames
//-------------------------------------------------------------------------------------------------------
void LayerObject::propagateMoveContour(int frameIndex, bool boundaryContour, bool selectedMultiple, QPointF shift, int actionId)
{
        //sprintf(_strbuf, "shift %f %f", shift.x(), shift.y());

	if(InRange(frameIndex)==false)
		return;
	int firstFrame,lastFrame;
	bool IsForwardLabel = false, IsBackwardLabel= false;

	if ((boundaryContour == false) && ((m_CurrentStroke>=0) || selectedMultiple == true))
	{
		saveChangedStrokes(frameIndex, actionId,true);

		QList<int> propList;

		if (selectedMultiple ==false)
			propList.push_back(m_CurrentStroke);
		else
		{
			for(int j=0;j<m_StrokeList[frameIndex].size();j++)
			{
			
				if (m_StrokeList[frameIndex][j].groupSelected())
				{
					propList.push_back(j);
				}
			}
		}

		for(int s = 0; s < propList.size();s++)
		{
			int currStroke = propList[s];

			int firstFrame = m_StartFrame;
			int lastFrame = m_EndFrame;

			//bool forward, backward;
			int first, last;
			for (int i=0;i < m_StrokeList[frameIndex][currStroke].numPoints();i++)
			{
				
				first = findContourFirstLocalLabeledFrame(m_StrokeList[frameIndex][currStroke],frameIndex,i,IsBackwardLabel);
				last = findContourLastLocalLabeledFrame(m_StrokeList[frameIndex][currStroke],frameIndex,i,IsForwardLabel );
		
				//if (backward ==true)
				//	IsBackwardLabel = true;
				//if (forward ==true)
				//	IsForwardLabel = true;

				if (first > firstFrame)
				{
					firstFrame = first;
				}
				if (last < lastFrame)
				{
					lastFrame = last;
				}
				
			}

			int strokeId = m_StrokeList[frameIndex][currStroke].getStrokeId();


			//QPointF shift=m_StrokeList[frameIndex][currStroke].centroid()-m_OrgContourLabel.centroid();
			double ratio;
			for(int i=firstFrame+1;i<frameIndex;i++)
			{
				
				//if(IsBackwardLabel)
				//{
					ratio=(double)(i-firstFrame)/(frameIndex-firstFrame);
					m_StrokeList[frameIndex][currStroke].getPrev(frameIndex-i)->shiftContour(shift*ratio);
					m_StrokeList[frameIndex][currStroke].getPrev(frameIndex-i)->getStroke()->forceRecompute();

					if(m_StrokeList[frameIndex][currStroke].getPrev(frameIndex-i)->isPolygon())
						m_StrokeList[frameIndex][currStroke].getPrev(frameIndex-i)->computeCentroid();
	
				//}
				//else
				//}
			}
			for(int i=frameIndex+1;i<lastFrame;i++)
			{
				if(IsForwardLabel)
				{
					ratio=1-(double)(i-frameIndex)/(lastFrame-frameIndex);
					m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->shiftContour(shift*ratio);
					m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->getStroke()->forceRecompute();
					if(m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->isPolygon())
						m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->computeCentroid();
				}
				else
				{
					QPointF newshift = m_StrokeList[frameIndex][currStroke].centroid() - m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->centroid();

					m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->shiftContour(newshift);

					m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->getStroke()->forceRecompute();
					if(m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->isPolygon())
						m_StrokeList[frameIndex][currStroke].getNext(i-frameIndex)->computeCentroid();
				}
			}

		}
	}
	else
	{



		firstFrame=findFirstGlobalLabeledFrame(frameIndex,IsForwardLabel);
		lastFrame=findLastGlobalLabeledFrame(frameIndex,IsBackwardLabel);

		QPointF shift=m_ContourLabelList[frameIndex].centroid()-m_OrgContourLabel.centroid();
		double ratio;
		for(int i=firstFrame+1;i<frameIndex;i++)
		{
			if(IsForwardLabel)
			{
				ratio=(double)(i-firstFrame)/(frameIndex-firstFrame);
				m_ContourLabelList[i].shiftContour(shift*ratio);
				if(m_ContourLabelList[i].isPolygon())
					m_ContourLabelList[i].computeCentroid();
			}
			else
				m_ContourLabelList[i]=m_ContourLabelList[frameIndex];
		}
		for(int i=frameIndex+1;i<lastFrame;i++)
		{
			if(IsBackwardLabel)
			{
				ratio=1-(double)(i-frameIndex)/(lastFrame-frameIndex);
				m_ContourLabelList[i].shiftContour(shift*ratio);
				if(m_ContourLabelList[i].isPolygon())
					m_ContourLabelList[i].computeCentroid();
			}
			else
				m_ContourLabelList[i]=m_ContourLabelList[frameIndex];
		}

	}
	

}

//-------------------------------------------------------------------------------------------------------
// function to scale the contour globally
//-------------------------------------------------------------------------------------------------------
void LayerObject::scaleContour(int frameIndex,const QPointF& point,int selectedBoundingBoxCorner,bool boundaryContour)
{
	if(InRange(frameIndex)==false)
		return;

	if (boundaryContour == true)
		m_ContourLabelList[frameIndex].scaleContour(m_OrgContourLabel,point-m_OrgCursorPoint,selectedBoundingBoxCorner);
	else if ((boundaryContour == false) && (m_CurrentStroke>=0)&& (m_CurrentStroke < m_StrokeList[frameIndex].size()))
		m_StrokeList[frameIndex][m_CurrentStroke].scaleContour(m_OrgContourLabel,point-m_OrgCursorPoint,selectedBoundingBoxCorner);
	else
		return;

	m_IsGlobalLabelList[frameIndex]=true;
}

void LayerObject::propagateScaleContour(int frameIndex)
{
	if(InRange(frameIndex)==false)
		return;
	int firstFrame,lastFrame;
	bool IsForwardLabel, IsBackwardLabel;
	firstFrame=findFirstGlobalLabeledFrame(frameIndex,IsForwardLabel);
	lastFrame=findLastGlobalLabeledFrame(frameIndex,IsBackwardLabel);

	double rate,ratio;
	rate=m_ContourLabelList[frameIndex].scale()/m_OrgContourLabel.scale();

	for(int i=firstFrame+1;i<frameIndex;i++)
	{
		if(IsForwardLabel)
		{
			ratio=(double)(i-firstFrame)/(frameIndex-firstFrame);
			if(m_ContourLabelList[i].isPolygon())
			{
				m_IsBoundaryDistanceCreatedList[i]=false;
				m_ContourLabelList[i].scaleContour((rate-1)*ratio+1);
				m_ContourLabelList[i].computeCentroid();
			}
			else
				m_ContourLabelList[i].scaleBoundingBox(ratio,m_ContourLabelList[frameIndex],m_ContourLabelList[firstFrame]);
		}
		else
			m_ContourLabelList[i]=m_ContourLabelList[frameIndex];
	}
	for(int i=frameIndex+1;i<lastFrame;i++)
	{
		if(IsBackwardLabel)
		{
			ratio=1-(double)(i-frameIndex)/(lastFrame-frameIndex);
			if(m_ContourLabelList[i].isPolygon())
			{
				m_IsBoundaryDistanceCreatedList[i]=false;
				m_ContourLabelList[i].scaleContour((rate-1)*ratio+1);
				m_ContourLabelList[i].computeCentroid();
			}
			else
				m_ContourLabelList[i].scaleBoundingBox(ratio,m_ContourLabelList[frameIndex],m_ContourLabelList[lastFrame]);
		}
		else
			m_ContourLabelList[i]=m_ContourLabelList[frameIndex];
	}
}

void LayerObject::removePoint(int curFrameIndex, bool boundaryContour)
{
	if (boundaryContour)
		for(int i=m_StartFrame;i<=m_EndFrame;i++)
			m_ContourLabelList[i].removePoint(selectedPointIndex);
	else if ((m_CurrentStroke>=0)&& (m_CurrentStroke < m_StrokeList[curFrameIndex].size()))
	{
		m_StrokeList[curFrameIndex][m_CurrentStroke].removePoint(selectedPointIndex);

		ContourLabel *nextLabel = m_StrokeList[curFrameIndex][m_CurrentStroke].getNext();
		while(nextLabel!=0)
		{
			nextLabel->removePoint(selectedPointIndex);
			nextLabel = nextLabel->getNext();
		}
		ContourLabel *prevLabel = m_StrokeList[curFrameIndex][m_CurrentStroke].getPrev();
		while(prevLabel!=0)
		{
			prevLabel->removePoint(selectedPointIndex);
			prevLabel = prevLabel->getPrev();
		}	
	}
			
}

void LayerObject::removeLabel(int frameIndex)
{
	m_ContourLabelList[frameIndex].removeLabel(selectedPointIndex);
}

void LayerObject::FormLayerDepth(int frameIndex,LayerDepth &layerDepth)
{
	QList<int> indexList;
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(m_IsDepthLabeledList[i]==true || i==frameIndex)
			indexList.append(i);
	m_IsDepthLabeledList[frameIndex]=true;
	layerDepth.Initialize(m_DepthList.size(),m_StartFrame,m_EndFrame,m_DepthList,indexList);
}

void LayerObject::SetLayerDepth(const LayerDepth& layerDepth)
{
	//layerDepth.printDepth();
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		m_DepthList[i]=layerDepth.depth(i);
	//printLayerDepth();
}

void LayerObject::printLayerDepth()
{
	std::cout<<std::endl;
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		std::cout<<m_DepthList[i]<<std::endl;
}

//-------------------------------------------------------------------------------------------------------------------
// file IO, to write xml file of the object
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::writeXML(XmlWriter &xw,int id)
{
	xw.writeOpenTag("object");
		xw.writeTaggedString("name",m_LayerName);
		xw.writeTaggedString("deleted","0");
		xw.writeTaggedString("verified","0");
		xw.writeDate();
		xw.writeTaggedString("id",QString::number(id));
		xw.writeTaggedString("username","anonymous");
		xw.writeTaggedString("numFrames",QString::number(m_NumFrames));
		xw.writeTaggedString("startFrame",QString::number(m_StartFrame));
		xw.writeTaggedString("endFrame",QString::number(m_EndFrame));
		xw.writeTaggedString("createdFrame",QString::number(m_CreatedFrame));
		xw.writeOpenTag("labels");
			for(int i=m_StartFrame;i<=m_EndFrame;i++)
			{
				xw.writeOpenTag("frame");
					xw.writeTaggedString("index",QString::number(i));
					xw.writeTaggedString("depth",QString::number(m_DepthList[i]));
					xw.writeTaggedString("globalLabel",QString::number(m_IsGlobalLabelList[i]));
					xw.writeTaggedString("localLabel",QString::number(m_IsLocalLabelList[i]));
					xw.writeTaggedString("tracked", QString::number(m_IsTrackedList[i]));
					xw.writeTaggedString("depthLabel",QString::number(m_IsDepthLabeledList[i]));
					xw.writeTaggedString("isPolygon",QString::number(m_ContourLabelList[i].isPolygon()));

				
					if(m_ContourLabelList[i].isPolygon()) // output contour
					{
						xw.writeOpenTag("polygon");
							for(int j=0;j<m_ContourLabelList[i].size();j++)
							{
								xw.writeOpenTag("pt");
									xw.writeTaggedString("x",QString::number(m_ContourLabelList[i].x(j)));
									xw.writeTaggedString("y",QString::number(m_ContourLabelList[i].y(j)));
									xw.writeTaggedString("labeled",QString::number(m_ContourLabelList[i].labeled(j)));
								xw.writeCloseTag("pt");
							}
						xw.writeCloseTag("polygon");
					}
					else // output bounding box
					{
						xw.writeOpenTag("boundingBox");
							QRectF boundingBox=m_ContourLabelList[i].boundingBox();
							xw.writeOpenTag("topLeft");
								xw.writeTaggedString("x",QString::number(boundingBox.topLeft().x()));
								xw.writeTaggedString("y",QString::number(boundingBox.topLeft().y()));
							xw.writeCloseTag("topLeft");
							xw.writeOpenTag("bottomRight");
								xw.writeTaggedString("x",QString::number(boundingBox.bottomRight().x()));
								xw.writeTaggedString("y",QString::number(boundingBox.bottomRight().y()));
							xw.writeCloseTag("bottomRight");
						xw.writeCloseTag("boundingBox");
					}
					
				xw.writeCloseTag("frame");
			}
		xw.writeCloseTag("labels");

	xw.writeCloseTag("object");
}

//-------------------------------------------------------------------------------------------------------------------
// function to cut the labeling
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::cutLabeling(int frameIndex)
{
	if(frameIndex>m_CreatedFrame)
	{
		if(frameIndex>m_EndFrame)
			return;
		m_EndFrame=frameIndex;
		for(int i=m_EndFrame+1;i<m_NumFrames;i++)
		{
			m_IsGlobalLabelList[i]=false;
			m_IsLocalLabelList[i]=false;
			m_IsTrackedList[i]=false;
			m_IsDepthLabeledList[i]=false;
		}
		return;
	}
	if(frameIndex<m_CreatedFrame)
	{
		if(frameIndex<m_StartFrame)
			return;
		m_StartFrame=frameIndex;
		for(int i=0;i<m_StartFrame;i++)
		{
			m_IsGlobalLabelList[i]=false;
			m_IsLocalLabelList[i]=false;
			m_IsTrackedList[i]=false;
			m_IsDepthLabeledList[i]=false;
		}
	}
}

void LayerObject::cutForward(int frameIndex)
{
	if(frameIndex<m_StartFrame)
		return;
	m_StartFrame=frameIndex;
	m_IsGlobalLabelList[frameIndex]=true;
	for(int i=0;i<m_StartFrame;i++)
	{
		m_IsGlobalLabelList[i]=false;
		m_IsLocalLabelList[i]=false;
		m_IsTrackedList[i]=false;
		m_IsDepthLabeledList[i]=false;
	}
}

void LayerObject::cutBackward(int frameIndex)
{
	if(frameIndex>m_EndFrame)
		return;
	m_EndFrame=frameIndex;
	m_IsGlobalLabelList[frameIndex]=true;
	for(int i=m_EndFrame+1;i<m_NumFrames;i++)
	{
		m_IsGlobalLabelList[i]=false;
		m_IsLocalLabelList[i]=false;
		m_IsTrackedList[i]=false;
		m_IsDepthLabeledList[i]=false;
	}
}

//-------------------------------------------------------------------------------------------------------------------
// function to remove all the tracking
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::removeAllTracking()
{
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		if(i==m_CreatedFrame)
			continue;
		m_IsGlobalLabelList[i]=false;
		m_IsLocalLabelList[i]=false;
		m_IsTrackedList[i]=false;
		m_IsDepthLabeledList[i]=false;
	}
	m_StartFrame=m_CreatedFrame;
	m_EndFrame=m_CreatedFrame;
}

//-------------------------------------------------------------------------------------------------------------------
// function to enforce the current frame to be a key frame
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::enforceKeyFrame(int frameIndex, bool boundaryContour)
{

	if (boundaryContour == true)
	{
		if(frameIndex>=m_StartFrame && frameIndex<=m_EndFrame)
			m_ContourLabelList[frameIndex].setUniformLabels(true);
	}
	else if ((boundaryContour == false) && (m_CurrentStroke>=0)&& (m_CurrentStroke < m_StrokeList[frameIndex].size()))
	{
		if(frameIndex>=m_StartFrame && frameIndex<=m_EndFrame)
			m_StrokeList[frameIndex][m_CurrentStroke].setUniformLabels(true);
	}
}

void LayerObject::removeUserLabels(int frameIndex)
{
	if(frameIndex>=m_StartFrame && frameIndex<=m_EndFrame)
		m_ContourLabelList[frameIndex].setUniformLabels(false);
}

void LayerObject::extendDepth()
{
	for(int i=0;i<m_StartFrame;i++)
		m_DepthList[i]=m_DepthList[m_StartFrame];
	for(int i=m_EndFrame+1;i<m_NumFrames;i++)
		m_DepthList[i]=m_DepthList[m_EndFrame];
}


//-------------------------------------------------------------------------------------------------------------------
// functions to handle tracking
//-------------------------------------------------------------------------------------------------------------------
bool LayerObject::IsForwardTrackable(int frameIndex)
{
	if(frameIndex==m_EndFrame && m_EndFrame<m_NumFrames-1)
		return true;
	else
		return false;

	//if(frameIndex==m_NumFrames-1)
	//	return false;
	//if(m_IsTrackedList[frameIndex]==true && m_IsTrackedList[frameIndex+1]==false)
	//	return true;
	//else
	//	return false;
}

bool LayerObject::IsBackwardTrackable(int frameIndex)
{
	if(frameIndex==m_StartFrame && m_StartFrame>0)
		return true;
	else
		return false;
	//if(frameIndex==0)
	//	return false;
	//if(m_IsTrackedList[frameIndex]==true && m_IsTrackedList[frameIndex-1]==false)
	//	return true;
	//else
	//	return false;
}

void LayerObject::TrackToFrame(int frameIndex, const QList<QPointF> &contour)
{
	if(m_StartFrame>frameIndex)
		m_StartFrame=frameIndex;
	if(m_EndFrame<frameIndex)
		m_EndFrame=frameIndex;
	m_ContourLabelList[frameIndex].loadPoints(contour);
	m_IsTrackedList[frameIndex]=true;
}

//-------------------------------------------------------------------------------------------------------------------
// function to handle layer mask
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::CreateMask(int frameIndex,int width,int height)
{
	if(frameIndex<startFrame() || frameIndex>endFrame())
		return;
	m_LayerMaskList[frameIndex].allocate(width,height);
	m_LayerMaskList[frameIndex].setAllZeros();
	if(m_ContourLabelList[frameIndex].isPolygon()) // the contour is defined as a polygon
		m_LayerMaskList[frameIndex].inpolygon(m_ContourLabelList[frameIndex].PointList());
	else // the contour is defined as a bounding box
	{
		QRectF rect=m_ContourLabelList[frameIndex].boundingBox();
		QList<QPointF> pointList;
		pointList.append(rect.topLeft());
		pointList.append(rect.topRight());
		pointList.append(rect.bottomRight());
		pointList.append(rect.bottomLeft());
		m_LayerMaskList[frameIndex].inpolygon(pointList);
	}

	QRect rect=m_LayerMaskList[frameIndex].boundingBox();

	// adjust the bounding box
	rect.adjust(-10,-10,10,10);
	if(rect.left()<0)
		rect.setLeft(0);
	if(rect.right()>width-1)
		rect.setRight(width-1);
	if(rect.top()<0)
		rect.setTop(0);
	if(rect.bottom()>height-1)
		rect.setBottom(height-1);

	m_FullBoundingBoxList[frameIndex]=rect;

	
}


//-------------------------------------------------------------------------------------------------------------------
// function to create the visible mask
// this function must be called after calling CreateMask
// the visible mask is 1 if mask is 1 and omask is 0
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::CreateVisibleMask(int frameIndex, const BinaryImage& OMask)
{
	if(frameIndex<startFrame() || frameIndex>endFrame())
		return;
	m_VisibleLayerMaskList[frameIndex]=m_LayerMaskList[frameIndex];
	m_VisibleLayerMaskList[frameIndex].subtract(OMask);
	QRect rect=m_VisibleLayerMaskList[frameIndex].boundingBox();
	if(!rect.isEmpty()) // if the bounding box is not empty, or there are visible pixels
	{ 
		int width,height;
		width=m_LayerMaskList[frameIndex].width();
		height=m_LayerMaskList[frameIndex].height();

		// adjust the bounding box
		rect.adjust(-5,-5,5,5);
		if(rect.left()<0)
			rect.setLeft(0);
		if(rect.right()>width-1)
			rect.setRight(width-1);
		if(rect.top()<0)
			rect.setTop(0);
		if(rect.bottom()>height-1)
			rect.setBottom(height-1);
	}
	m_BoundingBoxList[frameIndex]=rect;
}

//-------------------------------------------------------------------------------------------------------------------
// function to create the stroke image for this layer to store the system generated strokes
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::CreateStrokeImage(int frameIndex)
{
	if(frameIndex<startFrame() || frameIndex>endFrame())
		return;

	//if ((m_FullBoundingBoxList[frameIndex].width() >=512) || (m_FullBoundingBoxList[frameIndex].height() >=512))
	//	return;

	QImage img(m_FullBoundingBoxList[frameIndex].width(),m_FullBoundingBoxList[frameIndex].height(),QImage::Format_ARGB32_Premultiplied);
	img.fill(0);
	m_StrokeImageList[frameIndex] = img;
	m_StrokeImageStateList[frameIndex] = NO_IMAGE;

        //sprintf(_strbuf, "created stroke image size %i %i", m_BoundingBoxList[frameIndex].width(), m_BoundingBoxList[frameIndex].height());
	//img.save(QString("images/")+m_LayerName+QString::number(frameIndex)+QString("-initstrokeimage.jpg"));

}


//-------------------------------------------------------------------------------------------------------------------
// function to create the boundary distance image for this layer
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::CreateBoundaryDistanceImage(int frameIndex,int width,int height)
{
	if(frameIndex<startFrame() || frameIndex>endFrame())
		return;

	sprintf(_strbuf, "Creating boundary distance for layer: %s at frame %i", m_LayerName.toAscii().data(),frameIndex);
	LOG_STR_COUT(_strbuf);

	int top=m_FullBoundingBoxList[frameIndex].top();
	int left=m_FullBoundingBoxList[frameIndex].left();
	int bottom=m_FullBoundingBoxList[frameIndex].bottom();
	int right=m_FullBoundingBoxList[frameIndex].right();

	if ((right <= left) || (bottom <= top))
		return;

	DImage distImg(right-left+1,bottom-top+1);

	image<uchar> *boundary = new image<uchar>(right-left+1,bottom-top+1);

	for(int x=left;x<= right;x++)
	for(int y=top;y<= bottom;y++)
	{
		//If this is a pixel in the mask, search the surrounding area
		//to see if this is a boundary pixel
		if (m_LayerMaskList[frameIndex].getPixel(x,y) == 1)
		{
			for(int i=x-1;i<=x+1;i++)
			for(int j=y-1;j<=y+1;j++)
			{
				//If the surrounding pixel is out of the image, or not part
				//of the mask then we include it as a boundary point
				if ((i < 0) || (j < 0)  || (i >= width) || (j >= height) ||
					(m_LayerMaskList[frameIndex].getPixel(i,j) == 0))
				{
                                        //sprintf(_strbuf, "writing to %i %i", j,i);
					boundary->access[y-top][x-left] = 1;
				}
			}
		}
	}

	//Call the distance transform
	image<float> *distance = dt(boundary);

	for(int x=0;x< right-left+1;x++)
	for(int y=0;y< bottom-top+1;y++)
	{
		distImg.setPixelValue(sqrt(imRef(distance, x, y)),x,y);
	}

	distImg.GaussianSmoothing(distImg,2,6);
	
	DImage boundaryDx, boundaryDy;
	distImg.dx(boundaryDx,true);
	distImg.dy(boundaryDy,true);

	//boundaryDx.Multiplywith(100);
	//boundaryDy.Multiplywith(100);
	//boundaryDx.imwrite(m_ProjectDir+QString("boundx-")+QString::number(frameIndex)+QString(".png"));
	//boundaryDy.imwrite(m_ProjectDir+QString("boundy-")+QString::number(frameIndex)+QString(".png"));

	m_BoundaryDistanceDxImageList[frameIndex].fromDoublePrecision(boundaryDx);
	m_BoundaryDistanceDyImageList[frameIndex].fromDoublePrecision(boundaryDy);
	m_BoundaryDistanceImageList[frameIndex].fromDoublePrecision(distImg);
	
	m_IsBoundaryDistanceCreatedList[frameIndex] = true;

	delete boundary;
	delete distance;

}


void LayerObject::clearBoundaryDistanceImages(){
	
	for (int i=0;i < m_IsBoundaryDistanceCreatedList.size();i++)
	{
		if (m_IsBoundaryDistanceCreatedList[i]==true)
		{
			m_BoundaryDistanceImageList[i]= IntegerImage();
			m_BoundaryDistanceDxImageList[i]= IntegerImage();
			m_BoundaryDistanceDyImageList[i]= IntegerImage();
		}
		m_IsBoundaryDistanceCreatedList[i]=false;
	}

}


bool LayerObject::getBoundaryFlow(bool forward, int frameIndex, float x, float y, float *flowx, float *flowy)
{
	if (((forward==true) && (frameIndex<startFrame() || frameIndex>=endFrame())) ||
		((forward==false) && (frameIndex<=startFrame() || frameIndex>endFrame())))
		return false;

	QList<QPointF> curPts = m_ContourLabelList[frameIndex].PointList();

	QList<QPointF> nextPts;
	if (forward == true)
		nextPts = m_ContourLabelList[frameIndex+1].PointList();
	else
		nextPts = m_ContourLabelList[frameIndex-1].PointList();


	Vec2f boundaryDir(0,0);
	Vec2f currPos(x,y);

	double  weightSum = 0;
	
	//get the sum of all distances to all the boundary points
	for (int j=0; j < curPts.size();j++)
	{
		Vec2f pos(curPts[j].x(), curPts[j].y());
		double distance = currPos.distanceTo(pos);
		weightSum +=exp(-0.1*distance);
	}

	//use the distances to compute a weighted average of the displacement
	for (int j=0; j < curPts.size();j++)
	{
		Vec2f pos(curPts[j].x(), curPts[j].y());

		double distance = currPos.distanceTo(pos);

		double weight = exp(-0.1*distance);

		Vec2f displacement(nextPts[j].x() - curPts[j].x(), nextPts[j].y() - curPts[j].y());

		displacement *= (float) (weight/weightSum);
		boundaryDir += displacement;
	}

	*flowx = boundaryDir.x();
	*flowy = boundaryDir.y();

	return true;
}


bool LayerObject::getUserStrokeFlow(bool forward, int frameIndex, float x, float y,QVector<Vec2f>& curPts,QVector<Vec2f>& disp, float *flowx, float *flowy)
{

	if (disp.size()==0)
	{
		*flowx = 0;
		*flowy = 0;
		return false;
	}


	Vec2f dir(0,0);
	float minUserDist =0;

	Vec2f currPos(x,y);
        //sprintf(_strbuf, "pos %f %f", currPos.x(), currPos.y());

	double  distSum = 0;
	
	//get the sum of all distances to all the stroke points
	for (int j=0; j < curPts.size();j++)
	{

		double distance = currPos.distanceTo(curPts[j]);
		if ( distance== 0)
			continue;

		if (1.0/distance > minUserDist)
			minUserDist = 1.0/distance;

		double dist = exp(-0.005*distance);
		Vec2f& displacement = disp[j];
		if (m_LayerMaskList[frameIndex].getPixel(x+displacement.x(),y+displacement.y()) == 0)
		{
			dist = dist / 1000.;
			//continue;
		}

		distSum +=dist;
	}

	//use the distances to compute a weighted average of the displacement
	for (int j=0; j < curPts.size();j++)
	{

		
		double distance = currPos.distanceTo(curPts[j]);
		if ( distance== 0)
			continue;
		double dist = exp(-0.005*distance);
                //sprintf(_strbuf, "dist %f", dist);
		Vec2f displacement = disp[j];
                //sprintf(_strbuf, "\tdisplacement %f %f", displacement.x(), displacement.y());

		if (m_LayerMaskList[frameIndex].getPixel(x+displacement.x(),y+displacement.y()) == 0)
		{
			dist = dist / 1000.;
			//continue;
		}

		displacement *= (float) (dist/distSum);
                //sprintf(_strbuf, "\tdisplacement2 %f %f", displacement.x(), displacement.y());
		dir += displacement;
                //sprintf(_strbuf, "\tweightedDisplacement %f %f", userDir.x(), userDir.y());
	}


	*flowx = dir.x();
	*flowy = dir.y();
	return true;
}

/*

bool LayerObject::getUserStrokeFlow2(bool forward, int frameIndex, float x, float y, float *flowx, float *flowy)
{
	if (((forward==true) && (frameIndex<startFrame() || frameIndex>=endFrame())) ||
		((forward==false) && (frameIndex<=startFrame() || frameIndex>endFrame())))
		return false;


	QVector<Vec2f> curPts;
	QVector<Vec2f> disp;


	for (int k=0;k < m_StrokeList[frameIndex].size();k++)
	{
	
		QList<QPointF> pts = m_StrokeList[frameIndex][k].PointList();

		ContourLabel *nextStroke;
		if (forward == true)
			nextStroke= m_StrokeList[frameIndex][k].getNext();
		else
			nextStroke= m_StrokeList[frameIndex][k].getPrev();

		if (nextStroke == 0)
			continue;


		QList<QPointF> nextPts = nextStroke->PointList();

		assert(nextPts.size() == pts.size());

		for(int n=0;n < pts.size();n++)
		{
			curPts.push_back(Vec2f(pts[n].x(), pts[n].y()));
			disp.push_back(Vec2f(nextPts[n].x()-pts[n].x(), nextPts[n].y()-pts[n].y()));
		}
	}


	QList<QPointF> boundPts = m_ContourLabelList[frameIndex].PointList();
	QList<QPointF> nextPts;
	if (forward == true)
		nextPts = m_ContourLabelList[frameIndex+1].PointList();
	else
		nextPts = m_ContourLabelList[frameIndex-1].PointList();

	
	//get the sum of all distances to all the boundary points
	//for (int n=0; n < boundPts.size();n++)
	//{

	//	curPts.push_back(Vec2f(boundPts[n].x(), boundPts[n].y()));
	//	disp.push_back(Vec2f(nextPts[n].x()-boundPts[n].x(), nextPts[n].y()-boundPts[n].y()));
	//}
	


	Vec2f dir(0,0);
	float minUserDist =0;

	Vec2f currPos(x,y);
        //sprintf(_strbuf, "pos %f %f", currPos.x(), currPos.y());

	double  distSum = 0;
	
	//get the sum of all distances to all the stroke points
	for (int j=0; j < curPts.size();j++)
	{

		double distance = currPos.distanceTo(curPts[j]);
		if ( distance== 0)
			continue;

		if (1.0/distance > minUserDist)
			minUserDist = 1.0/distance;

		double dist = exp(-0.005*distance);
		Vec2f& displacement = disp[j];
		if (m_LayerMaskList[frameIndex].getPixel(x+displacement.x(),y+displacement.y()) == 0)
		{
			dist = dist / 1000.;
			//continue;
		}

		distSum +=dist;
	}

	//use the distances to compute a weighted average of the displacement
	for (int j=0; j < curPts.size();j++)
	{

		
		double distance = currPos.distanceTo(curPts[j]);
		if ( distance== 0)
			continue;
		double dist = exp(-0.005*distance);
                //sprintf(_strbuf, "dist %f", dist);
		Vec2f displacement = disp[j];
                //sprintf(_strbuf, "\tdisplacement %f %f", displacement.x(), displacement.y());

		if (m_LayerMaskList[frameIndex].getPixel(x+displacement.x(),y+displacement.y()) == 0)
		{
			dist = dist / 1000.;
			//continue;
		}

		displacement *= (float) (dist/distSum);
                //sprintf(_strbuf, "\tdisplacement2 %f %f", displacement.x(), displacement.y());
		dir += displacement;
                //sprintf(_strbuf, "\tweightedDisplacement %f %f", userDir.x(), userDir.y());
	}


	*flowx = dir.x();
	*flowy = dir.y();



	return true;
}
*/





/*

//-------------------------------------------------------------------------------------------------------------------
// function to create the user stroke flow image for this layer
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::CreateUserStrokeFlow(int frameIndex)
{
	if(frameIndex<startFrame() || frameIndex>=endFrame())
		return;

	
	QList<Vec2f> curPts;
	QList<Vec2f> disp;

	for (int i=0;i < m_StrokeList[frameIndex].size();i++)
	{
		if (m_StrokeList[frameIndex][i].IsGuideStroke())
		{
			QList<QPointF> pts = m_StrokeList[frameIndex][i].PointList();

			ContourLabel *nextStroke = m_StrokeList[frameIndex][i].getNext();
			if (nextStroke == 0)
				continue;

			QList<QPointF> nextPts = nextStroke->PointList();

			assert(nextPts.size() == pts.size());

			for(int j=0;j < pts.size();j++)
			{
				curPts.push_back(Vec2f(pts[j].x(), pts[j].y()));
				disp.push_back(Vec2f(nextPts[j].x()-pts[j].x(), nextPts[j].y()-pts[j].y()));
			}
		}
	}

	if (curPts.size() == 0)
	{
                //sprintf(_strbuf, "No user strokes found");
		return;
	}
	cout << "Creating user stroke flow for layer: "<< m_LayerName.toAscii().data() <<" at frame "<<frameIndex<< std::endl;


	int top=m_FullBoundingBoxList[frameIndex].top();
	int left=m_FullBoundingBoxList[frameIndex].left();
	int bottom=m_FullBoundingBoxList[frameIndex].bottom();
	int right=m_FullBoundingBoxList[frameIndex].right();

	DImage vx(right-left+1,bottom-top+1),vy(right-left+1,bottom-top+1);


	for(int x=left;x< right;x+=2)
	for(int y=top;y< bottom;y+=2)
	{

		if (m_LayerMaskList[frameIndex].getPixel(x,y) == 1)
		{

			Vec2f dir(0,0);
			float minUserDist =0;
		
			Vec2f currPos(x,y);
                        //sprintf(_strbuf, "pos %f %f", currPos.x(), currPos.y());

			double  distSum = 0;
			
			//get the sum of all distances to all the stroke points
			for (int j=0; j < curPts.size();j++)
			{

				double distance = currPos.distanceTo(curPts[j]);
				if ( distance== 0)
					continue;

				if (1.0/distance > minUserDist)
					minUserDist = 1.0/distance;

				double dist = exp(-0.1*distance);
				distSum +=dist;
			}

			//use the distances to compute a weighted average of the displacement
			for (int j=0; j < curPts.size();j++)
			{

				
				double distance = currPos.distanceTo(curPts[j]);
				if ( distance== 0)
					continue;
				double dist = exp(-0.1*distance);
                                //sprintf(_strbuf, "dist %f", dist);
				Vec2f displacement = disp[j];
                                //sprintf(_strbuf, "\tdisplacement %f %f", displacement.x(), displacement.y());
				displacement *= (float) (dist/distSum);
                                //sprintf(_strbuf, "\tdisplacement2 %f %f", displacement.x(), displacement.y());
				dir += displacement;
                                //sprintf(_strbuf, "\tweightedDisplacement %f %f", userDir.x(), userDir.y());
			}
			vx.setPixelValue(dir.x(),x-left,y-top);
			vy.setPixelValue(dir.y(),x-left,y-top);

                        //sprintf(_strbuf, "%f %f", boundaryDir.x(), boundaryDir.y());
		}
	}

	for(int x=1;x< vx.width()-1;x++)
	for(int y=1;y< vx.height()-1;y++)
	{
		
		if (vx.val(x,y) == 0)
		{

                        //sprintf(_strbuf, "1- xy %i %i: %f", x,y,vx.val(x,y));

			if ((vx.val(x-1,y) != 0) && (vx.val(x+1,y) != 0))
			{
				vx.setPixelValue((vx.val(x+1,y)+vx.val(x-1,y))/2.,x,y);
				vy.setPixelValue((vy.val(x+1,y)+vy.val(x-1,y))/2.,x,y);
			}
			else if ((vx.val(x,y-1) != 0) && (vx.val(x,y+1) != 0))
			{
				vx.setPixelValue((vx.val(x,y-1)+vx.val(x,y+1))/2.,x,y);
				vy.setPixelValue((vy.val(x,y-1)+vy.val(x,y+1))/2.,x,y);
			}
			else if ((vx.val(x-1,y-1) != 0) && (vx.val(x+1,y+1) != 0))
			{
				vx.setPixelValue((vx.val(x-1,y-1)+vx.val(x-1,y+1)+vx.val(x+1,y+1)+vx.val(x+1,y-1))/4.,x,y);
				vy.setPixelValue((vy.val(x-1,y-1)+vy.val(x-1,y+1)+vy.val(x+1,y+1)+vy.val(x+1,y-1))/4.,x,y);
			}
                        //sprintf(_strbuf, "xy %i %i: %f %f", x,y,vx.val(x,y),vy.val(x,y));
		}
	}
	vx.imwrite(QString("images/vx.jpg"),-50,50);
	vy.imwrite(QString("images/vy.jpg"),-50,50);

	FlowField m_FlowField;
	m_FlowField.fromDoublePrecision(vx,vy);
	m_FlowField.left()=left;
	m_FlowField.top()=top;
	m_FlowField.right()=right;
	m_FlowField.bottom()=bottom;

	m_UserStrokeFlowList[frameIndex] = m_FlowField;
	m_IsUserStrokeFlowCreatedList[frameIndex] = true;

}
*/


// function to judge whether a layer is visible
bool LayerObject::IsLayerVisible(int frameIndex)
{
	if(m_BoundingBoxList[frameIndex].isEmpty())
		return false;
	// count the number of pixels
	if(m_VisibleLayerMaskList[frameIndex].sum()<100)
		return false;
	return true;
}

bool LayerObject::IsWithinLayer(int frameIndex,const QPointF& point, bool visibleMask)
{

	if((visibleMask==false) && (m_LayerMaskList[frameIndex].getPixel(point.x(),point.y()))) // if the layer mask is 1
		return true;
	if((visibleMask==true) && (m_VisibleLayerMaskList[frameIndex].getPixel(point.x(),point.y()))) // if the layer mask is 1
		return true;
	else 
		return false;

}


void LayerObject::clearMotionAnalysis()
{
	QVector<Motion> layerMotionVector(m_NumFrames);
	m_LayerMotionList=QList<Motion>::fromVector(layerMotionVector);

	QVector<Motion> layerReverseMotionVector(m_NumFrames);
	m_LayerReverseMotionList=QList<Motion>::fromVector(layerReverseMotionVector);

}

//-------------------------------------------------------------------------------------------------------------------
// functions for motion analysis
//-------------------------------------------------------------------------------------------------------------------
void LayerObject::MotionAnalysis(int frameIndex, const DImage &image1, const DImage &image2,int& total,QProgressDialog* progressDialog,time_t* start,int sum)
{
	if(frameIndex<startFrame() || frameIndex>endFrame()-1)
		return;
	

	DImage patch1,patch2;
	DImage mask1,mask2,Mask1,Mask2;
	int top,left,bottom,right;

	if(m_BoundingBoxList[frameIndex].isEmpty()) // if the layer is outside the image boundary, no need to compute flow
		return;

	// get the bounding box that incorporate the whole
	top=__min(m_BoundingBoxList[frameIndex].top(),m_BoundingBoxList[frameIndex+1].top());
	left=__min(m_BoundingBoxList[frameIndex].left(),m_BoundingBoxList[frameIndex+1].left());
	bottom=__max(m_BoundingBoxList[frameIndex].bottom(),m_BoundingBoxList[frameIndex+1].bottom());
	right=__max(m_BoundingBoxList[frameIndex].right(),m_BoundingBoxList[frameIndex+1].right());

	if(progressDialog!=NULL)
	{
		// crop the images
		image1.crop(patch1,left,top,right-left+1,bottom-top+1);
		image2.crop(patch2,left,top,right-left+1,bottom-top+1);

		// crop the mask
		m_VisibleLayerMaskList[frameIndex].export2Image(mask1);
		m_VisibleLayerMaskList[frameIndex+1].export2Image(mask2);
		mask1.crop(Mask1,left,top,right-left+1,bottom-top+1);
		mask2.crop(Mask2,left,top,right-left+1,bottom-top+1);
	}
	// loop over all the choices of the motion parameters
	time_t now;
	for(int i=0;i<m_LayerMotionPara.numFlowPara();i++)
	{
		if(!m_LayerMotionPara.flowParaList()[i].IsActive()) // if the flow is set to be inactive, then no need of computing
			continue;

		bool IsAlreadyComputed=false;
		QList<int> IdentityList;
		// check whether the parameter has been computed
		for(int j=0;j<m_LayerMotionList[frameIndex].flowFieldList().size();j++)
			if(m_LayerMotionList[frameIndex].flowFieldList()[j].opticalFlowPara()==m_LayerMotionPara.flowParaList()[i]) // if the model parameters are the same
			{
			
			
				if(m_LayerMotionList[frameIndex].flowFieldList()[j].algorithmPara()==m_LayerMotionPara.algorithmPara())
				{
					IsAlreadyComputed=true;
					break;
				}
				else
				
					IdentityList.append(j);
			}
		if(IsAlreadyComputed)
			continue;
		if(IdentityList.size()>0) // remove the flow field with the same parameter setting as the specified
			for(int j=IdentityList.size()-1;j>=0;j--)
				m_LayerMotionList.removeAt(IdentityList[j]);
		
		if(IsLayerVisible(frameIndex)==false || IsLayerVisible(frameIndex+1)==false) // if one of the two frames is invisible
		{
			if(progressDialog==NULL) // just count the time
				continue;
			top=m_BoundingBoxList[frameIndex].top();
			left=m_BoundingBoxList[frameIndex].left();
			bottom=m_BoundingBoxList[frameIndex].bottom();
			right=m_BoundingBoxList[frameIndex].right();
			
			// the motion will be translation
			DImage vx1(right-left+1,bottom-top+1),vy1(right-left+1,bottom-top+1);
			m_ContourLabelList[frameIndex+1].computeCentroid();
			m_ContourLabelList[frameIndex].computeCentroid();
			QPointF flow=m_ContourLabelList[frameIndex+1].centroid()-m_ContourLabelList[frameIndex].centroid();
			vx1.setValue(flow.x());
			vy1.setValue(flow.y());
			FlowField m_FlowField;
			m_FlowField.fromDoublePrecision(vx1,vy1);
			m_FlowField.algorithmPara()=m_LayerMotionPara.algorithmPara();
			m_FlowField.opticalFlowPara()=m_LayerMotionPara.flowParaList()[i];
			m_FlowField.left()=left;
			m_FlowField.top()=top;
			m_FlowField.right()=right;
			m_FlowField.bottom()=bottom;

			// add to the list
			m_LayerMotionList[frameIndex].flowFieldList().append(m_FlowField);

			//reverse added by peter
			DImage vx2(right-left+1,bottom-top+1),vy2(right-left+1,bottom-top+1);
			m_ContourLabelList[frameIndex+1].computeCentroid();
			m_ContourLabelList[frameIndex].computeCentroid();
			vx2.setValue(-1*flow.x());
			vy2.setValue(-1*flow.y());
			FlowField m_FlowField2;
			m_FlowField2.fromDoublePrecision(vx2,vy2);
			m_FlowField2.algorithmPara()=m_LayerMotionPara.algorithmPara();
			m_FlowField2.opticalFlowPara()=m_LayerMotionPara.flowParaList()[i];
			m_FlowField2.left()=left;
			m_FlowField2.top()=top;
			m_FlowField2.right()=right;
			m_FlowField2.bottom()=bottom;

			// add to the list
			m_LayerReverseMotionList[frameIndex+1].flowFieldList().append(m_FlowField2);

			// set the motion parameter to be computed
			m_LayerMotionPara.flowParaList()[i].IsComputed()=true;
			progressDialog->setValue(total);
			continue;
		}
		total+=(bottom-top+1)*(right-left+1)*log((double)(right-left+1));
		if(progressDialog==NULL)
			continue;

		// now compute the optical flow using the parameters
		DImage vx1(right-left+1,bottom-top+1),vy1(right-left+1,bottom-top+1),vx2(right-left+1,bottom-top+1),vy2(right-left+1,bottom-top+1);
		DImage warpI2,warpI1;
		
		vx1.reset();
		vy1.reset();
		if(m_LayerMotionPara.flowParaList()[i].regularization()==OpticalFlowPara::membrane)
			OpticalFlowBidir::SetThinPlate(false);
		else
			OpticalFlowBidir::SetThinPlate(true);
		OpticalFlowBidir::setEta(m_LayerMotionPara.flowParaList()[i].eta());

		// load the parameters
		double alpha=m_LayerMotionPara.flowParaList()[i].alpha();
		double gamma=m_LayerMotionPara.flowParaList()[i].gamma();
		double ratio=m_LayerMotionPara.algorithmPara().ratio();
		int minWidth=m_LayerMotionPara.algorithmPara().minWidth();
		int nIterations=m_LayerMotionPara.algorithmPara().nIterations();
		int nIRLSIterations=m_LayerMotionPara.algorithmPara().nIRLSIterations();
		int nCGIterations=m_LayerMotionPara.algorithmPara().nCGIterations();

		// double check whether the images are ok
		//patch1.imwrite("patch1.jpg");
		//patch2.imwrite("patch2.jpg");
		//Mask1.imwrite("mask1.jpg");
		//Mask2.imwrite("mask2.jpg");

		// we need to handle minWidth because the patch can be too small
		if(patch1.width()<minWidth*2)
			minWidth=__max(patch1.width()/4,5);

		OpticalFlowBidir::Coarse2FineFlowBidir(vx1,vy1,warpI2,vx2,vy2,warpI1,patch1,patch2,Mask1,Mask2,alpha,gamma,ratio,minWidth,nIterations,nIRLSIterations,nCGIterations);


		// write the flow field 
		FlowField m_FlowField;
		m_FlowField.fromDoublePrecision(vx1,vy1);
		m_FlowField.algorithmPara()=m_LayerMotionPara.algorithmPara();
		m_FlowField.opticalFlowPara()=m_LayerMotionPara.flowParaList()[i];
		m_FlowField.left()=left;
		m_FlowField.top()=top;
		m_FlowField.right()=right;
		m_FlowField.bottom()=bottom;

		FlowField m_FlowField2;
		m_FlowField2.fromDoublePrecision(vx2,vy2);
		m_FlowField2.algorithmPara()=m_LayerMotionPara.algorithmPara();
		m_FlowField2.opticalFlowPara()=m_LayerMotionPara.flowParaList()[i];
		m_FlowField2.left()=left;
		m_FlowField2.top()=top;
		m_FlowField2.right()=right;
		m_FlowField2.bottom()=bottom;

		// add to the list
		m_LayerMotionList[frameIndex].flowFieldList().append(m_FlowField);
		m_LayerReverseMotionList[frameIndex+1].flowFieldList().append(m_FlowField2);

		// set the motion parameter to be computed
		m_LayerMotionPara.flowParaList()[i].IsComputed()=true;
		progressDialog->setValue(total);
		
		time(&now);
		double diff=difftime(now,*start);
		QString text("Computing optical flow: ");
		text+=QString::number((int)diff)+" seconds passed, ";
		diff*=(double)sum/total-1;
		text+=QString::number((int)diff)+" seconds to go";
		progressDialog->setLabelText(text);
		progressDialog->update();
		QApplication::processEvents();
	}
}

void LayerObject::ComposeMotion(int frameIndex, DImage &vx, DImage &vy,const BiImage& refImage)
{
	if(m_BoundingBoxList[frameIndex].isEmpty())
		return;

	int left,top,right,bottom,width,height;
	// get the bounding box of the layer mask
	int _left,_top,_right,_bottom;
	QRect layerBoundingBox=m_LayerMaskList[frameIndex].boundingBox();
	_left=layerBoundingBox.left();
	_top=layerBoundingBox.top();
	_right=layerBoundingBox.right();
	_bottom=layerBoundingBox.bottom();
	DImage flowVx,flowVy;

	bool IsMotionSelected=false;
	int x,y,u,v;// x and y are the coordinates in the layer mask , and u and v are the coordinate in the flow field

	if(m_TabIndexList[frameIndex]==0) // the motion is optical flow
		if(m_LayerMotionList[frameIndex].flowFieldList().size()>0 && m_SelectedRow>=0 && m_SelectedRow<m_LayerMotionList[frameIndex].flowFieldList().size())
		{
			// get the bounding box of the flow
			top=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].top();
			left=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].left();
			right=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].right();
			bottom=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].bottom();
			width=right-left+1;
			height=bottom-top+1;

			// get the selected flow field
			m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].toDoublePrecision(flowVx,flowVy);

			// bilateral smooth the flow field
			//BiImage layerImage;
			//refImage.crop(layerImage,left,top,width,height);
			//layerImage.BilateralFiltering(flowVx,6,2.5,16);
			//layerImage.BilateralFiltering(flowVy,6,2.5,16);

			//DImage temp;
			//flowVx.concatenate(temp,flowVy);
			//layerImage.BilateralFiltering(temp,7,2,10);
			//temp.separate(1,flowVx,flowVy);

			IsMotionSelected=true;
		}
	if(m_TabIndexList[frameIndex]==2) // parametric motion
		if(m_CorrespondenceList[frameIndex].size()>0)
		{
			top=_top;left=_left;
			bottom=_bottom;right=_right;
			width=right-left+1;
			height=bottom-top+1;
			Correspondence::setFlowDimension(left,top,right-left+1,bottom-top+1);
			m_CorrespondenceList[frameIndex].estimateMotion();
			m_CorrespondenceList[frameIndex].generateFlowField(flowVx,flowVy,_left,_top,_right,_bottom);
			IsMotionSelected=true;
		}
	if(IsMotionSelected) // if there is no motion selected then output zero
	{
		//m_LayerMaskList[frameIndex].imwrite("layerMask.bmp");
		for(int i=_top;i<=_bottom;i++)
			for(int j=_left;j<=_right;j++)
			{
				y=i-_top;
				x=j-_left;
				v=i-top;
				u=j-left;
				if(u<0 || u>=width || v<0 || v>=height)
					continue;
				if(m_LayerMaskList[frameIndex].getPixel(j,i)) // if the layer mask is 1
				{
					double dx = flowVx.data()[v*flowVx.width()+u];
					double dy = flowVy.data()[v*flowVx.width()+u];
					if (dx >=10)
						dx=10;
					if (dx <-10)
						dx=-10;
					if (dy >=10)
						dy=10;
					if (dy <-10)
						dy=-10;
					
					vx.data()[i*vx.width()+j]=dx;
					vy.data()[i*vx.width()+j]=dy;
				}
			}
	}
	else		// if the motion is not selected then set the motion to be zero
		for(int i=_top;i<=_bottom;i++)
			for(int j=_left;j<=_right;j++)
			{
				y=i-_top;
				x=j-_left;
				if(m_LayerMaskList[frameIndex].getPixel(j,i)) // if the layer mask is 1
				{
					vx.data()[i*vx.width()+j]=0;
					vy.data()[i*vx.width()+j]=0;
				}
			}
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// function to output layer motion
//-----------------------------------------------------------------------------------------------------------------------------------------
void LayerObject::OutputLayerMotion(int frameIndex,const QString& filename)
{
	if(m_BoundingBoxList[frameIndex].isEmpty())
		return;

	int left,top,right,bottom,width,height;
	// get the bounding box of the layer mask
	int _left,_top,_right,_bottom;
	QRect layerBoundingBox=m_LayerMaskList[frameIndex].boundingBox();
	_left=layerBoundingBox.left();
	_top=layerBoundingBox.top();
	_right=layerBoundingBox.right();
	_bottom=layerBoundingBox.bottom();

	DImage flowVx,flowVy;

	bool IsMotionSelected=false;

	if(m_TabIndexList[frameIndex]==0) // the motion is optical flow
		if(m_LayerMotionList[frameIndex].flowFieldList().size()>0 && m_SelectedRow>=0 && m_SelectedRow<m_LayerMotionList[frameIndex].flowFieldList().size())
		{
			// get the bounding box of the flow
			top=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].top();
			left=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].left();
			right=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].right();
			bottom=m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].bottom();
			width=right-left+1;
			height=bottom-top+1;

			// get the selected flow field
			m_LayerMotionList[frameIndex].flowFieldList()[m_SelectedRow].toDoublePrecision(flowVx,flowVy);
			IsMotionSelected=true;
		}
	if(m_TabIndexList[frameIndex]==2) // parametric motion
		if(m_CorrespondenceList[frameIndex].size()>0)
		{
			top=_top;left=_left;
			bottom=_bottom;right=_right;
			width=right-left+1;
			height=bottom-top+1;
			Correspondence::setFlowDimension(left,top,right-left+1,bottom-top+1);
			m_CorrespondenceList[frameIndex].estimateMotion();
			m_CorrespondenceList[frameIndex].generateFlowField(flowVx,flowVy,_left,_top,_right,_bottom);
			IsMotionSelected=true;
		}
	flowVx.writeImage(filename+"_vx.flw");
	flowVy.writeImage(filename+"_vy.flw");
	double Max,Min;
	Max=max(flowVx.maximum(),flowVy.maximum());
	Min=min(flowVx.minimum(),flowVy.minimum());
	flowVx.imwrite(filename+"_vx.jpg",Min,Max,99);
	flowVy.imwrite(filename+"_vy.jpg",Min,Max,99);
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// functions for FILE IO
//-----------------------------------------------------------------------------------------------------------------------------------------
bool LayerObject::writeLayerObject(QFile &file) const
{
	// write the number of frames
	if(file.write((char *)&m_NumFrames,sizeof(int))!=sizeof(int))
		return false;
	file.write((char *)&m_StartFrame,sizeof(int));
	file.write((char *)&m_EndFrame,sizeof(int));
	file.write((char *)&m_CreatedFrame,sizeof(int));
	file.write((char *)&m_MaxStrokeId,sizeof(int));
	
	QVideoProcessing::writeString(file,m_LayerName);

	// write the contour label
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(!m_ContourLabelList[i].writeContourLabel(file))
			return false;

	if(!QVideoProcessing::writeQListSegment<double>(file,m_DepthList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::writeQListSegment<bool>(file,m_IsGlobalLabelList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::writeQListSegment<bool>(file,m_IsLocalLabelList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::writeQListSegment<bool>(file,m_IsTrackedList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::writeQListSegment<bool>(file,m_IsDepthLabeledList,m_StartFrame,m_EndFrame))
		return false;

	// to write the layer information
	// output layer masks
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(!m_LayerMaskList[i].writeBinaryImage(file))
			return false;

	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(!m_VisibleLayerMaskList[i].writeBinaryImage(file))
			return false;


	bool f=false;
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		
		file.write((char *)&f,sizeof(bool));
		//file.write(false,sizeof(bool));
	}
	
	/*
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		if (m_IsBoundaryDistanceCreatedList[i] == true)
			if(!m_BoundaryDistanceImageList[i].writeImage(file))
				return false;
	}
	*/
	

	for(int i=m_StartFrame;i<m_EndFrame;i++)
	{
		//file.write((char *)&m_IsBoundaryFlowCreatedList[i],sizeof(bool));
		file.write((char *)&f,sizeof(bool));
	}
	/*
	for(int i=m_StartFrame;i<m_EndFrame;i++)
		if (m_IsBoundaryFlowCreatedList[i]==true)
		{
			if(!m_BoundaryFlowList[i].writeFlowField(file))
				return false;
		}
	

	for(int i=m_StartFrame+1;i<=m_EndFrame;i++)
		if (m_IsBoundaryFlowCreatedList[i]==true)
		{
			if(!m_BoundaryReverseFlowList[i].writeFlowField(file))
				return false;
		}
	
	*/ 

	// output the bouding boxes of the visible masks
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(!QVideoProcessing::writeQRect(file,m_BoundingBoxList[i]))
			return false;

	// output the motion para
	if(!m_LayerMotionPara.writeMotionPara(file))
		return false;

	// output the motion
	for(int i=m_StartFrame;i<m_EndFrame;i++) // note that for this one there is no need to save the last frame
		if(!m_LayerMotionList[i].writeMotion(file))
			return false;

	for(int i=m_StartFrame+1;i<=m_EndFrame;i++) // note that for this one there is no need to save the last frame
		if(!m_LayerReverseMotionList[i].writeMotion(file))
			return false;

	// the following code is for >1.00
	file.write((char *)&m_SelectedRow,sizeof(int));

	for(int i=m_StartFrame;i<m_EndFrame;i++)
		if(!m_CorrespondenceList[i].writeCorrespondence(file))
			return false;

	// the line below is for version <1.20
	// file.write((char *)&m_TabIndex,sizeof(int)); 
	if(!QVideoProcessing::writeQListSegment<int>(file,m_TabIndexList,m_StartFrame,m_EndFrame))
		return false;


	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{
		int numStrokes = 0;

		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if ((m_StrokeList[i][j].deleted() == false))
				numStrokes++;
		}

		file.write((char *)&numStrokes,sizeof(int));

		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].deleted() == true)
				continue;

			if ((m_StrokeList[i][j].writeContourLabel(file) == false))
				return false;
		}
	}
	char strbuf[1024];
        sprintf(strbuf, "looking for images in directory: %s", m_ProjectDir.toAscii().data());
	LOG_STR_COUT(strbuf);

    QDir dir(m_ProjectDir);
    dir.mkdir("regionTextures");

	//write out the stroke images
	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{
		//if (m_StrokeImageStateList[i] == TEXTURE_EXISTS)
		if ((m_StrokeImageStateList[i] == TEXTURE_EXISTS) || (m_StrokeImageStateList[i] == CREATE_TEXTURE))
		{
			//m_StrokeImageList[i].save(dir+m_LayerName+QString("-strokes-")+QString::number(i)+QString(".png"),0,100);



            QString imgfname = m_ProjectDir+QString("regionTextures/")+m_LayerName+QString("-strokes-")+QString::number(i)+QString(".img");

			FILE * pFile = fopen (imgfname.toAscii(),"wb");
			if (pFile!=NULL)
			{
		
				int w=m_StrokeImageList[i].width();
				int h=m_StrokeImageList[i].height();
				fwrite(&w , sizeof(int) , 1, pFile );
				fwrite(&h , sizeof(int) , 1, pFile );

				bool blankImage = true;
				for(int x=0;x < w;x++)
				for(int y=0;y < h;y++)
				{
					if (qAlpha(m_StrokeImageList[i].pixel(x,y)) > 0)
					{
						x=w;y=h;
						blankImage=false;
					}

				}
				if (blankImage==true)
				{
		
					sprintf(strbuf, "Maybe error. Writing out a blank image at frame %i",i);
					LOG_STR_COUT(strbuf);
					fclose(pFile);
					continue;
					//return false;
				}

				const uchar *data =  m_StrokeImageList[i].bits();	

			    int bytesWritten = fwrite(data , sizeof(uchar) , m_StrokeImageList[i].numBytes(), pFile );

				if (bytesWritten!= m_StrokeImageList[i].numBytes())
				{
					LOG_STR_COUT("ERROR: didn't write the correct number of bytes");
					fclose(pFile);
					continue;
				}
				fflush(pFile);
			    fclose(pFile);
			}
		
		}
	}



	//Write out generated layers
	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{
		

		int numLayers = m_GeneratedStrokeList[i].size();
		file.write((char *)&numLayers,sizeof(int));


        QMap<int,QVector<Stroke *> >::const_iterator itr = m_GeneratedStrokeList[i].begin();
		while (itr != m_GeneratedStrokeList[i].end()) 
		{
                        const QVector<Stroke *> strokeV = itr.value();

			int numLayerStroke = strokeV.size();
			file.write((char *)&numLayerStroke,sizeof(int));
			int genId = itr.key();
			file.write((char *)&genId,sizeof(int));

			for(int j=0;j < strokeV.size();j++)
			{
				strokeV[j]->writeStroke(file);
			}
			++itr;
		}
	}



	QString imgfname = m_ProjectDir+QString("strokeLayerProperties/")+m_LayerName+QString(".xml");
	QFile xmlfile(imgfname);

	if(xmlfile.open(QIODevice::WriteOnly)==false)
	{
		LOG_STR_COUT("Problem creating xml file");
		return true;
	}
	//	return false;

	XmlWriter xw(&xmlfile);
	xw.setAutoNewLine(true);

	xw.writeOpenTag("StrokeLayerPropertiesList");

        QMap<int,StrokeLayerProperties>::const_iterator itr = m_GeneratedStrokeParameterList.begin();
	while (itr != m_GeneratedStrokeParameterList.end()) 
	{

		 StrokeLayerProperties slp = itr.value();

		slp.writeStrokeLayerPropertiesToXML(xw);

		++itr;
	}
	xw.writeCloseTag("StrokeLayerPropertiesList");

	return true;
}

bool LayerObject::readLayerObject(QFile &file,int nFrames,QString& projectDir)
{

	// write the number of frames
	if(file.read((char *)&m_NumFrames,sizeof(int))!=sizeof(int))
		return false;
	if(m_NumFrames!=nFrames)
		return false;

	m_ProjectDir = projectDir;

	file.read((char *)&m_StartFrame,sizeof(int));
	file.read((char *)&m_EndFrame,sizeof(int));
	file.read((char *)&m_CreatedFrame,sizeof(int));
	file.read((char *)&m_MaxStrokeId,sizeof(int));

	
	QVideoProcessing::readString(file,m_LayerName);

        sprintf(_strbuf, "Reading in layer: %s",m_LayerName.toAscii().data());
	LOG_STR_COUT(_strbuf);


	// during creating all the buffers are released and initialized
	create(m_NumFrames);

	// write the contour label
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		if(!m_ContourLabelList[i].readContourLabel(file))
			return false;
		m_ContourLabelList[i].computeCentroid();
	}

	if(!QVideoProcessing::readQListSegment<double>(file,m_DepthList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::readQListSegment<bool>(file,m_IsGlobalLabelList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::readQListSegment<bool>(file,m_IsLocalLabelList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::readQListSegment<bool>(file,m_IsTrackedList,m_StartFrame,m_EndFrame))
		return false;
	if(!QVideoProcessing::readQListSegment<bool>(file,m_IsDepthLabeledList,m_StartFrame,m_EndFrame))
		return false;

	// to read the layer information
	// output layer masks
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(!m_LayerMaskList[i].readBinaryImage(file))
			return false;

	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(!m_VisibleLayerMaskList[i].readBinaryImage(file))
			return false;

	/*

	DImage mask1;
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		m_VisibleLayerMaskList[i].export2Image(mask1);
		QString layer = m_LayerName;
		mask1.imwrite(QString("images/visibleMask-").append(layer).append(QString::number(i)).append(".jpg"));
	}
	*/
	/*
			for(int i=m_StartFrame;i<=m_EndFrame;i++)
				file.write((char *)&m_IsBoundaryDistanceCreatedList[i],sizeof(bool));

			for(int i=m_StartFrame;i<=m_EndFrame;i++)
			{
				if (m_IsBoundaryDistanceCreatedList[i] == true)
					if(!m_BoundaryDistanceImageList[i].writeImage(file))
						return false;
			}

			for(int i=m_StartFrame;i<m_EndFrame;i++)
				file.write((char *)&m_IsBoundaryFlowCreatedList[i],sizeof(bool));

			for(int i=m_StartFrame;i<m_EndFrame;i++)
				if (m_IsBoundaryFlowCreatedList[i]==true)
					if(!m_BoundaryFlowList[i].writeFlowField(file))
						return false;
*/
	//CreateBoundaryDistanceImage(1,256,256);

	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		file.read((char *)&m_IsBoundaryDistanceCreatedList[i],sizeof(bool));
		//m_IsBoundaryDistanceCreatedList[i] = false;
	}

	FImage dist;
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		if (m_IsBoundaryDistanceCreatedList[i]==true)
		{
			//if(!m_BoundaryDistanceImageList[i].readImage(file))
			//	return false;
			dist.readImage(file);
		}
		m_IsBoundaryDistanceCreatedList[i] = false;
	}
	
	for(int i=m_StartFrame;i<m_EndFrame;i++)
		file.read((char *)&m_IsBoundaryFlowCreatedList[i],sizeof(bool));

	FlowField temp;
	for(int i=m_StartFrame;i<m_EndFrame;i++)
	{
		if (m_IsBoundaryFlowCreatedList[i]==true)
		{
			temp.readFlowField(file);
		}
	}

	for(int i=m_StartFrame+1;i<=m_EndFrame;i++)
	{
		if (m_IsBoundaryFlowCreatedList[i]==true)
		{
			temp.readFlowField(file);
		}
	}

	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		m_IsBoundaryFlowCreatedList[i] = false;
	}

/*
	for(int i=m_StartFrame;i<m_EndFrame;i++)
		if(!m_BoundaryFlowList[i].readFlowField(file))
			return false;
	for(int i=m_StartFrame;i<m_EndFrame;i++)
		file.read((char *)&m_IsBoundaryFlowCreatedList[i],sizeof(bool));
*/
	// output the bouding boxes of the visible masks
	for(int i=m_StartFrame;i<=m_EndFrame;i++)
		if(!QVideoProcessing::readQRect(file,m_BoundingBoxList[i]))
			return false;



	// output the motion para
	if(!m_LayerMotionPara.readMotionPara(file))
		return false;

	// output the motion
	for(int i=m_StartFrame;i<m_EndFrame;i++) // note that for this one there is no need to load the last frame
		if(!m_LayerMotionList[i].readMotion(file))
			return false;

	for(int i=m_StartFrame+1;i<=m_EndFrame;i++) // note that for this one there is no need to load the last frame
		if(!m_LayerReverseMotionList[i].readMotion(file))
			return false;

	// the following code is for version >1.01
	if(VideoLabeling::curVersion()>1.01)
	{
		file.read((char *)&m_SelectedRow,sizeof(int));

		for(int i=m_StartFrame;i<m_EndFrame;i++)
			if(!m_CorrespondenceList[i].readCorrespondence(file))
				return false;

		if(VideoLabeling::curVersion()<1.19)
		{
			int m_TabIndex;
			file.read((char *)&m_TabIndex,sizeof(int));
			for(int i=0;i<m_NumFrames;i++)
				m_TabIndexList[i]=m_TabIndex;
		}
		else
			if(!QVideoProcessing::readQListSegment<int>(file,m_TabIndexList,m_StartFrame,m_EndFrame))
				return false;
	}



	QVector<QRect> fullBoundingBoxVector(m_NumFrames);
	m_FullBoundingBoxList=QList<QRect>::fromVector(fullBoundingBoxVector);


	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{
		QRect rect=m_LayerMaskList[i].boundingBox();

		int width,height;
		width=m_LayerMaskList[i].width();
		height=m_LayerMaskList[i].height();


		// adjust the bounding box
		rect.adjust(-10,-10,10,10);
		if(rect.left()<0)
			rect.setLeft(0);
		if(rect.right()>width-1)
			rect.setRight(width-1);
		if(rect.top()<0)
			rect.setTop(0);
		if(rect.bottom()>height-1)
			rect.setBottom(height-1);

		m_FullBoundingBoxList[i]=rect;

         if ((QString::compare(m_LayerName,QString("Background"))==0)&& (m_ContourLabelList[i].numPoints()==0))
		{

            m_ContourLabelList[i].addPoint(QPointF(0,0),false);
            m_ContourLabelList[i].addPoint(QPointF(width,0),false);
            m_ContourLabelList[i].addPoint(QPointF(width,height),false);
            m_ContourLabelList[i].addPoint(QPointF(0,height),false);

		}
	}





	//int idx = file.fileName().lastIndexOf(QChar('/'));
	//m_ProjectDir = file.fileName();
	//m_ProjectDir.truncate(idx+1);

	//cout << "directory: "<< m_ProjectDir.toAscii().data()<< std::endl;
    sprintf(_strbuf, "Loading stroke images for layer: %s", m_LayerName.toAscii().data());
	LOG_STR_COUT(_strbuf);

	for(int i=m_StartFrame;i<=m_EndFrame;i++)
	{

		CreateStrokeImage(i);

		QImage loadImg, loadAlpha;
	
		//m_StrokeImageList[i].load(m_ProjectDir+m_LayerName+QString("-strokes-alpha-")+QString::number(i)+QString(".xpm"));

        QString imgfname =m_ProjectDir+QString("regionTextures/")+m_LayerName+QString("-strokes-")+QString::number(i)+QString(".img");
		FILE * pFile;
		pFile = fopen (imgfname.toAscii(),"rb");
		if (pFile!=NULL)
		{
			

			int *dimbuffer = (int*) malloc (sizeof(int)*2);

			int readin = fread (dimbuffer , sizeof(int),2,pFile );
	
			int w=dimbuffer[0];
			int h=dimbuffer[1];
                        //sprintf(_strbuf, "read in dim %i %i", w,h);

			m_StrokeImageList[i] = QImage(w,h,QImage::Format_ARGB32_Premultiplied);

                        //sprintf(_strbuf, "looking for bytes %i", m_StrokeImageList[i].numBytes());

			free(dimbuffer);
			uchar *buffer = (uchar*) malloc (sizeof(uchar)*m_StrokeImageList[i].numBytes());
			

            readin = fread (buffer , sizeof(uchar),m_StrokeImageList[i].numBytes(),pFile );
                        //sprintf(_strbuf, "read in %i, %i %i", readin, feof(pFile), ferror (pFile));
			if (readin ==m_StrokeImageList[i].numBytes())
			{

				int r,g,b,a;
				int m = 0;

			
				for(int y=0;y < m_StrokeImageList[i].height();y++)
				for(int x=0;x < m_StrokeImageList[i].width();x++)
				{
					
					b = buffer[m];
					g = buffer[m+1];
					r = buffer[m+2];
					a = buffer[m+3];
	
					m_StrokeImageList[i].setPixel(x, y,qRgba ( r, g, b, a ) );
			
					m +=4;
				}

				//m_StrokeImageList[i].save(m_ProjectDir+m_LayerName+QString("-savedstrokes-")+QString::number(i)+QString(".png"));
		
				//}
			}
			else
			{
            sprintf(_strbuf, "bytes don't match, %i %i", readin,m_StrokeImageList[i].numBytes());
				LOG_STR_COUT(_strbuf);

			}
			free(buffer);
		    fclose (pFile);
			m_StrokeImageStateList[i] = CREATE_TEXTURE;
		}

		//}

	
	}


	int width =m_LayerMaskList[0].width();
	int height=m_LayerMaskList[0].height();

	
	
	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{
		

		int numStrokes ;
		file.read((char *)&numStrokes,sizeof(int));
		QList<ContourLabel> contourList;
		m_StrokeList[i] = contourList;
		for(int j=0;j < numStrokes;j++)
		{
			ContourLabel newcontour;

			if (newcontour.readContourLabel(file) == false)
				return false;

		
			newcontour.setImWidth(width);
			newcontour.setImHeight(height);
			newcontour.recomputeStroke();

			m_StrokeList[i].push_back(newcontour);
			if (newcontour.getStrokeId() >= m_MaxStrokeId)
				m_MaxStrokeId = newcontour.getStrokeId() + 1;
			
			
		}	


		m_StrokeImageTextureList[i] = 0;
	}

	checkStrokeLinks();


	for(int i=m_StartFrame;i<m_EndFrame;i++) 
		createBoundaryKDTree(i);


	bool anyLayers=false;
	//read in  generated layers
	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{

		int numLayers;
		file.read((char *)&numLayers,sizeof(int));

		if (numLayers==0)
		{
			CreateStrokeImage(i);
		//	m_StrokeImageStateList[i] = NO_IMAGE;
		}
		anyLayers=true;

		for(int j=0;j < numLayers;j++)
		{

			QVector<Stroke *> strokeVect;

			int numLayerStroke;
			file.read((char *)&numLayerStroke,sizeof(int));
			int genId;
			file.read((char *)&genId,sizeof(int));

			for(int j=0;j < numLayerStroke;j++)
			{
				Stroke * str = new Stroke();
				str->readStroke(file);
				strokeVect.push_back(str);
			}

			m_GeneratedStrokeList[i].insert(genId,strokeVect);

		}
	}

	if (anyLayers)
	{

		QDomDocument doc("mydocument");
		QString fname = m_ProjectDir+QString("strokeLayerProperties/")+m_LayerName+QString(".xml");
		QFile file(fname);
		if (!file.open(QIODevice::ReadOnly))
		{
			 LOG_STR_COUT("problem with reading in xml file");
			 return true;
		}
		if (!doc.setContent(&file)) 
		{
			 LOG_STR_COUT("problem reading in xml document");
			 file.close();
			 return true;
		}
		file.close();

		QDomElement root=doc.documentElement();

		if (root.tagName()=="StrokeLayerPropertiesList")
		{

			QDomNode n=root.firstChild();
			while(!n.isNull())
			{

				QDomElement e=n.toElement();
				StrokeLayerProperties slp;
				slp.readStrokeLayerPropertiesFromXML(e);
				m_GeneratedStrokeParameterList.insert(slp.generationId,slp);
				n=n.nextSibling();
			}
		}
	}

	return true;
}


void LayerObject::checkForDuplicateStokes()
{

	QList<int> problemStrokes;
	for(int i=0;i < m_NumFrames;i++)
	{

		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].deleted()==false)
			{
				for(int k=j+1;k < m_StrokeList[i].size();k++)
				{
					if ((m_StrokeList[i][j].getStrokeId()==m_StrokeList[i][k].getStrokeId()) &&
						(m_StrokeList[i][k].deleted()==false))
					{

						problemStrokes.push_back(m_StrokeList[i][k].getStrokeId());
						//deleteStroke(i,k);

						//k--;
						LOG_STR_COUT("Error, duplicate active strokes");
					}
			
				}
			}
		}
	}

	if (problemStrokes.size()==0)
		return;

	LOG_STR_COUT("Sorry. I ran into a problem with one of the strokes. I'm going to delete it. :P");

	for(int i=0;i < m_NumFrames;i++)
	{

		for(int j=m_StrokeList[i].size()-1;j >-1 ;j--)
		{
			
			for (int k=0;k < problemStrokes.size();k++)
			{
				if (m_StrokeList[i][j].getStrokeId()==problemStrokes[k])
				{
					m_StrokeList[i].removeAt(j);
		
					break;
				}
			}

		}
	}

}

void LayerObject::deleteStroke(int frame, int stroke)
{

	if( m_StrokeList.size()< frame)
		return;

	if (m_StrokeList[frame].size() < stroke)
		return;


	ContourLabel *prev = m_StrokeList[frame][stroke].getPrev();
	if ((prev !=0) && (prev->getNext()==&(m_StrokeList[frame][stroke])))
	{
		prev->setNext(0);
	}

	m_StrokeList[frame][stroke].setPrev(0);

	ContourLabel *next = m_StrokeList[frame][stroke].getNext();
	if ((next !=0) &&  (next->getPrev()==&(m_StrokeList[frame][stroke])))
		next->setPrev(0);

	m_StrokeList[frame][stroke].setNext(0);

	m_StrokeList[frame].removeAt(stroke);
}


void LayerObject::checkForSelectionProblems()
{
	QList<int> problemStrokes;

	for(int i=m_StartFrame;i<m_EndFrame;i++) 
	{
		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].deleted()==false)
			{
				for(int k=0;k < m_StrokeList[i+1].size();k++)
				{	
					if ((m_StrokeList[i][j].getStrokeId() == m_StrokeList[i+1][k].getStrokeId())
						&& (m_StrokeList[i+1][k].deleted()==false) &&
						(m_StrokeList[i+1][k].groupSelected()!=m_StrokeList[i][j].groupSelected()))
					{
					
						problemStrokes.push_back(m_StrokeList[i][j].getStrokeId());
					}
				}
			}
		}
	}
	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{
		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			for(int k=0;k < problemStrokes.size();k++)
			{
				if (m_StrokeList[i][j].getStrokeId() ==problemStrokes[k])
				{
					m_StrokeList[i][j].groupDeselectContour();
				}
			}
		}
	}


}

void LayerObject::checkStrokeLinks()
{
	for(int i=m_StartFrame;i<m_EndFrame;i++) 
	{
		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].deleted()==false)
			{
				for(int k=0;k < m_StrokeList[i+1].size();k++)
				{	
					if ((m_StrokeList[i][j].getStrokeId() == m_StrokeList[i+1][k].getStrokeId())
						&& (m_StrokeList[i+1][k].deleted()==false))
					{
						m_StrokeList[i][j].setNext(&m_StrokeList[i+1][k]);
						m_StrokeList[i+1][k].setPrev(&m_StrokeList[i][j]);
					}
				}
			}
		}
	}
}

void LayerObject::createStrokeKDTrees()
{

	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{
		createStrokeKDTree(i,true);
		createStrokeKDTree(i,false);
	}
}


void LayerObject::createStrokeKDTree(int curFrameIndex,bool guideTree)
{
	
	if ((guideTree == true) && (m_GuideStrokeKDTreeList[curFrameIndex] != 0))
	{
		delete m_GuideStrokeKDTreeList[curFrameIndex];
		m_GuideStrokeKDTreeList[curFrameIndex] = 0;
	}
	if ((guideTree == false) && (m_ColorStrokeKDTreeList[curFrameIndex] != 0))
	{
		delete m_ColorStrokeKDTreeList[curFrameIndex];
		m_ColorStrokeKDTreeList[curFrameIndex] = 0;
	}


	int strokeCount = 0;
	for(int j=0;j < m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].deleted() == true)
			continue;

		if ((m_StrokeList[curFrameIndex][j].IsGuideStroke() == true) && (guideTree==true))
			strokeCount++;
		if ((m_StrokeList[curFrameIndex][j].IsColorStroke() == true) && (guideTree==false))
			strokeCount++;
	}
	if (strokeCount == 0)
		return;

	int height = m_LayerMaskList[curFrameIndex].height();


	ANNpointArray		dataPts;				// data points
	ANNkd_tree*			kdTree;					// search structure

	int nPts = 0;

	for(int j=0;j < m_StrokeList[curFrameIndex].size();j++)
	{
		if ((m_StrokeList[curFrameIndex][j].IsGuideStroke() == false) && (guideTree==true))
			continue;
		if ((m_StrokeList[curFrameIndex][j].IsColorStroke() == false) && (guideTree==false))
			continue;

		if (m_StrokeList[curFrameIndex][j].deleted() == true)
			continue;

		QVector<Point> *pointList = m_StrokeList[curFrameIndex][j].getStroke()->getLimitCurve();

		nPts +=pointList->size();
	}


	dataPts = annAllocPts(nPts, 2);			// allocate data points

	nPts=0;

	for(int j=0;j < m_StrokeList[curFrameIndex].size();j++)
	{
		if ((m_StrokeList[curFrameIndex][j].IsGuideStroke() == false) && (guideTree==true))
			continue;
		if ((m_StrokeList[curFrameIndex][j].IsColorStroke() == false) && (guideTree==false))
			continue;

		if (m_StrokeList[curFrameIndex][j].deleted() == true)
			continue;

		QVector<Point> *pointList = m_StrokeList[curFrameIndex][j].getStroke()->getLimitCurve();

		for (int k=0;k < pointList->size();k++)
		{
			dataPts[nPts][0] = pointList->at(k).x();
			dataPts[nPts][1] = height-pointList->at(k).y();
			nPts++;
		}
	}
	
	kdTree = new ANNkd_tree(					// build search structure
					dataPts,					// the data points
					nPts,						// number of points
					2);						// dimension of space

	if (guideTree == true)
		m_GuideStrokeKDTreeList[curFrameIndex] = kdTree;
	else
		m_ColorStrokeKDTreeList[curFrameIndex] = kdTree;

	//annDeallocPts(dataPts);
}



void LayerObject::createBoundaryKDTree(int curFrameIndex)
{

	if (m_BoundaryKDTreeList[curFrameIndex] != 0)
	{
		delete m_BoundaryKDTreeList[curFrameIndex];
		m_BoundaryKDTreeList[curFrameIndex] = 0;
	}

	QList<QPointF> boundPts = m_ContourLabelList[curFrameIndex].PointList();


	ANNpointArray		dataPts;				// data points
	ANNkd_tree*			kdTree;					// search structure

	int nPts = 0;
	dataPts = annAllocPts(boundPts.size(), 2);			// allocate data points

        for (int k=0;k < boundPts.size();k++)
	{
		dataPts[nPts][0] = boundPts[k].x();
		dataPts[nPts][1] = boundPts[k].y();
		nPts++;
	}
	
	kdTree = new ANNkd_tree(					// build search structure
					dataPts,					// the data points
					nPts,						// number of points
					2);						// dimension of space

        m_BoundaryKDTreeList[curFrameIndex] = kdTree;

	//annDeallocPts(dataPts);

}


//----------------------------------------------------------------------------------------
// function to reduce frame range
// it returns true if the object is out of the range and the parent object would 
// delete it
//----------------------------------------------------------------------------------------
bool LayerObject::reduceFrameRange(int start, int end)
{
	// remove the last part
	for(int i=end+1;i<m_NumFrames;i++)
	{
		m_ContourLabelList.removeLast();
		m_DepthList.removeLast();
		m_IsGlobalLabelList.removeLast();
		m_IsLocalLabelList.removeLast();
		m_IsTrackedList.removeLast();
		m_IsDepthLabeledList.removeLast();
		m_LayerMaskList.removeLast();
		m_VisibleLayerMaskList.removeLast();
		m_BoundingBoxList.removeLast();
		m_FullBoundingBoxList.removeLast();
		m_LayerMotionList.removeLast();
		m_LayerReverseMotionList.removeLast();
		m_CorrespondenceList.removeLast();
		m_TabIndexList.removeLast();
	}
	// remove the first part
	for(int i=0;i<start;i++)
	{
		m_ContourLabelList.removeFirst();
		m_DepthList.removeFirst();
		m_IsGlobalLabelList.removeFirst();
		m_IsLocalLabelList.removeFirst();
		m_IsTrackedList.removeFirst();
		m_IsDepthLabeledList.removeFirst();
		m_LayerMaskList.removeFirst();
		m_VisibleLayerMaskList.removeFirst();
		m_BoundingBoxList.removeFirst();
		m_FullBoundingBoxList.removeFirst();
		m_LayerMotionList.removeFirst();
		m_LayerReverseMotionList.removeFirst();
		m_CorrespondenceList.removeFirst();
		m_TabIndexList.removeFirst();
	}
	m_NumFrames=end-start+1;
	if(m_StartFrame> end || m_EndFrame<start)
		return true;
	
	m_StartFrame-=start;
	m_StartFrame=__max(__min(m_StartFrame,m_NumFrames-1),0);
	m_EndFrame-=start;
	m_EndFrame=__min(__max(m_EndFrame,0),m_NumFrames-1);
	m_CreatedFrame-=start;
	m_CreatedFrame=__min(__max(m_CreatedFrame,m_StartFrame),m_EndFrame);

	return false;
}

//----------------------------------------------------------------------------------------
// function to get the selected motion for display purpose
//----------------------------------------------------------------------------------------
bool LayerObject::getSelectedMotion(int frameIndex, int row, QImage &patch1, QImage &patch2, DImage &vx, DImage &vy,const QImage& frame1,const QImage& frame2)
{
	bool IsMatched=false;
	int j;
	// if the row indicates a valid parameter
	if(row>=0 && row<m_LayerMotionList[frameIndex].flowFieldList().size()) 
		for(j=0;j<m_LayerMotionList[frameIndex].flowFieldList().size();j++)
			if(m_LayerMotionList[frameIndex].flowFieldList()[j].opticalFlowPara()==m_LayerMotionPara.flowParaList()[row] && 
				m_LayerMotionList[frameIndex].flowFieldList()[j].algorithmPara()==m_LayerMotionPara.algorithmPara())
			{
				IsMatched=true;
				break;
			}
	if(m_BoundingBoxList[frameIndex].isEmpty()) // if the contour is outside image boundary, not motion
		IsMatched=false;
	int left,top,right,bottom;
	if(IsMatched)
	{
		//get the bounding box
		
		left=m_LayerMotionList[frameIndex].flowFieldList()[j].left();
		top=m_LayerMotionList[frameIndex].flowFieldList()[j].top();
		right=m_LayerMotionList[frameIndex].flowFieldList()[j].right();
		bottom=m_LayerMotionList[frameIndex].flowFieldList()[j].bottom();
		patch1=frame1.copy(left,top,right-left+1,bottom-top+1);
		patch2=frame2.copy(left,top,right-left+1,bottom-top+1);
		m_LayerMotionList[frameIndex].flowFieldList()[j].toDoublePrecision(vx,vy);
		return true;
	}
	else
	{
		left=__min(m_BoundingBoxList[frameIndex].left(),m_BoundingBoxList[frameIndex+1].left());
		top=__min(m_BoundingBoxList[frameIndex].top(),m_BoundingBoxList[frameIndex+1].top());
		right=__max(m_BoundingBoxList[frameIndex].right(),m_BoundingBoxList[frameIndex+1].right());
		bottom=__max(m_BoundingBoxList[frameIndex].bottom(),m_BoundingBoxList[frameIndex+1].bottom());
		patch1=frame1.copy(left,top,right-left+1,bottom-top+1);
		patch2=frame2.copy(left,top,right-left+1,bottom-top+1);
		return false;
	}
}

void LayerObject::getSelectedMotion(int frameIndex,QImage& patch1,QImage& patch2,DImage& vx,DImage& vy,const QImage& frame1,const QImage& frame2)
{
	int left,top,right,bottom;
	left=__min(m_BoundingBoxList[frameIndex].left(),m_BoundingBoxList[frameIndex+1].left());
	top=__min(m_BoundingBoxList[frameIndex].top(),m_BoundingBoxList[frameIndex+1].top());
	right=__max(m_BoundingBoxList[frameIndex].right(),m_BoundingBoxList[frameIndex+1].right());
	bottom=__max(m_BoundingBoxList[frameIndex].bottom(),m_BoundingBoxList[frameIndex+1].bottom());
	patch1=frame1.copy(left,top,right-left+1,bottom-top+1);
	patch2=frame2.copy(left,top,right-left+1,bottom-top+1);
	m_CorrespondenceList[frameIndex].generateFlowField(vx,vy,left,top,right,bottom);
}

void LayerObject::getLayerPatch(int frameIndex, QImage &patch, const QImage &frame)
{
	int left,top,right,bottom;
	left=m_BoundingBoxList[frameIndex].left();
	top=m_BoundingBoxList[frameIndex].top();
	right=m_BoundingBoxList[frameIndex].right();
	bottom=m_BoundingBoxList[frameIndex].bottom();
	patch=frame.copy(left,top,right-left+1,bottom-top+1);
}

//----------------------------------------------------------------------------------------
// function to add sparse correspondence
//----------------------------------------------------------------------------------------
void LayerObject::setCorrespondenceFlowDimension(int frameIndex)
{
	int left,top,right,bottom;
	left=__min(m_BoundingBoxList[frameIndex].left(),m_BoundingBoxList[frameIndex+1].left());
	top=__min(m_BoundingBoxList[frameIndex].top(),m_BoundingBoxList[frameIndex+1].top());
	right=__max(m_BoundingBoxList[frameIndex].right(),m_BoundingBoxList[frameIndex+1].right());
	bottom=__max(m_BoundingBoxList[frameIndex].bottom(),m_BoundingBoxList[frameIndex+1].bottom());
	Correspondence::setFlowDimension(left,top,right-left+1,bottom-top+1);
}


void LayerObject::addCorrespondence(int frameIndex, const QPointF &point1, const QPointF &point2)
{
	m_CorrespondenceList[frameIndex].addCorrespondence(point1,point2);
}

// to add uncertain matching
void LayerObject::addCorrespondence(int frameIndex,const QPointF& point1,const SortQueue<Point>& sort)
{
	m_CorrespondenceList[frameIndex].addCorrespondence(point1,sort);
}

bool LayerObject::IsParametricMotion(int frameIndex) const
{
	return m_CorrespondenceList[frameIndex].IsParametricMotion();
}

void LayerObject::estimateMotion(int frameIndex)
{
	// when flow estimate is invalidated, the piecewise smooth flow field need to be estimated
	m_CorrespondenceList[frameIndex].invalidateFlowEstimate();
	m_CorrespondenceList[frameIndex].estimateMotion();
}

//Create a new stroke at the current point with the given properties
void LayerObject::createStroke(int frameIndex,const QPointF& point,StrokeProperties& sp, float pressure,int strokeId)
{
    sprintf(_strbuf, "starting new stroke at point %f %f", point.x(), point.y());
	LOG_STR_COUT(_strbuf);

	ContourLabel newStroke;
	newStroke.setStrokeId(strokeId);
	newStroke.setIsStroke(true);
	newStroke.addPoint(point,true,pressure);
	newStroke.setStrokeProperties(sp);
	newStroke.setBeingDrawn(true);
	newStroke.setIsStroke(true);

	m_StrokeList[frameIndex].push_back(newStroke);
}


//Add a new point to the current stroke we're creating
void LayerObject::addStrokePoint(int frameIndex,const QPointF& point,StrokeProperties& sp, double ScaleFactor,float pressure)
{

	int strokeListSize =m_StrokeList[frameIndex].size();

	QPointF lastPoint = m_StrokeList[frameIndex][strokeListSize-1].getLastPoint();
	float dist = sqrt(pow((point.x()-lastPoint.x()),2) + pow((point.y()-lastPoint.y()),2));

        //sprintf(_strbuf, "dist %f, scale %f, %f", dist,sqrt(pow(ScaleFactor,1.75)),MyPreference::Threshold_StrokePointDistance/sqrt(ScaleFactor),MyPreference::Threshold_StrokePointDistance/sqrt(pow(ScaleFactor,1.75)));
	if (dist > MyPreference::Threshold_StrokePointDistance/sqrt(pow(ScaleFactor,2)))
	{
		m_StrokeList[frameIndex][strokeListSize-1].addPoint(point,true,pressure);
		m_StrokeList[frameIndex][strokeListSize-1].setStrokeProperties(sp);
	}
}

void LayerObject::finishedStroke(int frameIndex,bool showColorStroke, bool showGuideStroke, bool showSeedStroke, bool showRegularStroke)
{
	LOG_STR_COUT("finished stroke");
	



	int strokeListSize =m_StrokeList[frameIndex].size();


	if (m_StrokeList[frameIndex][strokeListSize-1].size() < 2)
		m_StrokeList[frameIndex].pop_back();
	else
	{
		m_StrokeList[frameIndex][strokeListSize-1].setBeingDrawn(false);

		m_StrokeList[frameIndex][strokeListSize-1].setHidden(showColorStroke,showGuideStroke,showSeedStroke,showRegularStroke);

		m_StrokeList[frameIndex][strokeListSize-1].groupSelectContour();
	}
	
	
	//
}

//Render all the strokes for a frame
void LayerObject::renderStrokes(int frameIndex)
{
	for(int i=0;i < m_StrokeList[frameIndex].size();i++)
	{
		LOG_STR_COUT("drawing strokes");
		m_StrokeList[frameIndex][i].drawStroke();
	}
}




//delete all the strokes for a given frame
void LayerObject::deleteUserStrokes(int frameIndex, int deleteId)
{
	m_CurrentStroke = -1;

	for(int i=0;i< m_StrokeList[frameIndex].size();i++)
	{
		m_StrokeList[frameIndex][i].deleteStroke(deleteId);
	}

}



void LayerObject::deleteCurrentStrokeOneDirection(int frameIndex, bool forward, int deleteId)
{
        sprintf(_strbuf, "starting stroke %i delete at frameIndex %i, id %i", m_CurrentStroke, frameIndex, deleteId);
	LOG_STR_COUT(_strbuf);

	if ((m_CurrentStroke < 0)|| (m_CurrentStroke >= m_StrokeList[frameIndex].size()))
		return;

	int strokeId = m_StrokeList[frameIndex][m_CurrentStroke].getStrokeId();

	if ((forward == true) && (m_StrokeList[frameIndex][m_CurrentStroke].getPrev() !=0))
		m_StrokeList[frameIndex][m_CurrentStroke].getPrev()->setNext(0);

	if ((forward == false) && (m_StrokeList[frameIndex][m_CurrentStroke].getNext() !=0))
		m_StrokeList[frameIndex][m_CurrentStroke].getNext()->setPrev(0);

	for(int i=0;i < m_NumFrames;i++)
	{
		if (((forward == true) && (i+m_StartFrame <frameIndex)) || 
			((forward == false) && (i+m_StartFrame >frameIndex)))
			continue;

		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].getStrokeId() == strokeId)
			{
				m_StrokeList[i][j].deleteStroke(deleteId);
			}
		}
	}
	m_CurrentStroke =-1;

}



void LayerObject::undoStroke(int strokeId)
{

	for(int i=0;i < m_NumFrames;i++)
	{

		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].getStrokeId() == strokeId)
			{
				deleteStroke(i,j);
				//m_StrokeList[i].removeAt(j);
			}
		}
	}
	m_CurrentStroke =-1;

}



void LayerObject::deleteCurrentStroke(int frameIndex, int deleteId, int startDeleteFrame,int endDeleteFrame,int *firstFrame, int *lastFrame)
{

        sprintf(_strbuf, "starting stroke %i delete at frameIndex %i, id %i", m_CurrentStroke, frameIndex, deleteId);
	LOG_STR_COUT(_strbuf);

	if ((m_CurrentStroke < 0))
		return;

	if (m_StrokeList[frameIndex].size() < m_CurrentStroke)
	{
		LOG_STR_COUT("strange. current stroke doesn't exist...");
		m_CurrentStroke=-1;
		return;
	}

	if ((startDeleteFrame < 0) || (startDeleteFrame >= m_NumFrames))
		return;
	else if ((endDeleteFrame < 0) || (endDeleteFrame >= m_NumFrames))
		return;

	int strokeId = m_StrokeList[frameIndex][m_CurrentStroke].getStrokeId();

	for(int i=startDeleteFrame;i <= endDeleteFrame;i++)
	{

		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].getStrokeId() == strokeId)
			{
				m_StrokeList[i][j].deleteStroke(deleteId);

				if (i < *firstFrame)
					*firstFrame = i;

				if (i > *lastFrame)
					*lastFrame = i;
			}
		}
	}
	m_CurrentStroke =-1;

}


void LayerObject::undoDeletedStrokes(int deleteId)
{
    sprintf(_strbuf, "undoing delete id %i", deleteId);
	LOG_STR_COUT(_strbuf);

	if (deleteId<0)
		return;

	for(int i=0;i < m_NumFrames;i++)
	{

		for(int j=0;j < m_StrokeList[i].size();j++)
		{
			if (m_StrokeList[i][j].getDeleteId() == deleteId)
			{
				//check that we haven't replicated this stroke again
			
				bool doDelete =true;
				for(int k=0;k < m_StrokeList[i].size();k++)
				{
					if ((m_StrokeList[i][j].getStrokeId()==m_StrokeList[i][k].getStrokeId()) &&
						(m_StrokeList[i][k].deleted()==false))
					{
						//m_StrokeList[i].removeAt(k);
						//k--;
						doDelete=false;
					}
			
				}
			
				if (doDelete)
					m_StrokeList[i][j].deleteStroke(-1);
			}
		}
	}
	checkStrokeLinks();
	checkForDuplicateStokes();
	checkForSelectionProblems();
}



//delete all the strokes for all the frames
void LayerObject::deleteAllUserStrokes()
{
	m_CurrentStroke = -1;
	
	for(int i=0;i < m_NumFrames;i++)
		m_StrokeList[i].clear();
}


//delete all the strokes for a given frame
void LayerObject::deleteStrokeImages(int frameIndex)
{
	CreateStrokeImage(frameIndex);
	//signal to free the texture
	m_StrokeImageTextureList[frameIndex] = -1*abs(m_StrokeImageTextureList[frameIndex]);
	m_StrokeImageStateList[frameIndex] = FREE_TEXTURE;

	QDir undoDir(m_ProjectDir+QString("undo/"));
	undoDir.remove(m_LayerName+QString("-strokes-")+QString::number(frameIndex)+QString(".img"));


}



//delete all the generated strokes for all the frames
void LayerObject::deleteAllGeneratedStrokes()
{
	m_GeneratedStrokeParameterList.clear();

	for(int i=0;i < m_NumFrames;i++)
	{
		deleteStrokeImages(i);
		

		
		m_GeneratedStrokeList[i].clear();
		

	}
}

void LayerObject::refreshAllStrokeImages()
{
	LOG_STR_COUT("refreshing stroke frames");
	for(int i=0;i < m_NumFrames;i++)
	{
		
		m_StrokeImageStateList[i] = CREATE_TEXTURE;
	}
}




QImage& LayerObject::getStrokeImage(int frameIndex)
{
	return m_StrokeImageList[frameIndex];
}


void LayerObject::setStrokeImage(int frameIndex,QImage& img, int generateId)
{


	QDir projectDir(m_ProjectDir);
    projectDir.mkdir(QString("undo"));

	const uchar *data =  m_StrokeImageList[frameIndex].bits();	
	QString imgfname = m_ProjectDir+QString("undo/")+m_LayerName+QString("-frame-")+QString::number(frameIndex)+QString("-")+QString::number(generateId)+QString(".img");

	FILE * pFile = fopen (imgfname.toAscii(),"wb");
	if (pFile!=NULL)
	{
		int w = m_StrokeImageList[frameIndex].width() ;
		int h = m_StrokeImageList[frameIndex].height() ;
		fwrite(&w , sizeof(int) , 1, pFile );
		fwrite(&h , sizeof(int) , 1, pFile );
	    fwrite(data , sizeof(uchar) , m_StrokeImageList[frameIndex].numBytes(), pFile );
	    fclose (pFile);
	}


	QImage newimg = img.convertToFormat(QImage::Format_ARGB32_Premultiplied);
	m_StrokeImageList[frameIndex] = newimg;

}


QList<Stroke *> LayerObject::getVisibleStrokes(int frameIndex,bool getGuide,bool getColor)
{
	QList<Stroke *> userStrokes;
	for (int i=0;i < m_StrokeList[frameIndex].size();i++)
	{
		if ((m_StrokeList[frameIndex][i].deleted()==false) && (m_StrokeList[frameIndex][i].getStroke()->properties.isVisible==true))
		{
			if ((m_StrokeList[frameIndex][i].IsColorStroke() == true) && (getColor==false))
				continue;

			if ((m_StrokeList[frameIndex][i].IsGuideStroke() == true) && (getGuide==false))
				continue;	
			
			userStrokes.push_back(m_StrokeList[frameIndex][i].getStroke());

		}
	}
	return userStrokes;
}

	

QList<Stroke *> LayerObject::getColorStrokes(int frameIndex)
{
	QList<Stroke *> colorStrokes;
	for (int i=0;i < m_StrokeList[frameIndex].size();i++)
	{
		if ((m_StrokeList[frameIndex][i].deleted()==false) && (m_StrokeList[frameIndex][i].IsColorStroke() == true))
			colorStrokes.push_back(m_StrokeList[frameIndex][i].getStroke());
	}
	return colorStrokes;
}

QList<Stroke *> LayerObject::getGuideStrokes(int frameIndex)
{
	QList<Stroke *> guideStrokes;
	for (int i=0;i < m_StrokeList[frameIndex].size();i++)
	{
		if ((m_StrokeList[frameIndex][i].deleted()==false) && (m_StrokeList[frameIndex][i].IsGuideStroke() == true))
			guideStrokes.push_back(m_StrokeList[frameIndex][i].getStroke());
	}
	return guideStrokes;
}



void LayerObject::eraseUserStrokes(int frameIndex, QPointF& p1,QPointF& p2,int strokeWidth)
{
	/*
	m_Painter.setPen(QPen(QColor(255,255,255,255),strokeWidth, Qt::SolidLine, Qt::RoundCap,Qt::MiterJoin));

	if (m_CurrentPaintFrame < 0)
		return;

	m_Painter.setBrush(QBrush(QColor(255,255,255,255), Qt::SolidPattern));

	const QRect bbox = boundingBox(m_CurrentPaintFrame);

	QPointF boundingBoxP1(p1.x()-(float)bbox.left(),p1.y()-(float)bbox.top());
	QPointF boundingBoxP2(p2.x()-(float)bbox.left(),p2.y()-(float)bbox.top());

        sprintf(_strbuf, "bbox %i %i %i %i", bbox.left(), bbox.right(), bbox.top(), bbox.bottom());
        sprintf(_strbuf, "p1 %f %f p2 %f %f", boundingBoxP1.x(), boundingBoxP1.y(), boundingBoxP2.x(), boundingBoxP2.y());

	m_Painter.drawLine(boundingBoxP1,boundingBoxP2);
	m_StrokeImageStateList[m_CurrentPaintFrame] = CREATE_TEXTURE;
	*/
		
	Vec2f eraseDisp(p2.x()-p1.x(), p2.y()-p1.y());

	int pointCount = 0;
	int pointThresh = 5;

	for (int i=0;i < m_StrokeList[frameIndex].size();i++)
	{	
		QList<QPointF> pts = m_StrokeList[frameIndex][i].PointList();
		
		QList<bool> toDelete;
		

		for(int j=0;j < pts.size();j++)
		{

			Vec2f strokeEraseDisp(pts[j].x()-p1.x(), pts[j].y()-p1.y());

			if (strokeEraseDisp.Len() < strokeWidth)
			{
				toDelete.push_back(true);
				
			}
			else
				toDelete.push_back(false);

			
		}

		for(int j=0;j < pts.size();j++)
		{
			if (toDelete[j] == true)
			{
				pointCount++;

				if (pointCount > pointThresh)
					return;

				m_StrokeList[frameIndex][i].removePoint(j);

				ContourLabel *nextLabel = m_StrokeList[frameIndex][i].getNext();
				while(nextLabel!=0)
				{
					nextLabel->removePoint(j);
					nextLabel = nextLabel->getNext();
				}
				ContourLabel *prevLabel = m_StrokeList[frameIndex][i].getPrev();
				while(prevLabel!=0)
				{
					prevLabel->removePoint(j);
					prevLabel = prevLabel->getPrev();
				}	
			}
		}
	
		if (m_StrokeList[frameIndex][i].PointList().size() <=1)
		{
			int strokeId = m_StrokeList[frameIndex][i].getStrokeId();
			for(int k=0;k < m_NumFrames;k++)
			{
				for(int j=0;j < m_StrokeList[k].size();j++)
				{
					if (m_StrokeList[k][j].getStrokeId() == strokeId)
					{
						deleteStroke(k,j);
						//m_StrokeList[k].removeAt(j);
					}
				}
			}

			if (i == m_CurrentStroke)
				m_CurrentStroke = -1;
		}
		

	}

}


void LayerObject::deselectAllStrokes()
{
	
	for(int i=0;i < m_NumFrames;i++)
	for(int k=0;k < m_StrokeList[i].size();k++)
		m_StrokeList[i][k].groupDeselectContour();

}


void LayerObject::selectMultipleContours(int curFrameIndex, QPointF point, int strokeWidth, bool selecting)
{
	QVector<int> toSelect;

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].deleted()==true)
			continue;

		if (m_StrokeList[curFrameIndex][j].groupSelected()==selecting)
			continue;

		float dist = m_StrokeList[curFrameIndex][j].distanceToContour(point);
		if (dist < strokeWidth)
		{
            //sprintf(_strbuf, "matched stroke %i with distance %f",j, dist);
			toSelect.push_back(m_StrokeList[curFrameIndex][j].getStrokeId());
		}
	}

	for(int i=0;i < m_NumFrames;i++)
	{

		for(int k=0;k < m_StrokeList[i].size();k++)
		{

			for (int n=0;n < toSelect.size();n++)
			{
				if (m_StrokeList[i][k].getStrokeId() == toSelect[n])
				{

					if (selecting)
						m_StrokeList[i][k].groupSelectContour();
					else
						m_StrokeList[i][k].groupDeselectContour();

				}
			}
		}
	}
}



void LayerObject::pushSelectedStrokes(int curFrameIndex, int strokeWidth, QPointF& p1,QPointF& p2)
{
	Vec2f pushDisp(p2.x()-p1.x(), p2.y()-p1.y());



	for(int i=0;i<m_StrokeList[curFrameIndex].size();i++)
	{
	
		if (m_StrokeList[curFrameIndex][i].groupSelected())
		{

			QList<QPointF> pts = m_StrokeList[curFrameIndex][i].PointList();
			
			for(int j=0;j < pts.size();j++)
			{

				Vec2f strokePushDisp(pts[j].x()-p1.x(), pts[j].y()-p1.y());

				if (strokePushDisp.Len() < strokeWidth)
				{
					m_StrokeList[curFrameIndex][i].updatePoint(QPointF(pts[j].x()+pushDisp.x(), pts[j].y()+pushDisp.y()), j);
				}
			}
			
			m_StrokeList[curFrameIndex][i].recomputeStroke();
		}
	}

}




void LayerObject::saveChangedStrokes(int curFrameIndex, int actionId,bool checkIfMoved, int saveStrokeId)
{

	QList<int> saveStrokeIds;


    QList<QList<ContourLabel> > m_PreviousStrokeList;

    QVector<QList<ContourLabel> > frameStrokeVector(m_NumFrames);
    m_PreviousStrokeList=QList<QList<ContourLabel> >::fromVector(frameStrokeVector);

	for(int i=0;i < m_NumFrames;i++)
	{
		QList<ContourLabel> contourList;
		m_PreviousStrokeList[i] = contourList;
	}



	for(int i=0;i<m_StrokeList[curFrameIndex].size();i++)
	{
		if (m_StrokeList[curFrameIndex][i].groupSelected())
		{

			int origStroke=-1;

			for (int j=0;j < m_StrokeListCopy.size();j++)
			{
				if (m_StrokeListCopy[j].getStrokeId() == m_StrokeList[curFrameIndex][i].getStrokeId())
				{
					if (origStroke!=-1)
					{
						LOG_STR_COUT("ERROR: duplicate match found");
						return;
					}
					origStroke = j;
				}
			}
			if (origStroke==-1)
			{
				LOG_STR_COUT("ERROR: Couldn't find previous stroke");
				return;
			}




			QList<QPointF> pts = m_StrokeList[curFrameIndex][i].PointList();
			QList<QPointF> origpts = m_StrokeListCopy[origStroke].PointList();

			bool strokeMoved= false;
			for(int j=0;j < pts.size();j++)
			{
				Vec2f strokePushDisp(pts[j].x()-origpts[j].x(), pts[j].y()-origpts[j].y());
				if ((m_StrokeList[curFrameIndex][i].labeled(j)==true) && (strokePushDisp.Len()>0))
				{
					strokeMoved = true;
				}
			}

			if ((checkIfMoved==true)&& (strokeMoved==false) && (saveStrokeId!=m_StrokeList[curFrameIndex][i].getStrokeId()))
				continue;


			//copy this stroke from the saved list
			m_PreviousStrokeList[curFrameIndex].push_back(m_StrokeListCopy[origStroke]);

			//copy all the strokes before/after before propagating the point
			for(int f=0;f < m_NumFrames;f++)
			for(int k=0;k < m_StrokeList[f].size();k++)
			{
				if ((f==curFrameIndex) || (m_StrokeList[f][k].deleted()==true))
					continue;

				if (m_StrokeList[f][k].getStrokeId() == m_StrokeList[curFrameIndex][i].getStrokeId())
				{
					m_PreviousStrokeList[f].push_back(m_StrokeList[f][k]);	
				}
			}
		}
	}


	m_PreviousStrokeLists.insert(actionId,m_PreviousStrokeList);
}




void LayerObject::propagatePushedPoints(int curFrameIndex, int actionId)
{

	saveChangedStrokes(curFrameIndex, actionId,true);


	int firstFrame,lastFrame;
	bool IsForwardLabeled,IsBackwardLabeled;


	for(int i=0;i<m_StrokeList[curFrameIndex].size();i++)
	{
		if (m_StrokeList[curFrameIndex][i].groupSelected())
		{

			int origStroke=-1;

			for (int j=0;j < m_StrokeListCopy.size();j++)
			{
				if (m_StrokeListCopy[j].getStrokeId() == m_StrokeList[curFrameIndex][i].getStrokeId())
				{
					if (origStroke!=-1)
					{
						LOG_STR_COUT("ERROR: duplicate match found");
						return;
					}
					origStroke = j;
				}
			}
			if (origStroke==-1)
			{
				LOG_STR_COUT("ERROR: Couldn't find previous stroke");
				return;
			}



			QList<QPointF> pts = m_StrokeList[curFrameIndex][i].PointList();
			QList<QPointF> origpts = m_StrokeListCopy[origStroke].PointList();

			bool strokeMoved= false;
			for(int j=0;j < pts.size();j++)
			{
				Vec2f strokePushDisp(pts[j].x()-origpts[j].x(), pts[j].y()-origpts[j].y());
				if ((m_StrokeList[curFrameIndex][i].labeled(j)==true) && (strokePushDisp.Len()>0))
				{
					strokeMoved = true;
				}
			}

			if (strokeMoved==false)
				continue;

		    sprintf(_strbuf, "propagating changed points");
			LOG_STR_COUT(_strbuf);
			//propagate the changed points
			for(int j=0;j < pts.size();j++)
			{
				Vec2f strokePushDisp(pts[j].x()-origpts[j].x(), pts[j].y()-origpts[j].y());
				if ((m_StrokeList[curFrameIndex][i].labeled(j)==true) && (strokePushDisp.Len()>0))
				{
					firstFrame=findContourFirstLocalLabeledFrame(m_StrokeList[curFrameIndex][i],curFrameIndex,j,IsBackwardLabeled);
					lastFrame=findContourLastLocalLabeledFrame(m_StrokeList[curFrameIndex][i],curFrameIndex,j,IsForwardLabeled);
					propagateContourUpdatePoint(m_StrokeList[curFrameIndex][i],curFrameIndex,j,lastFrame,IsForwardLabeled,firstFrame,IsBackwardLabeled);
				}
			}
			
			m_StrokeList[curFrameIndex][i].recomputeStroke();
		}
	}
}



void LayerObject::shiftSelectedStrokes(int curFrameIndex, QPointF shift)
{

        //sprintf(_strbuf, "shifting selected from frame %i with id %i", curFrameIndex,deleteId);

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
	
		if (m_StrokeList[curFrameIndex][j].groupSelected())
		{
			m_StrokeList[curFrameIndex][j].shiftContour(shift);	
			//m_StrokeList[curFrameIndex][j].getStroke()->forceRecompute();
			m_StrokeList[curFrameIndex][j].setUniformLabels(true);
		}
	}
	
}


void LayerObject::fixSelectedStrokes(int curFrameIndex)
{
	if ((curFrameIndex < 0) || (curFrameIndex >= m_NumFrames))
		return;

	bool setLabelsVal = false;

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].groupSelected())
		{

			for (int i=0; i < m_StrokeList[curFrameIndex][j].numPoints();i++)
			{
				if (m_StrokeList[curFrameIndex][j].labeled(i)==false)
					setLabelsVal = true;
			}	
		}
	}

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].groupSelected())
		{
			m_StrokeList[curFrameIndex][j].setUniformLabels(setLabelsVal);
		}
	}
}



void LayerObject::deleteSelectedStrokes(int curFrameIndex, int deleteId,int startDeleteFrame, int endDeleteFrame, int *firstFrame, int *lastFrame)
{

	if ((startDeleteFrame < 0) || (startDeleteFrame >= m_NumFrames))
		return;
	else if ((endDeleteFrame < 0) || (endDeleteFrame >= m_NumFrames))
		return;

	QVector<int> toDelete;

        sprintf(_strbuf, "deleting selected from frame %i with id %i", curFrameIndex,deleteId);
	LOG_STR_COUT(_strbuf);

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
	
		if (m_StrokeList[curFrameIndex][j].groupSelected())
		{
				toDelete.push_back(m_StrokeList[curFrameIndex][j].getStrokeId());

				if (j == m_CurrentStroke)
					 m_CurrentStroke =-1;
		}
	}
	for(int i=startDeleteFrame;i <= endDeleteFrame;i++)
	{

		for(int k=0;k < m_StrokeList[i].size();k++)
		{

			for (int n=0;n < toDelete.size();n++)
			{
				if (m_StrokeList[i][k].getStrokeId() == toDelete[n])
				{
					if (i < *firstFrame)
						*firstFrame = i;

					if (i > *lastFrame)
						*lastFrame = i;

					m_StrokeList[i][k].deleteStroke(deleteId);
				}
			}
		}
	}

}
	




void LayerObject::regularizeContour(const ContourLabel& lastFrameContour,ContourLabel& currFrameContour,float prevStrokeWeight, float flowWeight)
{

	float prevStrokeFactor = prevStrokeWeight/ ( prevStrokeWeight+flowWeight);
	float flowFactor = flowWeight/ ( prevStrokeWeight+flowWeight);

        //sprintf(_strbuf, "factors %f %f", prevStrokeFactor,flowFactor);

	QList<QPointF> oldptlist = lastFrameContour.PointList();

	float offsetx, offsety;

	QList<QPointF> currptlist = currFrameContour.PointList();

	//do first point by looping to compare with last
	float currdiffx = currptlist[0].x()-currptlist[currptlist.size()-1].x();
	float currdiffy = currptlist[0].y()-currptlist[currptlist.size()-1].y();

	float olddiffx = oldptlist[0].x()-oldptlist[currptlist.size()-1].x();
	float olddiffy = oldptlist[0].y()-oldptlist[currptlist.size()-1].y();

	offsetx = prevStrokeFactor*olddiffx + flowFactor*currdiffx;
	offsety = prevStrokeFactor*olddiffy + flowFactor*currdiffy;

	currFrameContour.updatePoint(QPointF(currptlist[currptlist.size()-1].x()+offsetx,currptlist[currptlist.size()-1].y()+offsety),0,false);


	for (int p=1;p < oldptlist.size();p++)
	{
		QList<QPointF> currptlist = currFrameContour.PointList();
	
		float currdiffx = currptlist[p].x()-currptlist[p-1].x();
		float currdiffy = currptlist[p].y()-currptlist[p-1].y();

		float olddiffx = oldptlist[p].x()-oldptlist[p-1].x();
		float olddiffy = oldptlist[p].y()-oldptlist[p-1].y();

		offsetx = prevStrokeFactor*olddiffx + flowFactor*currdiffx;
		offsety = prevStrokeFactor*olddiffy + flowFactor*currdiffy;

		//testing...
		if (!((flowFactor ==10) && (prevStrokeFactor==0)))
			currFrameContour.updatePoint(QPointF(currptlist[p-1].x()+offsetx,currptlist[p-1].y()+offsety),p,false);
	}

}



void LayerObject::undoGeneratedStrokes(int generateId)
{
        sprintf(_strbuf, "undoing with id %i", generateId);
	LOG_STR_COUT(_strbuf);

	m_GeneratedStrokeParameterList.remove(generateId);

	for (int i =m_StartFrame;i < m_EndFrame;i++)
	{

		if (m_GeneratedStrokeList[i].contains(generateId)==false)
			continue;

		

		m_GeneratedStrokeList[i].remove(generateId);

		//QString fname = m_ProjectDir+QString("undo/")+m_LayerName+QString("-frame-")+QString::number(i)+QString("-")+QString::number(generateId)+QString(".img");
		//QString layerfname = ;

		//QDir projectDir(m_ProjectDir+QString("strokeLayers/"));
		//projectDir.remove(m_LayerName+QString("-frame-")+QString::number(i)+QString("-")+QString::number(generateId)+QString(".layer"));

		QString imgfname = m_ProjectDir+QString("undo/")+m_LayerName+QString("-frame-")+QString::number(i)+QString("-")+QString::number(generateId)+QString(".img");

		FILE * pFile;
		pFile = fopen (imgfname.toAscii(),"rb");
		if (pFile!=NULL)
		{

			fseek (pFile , 0 , SEEK_END);
			long lSize = ftell (pFile);
			rewind (pFile);

    sprintf(_strbuf, "file size %i", lSize);
			LOG_STR_COUT(_strbuf);


			uchar *buffer = (uchar*) malloc (sizeof(uchar)*lSize);

			int w ;
			int h ;
			fread(&w , sizeof(int) , 1, pFile );
			fread(&h , sizeof(int) , 1, pFile );


			int readin = fread (buffer , sizeof(uchar),lSize,pFile );
    sprintf(_strbuf, "read in %i, %i %i", readin, feof(pFile), ferror (pFile));
			LOG_STR_COUT(_strbuf);

			if (readin ==lSize-8)
			{

				QImage img(w,h,QImage::Format_ARGB32_Premultiplied);

				int r,g,b,a;
				int m = 0;

				for(int y=0;y < h;y++)
				for(int x=0;x < w;x++)
				{
					
					b = buffer[m];
					g = buffer[m+1];
					r = buffer[m+2];
					a = buffer[m+3];

					img.setPixel(x, y,qRgba ( r, g, b, a ) );
				
					m +=4;
				}
				m_StrokeImageList[i] = img;
				m_StrokeImageStateList[i] = CREATE_TEXTURE;
			}

			free(buffer);
			fclose (pFile);
		}

	}
}

void LayerObject::saveGeneratedStrokes(int curFrameIndex,QVector<Stroke *>& generatedStrokes,int generateId)
{
	
	QDir projectDir(m_ProjectDir);
    projectDir.mkdir(QString("strokeLayers"));

	QString layerfname = m_ProjectDir+QString("strokeLayers/")+m_LayerName+QString("-frame-")+QString::number(curFrameIndex)+QString("-")+QString::number(generateId)+QString(".layer");

	QFile layerfile(layerfname);
	if(layerfile.open(QIODevice::WriteOnly)==false)
	{
		//for(int j=0;j < generatedStrokes.size();j++)
		//	delete(generatedStrokes[j]);

		return;
	}


	int numLayerStroke = generatedStrokes.size();
	layerfile.write((char *)&numLayerStroke,sizeof(int));

	for(int j=0;j < generatedStrokes.size();j++)
	{
		if (generatedStrokes[j]->writeStroke(layerfile)==false)
		{
    sprintf(_strbuf, "problem writing out stroke file at frame %i",j);
			LOG_STR_COUT(_strbuf);
		}
		//delete(generatedStrokes[j]);
	}
	layerfile.close();
	
	QVector<Stroke *> empty;

	m_GeneratedStrokeList[curFrameIndex].insert(generateId, empty);


}
void LayerObject::saveGeneratedStrokesParameters(int generateId,StrokeLayerProperties& slp)
{

	m_GeneratedStrokeParameterList.insert(generateId, slp);
}

void LayerObject::redrawGeneratedStrokes(int curFrameIndex)
{

	//need to read in from file
	/*

	QMap<int,QVector<Stroke *>>::iterator i = m_GeneratedStrokeList[curFrameIndex].begin();
	while (i != m_GeneratedStrokeList[curFrameIndex].end()) 
	{
		QVector<Stroke *>& strokeV = i.value();

		for(int j=0;j < strokeV.size();j++)
		{
			strokeV[j]->render();
		}
		++i;
	}

	*/

}

bool LayerObject::copySelectedStrokes(int curFrameIndex)
{
	m_StrokeListCopy.clear();
	//m_StrokeListCopy = m_StrokeList[curFrameIndex];

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].groupSelected())
			m_StrokeListCopy.push_back(m_StrokeList[curFrameIndex][j]);

	}

	return (m_StrokeListCopy.size()>0);
}

int LayerObject::getSelectedStrokes(int curFrameIndex,QList<QList<ContourLabel> >& selectedStrokes)
{

	QVector<int> toSelect;

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
		if (m_StrokeList[curFrameIndex][j].groupSelected())
			toSelect.push_back(m_StrokeList[curFrameIndex][j].getStrokeId());
	}

	for (int i =m_StartFrame;i < m_EndFrame;i++)
	{
		for(int j=0;j<m_StrokeList[i].size();j++)
		{
			for (int k=0;k< toSelect.size();k++)
			{
				if (toSelect[k]==m_StrokeList[i][j].getStrokeId())
					selectedStrokes[i].push_back(m_StrokeList[i][j]);
			}
		}
	}

	return toSelect.size();

}

void LayerObject::addNewSelectedStrokes(QList<QList<ContourLabel> >& selectedStrokes)
{

	for (int i=0;i < m_NumFrames;i++)
	{
		for(int j=0;j < selectedStrokes[i].size();j++)
			m_StrokeList[i].append(selectedStrokes[i][j]);
	}
	checkStrokeLinks();
	checkForDuplicateStokes();
	checkForSelectionProblems();
}


void LayerObject::undoChangedStrokes(int actionId)
{

    QList<QList<ContourLabel> > m_PreviousStrokeList = m_PreviousStrokeLists.take(actionId);

	if(m_PreviousStrokeList.size() !=m_NumFrames)
	{
		return;
	}


	QSet<int> previousStrokeIds;

	for(int i=0;i < m_NumFrames;i++)
	for (int n=0;n < m_PreviousStrokeList[i].size();n++)
	{
		previousStrokeIds.insert(m_PreviousStrokeList[i][n].getStrokeId());
	}


	for(int i=0;i < m_NumFrames;i++)
	{

		for(int k=m_StrokeList[i].size()-1;k >=0 ;k--)
		{
			if (!previousStrokeIds.contains(m_StrokeList[i][k].getStrokeId()))
				continue;

			bool foundStroke = false;
			for (int n=0;n < m_PreviousStrokeList[i].size();n++)
			{
				if (m_StrokeList[i][k].getStrokeId() == m_PreviousStrokeList[i][n].getStrokeId())
				{
					m_StrokeList[i][k] = m_PreviousStrokeList[i][n];
					foundStroke = true;
				}
			}

			if ((foundStroke==false)&& (m_StrokeList[i][k].deleted()==false))
			{
				deleteStroke(i,k);
				//m_StrokeList[i].removeAt(k);
                //sprintf(_strbuf, "removing stroke %i from frame %i", k, i);
				LOG_STR_COUT(_strbuf);
			}
		}

	}

	checkStrokeLinks();
	checkForDuplicateStokes();
	checkForSelectionProblems();

}


bool LayerObject::applyOverdrawStroke(int curFrameIndex,QList<QImage>& frameList, double colorUpdate, int actionId, QVector<QPointF>& strokePoints, float imageScaleFactor, float strokeScalingFactor,bool redrawStroke)
{
	float minDist =9999;

	int overdrawnStroke =-1;
	int overdrawnStrokeStartPoint =-1;
	int overdrawnStrokeEndPoint =-1;

	if (strokePoints.size() <=1)
		return false;

	QPointF projectPoint;

	for(int j=0;j<m_StrokeList[curFrameIndex].size();j++)
	{
	
		if (m_StrokeList[curFrameIndex][j].groupSelected())
		{

				float dist = m_StrokeList[curFrameIndex][j].minPointDistance(strokePoints[0],overdrawnStrokeStartPoint);

				if (dist < minDist)
				{
					overdrawnStroke =j;
					minDist = dist;
				}
					 
		}
	}	
	if (overdrawnStroke==-1)
		return false;


	int projectStartPoint, projectEndPoint;

	m_StrokeList[curFrameIndex][overdrawnStroke].minPointDistance(strokePoints[0],overdrawnStrokeStartPoint);
	m_StrokeList[curFrameIndex][overdrawnStroke].minPointDistance(strokePoints[strokePoints.size()-1],overdrawnStrokeEndPoint);


	if (redrawStroke==false)
	{

		
		m_StrokeList[curFrameIndex][overdrawnStroke].minLineSegDistance(strokePoints[0],projectStartPoint,projectPoint);
		m_StrokeList[curFrameIndex][overdrawnStroke].minLineSegDistance(strokePoints[strokePoints.size()-1],projectEndPoint,projectPoint);

		
		if (projectStartPoint==overdrawnStrokeStartPoint)
		{
			overdrawnStrokeStartPoint++;
		}
		if (projectEndPoint+1==overdrawnStrokeEndPoint)
		{
			overdrawnStrokeEndPoint--;
		}
	}
	else
	{

		if (overdrawnStrokeStartPoint < overdrawnStrokeEndPoint)
		{
			overdrawnStrokeStartPoint=0;
			overdrawnStrokeEndPoint=m_StrokeList[curFrameIndex][overdrawnStroke].numPoints()-1;
		}
		else
		{
			overdrawnStrokeStartPoint=m_StrokeList[curFrameIndex][overdrawnStroke].numPoints()-1;
			overdrawnStrokeEndPoint=0;
		}
	}



        //sprintf(_strbuf, "Start %i, end %i", overdrawnStrokeStartPoint, overdrawnStrokeEndPoint);


	if (overdrawnStrokeStartPoint==overdrawnStrokeEndPoint)
		return false;
	else if (overdrawnStrokeEndPoint < overdrawnStrokeStartPoint)
	{	
		

		int temp = overdrawnStrokeStartPoint;
		overdrawnStrokeStartPoint = overdrawnStrokeEndPoint;
		overdrawnStrokeEndPoint = temp;

		QVector<QPointF> flipStrokePoints;
		for (int i=strokePoints.size()-1;i>=0;i--)
		{
			flipStrokePoints.push_back(strokePoints[i]);
		}

		strokePoints = flipStrokePoints;
	}


	ContourLabel overdrawStroke = m_StrokeList[curFrameIndex][overdrawnStroke];


	const QList<QPointF>& ptlist = m_StrokeList[curFrameIndex][overdrawnStroke].PointList();

	overdrawStroke.removeAllPoints();


	QList<QPointF> overStrokePoints;

	for(int i=0;i < overdrawnStrokeStartPoint;i++)
		overdrawStroke.addPoint(ptlist[i],m_StrokeList[curFrameIndex][overdrawnStroke].labeled(i));

	QPointF lastPoint(0,0);

	if (overdrawnStrokeStartPoint>0)
		lastPoint = ptlist[overdrawnStrokeStartPoint-1];

	for(int i=0;i < strokePoints.size();i++)
	{
		int nearPoint=-1;

		float dist = sqrt(pow((strokePoints[i].x()-lastPoint.x()),2) + pow((strokePoints[i].y()-lastPoint.y()),2));
		if (dist > 2*MyPreference::Threshold_StrokePointDistance/sqrt(pow((double)imageScaleFactor,1.75)))
		{
			
			if (redrawStroke==true)
			{
				overStrokePoints.push_back(strokePoints[i]);
				overdrawStroke.addPoint(strokePoints[i],true);
				lastPoint = strokePoints[i];
				continue;
			}

			m_StrokeList[curFrameIndex][overdrawnStroke].minPointDistance(strokePoints[i],nearPoint);

			//m_StrokeList[curFrameIndex][overdrawnStroke].minLineSegDistance(strokePoints[i],nearPoint,projectPoint,-1000);

                        float endDist = min ((double)i,(double) strokePoints.size()-1-i)+1;
			float numTransitionPoints=5;
                        float currStrokeScalingFactor =strokeScalingFactor*min((double) endDist/numTransitionPoints,(double)1.0);
		
			QPointF closestPoint =  m_StrokeList[curFrameIndex][overdrawnStroke].getPoint(nearPoint);
			QPointF newPoint = closestPoint + currStrokeScalingFactor*(strokePoints[i] -closestPoint); 
	
	
			dist = 100;
			if (overdrawStroke.numPoints()>0)
				dist = sqrt(pow((overdrawStroke.getLastPoint().x()-newPoint.x()),2) + pow((overdrawStroke.getLastPoint().y()-newPoint.y()),2));
			if (dist > 2*MyPreference::Threshold_StrokePointDistance/pow((double)imageScaleFactor,1.75))
			{
				overStrokePoints.push_back(newPoint);
				overdrawStroke.addPoint(newPoint,true);
				lastPoint = strokePoints[i];
			}
			
			
		}
	}

	if (overStrokePoints.size()==0)
		return false; 


	//for(int i=0;i < overdrawStroke.numPoints();i++)
	//{
        //	sprintf(_strbuf, "in labelled %i %i", i, overdrawStroke.labeled(i));
	//}

	
	ContourLabel* copyStroke;
	for (int i=0;i < m_StrokeListCopy.size();i++)
	{
		if (m_StrokeListCopy[i].getStrokeId() == m_StrokeList[curFrameIndex][overdrawnStroke].getStrokeId())
		{
			copyStroke = &(m_StrokeListCopy[i]);
		}
	}
	

	ContourLabel *c = &(overdrawStroke);

        //sprintf(_strbuf, "current num points %i copy %i", ptlist.size(),copyStroke->numPoints());


	float oldLength=0, newLength=0;

	float oldCurveDistSum=0;
	float newCurveDistSum=0;
	QPointF disp, disp1, disp2;

	
	for(int i=overdrawnStrokeStartPoint-1;i <= overdrawnStrokeEndPoint;i++)
	{
	
		if ((i==-1) || (ptlist.size() <= i+1 ))
		{
			continue;
		}
		disp=ptlist[i+1]- ptlist[i];
		oldCurveDistSum+=pabs(disp);
	}

	if(oldCurveDistSum <=0.01)
		return false;

	if (overdrawnStrokeStartPoint>0)
	{
		newCurveDistSum+=pabs(overStrokePoints[0]-ptlist[overdrawnStrokeStartPoint-1]);

		disp1= overStrokePoints[0]-ptlist[overdrawnStrokeStartPoint-1];
		newLength=pabs(disp1);
		disp2=ptlist[overdrawnStrokeStartPoint]- ptlist[overdrawnStrokeStartPoint-1];
		oldLength=pabs(disp2);

	}
	for(int i=0;i < overStrokePoints.size()-1;i++)
	{
		disp=overStrokePoints[i+1]- overStrokePoints[i];
		newCurveDistSum+=pabs(disp);
	}
	if (overdrawnStrokeEndPoint+1<ptlist.size())
		newCurveDistSum+=pabs(ptlist[overdrawnStrokeEndPoint+1]-overStrokePoints[overStrokePoints.size()-1]);

	if(newCurveDistSum <=0.01)
		return false;

        //sprintf(_strbuf, "old length %f, new length %f", oldCurveDistSum, newCurveDistSum);

	
	//if (overdrawnStrokeStartPoint>0)
	//	oldLength = pabs(ptlist[overdrawnStrokeStartPoint]- ptlist[overdrawnStrokeStartPoint-1]);
	//float newLength=pabs(overStrokePoints[0]-ptlist[overdrawnStrokeStartPoint-1]);
	int currOldPoint=overdrawnStrokeStartPoint-1, currNewPoint=-1;



	saveChangedStrokes(curFrameIndex, actionId,true,m_StrokeList[curFrameIndex][overdrawnStroke].getStrokeId());


	int addedPoints1 = 0,addedPoints2 = 0,curAddPoint=0;

	while((oldLength+0.01 < oldCurveDistSum) || (newLength+0.01 < newCurveDistSum))
	{
		
                //sprintf(_strbuf, "\nold length %f (%f) new length %f (%f)", oldLength, newLength,oldLength /oldCurveDistSum,newLength/ newCurveDistSum);
	
                //sprintf(_strbuf, "currnewpoint %i curroldpoint %i ",currNewPoint,currOldPoint);
	

		//we need to insert a point into the new stroke
		if (oldLength/oldCurveDistSum < newLength/newCurveDistSum)
		{
			
			float ratio = 1-((newLength/newCurveDistSum - oldLength/oldCurveDistSum) / (pabs(disp1)/newCurveDistSum));

			if (!((ratio >=0)&&(ratio <=1.00)))
			{
				LOG_STR_COUT("prob");
			}

			

			QPointF newPoint;

			if (currNewPoint==-1)
				newPoint= ratio*overStrokePoints[0] + (1-ratio)*ptlist[overdrawnStrokeStartPoint-1];
			else if (overStrokePoints.size()-1 == currNewPoint)
				newPoint= ratio*ptlist[overdrawnStrokeEndPoint+1] + (1-ratio)*overStrokePoints[currNewPoint];
			else if (overStrokePoints.size()-1 >= currNewPoint+1)
				newPoint= ratio*overStrokePoints[currNewPoint+1] + (1-ratio)*overStrokePoints[currNewPoint];
			else
			{
				LOG_STR_COUT("prob");
				break;
			}


			

			overdrawStroke.insertPoint(newPoint,overdrawnStrokeStartPoint+1+currNewPoint+addedPoints1,true);

                //	sprintf(_strbuf, "adding an old point with ratio %f between %i %i at %i",ratio,currNewPoint,currNewPoint+1,currNewPoint+addedPoints1);

			currOldPoint++;

			if (ptlist.size() > currOldPoint+1)
				disp2=ptlist[currOldPoint+1]- ptlist[currOldPoint];

			oldLength+=pabs(disp2);
			

			addedPoints1++;
			curAddPoint=0;
		}
		//we need to insert the point into old strokes
		else if (oldLength/oldCurveDistSum > newLength/newCurveDistSum)
		{
			

			float ratio = 1-((oldLength/oldCurveDistSum-newLength/newCurveDistSum ) / (pabs(disp2)/oldCurveDistSum));

                        //sprintf(_strbuf, "adding a new point with ratio %f",ratio);
			
			if (!((ratio >=0)&&(ratio <=1.00)))
			{
				LOG_STR_COUT("prob");
			}

			QPointF newPoint = ratio*ptlist[currOldPoint+1] + (1-ratio)*ptlist[currOldPoint];

			copyStroke->insertPoint(newPoint,currOldPoint+1+addedPoints2,false);

			ContourLabel *c = copyStroke;
			while ((c= c->getNext()) != 0)
			{

				newPoint = ratio*c->getPoint(currOldPoint+1+addedPoints2) + (1-ratio)*c->getPoint(currOldPoint+addedPoints2-curAddPoint);

				bool labeled = c->labeled(currOldPoint+1+addedPoints2) && c->labeled(currOldPoint+addedPoints2-curAddPoint);
				c->insertPoint(newPoint, currOldPoint+1+addedPoints2,labeled);
				}
			c = copyStroke;
			while ((c= c->getPrev()) != 0)
			{

				bool labeled = c->labeled(currOldPoint+1+addedPoints2) && c->labeled(currOldPoint+addedPoints2-curAddPoint);
				newPoint = ratio*c->getPoint(currOldPoint+1+addedPoints2) + (1-ratio)*c->getPoint(currOldPoint+addedPoints2-curAddPoint);
				c->insertPoint(newPoint, currOldPoint+1+addedPoints2,labeled);
				
			}

			currNewPoint++;
			

			//if (currNewPoint==-1)
			//	disp1=ptlist[overdrawnStrokeEndPoint+1]- overStrokePoints[currNewPoint];
			if (overStrokePoints.size()-1 == currNewPoint)
				disp1=ptlist[overdrawnStrokeEndPoint+1]- overStrokePoints[currNewPoint];
			else if (overStrokePoints.size()-1 >= currNewPoint+1)
				disp1=overStrokePoints[currNewPoint+1]- overStrokePoints[currNewPoint];
			else
			{
				sprintf(_strbuf, "prob, overstroke size %i, currnewpoint %i",overStrokePoints.size(),currNewPoint);
				LOG_STR_COUT(_strbuf);
			}
			newLength+=pabs(disp1);
			
			curAddPoint++;
			addedPoints2++;
		}
		//matches fine, so just skip this point
		else
		{
                        //sprintf(_strbuf, "skipping point");
			currOldPoint++;
			currNewPoint++;

			if (overStrokePoints.size() <= currNewPoint+1)
				disp1=ptlist[overdrawnStrokeEndPoint+1]- overStrokePoints[currNewPoint];
			else
				disp1=overStrokePoints[currNewPoint+1]- overStrokePoints[currNewPoint];

			disp2=ptlist[currOldPoint+1]- ptlist[currOldPoint];
		

			oldLength+=pabs(disp2);
			newLength+=pabs(disp1);
			
			curAddPoint=0;
		}

	}



	for(int i=overdrawnStrokeEndPoint+1;i < ptlist.size();i++)
	{
		overdrawStroke.addPoint(ptlist[i],m_StrokeList[curFrameIndex][overdrawnStroke].labeled(i));
	}
	

	c = &(overdrawStroke);
	while ((c= c->getNext()) != 0)
	{
                //sprintf(_strbuf, "forward num points %i", );
		if (overdrawStroke.numPoints()!=c->numPoints())
		{

    sprintf(_strbuf, "problem, the number of points from previous frame %i doesn't match current %i", c->numPoints(), overdrawStroke.numPoints());
			LOG_STR_COUT(_strbuf);

			
			undoChangedStrokes(actionId);
			return false;
		}
	}
	c = &(overdrawStroke);
	while ((c= c->getPrev()) != 0)
	{
		if (overdrawStroke.numPoints()!=c->numPoints())
		{
    sprintf(_strbuf, "problem, the number of points from previous frame %i doesn't match current %i", c->numPoints(), overdrawStroke.numPoints());
			LOG_STR_COUT(_strbuf);

			undoChangedStrokes(actionId);
			return false;
		}
	}

        //sprintf(_strbuf, "current num points %i copy %i", overdrawStroke.numPoints(),copyStroke->numPoints());

	m_StrokeList[curFrameIndex][overdrawnStroke] = overdrawStroke;

	m_StrokeList[curFrameIndex][overdrawnStroke].recomputeStroke();


	

	//propagate all changes to stroke 

	m_CurrentStroke = overdrawnStroke;


	for(int i=0;i < overdrawStroke.numPoints();i++)
	{
		selectedPointIndex = i;
		propagateUpdatePoint(curFrameIndex,false);
	}

	updateSelectedStrokeColor(curFrameIndex,frameList,colorUpdate,m_StrokeList[curFrameIndex][overdrawnStroke].getStrokeId());
	
	m_StrokeList[curFrameIndex][overdrawnStroke].removeRedundantPoints(imageScaleFactor);

	m_StrokeList[curFrameIndex][overdrawnStroke].recomputeStroke();

        //sprintf(_strbuf, "new num points %i", m_StrokeList[curFrameIndex][overdrawnStroke].numPoints());

	return true;

}

FlowField& LayerObject::layerFlow(int frame) 
{ 
	if(flowExists(frame)==false)
	{
		LOG_STR_COUT("error, accessing flow which doesn't exist");
	}
		
	return m_LayerMotionList[frame].flowFieldList()[m_SelectedRow];
}

FlowField& LayerObject::layerReverseFlow(int frame) 
{
	if(reverseFlowExists(frame)==false)
	{
                //sprintf(_strbuf, "error, accessing reverse flow which doesn't exist");
        FlowField newFlow;
        return newFlow;
	}
	return m_LayerReverseMotionList[frame].flowFieldList()[m_SelectedRow];
}

int LayerObject::maxGenerateStrokeId()
{

	int maxGenId=-1;

	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	{
		
        QMap<int,QVector<Stroke *> >::iterator itr = m_GeneratedStrokeList[i].begin();
		while (itr != m_GeneratedStrokeList[i].end()) 
		{
			int genId = itr.key();

			if (genId > maxGenId)
				maxGenId=genId;

			++itr;
		}
	}
	return maxGenId;
}


void LayerObject::showStrokeTypes(bool showColorStroke, bool showGuideStroke,bool showSeedStrokes,  bool showRegularStroke)
{
	for(int i=m_StartFrame;i<=m_EndFrame;i++) 
	for(int k=0;k < m_StrokeList[i].size();k++)
	{
		m_StrokeList[i][k].setHidden(showColorStroke,showGuideStroke,showSeedStrokes,showRegularStroke);
	}

}
	
void LayerObject::printGlobalLabels()
{
	for(int i=0;i < m_IsGlobalLabelList.size();i++)
	{
                sprintf(_strbuf, "%i %i",i,m_IsGlobalLabelList[i]);
	}
}


void LayerObject::removeFrames(int frameIndex, bool forward)
{
	if ((frameIndex<0) || (frameIndex>=m_NumFrames))
		return;
	if (forward)
		m_EndFrame=frameIndex;	
	else
		m_StartFrame=frameIndex;

	for(int i=0; i < m_NumFrames;i++)
	{
		if (InRange(i)==false)
		{
			
			QList<ContourLabel> contourList;
			m_StrokeList[i] = contourList;

			deleteStrokeImages(i);
		
			if (m_BoundaryKDTreeList[i]!=0)
			{
				delete m_BoundaryKDTreeList[i];
				m_BoundaryKDTreeList[i]=0;
			}
			if (m_GuideStrokeKDTreeList[i]!=0)
			{
				delete m_GuideStrokeKDTreeList[i];
				m_GuideStrokeKDTreeList[i]=0;
			}
			if (m_ColorStrokeKDTreeList[i]!=0)
			{
				delete m_ColorStrokeKDTreeList[i];
				m_ColorStrokeKDTreeList[i]=0;
			}	
		
		}
	}
}
