#include <QtGui>
#include <QDomDocument>
#include "MainWindow.h"
#include "QVideoProcessing.h"
#include "ReduceFrameRange.h"


MainWindow::MainWindow(void)
{
	setWindowTitle(tr("AniPaint 0.12"));
        setWindowIcon(QIcon("icons/camera-video.png"));
	resize(900,760);

	// create the inspector window
	inspector=new Inspector(this,Qt::Tool);
	flowFieldViewer=new FlowFieldViewer(this,Qt::Tool);
	magViewer=new MagViewer(this,Qt::Tool);
	paintOptions=new PaintOptions(this,Qt::Tool);
        paintOptions->setGeometry(100   ,100,250,250);

	// creating the scoll area for ImageViewer
    scrollArea = new QScrollArea;
	imageViewer=new ImageViewer(scrollArea);

	
	scrollArea->setBackgroundRole(QPalette::Dark);
	scrollArea->setWidget(imageViewer);
	setCentralWidget(scrollArea);
	connect(imageViewer,SIGNAL(adjustScrollBar(double)),this,SLOT(adaptScrollBar(double)));

	connect(imageViewer,SIGNAL(finishedPainting()),this,SLOT(finishedPainting()));

	pVideoLabeling=new VideoLabeling(this);

	// the widgets for changing the frame index
	spinBoxFrameIndex=new QSpinBox;
	sliderFrameIndex=new QSlider(Qt::Horizontal);
	spinBoxFrameIndex->setRange(1,10);
	sliderFrameIndex->setRange(1,10);
	sliderFrameIndex->setFixedWidth(550);
	connect(spinBoxFrameIndex,SIGNAL(valueChanged(int)),sliderFrameIndex,SLOT(setValue(int)));
	connect(sliderFrameIndex,SIGNAL(valueChanged(int)),spinBoxFrameIndex,SLOT(setValue(int)));
	
	depthControl=new DepthControl(this); 
	 
	sliderFrameIndex->setValue(1);

	//sliderFrameIndex->setTickPosition(QSlider::TicksBothSides);

	strokePainter=new StrokePainter(this);
	strokePainter->hide();


	createActions();
	createMenus();
	createToolBar();

	binding();

	IsProjFileSpecified=false;
	
	//inspector->show();
	//magViewer->show();
	//flowFieldViewer->show();
	paintOptions->show();
	
	//importXML();
	
	currentlyPlaying=false;

	load();

}

MainWindow::~MainWindow(void)
{
	delete pVideoLabeling;
}



void MainWindow::adaptScrollBar(double factor)
{
	QScrollBar *scrollBar;
	scrollBar=scrollArea->horizontalScrollBar();
	scrollBar->setValue(int(factor * scrollBar->value() + ((factor - 1) * scrollBar->pageStep()/2)));

	scrollBar=scrollArea->verticalScrollBar();
	scrollBar->setValue(int(factor * scrollBar->value() + ((factor - 1) * scrollBar->pageStep()/2)));
}

void MainWindow::resizeEvent(QResizeEvent *event)
{
	imageViewer->setParentSize(scrollArea->frameSize());
}

void MainWindow::binding()
{
	connect(pVideoLabeling,SIGNAL(NumFramesChanged(int)),this,SLOT(setFrameIndexRange(int)));
	connect(pVideoLabeling,SIGNAL(ImageChanged(const QImage&)),imageViewer,SLOT(loadImage(const QImage&)));
	connect(pVideoLabeling,SIGNAL(FrameControlChanged(bool,bool,bool,bool)),this,SLOT(setFrameControl(bool,bool,bool,bool)));
	connect(pVideoLabeling,SIGNAL(FrameIndexChanged(int)),sliderFrameIndex,SLOT(setValue(int)));
	connect(pVideoLabeling,SIGNAL(Update()),imageViewer,SLOT(redraw()));
	connect(pVideoLabeling,SIGNAL(changeLayerDepth(const QList<LayerDepth>&,int,int)),this,SLOT(setLayerDepth(const QList<LayerDepth>&,int,int)));
	connect(depthControl,SIGNAL(LayerDepthInterpolated(const LayerDepth&)),pVideoLabeling,SLOT(setLayerDepth(const LayerDepth&)));
	
	
	//connect from pVideoLabeling to strokePainter
	connect(pVideoLabeling,SIGNAL(paintStrokes(ImageViewer&, QPainter&, QList<QImage>& ,QList<LayerObject>&, int)),strokePainter,SLOT(paintStrokes(ImageViewer&, QPainter&,QList<QImage>& , QList<LayerObject>&, int)));
	connect(pVideoLabeling,SIGNAL(strokesChanged(int)),strokePainter,SLOT(strokesChanged(int)));
	connect(pVideoLabeling,SIGNAL(NumFramesChanged(int)),strokePainter,SLOT(numFramesChanged(int)));
	connect(pVideoLabeling,SIGNAL(generateStrokes(QString&,LayerObject&, QList<LayerObject>&, QList<QImage>&,QStringList&,int,int,StrokeLayerProperties *,bool)),strokePainter,SLOT(generateStrokes(QString&,LayerObject&,  QList<LayerObject>&,QList<QImage>&,QStringList&,int,int,StrokeLayerProperties *,bool)));
	connect(pVideoLabeling,SIGNAL(renderStrokes(int,int,float, QList<LayerObject>&, QList<QImage>&,QString)),strokePainter,SLOT(renderStrokes(int,int,float, QList<LayerObject>&, QList<QImage>&,QString)));
	
	connect(pVideoLabeling,SIGNAL(frameChanged(const QImage*,const QImage*)),strokePainter,SLOT(frameChanged(const QImage*,const QImage*)));
	connect(pVideoLabeling,SIGNAL(setCurrentColor(QColor&, float, float)),strokePainter,SLOT(setCurrentColor(QColor&,float,float)));
	connect(pVideoLabeling,SIGNAL(redrawStrokes()),strokePainter,SLOT(redraw()));
	connect(pVideoLabeling,SIGNAL(setCurrentColorForFilledStroke(Stroke*, QImage&)),strokePainter,SLOT(setCurrentColorForFilledStroke(Stroke*, QImage&)));
	

	connect(pVideoLabeling,SIGNAL(updatePropertiesForSelectedStroke(StrokeProperties& )),paintOptions,SLOT(updatePropertiesForSelectedStroke(StrokeProperties&)));
	connect(pVideoLabeling,SIGNAL(showUserStrokeOptions(bool)),paintOptions,SLOT(showUserStrokeOptions(bool)));


	//connect from imageViewer to pVideoLabeling
	connect(imageViewer,SIGNAL(viewerMouseEvent(ImageViewer& , ImageViewer::MouseAction& ,QPoint& ,float,Qt::KeyboardModifiers&)),pVideoLabeling,SLOT(ProcessMouseEvent(ImageViewer&, ImageViewer::MouseAction&,QPoint&,float,Qt::KeyboardModifiers&)));
	connect(imageViewer,SIGNAL(paintingImageViewer(ImageViewer&,QPainter&,QImage&)),pVideoLabeling,SLOT(Paint(ImageViewer&,QPainter&,QImage&)));
	

	//connect from main window to pVideoLabeling
	 connect(sliderFrameIndex,SIGNAL(valueChanged(int)),pVideoLabeling,SLOT(gotoFrame(int)));
	 connect(nextFrameAct,SIGNAL(triggered()),pVideoLabeling,SLOT(gotoNextFrame()));
	 connect(preFrameAct,SIGNAL(triggered()),pVideoLabeling,SLOT(gotoPreFrame()));
	 connect(firstFrameAct,SIGNAL(triggered()),pVideoLabeling,SLOT(gotoFirstFrame()));
	 connect(lastFrameAct,SIGNAL(triggered()),pVideoLabeling,SLOT(gotoLastFrame()));
	 connect(playAct,SIGNAL(triggered()),this,SLOT(playpause()));


	 // connect from pVideoLabeling to inspector
	 connect(pVideoLabeling,SIGNAL(loadLayerMotionPara(const MotionPara&)),inspector,SLOT(paraLoaded(const MotionPara&)));
	 connect(pVideoLabeling,SIGNAL(layerDeSelected()),inspector,SLOT(clearPara()));
	 connect(pVideoLabeling,SIGNAL(SearchingWindowSizeChanged(int,int)),inspector,SLOT(setSearchingWindowSize(int,int)));
	 connect(pVideoLabeling,SIGNAL(motionTypeChanged(int)),inspector,SLOT(setMotionType(int)));
	 connect(pVideoLabeling,SIGNAL(setRow(int)),inspector,SLOT(selectRow(int)));
	 connect(pVideoLabeling,SIGNAL(setTab(int)),inspector,SLOT(selectTab(int)));
	 connect(pVideoLabeling,SIGNAL(lambdaChanged(double)),inspector,SLOT(setLambda(double)));

	 // connect from inspector to pVideoLabeling
	 connect(inspector,SIGNAL(paraModified(const MotionPara&)),pVideoLabeling,SLOT(updateLayerMotionPara(const MotionPara&)));
	 connect(inspector,SIGNAL(selectedRow(int)),pVideoLabeling,SLOT(motionParaRowSelected(int)));
	 connect(inspector,SIGNAL(tabChanged(int)),pVideoLabeling,SLOT(tabChanged(int)));
	 connect(inspector,SIGNAL(searchSizeChanged(int)),pVideoLabeling,SLOT(setSearchSize(int)));
	 connect(inspector,SIGNAL(winSizeChanged(int)),pVideoLabeling,SLOT(setWinSize(int)));
	 connect(inspector,SIGNAL(uncertainMatchingChanged(bool)),pVideoLabeling,SLOT(setUncertainMatching(bool)));
	 connect(inspector,SIGNAL(motionTypeSelected(int)),pVideoLabeling,SLOT(setParametricMotionType(int)));
	 connect(inspector,SIGNAL(clearLabeling()),pVideoLabeling,SLOT(clearLabeling()));
	 connect(inspector,SIGNAL(manualMatchingChanged(bool)),pVideoLabeling,SLOT(setManualMatching(bool)));
	 connect(inspector,SIGNAL(redoMatching()),pVideoLabeling,SLOT(redoMatching()));
	 connect(inspector,SIGNAL(lambdaChanged(double)),pVideoLabeling,SLOT(setLambda(double)));
	 connect(inspector,SIGNAL(grabFeatures()),pVideoLabeling,SLOT(grabFeatures()));

	// connect from pVideoLabeling to flowFieldViewer
	connect(pVideoLabeling,SIGNAL(motionSelected(const QImage&, const QImage&, const DImage&, const DImage&)),flowFieldViewer,SLOT(loadImage(const QImage&, const QImage&, const DImage&, const DImage&)));
	connect(pVideoLabeling,SIGNAL(motionSelected(const QImage&, const QImage&)),flowFieldViewer,SLOT(loadImage(const QImage&, const QImage&)));
	connect(pVideoLabeling,SIGNAL(motionSelected(const QImage&)),flowFieldViewer,SLOT(loadImage(const QImage&)));
	 
	// connect from pVideoLabeling to magViewer
	connect(pVideoLabeling,SIGNAL(frameChanged(const QImage*,const QImage*)),magViewer,SLOT(loadImages(const QImage*,const QImage*)));
	connect(pVideoLabeling,SIGNAL(pointSelected(const QPointF&,const QPointF&)),magViewer,SLOT(updatePoints(const QPointF&,const QPointF&)));


	//connect the paint options widget to the stroke painter
	connect(paintOptions,SIGNAL(loadTexture(QString&)),strokePainter,SLOT(loadTextureForCurrentStroke(QString&)));
	connect(paintOptions,SIGNAL(strokeStartWidthChanged(int)),strokePainter,SLOT(strokeStartWidthChanged(int)));
	connect(paintOptions,SIGNAL(strokeStartWidthChanged(int)),pVideoLabeling,SLOT(setStrokeWidth(int)));
	connect(paintOptions,SIGNAL(strokeEndWidthChanged(int)),strokePainter,SLOT(strokeEndWidthChanged(int)));
	connect(paintOptions,SIGNAL(guideStroke(bool)),strokePainter,SLOT(guideStroke(bool)));
	connect(paintOptions,SIGNAL(useImageColor(bool)),strokePainter,SLOT(useImageColor(bool)));
	connect(paintOptions,SIGNAL(useNewColor(QColor)),strokePainter,SLOT(useColor(QColor)));
	connect(paintOptions,SIGNAL(alphaChanged(double)),strokePainter,SLOT(alphaChanged(double)));
	//connect(paintOptions,SIGNAL(colorVarianceChanged(int)),strokePainter,SLOT(colorVarianceChanged(int)));
	connect(paintOptions,SIGNAL(orientationChanged(int)),strokePainter,SLOT(orientationChanged(int)));

	connect(paintOptions,SIGNAL(loadPaperTexture(QString&)),strokePainter,SLOT(loadPaperTexture(QString&)));
	connect(paintOptions,SIGNAL(usePaperColor(QColor)),strokePainter,SLOT(usePaperColor(QColor)));


	connect(paintOptions,SIGNAL(hueOffsetChanged(int)),strokePainter,SLOT(hueOffsetChanged(int)));
	connect(paintOptions,SIGNAL(saturationOffsetChanged(int)),strokePainter,SLOT(saturationOffsetChanged(int)));
	connect(paintOptions,SIGNAL(valueOffsetChanged(int)),strokePainter,SLOT(valueOffsetChanged(int)));
	connect(paintOptions,SIGNAL(hueNoiseChanged(double)),strokePainter,SLOT(hueNoiseChanged(double)));
	connect(paintOptions,SIGNAL(saturationNoiseChanged(double)),strokePainter,SLOT(saturationNoiseChanged(double)));
	connect(paintOptions,SIGNAL(valueNoiseChanged(double)),strokePainter,SLOT(valueNoiseChanged(double)));

	connect(paintOptions,SIGNAL(hueOffsetChanged(int)),pVideoLabeling,SLOT(hueOffsetChanged(int)));
	connect(paintOptions,SIGNAL(saturationOffsetChanged(int)),pVideoLabeling,SLOT(saturationOffsetChanged(int)));
	connect(paintOptions,SIGNAL(valueOffsetChanged(int)),pVideoLabeling,SLOT(valueOffsetChanged(int)));
	connect(paintOptions,SIGNAL(hueNoiseChanged(double)),pVideoLabeling,SLOT(hueNoiseChanged(double)));
	connect(paintOptions,SIGNAL(saturationNoiseChanged(double)),pVideoLabeling,SLOT(saturationNoiseChanged(double)));
	connect(paintOptions,SIGNAL(valueNoiseChanged(double)),pVideoLabeling,SLOT(valueNoiseChanged(double)));

	//color stroke properties
	connect(paintOptions,SIGNAL(colorStroke(bool)),strokePainter,SLOT(colorStroke(bool)));
	connect(paintOptions,SIGNAL(visibleStroke(bool)),strokePainter,SLOT(visibleStroke(bool)));
	connect(paintOptions,SIGNAL(fillStroke(bool)),strokePainter,SLOT(fillStroke(bool)));
	connect(paintOptions,SIGNAL(seedStroke(bool)),strokePainter,SLOT(seedStroke(bool)));
	connect(paintOptions,SIGNAL(addVisibleStrokesToLayer(bool)),strokePainter,SLOT(addVisibleStrokesToLayer(bool)));
	
	//connect(paintOptions,SIGNAL(darkenOccludedRegionsChanged(bool)),strokePainter,SLOT(darkenOccludedRegionsChanged(bool)));
	connect(paintOptions,SIGNAL(shiftToEdgesChanged(bool)),strokePainter,SLOT(shiftToEdgesChanged(bool)));
	connect(paintOptions,SIGNAL(optimizeStrokesChanged(bool)),strokePainter,SLOT(optimizeStrokesChanged(bool)));
	connect(paintOptions,SIGNAL(useStrokesFromAllLayersChanged(bool)),strokePainter,SLOT(useStrokesFromAllLayersChanged(bool)));





	//filling parameters
	connect(paintOptions,SIGNAL(fixedOrientationFactorChanged(int)),strokePainter,SLOT(fixedOrientationFactorChanged(int)));
	connect(paintOptions,SIGNAL(includeReverseDirection(bool)),strokePainter,SLOT(includeReverseDirection(bool)));
	connect(paintOptions,SIGNAL(followBoundaryFactorChanged(int)),strokePainter,SLOT(followBoundaryFactorChanged(int)));
	connect(paintOptions,SIGNAL(followImageFactorChanged(int)),strokePainter,SLOT(followImageFactorChanged(int)));
	connect(paintOptions,SIGNAL(followUserStrokesFactorChanged(int)),strokePainter,SLOT(followUserStrokesFactorChanged(int)));
	connect(paintOptions,SIGNAL(orientationVarianceChanged(double)),strokePainter,SLOT(orientationVarianceChanged(double)));
	connect(paintOptions,SIGNAL(paintOccludedRegionsChanged(bool)),strokePainter,SLOT(paintOccludedRegionsChanged(bool)));
	connect(paintOptions,SIGNAL(startInOccludedRegionsChanged(bool)),strokePainter,SLOT(startInOccludedRegionsChanged(bool)));
	
	connect(paintOptions,SIGNAL(useOccludingLayerFlowChanged(bool)),strokePainter,SLOT(useOccludingLayerFlowChanged(bool)));
	connect(paintOptions,SIGNAL(clipToSeedRegionsChanged(bool)),strokePainter,SLOT(clipToSeedRegionsChanged(bool)));
	connect(paintOptions,SIGNAL(fillEmptyRegionsChanged(bool)),strokePainter,SLOT(fillEmptyRegionsChanged(bool)));
	connect(paintOptions,SIGNAL(fillColorDifferenceChanged(bool)),strokePainter,SLOT(fillColorDifferenceChanged(bool)));
	connect(paintOptions,SIGNAL(fillUserSeedChanged(bool)),strokePainter,SLOT(fillUserSeedChanged(bool)));

	connect(paintOptions,SIGNAL(maxStrokeLengthChanged(int)),strokePainter,SLOT(maxStrokeLengthChanged(int)));
	connect(paintOptions,SIGNAL(minStrokeLengthChanged(int)),strokePainter,SLOT(minStrokeLengthChanged(int)));
	connect(paintOptions,SIGNAL(strokeDensityChanged(int)),strokePainter,SLOT(strokeDensityChanged(int)));
	connect(paintOptions,SIGNAL(densityWidthAddChanged(double)),strokePainter,SLOT(densityWidthAddChanged(double)));
	connect(paintOptions,SIGNAL(densityBrightnessFactorChanged(double)),strokePainter,SLOT(densityBrightnessFactorChanged(double)));
	
	connect(paintOptions,SIGNAL(minBrightnessChanged(int)),strokePainter,SLOT(minBrightnessChanged(int)));
	connect(paintOptions,SIGNAL(maxBrightnessChanged(int)),strokePainter,SLOT(maxBrightnessChanged(int)));
	
	connect(paintOptions,SIGNAL(magnitudeThresholdChanged(int)),strokePainter,SLOT(magnitudeThresholdChanged(int)));
	//connect(paintOptions,SIGNAL(edgeThresholdChanged(double)),strokePainter,SLOT(edgeThresholdChanged(double)));
	
	connect(paintOptions,SIGNAL(colorDifferenceChanged(double)),strokePainter,SLOT(colorDifferenceChanged(double)));
	connect(paintOptions,SIGNAL(maxStrokeAngleChanged(int)),strokePainter,SLOT(maxStrokeAngleChanged(int)));
	connect(paintOptions,SIGNAL(radiusFactorChanged(double)),strokePainter,SLOT(radiusFactorChanged(double)));

	//connect(paintOptions,SIGNAL(fillFramesForwardChanged(int)),strokePainter,SLOT(fillFramesForwardChanged(int)));
	//connect(paintOptions,SIGNAL(fillFramesBackwardChanged(int)),strokePainter,SLOT(fillFramesBackwardChanged(int)));
	connect(paintOptions,SIGNAL(colorUpdateChanged(double)),strokePainter,SLOT(colorUpdateChanged(double)));
	connect(paintOptions,SIGNAL(colorUpdateChanged(double)),pVideoLabeling,SLOT(colorUpdateChanged(double)));
	
	connect(paintOptions,SIGNAL(fadeRateChanged(int)),strokePainter,SLOT(fadeRateChanged(int)));
	connect(paintOptions,SIGNAL(followStrokesFactorChanged(double)),strokePainter,SLOT(followStrokesFactorChanged(double)));
	connect(paintOptions,SIGNAL(followGradientFactorChanged(double)),strokePainter,SLOT(followGradientFactorChanged(double)));
	//connect(paintOptions,SIGNAL(followFlowFactorChanged(double)),strokePainter,SLOT(followFlowFactorChanged(double)));
	
	connect(paintOptions,SIGNAL(orientationFactorChanged(double)),strokePainter,SLOT(orientationFactorChanged(double)));
	connect(paintOptions,SIGNAL(coherenceFactorChanged(double)),strokePainter,SLOT(coherenceFactorChanged(double)));
	connect(paintOptions,SIGNAL(colorFactorChanged(double)),strokePainter,SLOT(colorFactorChanged(double)));
	connect(paintOptions,SIGNAL(rigidityFactorChanged(double)),strokePainter,SLOT(rigidityFactorChanged(double)));
	connect(paintOptions,SIGNAL(densityFactorChanged(double)),strokePainter,SLOT(densityFactorChanged(double)));
	

	connect(paintOptions,SIGNAL(followStrokesFactorChanged(double)),pVideoLabeling,SLOT(followStrokesFactorChanged(double)));
	connect(paintOptions,SIGNAL(followGradientFactorChanged(double)),pVideoLabeling,SLOT(followGradientFactorChanged(double)));
	
	connect(paintOptions,SIGNAL(minFramesChanged(int)),strokePainter,SLOT(minFramesChanged(int)));
	connect(paintOptions,SIGNAL(maxFramesChanged(int)),strokePainter,SLOT(maxFramesChanged(int)));
	connect(paintOptions,SIGNAL(maxFlowAngleChanged(int)),strokePainter,SLOT(maxFlowAngleChanged(int)));
	connect(paintOptions,SIGNAL(maxStrokesChanged(int)),strokePainter,SLOT(maxStrokesChanged(int)));


	connect(paintOptions,SIGNAL(fixedMovementDirectionChanged(int)),strokePainter,SLOT(fixedMovementDirectionChanged(int)));
	connect(paintOptions,SIGNAL(fixedMovementRateChanged(int)),strokePainter,SLOT(fixedMovementRateChanged(int)));
	connect(paintOptions,SIGNAL(fixedMovementDirectionChanged(int)),pVideoLabeling,SLOT(fixedMovementDirectionChanged(int)));
	connect(paintOptions,SIGNAL(fixedMovementRateChanged(int)),pVideoLabeling,SLOT(fixedMovementRateChanged(int)));

	
	connect(paintOptions,SIGNAL(movementVarianceChanged(double)),strokePainter,SLOT(movementVarianceChanged(double)));
	

	connect(paintOptions,SIGNAL(seedOrderChanged( const QString &)),strokePainter,SLOT(seedOrderChanged( const QString &)));

	connect(paintOptions,SIGNAL(optimizeStrokeWidthsChanged(bool)),strokePainter,SLOT(optimizeStrokeWidthsChanged(bool)));
	connect(paintOptions,SIGNAL(oswColorFactorChanged(double)),strokePainter,SLOT(oswColorFactorChanged(double)));
	connect(paintOptions,SIGNAL(oswCoherenceFactorChanged(double)),strokePainter,SLOT(oswCoherenceFactorChanged(double)));
	connect(paintOptions,SIGNAL(oswRegularizationFactorChanged(double)),strokePainter,SLOT(oswRegularizationFactorChanged(double)));
	connect(paintOptions,SIGNAL(oswEdgeFactorChanged(double)),strokePainter,SLOT(oswEdgeFactorChanged(double)));
	connect(paintOptions,SIGNAL(oswGrowFactorChanged(double)),strokePainter,SLOT(oswGrowFactorChanged(double)));

	
	connect(paintOptions,SIGNAL(useBoundaryFlow(bool)),strokePainter,SLOT(useBoundaryFlow(bool)));
	connect(paintOptions,SIGNAL(useStrokeFlow(bool)),strokePainter,SLOT(useStrokeFlow(bool)));
	connect(paintOptions,SIGNAL(oldBlend(bool)),strokePainter,SLOT(oldBlend(bool)));
	connect(paintOptions,SIGNAL(multiPassRender(bool)),strokePainter,SLOT(multiPassRender(bool)));
	
	connect(paintOptions,SIGNAL(startHeavySampleChanged(int)),strokePainter,SLOT(startHeavySampleChanged(int)));
	connect(paintOptions,SIGNAL(endHeavySampleChanged(int)),strokePainter,SLOT(endHeavySampleChanged(int)));
	
	connect(paintOptions,SIGNAL(fixedMovementFactorChanged(int)),strokePainter,SLOT(fixedFlowFactorChanged(int)));
	connect(paintOptions,SIGNAL(followBoundaryMovementFactorChanged(int)),strokePainter,SLOT(boundaryFlowFactorChanged(int)));
	connect(paintOptions,SIGNAL(followOpticalFlowFactorChanged(int)),strokePainter,SLOT(imageFlowFactorChanged(int)));
	connect(paintOptions,SIGNAL(followUserStrokeMovementFactorChanged(int)),strokePainter,SLOT(userFlowFactorChanged(int)));


	connect(paintOptions,SIGNAL(fixedMovementFactorChanged(int)),pVideoLabeling,SLOT(fixedFlowFactorChanged(int)));
	connect(paintOptions,SIGNAL(followBoundaryMovementFactorChanged(int)),pVideoLabeling,SLOT(boundaryFlowFactorChanged(int)));
	connect(paintOptions,SIGNAL(followOpticalFlowFactorChanged(int)),pVideoLabeling,SLOT(imageFlowFactorChanged(int)));
	connect(paintOptions,SIGNAL(followUserStrokeMovementFactorChanged(int)),pVideoLabeling,SLOT(userFlowFactorChanged(int)));

	/*
	//motion parameters

	connect(paintOptions,SIGNAL(directionVarianceChanged(int)),strokePainter,SLOT(directionVarianceChanged(int)));
	connect(paintOptions,SIGNAL(scaleStrokesChanged(bool)),strokePainter,SLOT(scaleStrokesChanged(bool)));
	connect(paintOptions,SIGNAL(minStrokeScaleChanged(int)),strokePainter,SLOT(minStrokeScaleChanged(int)));
	connect(paintOptions,SIGNAL(maxStrokeScaleChanged(int)),strokePainter,SLOT(maxStrokeScaleChanged(int)));
	
	*/

	connect(strokePainter,SIGNAL(strokePropertiesChanged(StrokeProperties&)),pVideoLabeling,SLOT(setStrokeProperties(StrokeProperties&)));
	connect(strokePainter,SIGNAL(strokePropertiesChanged(QString, QString)),pVideoLabeling,SLOT(setStrokeProperties(QString, QString)));

	connect(strokePainter,SIGNAL(generatingStrokes(bool)),flowFieldViewer,SLOT(generatingStrokes(bool)));



	connect(paintOptions,SIGNAL(boundaryBlurChanged(double)),pVideoLabeling,SLOT(boundaryBlurChanged(double)));
	connect(paintOptions,SIGNAL(medianBlurChanged(int)),pVideoLabeling,SLOT(medianBlurChanged(int)));
	
}

void MainWindow::createActions()
{
	//-----------------------------------------------------------------------------------------------------------------
	// the actions of file IO, closing or opening the program
     openFolderAct = new QAction(tr("&Open folder"), this);
     openFolderAct->setShortcut(tr("Ctrl+O"));
     openFolderAct->setIcon(QIcon("icons/Open16.png"));
     connect(openFolderAct, SIGNAL(triggered()), this, SLOT(openFolder()));

	 loadProjectAct=new QAction(tr("&Load project"),this);
	 loadProjectAct->setShortcut(tr("Ctrl+P"));
         loadProjectAct->setIcon(QIcon("icons/folder-open.png"));
	 connect(loadProjectAct,SIGNAL(triggered()),this,SLOT(load()));

	 importXMLAct=new QAction(tr("&Import xml"),this);
	 importXMLAct->setShortcut(tr("Ctrl+I"));
         importXMLAct->setIcon(QIcon("icons/format-indent-less.png"));
	 connect(importXMLAct,SIGNAL(triggered()),this,SLOT(importXML()));

	 exportXMLAct=new QAction(tr("&Export xml"),this);
	 exportXMLAct->setShortcut(tr("Ctrl+E"));
         exportXMLAct->setIcon(QIcon("icons/format-indent-more.png"));
	 connect(exportXMLAct,SIGNAL(triggered()),this,SLOT(exportXML()));


	 saveAct = new QAction(tr("&Save project"),this);
	 saveAct->setShortcut(tr("Ctrl+S"));
         saveAct->setIcon(QIcon("icons/Save16.png"));
	 saveAct->setEnabled(false);
	 connect(saveAct,SIGNAL(triggered()),this,SLOT(save()));

	 saveAsAct=new QAction(tr("Save &as project"),this);
	// saveAsAct->setShortcut(tr("Ctrl+A"));
	 saveAsAct->setEnabled(false);
	 connect(saveAsAct,SIGNAL(triggered()),this,SLOT(saveAs()));


	 runScriptAct=new QAction(tr("&Run script"),this);
	 runScriptAct->setEnabled(true);
	 connect(runScriptAct,SIGNAL(triggered()),this,SLOT(runScript()));

	 exportAct=new QAction(tr("&Export labeling"),this);
	// exportAct->setShortcut(tr("Ctrl+E"));
	 exportAct->setEnabled(false);
	 connect(exportAct,SIGNAL(triggered()),this,SLOT(exportLabeling()));

	 outputDepthAct=new QAction(tr("Output &depth"),this);
	 //outputDepthAct->setShortcut(tr("Ctrl+D"));
	 outputDepthAct->setEnabled(false);
	 connect(outputDepthAct,SIGNAL(triggered()),this,SLOT(outputDepthMap()));

	 outputMotionAct=new QAction(tr("Output &motion"),this);
	// outputMotionAct->setShortcut(tr("Ctrl+M"));
	 outputMotionAct->setEnabled(false);
	 connect(outputMotionAct,SIGNAL(triggered()),this,SLOT(outputMotion()));
	 connect(pVideoLabeling,SIGNAL(setMotionAnalysis(bool)),outputMotionAct,SLOT(setEnabled(bool)));

	 outputLayerMotionAct=new QAction(tr("Output &layer motion"),this);
	// outputLayerMotionAct->setShortcut(tr("Ctrl+K"));
	 outputLayerMotionAct->setEnabled(false);
	 connect(outputLayerMotionAct,SIGNAL(triggered()),this,SLOT(outputLayerMotion()));
	 connect(pVideoLabeling,SIGNAL(setMotionAnalysis(bool)),outputLayerMotionAct,SLOT(setEnabled(bool)));


	 exitAct = new QAction(tr("E&xit"), this);
     exitAct->setShortcut(tr("Ctrl+Q"));
     connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	 //-----------------------------------------------------------------------------------------------------------------
	 // the actions of zoom in, zoom out and set to normal size
     zoomInAct = new QAction(tr("Zoom &In"), this);
	 zoomInAct->setShortcut(tr("="));
     zoomInAct->setEnabled(true);
         zoomInAct->setIcon(QIcon("icons/zoom_in.png"));
     connect(zoomInAct, SIGNAL(triggered()), imageViewer, SLOT(zoomIn()));
	 connect(imageViewer,SIGNAL(setZoomIn(bool)),zoomInAct,SLOT(setEnabled(bool)));

     zoomOutAct = new QAction(tr("Zoom &Out"), this);
	 zoomOutAct->setShortcut(tr("-"));
     zoomOutAct->setEnabled(true);
         zoomOutAct->setIcon(QIcon("icons/zoom_out.png"));
     connect(zoomOutAct, SIGNAL(triggered()), imageViewer, SLOT(zoomOut()));
	 connect(imageViewer,SIGNAL(setZoomOut(bool)),zoomOutAct,SLOT(setEnabled(bool)));

     normalSizeAct = new QAction(tr("&Normal Size"), this);
     normalSizeAct->setShortcut(tr("S"));
     normalSizeAct->setEnabled(true);
         normalSizeAct->setIcon(QIcon("icons/zoom_previous.png"));
     connect(normalSizeAct, SIGNAL(triggered()), imageViewer, SLOT(normalSize()));

	 //-----------------------------------------------------------------------------------------------------------------
	 // the actions of "actions"

	 reduceFrameRangeAct = new QAction(tr("Reduce frame range"),this);
	 //reduceFrameRangeAct->setShortcut(tr("Ctrl+N"));
	 reduceFrameRangeAct->setEnabled(false);
         reduceFrameRangeAct->setIcon(QIcon("icons/ReduceRange.png"));
	 connect(reduceFrameRangeAct,SIGNAL(triggered()),this,SLOT(reduceFrameRange()));

	 //-----------------------------------------------------------------------------------------------------------------
	 // the actions of showing or hiding the video by frames
	showInspectorAct=new QAction(tr("Show inspector"),this);
	showInspectorAct->setShortcut(tr("Ctrl+H"));
	showInspectorAct->setEnabled(true);
	connect(showInspectorAct,SIGNAL(triggered()),inspector,SLOT(show()));

	showFlowFieldViewerAct=new QAction(tr("Show flow field viewer"),this);
	showFlowFieldViewerAct->setEnabled(true);
	connect(showFlowFieldViewerAct,SIGNAL(triggered()),flowFieldViewer,SLOT(show()));

	hideInspectorAct=new QAction(tr("Hide inspector"),this);
	hideInspectorAct->setShortcut(tr("Ctrl+Shift+H"));
	hideInspectorAct->setEnabled(true);
	connect(hideInspectorAct,SIGNAL(triggered()),inspector,SLOT(hide()));

	hideFlowFieldViewerAct=new QAction(tr("Hide flow field viewer"),this);
	hideFlowFieldViewerAct->setShortcut(tr("Ctrl+Shift+F"));
	hideFlowFieldViewerAct->setEnabled(true);
	connect(hideFlowFieldViewerAct,SIGNAL(triggered()),flowFieldViewer,SLOT(hide()));

	 //-----------------------------------------------------------------------------------------------------------------
	 // the actions of showing or hiding various parts of the video
	//showImage=new QAction(tr("Show/Hide Image"),this);
	//showImage->setShortcut(tr("Ctrl+G"));
	//showImage->setEnabled(true);
	//connect(showImage,SIGNAL(triggered()),imageViewer,SLOT(showImage()));

	showStrokeSpines=new QAction(tr("Show Stroke Spines"),this);
	showStrokeSpines->setShortcut(tr("Ctrl+C"));
	showStrokeSpines->setEnabled(true);
	showStrokeSpines->setCheckable(true);
	showStrokeSpines->setChecked(true);
        showStrokeSpines->setIcon(QIcon("icons/strokespine.png"));
	connect(showStrokeSpines,SIGNAL(triggered()),pVideoLabeling,SLOT(showStrokeSpines()));

        /*
	showStrokes=new QAction(tr("Show Painting"),this);
	showStrokes->setShortcut(tr("Ctrl+T"));
	showStrokes->setEnabled(true);
	showStrokes->setCheckable(true);
	showStrokes->setChecked(true);
        showStrokes->setIcon(QIcon("icons/painting.png"));
	connect(showStrokes,SIGNAL(triggered()),pVideoLabeling,SLOT(showStrokes()));
        */

	showLayerBoundaries=new QAction(tr("Show Layer Boundaries"),this);
	showLayerBoundaries->setShortcut(tr("Ctrl+V"));
	showLayerBoundaries->setEnabled(true);
	showLayerBoundaries->setCheckable(true);
	showLayerBoundaries->setChecked(true);
        showLayerBoundaries->setIcon(QIcon("icons/layerspine.png"));
	connect(showLayerBoundaries,SIGNAL(triggered()),pVideoLabeling,SLOT(showLayerBoundaries()));




	showColorStrokes=new QAction(tr("Show Color Strokes"),this);
	showColorStrokes->setEnabled(true);
	showColorStrokes->setCheckable(true);
	showColorStrokes->setChecked(true);
	connect(showColorStrokes,SIGNAL(toggled(bool)),pVideoLabeling,SLOT(showColorStrokes(bool)));

	showGuideStrokes=new QAction(tr("Show Guide Strokes"),this);
	showGuideStrokes->setEnabled(true);
	showGuideStrokes->setCheckable(true);
	showGuideStrokes->setChecked(true);
	connect(showGuideStrokes,SIGNAL(toggled(bool)),pVideoLabeling,SLOT(showGuideStrokes(bool)));


	showSeedStrokes=new QAction(tr("Show Seed Strokes"),this);
	showSeedStrokes->setEnabled(true);
	showSeedStrokes->setCheckable(true);
	showSeedStrokes->setChecked(true);
	connect(showSeedStrokes,SIGNAL(toggled(bool)),pVideoLabeling,SLOT(showSeedStrokes(bool)));

	showRegularStrokes=new QAction(tr("Show Regular Strokes (Not Color/Guide/Seed)"),this);
	showRegularStrokes->setEnabled(true);
	showRegularStrokes->setCheckable(true);
	showRegularStrokes->setChecked(true);
	connect(showRegularStrokes,SIGNAL(toggled(bool)),pVideoLabeling,SLOT(showRegularStrokes(bool)));


	/*
	stroke actions
	*/
	generateStrokeLayer=new QAction(tr("Generate Stroke Layer"),this);
	generateStrokeLayer->setEnabled(true);
    generateStrokeLayer->setIcon(QIcon("icons/redraw.png"));
	generateStrokeLayer->setShortcut(tr("g"));
	connect(generateStrokeLayer,SIGNAL(triggered()),pVideoLabeling,SLOT(generateStrokeLayer()));

	generateStrokeLayersFromList=new QAction(tr("Generate Stroke Layers From Parameter List"),this);
	generateStrokeLayersFromList->setEnabled(true);
    generateStrokeLayersFromList->setIcon(QIcon("icons/redraw.png"));
	connect(generateStrokeLayersFromList,SIGNAL(triggered()),pVideoLabeling,SLOT(generateStrokeLayersFromList()));

	deleteStrokeLayers=new QAction(tr("Delete Stroke Layers"),this);
	deleteStrokeLayers->setEnabled(true);
	deleteStrokeLayers->setShortcut(tr("Ctrl+D"));
	connect(deleteStrokeLayers,SIGNAL(triggered()),pVideoLabeling,SLOT(deleteStrokeLayers()));

	undoLastLayer=new QAction(tr("Undo Last Generated Layer"),this);
	undoLastLayer->setEnabled(true);
	undoLastLayer->setShortcut(tr("l"));
	connect(undoLastLayer,SIGNAL(triggered()),pVideoLabeling,SLOT(undoLastLayer()));

	changeRegionBoundaryBlur=new QAction(tr("Change Region Boundary Blur"),this);
	changeRegionBoundaryBlur->setEnabled(true);
	//changeRegionBoundaryBlur->setShortcut(tr("b"));
	connect(changeRegionBoundaryBlur,SIGNAL(triggered()),pVideoLabeling,SLOT(changeRegionBoundaryBlur()));

	replicateSelectedStrokes=new QAction(tr("Move Selected Strokes Through Time"),this);
	replicateSelectedStrokes->setEnabled(true);
	replicateSelectedStrokes->setShortcut(tr("r"));
        replicateSelectedStrokes->setIcon(QIcon("icons/hsplit.png"));
	connect(replicateSelectedStrokes,SIGNAL(triggered()),pVideoLabeling,SLOT(replicateSelectedStrokes()));

	fixSelectedStrokes=new QAction(tr("Fix Selected Strokes"),this);
	fixSelectedStrokes->setEnabled(true);
	fixSelectedStrokes->setShortcut(tr("Ctrl+F"));
	connect(fixSelectedStrokes,SIGNAL(triggered()),pVideoLabeling,SLOT(fixSelectedStrokes()));

	deleteSelectedStrokes=new QAction(tr("Delete Selected Strokes"),this);
	deleteSelectedStrokes->setEnabled(true);
	QList<QKeySequence> shortcuts;
	shortcuts.push_back(tr("d"));
	shortcuts.push_back(tr("Del"));
    deleteSelectedStrokes->setShortcuts(shortcuts);
    deleteSelectedStrokes->setIcon(QIcon("icons/standardbutton-delete-32.png"));
	connect(deleteSelectedStrokes,SIGNAL(triggered()),pVideoLabeling,SLOT(deleteSelectedStrokes()));

	deleteSelectedStrokesForward=new QAction(tr("Delete Selected Strokes Forward"),this);
	deleteSelectedStrokesForward->setEnabled(true);
	deleteSelectedStrokesForward->setShortcut(tr("f"));
    deleteSelectedStrokesForward->setIcon(QIcon("icons/delete-left.png"));
	connect(deleteSelectedStrokesForward,SIGNAL(triggered()),pVideoLabeling,SLOT(deleteSelectedStrokesForward()));

	deleteSelectedStrokesBackward=new QAction(tr("Delete Selected Strokes Backward"),this);
	deleteSelectedStrokesBackward->setEnabled(true);
	deleteSelectedStrokesBackward->setShortcut(tr("b"));
        deleteSelectedStrokesBackward->setIcon(QIcon("icons/delete-right.png"));
	connect(deleteSelectedStrokesBackward,SIGNAL(triggered()),pVideoLabeling,SLOT(deleteSelectedStrokesBackward()));

	undoLastAction=new QAction(tr("Undo"),this);
	undoLastAction->setEnabled(true);
	undoLastAction->setShortcut(tr("Ctrl+Z"));
        undoLastAction->setIcon(QIcon("icons/edit-clear.png"));
	connect(undoLastAction,SIGNAL(triggered()),pVideoLabeling,SLOT(undoLastAction()));


	increaseStrokeWidthAction=new QAction(tr("Increase Stroke Width"),this);
	increaseStrokeWidthAction->setEnabled(true);
	increaseStrokeWidthAction->setShortcut(tr("w"));
	connect(increaseStrokeWidthAction,SIGNAL(triggered()),paintOptions,SLOT(increaseStrokeWidth()));


	decreaseStrokeWidthAction=new QAction(tr("Decrease Stroke Width"),this);
	decreaseStrokeWidthAction->setEnabled(true);
	decreaseStrokeWidthAction->setShortcut(tr("q"));
	connect(decreaseStrokeWidthAction,SIGNAL(triggered()),paintOptions,SLOT(decreaseStrokeWidth()));

	// strokeActionsMenu->addAction(deleteStrokesForward);
	// strokeActionsMenu->addAction(deleteStrokesBackward);

	renderAction=new QAction(tr("Render Frames"),this);
	renderAction->setEnabled(true);
	renderAction->setShortcut(tr("Ctrl+R"));
        renderAction->setIcon(QIcon("icons/applications-multimedia.png"));
	connect(renderAction,SIGNAL(triggered()),strokePainter,SLOT(renderFrames()));
	connect(renderAction,SIGNAL(triggered()),imageViewer,SLOT(redraw()));

	renderWithLightingAction=new QAction(tr("Render With Lighting"),this);
	renderWithLightingAction->setEnabled(true);
	renderWithLightingAction->setCheckable(true);
	renderWithLightingAction->setChecked(true);
	connect(renderWithLightingAction,SIGNAL(toggled(bool)),strokePainter,SLOT(renderWithLighting(bool)));

	renderWithCurrentTextureAction=new QAction(tr("Use Current Stroke Texture For Rendering"),this);
	renderWithCurrentTextureAction->setEnabled(true);
	renderWithCurrentTextureAction->setCheckable(true);
	renderWithCurrentTextureAction->setChecked(false);
	connect(renderWithCurrentTextureAction,SIGNAL(toggled(bool)),strokePainter,SLOT(renderWithCurrentTexture(bool)));



	 //-----------------------------------------------------------------------------------------------------------------
	 // the actions of viewing the video by frames

	 nextFrameAct=new QAction(tr("Next frame"),this);
	 QList<QKeySequence> mKeyList;
	 mKeyList.append(QKeySequence(tr("y")));
	 mKeyList.append(QKeySequence(tr("Right")));
	 nextFrameAct->setShortcuts(mKeyList);
	 nextFrameAct->setEnabled(false);
         nextFrameAct->setIcon(QIcon("icons/go-next.png"));
	 
	 playAct=new QAction(tr("Play"),this);
	 playAct->setEnabled(false);
	 playAct->setShortcut(tr("Space"));
         playAct->setIcon(QIcon("icons/play.png"));

	/*
	 pauseAct=new QAction(tr("Pause"),this);
	 pauseAct->setEnabled(false);
	 pauseAct->setShortcut(tr("Space"));
         pauseAct->setIcon(QIcon("icons/pause.png"));
 */

	 preFrameAct=new QAction(tr("Previous frame"),this);
	
	 mKeyList.clear();
	 mKeyList.append(QKeySequence(tr("t")));
	 mKeyList.append(QKeySequence(tr("Left")));
	 preFrameAct->setShortcuts(mKeyList);
	 preFrameAct->setEnabled(false);
         preFrameAct->setIcon(QIcon("icons/go-previous.png"));
	 

	 firstFrameAct=new QAction(tr("First frame"),this);
	 firstFrameAct->setShortcut(tr("Home"));
	 firstFrameAct->setEnabled(false);
         firstFrameAct->setIcon(QIcon("icons/go-first.png"));


	 
	 lastFrameAct=new QAction(tr("Last frame"),this);
	 lastFrameAct->setShortcut(tr("End"));
	 lastFrameAct->setEnabled(false);
         lastFrameAct->setIcon(QIcon("icons/go-last.png"));
	
	 //-----------------------------------------------------------------------------------------------------------------
	 // the actions of layering and motion labeling
	
	createLayerAct=new QAction(tr("Create New Region"),this);
	createLayerAct->setEnabled(true);
        createLayerAct->setIcon(QIcon("icons/region-create.png"));
	connect(createLayerAct,SIGNAL(triggered()),pVideoLabeling,SLOT(createLayerMode()));
	 
	generateLayerAct=new QAction(tr("Generate Region Masks"),this);
	generateLayerAct->setEnabled(false);
        generateLayerAct->setIcon(QIcon("icons/animation.png"));
	connect(generateLayerAct,SIGNAL(triggered()),pVideoLabeling,SLOT(GenerateLayers()));

	motionAnalysisAct=new QAction(tr("Motion Analysis"),this);
	motionAnalysisAct->setEnabled(false);
        motionAnalysisAct->setIcon(QIcon("icons/start-here.png"));
	connect(motionAnalysisAct,SIGNAL(triggered()),pVideoLabeling,SLOT(MotionAnalysis()));
	connect(pVideoLabeling,SIGNAL(setMotionAnalysis(bool)),motionAnalysisAct,SLOT(setEnabled(bool)));


	createStrokeAct=new QAction(tr("Draw Stroke"),this);
	createStrokeAct->setEnabled(true);
        createStrokeAct->setIcon(QIcon("icons/paintbrush.png"));
	createStrokeAct->setShortcut(tr("3"));
	connect(createStrokeAct,SIGNAL(triggered()),pVideoLabeling,SLOT(createStrokeMode()));

	overdrawContourAct=new QAction(tr("Fix Stroke"),this);
	overdrawContourAct->setEnabled(true);
        overdrawContourAct->setIcon(QIcon("icons/overdraw.png"));
	overdrawContourAct->setShortcut(tr("4"));
	connect(overdrawContourAct,SIGNAL(triggered()),pVideoLabeling,SLOT(overdrawContourMode()));


	selectContourAct=new QAction(tr("Select Single Stroke/Region"),this);
	selectContourAct->setEnabled(true);
	selectContourAct->setShortcut(tr("1"));
        selectContourAct->setIcon(QIcon("icons/arrow.png"));
	connect(selectContourAct,SIGNAL(triggered()),pVideoLabeling,SLOT(selectContourMode()));


	drawStrokeSelectAct=new QAction(tr("Select Multiple Strokes"),this);
	drawStrokeSelectAct->setEnabled(true);
        drawStrokeSelectAct->setIcon(QIcon("icons/arrow-multiple.png"));
	drawStrokeSelectAct->setShortcut(tr("2"));
	connect(drawStrokeSelectAct,SIGNAL(triggered()),pVideoLabeling,SLOT(selectMultipleStrokeMode()));




	moveSelectedStrokesAct=new QAction(tr("Move Selected Strokes"),this);
	moveSelectedStrokesAct->setEnabled(true);
        moveSelectedStrokesAct->setIcon(QIcon("icons/move.png"));
	moveSelectedStrokesAct->setShortcut(tr("m"));
	moveSelectedStrokesAct->setShortcut(tr("5"));
	connect(moveSelectedStrokesAct,SIGNAL(triggered()),pVideoLabeling,SLOT(moveSelectedStrokes()));

	pushSelectedStrokesAct=new QAction(tr("Push Selected Strokes"),this);
	pushSelectedStrokesAct->setEnabled(true);
        pushSelectedStrokesAct->setIcon(QIcon("icons/select.png"));
	pushSelectedStrokesAct->setShortcut(tr("p"));
	pushSelectedStrokesAct->setShortcut(tr("6"));
	connect(pushSelectedStrokesAct,SIGNAL(triggered()),pVideoLabeling,SLOT(pushSelectedStrokes()));


	colorPickerAct=new QAction(tr("Pick Color"),this);
	colorPickerAct->setEnabled(true);
        colorPickerAct->setIcon(QIcon("icons/dropper.png"));
	colorPickerAct->setShortcut(tr("7"));
	connect(colorPickerAct,SIGNAL(triggered()),pVideoLabeling,SLOT(pickColorMode()));



	//selectAllStrokesAct=new QAction(tr("Select All Strokes"),this);
	//selectAllStrokesAct->setEnabled(true);
	//selectAllStrokesAct->setShortcut(tr("CTRL+A"));
	//connect(drawStrokeSelectAct,SIGNAL(triggered()),pVideoLabeling,SLOT(selectAllStrokes()));


}

void MainWindow::createMenus()
{
	 fileMenu = new QMenu(tr("File"), this);
     fileMenu->addAction(openFolderAct);
	 fileMenu->addAction(loadProjectAct);
	 fileMenu->addAction(importXMLAct);
	 fileMenu->addAction(exportXMLAct);
	 fileMenu->addAction(saveAct);
	 fileMenu->addAction(saveAsAct);
	 fileMenu->addAction(runScriptAct);
	 fileMenu->addAction(exportAct);
	 fileMenu->addAction(outputDepthAct);
	 fileMenu->addAction(outputMotionAct);
	 fileMenu->addAction(outputLayerMotionAct);
	 fileMenu->addAction(exitAct);

	 viewMenu=new  QMenu(tr("View"), this);
     viewMenu->addAction(zoomInAct);
     viewMenu->addAction(zoomOutAct);
     viewMenu->addAction(normalSizeAct);

	 actionMenu=new QMenu(tr("Video Actions"),this);
	 actionMenu->addAction(reduceFrameRangeAct);
	
	 windowMenu=new QMenu(tr("Window"),this);
	 windowMenu->addAction(showInspectorAct);
	 windowMenu->addAction(showFlowFieldViewerAct);
	 windowMenu->addSeparator();
	 windowMenu->addAction(hideInspectorAct);
	 windowMenu->addAction(hideFlowFieldViewerAct);

	 displayMenu=new QMenu(tr("Display"),this);
	 //displayMenu->addAction(showImage);
	 displayMenu->addAction(showStrokeSpines);
         //displayMenu->addAction(showStrokes);
	 displayMenu->addAction(showLayerBoundaries);
	 displayMenu->addSeparator();
	 displayMenu->addAction(showColorStrokes);
	 displayMenu->addAction(showGuideStrokes);
	 displayMenu->addAction(showSeedStrokes);
	 displayMenu->addAction(showRegularStrokes);

	 strokeToolsMenu=new QMenu(tr("Stroke Tools"),this);
	 strokeToolsMenu->addAction(selectContourAct);
	 strokeToolsMenu->addAction(drawStrokeSelectAct);
	 strokeToolsMenu->addAction(createStrokeAct);
	 strokeToolsMenu->addAction(overdrawContourAct);
	 strokeToolsMenu->addSeparator();

	 strokeToolsMenu->addAction(moveSelectedStrokesAct);
	 strokeToolsMenu->addAction(pushSelectedStrokesAct);
	 strokeToolsMenu->addAction(colorPickerAct);


	 strokeActionsMenu=new QMenu(tr("Stroke Actions"),this);
	 strokeActionsMenu->addAction(undoLastAction);
	 strokeActionsMenu->addAction(undoLastLayer);
	 strokeActionsMenu->addSeparator();
	 strokeActionsMenu->addAction(generateStrokeLayer);
	 strokeActionsMenu->addAction(generateStrokeLayersFromList);
	 strokeActionsMenu->addAction(deleteStrokeLayers);
	 
	 strokeActionsMenu->addAction(replicateSelectedStrokes);
	 strokeActionsMenu->addAction(fixSelectedStrokes);
	 strokeActionsMenu->addAction(deleteSelectedStrokes);
	 strokeActionsMenu->addAction(deleteSelectedStrokesForward);
	 strokeActionsMenu->addAction(deleteSelectedStrokesBackward);
	 
	 strokeActionsMenu->addSeparator();
	 strokeActionsMenu->addAction(increaseStrokeWidthAction);
	 strokeActionsMenu->addAction(decreaseStrokeWidthAction);

	 strokeActionsMenu->addSeparator();
	 strokeActionsMenu->addAction(renderAction);
	 strokeActionsMenu->addAction(renderWithLightingAction);
	 strokeActionsMenu->addAction(renderWithCurrentTextureAction);
	 


	 menuBar()->addMenu(fileMenu);
	 menuBar()->addMenu(viewMenu);
	 menuBar()->addMenu(actionMenu);
	 menuBar()->addMenu(windowMenu);
	 menuBar()->addMenu(displayMenu);
	 menuBar()->addMenu(strokeToolsMenu);
	 menuBar()->addMenu(strokeActionsMenu);
}

void MainWindow::createToolBar()
{
	fileToolBar=addToolBar(tr("&File (open, load project, save project)"));
	fileToolBar->addAction(openFolderAct);
	fileToolBar->addAction(loadProjectAct);
	//fileToolBar->addAction(importXMLAct);
	//fileToolBar->addAction(exportXMLAct);
	fileToolBar->addAction(saveAct);

	// create the tool bar for zoom-in and zoom-out
	zoomToolBar=addToolBar(tr("&Zoom (zoom-in, zoom-out, reset), Stroke Tools"));
	zoomToolBar->addAction(zoomInAct);
	zoomToolBar->addAction(zoomOutAct);
	zoomToolBar->addAction(normalSizeAct);
	zoomToolBar->addSeparator();


	//
	zoomToolBar->addAction(createLayerAct);
	zoomToolBar->addAction(generateLayerAct);
	zoomToolBar->addAction(motionAnalysisAct);

	//zoomToolBar->addAction(reduceFrameRangeAct);

	zoomToolBar->addSeparator();
	zoomToolBar->addAction(showStrokeSpines);
	zoomToolBar->addAction(showLayerBoundaries);
        //zoomToolBar->addAction(showStrokes);


	zoomToolBar->addSeparator();
	zoomToolBar->addWidget(new QLabel("Stroke Tools: "));
	zoomToolBar->addAction(selectContourAct);
	zoomToolBar->addAction(drawStrokeSelectAct);
	zoomToolBar->addAction(createStrokeAct);
	zoomToolBar->addAction(overdrawContourAct);
	//zoomToolBar->addSeparator();
	zoomToolBar->addAction(moveSelectedStrokesAct);
	zoomToolBar->addAction(pushSelectedStrokesAct);
	zoomToolBar->addAction(colorPickerAct);


	addToolBarBreak();

	strokeActionToolBar=addToolBar(tr("&Stroke Actions"));



	strokeActionToolBar->addAction(undoLastAction);

	strokeActionToolBar->addSeparator();

	strokeActionToolBar->addWidget(new QLabel("Start Frame "));
	startFrameSpinBox=new QSpinBox;
	startFrameSpinBox->setRange(1,300);
	startFrameSpinBox->setSingleStep(1);
	strokeActionToolBar->addWidget(startFrameSpinBox);
	connect(startFrameSpinBox,SIGNAL(valueChanged(int)),strokePainter,SLOT(requestedStartFrameChanged(int)));

	strokeActionToolBar->addWidget(new QLabel("End Frame "));
	endFrameSpinBox=new QSpinBox;
	endFrameSpinBox->setRange(1,300);
	endFrameSpinBox->setSingleStep(1);
	endFrameSpinBox->setValue(50);
	strokeActionToolBar->addWidget(endFrameSpinBox);
	connect(endFrameSpinBox,SIGNAL(valueChanged(int)),strokePainter,SLOT(requestedEndFrameChanged(int)));

	strokeActionToolBar->addAction(generateStrokeLayer);
	strokeActionToolBar->addSeparator();

	strokeActionToolBar->addWidget(new QLabel("Move Strokes Backward "));
	QSpinBox* fillBackwardSpinBox=new QSpinBox;
	fillBackwardSpinBox->setRange(0,300);
	fillBackwardSpinBox->setSingleStep(1);
	fillBackwardSpinBox->setValue(0);
	strokeActionToolBar->addWidget(fillBackwardSpinBox);
	connect(fillBackwardSpinBox,SIGNAL(valueChanged(int)),pVideoLabeling,SLOT(fillFramesBackwardChanged(int)));

	strokeActionToolBar->addWidget(new QLabel("Forward "));
	QSpinBox* fillForwardSpinBox=new QSpinBox;
	fillForwardSpinBox->setRange(0,300);
	fillForwardSpinBox->setSingleStep(1);
	fillForwardSpinBox->setValue(10);
	strokeActionToolBar->addWidget(fillForwardSpinBox);
	connect(fillForwardSpinBox,SIGNAL(valueChanged(int)),pVideoLabeling,SLOT(fillFramesForwardChanged(int)));

	strokeActionToolBar->addAction(replicateSelectedStrokes);


	strokeActionToolBar->addSeparator();
	strokeActionToolBar->addAction(deleteSelectedStrokesBackward);
	strokeActionToolBar->addAction(deleteSelectedStrokes);
	strokeActionToolBar->addAction(deleteSelectedStrokesForward);
	
	strokeActionToolBar->addSeparator();

	strokeActionToolBar->addWidget(new QLabel("Render Scale "));
	QDoubleSpinBox* renderScaleSpinBox=new QDoubleSpinBox;
	renderScaleSpinBox->setRange(0.1,10);
	renderScaleSpinBox->setDecimals(1);
	renderScaleSpinBox->setSingleStep(0.1);
	renderScaleSpinBox->setValue(2);
	strokeActionToolBar->addWidget(renderScaleSpinBox);
	connect(renderScaleSpinBox,SIGNAL(valueChanged(double)),strokePainter,SLOT(renderScaleChanged(double)));

	strokeActionToolBar->addAction(renderAction);

	//QHBoxLayout *renderLayout = new QHBoxLayout;






	addToolBarBreak();

	transparencyToolBar=new QToolBar(tr("&StrokeTransparencies"));
	//transparencyToolBar->addAction(firstFrameAct);

	

 	QWidget* transparenciesWidget=new QWidget;
	QVBoxLayout *transvLayout= new QVBoxLayout;
	QHBoxLayout *transLayout = new QHBoxLayout;


	QSlider* imageTransparencySlider=new QSlider(Qt::Horizontal);
	imageTransparencySlider->setRange(0,100);
	imageTransparencySlider->setFixedWidth(150);
	connect(imageTransparencySlider,SIGNAL(valueChanged(int)),strokePainter,SLOT(imageTransparencyChanged(int)));
	connect(imageTransparencySlider,SIGNAL(valueChanged(int)),imageViewer,SLOT(redraw()));
	imageTransparencySlider->setValue(100);

	transLayout->addWidget(new QLabel("Image Transparency: "));
	transLayout->addWidget(imageTransparencySlider);

	QSlider* systemStrokeTransparencySlider=new QSlider(Qt::Horizontal);
	systemStrokeTransparencySlider->setRange(0,100);
	systemStrokeTransparencySlider->setFixedWidth(150);
	connect(systemStrokeTransparencySlider,SIGNAL(valueChanged(int)),strokePainter,SLOT(systemStrokeTransparencyChanged(int)));
	connect(systemStrokeTransparencySlider,SIGNAL(valueChanged(int)),imageViewer,SLOT(redraw()));
	
	systemStrokeTransparencySlider->setValue(100);

	transLayout->addWidget(new QLabel("System Stroke Transparency: "));
	transLayout->addWidget(systemStrokeTransparencySlider);

	QSlider* userStrokeTransparencySlider=new QSlider(Qt::Horizontal);
	userStrokeTransparencySlider->setRange(0,100);
	userStrokeTransparencySlider->setFixedWidth(150);
	connect(userStrokeTransparencySlider,SIGNAL(valueChanged(int)),strokePainter,SLOT(userStrokeTransparencyChanged(int)));
	connect(userStrokeTransparencySlider,SIGNAL(valueChanged(int)),imageViewer,SLOT(redraw()));
	
	userStrokeTransparencySlider->setValue(100);

	transLayout->addWidget(new QLabel("User Stroke Transparency: "));
	transLayout->addWidget(userStrokeTransparencySlider);

	transvLayout->addLayout(transLayout);
	transparenciesWidget->setLayout(transvLayout);
	transparencyToolBar->addWidget(transparenciesWidget);




	//-----------------------------------------------------------------------------------------------------------------
	 // create the toolbar for viewing video frames
	 videoToolBar=new QToolBar(tr("&Change frames"));
	 videoToolBar->addAction(firstFrameAct);
	 videoToolBar->addAction(preFrameAct);
	 videoToolBar->addAction(nextFrameAct);
	 videoToolBar->addAction(lastFrameAct);

	 videoToolBar->addAction(playAct);
	 videoToolBar->addSeparator();


 	QWidget* windowFrameIndex=new QWidget;
	//windowFrameIndex->setMaximumWidth(300);
	QLabel* labelFrameIndex=new QLabel(tr("Frame index:"));
	labelFrameIndex->setBuddy(spinBoxFrameIndex);
	QHBoxLayout *layoutFrameIndex = new QHBoxLayout;
	layoutFrameIndex->addWidget(labelFrameIndex);
	layoutFrameIndex->addWidget(spinBoxFrameIndex);
    layoutFrameIndex->addWidget(sliderFrameIndex);
	windowFrameIndex->setLayout(layoutFrameIndex);
	videoToolBar->addWidget(windowFrameIndex);


	
	addToolBarBreak();
	addToolBar(transparencyToolBar);

	addToolBarBreak();
	addToolBar(Qt::BottomToolBarArea,videoToolBar);



}

void MainWindow::setFrameIndexRange(int nFrames)
{
	spinBoxFrameIndex->setRange(1,nFrames);
	sliderFrameIndex->setRange(1,nFrames);
}

void MainWindow::setFrameControl(bool firstFrame,bool preFrame,bool nextFrame,bool lastFrame)
{
	firstFrameAct->setEnabled(firstFrame);
	preFrameAct->setEnabled(preFrame);
	nextFrameAct->setEnabled(nextFrame);
	lastFrameAct->setEnabled(lastFrame);
	playAct->setEnabled(true);
}

void MainWindow::setLayerDepth(const QList<LayerDepth>& layerDepthList,int activeLayerIndex,int frameIndex)
{
	depthControl->loadDepthList(layerDepthList,activeLayerIndex,frameIndex);
	depthControl->exec();
}

void MainWindow::openFolder(QString filespath,QString projectname,QString startFrame,QString endFrame)
{

	if (filespath.isEmpty())
		framePath=QFileDialog::getExistingDirectory(this,"Specify path for video frames",filespath);
	else
		framePath = filespath;
	if(!framePath.isEmpty())
	{
		QFileInfoList fileList=QVideoProcessing::findImageFilesSingleFormat(framePath,startFrame,endFrame);
		if(fileList.size()<2)
		{
			QMessageBox::warning(this,"Warning","No or not enough images found in the folder!");
			return;
		}
		if(framePath.right(1)=="/")
			framePath.chop(1);
		QVideoProcessing::separatePath_FileNames(framePath,projPath,projFileName);
		framePath+="/";
		projFileName+=".mgt";

		pVideoLabeling->Initialize(framePath,fileList);
		IsProjFileSpecified=false;
		saveAct->setEnabled(true);
		saveAsAct->setEnabled(true);
		exportAct->setEnabled(true);
		outputDepthAct->setEnabled(true);
		generateLayerAct->setEnabled(true);
		reduceFrameRangeAct->setEnabled(true);
		selectContourAct->setEnabled(true);
		createStrokeAct->setEnabled(true);
		overdrawContourAct->setEnabled(true);

		pVideoLabeling->GenerateLayers();

		saveAs(projectname,true);


		if(IsProjFileSpecified==false)
		{
			QMessageBox::warning(this,"Warning","You need to save a project file for AniPaint to work properly. If you save now, please restart AniPaint.");
			return;
		}
		strokePainter->clear();
		
		if(pVideoLabeling->readVideoLabeling(projPath+"/"+projFileName)==false)
		{
			QMessageBox::warning(this,"Warning","Fail to load the video labeling MGT file!");
			return;
		}

		QPoint imSize = imageViewer->getImageSize();
		printf("imsize %i %i\n", imSize.x(),imSize.y());
		strokePainter->setSize(imSize.x(), imSize.y());
		strokePainter->setGeometry(0,0,imSize.x(),imSize.y());
        



	}
}

void MainWindow::exportLabeling()
{
	outputPathName=QFileDialog::getExistingDirectory(this,"Specify path for output labeling",outputPathName);
	if(!outputPathName.isEmpty())
	{
		pVideoLabeling->OutputAllLabeling(outputPathName);
	}
}

void MainWindow::outputDepthMap()
{
	if(!pVideoLabeling->isLayerMaskGenerated())
	{
		QMessageBox::warning(this,"Warning","Layers haven't been generated!");
		return;
	}
	outputPathName=QFileDialog::getExistingDirectory(this,"Specify path for output labeling",outputPathName);
	if(!outputPathName.isEmpty())
	{
		pVideoLabeling->outputDepthMap(outputPathName);
	}
}

void MainWindow::outputMotion()
{
	if(!pVideoLabeling->isLayerMaskGenerated())
	{
		QMessageBox::warning(this,"Warning","Layers haven't been generated!");
		return;
	}
	outputPathName=QFileDialog::getExistingDirectory(this,"Specify path for output motion",outputPathName);
	if(!outputPathName.isEmpty())
	{
		pVideoLabeling->outputMotion(outputPathName);
	}
}

void MainWindow::outputLayerMotion()
{
	if(!pVideoLabeling->isLayerMaskGenerated())
	{
		QMessageBox::warning(this,"Warning","Layers haven't been generated!");
		return;
	}
	outputPathName=QFileDialog::getExistingDirectory(this,"Specify path for output layer motion",outputPathName);
	if(!outputPathName.isEmpty())
	{
		pVideoLabeling->outputLayerMotion(outputPathName);
	}
}


void MainWindow::importXML(QString filename)
{

	if (filename.isEmpty())
		filename=QFileDialog::getOpenFileName(this,"Import video labelme XML format",projPath,tr("XML file (*.xml)"));


	if(!filename.isEmpty())
	{
		if(pVideoLabeling->readVideoLabelMeXML(filename)==false)
		{
			QMessageBox::warning(this,"Warning","Fail to load the video labelme xml file!");
			return;
		}
		QVideoProcessing::separatePath_FileNames(filename,projPath,projFileName);
		projFileName.chop(3);
		projFileName+="mgt";
		//IsProjFileSpecified=true;
		saveAct->setEnabled(true);
		saveAsAct->setEnabled(true);
		exportAct->setEnabled(true);
		outputDepthAct->setEnabled(true);
		generateLayerAct->setEnabled(true);
		reduceFrameRangeAct->setEnabled(true);
		selectContourAct->setEnabled(true);
		createStrokeAct->setEnabled(true);
		overdrawContourAct->setEnabled(true);
		

		QPoint imSize = imageViewer->getImageSize();
		printf("imsize %i %i\n", imSize.x(),imSize.y());
		strokePainter->setSize(imSize.x(), imSize.y());
		strokePainter->setGeometry(0,0,imSize.x(),imSize.y());
                pVideoLabeling->GenerateLayers();
	}
}

void MainWindow::exportXML()
{
	QString xmlFileName=projFileName;
	xmlFileName.chop(3);
	xmlFileName+="xml";
	QString filename=QFileDialog::getSaveFileName(this,"Export the labeling to LabelMe XML format",projPath+"/"+xmlFileName,tr("XML file (*.xml)"));
	if(!filename.isEmpty())
	{
		if(pVideoLabeling->writeVideoLabelMeXML(filename)==false)
			QMessageBox::warning(this,"Warning!","Failed to write the VideoLabelMe XML file!");
		else
		{
			//QVideoProcessing::separatePath_FileNames(filename,projPath,projFileName);
			//IsProjFileSpecified=true;
		}
	}
}

void MainWindow::load(QString filename)
{
	 if (filename.isEmpty())
		filename=QFileDialog::getOpenFileName(this,"Load video labeling format",projPath,tr("MGT file (*.MGT)"));
			//filename=QString("D:/peter/Roto/Images/flower/rose/rose-user.mgt");
		//filename=QString("D:/peter/Roto/Images/birdmini/birdmini2.mgt");
		//filename=QString("D:/peter/Roto/Images/bird/bird2-red.mgt");
		//filename = QString("D:/peter/Roto/Images/dustin/dustin2-red.mgt");
	//filename("D:/peter/Roto/Images/tiger/tiger4.mgt");
    //filename("/Users/donovan/dustin/dustin2-red.mgt");
	if(!filename.isEmpty())
	{

		//delete pVideoLabeling;
		//pVideoLabeling=new VideoLabeling(this);

		//delete strokePainter;
		//strokePainter=new StrokePainter(this);
		//strokePainter->hide();

		imageViewer->normalSize();

		//binding();
		strokePainter->clear();
	
		
		if(pVideoLabeling->readVideoLabeling(filename)==false)
		{
			QMessageBox::warning(this,"Warning","Fail to load the video labeling MGT file!");
			return;
		}

		QVideoProcessing::separatePath_FileNames(filename,projPath,projFileName);


		IsProjFileSpecified=true;
		saveAct->setEnabled(true);
		saveAsAct->setEnabled(true);
		exportAct->setEnabled(true);
		outputDepthAct->setEnabled(true);
		generateLayerAct->setEnabled(true);
		reduceFrameRangeAct->setEnabled(true);

		QPoint imSize = imageViewer->getImageSize();
		printf("imsize %i %i\n", imSize.x(),imSize.y());
		strokePainter->setSize(imSize.x(), imSize.y());
		strokePainter->setGeometry(0,0,imSize.x(),imSize.y());
	}
}

void MainWindow::save()
{
	if(IsProjFileSpecified==true)
	{
		if(pVideoLabeling->writeVideoLabeling(projPath+"/"+projFileName)==false)
			QMessageBox::warning(this,"Warning!","Failed to write the VideoLabeling MGT file!");
	

		//VideoLabeling *testVideoLabeling=new VideoLabeling(this);
		//if (testVideoLabeling->readVideoLabeling(projPath+"/"+projFileName)==false)
		//	QMessageBox::warning(this,"Warning!","Problem reading back VideoLabeling MGT file!");
		//delete testVideoLabeling;
	}
	else
		saveAs();

}

void MainWindow::saveAs(QString filename,bool useFramePath)
{
	if (filename.isEmpty())
	{
		if (useFramePath)
			filename=QFileDialog::getSaveFileName(this,"Save the labeling to VideoLabeling format",framePath+"/"+projFileName,tr("MGT file (*.mgt)"));
		else
			filename=QFileDialog::getSaveFileName(this,"Save the labeling to VideoLabeling format",projPath+"/"+projFileName,tr("MGT file (*.mgt)"));
	}
	if(!filename.isEmpty())
	{
		QString oldDir = pVideoLabeling->projectDir();
		if(pVideoLabeling->writeVideoLabeling(filename)==false)
			QMessageBox::warning(this,"Warning!","Failed to write the VideoLabeling MGT file!");
		else
		{
			QVideoProcessing::separatePath_FileNames(filename,projPath,projFileName);
			IsProjFileSpecified=true;
            pVideoLabeling->copyVideoLabelingFiles(oldDir,pVideoLabeling->projectDir());
		}



		//VideoLabeling *testVideoLabeling=new VideoLabeling(this);
		//if (testVideoLabeling->readVideoLabeling(filename)==false)
		//	QMessageBox::warning(this,"Warning!","Problem reading back VideoLabeling MGT file!");
		//delete testVideoLabeling;

	}
}


void MainWindow::runScript(QString filename)
{
	if (filename.isEmpty())
		filename=QFileDialog::getOpenFileName(this,"Import script XML format",projPath,tr("XML file (*.xml)"));

	if (filename.isEmpty())
		return;

	QDomDocument doc("mydocument");
	QFile file(filename);
	if ((!file.open(QIODevice::ReadOnly))||(!doc.setContent(&file)))
	{
		QMessageBox::warning(this,"Warning","Problem with reading in xml file");
		file.close();
		return;
	}
	file.close();

	QDomElement root=doc.documentElement();

	if (root.tagName()=="ActionList")
	{
		QDomNode n=root.firstChild();
		while(!n.isNull())
		{
			QDomElement e=n.toElement();
		
			if(e.tagName()=="LoadFrames")
			{

				QString folderName;
				QString projectName;
				QString startFileName;
				QString endFileName;

				QDomNode obj_n=e.firstChild();
				while(!obj_n.isNull())
				{
					QDomElement el=obj_n.toElement();
					if(el.tagName()=="folderName")
						folderName = el.text();
					if(el.tagName()=="projectName")
						projectName = el.text();
					if(el.tagName()=="startFileName")
						startFileName = el.text();
					if(el.tagName()=="endFileName")
						endFileName= el.text();

					obj_n=obj_n.nextSibling();
				}

				openFolder(folderName,projectName,startFileName,endFileName);
			}
			if(e.tagName()=="LoadProject")
					load(e.text());
			if(e.tagName()=="SaveProject")
					saveAs(e.text());
			if(e.tagName()=="MotionAnalysis")
					pVideoLabeling->MotionAnalysis();
			if(e.tagName()=="GenerateStrokes")
					pVideoLabeling->generateStrokeLayersFromList(e.text());
			if(e.tagName()=="Render")
			{
				float renderScale=1;
				int requestStartFrame=0;
				int requestEndFrame=10000;
				QString outputDir("output");

				QDomNode obj_n=e.firstChild();
				while(!obj_n.isNull())
				{
					QDomElement el=obj_n.toElement();
					if(el.tagName()=="startFrame")
						requestStartFrame = el.text().toInt();
					if(el.tagName()=="endFrame")
						requestEndFrame = el.text().toInt();
					if(el.tagName()=="scale")
						renderScale = el.text().toFloat();
					if(el.tagName()=="outputDirectory")
						outputDir = el.text();

					obj_n=obj_n.nextSibling();
				}
				pVideoLabeling->renderPainting(requestStartFrame,requestEndFrame,renderScale,outputDir);
			}

			n=n.nextSibling();
		}
	}

}


void MainWindow::reduceFrameRange()
{
	ReduceFrameRange dialog(this,pVideoLabeling->NumFrames());
	if(dialog.exec()==QDialog::Accepted)
		pVideoLabeling->reduceFrameRange(dialog.startFrame()-1,dialog.endFrame()-1);
}

void MainWindow::playpause()
{
	if (currentlyPlaying==true)
	{
		currentlyPlaying=false;
		return;
	}

	currentlyPlaying=true;

	pVideoLabeling->deselectBoundary();

	if (sliderFrameIndex->value() < pVideoLabeling->NumFrames()-1)
	{
             time.start();

             sliderFrameIndex->setValue(sliderFrameIndex->value()+1);
	
	}
	else 
	{
		currentlyPlaying=false;
	}
}

void MainWindow::finishedPainting()
{
	if (currentlyPlaying)
	{
		if (sliderFrameIndex->value() < endFrameSpinBox->value())
        {
            int i=1;
            while (time.elapsed() < 80)
            {
                i++;
            }
            sliderFrameIndex->setValue(sliderFrameIndex->value()+1);
            time.start();

        }
		else 
		{
            int i=1;
            while (time.elapsed() < 80)
            {
                i++;
            }
            sliderFrameIndex->setValue(startFrameSpinBox->value());
            time.start();
			//currentlyPlaying=false;

		}
	}
}


