#include "OpticalFlowBidir.h"
#include "ImageProcessing.h"
#include "GaussianPyramid.h"
#include <cstdlib> 
#include <iostream>

using namespace std;

bool OpticalFlowBidir::IsThinPlate=false;
double OpticalFlowBidir::m_Eta=0.5;

DImage OpticalFlowBidir::imdx;
DImage OpticalFlowBidir::imdy;
DImage OpticalFlowBidir::imdt;
DImage OpticalFlowBidir::immask;
DImage OpticalFlowBidir::uu;
DImage OpticalFlowBidir::vv;
DImage OpticalFlowBidir::ux;
DImage OpticalFlowBidir::uy;
DImage OpticalFlowBidir::vx;
DImage OpticalFlowBidir::vy;
DImage OpticalFlowBidir::Phi_1st;
DImage OpticalFlowBidir::Psi_1st;
DImage OpticalFlowBidir::imdxy;
DImage OpticalFlowBidir::imdx2;
DImage OpticalFlowBidir::imdy2;
DImage OpticalFlowBidir::imdtdx;
DImage OpticalFlowBidir::imdtdy;
DImage OpticalFlowBidir::ImDxy;
DImage OpticalFlowBidir::ImDx2;
DImage OpticalFlowBidir::ImDy2;
DImage OpticalFlowBidir::ImDtDx;
DImage OpticalFlowBidir::ImDtDy;
DImage OpticalFlowBidir::A11;
DImage OpticalFlowBidir::A12;
DImage OpticalFlowBidir::A22;
DImage OpticalFlowBidir::b1;
DImage OpticalFlowBidir::b2;
DImage OpticalFlowBidir::foo1;
DImage OpticalFlowBidir::foo2;
DImage OpticalFlowBidir::warpur;
DImage OpticalFlowBidir::warpvr;
DImage OpticalFlowBidir::urdx;
DImage OpticalFlowBidir::urdy;
DImage OpticalFlowBidir::vrdx;
DImage OpticalFlowBidir::vrdy;
DImage OpticalFlowBidir::UU;
DImage OpticalFlowBidir::UV;
DImage OpticalFlowBidir::VV;
DImage OpticalFlowBidir::UT;
DImage OpticalFlowBidir::VT;


OpticalFlowBidir::OpticalFlowBidir(void)
{
}

OpticalFlowBidir::~OpticalFlowBidir(void)
{
}

void OpticalFlowBidir::FlowBidirIRLS(DImage &du, DImage &dv, const DImage &imdx, const DImage &imdy, const DImage &imdt, const DImage Mask, 
									                                    const DImage &u, const DImage &v, const DImage ur, const DImage vr, 
																		double alpha, double gamma, int nIRLSIterations, int nCGIterations)
{
	int imWidth,imHeight,nChannels,nPixels;
	imWidth=imdx.width();
	imHeight=imdx.height();
	nChannels=imdx.nchannels();
	nPixels=imWidth*imHeight;

	if(du.matchDimension(u)==false)
		du.allocate(u);
	if(dv.matchDimension(v)==false)
		dv.allocate(v);

	if(uu.matchDimension(du)==false)
		uu.allocate(du);
	if(vv.matchDimension(dv)==false)
		vv.allocate(dv);
	if(ux.matchDimension(du)==false)
		ux.allocate(du);
	if(uy.matchDimension(du)==false)
		uy.allocate(du);
	if(vx.matchDimension(dv)==false)
		vx.allocate(dv);
	if(vy.matchDimension(dv)==false)
		vy.allocate(dv);
	if(Phi_1st.matchDimension(du)==false)
		Phi_1st.allocate(du);
	if(Psi_1st.matchDimension(imdx)==false)
		Psi_1st.allocate(imdx);

	// warp the flow from ur,vr to u,v's coordinate
	warpFL(warpur,ur,ur,u,v);
	warpFL(warpvr,vr,vr,u,v);
	warpur.dx(urdx,true);
	warpur.dy(urdy,true);
	warpvr.dx(vrdx,true);
	warpvr.dy(vrdy,true);
	if(UU.matchDimension(u)==false)
		UU.allocate(u);
	if(VV.matchDimension(v)==false)
		VV.allocate(v);
	if(UV.matchDimension(u)==false)
		UV.allocate(v);
	if(UT.matchDimension(u)==false)
		UT.allocate(v);
	if(VT.matchDimension(u)==false)
		VT.allocate(v);

	// variables for conjugate gradient
	DImage r1,r2,p1,p2,q1,q2;
	double* rou;
	rou=new double[nCGIterations];

	double varepsilon_phi=pow(0.001,2);
	double varepsilon_psi=pow(0.001,2);

	du.reset();
	dv.reset();

	for(int hh=0;hh<nIRLSIterations;hh++)
	{
		// compute the derivatives of the current flow field
		if(hh==0)
		{
			uu.copyData(u);
			vv.copyData(v);
		}
		else
		{
			uu.Add(u,du);
			vv.Add(v,dv);
		}

		if(!IsThinPlate)
		{
			uu.dx(ux);
			uu.dy(uy);
			vv.dx(vx);
			vv.dy(vy);
		}
		else
		{
			uu.dxx(ux);
			uu.dyy(uy);
			vv.dxx(vx);
			vv.dyy(vy);
		}

		// compute the weight of phi
		Phi_1st.reset();
		double* phiData=Phi_1st.data();
		double temp;
		const double *uxData,*uyData,*vxData,*vyData;
		uxData=ux.data();
		uyData=uy.data();
		vxData=vx.data();
		vyData=vy.data();
		for(int i=0;i<nPixels;i++)
		{
			temp=uxData[i]*uxData[i]+uyData[i]*uyData[i]+vxData[i]*vxData[i]+vyData[i]*vyData[i];
			//phiData[i]=1/(2*sqrt(temp+varepsilon_phi));
			phiData[i]=alpha*m_Eta*pow(temp+varepsilon_phi,m_Eta-1);
			//if(!gradient.IsEmpty())
			//	phiData[i]*=0.1/(gradient.data()[i]+0.1);
		}
		// compute the nonlinear term of psi
		double* psiData=Psi_1st.data();
		const double *imdxData,*imdyData,*imdtData;
		const double *duData,*dvData;
		imdxData=imdx.data();
		imdyData=imdy.data();
		imdtData=imdt.data();
		duData=du.data();
		dvData=dv.data();
		if(nChannels==1)
		{
			for(int i=0;i<nPixels;i++)
			{
				temp=imdtData[i]+imdxData[i]*duData[i]+imdyData[i]*dvData[i];
				psiData[i]=1/(2*sqrt(temp*temp+varepsilon_psi));
				//if(!smoothgradient.IsEmpty())
				//	psiData[i]*=0.02/(smoothgradientdata[i]+0.02);
			}
		}
		else
		{
			for(int i=0;i<nPixels;i++)
				for(int k=0;k<nChannels;k++)
				{
					int offset=i*nChannels+k;
					temp=imdtData[offset]+imdxData[offset]*duData[i]+imdyData[offset]*dvData[i];
					psiData[offset]=1/(2*sqrt(temp*temp+varepsilon_psi));
					//if(!smoothgradient.IsEmpty())
					//	psiData[offset]*=0.02/(smoothgradientdata[i]+0.02);
				}
		}

		// compute the bidirectional constraint
		double *UUdata=UU.data(),*VVdata=VV.data(),*UVdata=UV.data(),*UTdata=UT.data(),*VTdata=VT.data();
		const double *urdxdata=urdx.data(),*urdydata=urdy.data(),*vrdxdata=vrdx.data(),*vrdydata=vrdy.data();
		const double *warpurdata=warpur.data(),*warpvrdata=warpvr.data();
		const double *udata=u.data(),*vdata=v.data();
		double temp1,temp2,gamma_1st=1;
		double h=0.5;
		if(gamma>0)
		{
			for(int i=0;i<nPixels;i++)
			{
				temp1=udata[i]+duData[i]+warpurdata[i]+urdxdata[i]*duData[i]+urdydata[i]*dvData[i];
				temp2=vdata[i]+dvData[i]+warpvrdata[i]+vrdxdata[i]*duData[i]+vrdydata[i]*dvData[i];
				//gamma_1st=1/(2*sqrt(temp1*temp1+temp2*temp2+.5));
				gamma_1st=h*pow(temp1*temp1+temp2*temp2+0.1,h-1);

				UUdata[i]=gamma_1st*((urdxdata[i]+1)*(urdxdata[i]+1)+vrdxdata[i]*vrdxdata[i]);
				VVdata[i]=gamma_1st*((vrdydata[i]+1)*(vrdydata[i]+1)+urdydata[i]*urdydata[i]);
				UVdata[i]=gamma_1st*((urdxdata[i]+1)*urdydata[i]+(vrdydata[i]+1)*vrdxdata[i]);
				UTdata[i]=gamma_1st*((urdxdata[i]+1)*(udata[i]+warpurdata[i])+vrdxdata[i]*(vdata[i]+warpvrdata[i]));
				VTdata[i]=gamma_1st*((vrdydata[i]+1)*(vdata[i]+warpvrdata[i])+urdydata[i]*(udata[i]+warpurdata[i]));
			}
			UU.Multiplywith(Mask);
			VV.Multiplywith(Mask);
			UV.Multiplywith(Mask);
			UT.Multiplywith(Mask);
			VT.Multiplywith(Mask);
		}
			
		// prepare the components of the large linear system
		ImDxy.Multiply(Psi_1st,imdx,imdy);
		ImDx2.Multiply(Psi_1st,imdx,imdx);
		ImDy2.Multiply(Psi_1st,imdy,imdy);
		ImDtDx.Multiply(Psi_1st,imdx,imdt);
		ImDtDy.Multiply(Psi_1st,imdy,imdt);

		if(nChannels>1)
		{
			ImDxy.collapse(imdxy);
			ImDx2.collapse(imdx2);
			ImDy2.collapse(imdy2);
			ImDtDx.collapse(imdtdx);
			ImDtDy.collapse(imdtdy);
		}
		else
		{
			imdxy.copyData(ImDxy);
			imdx2.copyData(ImDx2);
			imdy2.copyData(ImDy2);
			imdtdx.copyData(ImDtDx);
			imdtdy.copyData(ImDtDy);
		}
		// multiply with the mask
		imdxy.Multiplywith(Mask);
		imdx2.Multiplywith(Mask);
		imdy2.Multiplywith(Mask);
		imdtdx.Multiplywith(Mask);
		imdtdy.Multiplywith(Mask);

		// filtering
		//imdx2.smoothing(A11,3);
		//imdxy.smoothing(A12,3);
		//imdy2.smoothing(A22,3);
		imdx2.GaussianSmoothing(A11,1.2,3);
		imdxy.GaussianSmoothing(A12,1.2,3);
		imdy2.GaussianSmoothing(A22,1.2,3);

		// add epsilon to A11 and A22
		A11.Add(alpha*0.05);
		A22.Add(alpha*0.05);
		//A11.Add(Mask,alpha*0.1);
		//A22.Add(Mask,alpha*0.1);

		// form b
		//imdtdx.smoothing(b1,3);
		//imdtdy.smoothing(b2,3);
		imdtdx.GaussianSmoothing(b1,1.2,3);
		imdtdy.GaussianSmoothing(b2,1.2,3);

		// laplacian filtering of the current flow field
		if(!IsThinPlate)
		{
			Laplacian(foo1,u,Phi_1st);
			Laplacian(foo2,v,Phi_1st);
		}
		else
		{
			Laplacian2(foo1,u,Phi_1st);
			Laplacian2(foo2,v,Phi_1st);
		}
		double *b1Data,*b2Data;
		const double *foo1Data,*foo2Data;
		b1Data=b1.data();
		b2Data=b2.data();
		foo1Data=foo1.data();
		foo2Data=foo2.data();

		if(gamma>0)
		{
			for(int i=0;i<nPixels;i++)
			{
				b1Data[i]=-b1Data[i]-foo1Data[i]-gamma*UTdata[i];
				b2Data[i]=-b2Data[i]-foo2Data[i]-gamma*VTdata[i];
			}
			A11.Add(UU,gamma);
			A22.Add(VV,gamma);
			A12.Add(UV,gamma);
		}
		else
			for(int i=0;i<nPixels;i++)
			{
				b1Data[i]=-b1Data[i]-foo1Data[i];
				b2Data[i]=-b2Data[i]-foo2Data[i];
			}

		//-----------------------------------------------------------------------
		// conjugate gradient algorithm
		//-----------------------------------------------------------------------
		r1.copyData(b1);
		r2.copyData(b2);
		du.reset();
		dv.reset();

		for(int k=0;k<nCGIterations;k++)
		{
			rou[k]=r1.norm2()+r2.norm2();
			//cout<<rou[k]<<endl;
			if(rou[k]<1E-10)
				break;
			if(k==0)
			{
				p1.copyData(r1);
				p2.copyData(r2);
			}
			else
			{
				double ratio=rou[k]/rou[k-1];
				p1.Add(r1,p1,ratio);
				p2.Add(r2,p2,ratio);
			}

			// go through the large linear system
			foo1.Multiply(A11,p1);
			foo2.Multiply(A12,p2);
			q1.Add(foo1,foo2);
			if(!IsThinPlate)
				Laplacian(foo1,p1,Phi_1st);
			else
				Laplacian2(foo1,p1,Phi_1st);
			q1.Add(foo1);

			foo1.Multiply(A12,p1);
			foo2.Multiply(A22,p2);
			q2.Add(foo1,foo2);
			if(!IsThinPlate)
				Laplacian(foo2,p2,Phi_1st);
			else
				Laplacian2(foo2,p2,Phi_1st);

			q2.Add(foo2);

			double beta;
			beta=rou[k]/(p1.innerproduct(q1)+p2.innerproduct(q2));
			
			du.Add(p1,beta);
			dv.Add(p2,beta);

			r1.Add(q1,-beta);
			r2.Add(q2,-beta);
		}
		//-----------------------------------------------------------------------
		// end of conjugate gradient algorithm
		//-----------------------------------------------------------------------		
	}

	delete rou;
}

/*
void OpticalFlowBidir::Coarse2FineFlowBidir(DImage &vx1, DImage &vy1, DImage &warpI2, DImage &vx2, DImage &vy2, DImage &warpI1, 
											const DImage &Im1, const DImage &Im2, double alpha, double gamma, double ratio, int minWidth, int nIterations, int nIRLSIterations, int nCGIterations)
{
	// first build the pyramid of the two images
	GaussianPyramid GPyramid1;
	GaussianPyramid GPyramid2;
	if(IsDisplay)
		cout<<"Constructing pyramid...";
	GPyramid1.ConstructPyramid(Im1,ratio,minWidth);
	GPyramid2.ConstructPyramid(Im2,ratio,minWidth);
	if(IsDisplay)
		cout<<"done!"<<endl;
	
	// now iterate from the top level to the bottom
	DImage Image1,Image2,WarpImage2,WarpImage1;
	DImage du1,dv1,du2,dv2;

	for(int k=GPyramid1.nlevels()-1;k>=0;k--)
	{
		if(IsDisplay)
			cout<<"Pyramid level "<<k;
		int width=GPyramid1.Image(k).width();
		int height=GPyramid1.Image(k).height();
		im2feature(Image1,GPyramid1.Image(k));
		im2feature(Image2,GPyramid2.Image(k));


		if(k==GPyramid1.nlevels()-1) // if at the top level
		{
			vx1.allocate(width,height);
			vy1.allocate(width,height);
			WarpImage2.copyData(Image2);

			vx2.allocate(width,height);
			vy2.allocate(width,height);
			WarpImage1.copyData(Image1);
		}
		else
		{
			// propagate the flow field from the coarser to finer level
			vx1.imresize(width,height);
			vx1.Multiplywith(1/ratio);
			vy1.imresize(width,height);
			vy1.Multiplywith(1/ratio);
			warpFL(WarpImage2,Image1,Image2,vx1,vy1);

			vx2.imresize(width,height);
			vx2.Multiplywith(1/ratio);
			vy2.imresize(width,height);
			vy2.Multiplywith(1/ratio);
			warpFL(WarpImage1,Image2,Image1,vx2,vy2);

		}
		for(int i=0;i<nIterations;i++)
		{
			// forward flow
			getDxs(imdx,imdy,imdt,Image1,WarpImage2);
			genInImageMask(immask,vx1,vy1);
			FlowBidirIRLS(du1,dv1,imdx,imdy,imdt,immask,vx1,vy1,vx2,vy2,alpha,gamma,nIRLSIterations,nCGIterations);

			// backward flow
			getDxs(imdx,imdy,imdt,Image2,WarpImage1);
			genInImageMask(immask,vx2,vy2);
			FlowBidirIRLS(du2,dv2,imdx,imdy,imdt,immask,vx2,vy2,vx1,vy1,alpha,gamma,nIRLSIterations,nCGIterations);
			
			vx1.Add(du1,1);
			vy1.Add(dv1,1);
			vx2.Add(du2,1);
			vy2.Add(dv2,1);

			warpFL(WarpImage2,Image1,Image2,vx1,vy1);
			warpFL(WarpImage1,Image2,Image1,vx2,vy2);
		}
		
		if(IsDisplay)
			cout<<endl;
	}
	warpFL(warpI2,Im1,Im2,vx1,vy1);
	warpFL(warpI1,Im2,Im1,vx2,vy2);
}//*/

void OpticalFlowBidir::Coarse2FineFlowBidir(DImage &vx1, DImage &vy1, DImage &warpI2, DImage &vx2, DImage &vy2, DImage &warpI1, 
											const DImage &Im1, const DImage &Im2, const DImage& Mask1,const DImage& Mask2,
											double alpha, double gamma, double ratio, int minWidth, int nIterations, int nIRLSIterations, int nCGIterations)
{
	// first build the pyramid of the two images
	GaussianPyramid GPyramid1;
	GaussianPyramid GPyramid2;
	GaussianPyramid MaskPyramid1;
	GaussianPyramid MaskPyramid2;

	if(IsDisplay)
		cout<<"Constructing pyramid...";
	GPyramid1.ConstructPyramid(Im1,ratio,minWidth);
	GPyramid2.ConstructPyramid(Im2,ratio,minWidth);
	if(!Mask1.IsEmpty())
	{
		MaskPyramid1.ConstructPyramid(Mask1,ratio,minWidth);
		MaskPyramid2.ConstructPyramid(Mask2,ratio,minWidth);
	}
	

	if(IsDisplay)
		cout<<"done!"<<endl;
	
	// now iterate from the top level to the bottom
	DImage Image1,Image2,WarpImage2,WarpImage1;
	DImage du1,dv1,du2,dv2;

	for(int k=GPyramid1.nlevels()-1;k>=0;k--)
	{
		if(IsDisplay)
			cout<<"Pyramid level "<<k;
		int width=GPyramid1.Image(k).width();
		int height=GPyramid1.Image(k).height();
		im2feature(Image1,GPyramid1.Image(k));
		im2feature(Image2,GPyramid2.Image(k));

		// compute the derivative of the images
		//DImage gradient1,gradient2;
		//GPyramid1.Image(k).gradientmag(gradient1);
		//GPyramid2.Image(k).gradientmag(gradient2);
		//if(!Mask1.IsEmpty())
		//{
		//	gradient1.Multiplywith(MaskPyramid1.Image(k));
		//	gradient2.Multiplywith(MaskPyramid2.Image(k));
		//}
		//DImage smoothgradient1,smoothgradient2;
		//gradient1.GaussianSmoothing(smoothgradient1,2,4);
		//gradient2.GaussianSmoothing(smoothgradient2,2,4);

	//	cout<<"max: "<<smoothgradient1.max()<<endl;

		if(k==GPyramid1.nlevels()-1) // if at the top level
		{
			vx1.allocate(width,height);
			vy1.allocate(width,height);
			WarpImage2.copyData(Image2);

			vx2.allocate(width,height);
			vy2.allocate(width,height);
			WarpImage1.copyData(Image1);
		}
		else
		{
			// propagate the flow field from the coarser to finer level
			vx1.imresize(width,height);
			vx1.Multiplywith(1/ratio);
			vy1.imresize(width,height);
			vy1.Multiplywith(1/ratio);
			warpFL(WarpImage2,Image1,Image2,vx1,vy1);

			vx2.imresize(width,height);
			vx2.Multiplywith(1/ratio);
			vy2.imresize(width,height);
			vy2.Multiplywith(1/ratio);
			warpFL(WarpImage1,Image2,Image1,vx2,vy2);

		}
		//cout<<"Programs goes here!"<<endl;
		for(int i=0;i<nIterations;i++)
		{
			// forward flow
			getDxs(imdx,imdy,imdt,Image1,WarpImage2);
			if(Mask2.IsEmpty())
				genInImageMask(immask,vx1,vy1);
			else
				genInImageMask(immask,vx1,vy1,MaskPyramid2.Image(k));
			if(!Mask1.IsEmpty())
				immask.Multiplywith(MaskPyramid1.Image(k));

			//cout<<"Program goes here!"<<endl;

			//immask.imwrite("temp.bmp");
			FlowBidirIRLS(du1,dv1,imdx,imdy,imdt,immask,vx1,vy1,vx2,vy2,alpha,gamma,nIRLSIterations,nCGIterations);

			// backward flow
			getDxs(imdx,imdy,imdt,Image2,WarpImage1);
			if(Mask1.IsEmpty())
				genInImageMask(immask,vx2,vy2);
			else
				genInImageMask(immask,vx2,vy2,MaskPyramid1.Image(k));
			if(!Mask2.IsEmpty())
				immask.Multiplywith(MaskPyramid2.Image(k));

			//cout<<"Program goes here!"<<endl;

			//immask.imwrite("temp.bmp");
			

			FlowBidirIRLS(du2,dv2,imdx,imdy,imdt,immask,vx2,vy2,vx1,vy1,alpha*pow(ratio,k),gamma,nIRLSIterations,nCGIterations);
			
			vx1.Add(du1,1);
			vy1.Add(dv1,1);
			vx2.Add(du2,1);
			vy2.Add(dv2,1);

			warpFL(WarpImage2,Image1,Image2,vx1,vy1);
			warpFL(WarpImage1,Image2,Image1,vx2,vy2);
		}
		
		if(IsDisplay)
			cout<<endl;
	}
	warpFL(warpI2,Im1,Im2,vx1,vy1);
	warpFL(warpI1,Im2,Im1,vx2,vy2);
}