
#include <QtGui>
#include <QtGui/QApplication>
#include <QApplication>
#include <QHeaderView>
#include <QItemSelectionModel>
#include <QStandardItemModel>
#include <QSizePolicy>
#include <QSpacerItem>
#include <qcolordialog.h>
#include <iostream>
#include <QComboBox>
#include <QDomDocument>
#include "PaintOptions.h"
#include "MyPreference.h"


using namespace std;


PaintOptions::PaintOptions(QWidget *parent, Qt::WFlags flags)
	: QWidget(parent, flags)
{


	/*setup tabs and main layout*/
	createTabs();

	setMaximumWidth(400);

    setWindowIcon(QIcon("icons/preferences-system.png"));
	setWindowTitle(tr("Paint Options"));
}

PaintOptions::~PaintOptions()
{

}

void PaintOptions::createTabs()
{
	_strokeProperties = createStrokePropertiesWidget();
	_fillingParameters= createFillingParametersWidget();
	_motionParameters= createMotionParametersWidget();
	_backgroundParameters= createBackgroundParametersWidget();

	_tabGroup=new QTabWidget;
	_tabGroup->addTab(_strokeProperties,"Stroke Options");
	_tabGroup->addTab(_fillingParameters,"Filling Options");
	_tabGroup->addTab(_motionParameters,"Motion Options" );
    _tabGroup->addTab(_backgroundParameters,"Paper Options");

	QVBoxLayout* vGlobalLayout=new QVBoxLayout;
	vGlobalLayout->addWidget(_tabGroup);
	setLayout(vGlobalLayout);

}


QWidget* PaintOptions::createBackgroundParametersWidget()
{

	QWidget* backgroundParameters=new QWidget;
	QVBoxLayout* vLayout=new QVBoxLayout;

	vLayout->setAlignment(Qt::AlignTop);


	/* Texture layout */
	QGroupBox *groupBox=new QGroupBox(tr("Canvas:"));
	QVBoxLayout* vbox=new QVBoxLayout;

	//scrollArea=new QScrollArea;
	
	_PaperImageViewer=new ImageViewer;

	_PaperImageViewer->setParentSize(QSize(100,100));
	QImage* im = new QImage(QString("Paper/defaultpaper.pgm")); 

	if ( im == NULL )
		QMessageBox::warning(this,"Warning","Can't find the default paper in HOMEDIR/Paper/ \n");
	else
		_PaperImageViewer->loadImage(*im);

	_PaperImageViewer->setGeometry(0,0,100,100);
	//_PaperImageViewer->setFixedWidth(100);
	//_PaperImageViewer->setFixedHeight(100);
	vbox->addWidget(_PaperImageViewer);
	
	//_StrokeImageViewer->setMaxScale(1);
	//_StrokeImageViewer->setMinScale(0.2);
	//scrollArea->setWidget(_StrokeImageViewer);

	QHBoxLayout *hLayout=new QHBoxLayout;

	QPushButton *_loadPaperTextureButton=new QPushButton(tr("Load Paper Texture"));
	_loadPaperTextureButton->setFixedWidth(100);
	connect(_loadPaperTextureButton,SIGNAL(clicked()),this,SLOT(selectPaperTexture()));

	//vbox->addWidget(_StrokeImageViewer);
	hLayout->addWidget(_loadPaperTextureButton);

	_choosePaperColorButton=new QPushButton(tr("Paper Color"));
	_choosePaperColorButton->setFixedWidth(100);
	connect(_choosePaperColorButton,SIGNAL(clicked()),this,SLOT(selectPaperColor()));
	hLayout->addWidget(_choosePaperColorButton);

	//_imgColorForPaper=new QCheckBox("Use Image Color");
	//_imgColorForPaper->setChecked(MyPreference::useImageColorForPaper);
	//connect(_imgColorForPaper,SIGNAL(toggled(bool)),this,SIGNAL(useImageColorForPaper(bool)));
	//vbox->addWidget(_imgColorForPaper);

	vbox->addLayout(hLayout);

	groupBox->setLayout(vbox);
	vLayout->addWidget(groupBox);







	/*
	hLayout=new QHBoxLayout;
	QCheckBox* multiPassRender=new QCheckBox("Multi-pass Render");
	multiPassRender->setChecked(true);
	connect(multiPassRender,SIGNAL(toggled(bool)),this,SIGNAL(multiPassRender(bool)));

	hLayout->addWidget(multiPassRender);
	vLayout->addLayout(hLayout);
	hLayout=new QHBoxLayout;
	QCheckBox* oldBlend=new QCheckBox("Old Blend");
	oldBlend->setChecked(false);
	connect(oldBlend,SIGNAL(toggled(bool)),this,SIGNAL(oldBlend(bool)));

	hLayout->addWidget(oldBlend);
	vLayout->addLayout(hLayout);
	*/



             /*

	QDoubleSpinBox* boundaryBlurSpinBox=new QDoubleSpinBox;
	boundaryBlurSpinBox->setRange(0.0,50.0);
	boundaryBlurSpinBox->setDecimals(1);
	boundaryBlurSpinBox->setSingleStep(1);
	boundaryBlurSpinBox->setMaximumWidth(50);
	boundaryBlurSpinBox->setValue(MyPreference::boundaryBlur);
	connect(boundaryBlurSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(boundaryBlurChanged(double)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("Boundary Blur: "));
	hLayout->addWidget(boundaryBlurSpinBox);
	vLayout->addLayout(hLayout);

	QSpinBox* medianBlurSpinBox=new QSpinBox;
	medianBlurSpinBox->setRange(0,20);
	medianBlurSpinBox->setSingleStep(1);
	medianBlurSpinBox->setMaximumWidth(50);
	medianBlurSpinBox->setValue(0);
	connect(medianBlurSpinBox,SIGNAL(valueChanged(int)),this,SIGNAL(medianBlurChanged(int)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("Median Filter: "));
	hLayout->addWidget(medianBlurSpinBox);
	vLayout->addLayout(hLayout);



	radiusFactorSpinBox=new QDoubleSpinBox;
	radiusFactorSpinBox->setRange(0.0,1.0);
	radiusFactorSpinBox->setDecimals(2);
	radiusFactorSpinBox->setSingleStep(0.1);
	radiusFactorSpinBox->setMaximumWidth(50);
	radiusFactorSpinBox->setAlignment(Qt::AlignLeft);
	radiusFactorSpinBox->setValue(MyPreference::radiusFactor);
	connect(radiusFactorSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(radiusFactorChanged(double)));

	hLayout=new QHBoxLayout;
	QLabel *label = new QLabel("How much to blur source image ");
	label->setAlignment(Qt::AlignLeft);
	//label->setFixedWidth(300);
	hLayout->addWidget(label);
	hLayout->addWidget(radiusFactorSpinBox);
	vLayout->addLayout(hLayout);



	densityWidthAddSpinBox=new QDoubleSpinBox;
	densityWidthAddSpinBox->setRange(-100,100.0);
	densityWidthAddSpinBox->setDecimals(2);
	densityWidthAddSpinBox->setSingleStep(1);
	densityWidthAddSpinBox->setMaximumWidth(50);
	densityWidthAddSpinBox->setAlignment(Qt::AlignLeft);
	densityWidthAddSpinBox->setValue(0);
	connect(densityWidthAddSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(densityWidthAddChanged(double)));


	densityBrightnessFactorSpinBox=new QDoubleSpinBox;
	densityBrightnessFactorSpinBox->setRange(0,100.0);
	densityBrightnessFactorSpinBox->setDecimals(2);
	densityBrightnessFactorSpinBox->setSingleStep(1);
	densityBrightnessFactorSpinBox->setMaximumWidth(50);
	densityBrightnessFactorSpinBox->setAlignment(Qt::AlignLeft);
	densityBrightnessFactorSpinBox->setValue(0);
	connect(densityBrightnessFactorSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(densityBrightnessFactorChanged(double)));



	hLayout=new QHBoxLayout;
	label = new QLabel("Density Width Add/Bright Factor:");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(densityWidthAddSpinBox);
	hLayout->addWidget(densityBrightnessFactorSpinBox);
	vLayout->addLayout(hLayout);



	QSpinBox* startHeavySampleSpinBox=new QSpinBox;
	startHeavySampleSpinBox->setRange(0,1000);
	startHeavySampleSpinBox->setSingleStep(1);
	startHeavySampleSpinBox->setMaximumWidth(50);
	startHeavySampleSpinBox->setValue(0);
	connect(startHeavySampleSpinBox,SIGNAL(valueChanged(int)),this,SIGNAL(startHeavySampleChanged(int)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("Start Heavy Sample Frame: "));
	hLayout->addWidget(startHeavySampleSpinBox);
	vLayout->addLayout(hLayout);


	QSpinBox* endHeavySampleSpinBox=new QSpinBox;

	endHeavySampleSpinBox->setRange(0,1000);
	endHeavySampleSpinBox->setSingleStep(1);
	endHeavySampleSpinBox->setMaximumWidth(50);
	endHeavySampleSpinBox->setValue(0);
	connect(endHeavySampleSpinBox,SIGNAL(valueChanged(int)),this,SIGNAL(endHeavySampleChanged(int)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("End Heavy Sample Frame: "));
	hLayout->addWidget(endHeavySampleSpinBox);
	vLayout->addLayout(hLayout);






	_passOverGuideStrokes=new QCheckBox("System strokes can pass over guide strokes?");
	_passOverGuideStrokes->setChecked(true);
	connect(_passOverGuideStrokes,SIGNAL(toggled(bool)),this,SIGNAL(passOverGuideStrokes(bool)));
	vLayout->addWidget(_passOverGuideStrokes);

	QCheckBox* darkenOccludedRegions=new QCheckBox("Darken occluded regions?");
	darkenOccludedRegions->setChecked(false);
	connect(darkenOccludedRegions,SIGNAL(toggled(bool)),this,SIGNAL(darkenOccludedRegionsChanged(bool)));


	QCheckBox* useStrokesFromAllLayers=new QCheckBox("Use user strokes from all layers?");
	useStrokesFromAllLayers->setChecked(false);
	connect(useStrokesFromAllLayers,SIGNAL(toggled(bool)),this,SIGNAL(useStrokesFromAllLayersChanged(bool)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(useStrokesFromAllLayers);
	vLayout->addLayout(hLayout);

        */


	backgroundParameters->setLayout(vLayout);


	return backgroundParameters;

}


QWidget* PaintOptions::createMotionParametersWidget()
{

	QLabel *label;
	QWidget* motionParameters=new QWidget;

		

	QGroupBox *groupBox=new QGroupBox(tr("How will strokes flow in time?"));

	QGridLayout* gridLayout=new QGridLayout;

	//gridLayout->addWidget(new QLabel("Orientation Type"),1,1);
	//gridLayout->addWidget(new QLabel("Factor"),1,2);

	gridLayout->addWidget(new QLabel("Weights "),1,2);
	gridLayout->addWidget(new QLabel("With a fixed direction "),2,1);

	
	_fixedMovementFactor=new QSpinBox;
	_fixedMovementFactor->setRange(0,100);
	_fixedMovementFactor->setFixedWidth(40);
	_fixedMovementFactor->setSingleStep(5);
	_fixedMovementFactor->setValue(MyPreference::fixedOrientationFactor);
	connect(_fixedMovementFactor,SIGNAL(valueChanged(int)),this,SIGNAL(fixedMovementFactorChanged(int)));
	gridLayout->addWidget(_fixedMovementFactor,2,2);


	_movementDial = new QDial;
	//_movementDial->setFixedWidth(35);
	_movementDial->setFixedHeight(35);
	_movementDial->setWrapping(true);
	_movementDial->setMinimum(0);
	_movementDial->setMaximum(359);
	_movementDial->setValue(0);
	connect(_movementDial,SIGNAL(valueChanged(int)),this,SIGNAL(fixedMovementDirectionChanged(int)));
	gridLayout->addWidget(_movementDial,2,3);



	_fixedMovementRateSpin=new QSpinBox;
	QSlider* fixedMovementRateSlider=new QSlider(Qt::Horizontal);
	_fixedMovementRateSpin->setRange(0,100);
	fixedMovementRateSlider->setRange(0,100);
	_fixedMovementRateSpin->setValue(MyPreference::fixedMovementRate);
	fixedMovementRateSlider->setValue(MyPreference::fixedMovementRate);
	fixedMovementRateSlider->setFixedWidth(150);
	connect(_fixedMovementRateSpin,SIGNAL(valueChanged(int)),fixedMovementRateSlider,SLOT(setValue(int)));
	connect(fixedMovementRateSlider,SIGNAL(valueChanged(int)),_fixedMovementRateSpin,SLOT(setValue(int)));
	connect(fixedMovementRateSlider,SIGNAL(valueChanged(int)),this,SIGNAL(fixedMovementRateChanged(int)));


	label = new QLabel("Pixels Per Frame:");
	label->setAlignment(Qt::AlignRight);
	label->setFixedWidth(200);
	gridLayout->addWidget(label,3,1);
	gridLayout->addWidget(_fixedMovementRateSpin,3,2);
	gridLayout->addWidget(fixedMovementRateSlider,3,3);

	/*
	QCheckBox* useBoundaryFlowWithGradient=new QCheckBox("Use Boundary Flow with Gradient Vectors");
	useBoundaryFlowWithGradient->setChecked(false);
	connect(useBoundaryFlowWithGradient,SIGNAL(toggled(bool)),this,SIGNAL(useBoundaryFlowWithGradientChanged(bool)));
	gridLayout->addWidget(useBoundaryFlowWithGradient,4,1);
	*/

	gridLayout->addWidget(new QLabel("Following region boundaries"),4,1);

	_boundaryMovementFactor=new QSpinBox;
	_boundaryMovementFactor->setRange(0,100);
	_boundaryMovementFactor->setFixedWidth(40);
	_boundaryMovementFactor->setSingleStep(5);
	_boundaryMovementFactor->setValue(MyPreference::boundaryFactor);
	connect(_boundaryMovementFactor,SIGNAL(valueChanged(int)),this,SIGNAL(followBoundaryMovementFactorChanged(int)));
	gridLayout->addWidget(_boundaryMovementFactor,4,2);


	gridLayout->addWidget(new QLabel("Following the image"),5,1);

	_opticalFlowFactor=new QSpinBox;
	_opticalFlowFactor->setRange(0,100);
	_opticalFlowFactor->setFixedWidth(40);
	_opticalFlowFactor->setSingleStep(5);
	_opticalFlowFactor->setValue(MyPreference::imageGradientFactor);
	connect(_opticalFlowFactor,SIGNAL(valueChanged(int)),this,SIGNAL(followOpticalFlowFactorChanged(int)));
	gridLayout->addWidget(_opticalFlowFactor,5,2);


	gridLayout->addWidget(new QLabel("Following guide strokes"),6,1);

	_userStrokesMovementFactor=new QSpinBox;
	_userStrokesMovementFactor->setRange(0,100);
	_userStrokesMovementFactor->setFixedWidth(40);
	_userStrokesMovementFactor->setSingleStep(5);
	_userStrokesMovementFactor->setValue(MyPreference::userStrokesFactor);
	connect(_userStrokesMovementFactor,SIGNAL(valueChanged(int)),this,SIGNAL(followUserStrokeMovementFactorChanged(int)));
	gridLayout->addWidget(_userStrokesMovementFactor,6,2);

	/*
	gridLayout->addWidget(new QLabel("How much noise?"),7,1);

	QSlider* movementVarianceSlider=new QSlider(Qt::Horizontal);
	movementVarianceSlider->setRange(0,100);
	movementVarianceSlider->setFixedWidth(100);
	connect(movementVarianceSlider,SIGNAL(valueChanged(int)),this,SLOT(setMovementVariance(int)));
	movementVarianceSlider->setValue(0);

	gridLayout->addWidget(movementVarianceSlider,7,3);
	*/

	groupBox->setLayout(gridLayout);

	groupBox->resize(100,100);

	QVBoxLayout* topLayout=new QVBoxLayout;
	topLayout->setAlignment(Qt::AlignTop);
	topLayout->addWidget(groupBox);
	

	QHBoxLayout *hLayout;
	/*

	QCheckBox* scaleStrokes=new QCheckBox("Scale Strokes");
	scaleStrokes->setChecked(true);
	connect(scaleStrokes,SIGNAL(toggled(bool)),this,SIGNAL(scaleStrokesChanged(bool)));
	topLayout->addWidget(scaleStrokes);


	QSpinBox* strokeMinScaleSpin=new QSpinBox;
	QSlider* strokeMinScaleSlider=new QSlider(Qt::Horizontal);
	strokeMinScaleSpin->setRange(1,20);
	strokeMinScaleSlider->setRange(1,20);
	strokeMinScaleSlider->setValue(MyPreference::minStrokeScale);
	strokeMinScaleSpin->setValue(MyPreference::minStrokeScale);
	strokeMinScaleSlider->setFixedWidth(150);
	connect(strokeMinScaleSpin,SIGNAL(valueChanged(int)),strokeMinScaleSlider,SLOT(setValue(int)));
	connect(strokeMinScaleSlider,SIGNAL(valueChanged(int)),strokeMinScaleSpin,SLOT(setValue(int)));
	connect(strokeMinScaleSlider,SIGNAL(valueChanged(int)),this,SIGNAL(minStrokeScaleChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Min Scale For Strokes (1/current size):");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(strokeMinScaleSpin);
	hLayout->addWidget(strokeMinScaleSlider);
	topLayout->addLayout(hLayout);

	
	QSpinBox* strokeMaxScaleSpin=new QSpinBox;
	QSlider* strokeMaxScaleSlider=new QSlider(Qt::Horizontal);
	strokeMaxScaleSpin->setRange(1,20);
	strokeMaxScaleSlider->setRange(1,20);
	strokeMaxScaleSpin->setValue(MyPreference::maxStrokeScale);
	strokeMaxScaleSlider->setValue(MyPreference::maxStrokeScale);
	strokeMaxScaleSlider->setFixedWidth(150);
	connect(strokeMaxScaleSpin,SIGNAL(valueChanged(int)),strokeMaxScaleSlider,SLOT(setValue(int)));
	connect(strokeMaxScaleSlider,SIGNAL(valueChanged(int)),strokeMaxScaleSpin,SLOT(setValue(int)));
	connect(strokeMaxScaleSlider,SIGNAL(valueChanged(int)),this,SIGNAL(maxStrokeScaleChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Max Scale For Strokes (X current size):");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(strokeMaxScaleSpin);
	hLayout->addWidget(strokeMaxScaleSlider);
	topLayout->addLayout(hLayout);

	*/


	QCheckBox *showAdvancedMotionOptionsCheckbox=new QCheckBox("Show Advanced Options");
    showAdvancedMotionOptionsCheckbox->setChecked(false);
    connect(showAdvancedMotionOptionsCheckbox,SIGNAL(toggled(bool)),this,SLOT(showAdvancedMotionOptions(bool)));

	hLayout=new QHBoxLayout;
    hLayout->addWidget(showAdvancedMotionOptionsCheckbox);
    topLayout->addLayout(hLayout);

    _advancedMotionOptionsWidget = new QWidget();
	QVBoxLayout* advancedOptionsLayout=new QVBoxLayout;



	_colorUpdateSpinBox=new QDoubleSpinBox;
	_colorUpdateSpinBox->setRange(0.0,1.0);
	_colorUpdateSpinBox->setDecimals(3);
	_colorUpdateSpinBox->setSingleStep(0.05);
	_colorUpdateSpinBox->setMaximumWidth(50);
	_colorUpdateSpinBox->setValue(MyPreference::colorUpdate);
	connect(_colorUpdateSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(colorUpdateChanged(double)));

	hLayout=new QHBoxLayout;
	label = new QLabel("How much to update color at each frame");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_colorUpdateSpinBox);
	advancedOptionsLayout->addLayout(hLayout);


	_maxFramesSpin=new QSpinBox;
	QSlider* strokeMaxFramesSlider=new QSlider(Qt::Horizontal);
	_maxFramesSpin->setRange(1,1000);
	strokeMaxFramesSlider->setRange(1,1000);
	_maxFramesSpin->setValue(MyPreference::maxFrames);
	strokeMaxFramesSlider->setValue(MyPreference::maxFrames);
	strokeMaxFramesSlider->setFixedWidth(150);
	connect(_maxFramesSpin,SIGNAL(valueChanged(int)),strokeMaxFramesSlider,SLOT(setValue(int)));
	connect(strokeMaxFramesSlider,SIGNAL(valueChanged(int)),_maxFramesSpin,SLOT(setValue(int)));
	connect(strokeMaxFramesSlider,SIGNAL(valueChanged(int)),this,SIGNAL(maxFramesChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Maximum # of frames");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_maxFramesSpin);
	hLayout->addWidget(strokeMaxFramesSlider);
	advancedOptionsLayout->addLayout(hLayout);


	_minFramesSpin=new QSpinBox;
	QSlider* strokeMinFramesSlider=new QSlider(Qt::Horizontal);
	_minFramesSpin->setRange(1,100);
	strokeMinFramesSlider->setRange(1,100);
	_minFramesSpin->setValue(MyPreference::minFrames);
	strokeMinFramesSlider->setValue(MyPreference::minFrames);
	strokeMinFramesSlider->setFixedWidth(150);
	connect(_minFramesSpin,SIGNAL(valueChanged(int)),strokeMinFramesSlider,SLOT(setValue(int)));
	connect(strokeMinFramesSlider,SIGNAL(valueChanged(int)),_minFramesSpin,SLOT(setValue(int)));
	connect(strokeMinFramesSlider,SIGNAL(valueChanged(int)),this,SIGNAL(minFramesChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Minimum # of frames");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_minFramesSpin);
	hLayout->addWidget(strokeMinFramesSlider);
	advancedOptionsLayout->addLayout(hLayout);


	_fadeRateSpin=new QSpinBox;
	QSlider* fadeRateSlider=new QSlider(Qt::Horizontal);
	_fadeRateSpin->setRange(0,100);
	fadeRateSlider->setRange(0,100);
	_fadeRateSpin->setValue(MyPreference::fadeRate);
	fadeRateSlider->setValue(MyPreference::fadeRate);
	fadeRateSlider->setFixedWidth(150);
	connect(_fadeRateSpin,SIGNAL(valueChanged(int)),fadeRateSlider,SLOT(setValue(int)));
	connect(fadeRateSlider,SIGNAL(valueChanged(int)),_fadeRateSpin,SLOT(setValue(int)));
	connect(fadeRateSlider,SIGNAL(valueChanged(int)),this,SIGNAL(fadeRateChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("# of frames to fade in and out");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_fadeRateSpin);
	hLayout->addWidget(fadeRateSlider);
	advancedOptionsLayout->addLayout(hLayout);



	

	_quickOptions = new QWidget;
	QVBoxLayout *quickOptionsLayout=new QVBoxLayout;


	_maxStrokeAngleSpin=new QSpinBox;
	QSlider* maxAngleSlider=new QSlider(Qt::Horizontal);
	_maxStrokeAngleSpin->setRange(0,180);
	maxAngleSlider->setRange(0,180);
	_maxStrokeAngleSpin->setValue(MyPreference::maxStrokeAngle);
	maxAngleSlider->setValue(MyPreference::maxStrokeAngle);
	maxAngleSlider->setFixedWidth(150);
	connect(_maxStrokeAngleSpin,SIGNAL(valueChanged(int)),maxAngleSlider,SLOT(setValue(int)));
	connect(maxAngleSlider,SIGNAL(valueChanged(int)),_maxStrokeAngleSpin,SLOT(setValue(int)));
	connect(maxAngleSlider,SIGNAL(valueChanged(int)),this,SIGNAL(maxStrokeAngleChanged(int)));


	hLayout=new QHBoxLayout;
	label = new QLabel("Max angle between stroke points ");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_maxStrokeAngleSpin);
	hLayout->addWidget(maxAngleSlider);
	quickOptionsLayout->addLayout(hLayout);


	_maxFlowAngleSpin=new QSpinBox;
	QSlider* maxFlowAngleSlider=new QSlider(Qt::Horizontal);
	_maxFlowAngleSpin->setRange(0,180);
	maxFlowAngleSlider->setRange(0,180);
	_maxFlowAngleSpin->setValue(MyPreference::maxAngle);
	maxFlowAngleSlider->setValue(MyPreference::maxAngle);
	maxFlowAngleSlider->setFixedWidth(150);
	connect(_maxFlowAngleSpin,SIGNAL(valueChanged(int)),maxFlowAngleSlider,SLOT(setValue(int)));
	connect(maxFlowAngleSlider,SIGNAL(valueChanged(int)),_maxFlowAngleSpin,SLOT(setValue(int)));
	connect(maxFlowAngleSlider,SIGNAL(valueChanged(int)),this,SIGNAL(maxFlowAngleChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Max angle to update each frame");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_maxFlowAngleSpin);
	hLayout->addWidget(maxFlowAngleSlider);
	quickOptionsLayout->addLayout(hLayout);



	_followStrokesFactorSpin=new QDoubleSpinBox;
	_followStrokesFactorSpin->setRange(0,100);
	_followStrokesFactorSpin->setValue(MyPreference::followStrokesFactor);
	_followStrokesFactorSpin->setMaximumWidth(50);
	connect(_followStrokesFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(followStrokesFactorChanged(double)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Follow the last frame's stroke");
	label->setAlignment(Qt::AlignLeft);
	//label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_followStrokesFactorSpin);
	//hLayout->addWidget(followStrokesFactorSlider);
	

	/*
	QDoubleSpinBox* followFlowFactorSpin=new QDoubleSpinBox;
	followFlowFactorSpin->setRange(0,100);
	followFlowFactorSpin->setMaximumWidth(50);
	followFlowFactorSpin->setValue(MyPreference::followFlowFactor);
	connect(followFlowFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(followFlowFactorChanged(double)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Follow the flow");
	label->setAlignment(Qt::AlignLeft);
	label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(followFlowFactorSpin);
	quickOptionsLayout->addLayout(hLayout);
	
	*/
	_followGradientFactorSpin=new QDoubleSpinBox;
	_followGradientFactorSpin->setRange(0,100);
	_followGradientFactorSpin->setValue(MyPreference::followGradientFactor);
	_followGradientFactorSpin->setMaximumWidth(50);
	connect(_followGradientFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(followGradientFactorChanged(double)));


	label = new QLabel("Follow the current image");
	label->setAlignment(Qt::AlignLeft);
	//label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_followGradientFactorSpin);
	quickOptionsLayout->addLayout(hLayout);

	_quickOptions->setLayout(quickOptionsLayout);

	
	_optimizedOptions = new QWidget;
	QGridLayout* gridLyt=new QGridLayout;


	hLayout=new QHBoxLayout;

	gridLyt->addWidget(new QLabel("Weights: "),2,1);

	_orientationFactorSpin=new QDoubleSpinBox;
	_orientationFactorSpin->setRange(0,1000);
	_orientationFactorSpin->setValue(MyPreference::orientationFactor);
	_orientationFactorSpin->setDecimals(2);
	_orientationFactorSpin->setSingleStep(0.05);
	_orientationFactorSpin->setMaximumWidth(50);
	connect(_orientationFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(orientationFactorChanged(double)));

	label = new QLabel("Orientation ");
	label->setAlignment(Qt::AlignLeft);
	gridLyt->addWidget(label,3,1);
	gridLyt->addWidget(_orientationFactorSpin,4,1);

	_coherenceFactorSpin=new QDoubleSpinBox;
	_coherenceFactorSpin->setRange(0,1000);
	_coherenceFactorSpin->setValue(MyPreference::coherenceFactor);
	_coherenceFactorSpin->setDecimals(2);
	_coherenceFactorSpin->setSingleStep(0.05);
	_coherenceFactorSpin->setMaximumWidth(50);
	connect(_coherenceFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(coherenceFactorChanged(double)));

	label = new QLabel("Coherence ");
	label->setAlignment(Qt::AlignLeft);
	gridLyt->addWidget(label,3,2);
	gridLyt->addWidget(_coherenceFactorSpin,4,2);

	_colorFactorSpin=new QDoubleSpinBox;
	_colorFactorSpin->setRange(0,1000);
	_colorFactorSpin->setValue(MyPreference::colorFactor);
	_colorFactorSpin->setDecimals(2);
	_colorFactorSpin->setSingleStep(0.05);
	_colorFactorSpin->setMaximumWidth(50);
	connect(_colorFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(colorFactorChanged(double)));

	label = new QLabel("Color ");
	label->setAlignment(Qt::AlignLeft);
	gridLyt->addWidget(label,3,3);
	gridLyt->addWidget(_colorFactorSpin,4,3);
	
	_rigidityFactorSpin=new QDoubleSpinBox;
	_rigidityFactorSpin->setRange(0,1000);
	_rigidityFactorSpin->setValue(MyPreference::rigidityFactor);
	_rigidityFactorSpin->setDecimals(2);
	_rigidityFactorSpin->setSingleStep(0.05);
	_rigidityFactorSpin->setMaximumWidth(50);
	connect(_rigidityFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(rigidityFactorChanged(double)));

	label = new QLabel("Rigidity ");
	label->setAlignment(Qt::AlignLeft);
	gridLyt->addWidget(label,3,4);
	gridLyt->addWidget(_rigidityFactorSpin,4,4);


	_densityFactorSpin=new QDoubleSpinBox;
	_densityFactorSpin->setRange(0,1000);
	_densityFactorSpin->setValue(MyPreference::densityFactor);
	_densityFactorSpin->setDecimals(2);
	_densityFactorSpin->setSingleStep(0.05);
	_densityFactorSpin->setMaximumWidth(50);
	connect(_densityFactorSpin,SIGNAL(valueChanged(double)),this,SIGNAL(densityFactorChanged(double)));

	label = new QLabel("Density ");
	label->setAlignment(Qt::AlignLeft);
	gridLyt->addWidget(label,3,5);
	gridLyt->addWidget(_densityFactorSpin,4,5);

	_optimizedOptions->setLayout(gridLyt);


	 QGroupBox *bx = new QGroupBox(tr("Stroke Deformation"));

     _quickRadio = new QRadioButton(tr("Greedy (Faster)"));
     _optimizedRadio = new QRadioButton(tr("Optimized (Better)"));

	 connect(_optimizedRadio,SIGNAL(toggled(bool)),this,SLOT(strokeDeformWidget(bool)));

	 _optimizedRadio->setChecked(MyPreference::optimizeStrokes);

     QVBoxLayout *vbox = new QVBoxLayout;
     vbox->addWidget(_quickRadio);
	 vbox->addWidget(_quickOptions);
     vbox->addWidget(_optimizedRadio);
	 vbox->addWidget(_optimizedOptions);
     vbox->addStretch(1);
     bx->setLayout(vbox);

	 _optimizedOptions->show();
	 _quickOptions->hide();

	advancedOptionsLayout->addWidget(bx);




	hLayout=new QHBoxLayout;
	_optimizeStrokeWidths=new QCheckBox("Optimize Stroke Widths");
	_optimizeStrokeWidths->setChecked(MyPreference::optimizeStrokeWidths);
	connect(_optimizeStrokeWidths,SIGNAL(toggled(bool)),this,SIGNAL(optimizeStrokeWidthsChanged(bool)));

	hLayout->addWidget(_optimizeStrokeWidths);
	advancedOptionsLayout->addLayout(hLayout);


	hLayout=new QHBoxLayout;

    _oswColorFactorSpinBox=new QDoubleSpinBox;
    _oswColorFactorSpinBox->setRange(0.0,1000.0);
    _oswColorFactorSpinBox->setDecimals(3);
    _oswColorFactorSpinBox->setSingleStep(5);
    _oswColorFactorSpinBox->setMaximumWidth(50);
    _oswColorFactorSpinBox->setValue(MyPreference::oswColorFactor);
    connect(_oswColorFactorSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(oswColorFactorChanged(double)));



    hLayout->addWidget(new QLabel("Color Factor: "));
    hLayout->addWidget(_oswColorFactorSpinBox);

    _oswCoherenceFactorSpinBox=new QDoubleSpinBox;
    _oswCoherenceFactorSpinBox->setRange(0.0,1000.0);
    _oswCoherenceFactorSpinBox->setDecimals(3);
    _oswCoherenceFactorSpinBox->setSingleStep(1);
    _oswCoherenceFactorSpinBox->setMaximumWidth(50);
    _oswCoherenceFactorSpinBox->setValue(MyPreference::oswCoherenceFactor);
    connect(_oswCoherenceFactorSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(oswCoherenceFactorChanged(double)));

    hLayout->addWidget(new QLabel("Coherence Factor: "));
    hLayout->addWidget(_oswCoherenceFactorSpinBox);

    _oswRegularizationFactorSpinBox=new QDoubleSpinBox;
    _oswRegularizationFactorSpinBox->setRange(0.0,1000.0);
    _oswRegularizationFactorSpinBox->setDecimals(3);
    _oswRegularizationFactorSpinBox->setSingleStep(5);
    _oswRegularizationFactorSpinBox->setMaximumWidth(50);
    _oswRegularizationFactorSpinBox->setValue(MyPreference::oswRegularizationFactor);
    connect(_oswRegularizationFactorSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(oswRegularizationFactorChanged(double)));

    hLayout->addWidget(new QLabel("Regularization Factor: "));
    hLayout->addWidget(_oswRegularizationFactorSpinBox);

	advancedOptionsLayout->addLayout(hLayout);


	hLayout=new QHBoxLayout;
    _oswGrowFactorSpinBox=new QDoubleSpinBox;
    _oswGrowFactorSpinBox->setRange(0.0,1000.0);
    _oswGrowFactorSpinBox->setDecimals(3);
    _oswGrowFactorSpinBox->setSingleStep(5);
    _oswGrowFactorSpinBox->setMaximumWidth(50);
    _oswGrowFactorSpinBox->setValue(MyPreference::oswGrowFactor);
    connect(_oswGrowFactorSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(oswGrowFactorChanged(double)));

    hLayout->addWidget(new QLabel("Grow Factor: "));
    hLayout->addWidget(_oswGrowFactorSpinBox);

    _oswEdgeFactorSpinBox=new QDoubleSpinBox;
    _oswEdgeFactorSpinBox->setRange(0.0,1000.0);
    _oswEdgeFactorSpinBox->setDecimals(3);
    _oswEdgeFactorSpinBox->setSingleStep(5);
    _oswEdgeFactorSpinBox->setMaximumWidth(50);
    _oswEdgeFactorSpinBox->setValue(MyPreference::oswEdgeFactor);
    connect(_oswEdgeFactorSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(oswEdgeFactorChanged(double)));

    hLayout->addWidget(new QLabel("Edge Factor: "));
    hLayout->addWidget(_oswEdgeFactorSpinBox);

	advancedOptionsLayout->addLayout(hLayout);


	_advancedMotionOptionsWidget->setLayout(advancedOptionsLayout);
	_advancedMotionOptionsWidget->hide();
	topLayout->addWidget(_advancedMotionOptionsWidget);

	motionParameters->setLayout(topLayout);

	return motionParameters;
}


QWidget* PaintOptions::createFillingParametersWidget()
{

	QWidget* fillingParameters=new QWidget;

	QVBoxLayout* topLayout=new QVBoxLayout;


	QHBoxLayout *hLayout;
	QLabel *label;



	QGroupBox *groupBox=new QGroupBox(tr("How will strokes move in space? "));

	QGridLayout* gridLayout=new QGridLayout;

	gridLayout->addWidget(new QLabel("Weights "),1,2);
	gridLayout->addWidget(new QLabel("With a fixed direction"),2,1);

	_fixedOrientationFactor=new QSpinBox;
	_fixedOrientationFactor->setRange(0,100);
	_fixedOrientationFactor->setFixedWidth(40);
	_fixedOrientationFactor->setSingleStep(5);
	_fixedOrientationFactor->setValue(MyPreference::fixedOrientationFactor);
	connect(_fixedOrientationFactor,SIGNAL(valueChanged(int)),this,SIGNAL(fixedOrientationFactorChanged(int)));
	gridLayout->addWidget(_fixedOrientationFactor,2,2);



	_orientationDial = new QDial;
	//_orientationDial->setFixedWidth(50);
	_orientationDial->setFixedHeight(35);
	_orientationDial->setWrapping(true);
	_orientationDial->setMinimum(0);
	_orientationDial->setMaximum(359);
	_orientationDial->setValue(0);
	connect(_orientationDial,SIGNAL(valueChanged(int)),this,SIGNAL(orientationChanged(int)));
	gridLayout->addWidget(_orientationDial,2,3);


	gridLayout->addWidget(new QLabel("Following the region boundaries"),3,1);

	_followBoundaryFactor=new QSpinBox;
	_followBoundaryFactor->setRange(0,100);
	_followBoundaryFactor->setFixedWidth(40);
	_followBoundaryFactor->setSingleStep(5);
	_followBoundaryFactor->setValue(MyPreference::boundaryFactor);
	connect(_followBoundaryFactor,SIGNAL(valueChanged(int)),this,SIGNAL(followBoundaryFactorChanged(int)));
	gridLayout->addWidget(_followBoundaryFactor,3,2);

	gridLayout->addWidget(new QLabel("Following the image"),4,1);

	_followImageFactor=new QSpinBox;
	_followImageFactor->setRange(0,100);
	_followImageFactor->setFixedWidth(40);
	_followImageFactor->setSingleStep(5);
	_followImageFactor->setValue(MyPreference::imageGradientFactor);
	connect(_followImageFactor,SIGNAL(valueChanged(int)),this,SIGNAL(followImageFactorChanged(int)));
	gridLayout->addWidget(_followImageFactor,4,2);


	gridLayout->addWidget(new QLabel("Following guide strokes"),5,1);

	_followUserStrokesFactor=new QSpinBox;
	_followUserStrokesFactor->setRange(0,100);
	_followUserStrokesFactor->setFixedWidth(40);
	_followUserStrokesFactor->setSingleStep(5);
	_followUserStrokesFactor->setValue(MyPreference::userStrokesFactor);
	connect(_followUserStrokesFactor,SIGNAL(valueChanged(int)),this,SIGNAL(followUserStrokesFactorChanged(int)));
	gridLayout->addWidget(_followUserStrokesFactor,5,2);
	
	/*
	gridLayout->addWidget(new QLabel("How much noise? "),6,1);

	QSlider* orientationVarianceSlider=new QSlider(Qt::Horizontal);
	orientationVarianceSlider->setRange(0,100);
	orientationVarianceSlider->setFixedWidth(100);
	connect(orientationVarianceSlider,SIGNAL(valueChanged(int)),this,SLOT(setOrientationVariance(int)));
	orientationVarianceSlider->setValue(0);

	gridLayout->addWidget(orientationVarianceSlider,6,3);
	*/

	//QCheckBox* reverseDirection=new QCheckBox("Reverse direction sometimes? ");
	//reverseDirection->setChecked(MyPreference::reverseDirections);
	//connect(reverseDirection,SIGNAL(toggled(bool)),this,SIGNAL(includeReverseDirection(bool)));
	//gridLayout->addWidget(reverseDirection,7,1);


	groupBox->setLayout(gridLayout);

	topLayout->addWidget(groupBox);


    _maxStrokesSpin=new QSpinBox;
    //QSlider* maxStrokesSlider=new QSlider(Qt::Horizontal);
    _maxStrokesSpin->setRange(1,6000);
    //maxStrokesSlider->setRange(1,6000);
    _maxStrokesSpin->setValue(MyPreference::maxStrokes);
    _maxStrokesSpin->setMaximumWidth(50);
    //maxStrokesSlider->setValue(MyPreference::maxStrokes);
    //maxStrokesSlider->setFixedWidth(150);
    //connect(_maxStrokesSpin,SIGNAL(valueChanged(int)),maxStrokesSlider,SLOT(setValue(int)));
    //connect(maxStrokesSlider,SIGNAL(valueChanged(int)),_maxStrokesSpin,SLOT(setValue(int)));
    //connect(maxStrokesSlider,SIGNAL(valueChanged(int)),this,SIGNAL(maxStrokesChanged(int)));
    connect(_maxStrokesSpin,SIGNAL(valueChanged(int)),this,SIGNAL(maxStrokesChanged(int)));

    hLayout=new QHBoxLayout;
    label = new QLabel("Max # strokes to draw");
    label->setAlignment(Qt::AlignLeft);
    //label->setFixedWidth(300);
    hLayout->addWidget(label);
    hLayout->addWidget(_maxStrokesSpin);
    //hLayout->addWidget(maxStrokesSlider);
    topLayout->addLayout(hLayout);


    _colorDifferenceSpin=new QDoubleSpinBox;
    _colorDifferenceSpin->setRange(0.02,1);
    _colorDifferenceSpin->setSingleStep(0.1);
    _colorDifferenceSpin->setMaximumWidth(50);

    _colorDifferenceSpin->setValue(MyPreference::Threshold_ColorDifference);
    connect(_colorDifferenceSpin,SIGNAL(valueChanged(double)),this,SIGNAL(colorDifferenceChanged(double)));


	hLayout=new QHBoxLayout;
	label = new QLabel("Match stroke color to image: 0 closely - 1 loosely");
    label->setAlignment(Qt::AlignLeft);
    //label->setFixedWidth(100);
    hLayout->addWidget(label);
    hLayout->addWidget(_colorDifferenceSpin);
    //hLayout->addWidget(colorDifferenceSlider);
	topLayout->addLayout(hLayout);




	QCheckBox *showAdvancedFillingOptionsCheckbox=new QCheckBox("Show Advanced Options");
    showAdvancedFillingOptionsCheckbox->setChecked(false);
    connect(showAdvancedFillingOptionsCheckbox,SIGNAL(toggled(bool)),this,SLOT(showAdvancedFillingOptions(bool)));

	hLayout=new QHBoxLayout;
    hLayout->addWidget(showAdvancedFillingOptionsCheckbox);
    topLayout->addLayout(hLayout);

    _advancedFillingOptionsWidget = new QWidget();
	QVBoxLayout* advancedOptionsLayout=new QVBoxLayout;


	_strokeMinLengthSpin=new QSpinBox;
	_strokeMinLengthSpin->setRange(1,30);
	_strokeMinLengthSpin->setFixedWidth(40);
	_strokeMinLengthSpin->setValue(MyPreference::minStrokeLength);
	connect(_strokeMinLengthSpin,SIGNAL(valueChanged(int)),this,SIGNAL(minStrokeLengthChanged(int)));

	_strokeMaxLengthSpin=new QSpinBox;
	_strokeMaxLengthSpin->setRange(1,30);
	_strokeMaxLengthSpin->setFixedWidth(40);
	_strokeMaxLengthSpin->setValue(MyPreference::maxStrokeLength);
	_strokeMaxLengthSpin->setAlignment(Qt::AlignLeft);
	connect(_strokeMaxLengthSpin,SIGNAL(valueChanged(int)),this,SIGNAL(maxStrokeLengthChanged(int)));


	hLayout=new QHBoxLayout;
	label = new QLabel("Stroke length");
	label->setAlignment(Qt::AlignLeft);
	//label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_strokeMinLengthSpin);
	hLayout->addWidget(new QLabel(" to "));
	hLayout->addWidget(_strokeMaxLengthSpin);
	advancedOptionsLayout->addLayout(hLayout);



	_strokeDensitySpin=new QSpinBox;
	_strokeDensitySpin->setRange(0,100);
	_strokeDensitySpin->setValue(MyPreference::strokeDensity*1000.);
	_strokeDensitySpin->setMaximumWidth(50);
	_strokeDensitySpin->setSingleStep(5);
	connect(_strokeDensitySpin,SIGNAL(valueChanged(int)),this,SIGNAL(strokeDensityChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Strokes overlap (0 = no touching!) ");
	label->setAlignment(Qt::AlignLeft);
	hLayout->addWidget(label);
	hLayout->addWidget(_strokeDensitySpin);
	advancedOptionsLayout->addLayout(hLayout);
	

	_shiftToEdges=new QCheckBox("Shift strokes to edges?");
	_shiftToEdges->setChecked(false);
	connect(_shiftToEdges,SIGNAL(toggled(bool)),this,SIGNAL(shiftToEdgesChanged(bool)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(_shiftToEdges);
	advancedOptionsLayout->addLayout(hLayout);


    QGroupBox *strokeStartBox=new QGroupBox(tr("Where to draw strokes?"));
	QVBoxLayout* strokeStartLayout=new QVBoxLayout;


	_fillEmptyRegions=new QCheckBox("In empty canvas");
	_fillEmptyRegions->setChecked(MyPreference::fillEmptyRegions);
	connect(_fillEmptyRegions,SIGNAL(toggled(bool)),this,SIGNAL(fillEmptyRegionsChanged(bool)));

	_fillColorDifference=new QCheckBox("Color diff from image to canvas");
	_fillColorDifference->setChecked(MyPreference::fillColorDifference);
	connect(_fillColorDifference,SIGNAL(toggled(bool)),this,SIGNAL(fillColorDifferenceChanged(bool)));

	_fillUserSeed=new QCheckBox("User seed strokes");
	_fillUserSeed->setChecked(false);
	connect(_fillUserSeed,SIGNAL(toggled(bool)),this,SIGNAL(fillUserSeedChanged(bool)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(_fillEmptyRegions);
	hLayout->addWidget(_fillUserSeed);
	hLayout->addWidget(_fillColorDifference);
	strokeStartLayout->addLayout(hLayout);




	_minBrightnessSpin=new QSpinBox;
	_minBrightnessSpin->setRange(0,255);
	_minBrightnessSpin->setFixedWidth(40);
	_minBrightnessSpin->setSingleStep(5);
	_minBrightnessSpin->setValue(0);
	connect(_minBrightnessSpin,SIGNAL(valueChanged(int)),this,SIGNAL(minBrightnessChanged(int)));



	_maxBrightnessSpin=new QSpinBox;
	_maxBrightnessSpin->setRange(0,255);
	_maxBrightnessSpin->setFixedWidth(40);
	_maxBrightnessSpin->setSingleStep(5);
	_maxBrightnessSpin->setValue(255);
	connect(_maxBrightnessSpin,SIGNAL(valueChanged(int)),this,SIGNAL(maxBrightnessChanged(int)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Brightness range");
	label->setAlignment(Qt::AlignLeft);
	//label->setFixedWidth(210);
	hLayout->addWidget(label);
	hLayout->addWidget(_minBrightnessSpin);
	hLayout->addWidget(new QLabel(" to "));
	hLayout->addWidget(_maxBrightnessSpin);
	strokeStartLayout->addLayout(hLayout);


	_magThresholdSpin=new QSpinBox;
	_magThresholdSpin->setRange(1,500);
	_magThresholdSpin->setMaximumWidth(50);
	_magThresholdSpin->setValue(MyPreference::magnitudeThreshold);
	connect(_magThresholdSpin,SIGNAL(valueChanged(int)),this,SIGNAL(magnitudeThresholdChanged(int)));
	
	hLayout=new QHBoxLayout;
	label = new QLabel("Distance from edges/boundary/guide stroke ");
	label->setAlignment(Qt::AlignLeft);
	//label->setFixedWidth(200);
	hLayout->addWidget(label);
	hLayout->addWidget(_magThresholdSpin);
	//hLayout->addWidget(magThresholdSlider);
	strokeStartLayout->addLayout(hLayout);

	hLayout=new QHBoxLayout;
	_seedOrder = new QComboBox();
	_seedOrder->addItem("Random");
	_seedOrder->addItem("Light to Dark");
	_seedOrder->addItem("Dark to Light");
	_seedOrder->addItem("Edge");
	_seedOrder->addItem("Image Canvas Edge Difference");
	_seedOrder->addItem("Color Difference");
	QObject::connect(_seedOrder, SIGNAL(currentIndexChanged( const QString &)),this, SIGNAL(seedOrderChanged( const QString &)));

	hLayout->addWidget(new QLabel("Seed Order: "));
	hLayout->addWidget(_seedOrder);
	strokeStartLayout->addLayout(hLayout);

	_paintOccludedRegions=new QCheckBox("Draw strokes in occluded regions? ");
	_paintOccludedRegions->setChecked(MyPreference::paintOccludedRegions);
	connect(_paintOccludedRegions,SIGNAL(toggled(bool)),this,SIGNAL(paintOccludedRegionsChanged(bool)));

	_startInOccludedRegions=new QCheckBox("Start in occluded regions? ");
	_startInOccludedRegions->setChecked(MyPreference::startInOccludedRegions);
	connect(_startInOccludedRegions,SIGNAL(toggled(bool)),this,SIGNAL(startInOccludedRegionsChanged(bool)));


	_useOccludingLayerFlow=new QCheckBox("Use occluding region flow?");
	_useOccludingLayerFlow->setChecked(true);
	connect(_useOccludingLayerFlow,SIGNAL(toggled(bool)),this,SIGNAL(useOccludingLayerFlowChanged(bool)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(_paintOccludedRegions);
	
	hLayout->addWidget(_useOccludingLayerFlow);
	strokeStartLayout->addLayout(hLayout);



	_clipToSeedRegions=new QCheckBox("Strokes stay in seed regions?");
	_clipToSeedRegions->setChecked(false);
	connect(_clipToSeedRegions,SIGNAL(toggled(bool)),this,SIGNAL(clipToSeedRegionsChanged(bool)));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(_clipToSeedRegions);
	hLayout->addWidget(_startInOccludedRegions);
	strokeStartLayout->addLayout(hLayout);


	_addVisibleStrokesToLayer=new QCheckBox("Add visible strokes to layer after generation and switch to invisible?");
	_addVisibleStrokesToLayer->setChecked(MyPreference::addVisibleToStrokeLayer);
	connect(_addVisibleStrokesToLayer,SIGNAL(toggled(bool)),this,SIGNAL(addVisibleStrokesToLayer(bool)));
	strokeStartLayout->addWidget(_addVisibleStrokesToLayer);



	topLayout->setAlignment(Qt::AlignTop);
	strokeStartBox->setLayout(strokeStartLayout);
	advancedOptionsLayout->addWidget(strokeStartBox);

	_advancedFillingOptionsWidget->setLayout(advancedOptionsLayout);
	_advancedFillingOptionsWidget->hide();
	topLayout->addWidget(_advancedFillingOptionsWidget);

	fillingParameters->setLayout(topLayout);
	
	return fillingParameters;
}

QWidget* PaintOptions::createStrokePropertiesWidget()
{

	/* Stroke properties layout */

	QWidget* strokeProperties=new QWidget;
	QVBoxLayout* vLayout=new QVBoxLayout;

	vLayout->setAlignment(Qt::AlignTop);

	_layerParametersList = new QComboBox();
	setLayerParametersFromFile(_layerParametersList, QString("defaultStrokeLayerParameters.xml"));
	QObject::connect(_layerParametersList, SIGNAL(currentIndexChanged( int)),this, SLOT(layerParametersChanged(int)));

	QHBoxLayout *hLayout=new QHBoxLayout;

	hLayout->addWidget(new QLabel("Example Parameters:"));
	hLayout->addWidget(_layerParametersList);

	vLayout->addLayout(hLayout);


	//QPushButton* _saveParametersButton=new QPushButton(tr("Save Parameters"));
	//_saveParametersButton->setFixedWidth(100);
	//connect(_saveParametersButton,SIGNAL(clicked()),this,SLOT(saveParameters()));

	QPushButton* addCurrentParametersButton=new QPushButton(tr("Add New"));
	connect(addCurrentParametersButton,SIGNAL(clicked()),this,SLOT(addCurrentParameters()));

	QPushButton* replaceSelectedParametersButton=new QPushButton(tr("Replace Selected"));
	connect(replaceSelectedParametersButton,SIGNAL(clicked()),this,SLOT(replaceSelectedParameters()));

	QPushButton* deleteSelectedParametersButton=new QPushButton(tr("Delete Selected"));
	connect(deleteSelectedParametersButton,SIGNAL(clicked()),this,SLOT(deleteSelectedParameters()));

	QPushButton* saveParametersButton=new QPushButton(tr("Save"));
	connect(saveParametersButton,SIGNAL(clicked()),this,SLOT(saveParameters()));

	hLayout=new QHBoxLayout;
	hLayout->addWidget(addCurrentParametersButton);
	hLayout->addWidget(replaceSelectedParametersButton);
	hLayout->addWidget(deleteSelectedParametersButton);
	hLayout->addWidget(saveParametersButton);
	vLayout->addLayout(hLayout);

	/* Texture layout */
	_strokeGroupBox=new QGroupBox(tr("Automatic Stroke Options:"));
	QVBoxLayout* vbox=new QVBoxLayout;

	//scrollArea=new QScrollArea;
	

	_strokeListWidget = new QListWidget;
	//_strokeListWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);

	_strokeListWidget->setFixedSize(400,200);


	//QListBoxPixmap

	_strokeListWidget->setIconSize(QSize(1000,250));

	QStringList brushes;
    brushes << "Brushes/soft.png" << "Brushes/hayes2.png"<<"Brushes/hayes.pgm"  << "Brushes/paintbrush03.pgm" 
			<<"Brushes/paintbrush.pgm" << "Brushes/cylinder-fat.png"<< "Brushes/cylinder-thin.png"
			<< "Brushes/crayon01.pgm" << "Brushes/paintbrush04.pgm" << "Brushes/paintbrush09.pgm" << "Brushes/paintbrush10.pgm" 
			<< "Brushes/paintbrush05.pgm" << "Brushes/pointillist.pgm"<< "Brushes/arrow01.pgm"
			<< "Brushes/paintbrush01.pgm" << "Brushes/crayon06.pgm"<< "Brushes/crayon07.pgm"
			<< "Brushes/flower03.pgm" << "Brushes/wavy.pgm"<< "Brushes/chalk01.pgm"<< "Brushes/fish4.pgm";

	QListWidgetItem *item;

	QStringList::const_iterator constIterator;
	for (constIterator = brushes.constBegin(); constIterator != brushes.constEnd();++constIterator)
	{
		
		cout << (*constIterator).toLocal8Bit().constData() << endl;

		QPixmap imgpixmap(*constIterator);
		imgpixmap = imgpixmap.scaledToHeight(50,Qt::SmoothTransformation);
		QIcon imgicon(imgpixmap);
		item = new QListWidgetItem(imgicon,*constIterator);
		item->setBackgroundColor(QColor(0,0,0));
		item->setTextColor(QColor(255,255,255));
		_strokeListWidget->addItem(item);
	}
	_strokeListWidget->setCurrentRow(0);

	connect(_strokeListWidget,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(newTextureSelected(QListWidgetItem*)));

	//Q3ListBox *_strokeListBox = new Q3ListBox;
	//Q3ListBoxPixmap * lboxpix = new Q3ListBoxPixmap(imgpixmap);
	//_strokeListBox->insertItem(lboxpix);
	
	
	//_StrokeImageViewer=new ImageViewer;
	
	/*
	QImage *im = new QImage(QString("Brushes/soft.png")); 

	*im = im->scaled(100,20);

	if ( im == NULL )
		QMessageBox::warning(this,"Warning","Can't find the default brush in HOMEDIR/Brushes\n");
	else
		_StrokeImageViewer->loadImage(*im);


	QImage im(QString("Brushes/soft.png")); 
	im = im.scaled(200,50);
	_StrokeImageViewer->loadImage(im);

	_StrokeImageViewer->imageTransparencyChanged(20);

	vbox->addWidget(_StrokeImageViewer);
	*/


	vbox->addWidget(_strokeListWidget);

	hLayout=new QHBoxLayout;
	
	//_StrokeImageViewer->setMaxScale(1);
	//_StrokeImageViewer->setMinScale(0.2);
	//scrollArea->setWidget(_StrokeImageViewer);

	_loadTextureButton=new QPushButton(tr("Load Texture"));
	_loadTextureButton->setFixedWidth(100);
	connect(_loadTextureButton,SIGNAL(clicked()),this,SLOT(loadTexture()));

	//vbox->addWidget(_StrokeImageViewer);
	hLayout->addWidget(_loadTextureButton);

	_chooseColorButton=new QPushButton(tr("Stroke Color"));
	_chooseColorButton->setAutoFillBackground(true);
	_chooseColorButton->setFixedWidth(100);
	connect(_chooseColorButton,SIGNAL(clicked()),this,SLOT(selectColor()));
	hLayout->addWidget(_chooseColorButton);

	_imgColor=new QCheckBox("Use color from image?");
	_imgColor->setChecked(MyPreference::useImageColor);
	_imgColor->setShortcut(tr("Ctrl+R"));
	connect(_imgColor,SIGNAL(toggled(bool)),this,SIGNAL(useImageColor(bool)));
	hLayout->addWidget(_imgColor);

	vbox->addLayout(hLayout);

	hLayout=new QHBoxLayout;

	_fillStroke=new QCheckBox("Fill Inside? ");
	_fillStroke->setChecked(false);
	connect(_fillStroke,SIGNAL(toggled(bool)),this,SIGNAL(fillStroke(bool)));
	hLayout->addWidget(_fillStroke);
	

	vbox->addLayout(hLayout);



	QCheckBox *offsetCheckbox = new QCheckBox("Show color shifting and noise options?");
	connect(offsetCheckbox,SIGNAL(toggled(bool)),this,SLOT(offsetNoiseWidget(bool)));
	vbox->addWidget(offsetCheckbox);

    _offsetWidget = new QWidget();
    //offsetBox->setCheckable(true);
    //offsetBox->setChecked(false);

	QVBoxLayout* offsetvbox=new QVBoxLayout;





	_hueOffsetSpinBox=new QSpinBox;
	_hueOffsetSpinBox->setRange(-200,200);
	_hueOffsetSpinBox->setSingleStep(1);
	_hueOffsetSpinBox->setMaximumWidth(50);
	_hueOffsetSpinBox->setValue(0);
	connect(_hueOffsetSpinBox,SIGNAL(valueChanged(int)),this,SIGNAL(hueOffsetChanged(int)));

	_hueNoiseSpinBox=new QDoubleSpinBox;
	_hueNoiseSpinBox->setRange(0,50);
	_hueNoiseSpinBox->setSingleStep(1);
	_hueNoiseSpinBox->setDecimals(1);
	_hueNoiseSpinBox->setMaximumWidth(50);
	_hueNoiseSpinBox->setValue(0);
	connect(_hueNoiseSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(hueNoiseChanged(double)));

	hLayout=new QHBoxLayout;
	QLabel *label = new QLabel("Hue Offset:");
	label->setAlignment(Qt::AlignLeft);
	//label->setMaximumWidth(60);
	hLayout->addWidget(label);
	hLayout->addWidget(_hueOffsetSpinBox);
	label = new QLabel("Hue Noise:");
	label->setAlignment(Qt::AlignLeft);
	//label->setMaximumWidth(50);
	hLayout->addWidget(label);
	hLayout->addWidget(_hueNoiseSpinBox);
	offsetvbox->addLayout(hLayout);


	_saturationOffsetSpinBox=new QSpinBox;
	_saturationOffsetSpinBox->setRange(-200,200);
	_saturationOffsetSpinBox->setSingleStep(1);
	_saturationOffsetSpinBox->setMaximumWidth(50);
	_saturationOffsetSpinBox->setValue(0);
	connect(_saturationOffsetSpinBox,SIGNAL(valueChanged(int)),this,SIGNAL(saturationOffsetChanged(int)));

	_saturationNoiseSpinBox=new QDoubleSpinBox;
	_saturationNoiseSpinBox->setRange(0,50);
	_saturationNoiseSpinBox->setSingleStep(1);
	_saturationNoiseSpinBox->setDecimals(1);
	_saturationNoiseSpinBox->setMaximumWidth(50);
	_saturationNoiseSpinBox->setValue(0);
	connect(_saturationNoiseSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(saturationNoiseChanged(double)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Saturation Offset:");
	label->setAlignment(Qt::AlignLeft);
	//label->setMaximumWidth(60);
	hLayout->addWidget(label);
	hLayout->addWidget(_saturationOffsetSpinBox);
	label = new QLabel("Saturation Noise:");
	label->setAlignment(Qt::AlignLeft);
	//label->setMaximumWidth(50);
	hLayout->addWidget(label);
	hLayout->addWidget(_saturationNoiseSpinBox);
	offsetvbox->addLayout(hLayout);


	_valueOffsetSpinBox=new QSpinBox;
	_valueOffsetSpinBox->setRange(-200,200);
	_valueOffsetSpinBox->setSingleStep(1);
	_valueOffsetSpinBox->setMaximumWidth(50);
	_valueOffsetSpinBox->setValue(0);
	connect(_valueOffsetSpinBox,SIGNAL(valueChanged(int)),this,SIGNAL(valueOffsetChanged(int)));

	_valueNoiseSpinBox=new QDoubleSpinBox;
	_valueNoiseSpinBox->setRange(0,50);
	_valueNoiseSpinBox->setSingleStep(1);
	_valueNoiseSpinBox->setDecimals(1);
	_valueNoiseSpinBox->setMaximumWidth(50);
	_valueNoiseSpinBox->setValue(0);
	connect(_valueNoiseSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(valueNoiseChanged(double)));

	hLayout=new QHBoxLayout;
	label = new QLabel("Value Offset:");
	label->setAlignment(Qt::AlignLeft);
	//label->setMaximumWidth(60);
	hLayout->addWidget(label);
	hLayout->addWidget(_valueOffsetSpinBox);
	label = new QLabel("Value Noise:");
	label->setAlignment(Qt::AlignLeft);
	//label->setMaximumWidth(50);
	hLayout->addWidget(label);
	hLayout->addWidget(_valueNoiseSpinBox);
	offsetvbox->addLayout(hLayout);

	_offsetWidget->setLayout(offsetvbox);

	_offsetWidget->hide();
	vbox->addWidget(_offsetWidget);


	
	hLayout=new QHBoxLayout;
	_linkWidths=new QCheckBox("Link Widths");
	_linkWidths->setChecked(true);
	//connect(_guideStroke,SIGNAL(toggled(bool)),this,SIGNAL(guideStroke(bool)));
	hLayout->addWidget(_linkWidths);
	vbox->addLayout(hLayout);

	_spinBoxStrokeStartWidth=new QSpinBox;
	QSlider* sliderStrokeStartWidth=new QSlider(Qt::Horizontal);
	_spinBoxStrokeStartWidth->setRange(1,50);
    sliderStrokeStartWidth->setRange(1,50);
    sliderStrokeStartWidth->setFixedWidth(150);
	connect(_spinBoxStrokeStartWidth,SIGNAL(valueChanged(int)),sliderStrokeStartWidth,SLOT(setValue(int)));
	connect(sliderStrokeStartWidth,SIGNAL(valueChanged(int)),_spinBoxStrokeStartWidth,SLOT(setValue(int)));
	//connect(sliderStrokeStartWidth,SIGNAL(valueChanged(int)),this,SIGNAL(strokeStartWidthChanged(int)));
	connect(sliderStrokeStartWidth,SIGNAL(valueChanged(int)),this,SLOT(setStrokeStartWidth(int)));
	

	_spinBoxStrokeEndWidth=new QSpinBox;
	QSlider* sliderStrokeEndWidth=new QSlider(Qt::Horizontal);
	_spinBoxStrokeEndWidth->setRange(1,50);
	sliderStrokeEndWidth->setRange(1,50);
    sliderStrokeEndWidth->setFixedWidth(150);
	connect(_spinBoxStrokeEndWidth,SIGNAL(valueChanged(int)),sliderStrokeEndWidth,SLOT(setValue(int)));
	connect(sliderStrokeEndWidth,SIGNAL(valueChanged(int)),_spinBoxStrokeEndWidth,SLOT(setValue(int)));
	//connect(sliderStrokeEndWidth,SIGNAL(valueChanged(int)),this,SIGNAL(strokeEndWidthChanged(int)));
	connect(sliderStrokeEndWidth,SIGNAL(valueChanged(int)),this,SLOT(setStrokeEndWidth(int)));
	sliderStrokeEndWidth->setValue(MyPreference::endStrokeWidth);
	sliderStrokeStartWidth->setValue(MyPreference::startStrokeWidth);

	hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("Width:"));
	hLayout->addWidget(_spinBoxStrokeStartWidth);
	hLayout->addWidget(sliderStrokeStartWidth);
	vbox->addLayout(hLayout);


	hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("End Width:"));
	hLayout->addWidget(_spinBoxStrokeEndWidth);
	hLayout->addWidget(sliderStrokeEndWidth);
	vbox->addLayout(hLayout);





	_spinBoxAlpha=new QSpinBox;
	QSlider* sliderStrokeAlpha=new QSlider(Qt::Horizontal);
	_spinBoxAlpha->setRange(0,100);
	sliderStrokeAlpha->setRange(0,100);
	sliderStrokeAlpha->setSingleStep(10);
	sliderStrokeAlpha->setFixedWidth(150);
	connect(_spinBoxAlpha,SIGNAL(valueChanged(int)),sliderStrokeAlpha,SLOT(setValue(int)));
	connect(sliderStrokeAlpha,SIGNAL(valueChanged(int)),_spinBoxAlpha,SLOT(setValue(int)));
	connect(sliderStrokeAlpha,SIGNAL(valueChanged(int)),this,SLOT(setAlpha(int)));
	sliderStrokeAlpha->setValue(100);

	hLayout=new QHBoxLayout;
	hLayout->addWidget(new QLabel("Transparency: "));
	hLayout->addWidget(_spinBoxAlpha);
	hLayout->addWidget(sliderStrokeAlpha);
	vbox->addLayout(hLayout);




	_strokeGroupBox->setLayout(vbox);
	vLayout->addWidget(_strokeGroupBox);


    _userStrokeOptionsWidget = new QWidget();
	QVBoxLayout* userStrokeOptionsLayout=new QVBoxLayout;

	hLayout=new QHBoxLayout;

	_visibleStroke=new QCheckBox("Stroke Visible?");
	_visibleStroke->setChecked(true);
	//_visibleStroke->hide();
	connect(_visibleStroke,SIGNAL(toggled(bool)),this,SIGNAL(visibleStroke(bool)));

	hLayout->addWidget(_visibleStroke);

	userStrokeOptionsLayout->addLayout(hLayout);

	userStrokeOptionsLayout->addWidget(new QLabel("How does the user stroke affect automatic strokes?"));
	

	hLayout=new QHBoxLayout;




	_guideStroke=new QCheckBox("Guide Orientation? ");
	_guideStroke->setChecked(true);
	_guideStroke->setMaximumWidth(115);
	_guideStroke->setStyleSheet("border-color: blue; border-style: outset; border-width: 2px;");
	connect(_guideStroke,SIGNAL(toggled(bool)),this,SIGNAL(guideStroke(bool)));
	hLayout->addWidget(_guideStroke);

	_colorStroke=new QCheckBox("Affect Color? ");
	_colorStroke->setChecked(true);
	_colorStroke->setMaximumWidth(100);
	_colorStroke->setStyleSheet("border-color: red;  border-style: outset;border-width: 2px");
	connect(_colorStroke,SIGNAL(toggled(bool)),this,SIGNAL(colorStroke(bool)));
	hLayout->addWidget(_colorStroke);

	_seedStroke=new QCheckBox("Seed System? ");
	_seedStroke->setChecked(false);
	_seedStroke->setMaximumWidth(100);
	_seedStroke->setStyleSheet("border-color: green; border-style: outset; border-width: 2px");
	connect(_seedStroke,SIGNAL(toggled(bool)),this,SIGNAL(seedStroke(bool)));
	hLayout->addWidget(_seedStroke);
	userStrokeOptionsLayout->addLayout(hLayout);




	_userStrokeOptionsWidget->setLayout(userStrokeOptionsLayout);

	vLayout->addWidget(_userStrokeOptionsWidget);

	_userStrokeOptionsWidget->hide();

	strokeProperties->setLayout(vLayout);

	return strokeProperties;
}



void PaintOptions::setOrientationVariance(int orientationVariance)
{
	emit orientationVarianceChanged((double) orientationVariance / 100.);
}

void PaintOptions::setMovementVariance(int movementVariance)
{
	emit movementVarianceChanged((double) movementVariance / 25.);
}

void PaintOptions::setAlpha(int alpha)
{
	emit alphaChanged(((double) alpha)/100.);
}



void PaintOptions::loadTexture()
{
	printf("loading texture\n");
	QString filename=QFileDialog::getOpenFileName(this,"Select Stroke Texture","Brushes/",tr("All Files (*.*)"));
	if(!filename.isEmpty())
	{

		QList<QListWidgetItem *> itms = _strokeListWidget->findItems(filename,Qt::MatchContains| Qt::MatchExactly);
		if (itms.size()==0)
		{
			QListWidgetItem *item;

			QPixmap imgpixmap(filename);
			imgpixmap = imgpixmap.scaledToHeight(50,Qt::SmoothTransformation);
			QIcon imgicon(imgpixmap);
			item = new QListWidgetItem(imgicon,filename);
			item->setBackgroundColor(QColor(0,0,0));
			item->setTextColor(QColor(255,255,255));
			_strokeListWidget->addItem(item);
			_strokeListWidget->setCurrentItem(item);
		}

		emit loadTexture(filename);
		//setTexture(filename);
	}
}

void PaintOptions::newTextureSelected(QListWidgetItem * item)
{
	QString filename = item->text();
	if(!filename.isEmpty())
	{
		QImage im(filename);  
		if (im.width()==0)
			return;
//_strokeListWidget
		im = im.scaled(160,40);

		//if ( im == NULL )
		//{
		//	QMessageBox::warning(this,"Warning","Problem reading the image");
		//	return;
		//} 
		//_StrokeImageViewer->loadImage(im);

		emit loadTexture(filename);
	}

}


void PaintOptions::setTexture(QString filename)
{

	if(!filename.isEmpty())
	{
		//QImage im(filename);  

		QList<QListWidgetItem *> itms = _strokeListWidget->findItems(filename,Qt::MatchContains| Qt::MatchExactly);
		if (itms.size()==0)
		{
			QListWidgetItem *item;

			QPixmap imgpixmap(filename);
			imgpixmap = imgpixmap.scaledToHeight(50,Qt::SmoothTransformation);
			QIcon imgicon(imgpixmap);
			item = new QListWidgetItem(imgicon,filename);
			item->setBackgroundColor(QColor(0,0,0));
			item->setTextColor(QColor(255,255,255));
			_strokeListWidget->addItem(item);
			_strokeListWidget->setCurrentItem(item);
		}
		else
		{
			_strokeListWidget->setCurrentItem(itms[0]);
		}


		//if ( im == NULL )
		//{
		//	QMessageBox::warning(this,"Warning","Problem reading the image");
		//	return;
		//} 
		//_StrokeImageViewer->loadImage(im);

		emit loadTexture(filename);
	}
}




void PaintOptions::selectPaperTexture()
{
	printf("loading paper texture\n");
	QString filename=QFileDialog::getOpenFileName(this,"Select Paper Texture","Paper/",tr("All Files (*.*)"));
	if(!filename.isEmpty())
	{
		setPaperTexture(filename);
	}
}


void PaintOptions::setPaperTexture(QString filename)
{

	if(!filename.isEmpty())
	{
		
		QImage* im = new QImage(filename);  

		printf("width height %i %i\n", im->width(),im->height());

		if (im->width() > 200)
			*im = im->scaledToWidth(200);

		if (im->height() > 200)
			*im = im->scaledToHeight(200);

		printf("width height %i %i\n", im->width(),im->height());

		if ( im == NULL )
		{
			QMessageBox::warning(this,"Warning","Problem reading the image");
			return;
		} 
		_PaperImageViewer->loadImage(*im);

		delete im;

		emit loadPaperTexture(filename);
	}
}


void PaintOptions::selectColor()
{
	_col = QColorDialog::getColor(_col);
	if (!_col.isValid()) 
		return;
  
    QString colorString = QString("color: rgb(")+QString::number(_col.red())+QString(", ")+QString::number(_col.green())+QString(", ")+QString::number(_col.blue())+QString("); border-style: solid; border-width: 6px;");
	_chooseColorButton->setStyleSheet(colorString);

	emit useNewColor(_col);
}

void PaintOptions::selectPaperColor()
{
	_papercol = QColorDialog::getColor(_papercol);
	if (!_papercol.isValid()) 
		return;

         QString colorString = QString("color: rgb(")+QString::number(_papercol.red())+QString(", ")+QString::number(_papercol.green())+QString(", ")+QString::number(_papercol.blue())+QString("); border-style: solid; border-width: 6px;");
         _choosePaperColorButton->setStyleSheet(colorString);


	emit usePaperColor(_papercol);
}


void PaintOptions::setStrokeEndWidth(int strokeEndWidth)
{
	if (_linkWidths->isChecked())
	{
		_linkWidths->setChecked(false);
		_spinBoxStrokeStartWidth->setValue(strokeEndWidth);
		_linkWidths->setChecked(true);
	}

	emit strokeEndWidthChanged(strokeEndWidth);
}


void PaintOptions::setStrokeStartWidth(int strokeStartWidth)
{
	if (_linkWidths->isChecked())
	{

		_linkWidths->setChecked(false);
		_spinBoxStrokeEndWidth->setValue(strokeStartWidth);
		_linkWidths->setChecked(true);
	}

	emit strokeStartWidthChanged(strokeStartWidth);
}

void PaintOptions::increaseStrokeWidth()
{
	printf("increasing stroke width\n");
	_spinBoxStrokeStartWidth->setValue(min(_spinBoxStrokeStartWidth->value()+1,50));
	_spinBoxStrokeEndWidth->setValue(min(_spinBoxStrokeEndWidth->value()+1,50));
}

void PaintOptions::decreaseStrokeWidth()
{
	printf("decreasing stroke width\n");

	_spinBoxStrokeStartWidth->setValue(max(_spinBoxStrokeStartWidth->value()-1,1));
	_spinBoxStrokeEndWidth->setValue(max(_spinBoxStrokeEndWidth->value()-1,1));
}


void PaintOptions::showAdvancedFillingOptions(bool showAdvancedFillingOptions)
{
	if (showAdvancedFillingOptions)
		_advancedFillingOptionsWidget->show();
	else
		_advancedFillingOptionsWidget->hide();
}

void PaintOptions::showAdvancedMotionOptions(bool showAdvancedMotionOptions)
{
	if (showAdvancedMotionOptions)
		_advancedMotionOptionsWidget->show();
	else
		_advancedMotionOptionsWidget->hide();
}


void PaintOptions::showTestingOptions(bool showTestingOptions)
{
	if (showTestingOptions)
		_testingOptionsWidget->show();
	else
		_testingOptionsWidget->hide();
}

void PaintOptions::showUserStrokeOptions(bool showUserStrokeOptions)
{
	if (showUserStrokeOptions)
	{
		_strokeGroupBox->setTitle(QString("User Stroke Options"));
		_userStrokeOptionsWidget->show();
	}
	else
	{
		_strokeGroupBox->setTitle(QString("Automatic Stroke Options"));
		_userStrokeOptionsWidget->hide();

	}
}



void PaintOptions::offsetNoiseWidget(bool showwidget)
{
	if (showwidget)
		_offsetWidget->show();
	else
		_offsetWidget->hide();
}

void PaintOptions::strokeDeformWidget(bool showOptimizedOptions)
{

	if (showOptimizedOptions)
	{
		_optimizedOptions->show();
		_quickOptions->hide();
	}
	else
	{
		_optimizedOptions->hide();
		_quickOptions->show();
	}

	emit optimizeStrokesChanged(showOptimizedOptions);
}


void PaintOptions::updatePropertiesForSelectedStroke(StrokeProperties& prop)
{
	setTexture(prop.textureFilename);

	_col = prop.color;
	emit useNewColor(prop.color);

    QString colorString = QString("color: rgb(")+QString::number(_col.red())+QString(", ")+QString::number(_col.green())+QString(", ")+QString::number(_col.blue())+QString("); border-style: solid; border-width: 6px;");
    _chooseColorButton->setStyleSheet(colorString);



	_spinBoxAlpha->setValue(prop.alpha*100);
	_guideStroke->setChecked(prop.isGuide);
	_colorStroke->setChecked(prop.isColor);
	_visibleStroke->setChecked(prop.isVisible);
	_fillStroke->setChecked(prop.isFill);
	_seedStroke->setChecked(prop.isSeed);
	_imgColor->setChecked(prop.useImageColor);
	_spinBoxStrokeStartWidth->setValue(prop.startRadius);
	_spinBoxStrokeEndWidth->setValue(prop.endRadius);

}

void PaintOptions::loadLayerParameters(StrokeLayerProperties& slp)
{
	
	//stroke options tab
	
	_spinBoxStrokeStartWidth->setValue(slp.strokeProperties.startRadius);
	_spinBoxStrokeEndWidth->setValue(slp.strokeProperties.endRadius);
	_hueOffsetSpinBox->setValue(slp.hueOffset);
	_hueNoiseSpinBox->setValue(slp.hueNoise);
	_saturationOffsetSpinBox->setValue(slp.saturationOffset);
	_saturationNoiseSpinBox->setValue(slp.saturationNoise);
	_valueOffsetSpinBox->setValue(slp.valueOffset);
	_valueNoiseSpinBox->setValue(slp.valueNoise);
	_spinBoxAlpha->setValue(slp.strokeProperties.alpha*100);
	_imgColor->setChecked(slp.strokeProperties.useImageColor);

	if (slp.strokeProperties.useImageColor==false)
	{
		QString colorString = QString("color: rgb(")+QString::number(slp.strokeProperties.color.red())+QString(", ")+QString::number(slp.strokeProperties.color.green())+QString(", ")+QString::number(slp.strokeProperties.color.blue())+QString("); border-style: solid; border-width: 6px;");
		_chooseColorButton->setStyleSheet(colorString);
		emit useNewColor(slp.strokeProperties.color);
	}
	setTexture(slp.strokeProperties.textureFilename);

	_fillStroke->setChecked(slp.strokeProperties.isFill);
	_addVisibleStrokesToLayer->setChecked(slp.addVisibleStrokesToLayer);
	_maxStrokesSpin->setValue(slp.maxStrokes);
	_colorDifferenceSpin->setValue(slp.colorDifference);

	_fillEmptyRegions->setChecked(slp.fillEmptyRegions);
	_fillColorDifference->setChecked(slp.fillColorDifference);
	_fillUserSeed->setChecked(slp.fillUserSeed);


	//motion options tab
	_fixedMovementFactor->setValue(slp.fixedFlowFactor);
	_movementDial->setValue(slp.fixedMovementAngle*(180.0/PI)+90);
	_fixedMovementRateSpin->setValue(slp.fixedMovementRate);
	_boundaryMovementFactor->setValue(slp.boundaryFlowFactor);
	_opticalFlowFactor->setValue(slp.imageFlowFactor);
	_userStrokesMovementFactor->setValue(slp.userFlowFactor);
	_colorUpdateSpinBox->setValue(slp.colorUpdate);
	_maxFramesSpin->setValue(slp.maxFrames);
	_minFramesSpin->setValue(slp.minFrames);
	_fadeRateSpin->setValue(slp.fadeRate);
	_maxStrokeAngleSpin->setValue(slp.maxStrokeAngle);
	_maxFlowAngleSpin->setValue(slp.maxFlowAngle);
	_followStrokesFactorSpin->setValue(slp.followPreviousStrokesFactor);
	_followGradientFactorSpin->setValue(slp.followGradientFactor);

	_quickRadio->setChecked(!(slp.optimizeStrokes));
	_optimizedRadio->setChecked(slp.optimizeStrokes);


	_orientationFactorSpin->setValue(slp.orientationFactor);
	_coherenceFactorSpin->setValue(slp.coherenceFactor);
	_colorFactorSpin->setValue(slp.colorFactor);
	_rigidityFactorSpin->setValue(slp.rigidityFactor);
	_densityFactorSpin->setValue(slp.densityFactor);

	//filling options tab
	_fixedOrientationFactor->setValue(slp.fixedOrientationFactor);

	_orientationDial->setValue(slp.orientationAngle*(180.0/PI)+90);
	_followBoundaryFactor->setValue(slp.boundaryFactor);
	_followImageFactor->setValue(slp.imageGradientFactor);
	_followUserStrokesFactor->setValue(slp.userStrokesFactor);
	_strokeMinLengthSpin->setValue(slp.minStrokeLength);
	_strokeMaxLengthSpin->setValue(slp.maxStrokeLength);
	_strokeDensitySpin->setValue(slp.strokeDensity*1000);
	_minBrightnessSpin->setValue(slp.minBrightness);
	_maxBrightnessSpin->setValue(slp.maxBrightness);
	_magThresholdSpin->setValue(slp.magnitudeThreshold);
	_paintOccludedRegions->setChecked(slp.paintOccludedRegions);
	_startInOccludedRegions->setChecked(slp.startInOccludedRegions);
	_useOccludingLayerFlow->setChecked(slp.useOccludingLayerFlow);
	_clipToSeedRegions->setChecked(slp.clipToSeedRegions);
	_shiftToEdges->setChecked(slp.shiftToEdges);

	_optimizeStrokeWidths->setChecked(slp.optimizeStrokeWidths);
	_oswColorFactorSpinBox->setValue(slp.oswColorFactor);
	_oswEdgeFactorSpinBox->setValue(slp.oswEdgeFactor);
	_oswCoherenceFactorSpinBox->setValue(slp.oswCoherenceFactor);
	_oswRegularizationFactorSpinBox->setValue(slp.oswRegularizationFactor);
	_oswGrowFactorSpinBox->setValue(slp.oswGrowFactor);

	int idx =_seedOrder->findText(slp.seedOrder);
	if (idx > -1)
		_seedOrder->setCurrentIndex(idx);


}

void PaintOptions::layerParametersChanged(int slpIndex)
{
	if ((slpIndex <0) || (slpIndex >_slpList.size()))
		return;
	loadLayerParameters(_slpList[slpIndex]);
}

void PaintOptions::setLayerParametersFromFile(QComboBox* combo, QString filename)
{
	QDomDocument doc("mydocument");
	QFile file(filename);
	if ((!file.open(QIODevice::ReadOnly))||(!doc.setContent(&file)))
	{
		QMessageBox::warning(this,"Warning","Problem with reading in default layer parameters");
		file.close();
		return;
	}
	file.close();


	QDomElement root=doc.documentElement();

	if (root.tagName()=="StrokeLayerPropertiesList")
	{
		QDomNode n=root.firstChild();
		while(!n.isNull())
		{
			QDomElement e=n.toElement();
			StrokeLayerProperties slp;
			slp.readStrokeLayerPropertiesFromXML(e);
			combo->addItem(slp.layerParameterName);
			_slpList.push_back(slp);
			n=n.nextSibling();
		}
	}
}

void PaintOptions::deleteSelectedParameters()
{
	_slpList.removeAt(_layerParametersList->currentIndex());
	_layerParametersList->removeItem(_layerParametersList->currentIndex());
}

void PaintOptions::saveParameters()
{

	QFile xmlfile(QString("defaultStrokeLayerParameters.xml"));

	if(xmlfile.open(QIODevice::WriteOnly)==false)
	{
		LOG_STR_COUT("Problem writing default parameter xml file");
		return;
	}

	XmlWriter xw(&xmlfile);
	xw.setAutoNewLine(true);

	xw.writeOpenTag("StrokeLayerPropertiesList");

	for(int i=0;i < _slpList.size();i++)
	{
		_slpList[i].writeStrokeLayerPropertiesToXML(xw);
	}

	xw.writeCloseTag("StrokeLayerPropertiesList");

}

void PaintOptions::replaceSelectedParameters()
{
	QString layerParameterName =_slpList[_layerParametersList->currentIndex()].layerParameterName;
	StrokeLayerProperties slp;
	setLayerParametersFromInterface(slp);
	slp.layerParameterName=layerParameterName;
	_slpList[_layerParametersList->currentIndex()]=slp;
}

void PaintOptions::addCurrentParameters()
{
	
	QString layerParameterName=QInputDialog::getText(this,"Annotation","Please name the current parameters:",QLineEdit::Normal,QString(),0,Qt::Dialog);

	if (layerParameterName.length()==0)
		return;
	int idx = _layerParametersList->findText(layerParameterName);
	if (idx>=0)
	{
		QMessageBox msgBox;
		msgBox.setText("Name already exists. Overwrite?");
		msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
		int ret = msgBox.exec();
		if (ret==QMessageBox::Cancel)
		{
			return;
		}	
	}

	StrokeLayerProperties slp;
	setLayerParametersFromInterface(slp);
	slp.layerParameterName=layerParameterName;

	if (idx>=0)
	{
		//_layerParametersList->setItemData(idx,slp.layerParameterName);
		_slpList[idx]=slp;
		_layerParametersList->setCurrentIndex(idx);
	}
	else
	{
		_layerParametersList->addItem(slp.layerParameterName);
		_slpList.push_back(slp);
		_layerParametersList->setCurrentIndex(_layerParametersList->count()-1);
	}

}


void PaintOptions::setLayerParametersFromInterface(StrokeLayerProperties& slp)
{

	
	slp.maxStrokes=_maxStrokesSpin->value();
	slp.strokeProperties.textureFilename=_strokeListWidget->currentItem()->text();
	slp.strokeProperties.startRadius=_spinBoxStrokeStartWidth->value();
	slp.strokeProperties.endRadius=_spinBoxStrokeEndWidth->value();
	slp.strokeProperties.alpha=(((float)_spinBoxAlpha->value())/100.);
	slp.strokeProperties.useImageColor=_imgColor->isChecked();
	slp.strokeProperties.color=_col;
	slp.strokeProperties.isFill=_fillStroke->isChecked();

	slp.hueOffset=_hueOffsetSpinBox->value();
	slp.saturationOffset=_saturationOffsetSpinBox->value();
	slp.valueOffset=_valueOffsetSpinBox->value();
	slp.hueNoise=_hueNoiseSpinBox->value();
	slp.saturationNoise=_saturationNoiseSpinBox->value();
	slp.valueNoise=_valueNoiseSpinBox->value();
	slp.addVisibleStrokesToLayer=_addVisibleStrokesToLayer->isChecked();
	slp.fixedOrientationFactor=_fixedOrientationFactor->value();
	slp.boundaryFactor=_followBoundaryFactor->value();
	slp.imageGradientFactor=_followImageFactor->value();
	slp.userStrokesFactor=_followUserStrokesFactor->value();
	slp.orientationAngle=(360+(-1*_orientationDial->value()-90)%360) * (PI/180.0);
	slp.minStrokeLength=_strokeMinLengthSpin->value();
	slp.maxStrokeLength=_strokeMaxLengthSpin->value();
	slp.strokeDensity=_strokeDensitySpin->value()/1000.;
	slp.maxBrightness=_maxBrightnessSpin->value();
	slp.minBrightness=_minBrightnessSpin->value();
	slp.magnitudeThreshold=_magThresholdSpin->value();
	slp.colorDifference=_colorDifferenceSpin->value();
	slp.boundaryFlowFactor=_boundaryMovementFactor->value();
	slp.imageFlowFactor=_opticalFlowFactor->value();
	slp.userFlowFactor=_userStrokesMovementFactor->value();
	slp.fixedFlowFactor=_fixedMovementFactor->value();
	slp.fixedMovementAngle=(360+(-1*_movementDial->value()-90)%360) * (PI/180.0);
	slp.fixedMovementRate=_fixedMovementRateSpin->value();
	slp.minFrames=_minFramesSpin->value();
	slp.maxFrames=_maxFramesSpin->value();
	slp.colorUpdate=_colorUpdateSpinBox->value();
	slp.fadeRate=_fadeRateSpin->value();
	//slp.densityWidthAdd;
	//slp.densityBrightnessFactor;
	slp.followPreviousStrokesFactor=_followStrokesFactorSpin->value();
	slp.followGradientFactor=_followGradientFactorSpin->value();
	slp.maxFlowAngle=_maxFlowAngleSpin->value();
	slp.maxStrokeAngle=_maxStrokeAngleSpin->value();
	slp.optimizeStrokes=_optimizedRadio->isChecked();
	slp.orientationFactor=_orientationFactorSpin->value();
	slp.coherenceFactor=_coherenceFactorSpin->value();
	slp.rigidityFactor=_rigidityFactorSpin->value();
	slp.colorFactor=_colorFactorSpin->value();

	slp.densityFactor=_densityFactorSpin->value();
	slp.fillEmptyRegions=_fillEmptyRegions->isChecked();
	slp.fillColorDifference=_fillColorDifference->isChecked();
	slp.fillUserSeed=_fillUserSeed->isChecked();
	slp.paintOccludedRegions=_paintOccludedRegions->isChecked();
	slp.startInOccludedRegions=_startInOccludedRegions->isChecked();

	slp.useOccludingLayerFlow=_useOccludingLayerFlow->isChecked();
	slp.clipToSeedRegions = _clipToSeedRegions->isChecked();
	slp.shiftToEdges = _shiftToEdges->isChecked();
	slp.seedOrder =_seedOrder->itemText(_seedOrder->currentIndex());

	slp.optimizeStrokeWidths=_optimizeStrokeWidths->isChecked();
	slp.oswColorFactor=_oswColorFactorSpinBox->value();
	slp.oswEdgeFactor=_oswEdgeFactorSpinBox->value();
	slp.oswCoherenceFactor=_oswCoherenceFactorSpinBox->value();
	slp.oswRegularizationFactor=_oswRegularizationFactorSpinBox->value();
	slp.oswGrowFactor=_oswGrowFactorSpinBox->value();

}
	