#ifndef PAINTOPTIONS_H
#define PAINTOPTIONS_H


#include <QTabWidget>
#include <QWidget>
#include <QDial>
#include <QComboBox>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QRadioButton>
#include <QDoubleSpinBox>
#include <QSpinBox>
#include <QScrollArea>
#include <QCheckBox>
#include <QListWidget>
#include <QAbstractListModel>
#include "ImageViewer.h"
#include "StrokeProperties.h"
#include "StrokeLayerProperties.h"
#include "tri_logger.hpp"

#define PI 3.1415927

class PaintOptions : public QWidget
{
	Q_OBJECT

public:

	PaintOptions(QWidget *parent = 0, Qt::WFlags flags = 0);
	~PaintOptions();


protected:

signals:

	//stroke properties signals
	void loadTexture(QString& filename);
	void strokeStartWidthChanged(int startWidth);
	void strokeEndWidthChanged(int endWidth);


	void useImageColor(bool useImageColor);
	void useNewColor(QColor col);
	void alphaChanged(double alpha);
	void colorVarianceChanged(int strokeVariance);
	void hueOffsetChanged(int hueOffset);
	void hueNoiseChanged(double hueNoise);
	void saturationOffsetChanged(int saturationOffset);
	void saturationNoiseChanged(double saturationNoise);
	void valueOffsetChanged(int valueOffset);
	void valueNoiseChanged(double valueNoise);

	void guideStroke(bool isGuide);
	void visibleStroke(bool);
	void fillStroke(bool);
	void colorStroke(bool);
	void seedStroke(bool);
	void addVisibleStrokesToLayer(bool);
	void passOverGuideStrokes(bool);
	void darkenOccludedRegionsChanged(bool);
	void shiftToEdgesChanged(bool);
	void optimizeStrokesChanged(bool);
	void useStrokesFromAllLayersChanged(bool);

	//filling parameter signals
	void orientationChanged(int orientation);
	void includeReverseDirection(bool reverseDirection);
	void fixedOrientationFactorChanged(int fixedOrientationFactor);
	void followBoundaryFactorChanged(int boundaryFactor);
	void followImageFactorChanged(int imageGradientFactor);
	void followUserStrokesFactorChanged(int userStrokesFactor);
	void orientationVarianceChanged(double orientationVariance);
	void paintOccludedRegionsChanged(bool paintOccludedRegions);
	void startInOccludedRegionsChanged(bool startInOccludedRegions);
	void useOccludingLayerFlowChanged(bool useOccludingLayerFlow);
	void clipToSeedRegionsChanged(bool clipToSeedRegions);
	void fillEmptyRegionsChanged(bool fillEmptyRegions);
	void fillColorDifferenceChanged(bool fillColorDifference);
	void fillUserSeedChanged(bool fillUserSeed);
	void minStrokeLengthChanged(int minStrokeLength);
	void maxStrokeLengthChanged(int maxStrokeLength);
	void strokeDensityChanged(int strokeDensity);
	void densityWidthAddChanged(double densityWidthAdd);
	void densityBrightnessFactorChanged(double densityBrightnessFactor);

	void minBrightnessChanged(int minBrightness);
	void maxBrightnessChanged(int maxfiBrightness);

	void magnitudeThresholdChanged(int magnitudeThreshold);
	void colorDifferenceChanged(double colorDifference);
	void edgeThresholdChanged(double edgeThreshold);
	void maxStrokeAngleChanged(int maxStrokeAngle);

	void radiusFactorChanged(double radiusFactor);


	//motion parameter signals
	void fixedMovementFactorChanged(int fixedMovementFactor);
	void fixedMovementDirectionChanged(int fixedDirection);
	void fixedMovementRateChanged(int fixedMovementRate);
	void followBoundaryMovementFactorChanged(int boundaryMovementFactor);
	void followOpticalFlowFactorChanged(int opticalFlowFactor);
	void followUserStrokeMovementFactorChanged(int userStrokeMovementFactor);
	void movementVarianceChanged(double movementVariance);
	void scaleStrokesChanged(bool scaleStrokes);
	void minStrokeScaleChanged(int minStrokeScale);
	void maxStrokeScaleChanged(int maxStrokeScale);
	void fadeRateChanged(int fadeRate);
	void colorUpdateChanged(double colorUpdate);
	void followStrokesFactorChanged(double followStrokesFactor);
	void followGradientFactorChanged(double followGradientFactor);
	void followFlowFactorChanged(double followFlowFactor);


	void orientationFactorChanged(double colorFactor);
	void coherenceFactorChanged(double colorFactor);
	void colorFactorChanged(double colorFactor);
	void rigidityFactorChanged(double rigidityFactor);
	void densityFactorChanged(double densityFactor);


	void maxFlowAngleChanged(int maxFlowAngle);
	void maxStrokesChanged(int maxStrokes);
	void boundaryBlurChanged(double boundaryBlur);
	void medianBlurChanged(int medianBlur);

	//void systemStrokeTransparencyChanged(int strokeTransparency);
	//void userStrokeTransparencyChanged(int strokeTransparency);
	//void imageTransparencyChanged(int imageTransparency);

	//background
	void loadPaperTexture(QString& filename);
	void usePaperColor(QColor col);
	//void useImageColorForPaper(bool useImageColorForPaper);

	void minFramesChanged(int minFrames);
	void maxFramesChanged(int maxFrames);

	void seedOrderChanged( const QString &);

	void optimizeStrokeWidthsChanged(bool);
	void oswColorFactorChanged(double);
	void oswCoherenceFactorChanged(double);
	void oswRegularizationFactorChanged(double);
	void oswEdgeFactorChanged(double);
	void oswGrowFactorChanged(double);


	void saveFrames(bool saveFrames);
	void oldBlend(bool oldBlend);
	void multiPassRender(bool multiPassRender);
	void useBoundaryFlow(bool useBoundaryFlow);
	void useStrokeFlow(bool useStrokeFlow);

	void endHeavySampleChanged(int endHeavySample);
	void startHeavySampleChanged(int startHeavySample);




private slots:
	void loadTexture();
	void selectColor();
	void selectPaperTexture();
	void selectPaperColor();

	void newTextureSelected(QListWidgetItem * item);

	void setOrientationVariance(int orientationVariance);
	void setMovementVariance(int movementVariance);
	void setAlpha(int alpha);

	void updatePropertiesForSelectedStroke(StrokeProperties& prop);

	void setStrokeStartWidth(int startWidth);
	void setStrokeEndWidth(int endWidth);

	void increaseStrokeWidth();
	void decreaseStrokeWidth();

	void offsetNoiseWidget(bool);
	void strokeDeformWidget(bool);


	void layerParametersChanged(int slpIndex);
	void addCurrentParameters();
	void replaceSelectedParameters();
	void deleteSelectedParameters();
	void saveParameters();

	void showAdvancedFillingOptions(bool showAdvancedFillingOptions);
	void showAdvancedMotionOptions(bool showAdvancedMotionOptions);
	void showUserStrokeOptions(bool showUserStrokeOptions);
	void showTestingOptions(bool showTestingOptions);
private:
	QWidget *_offsetWidget;
	QWidget *_optimizedOptions;
	QWidget *_quickOptions;
	QRadioButton *_quickRadio;
    QRadioButton *_optimizedRadio;
	QWidget *_moreFillingOptionsWidget;
	QWidget *_moreMotionOptionsWidget;
	QPushButton *_loadTextureButton;
	QPushButton *_chooseColorButton, *_leftColorButton, *_rightColorButton, *_choosePaperColorButton;
	QColor _col, _papercol;
	QColor _leftcol;
	QColor _rightcol;
	QCheckBox *_guideStroke, *_colorStroke, *_imgColor, *_visibleStroke,*_fillStroke,*_seedStroke, *_addVisibleStrokesToLayer,*_passOverGuideStrokes, *_darkenOccludedRegions;


	QSpinBox *_spinBoxStrokeStartWidth, *_spinBoxStrokeEndWidth, *_spinBoxRightColorWidth, *_spinBoxLeftColorWidth,*_spinBoxAlpha;
	ImageViewer *_StrokeImageViewer, *_PaperImageViewer;
	QCheckBox *_linkWidths;

	QListWidget *_strokeListWidget;

	QSpinBox *_hueOffsetSpinBox,*_saturationOffsetSpinBox,*_valueOffsetSpinBox;
	QDoubleSpinBox *_hueNoiseSpinBox,*_saturationNoiseSpinBox,*_valueNoiseSpinBox;


	QSpinBox *_maxStrokesSpin;
	QDoubleSpinBox *_colorDifferenceSpin;
	QCheckBox *_fillEmptyRegions,*_fillColorDifference,*_fillUserSeed;

	QSpinBox *_fixedMovementFactor;
	QDial *_movementDial;
	QSpinBox *_fixedMovementRateSpin;
	QSpinBox *_boundaryMovementFactor;
	QSpinBox *_opticalFlowFactor;
	QSpinBox *_userStrokesMovementFactor;
	QDoubleSpinBox *_colorUpdateSpinBox;
	QSpinBox *_maxFramesSpin;
	QSpinBox *_minFramesSpin;
	QSpinBox *_fadeRateSpin;
	QSpinBox *_maxStrokeAngleSpin;
	QSpinBox *_maxFlowAngleSpin;
	QDoubleSpinBox *_followStrokesFactorSpin;
	QDoubleSpinBox *_followGradientFactorSpin;
	QDoubleSpinBox *_orientationFactorSpin;
	QDoubleSpinBox *_coherenceFactorSpin;
	QDoubleSpinBox *_colorFactorSpin;
	QDoubleSpinBox *_rigidityFactorSpin;
	QDoubleSpinBox *_densityFactorSpin;
	QSpinBox *_fixedOrientationFactor;
	QDial *_orientationDial;
	QSpinBox *_followBoundaryFactor;
	QSpinBox *_followImageFactor;
	QSpinBox *_followUserStrokesFactor;
	QSpinBox *_strokeMinLengthSpin;
	QSpinBox *_strokeMaxLengthSpin;
	QSpinBox *_strokeDensitySpin;
	QSpinBox *_minBrightnessSpin;
	QSpinBox *_maxBrightnessSpin;
	QSpinBox *_magThresholdSpin;
	QCheckBox *_paintOccludedRegions;
	QCheckBox *_startInOccludedRegions;
	QCheckBox *_useOccludingLayerFlow;
	QCheckBox *_clipToSeedRegions;
	QCheckBox *_shiftToEdges;
	QComboBox *_seedOrder;

	QList<StrokeLayerProperties> _slpList;
	QComboBox *_layerParametersList;

	QCheckBox* _optimizeStrokeWidths;
	QDoubleSpinBox* _oswColorFactorSpinBox;
	QDoubleSpinBox* _oswEdgeFactorSpinBox;
	QDoubleSpinBox* _oswRegularizationFactorSpinBox;
	QDoubleSpinBox* _oswGrowFactorSpinBox;
	QDoubleSpinBox* _oswCoherenceFactorSpinBox;


	QWidget* _advancedFillingOptionsWidget;
	QWidget* _advancedMotionOptionsWidget;
	QWidget* _userStrokeOptionsWidget;
	QWidget* _testingOptionsWidget;

	int _StrokeStartWidth,_StrokeEndWidth;

	QWidget* createFillingParametersWidget();
	QWidget* createStrokePropertiesWidget();
	QWidget* createMotionParametersWidget();
	QWidget* createBackgroundParametersWidget();

	QGroupBox* _strokeGroupBox;
	
	QWidget* _strokeProperties;
	QWidget* _fillingParameters;
	QWidget* _motionParameters;
	QWidget* _backgroundParameters;
	
	QTabWidget* _tabGroup;

	void createTabs();

	void setTexture(QString filename);
	void setPaperTexture(QString filename);


	void loadLayerParameters(StrokeLayerProperties& slp);
	void setLayerParametersFromFile(QComboBox* combo, QString filename);
	void setLayerParametersFromInterface(StrokeLayerProperties& slp);




};


#endif // PAINTOPTIONS_H
