#include <QtGui>
#include <QCheckBox>
#include <QDoubleSpinBox>
#include "ParameterDelegate.h"

ParameterDelegate::ParameterDelegate(QObject* parent) : QItemDelegate(parent)
{
}

ParameterDelegate::~ParameterDelegate(void)
{
}

//void ParameterDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
//{
//	if(index.column()==0)
//	{
//	}
//	else
//		QItemDelegate::paint(painter, option, index);
//}

// this function handles how to edit the date for each element
QWidget * ParameterDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	QCheckBox* checkBox;
	QDoubleSpinBox* doubleSpinBox;
	QComboBox* comboBox;
	switch(index.column()){
		case 0:
			checkBox=new QCheckBox(parent);
			return checkBox;
		case 1:
		case 2:
			doubleSpinBox=new QDoubleSpinBox(parent);
			doubleSpinBox->setRange(0.001,0.07);
			doubleSpinBox->setDecimals(3);
			doubleSpinBox->setSingleStep(0.001);
			return doubleSpinBox;
		case 3:
			doubleSpinBox=new QDoubleSpinBox(parent);
			doubleSpinBox->setRange(0.3,1);
			doubleSpinBox->setDecimals(2);
			doubleSpinBox->setSingleStep(0.01);
			return doubleSpinBox;
		case 4:
			comboBox=new QComboBox(parent);
			comboBox->addItem("membrane");
			comboBox->addItem("thinplate");
			comboBox->setEditable(false);
			return comboBox;
	}
}

// function to read the data from the model and display it in the viewer
void ParameterDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
	bool isChecked;
	double value;
	QString string;
	QCheckBox* checkBox;
	QDoubleSpinBox* doubleSpinBox;
	QComboBox* comboBox;
	switch(index.column()){
		case 0:
			isChecked=index.model()->data(index,Qt::DisplayRole).toBool();
			checkBox = static_cast<QCheckBox*>(editor);
			checkBox->setChecked(isChecked);
			break;
		case 1:
		case 2:
		case 3:
			value = index.model()->data(index,Qt::DisplayRole).toDouble();
			doubleSpinBox=static_cast<QDoubleSpinBox*>(editor);
			doubleSpinBox->setValue(value);
			//doubleSpinBox->setDecimals(3);
			break;
		case 4:
			string=index.model()->data(index,Qt::DisplayRole).toString();
			comboBox=static_cast<QComboBox*>(editor);
			if(string=="membrane")
				comboBox->setCurrentIndex(0);
			else
				comboBox->setCurrentIndex(1);
			break;
	}
}

// function to read the content of the editor and write it back to the model
void ParameterDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
	QCheckBox* checkBox;
	QDoubleSpinBox* doubleSpinBox;
	QComboBox* comboBox;
	switch(index.column()){
		case 0:
			checkBox = static_cast<QCheckBox*>(editor);
			model->setData(index,(int)checkBox->isChecked());
			break;
		case 1:
		case 2:
		case 3:
			doubleSpinBox=static_cast<QDoubleSpinBox*>(editor);
			doubleSpinBox->interpretText();
			model->setData(index,doubleSpinBox->value(),Qt::EditRole);
			break;
		case 4:
			comboBox=static_cast<QComboBox*>(editor);
			model->setData(index,comboBox->currentText(),Qt::EditRole);
			break;
	}
}

void ParameterDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	editor->setGeometry(option.rect);
}