#include "QVideoProcessing.h"
#include <QDir>
#include "Stochastic.h"

QVideoProcessing::QVideoProcessing(void)
{
}

QVideoProcessing::~QVideoProcessing(void)
{
}

QFileInfoList QVideoProcessing::findimagefiles(const QString& path)
{
    QDir dir;
	if (path.isEmpty()==false)
		dir.setPath(path);
	 dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
	 QStringList fileformat;
	 fileformat<<"*.bmp"<<"*.jpg"<<"*.png"<<"*.gif";
	 dir.setNameFilters(fileformat);
     dir.setSorting(QDir::Name);
	 

     QFileInfoList list = dir.entryInfoList();
	 return list;
}

QFileInfoList QVideoProcessing::findImageFilesSingleFormat(const QString& path,const QString& startFrame,const QString& endFrame)
{
	QDir dir;
	if (path.isEmpty()==false)
		dir.setPath(path);
	 dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
	 QFileInfoList list;

	 if (startFrame.length()==0)
	 {
		 QStringList fileformat;
		 fileformat<<"*.bmp"<<"*.jpg"<<"*.jpeg"<<"*.png"<<"*.gif";
		 
		 int numFiles[5]={0,0,0,0,0};
		 for (int i=0;i<5;i++)
		 {
			QStringList format(fileformat[i]);
			dir.setNameFilters(format);
			dir.setSorting(QDir::Name);
			list= dir.entryInfoList();
			numFiles[i]=list.size();
		 }

		 // find the format with the maximum number of files
		 int index=CStochastic::FindMax(5,numFiles);
		
		 QStringList format(fileformat[index]);
		 dir.setNameFilters(format);
		 dir.setSorting(QDir::Name);
		 list= dir.entryInfoList();
		 return list; 
	 }
	 else
	 {
		QString imgformat=QString("*")+startFrame.right(startFrame.size()-startFrame.lastIndexOf(QChar('.')));
		QStringList format(imgformat);
		dir.setNameFilters(format);
		dir.setSorting(QDir::Name);
		list= dir.entryInfoList();

		QFileInfoList filteredList;

		bool reading=false;
		for(int i=0;i < list.size();i++)
		{
			if (list[i].fileName()==startFrame)
				reading=true;
			if (list[i].fileName()==endFrame)
			{
				filteredList.push_back(list[i]);
				break;
			}

			if (reading)
				filteredList.push_back(list[i]);
		}
		return filteredList;
	 }


}

bool QVideoProcessing::FileListExist(const QString &pathName, const QStringList &fileList)
{
	QString folderName=pathName;
	if(folderName.right(1)!="/")
		folderName+="/";
	for(int i=0;i<fileList.size();i++)
	{
		QFileInfo fileInfo(folderName+fileList[i]);
		if(fileInfo.exists()==false)
			return false;
	}
	return true;
}

void QVideoProcessing::separatePath_FileNames(const QString& fullname,QString& pathname,QString& filename)
{
		QString FullName(fullname);
		if(FullName.right(1)=="/")
			FullName.chop(1);
		// separate file name and path name
		QStringList strlist=fullname.split("/");
		filename=strlist.last();
		pathname.clear();
		for (int j=0;j<strlist.size()-1;j++)
		{
			pathname+=strlist[j];
			if(j<strlist.size()-2)
				pathname+="/";
		}
}

bool QVideoProcessing::writeString(QFile &file, const QString &string)
{
	QByteArray byteArray=string.toAscii();
	int nBytes=byteArray.size();
	
	char* pBuffer;
	pBuffer=new char[nBytes];
	for(int i=0;i<nBytes;i++)
		pBuffer[i]=byteArray[i];

	file.write((char *)&nBytes,sizeof(int));
	if(file.write(pBuffer,nBytes)!=nBytes)
	{
		delete pBuffer;
		return false;
	}

	delete pBuffer;
	return true;
}

bool QVideoProcessing::readString(QFile &file,QString& string)
{
	int nBytes;
	file.read((char *)&nBytes,sizeof(int));
	if(nBytes<0 || nBytes>10000000)
		return false;
	char* pBuffer;
	pBuffer=new char[nBytes+1];
	if(file.read(pBuffer,nBytes)!=nBytes)
	{
		delete pBuffer;
		return false;
	}
	pBuffer[nBytes]='\0';
	string=QString(pBuffer);
	return true;
}

bool QVideoProcessing::writeQRectF(QFile &file, const QRectF &rectF)
{
	double left=rectF.left();
	double top=rectF.top();
	double right=rectF.right();
	double bottom=rectF.bottom();
	if(file.write((char *)&left,sizeof(double))!=sizeof(double))
		return false;
	if(file.write((char *)&top,sizeof(double))!=sizeof(double))
		return false;
	if(file.write((char *)&right,sizeof(double))!=sizeof(double))
		return false;
	if(file.write((char *)&bottom,sizeof(double))!=sizeof(double))
		return false;
	return true;
}

bool QVideoProcessing::readQRectF(QFile &file, QRectF &rectF)
{
	double left,top,right,bottom;
	if(file.read((char *)&left,sizeof(double))!=sizeof(double))
		return false;
	if(file.read((char *)&top,sizeof(double))!=sizeof(double))
		return false;
	if(file.read((char *)&right,sizeof(double))!=sizeof(double))
		return false;
	if(file.read((char *)&bottom,sizeof(double))!=sizeof(double))
		return false;
	rectF=QRectF(left,top,right-left+1,bottom-top+1);
	return true;
}

bool QVideoProcessing::writeQRect(QFile &file, const QRect &rect)
{
	int left=rect.left();
	int top=rect.top();
	int right=rect.right();
	int bottom=rect.bottom();
	if(file.write((char *)&left,sizeof(int))!=sizeof(int))
		return false;
	if(file.write((char *)&top,sizeof(int))!=sizeof(int))
		return false;
	if(file.write((char *)&right,sizeof(int))!=sizeof(int))
		return false;
	if(file.write((char *)&bottom,sizeof(int))!=sizeof(int))
		return false;
	return true;
}

bool QVideoProcessing::readQRect(QFile &file, QRect &rect)
{
	int left,top,right,bottom;
	if(file.read((char *)&left,sizeof(int))!=sizeof(int))
		return false;
	if(file.read((char *)&top,sizeof(int))!=sizeof(int))
		return false;
	if(file.read((char *)&right,sizeof(int))!=sizeof(int))
		return false;
	if(file.read((char *)&bottom,sizeof(int))!=sizeof(int))
		return false;
	rect=QRect(left,top,right-left+1,bottom-top+1);
	return true;
}

