#ifndef _QVideoProcessing_h
#define _QVideoProcessing_h

#include <QFileInfo>
#include <QFile>
#include <QList>
#include <QVector>
#include <QRectF>
class QString;

class QVideoProcessing
{
public:
	QVideoProcessing(void);
public:
	~QVideoProcessing(void);
	static QFileInfoList findimagefiles(const QString& path);
	static QFileInfoList findImageFilesSingleFormat(const QString& path,const QString& startFrame,const QString& endFrame);
	static bool FileListExist(const QString& pathName,const QStringList& fileList);
	static void separatePath_FileNames(const QString& fullname,QString& pathname,QString& filename);

	template <class T>
	static bool writeQListSegment(QFile& file,const QList<T>& list,int start=-1,int end=-1);
	template <class T>
	static bool readQListSegment(QFile& file,QList<T>& list,int start=-1,int end=-1);

	template <class T>
	static bool writeQVector(QFile& file,const QVector<T>& vector);
	template <class T>
	static bool readQVector(QFile& file,QVector<T>& vector);

	template <class T>
	static bool writeQList(QFile& file,const QList<T>& list);
	template <class T>
	static bool readQList(QFile& file,QList<T>& list);


	static bool writeString(QFile& file,const QString& string);
	static bool readString(QFile& file,QString& string);

	static bool writeQRect(QFile& file,const QRect& rect);
	static bool readQRect(QFile& file,QRect& rect);

	static bool writeQRectF(QFile& file,const QRectF& rectF);
	static bool readQRectF(QFile& file,QRectF& rectF);
};

template <class T>
bool QVideoProcessing::writeQListSegment(QFile& file,const QList<T> &list, int start, int end)
{
	if(start==-1)
		start=0;
	if(end==-1)
		end=list.size()-1;
	if(start>end || end>list.size()-1) // the end cannot be bigger than the length of the list, and the order cannot be reversed
		return false;

	T* pBuffer;
	pBuffer=new T[end-start+1];
	for(int i=start;i<=end;i++)
		pBuffer[i-start]=list[i];
	
	if(file.write((char *)pBuffer,sizeof(T)*(end-start+1))!=sizeof(T)*(end-start+1))
	{
		delete []pBuffer;
		return false;
	}

	delete []pBuffer;
	return true;
}

template <class T>
bool QVideoProcessing::readQListSegment(QFile& file,QList<T>& list,int start,int end)
{
	if(start==-1)
		start=0;
	if(end==-1)
		end=list.size()-1;
	if(start>end || end>list.size()-1) // the end cannot be bigger than the length of the list, and the order cannot be reversed
		return false;

	T* pBuffer;
	pBuffer=new T[end-start+1];
	if(file.read((char *)pBuffer,sizeof(T)*(end-start+1))!=sizeof(T)*(end-start+1))
	{
		delete []pBuffer;
		return false;
	}
	for(int i=start;i<=end;i++)
		list[i]=pBuffer[i-start];

	delete []pBuffer;
	return true;
}

template <class T>
bool QVideoProcessing::writeQVector(QFile& file,const QVector<T>& vector)
{
	int num=vector.size();
	file.write((char *)&num,sizeof(int));
	if(num==0)
		return true;
	T* pBuffer=new T[num];
	for(int i=0;i<num;i++)
		pBuffer[i]=vector[i];
	if(file.write((char *)pBuffer,sizeof(T)*num)!=sizeof(T)*num)
	{
		delete pBuffer;
		return false;
	}
	delete pBuffer;
	return true;
}

template <class T>
bool QVideoProcessing::readQVector(QFile& file,QVector<T>& vector)
{
	int num;
	file.read((char *)&num,sizeof(int));
	vector.clear();
	if(num==0)
		return true;
	vector.resize(num);
	T* pBuffer=new T[num];
	if(file.read((char *)pBuffer,sizeof(T)*num)!=sizeof(T)*num)
	{
		delete pBuffer;
		return false;
	}
	for(int i=0;i<num;i++)
		vector[i]=pBuffer[i];
	delete pBuffer;
	return true;
}

template <class T>
bool QVideoProcessing::writeQList(QFile &file, const QList<T> &list)
{
	return writeQVector(file,QVector<T>::fromList(list));
}

template <class T>
bool QVideoProcessing::readQList(QFile& file,QList<T>& list)
{
	QVector<T> vector;
	if(!readQVector(file,vector))
		return false;
	list=QList<T>::fromVector(vector);
	return true;
}


#endif