//Aaron Hertzmann's stroke code modified by Peter O'Donovan

#include "Stroke.h"

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <sstream>
#include <iomanip>

using std::stringstream;
using std::cout;
using std::cerr;
using std::endl;
using std::ends;


#ifdef SCAN_CONVERT
#include "polyfill.h"
#endif

/*
#ifndef SCAN_CONVERT
GLUquadricObj * Stroke::qobj = NULL;
#endif
*/



Stroke::Stroke()
{
  limit = temp = NULL;
  computed = GL_FALSE;
  //curveType = CUBIC_BSPLINE;

  useTexture = true;

  //numLevels = 3;

  z = 0;
/*
#ifndef SCAN_CONVERT
  if (qobj == NULL)
    {
      qobj = gluNewQuadric();
    }
#endif
	*/

  //UFREQ = .005;
  //VFREQ = .5;
  //USTART = 0;
  //VSTART = 0.5;
  filledRegionList =0;
  nextStroke=0;
  ageChecked=false;
  age=1;
  dying=false;
}


Stroke::~Stroke()
{
  if (limit != NULL)
  {
    delete limit;
	limit = NULL;
  }

  if (temp != NULL)
  {
    delete temp;
	temp = NULL;
  }

  if (filledRegionList>0)
  {
	glDeleteLists(filledRegionList,1);
	filledRegionList =0;
  }

  computed = GL_FALSE;
}


Stroke::Stroke(const Stroke& stroke)
{

	properties=stroke.properties;
	control = stroke.control;
	//pressure = stroke.pressure;
	useTexture = stroke.useTexture;
	limit = NULL;
	temp = NULL;
	computed = GL_FALSE;
	useTexture = stroke.useTexture;
	filledRegionList = 0;
	z = stroke.z;
	nextStroke=stroke.nextStroke;
	ageChecked=stroke.ageChecked;
	age=stroke.age;
}

Stroke& Stroke::operator = (const Stroke& stroke)
{

	properties=stroke.properties;
	control = stroke.control;
	//pressure = stroke.pressure;
	useTexture = stroke.useTexture;
	computed = GL_FALSE;
	limit = NULL;
	temp = NULL;
	filledRegionList = 0;
	z = stroke.z;
	nextStroke=stroke.nextStroke;
	ageChecked=stroke.ageChecked;
	age=stroke.age;
	return *this;
}

bool Stroke::writeStroke(QFile& file) const
{
	if (properties.writeStrokeProperties(file) ==false)
		return false;

	//printf("write stroke\n");

	int num_Points=control.size();
	file.write((char *)&num_Points,sizeof(int));
	double x,y;
	// write the list
	for(int i=0;i<num_Points;i++)
	{
		x=control[i].x();
		y=control[i].y();
		file.write((char*)&x,sizeof(double));
		file.write((char*)&y,sizeof(double));
	}
	return true;

}
bool Stroke::readStroke(QFile& file)
{
	if (properties.readStrokeProperties(file) ==false)
		return false;

	int num_Points;
	if(file.read((char *)&num_Points,sizeof(int))!=sizeof(int))
		return false;
	double x,y;
	for(int i=0;i<num_Points;i++)
	{
		file.read((char *)&x,sizeof(double));
		file.read((char *)&y,sizeof(double));
		if ((x<-1000) || (y<-1000) || (x>1000) || (y>1000))
			return false;
		add(x,y);
	}

	computed = GL_FALSE;
	limit = NULL;
	temp = NULL;

	return true;
}


void Stroke::add(float x,float y,float press)
{
 // if (control.empty() || (control.back().x() != x && control.back().y() != y))
  //  {
	 // printf("added %f %f with pressure %f\n", x, y, press);
      control.push_back(Point(x,y));
	 // pressure.push_back(press);
      computed = GL_FALSE;
  //  }
}

void Stroke::clear()
{
  computed = GL_FALSE;
  //control.erase(control.begin(),control.end());
  control.clear();
  //pressure.clear();

  cleanup();

  //limit->erase(control.begin(),control.end());
  //USTART = 0;
  //VSTART = 0.5;
}


void Stroke::cleanup()
{
  if (limit != NULL)
  {
    delete limit;
	limit = NULL;
  }

  if (temp != NULL)
  {
    delete temp;
	temp = NULL;
  }

  if (filledRegionList>0)
  {
	glDeleteLists(filledRegionList,1);
	filledRegionList =0;
  }

  computed = GL_FALSE;
  nextStroke=0;
}


void Stroke::drawLines(QVector<Point> *curve)
{
  glDisable(GL_TEXTURE_2D);
  glBegin(GL_LINE_STRIP);
  for(int i=0;i<curve->size();i++)
    {
      Point & p = (*curve)[i];
      glVertex2i(p.x(),p.y());
    }
  glEnd();
}

void Stroke::forceRecompute()
{
  //printf("recomputing\n");
  computed = false;

}

void Stroke::discPoint(float x,float y,float brushRadius)
{
	/*
#ifdef SCAN_CONVERT
  // really ought to implement scan-conversion of the circle here
  
  glColor3ub(0,0,255);
  drawCap(Point(x,y),1,0,0,0);
  drawCap(Point(x,y),-1,0,0,0);
#else
  glPushMatrix();
  glTranslatef(x,y,z);
  gluDisk(qobj,0,brushRadius,NUM_SLICES,1);
  glPopMatrix();
#endif
  */
}

void Stroke::drawCap(const Point & p0, float dx, float dy,
		     float texU,float texV)
{
#ifdef SCAN_CONVERT
  glDisable(GL_TEXTURE_2D);

  float theta = atan2(-dy,-dx);

  Point p1(p0.x() - radius * dx, p0.y() - radius*dy);

  for(int i=1;i<NUM_SLICES-1;i++)
    {
      float dx1 = cos(theta + i * M_PI / NUM_SLICES);
      float dy1 = sin(theta + i * M_PI / NUM_SLICES);

      Point p2(p0.x() + radius * dx1, p0.y() + radius*dy1);

      drawTriangle(p0,p1,p2);

      p1 = p2;
    }

  drawTriangle(p0,p1,Point(p0.x()+radius*dx,p0.y()+radius*dy));
#else
  float theta = atan2(-dy,-dx);

  glPushMatrix();
  glTranslatef(p0.x(),p0.y(),z);

  glBegin(GL_TRIANGLE_FAN);
  //        glVertex3f(p0->x, p0->y,z);
  glTexCoord2f(texU,texV);
  glVertex3i(0,0,0);


  glTexCoord2f(texU+UFREQ*properties.startRadius * dx,texV+VFREQ*dy);
  glVertex3f(-properties.startRadius*dx,-properties.startRadius*dy,0);

  for(int i=1;i<=NUM_SLICES-1;i++)
    {
      float dx1 = cos(theta + i * 3.14159265 / NUM_SLICES);
      float dy1 = sin(theta + i * 3.14159265 / NUM_SLICES);

      // these 
      glTexCoord2f(texU-UFREQ*properties.startRadius * dx1,texV-VFREQ*dy1);

      glVertex3i(properties.startRadius * dx1, properties.startRadius * dy1, 0);
      //glVertex3f(p0->x + radius * dx1, p0->y + radius * dy1, z);
    }

  glTexCoord2f(texU-UFREQ*properties.startRadius * dx,texV-VFREQ*dy);
  glVertex3f(properties.startRadius*dx,properties.startRadius*dy,0);

  glEnd();

  glPopMatrix();
#endif
}

void Stroke::drawThickCurve(QVector<Point> * curve,bool cap)
{

 //printf("size of control %i, curve %i\n", control.size(), curve->size());
  
/*
  if (useTexture)
  {
    //glEnable(GL_TEXTURE_2D);
	//glPushAttrib(GL_TEXTURE_BIT);
  }

  else
    glDisable(GL_TEXTURE_2D);
	*/
	
  glBindTexture(GL_TEXTURE_2D,properties.texture);


  float radius;
  int i=0;
  float dx,dy,mag;
  Point p0;
  Point p1;
  Point p2;

  if (curve->empty())
    return;

  p0 = (*curve)[0];


  if (curve->size() == 1)
    {
   //   if (cap)
	//discPoint(p0.x(),p0.y(),radius);
	  if (useTexture)
	  {

		//glPopAttrib();
	  }
      return;
    }

  p1 = (*curve)[1];

  dx = p1.y() - p0.y();
  dy = p0.x() - p1.x();

  mag = sqrt(dx*dx + dy*dy);

  dx /= mag;
  dy /= mag;

  float textureU = USTART;
  float textureV = VSTART;

  float distTotal = 0;

  for(int i=1;i<curve->size()-1;i++)
    {
      Point point0 = (*curve)[i-1];
      Point point1 = (*curve)[i];

      float dist = sqrt((point1.x()-point0.x())*(point1.x()-point0.x())+(point1.y()-point0.y())*(point1.y()-point0.y()));
      distTotal += dist;
  }


  //printf("VSTART %f\n", VSTART);

 // glColor3ub(0,0,255);

  if (cap)
    drawCap(p0, dx, dy,textureU,textureV);

  /*
#ifdef SCAN_CONVERT
  glDisable(GL_TEXTURE_2D);

  Point v0(p0.x() + radius * dx, p0.y() + radius * dy);
  Point v1(p0.x() - radius * dx, p0.y() - radius * dy);

#else
  */
  glBegin(GL_TRIANGLE_STRIP);

 //printf("%f %f  + %f\n",textureU,textureV, VFREQ);
  glTexCoord2f(textureU,textureV + VFREQ);



  radius = getPointRadius(0,1,curve);

  glVertex3f(p0.x() + radius * dx, p0.y() + radius * dy,z);
  
 //printf("%f %f  - %f\n",textureU,textureV, VFREQ);
  glTexCoord2f(textureU,textureV - VFREQ);
  glVertex3f(p0.x() - radius * dx, p0.y() - radius * dy,z);
//#endif


#ifdef SCAN_CONVERT
  // draw the patch in between with extra subdivision to match the caps,
  // to prevent holes from appearing in the stroke
  if (curve->size() >= 2)
    {
      float dist = sqrt((p1.x()-p0.x())*(p1.x()-p0.x())+(p1.y()-p0.y())*(p1.y()-p0.y()));
      textureU += UFREQ * dist;
	  radius = (1.0-textureU)*properties.startRadius + (textureU)*properties.endRadius);
      v0 = Point(p0.x() + radius * dx, p0.y() + radius * dy);
      v1 = Point(p0.x() - radius * dx, p0.y() - radius * dy);
      Point v2(p1.x() + radius * dx, p1.y() + radius * dy);
      Point v3(p1.x() - radius * dx, p1.y() - radius * dy);
	  
      glColor3ub(255,0,0);
      //      drawTriangle(v2,v3,v0);
      drawTriangle(v2,p1,v0);
      glColor3ub(255,255,0);
      drawTriangle(p1,v3,v0);

      glColor3ub(0,255,0);
      //      drawTriangle(v0,v1,v3);
      drawTriangle(v0,p0,v3);
      glColor3ub(0,255,255);
      drawTriangle(p0,v1,v3);

      glColor3ub(0,0,255);

      if (curve->size() == 2)
	{
	  drawCap(p1,-dx,-dy,textureU,textureV);
	  return;
	}


      v0 = v2;
      v1 = v3;
    }
#endif

#ifdef SCAN_CONVERT
  for(i=2;i<curve->size()-1;i++)
#else
  for(i=1;i<curve->size()-1;i++)
#endif
    {
      p0 = (*curve)[i-1];
      p1 = (*curve)[i];
      p2 = (*curve)[i+1];

      dx = p2.y() - p0.y();
      dy = p0.x() - p2.x();

      mag = sqrt(dx*dx + dy*dy);

      dx /= mag;
      dy /= mag;

      float dist = sqrt((p1.x()-p0.x())*(p1.x()-p0.x())+(p1.y()-p0.y())*(p1.y()-p0.y()));
      textureU += dist/distTotal;

#ifdef SCAN_CONVERT
      Point v2(p1.x() + radius * dx, p1.y() + radius * dy);
      Point v3(p1.x() - radius * dx, p1.y() - radius * dy);

      glColor3ub(255,0,0);
      drawTriangle(v2,v3,v0);

      glColor3ub(0,255,0);
      drawTriangle(v0,v1,v3);
	  
      v0 = v2;
      v1 = v3;
#else
	  radius = getPointRadius(textureU,i,curve);
      glTexCoord2f(textureU,textureV + VFREQ);
      glVertex3f(p1.x() + radius * dx, p1.y() + radius * dy,z);
	  //printf("%f %f  + %f\n",textureU,textureV, VFREQ);
      glTexCoord2f(textureU,textureV - VFREQ);
      glVertex3f(p1.x() - radius * dx, p1.y() - radius * dy,z);
	  //printf("%f %f  - %f\n",textureU,textureV, VFREQ);
#endif

    }

  p0 = (*curve)[curve->size()-2];
  p1 = (*curve)[curve->size()-1];
    
  dx = p1.y() - p0.y();
  dy = p0.x() - p1.x();
    
  mag = sqrt(dx*dx + dy*dy);

  dx /= mag;
  dy /= mag;
      
  textureU +=  mag/distTotal;
  
#ifdef SCAN_CONVERT
  Point v2(p1.x() + radius * dx, p1.y() + radius * dy);
  Point v3(p1.x() - radius * dx, p1.y() - radius * dy);
      
  glColor3ub(255,0,0);
  //  drawTriangle(v2,v3,v0);
  drawTriangle(v2,p1,v0);
  glColor3ub(255,255,0);
  drawTriangle(p1,v3,v0);

  glColor3ub(0,255,0);
    drawTriangle(v0,v3,v1);
    //  drawTriangle(v0,p0,v3);
    //  glColor3ub(0,255,255);
    //  drawTriangle(p0,v1,v3);

#else
  radius = getPointRadius(textureU,i,curve);
  glTexCoord2f(textureU,textureV + VFREQ);
  glVertex3f(p1.x() + radius * dx, p1.y() + radius * dy,z);
  glTexCoord2f(textureU,textureV - VFREQ);
  glVertex3f(p1.x() - radius * dx, p1.y() - radius * dy,z);
      
  glEnd();
#endif



  //glColor3ub(0,0,255);

  if (cap)
    {
      drawCap(p1, -dx, -dy,textureU,textureV);
    }

  if (useTexture)
  {
	//glPopAttrib();
  }
}

float Stroke::getPointRadius(float textureU, int curvePoint,QVector<Point>* curve)
{
  float radius;

  //printf("curve size %i, point %i, pressure size %i\n", curve->size(), curvePoint,pressure.size());
  float frac = (((float) curvePoint)/ (float)curve->size());

  float pos = ((float)control.size())*frac;
  int ipos = pos;

  float rem = pos - ((float)ipos);

 

  float press=1;

  /*
  press = pressure[0];
  if (ipos >= pressure.size()-1)
	press = pressure[ipos];
  else
	press = rem*pressure[ipos] + (1.0-rem)*pressure[ipos+1];
	*/

 // printf("pos %f, ipos %i, rem %f, press %f\n", pos, ipos, rem,press);

  radius = press*properties.startRadius;

  radius = (1.0-textureU)*properties.startRadius + (textureU)*properties.endRadius;

  return radius;
}


void Stroke::drawControl()
{
  glColor3ub(255,0,0);
  drawLines(&control);
}

void Stroke::drawLineCurve()
{
  drawLines(limit);
}

QVector<Point>*  Stroke::getLimitCurve()
{
  if (!computed)
  {
      computeLimitCurve();
      computed = GL_TRUE;
  }
  return limit;
}

void Stroke::render()
{



  if (control.size() == 0)
	  return;

  if (!computed)
  {
     computeLimitCurve();
     computed = GL_TRUE;


	// printf("rendering but not computed\n");
	 if (properties.isFill)
	 {
		tesselate(limit);
	 }
  }

  if (properties.isFill)
  {
		renderFilled();
  }
  else
  {
       float r= ((float)properties.color.red())/255.;
       float g= ((float)properties.color.green())/255.;
       float b= ((float)properties.color.blue())/255.;

     if (r > 1)
		 r=1;
	 if (g > 1)
		 g=1;
	 if (b > 1)
		 b=1;
     if (r <0)
		 r=0;
	 if (g <0)
		 g=0;
	 if (b <0)
		 b=0;


	 glColor4f(r,g,b,properties.alpha);
  
	 drawThickCurve(limit,false);

  }
}

void Stroke::subdivideCubicBSpline(QVector<Point> * inputCurve,
                                   QVector<Point> * outputCurve)
{
  outputCurve->erase(outputCurve->begin(),outputCurve->end());

  //    printf("ic-count=%d\n",inputCurve->count);

  if (inputCurve->size() < 1)
    return;

  Point pi0;
  Point pi1;
  Point pi2;

  pi0 = (*inputCurve)[0];

  outputCurve->push_back(Point(pi0.x(),pi0.y()));

  if (inputCurve->size() == 1)
    return;

  if (inputCurve->size() == 2)
    {
      pi1 = (*inputCurve)[1];

      outputCurve->push_back(Point(pi1.x(),pi1.y()));

      return;
    }

  pi1 = (*inputCurve)[1];

  outputCurve->push_back(Point((pi0.x() + pi1.x())/2,(pi0.y() + pi1.y())/2));

  for(int i=1;i<inputCurve->size()-1;i++)
    {
      pi0 = (*inputCurve)[i-1];
      pi1 = (*inputCurve)[i];
      pi2 = (*inputCurve)[i+1];

      outputCurve->push_back(Point( (pi0.x() + 6*pi1.x() + pi2.x())/8,
				    (pi0.y() + 6*pi1.y() + pi2.y())/8));

      outputCurve->push_back(Point( (pi1.x() + pi2.x())/2,(pi1.y() + pi2.y())/2));
    }
	
  outputCurve->push_back(Point(pi2.x(),pi2.y()));
}

void Stroke::subdivideFourPoint(QVector<Point> * inputCurve,
                                QVector<Point> * outputCurve)
{
  outputCurve->erase(outputCurve->begin(),outputCurve->end());

  if (inputCurve->size() < 1)
    return;

  Point pi0;
  Point pi1;
  Point pi2;
  Point pi3;

  if (inputCurve->size() == 1)
    {
      pi0 = (*inputCurve)[0];
      outputCurve->push_back(Point(pi0.x(),pi0.y()));

      return;
    }

  if (inputCurve->size() == 2)
    {
      pi0 = (*inputCurve)[0];
      pi1 = (*inputCurve)[1];
	
      outputCurve->push_back(Point(pi0.x(),pi0.y()));
      outputCurve->push_back(Point((pi0.x()+pi1.x())/2,(pi0.y()+pi1.y())/2));
      outputCurve->push_back(Point(pi1.x(),pi1.y()));
	
      return;
    }

  pi0 = (*inputCurve)[0];
  pi1 = (*inputCurve)[1];

  Point piminus1(2*pi0.x() - pi1.x(),2*pi0.y() - pi1.y());

  pi0 = (*inputCurve)[inputCurve->size()-1];
  pi1 = (*inputCurve)[inputCurve->size()-2];

  Point piplus1(2*pi0.x() - pi1.x(),2*pi0.y() - pi1.y());
    
  for(int i=0;i<inputCurve->size()-1;i++)
    {
      pi0 = (i==0 ? piminus1 : (*inputCurve)[i-1]);
      pi1 = (*inputCurve)[i];
      pi2 = (*inputCurve)[i+1];
      pi3 = (i==inputCurve->size()-2? piplus1:(*inputCurve)[i+2]);

      outputCurve->push_back(Point( pi1.x(), pi1.y()));

      outputCurve->push_back(Point( (-pi0.x() + 9*pi1.x() + 9*pi2.x() - pi3.x())/16,
				    (-pi0.y() + 9*pi1.y() + 9*pi2.y() - pi3.y())/16));
    }

  pi0 = inputCurve->back();

  outputCurve->push_back(Point(pi0.x(),pi0.y()));
}

void Stroke::subdivide(QVector<Point> * inputCurve, QVector<Point> * outputCurve)
{
	 subdivideCubicBSpline(inputCurve,outputCurve);
	/*
  switch(curveType)
    {
    case CUBIC_BSPLINE:
      subdivideCubicBSpline(inputCurve,outputCurve);
      break;

    case FOUR_POINT:
      subdivideFourPoint(inputCurve,outputCurve);
      break;

    default:
      printf( "Illegal subdivision scheme selected\n");
      exit(-1);
    }
	*/
}

void Stroke::computeLimitCurve()
{
  //    printf("Computing limit curve.  Input length = %d\n",control.count);

  if (limit == NULL)
    limit = new QVector<Point>();

  if (temp == NULL)
    temp = new QVector<Point>();

  QVector<Point> control2 = control;

  if (properties.isFill)
  {
	if (control.size() <1)
	  return;

	Point p1 = control[0];
	Point p2 = control[control.size()-1];

	if (sqrt((p1.x()-p2.x())*(p1.x()-p2.x()) + (p1.y()-p2.y())*(p1.y()-p2.y())) > 3)
	{
		//control2.push_back(Point(p1.x()-1,p1.y()-1));
	}
  }


  subdivide(&control2,limit);

  //    limit->print();
  //    printf(" count = %d\n",limit->size());

  for(int i=0;i<NUMLEVELS/2;i++)
    {
      subdivide(limit,temp);
      subdivide(temp,limit);
    }
}





void Stroke::renderFilled()
{

    if (!computed)
    {
		computeLimitCurve();
		computed = GL_TRUE;
    }

	if (filledRegionList==0)
		tesselate(limit);

	if (filledRegionList>0)
	{

		glDisable(GL_TEXTURE_2D);
		//printf("%f %f %f %f\n", ((float)properties.color.red())/255.,((float)properties.color.green())/255.,((float)properties.color.blue())/255.,properties.alpha);
		glColor4f(((float)properties.color.red())/255.,((float)properties.color.green())/255.,((float)properties.color.blue())/255.,properties.alpha);
		glCallList(filledRegionList);
		glEnable(GL_TEXTURE_2D);
	}

}

static void CALLBACK tessErr(GLenum errno) {
  printf("Tesselation Error\n");
  const GLubyte *errString = gluErrorString(errno);
  printf("tessErr: OpenGL error: %s\n",errString);
  assert(0);
}

static void CALLBACK tessCombine(GLdouble coords[3], GLdouble* vd[4], GLfloat weight[4], void **outData) {
  GLdouble *vertex = (GLdouble *) malloc(3*sizeof(GLdouble));
  memcpy(vertex, coords, 3*sizeof(GLdouble));
  *outData = vertex;

 // printf("combine vertex data\n");
  //for (int i=0; i<4; i++)
  //  printf("%f %f %f\n",vd[i][0], vd[i][1], vd[i][2]);
}

/*
//void __stdcall tessEndCallback(void)
static void CALLBACK tessEndCallback(void)
{
	glEnd();
}

//void __stdcall tessBeginCallback(void)
static void CALLBACK tessBeginCallback(void)
{
	glBegin(GL_POLYGON);
}
*/

static void CALLBACK
begin(GLenum type)
{
 // assert(type == GL_LINE_LOOP);
  glBegin(type);
}

static void CALLBACK
vertex(void *data)
{
  GLdouble *v = (GLdouble *) data;
  glVertex3dv(v);
}

static void CALLBACK
end(void)
{
  glEnd();
}

void Stroke::tesselate(QVector<Point> * curve)
{

	if (filledRegionList>0)
	{
		glDeleteLists(filledRegionList,1);
		filledRegionList =0;
	}

	filledRegionList = glGenLists(1);  
    if(!filledRegionList) return;    

	//printf("created new list %i\n", filledRegionList);

    GLUtesselator *tess = gluNewTess();
	if(!tess)  return;

	GLdouble *v = new GLdouble[curve->size()*3];
	for(int i=0;i<curve->size();i++)
	{
		Point & p = (*curve)[i];
		v[i*3] = p.x(); 
		v[i*3+1] = p.y(); 
		v[i*3+2] = 0;
	}

    //gluTessCallback(tess, GLU_TESS_BEGIN, (GLvoid (__stdcall *) ( )) glBegin);
    //gluTessCallback(tess, GLU_TESS_VERTEX, (GLvoid (__stdcall *) ( )) glVertex3dv);
    //gluTessCallback(tess, GLU_TESS_END, (GLvoid (__stdcall *) ( )) glEnd);
	//gluTessCallback(tess, GLU_TESS_ERROR, (void (__stdcall *)(void))  &tessErr);
	//gluTessCallback(tess, GLU_TESS_COMBINE, (void (__stdcall *)(void))  &tessCombine);

    gluTessCallback(tess, GLU_TESS_BEGIN, (void (CALLBACK*)())begin);
    gluTessCallback(tess, GLU_TESS_VERTEX, (void (CALLBACK*)())vertex);
    gluTessCallback(tess, GLU_TESS_END, (void (CALLBACK*)())end);
        //gluTessCallback(tess, GLU_TESS_ERROR, (void (__stdcall *)(void))  &tessErr);
        //gluTessCallback(tess, GLU_TESS_COMBINE, (void (__stdcall *)(void))  &tessCombine);
   gluTessCallback(tess, GLU_TESS_ERROR, (void (CALLBACK*)())tessErr);
   gluTessCallback(tess, GLU_TESS_COMBINE,  (void (CALLBACK*)())tessCombine);

	int windingRule =2;
    switch (windingRule) {
            case 1 : gluTessProperty(tess, GLU_TESS_WINDING_RULE,GLU_TESS_WINDING_ODD); break;
            case 2 : gluTessProperty(tess, GLU_TESS_WINDING_RULE,GLU_TESS_WINDING_NONZERO); break;
            case 3 : gluTessProperty(tess, GLU_TESS_WINDING_RULE,GLU_TESS_WINDING_POSITIVE); break;
            case 4 : gluTessProperty(tess, GLU_TESS_WINDING_RULE,GLU_TESS_WINDING_NEGATIVE); break;
            case 5 : gluTessProperty(tess, GLU_TESS_WINDING_RULE,GLU_TESS_WINDING_ABS_GEQ_TWO); break;
    }

	//gluTessProperty(tess, GLU_TESS_BOUNDARY_ONLY,GL_TRUE);

	GLenum errCode;
	const GLubyte *errString;

	try {

		glNewList(filledRegionList, GL_COMPILE);
	    
		if ((errCode = glGetError()) != GL_NO_ERROR) {
			errString = gluErrorString(errCode);
			fprintf (stdout, "tesselate: OpenGL Error: %s\n", errString);
		   assert(glGetError() == GL_NO_ERROR);
		}

		gluTessBeginPolygon(tess, NULL);                   // with NULL data
		gluTessBeginContour(tess);

		for (int i=0; i<curve->size(); i++)
			gluTessVertex(tess, v+i*3, v+i*3);

		gluTessEndContour(tess);
		gluTessEndPolygon(tess);

		//_heapchk();
	
	}
	catch( char * str ) {
		cout << "Exception raised: " << str << '\n';
	}


	if ((errCode = glGetError()) != GL_NO_ERROR) {
		errString = gluErrorString(errCode);
		fprintf (stdout, "tesselate2: OpenGL Error: %s\n", errString);
	   assert(glGetError() == GL_NO_ERROR);
	}

    glEndList();

    gluDeleteTess(tess);        // delete after tessellation
	delete v;

}




