#ifndef STROKE_H
#define STROKE_H

#include <QtOpenGL>
#include <QFile>
#include <QVector>
#include <math.h>
#include "Point.h"
#include "StrokeProperties.h"
#include <GL/glut.h>
#include "jl_vectors.h"

#ifndef CALLBACK
#define CALLBACK
#endif

#define NUM_SLICES 20
#define UFREQ 0.005
#define VFREQ 0.5
#define USTART 0
#define VSTART 0.5
#define NUMLEVELS 3

typedef enum { CUBIC_BSPLINE, FOUR_POINT } CurveType;



class Stroke
{

  QVector<Point> control;
 // QVector<float> pressure;

  QVector<Point> * limit;
  QVector<Point> * temp;

private:


  bool computed;

  //int numLevels;

  GLuint filledRegionList; 

  //static GLUquadricObj * qobj;
public:

  float z;

  StrokeProperties properties;

  bool useTexture;

  //  feedforward testing 
  QVector<Vec2f> strokePoints;
  QVector<Vec2f> strokeAngles;
  Vec2f strokeWidths;
  int strokePointCount;
  Vec2f anchorPoint;
  Stroke *nextStroke;
  bool ageChecked;
  bool dying;
  int age;

  //CurveType curveType;

  Stroke();
  ~Stroke();
  Stroke(const Stroke& stroke);
  Stroke& operator =(const Stroke& stroke);

  void add(float x, float y,float press=1);
  void clear();
  void cleanup();
  static void drawLines(QVector<Point> * curve);
  void forceRecompute();
  void discPoint(float x,float y,float brushRadius);
  void drawCap(const Point & p0, float dx, float dy,float texU,float texV);
  void drawThickCurve(QVector<Point> * curve,bool cap=true);
  void drawControl();
  void drawLineCurve();
  void render();

  void tesselate(QVector<Point> * curve);
  void renderFilled();
 // void CALLBACK tessBeginCB(GLenum which);
 // void CALLBACK tessEndCB();
  //void CALLBACK tessErrorCB(GLenum errorCode);
  //void CALLBACK tessVertexCB(const GLvoid *data);


  QVector<Point>* getLimitCurve();
  QVector<Point>& getControlPoints(){return control;};
 // QVector<float> getPressures(){return pressure;};
  void subdivideCubicBSpline(QVector<Point> * inputCurve,
                             QVector<Point> * outputCurve);
  void subdivideFourPoint(QVector<Point> * inputCurve,
                          QVector<Point> * outputCurve);
  void subdivide(QVector<Point> * inputCurve,
                 QVector<Point> * outputCurve);
  void computeLimitCurve();
  inline void setProperties(StrokeProperties& sp){computed= false; properties = sp;};

  float getPointRadius(float textureU,int curvePoint,QVector<Point> * curve);

  bool writeStroke(QFile& file) const;
  bool readStroke(QFile& file);

};


#endif
