
#include "StrokeLayerProperties.h"
#include "MyPreference.h"
#include "tri_logger.hpp"
#include <iostream>
using namespace std;

StrokeLayerProperties::StrokeLayerProperties()
{
	initialize();
}




StrokeLayerProperties::~StrokeLayerProperties()
{

}  

/*
StrokeLayerProperties& StrokeLayerProperties::operator =(const StrokeLayerProperties &StrokeLayerProperties)
{
	copyData(StrokeLayerProperties);
	return *this;
}

void StrokeLayerProperties::copyData(const StrokeLayerProperties &StrokeLayerProperties)
{
	strokeProperties = StrokeLayerProperties.strokeProperties;
	actualStrokesPerFrame = StrokeLayerProperties.actualStrokesPerFrame;
	date = StrokeLayerProperties.date;
	startTime = StrokeLayerProperties.startTime;
	layerName = StrokeLayerProperties.layerName;

}
*/


void StrokeLayerProperties::initialize()
{
	minStrokeLength = MyPreference::minStrokeLength;
	maxStrokeLength = MyPreference::maxStrokeLength;

	orientationAngle = 0;
	reverseDirection = MyPreference::reverseDirections;
	fixedOrientationFactor = MyPreference::fixedOrientationFactor;
	boundaryFactor = MyPreference::boundaryFactor;
	imageGradientFactor = MyPreference::imageGradientFactor;
	userStrokesFactor = MyPreference::userStrokesFactor;


	fillEmptyRegions= MyPreference::fillEmptyRegions;
	fillColorDifference= MyPreference::fillColorDifference;
	fillUserSeed = false;

	strokeDensity = MyPreference::strokeDensity;
	densityWidthAdd = 0;
	densityBrightnessFactor = 0;
	maxBrightness = 255;
	minBrightness = 0;
	magnitudeThreshold = MyPreference::magnitudeThreshold;
	colorDifference = MyPreference::Threshold_ColorDifference;
//	edgeThreshold = MyPreference::Threshold_Edge;

	requestedStartFrame = 0;
	requestedEndFrame = 50;
	colorUpdate = MyPreference::colorUpdate;
	fadeRate = MyPreference::fadeRate;
	followPreviousStrokesFactor =  MyPreference::followStrokesFactor;
	followGradientFactor =  MyPreference::followGradientFactor;
	
	orientationFactor=MyPreference::orientationFactor;
	coherenceFactor=MyPreference::coherenceFactor;
	rigidityFactor =  MyPreference::rigidityFactor;
	colorFactor =  MyPreference::colorFactor;
	densityFactor = MyPreference::densityFactor;

	minFrames =  MyPreference::minFrames;
	maxFrames =  MyPreference::maxFrames;
	maxFlowAngle = MyPreference::maxAngle;
	maxStrokeAngle = MyPreference::maxStrokeAngle;
	maxStrokes = MyPreference::maxStrokes;
	fixedMovementRate = MyPreference::fixedMovementRate;
	fixedMovementAngle = 0;
	movementVariance = 0;

	
	magWeight = MyPreference::magWeight;
	distWeight = MyPreference::distWeight;
	boundaryFlowFactor= MyPreference::boundaryFactor;
	imageFlowFactor= MyPreference::imageGradientFactor;
	userFlowFactor= MyPreference::userStrokesFactor;
	fixedFlowFactor= MyPreference::fixedOrientationFactor;


	radiusFactor = MyPreference::radiusFactor;

	useOccludingLayerFlow = true;
	clipToSeedRegions=false;

	shiftToEdges=false;
	optimizeStrokes=MyPreference::optimizeStrokes;
	addVisibleStrokesToLayer=MyPreference::addVisibleToStrokeLayer;

	hueOffset = 0;
	saturationOffset = 0;
	valueOffset = 0;
	hueNoise = 0;
	saturationNoise = 0;
	valueNoise = 0;


	paintOccludedRegions= MyPreference::paintOccludedRegions;
	startInOccludedRegions= MyPreference::startInOccludedRegions;

	relativeWidthShift=0.5;


	//not used for now
	orientationVariance = 0;
//	passOverGuideStrokes=false;
//	darkenOccludedRegions=false;
	useStrokesFromAllLayers=false;

	date=QString("");
	startTime=QString("");
	getInputTime=-1;
	strokeGenTime=-1;
	renderTime=-1;
	overallTime=-1;
	generationId=-1;
	actualStrokesPerFrame.clear();
	distinctStrokes=0;

	seedOrder=QString("Random");

	generationTimelimit=1000;

	optimizeStrokeWidths=MyPreference::optimizeStrokeWidths;
	oswColorFactor=MyPreference::oswColorFactor;
	oswEdgeFactor=MyPreference::oswEdgeFactor;
	oswGrowFactor=MyPreference::oswGrowFactor;
	oswCoherenceFactor=MyPreference::oswCoherenceFactor;
	oswRegularizationFactor=MyPreference::oswRegularizationFactor;

	relativeWidthShift=0.5;

	contrastStretch=false;

}


bool StrokeLayerProperties::writeStrokeLayerPropertiesToXML(XmlWriter& xw) const
{
	StrokeLayerProperties initialTest;

	xw.writeOpenTag("StrokeLayerProperties");
		
		
		xw.writeOpenTag("StrokeLayerParameters");
			
			xw.writeTaggedString("layerParameterName",layerParameterName);
			xw.writeTaggedString("layerName",layerName);
			xw.writeTaggedString("requestedStartFrame",QString::number(requestedStartFrame));
			xw.writeTaggedString("requestedEndFrame",QString::number(requestedEndFrame));
			
			xw.writeTaggedString("maxStrokes",QString::number(maxStrokes));
			
			xw.writeTaggedString("strokeRadius",QString::number(strokeProperties.startRadius));

			xw.writeTaggedString("alpha",QString::number(strokeProperties.alpha));
			xw.writeTaggedString("textureFilename",strokeProperties.textureFilename);
			xw.writeAtomTag("useImageColor",AttrMap(QString("value"),QString::number(strokeProperties.useImageColor)));
			xw.writeAtomTag("isFill",AttrMap(QString("value"),QString::number(strokeProperties.isFill)));


			xw.writeTaggedString("red",QString::number(strokeProperties.color.red()));
			xw.writeTaggedString("blue",QString::number(strokeProperties.color.blue()));
			xw.writeTaggedString("green",QString::number(strokeProperties.color.green()));


			xw.writeTaggedString("hueOffset",QString::number(hueOffset));
			xw.writeTaggedString("saturationOffset",QString::number(saturationOffset));
			xw.writeTaggedString("valueOffset",QString::number(valueOffset));
			xw.writeTaggedString("hueNoise",QString::number(hueNoise));
			xw.writeTaggedString("saturationNoise",QString::number(saturationNoise));
			xw.writeTaggedString("valueNoise",QString::number(valueNoise));
			
			xw.writeTaggedString("radiusFactor",QString::number(radiusFactor));

			xw.writeAtomTag("addVisibleStrokesToLayer",AttrMap(QString("value"),QString::number(addVisibleStrokesToLayer)));
		
			xw.writeTaggedString("fixedOrientationFactor",QString::number(fixedOrientationFactor));
			xw.writeTaggedString("boundaryFactor",QString::number(boundaryFactor));
			xw.writeTaggedString("imageGradientFactor",QString::number(imageGradientFactor));
			xw.writeTaggedString("userStrokesFactor",QString::number(userStrokesFactor));
			xw.writeTaggedString("orientationAngle",QString::number(orientationAngle));
			
			xw.writeTaggedString("magWeight",QString::number(magWeight));
			xw.writeTaggedString("distWeight",QString::number(distWeight));
			
			xw.writeTaggedString("minStrokeLength",QString::number(minStrokeLength));
			xw.writeTaggedString("maxStrokeLength",QString::number(maxStrokeLength));
			xw.writeTaggedString("strokeDensity",QString::number(strokeDensity));
			xw.writeTaggedString("maxBrightness",QString::number(maxBrightness));
			xw.writeTaggedString("minBrightness",QString::number(minBrightness));
			
			xw.writeTaggedString("magnitudeThreshold",QString::number(magnitudeThreshold));
			//xw.writeTaggedString("edgeThreshold",QString::number(edgeThreshold));
			xw.writeTaggedString("colorDifference",QString::number(colorDifference));
			
			xw.writeTaggedString("boundaryFlowFactor",QString::number(boundaryFlowFactor));
			xw.writeTaggedString("imageFlowFactor",QString::number(imageFlowFactor));
			xw.writeTaggedString("userFlowFactor",QString::number(userFlowFactor));
			xw.writeTaggedString("fixedFlowFactor",QString::number(fixedFlowFactor));
			xw.writeTaggedString("fixedMovementAngle",QString::number(fixedMovementAngle));
			xw.writeTaggedString("fixedMovementRate",QString::number(fixedMovementRate));
			
			
			xw.writeTaggedString("minFrames",QString::number(minFrames));
			xw.writeTaggedString("maxFrames",QString::number(maxFrames));
			xw.writeTaggedString("colorUpdate",QString::number(colorUpdate));
			xw.writeTaggedString("fadeRate",QString::number(fadeRate));
			
			xw.writeTaggedString("densityWidthAdd",QString::number(densityWidthAdd));
			xw.writeTaggedString("densityBrightnessFactor",QString::number(densityBrightnessFactor));


			xw.writeOpenTag("StrokeDeformationParameters");
			if (optimizeStrokes)
			{
				xw.writeAtomTag("optimizeStrokes",AttrMap(QString("value"),QString::number(optimizeStrokes)));
				xw.writeTaggedString("orientationFactor",QString::number(orientationFactor));
				xw.writeTaggedString("coherenceFactor",QString::number(coherenceFactor));
				xw.writeTaggedString("rigidityFactor",QString::number(rigidityFactor));
				xw.writeTaggedString("colorFactor",QString::number(colorFactor));
				xw.writeTaggedString("densityFactor",QString::number(densityFactor));
			}
			else
			{
				xw.writeTaggedString("followPreviousStrokesFactor",QString::number(followPreviousStrokesFactor));
				xw.writeTaggedString("followGradientFactor",QString::number(followGradientFactor));
				xw.writeTaggedString("maxFlowAngle",QString::number(maxFlowAngle));
				xw.writeTaggedString("maxStrokeAngle",QString::number(maxStrokeAngle));
			}

			xw.writeCloseTag("StrokeDeformationParameters");
			
			xw.writeAtomTag("fillEmptyRegions",AttrMap(QString("value"),QString::number(fillEmptyRegions)));
			xw.writeAtomTag("fillColorDifference",AttrMap(QString("value"),QString::number(fillColorDifference)));
			xw.writeAtomTag("fillUserSeed",AttrMap(QString("value"),QString::number(fillUserSeed)));
			xw.writeAtomTag("paintOccludedRegions",AttrMap(QString("value"),QString::number(paintOccludedRegions)));
			xw.writeAtomTag("startInOccludedRegions",AttrMap(QString("value"),QString::number(startInOccludedRegions)));
			xw.writeAtomTag("useOccludingLayerFlow",AttrMap(QString("value"),QString::number(useOccludingLayerFlow)));
			xw.writeAtomTag("clipToSeedRegions",AttrMap(QString("value"),QString::number(clipToSeedRegions)));
			xw.writeAtomTag("shiftToEdges",AttrMap(QString("value"),QString::number(shiftToEdges)));

			xw.writeTaggedString("seedOrder",seedOrder);

			xw.writeAtomTag("optimizeStrokeWidths",AttrMap(QString("value"),QString::number(optimizeStrokeWidths)));
			xw.writeTaggedString("oswColorFactor",QString::number(oswColorFactor));
			xw.writeTaggedString("oswCoherenceFactor",QString::number(oswCoherenceFactor));
			xw.writeTaggedString("oswRegularizationFactor",QString::number(oswRegularizationFactor));
			xw.writeTaggedString("oswGrowFactor",QString::number(oswGrowFactor));
			xw.writeTaggedString("oswEdgeFactor",QString::number(oswEdgeFactor));

			xw.writeAtomTag("contrastStretch",AttrMap(QString("value"),QString::number(contrastStretch)));


			xw.writeTaggedString("generationTimelimit",QString::number(generationTimelimit));

		xw.writeCloseTag("StrokeLayerParameters");


if (generationId>-1)
		{
			xw.writeOpenTag("StrokeGenerationInformation");
				xw.writeTaggedString("generationId",QString::number(generationId));
				xw.writeTaggedString("startFrame",QString::number(startFrame));
				xw.writeTaggedString("endFrame",QString::number(endFrame));
				xw.writeTaggedString("date",date);
				xw.writeTaggedString("startTime",startTime);
				xw.writeTaggedString("getInputTime",QString::number(getInputTime));
				xw.writeTaggedString("strokeGenTime",QString::number(strokeGenTime));
				xw.writeTaggedString("renderTime",QString::number(renderTime));
				xw.writeTaggedString("overallTime",QString::number(overallTime));
				xw.writeTaggedString("actualRadius",QString::number(actualRadius));
				xw.writeTaggedString("gridSize",QString::number(gridSize));
				xw.writeTaggedString("distinctStrokes",QString::number(distinctStrokes));

				xw.writeOpenTag("actualStrokesPerFrame");
				for(int i=0;i < actualStrokesPerFrame.size();i++)
				{
					xw.writeTaggedString("numStrokes",QString::number(actualStrokesPerFrame[i]));

				}
				xw.writeCloseTag("actualStrokesPerFrame");



			xw.writeCloseTag("StrokeGenerationInformation");
		}
		

	xw.writeCloseTag("StrokeLayerProperties");

	return true;
}


bool StrokeLayerProperties::readStrokeLayerPropertiesFromXML(QDomElement& root)
{
	 //check that the root is called StrokeLayerParameters

	initialize();

	if(root.tagName()!="StrokeLayerProperties")
	{
		return false;
	}


	QDomNode u=root.firstChild();
	while(!u.isNull())
	{

		QDomElement upperU=u.toElement();

		if(upperU.tagName()=="StrokeGenerationInformation") // parse the layer object
		{
			 QDomNode n=upperU.firstChild();
			 while(!n.isNull())
			 {
				QDomElement e=n.toElement();

				if(e.tagName()=="date")
					date=e.text();
				if(e.tagName()=="startTime")
					startTime=e.text();
				if(e.tagName()=="getInputTime")
					getInputTime=e.text().toInt();
				if(e.tagName()=="strokeGenTime")
					strokeGenTime=e.text().toInt();
				if(e.tagName()=="renderTime")
					renderTime=e.text().toInt();
				if(e.tagName()=="overallTime")
					overallTime=e.text().toInt();
				if(e.tagName()=="generationId")
					generationId=e.text().toInt();
				if(e.tagName()=="startFrame")
					startFrame=e.text().toInt();
				if(e.tagName()=="endFrame")
					endFrame=e.text().toInt();
				if(e.tagName()=="actualRadius")
					actualRadius=e.text().toFloat();
				if(e.tagName()=="gridSize")
					gridSize=e.text().toInt();
				if(e.tagName()=="distinctStrokes")
					distinctStrokes=e.text().toInt();


				if(e.tagName()=="actualStrokesPerFrame")
				{
					QDomNode obj_n=e.firstChild();
					while(!obj_n.isNull())
					{
						QDomElement el=obj_n.toElement();
						if(el.tagName()=="numStrokes")
						{
							actualStrokesPerFrame.push_back(el.text().toInt());
						}

						obj_n=obj_n.nextSibling();
					}
							
				}
				n=n.nextSibling();
			 }
		}
		if(upperU.tagName()=="StrokeLayerParameters") // parse the layer object
		{

			QDomNode n=upperU.firstChild();
			while(!n.isNull())
			{
				QDomElement e=n.toElement();
				if(e.tagName()=="layerParameterName")
					layerParameterName=e.text();
				if(e.tagName()=="layerName")
					layerName=e.text();
				if(e.tagName()=="requestedStartFrame")
					requestedStartFrame=e.text().toInt();
				if(e.tagName()=="requestedEndFrame")
					requestedEndFrame=e.text().toInt();
				if(e.tagName()=="maxStrokes")
					maxStrokes=e.text().toInt();
				if(e.tagName()=="strokeRadius")
				{

					strokeProperties.startRadius=e.text().toInt();
					strokeProperties.endRadius=e.text().toInt();
				}
				if(e.tagName()=="strokeStartRadius")
				{
					strokeProperties.startRadius=e.text().toInt();
				}
				if(e.tagName()=="strokeEndRadius")
				{
					strokeProperties.endRadius=e.text().toInt();
				}
				if(e.tagName()=="alpha")
					strokeProperties.alpha=e.text().toFloat();
				if(e.tagName()=="textureFilename")
					strokeProperties.textureFilename=e.text();
				if(e.tagName()=="useImageColor")
					strokeProperties.useImageColor=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="isFill")
					strokeProperties.isFill=(bool)e.attribute(QString("value")).toInt();


				if(e.tagName()=="red")
					strokeProperties.color.setRed(e.text().toInt());
				if(e.tagName()=="blue")
					strokeProperties.color.setBlue(e.text().toInt());
				if(e.tagName()=="green")
					strokeProperties.color.setGreen(e.text().toInt());

				if(e.tagName()=="hueOffset")
					hueOffset=e.text().toFloat();
				if(e.tagName()=="saturationOffset")
					saturationOffset=e.text().toFloat();
				if(e.tagName()=="valueOffset")
					valueOffset=e.text().toFloat();
				if(e.tagName()=="hueNoise")
					hueNoise=e.text().toFloat();
				if(e.tagName()=="saturationNoise")
					saturationNoise=e.text().toFloat();
				if(e.tagName()=="valueNoise")
					valueNoise=e.text().toFloat();
				if(e.tagName()=="radiusFactor")
					radiusFactor=e.text().toFloat();
				if(e.tagName()=="addVisibleStrokesToLayer")
					addVisibleStrokesToLayer=(bool)e.attribute(QString("value")).toInt();

				if(e.tagName()=="fixedOrientationFactor")
					fixedOrientationFactor=e.text().toInt();
				if(e.tagName()=="boundaryFactor")
					boundaryFactor=e.text().toInt();
				if(e.tagName()=="imageGradientFactor")
					imageGradientFactor=e.text().toInt();
				if(e.tagName()=="userStrokesFactor")
					userStrokesFactor=e.text().toInt();
				if(e.tagName()=="orientationAngle")
					orientationAngle=e.text().toFloat();
				if(e.tagName()=="magWeight")
					magWeight=e.text().toFloat();
				if(e.tagName()=="distWeight")
					distWeight=e.text().toFloat();
				if(e.tagName()=="minStrokeLength")
					minStrokeLength=e.text().toInt();
				if(e.tagName()=="maxStrokeLength")
					maxStrokeLength=e.text().toInt();
				if(e.tagName()=="strokeDensity")
					strokeDensity=e.text().toFloat();
				if(e.tagName()=="orientationAngle")
					orientationAngle=e.text().toFloat();
				if(e.tagName()=="maxBrightness")
					maxBrightness=e.text().toInt();
				if(e.tagName()=="minBrightness")
					minBrightness=e.text().toInt();
				if(e.tagName()=="magnitudeThreshold")
					magnitudeThreshold=e.text().toInt();
				//if(e.tagName()=="edgeThreshold")
				//	edgeThreshold=e.text().toFloat();
				if(e.tagName()=="colorDifference")
					colorDifference=e.text().toFloat();
				if(e.tagName()=="boundaryFlowFactor")
					boundaryFlowFactor=e.text().toInt();
				if(e.tagName()=="imageFlowFactor")
					imageFlowFactor=e.text().toInt();
				if(e.tagName()=="userFlowFactor")
					userFlowFactor=e.text().toInt();
				if(e.tagName()=="fixedFlowFactor")
					fixedFlowFactor=e.text().toInt();
				if(e.tagName()=="fixedMovementAngle")
					fixedMovementAngle=e.text().toFloat();
				if(e.tagName()=="fixedMovementRate")
					fixedMovementRate=e.text().toInt();
				if(e.tagName()=="minFrames")
					minFrames=e.text().toInt();
				if(e.tagName()=="maxFrames")
					maxFrames=e.text().toInt();
				if(e.tagName()=="colorUpdate")
					colorUpdate=e.text().toFloat();
				if(e.tagName()=="fadeRate")
					fadeRate=e.text().toInt();
				if(e.tagName()=="densityWidthAdd")
					densityWidthAdd=e.text().toFloat();
				if(e.tagName()=="densityBrightnessFactor")
					densityBrightnessFactor=e.text().toFloat();
				if(e.tagName()=="densityWidthAdd")
					densityWidthAdd=e.text().toFloat();
				

				if(e.tagName()=="StrokeDeformationParameters") // parse the layer object
				{

					 QDomNode obj_n=e.firstChild();
					 while(!obj_n.isNull())
					 {
						QDomElement e=obj_n.toElement();


						if(e.tagName()=="followPreviousStrokesFactor")
							followPreviousStrokesFactor=e.text().toFloat();
						if(e.tagName()=="followGradientFactor")
							followGradientFactor=e.text().toFloat();
						if(e.tagName()=="maxFlowAngle")
							maxFlowAngle=e.text().toInt();
						if(e.tagName()=="maxStrokeAngle")
							maxStrokeAngle=e.text().toInt();

						if(e.tagName()=="optimizeStrokes")
							optimizeStrokes=(bool)e.attribute(QString("value")).toInt();
						if(e.tagName()=="orientationFactor")
							orientationFactor=e.text().toFloat();
						if(e.tagName()=="coherenceFactor")
							coherenceFactor=e.text().toFloat();
						if(e.tagName()=="rigidityFactor")
							rigidityFactor=e.text().toFloat();
						if(e.tagName()=="colorFactor")
							colorFactor=e.text().toFloat();
						if(e.tagName()=="densityFactor")
							densityFactor=e.text().toFloat();

						obj_n=obj_n.nextSibling();
					 }
				}
				if(e.tagName()=="fillEmptyRegions")
					fillEmptyRegions=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="fillUserSeed")
					fillUserSeed=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="fillColorDifference")
					fillColorDifference=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="paintOccludedRegions")
					paintOccludedRegions=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="startInOccludedRegions")
					startInOccludedRegions=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="useOccludingLayerFlow")
					useOccludingLayerFlow=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="clipToSeedRegions")
					clipToSeedRegions=(bool)e.attribute(QString("value")).toInt();
				if(e.tagName()=="shiftToEdges")
					shiftToEdges=(bool)e.attribute(QString("value")).toInt();


				if(e.tagName()=="seedOrder")
					seedOrder=e.text();

				if(e.tagName()=="optimizeStrokeWidths")
					optimizeStrokeWidths=(bool)e.attribute(QString("value")).toInt();

				if(e.tagName()=="oswColorFactor")
					oswColorFactor=e.text().toFloat();
				if(e.tagName()=="oswCoherenceFactor")
					oswCoherenceFactor=e.text().toFloat();
				if(e.tagName()=="oswRegularizationFactor")
					oswRegularizationFactor=e.text().toFloat();
				if(e.tagName()=="oswGrowFactor")
					oswGrowFactor=e.text().toFloat();
				if(e.tagName()=="oswEdgeFactor")
					oswEdgeFactor=e.text().toFloat();

				if(e.tagName()=="contrastStretch")
					contrastStretch=(bool)e.attribute(QString("value")).toInt();

				if(e.tagName()=="generationTimelimit")
					generationTimelimit=e.text().toInt();

				// end parsing the element at the top level
				n=n.nextSibling();
			}
		}

		u=u.nextSibling();
	}

	return true;
}


void StrokeLayerProperties::logLayerProperties()
{
	char strbuf[2048];
	sprintf(strbuf, "layerName= %s" ,layerName.toAscii().data() );
	LOG_STR(strbuf);

	LOG(startFrame);
	LOG(endFrame);
	LOG(requestedStartFrame);
	LOG(requestedEndFrame);
	LOG(maxStrokes);
	LOG(strokeProperties.alpha);
	LOG(strokeProperties.startRadius);
	LOG(strokeProperties.texture);
	sprintf(strbuf, "strokeProperties.textureFilename= %s" ,strokeProperties.textureFilename.toAscii().data() );
	LOG_STR(strbuf);
	LOG(strokeProperties.useImageColor);
	LOG(strokeProperties.startRadius);
	LOG(strokeProperties.endRadius);
	LOG(actualRadius);
	LOG(gridSize);
	LOG(hueOffset);
	LOG(saturationOffset);
	LOG(valueOffset);
	LOG(hueNoise);
	LOG(saturationNoise);
	LOG(valueNoise);
	LOG(radiusFactor);
	LOG(addVisibleStrokesToLayer);
	LOG(fixedOrientationFactor);
	LOG(boundaryFactor);
	LOG(imageGradientFactor);
	LOG(userStrokesFactor);
	LOG(orientationAngle);
	LOG(magWeight);
	LOG(distWeight);
	LOG(minStrokeLength);
	LOG(maxStrokeLength);
	LOG(strokeDensity);
	LOG(maxBrightness);
	LOG(minBrightness);
	LOG(magnitudeThreshold);
	//LOG(edgeThreshold);
	LOG(colorDifference);
	LOG(boundaryFlowFactor);
	LOG(imageFlowFactor);
	LOG(userFlowFactor);
	LOG(fixedFlowFactor);
	LOG(fixedMovementAngle);
	LOG(fixedMovementRate);
	LOG(minFrames);
	LOG(maxFrames);
	LOG(colorUpdate);
	LOG(fadeRate);
	LOG(densityWidthAdd);
	LOG(densityBrightnessFactor);
	LOG(followPreviousStrokesFactor);
	LOG(followGradientFactor);
	LOG(maxFlowAngle);
	LOG(maxStrokeAngle);
	LOG(optimizeStrokes);
	LOG(orientationFactor);
	LOG(coherenceFactor);
	LOG(rigidityFactor);
	LOG(colorFactor);
	LOG(densityFactor);
	LOG(fillEmptyRegions);
	LOG(fillColorDifference);
	LOG(fillUserSeed);
	LOG(paintOccludedRegions);
	LOG(startInOccludedRegions);
	LOG(useOccludingLayerFlow);
	LOG(clipToSeedRegions);
	LOG(shiftToEdges);
	LOG(optimizeStrokeWidths);
	LOG(oswColorFactor);
	LOG(oswCoherenceFactor);
	LOG(oswRegularizationFactor);
	LOG(oswGrowFactor);
	LOG(oswEdgeFactor);
	LOG(contrastStretch);

	LOG(generationTimelimit);
}