#ifndef StrokeLayerProperties_H
#define StrokeLayerProperties_H

#include <QDomDocument>
#include <QColor>
#include <QFile>
#include <QString>
#include <QList>
#include "StrokeProperties.h"
#include "XmlWriter.h"

class StrokeLayerProperties
{

public:

	
	//generation info
	
	QList<int> actualStrokesPerFrame;
	QString date;
	QString startTime;
	int getInputTime;
	int strokeGenTime;
	int renderTime;
	int overallTime;
	float actualRadius;
	int gridSize;
	int generationId;
	int distinctStrokes;


	//stroke layer parameters
	QString layerParameterName;


	QString layerName;

	int startFrame;
	int endFrame;
	int requestedStartFrame;
	int requestedEndFrame;

	int maxStrokes;

	//stroke properties

	StrokeProperties strokeProperties;



	//color shifts and noise
	int hueOffset;
	int saturationOffset;
	int  valueOffset;
	double hueNoise;
	double saturationNoise;
	double valueNoise;

	float radiusFactor;
	bool addVisibleStrokesToLayer;

	//spatial factors 
	int fixedOrientationFactor;
	int boundaryFactor;
	int imageGradientFactor;
	int userStrokesFactor;
	double orientationAngle;

	float magWeight;
	float distWeight;

	int minStrokeLength;
	int maxStrokeLength;
	float strokeDensity;
	int maxBrightness;
	int minBrightness;
	
	double magnitudeThreshold;

	double colorDifference;

	//temporal factors
	float boundaryFlowFactor;
	float imageFlowFactor;
	float userFlowFactor;
	float fixedFlowFactor;
	double fixedMovementAngle;
	int fixedMovementRate;


	int minFrames;
	int maxFrames;

	double colorUpdate;
	int fadeRate;


	float densityWidthAdd;
	float densityBrightnessFactor;

	//greedy factors
	float followPreviousStrokesFactor;
	float followGradientFactor;
	int maxFlowAngle;
	int maxStrokeAngle;

	//optimization factors
	bool optimizeStrokes;
	float orientationFactor;
	float coherenceFactor;
	float rigidityFactor;
	float colorFactor;
	float densityFactor;

	//starting conditions
	bool fillEmptyRegions;
	bool fillColorDifference;
	bool fillUserSeed;
	bool paintOccludedRegions;
	bool startInOccludedRegions;
	bool useOccludingLayerFlow;
	bool clipToSeedRegions;
	bool shiftToEdges;

	QString seedOrder;


	int generationTimelimit;

	//testing
	bool optimizeStrokeWidths;
	float relativeWidthShift;

	float oswColorFactor;
	float oswCoherenceFactor;
	float oswRegularizationFactor;
	float oswGrowFactor;
	float oswEdgeFactor;

	bool contrastStretch;

	
	//deprecated
	double orientationVariance;
	float movementVariance;
	bool reverseDirection;
	bool useStrokesFromAllLayers;
	//bool darkenOccludedRegions;
	//
	//double edgeThreshold;
	//double boundaryBlur;
	//bool passOverGuideStrokes;

	StrokeLayerProperties();
	~StrokeLayerProperties();
	//void copyData(const StrokeLayerProperties& StrokeLayerProperties);
	//StrokeLayerProperties& operator=(const StrokeLayerProperties& StrokeLayerProperties);
	void initialize();
	void logLayerProperties();

	bool readStrokeLayerPropertiesFromXML(QDomElement& root);
	bool writeStrokeLayerPropertiesToXML(XmlWriter& xw) const;

};

#endif