
#include "StrokeProperties.h"
#include "MyPreference.h"
#include "QVideoProcessing.h"


StrokeProperties::StrokeProperties()
{
	startRadius = MyPreference::startStrokeWidth;
	endRadius = MyPreference::endStrokeWidth;
	useImageColor = MyPreference::useImageColor;
	
	color.setRed(255);
	color.setGreen(255);
	color.setBlue(255);

	initColor.setRed(1);
	initColor.setGreen(1);
	initColor.setBlue(1);

	alpha = 1;
	texture = -1;
	textureFilename = QString("");

	//color stroke properties
	isColor = true;
	isGuide = true;
	isVisible = true;
	isFill = false;
	isSeed = false;

	hueOffset=0;
	saturationOffset=0;
	valueOffset=0;


}


StrokeProperties::StrokeProperties(int startR, int endR, int tex)
{
	startRadius = startR;
	endRadius = endR;
	texture = tex;

}


StrokeProperties::~StrokeProperties()
{

}  

StrokeProperties& StrokeProperties::operator =(const StrokeProperties &strokeProperties)
{
	copyData(strokeProperties);
	return *this;
}

void StrokeProperties::copyData(const StrokeProperties &strokeProperties)
{
	startRadius = strokeProperties.startRadius;
	endRadius = strokeProperties.endRadius;
	useImageColor = strokeProperties.useImageColor;
	color = strokeProperties.color;
	alpha = strokeProperties.alpha;
	texture = strokeProperties.texture;
	textureFilename = strokeProperties.textureFilename;
	isColor = strokeProperties.isColor;
	isGuide = strokeProperties.isGuide;
	isVisible = strokeProperties.isVisible;
	isFill = strokeProperties.isFill;
	isSeed = strokeProperties.isSeed;

	hueOffset=strokeProperties.hueOffset;
	saturationOffset=strokeProperties.saturationOffset;
	valueOffset=strokeProperties.valueOffset;

}




bool StrokeProperties::writeStrokeProperties(QFile& file) const
{

	int red = color.red();
	int green = color.green();
	int blue = color.blue();

	int placeholderint=0;
	bool placeholderbool=false;
	float placeholderfloat=0;

	file.write((char *)&isGuide,sizeof(bool));
	file.write((char *)&startRadius,sizeof(int));
	file.write((char *)&endRadius,sizeof(int));
	file.write((char *)&placeholderfloat,sizeof(float));
	file.write((char *)&texture,sizeof(int));
	QVideoProcessing::writeString(file,textureFilename);
	file.write((char *)&useImageColor,sizeof(bool));
	file.write((char *)&red,sizeof(int));
	file.write((char *)&green,sizeof(int));
	file.write((char *)&blue,sizeof(int));
	file.write((char *)&alpha,sizeof(float));
	file.write((char *)&placeholderint,sizeof(int));
	file.write((char *)&placeholderint,sizeof(int));


	file.write((char *)&isColor,sizeof(bool));
	file.write((char *)&isVisible,sizeof(bool));

	file.write((char *)&placeholderbool,sizeof(bool));


	float hOffset=hueOffset;
	float vOffset=valueOffset;
	float sOffset= saturationOffset;
	file.write((char *)&hOffset,sizeof(float));
	file.write((char *)&sOffset,sizeof(float));
	file.write((char *)&vOffset,sizeof(float));

	file.write((char *)&placeholderfloat,sizeof(float));
	file.write((char *)&placeholderfloat,sizeof(float));
	file.write((char *)&placeholderint,sizeof(int));
	file.write((char *)&placeholderint,sizeof(int));

	int seed = (int) isSeed;
	int color = (int) isColor;
	int fill = (int) isFill;
	int guide = (int) isGuide;
	
	file.write((char *)&seed,sizeof(int));
	file.write((char *)&color,sizeof(int));
	file.write((char *)&fill,sizeof(int));
	file.write((char *)&guide,sizeof(int));

	return true;
}
bool StrokeProperties::readStrokeProperties(QFile& file)
{

	int placeholderint=0;
	bool placeholderbool=false;
	float placeholderfloat=0;

	file.read((char *)&isGuide,sizeof(bool));
	file.read((char *)&startRadius,sizeof(int));
	file.read((char *)&endRadius,sizeof(int));
	file.read((char *)&placeholderfloat,sizeof(float));
	file.read((char *)&texture,sizeof(int));
	texture = -1;
	QVideoProcessing::readString(file,textureFilename);


	int idx = textureFilename.indexOf(QString("Brushes/"));
	textureFilename = textureFilename.right(textureFilename.size() - idx);


	file.read((char *)&useImageColor,sizeof(bool));

	int red, green, blue;
	file.read((char *)&red,sizeof(int));
	file.read((char *)&green,sizeof(int));
	file.read((char *)&blue,sizeof(int));

	if ((red < 0) || (green < 0) || (blue < 0))
	{

	printf("prob\n");
	return false;
	}

	color.setRed(red);
	color.setGreen(green);
	color.setBlue(blue);

	file.read((char *)&alpha,sizeof(float));
	file.read((char *)&placeholderint,sizeof(int));
	file.read((char *)&placeholderint,sizeof(int));


	file.read((char *)&isColor,sizeof(bool));
	file.read((char *)&isVisible,sizeof(bool));


	file.read((char *)&placeholderbool,sizeof(bool));



	float hOffset,sOffset,vOffset;
	file.read((char *)&hOffset,sizeof(float));
	file.read((char *)&vOffset,sizeof(float));
	file.read((char *)&sOffset,sizeof(float));
	hueOffset=hOffset;
	valueOffset=sOffset;
	saturationOffset=vOffset;


	file.read((char *)&placeholderfloat,sizeof(float));
	file.read((char *)&placeholderfloat,sizeof(float));
	file.read((char *)&placeholderint,sizeof(int));
	file.read((char *)&placeholderint,sizeof(int));


	int seed;
	int color;
	int fill;
	int guide;

	file.read((char *)&seed,sizeof(int));
	file.read((char *)&color,sizeof(int));
	file.read((char *)&fill,sizeof(int));
	file.read((char *)&guide,sizeof(int));
	isSeed = (bool) seed;
	isColor = (bool) color;
	isFill = (bool) fill;
	isGuide = (bool) guide;

	return true;
}



void StrokeProperties::updateStrokeProperty(QString prop, QString val)
{

	//printf("%f %i \n", val.toFloat(), val.toInt());

	if (prop.compare(QString("isGuide"))==0)
		isGuide = (bool)val.toFloat();
	else if (prop.compare(QString("isColor"))==0)
		isColor = (bool)val.toFloat();
	else if (prop.compare(QString("isVisible"))==0) 
		isVisible = (bool)val.toFloat();
	else if (prop.compare(QString("isFill"))==0)
		isFill = (bool)val.toFloat();
	else if (prop.compare(QString("isSeed"))==0)
		isSeed = (bool)val.toFloat();
	else
		printf("don't recognize the property name\n");

}
/*
bool StrokeProperties::writeXML(XmlWriter& xw) const
{
	return true;
}

bool StrokeProperties::readXML(XmlWriter& xw) const
{
	return true;
}
*/