#include <QtGui>
#include <QString>
#include <QSet>
#include <QDomDocument>
#include <QTabletEvent>
#include <QDir>
#include <QProgressDialog>
#include "VideoLabeling.h"
#include "MyPreference.h"
#include "QVideoProcessing.h"
#include "ContourTracking.h"
#include "Image.h"
#include "ImPyramid.h"
#include "PyrdPatchSet.h"
#include "time.h"
#include "ImageMatching.h"
#include <math.h>
#include <stdio.h>
#include "tri_logger.hpp"

double VideoLabeling::m_CurVersion=1.20;

VideoLabeling::VideoLabeling(MainWindow* parent):VideoSequence()
{
	m_Version="1.20";

	selectedLayerIndex=-1;
	labelState=Idle;
	pMainWindow=parent;
	IsMouseAllowed=true;
	IsBgLayerCreated=false;
	IsLayerMaskGenerated=false;
	IsBoundarySelected=true;

	// this is a state variable, no need to save
	tabIndex=0;
	IsEditingCorrespondence=false;
	m_WinSize=15;
	m_SearchSize=22;
	m_IsPyramidCreated=false;
	m_IsUncertainMatching=false;
	m_IsManualMatching=false;

	IsMaskPyramidComputed=false;
	
	m_CurrentMode=SelectMode;

	m_FillFramesBackward =MyPreference::fillFramesBackward;
	m_FillFramesForward = MyPreference::fillFramesForward;
	m_MaxStrokeId=20;
	m_GeneratedStrokesId = 0;
	m_ChangeStrokeId=0;

	
	_strokeWidth=MyPreference::startStrokeWidth;


	_hueOffset = 0;
	_saturationOffset = 0;
	_valueOffset = 0;
	_hueNoise = 0;
	_saturationNoise = 0;
	_valueNoise = 0;

	_boundaryFlowFactor= MyPreference::boundaryFactor;
	_imageFlowFactor= MyPreference::imageGradientFactor;
	_userFlowFactor= MyPreference::userStrokesFactor;
	_fixedFlowFactor= MyPreference::fixedOrientationFactor;

	_fixedMovementAngle = 0;
	_fixedMovementRate =MyPreference::fixedMovementRate;

	m_CurrDeleteId = 0;

	m_ShowLayerBoundaries=true;
	m_ShowStrokeSpines = true;
	m_ShowStrokes = true;
	m_ShowColorStrokes=true;
	m_ShowGuideStrokes=true;
	m_ShowSeedStrokes=true;
	m_ShowRegularStrokes=true;

	_followPreviousStrokesFactor = MyPreference::followStrokesFactor;
	_followGradientFactor = MyPreference::followGradientFactor;

    QPixmap msCursor("icons/arrow-multiple.png");
	m_MultipleSelectCursor = QCursor(msCursor,7,3);

    QPixmap drawCursor("icons/paintbrush-cursor.png");
	m_DrawCursor = QCursor(drawCursor,4,2);

    QPixmap overdrawCursor("icons/overdraw-cursor.png");
	m_OverdrawCursor = QCursor(overdrawCursor,4,2);

    QPixmap pushCursor("icons/select.png");
	m_PushCursor = QCursor(pushCursor,1,pushCursor.height()-1);

    QPixmap CreateRegionCursor("icons/region-cursor.png");
	m_CreateRegionCursor = QCursor(CreateRegionCursor,4,2);
	
	m_OverdrawStrokeFromLayer=-1;

	_colorUpdate=0.3;



}

VideoLabeling::VideoLabeling(const VideoLabeling &videoLabeling):VideoSequence(videoLabeling)
{
	copyData(videoLabeling);
}


VideoLabeling::~VideoLabeling(void)
{
}

void VideoLabeling::copyData(const VideoLabeling &videoLabeling)
{
        sprintf(_strbuf, "copying video labelling");
	LOG_STR_COUT(_strbuf);
	m_Version=videoLabeling.m_Version;

	VideoSequence::copyData(videoLabeling);
	m_LayerObjectList=videoLabeling.m_LayerObjectList;
	selectedLayerIndex=videoLabeling.selectedLayerIndex;
	labelState=videoLabeling.labelState;
	selectedBoundingBoxCorner=videoLabeling.selectedBoundingBoxCorner;
	IsBgLayerCreated=videoLabeling.IsBgLayerCreated;
	IsLayerMaskGenerated=videoLabeling.IsLayerMaskGenerated;
	m_ImPyramidList=videoLabeling.m_ImPyramidList;
	m_IsPyramidCreated=videoLabeling.m_IsPyramidCreated;

	tabIndex=videoLabeling.tabIndex;
	IsEditingCorrespondence=videoLabeling.IsEditingCorrespondence;
	m_SearchSize=videoLabeling.m_SearchSize;
	m_WinSize=videoLabeling.m_WinSize;
	m_IsUncertainMatching=videoLabeling.m_IsUncertainMatching;
	m_IsManualMatching=videoLabeling.m_IsManualMatching;

	IsMaskPyramidComputed=videoLabeling.IsMaskPyramidComputed;
	m_MaskPyramid=videoLabeling.m_MaskPyramid;

	m_CurrentStrokeProperties = videoLabeling.m_CurrentStrokeProperties;


}

VideoLabeling& VideoLabeling::operator =(const VideoLabeling &videoLabeling)
{
	copyData(videoLabeling);
	return *this;
}

void VideoLabeling::clear()
{
	 fileList.clear();
	 frameList.clear();
	 framesRead=false;
	 m_GeneratedStrokeLayerList.clear();
	 m_ActionHistory.clear();
	 m_LayerObjectList.clear();
	 m_ImPyramidList.clear();
	 m_IsPyramidCreated=false;

	 selectedLayerIndex=0;
	 curFrameIndex=0;
	 labelState=Idle;
	 IsBgLayerCreated=false;
	 IsLayerMaskGenerated=false;
	 IsMaskPyramidComputed=false;
	
	//m_IsUncertainMatching=false;
	//m_IsManualMatching=false;
}


void VideoLabeling::setupSelectedLayer()
{

	m_LayerObjectList[selectedLayerIndex].deselectCurrentStroke();
	IsBoundarySelected = true;
	labelState=Selected;
	emit showUserStrokeOptions(false);
	m_IsLoadingStrokeProperties = true;
	emit updatePropertiesForSelectedStroke(m_CurrentStrokeProperties);
	m_IsLoadingStrokeProperties = false;
	tabIndex=m_LayerObjectList[selectedLayerIndex].tabIndex(curFrameIndex);
	emit setTab(tabIndex);
	IsMaskPyramidComputed=false;
	if(tabIndex==2)
	{
		IsEditingCorrespondence=true;
		if(IsLayerMaskGenerated==false)
			GenerateLayers();
		else
		{
			QImage patch1,patch2;
			DImage vx,vy;
			if(curFrameIndex<numFrames-1)
			{
				m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);
				emit motionSelected(patch1,patch2,vx,vy);
				m_LayerObjectList[selectedLayerIndex].setCorrespondenceFlowDimension(curFrameIndex);
			}
			else
			{
				m_LayerObjectList[selectedLayerIndex].getLayerPatch(curFrameIndex,patch1,m_Image);
				emit motionSelected(patch1);
			}
		}
		emit motionTypeChanged(m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex).getMotionType());
		emit lambdaChanged(m_LayerObjectList[selectedLayerIndex].lambda(curFrameIndex));
	}
	emit loadLayerMotionPara(m_LayerObjectList[selectedLayerIndex].LayerMotionPara());
	if(tabIndex==0) // load the optical flow parameter
		emit setRow(m_LayerObjectList[selectedLayerIndex].selectedRow());


}

//------------------------------------------------------------------------------------------------------
// function to process mouse event based on the state variable
//------------------------------------------------------------------------------------------------------
void VideoLabeling::ProcessMouseEvent(ImageViewer& imageViewer,ImageViewer::MouseAction& MouseAction,QPoint& pos,float pressure,Qt::KeyboardModifiers& km)
{
	try{

	if(IsVideoLoaded==false)
		return;
	if(IsMouseAllowed==false)
		return;
	//---------------------------------------------------------------------
	// compute the logical coordinates on the image
	//---------------------------------------------------------------------
	QPointF point;
	QPoint ScreenOffset=imageViewer.getPaintingOffset();
	double ScaleFactor=imageViewer.getScaleFactor();
	point.setX((double)(pos.x()-ScreenOffset.x())/ScaleFactor);
	point.setY((double)(pos.y()-ScreenOffset.y())/ScaleFactor);

	static double lastX = -1;
	static double lastY = -1;

	int preSelectedLayerIndex,preSelectedStrokeIndex;
	bool IsPointSelected;
	int popupMenuSelection;
	QString objname;

	static QPointF moveStart;


	bool isShiftPressed = false;
	if (km & Qt::ShiftModifier)
	{
		isShiftPressed = true;
	}
	bool isControlPressed = false;
	if (km & Qt::ControlModifier)
	{
		isControlPressed = true;
	}


	bool isPolygon =false;
	bool selectObject = false;
	bool anySelected=false;

    int frame = curFrameIndex;
    //int strokeWidth = _strokeWidth;

	//---------------------------------------------------------------------
	// the branches of this function is based on the mouse action
	// and the labeling state (mouse_action x state)-->state
	//---------------------------------------------------------------------
	switch(MouseAction){
		case ImageViewer::LeftClick: // left click
			switch(labelState){
				//---------------------------------------------------------------------------
				// after left click, the Idel and CursorOver state will be activated to 
				// either CreateContour or Edit_InsertPoint, based on whether a
				// contour is selected
				//---------------------------------------------------------------------------					
				case Idle:
				case CursorOver:
					if(m_CurrentMode==SelectMode)
					{
						if((selectLayerObject(point,ScaleFactor)==true)) // one contontour is selected
							setupSelectedLayer();
						
					}
					else if (m_CurrentMode==CreateLayerMode)
					{
						// an empty spot is clicked
						labelState=CreateContour;
						createLayerObject(point);
						point.setY(m_Image.height()-point.y());
	
					}
					else if (m_CurrentMode==CreateStrokeMode)
					{
		
						if (m_LayerObjectList.size()!=0)
						{
							labelState=CreateStroke;
                            //sprintf(_strbuf, "pos %f %f %f", point.x(), point.y(),ScaleFactor );
							startStroke(point,pressure, ScaleFactor);
						}
					}
					
					emit Update();
					break;


				case CursorOverStroke:
					if((m_CurrentMode==SelectMode) &&(selectStroke(point,ScaleFactor)==true)) //check that a stroke has been selected
					{
						IsBoundarySelected = false;
						labelState=Selected;
						emit showUserStrokeOptions(true);
						m_IsLoadingStrokeProperties = true;
						m_LayerObjectList[selectedLayerIndex].strokeClicked(curFrameIndex);
						emit updatePropertiesForSelectedStroke(m_LayerObjectList[selectedLayerIndex].getSelectedStrokeProperties(curFrameIndex));
						m_IsLoadingStrokeProperties = false;
					}
					emit Update();
					break;
				//---------------------------------------------------------------------------
				// if the current state is CreateContour, then clicking adds a new
				// point to the existing contour and the state doesn't change. 
				// update the plot.
				//---------------------------------------------------------------------------
				case CreateContour:
					//point.setY(m_Image.height()-point.y());
					m_LayerObjectList[selectedLayerIndex].addPoint(curFrameIndex,point,false);
					emit Update();
					break;
				//---------------------------------------------------------------------------
				// if the current state is Edit_InsertPoint, first check whether the click
				// is outside the contour (so the contour is disselected). Otherwise, 
				// check whether a point is selected or a line segment is selected
				//---------------------------------------------------------------------------
				case Selected:
					if(IsEditingCorrespondence) // the procedure of editing correspondence by feature points
					{
						// if check outside the bounding box of the object, then deselect the object
						if(!m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).boundingBox().contains(point))
						{
							labelState=Idle;
							emit showUserStrokeOptions(true);
							m_IsLoadingStrokeProperties = true;
							emit updatePropertiesForSelectedStroke(m_CurrentStrokeProperties);
							m_IsLoadingStrokeProperties = false;
							IsEditingCorrespondence=false;		
							emit layerDeSelected();
							emit Update();
							
							break;
						}
						if(curFrameIndex==numFrames-1) // if last frame, not editing
						{
							QMessageBox::warning(pMainWindow,"Warning","You are viewing the last frame, no correspondence to label!");
							return;
						}
						if(m_IsPyramidCreated==false) // build the pyramid of the whole sequence if it has not been built
						{
							buildPyramid();
							m_IsPyramidCreated=true;
						}
						if(IsLayerMaskGenerated==false)
							GenerateLayers();

						// set the dimension of the layer for correspondence to get read for piecewise smooth flow interpolation
						m_LayerObjectList[selectedLayerIndex].setCorrespondenceFlowDimension(curFrameIndex);
						Correspondence& curCorrespondence=m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex);

						if(m_IsManualMatching==false) //automatic matching
						{
							int x1,y1;
							x1=point.x();
							y1=point.y();
							double x2,y2;
							x2=x1;
							y2=y1;

							// set the masks to get read for matching
							if(IsMaskPyramidComputed==false)
							{
								m_MaskPyramid.GenerateMask(m_LayerObjectList[selectedLayerIndex].visibleMask(curFrameIndex),6);
								ImageMatching::setMask(&m_MaskPyramid);
								IsMaskPyramidComputed=true;
							}

							if(m_IsUncertainMatching==false)
							{
								ImageMatching::setQueueRatio(6);
								ImageMatching::MatchCoarse2Fine(x1,y1,x2,y2,m_ImPyramidList[curFrameIndex],m_ImPyramidList[curFrameIndex+1],m_SearchSize,m_WinSize,(double)m_WinSize/2);
								ImageMatching::LucasKanade(x1,y1,x2,y2,m_ImPyramidList[curFrameIndex],m_ImPyramidList[curFrameIndex+1],m_WinSize-2,(double)(m_WinSize-2)/2);
								curCorrespondence.addCorrespondence(QPointF(x1,y1),QPointF(x2,y2));
							}
							else
							{
								SortQueue<Point> sort(m_SearchSize*m_SearchSize);
								ImageMatching::setQueueRatio(m_SearchSize);
								
								ImageMatching::MatchCoarse2Fine(x1,y1,x2,y2,sort,m_ImPyramidList[curFrameIndex],m_ImPyramidList[curFrameIndex+1],m_SearchSize,m_WinSize,(double)m_WinSize/2);
								curCorrespondence.addCorrespondence(QPointF(x1,y1),sort);
								if(curCorrespondence.IsParametricMotion())
								{
									// estimate the global motion to predict (x2,y2) better
									curCorrespondence.estimateMotion();
									x2=curCorrespondence.getPoint2().x();
									y2=curCorrespondence.getPoint2().y();
									curCorrespondence.removePoint();

									sort.reset();
									ImageMatching::MatchCoarse2Fine(x1,y1,x2,y2,sort,m_ImPyramidList[curFrameIndex],m_ImPyramidList[curFrameIndex+1],m_SearchSize,m_WinSize,(double)m_WinSize/2);
									curCorrespondence.addCorrespondence(QPointF(x1,y1),sort);
								}

							}
							//curCorrespondence.invalidateFlowEstimate();
							curCorrespondence.estimateMotion();

							//if(curCorrespondence.isMotionTypeSpecified())
							emit motionTypeChanged(curCorrespondence.getMotionType());
							
							QImage patch1,patch2;
							DImage vx,vy;
							m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);

							emit motionSelected(patch1,patch2,vx,vy);
							emit Update();

							emit pointSelected(point,curCorrespondence.getPoint2());
						}
						else // manual matching
						{
							curCorrespondence.addCorrespondence(point,point);
							emit pointSelected(point,point);
							labelState=ModifySecond;
							m_PrePoint=point;
							emit Update();
						}
						return;
					}
					preSelectedLayerIndex=selectedLayerIndex;
					preSelectedStrokeIndex = m_LayerObjectList[selectedLayerIndex].currentStroke();
					
		
					if((m_ShowLayerBoundaries)&& (m_LayerObjectList[selectedLayerIndex].isPolygon(curFrameIndex)) && (selectLayerObject(point,ScaleFactor)))
					{
						IsBoundarySelected= true;
						m_LayerObjectList[selectedLayerIndex].deselectCurrentStroke();
						
						emit showUserStrokeOptions(false);
						m_IsLoadingStrokeProperties = true;
						emit updatePropertiesForSelectedStroke(m_CurrentStrokeProperties);
						m_IsLoadingStrokeProperties = false;
						
						
						// other contour is selected
						if(preSelectedLayerIndex!=selectedLayerIndex)
						{
							//labelState=Idle;
							//emit layerDeSelected();
							// the layer motion parameter is changed
							IsMaskPyramidComputed=false;
							emit loadLayerMotionPara(m_LayerObjectList[selectedLayerIndex].LayerMotionPara());
							emit setTab(m_LayerObjectList[selectedLayerIndex].tabIndex(curFrameIndex));
						}
						else    // else check whether a line segment or point is selected
						{
							if (selectPoint(point,ScaleFactor,IsBoundarySelected))
								labelState=MovePoint;
						}
					}
					else if (selectStroke(point,ScaleFactor) == true)
					{
							int selectedStrokeIndex = m_LayerObjectList[selectedLayerIndex].currentStroke();
							IsBoundarySelected= false;
							
                                    sprintf(_strbuf, "selected stroke %i, old %i",selectedStrokeIndex,preSelectedStrokeIndex);
							LOG_STR_COUT(_strbuf);
							if ((preSelectedLayerIndex==selectedLayerIndex) && 
								(preSelectedStrokeIndex==selectedStrokeIndex) && 
								(selectPoint(point,ScaleFactor,IsBoundarySelected)))
							{
								m_LayerObjectList[selectedLayerIndex].copySelectedStrokes(curFrameIndex);
								labelState=MovePoint;
							}

							emit showUserStrokeOptions(true);

							m_IsLoadingStrokeProperties = true;
							emit updatePropertiesForSelectedStroke(m_LayerObjectList[selectedLayerIndex].getSelectedStrokeProperties(curFrameIndex));
							m_IsLoadingStrokeProperties = false;
							emit Update();
							
							break;

					}
					else // if global changes are selected
					{
				
						//deselectAllStrokes();

						if(m_LayerObjectList[selectedLayerIndex].IsCentroidSelected(curFrameIndex,point,ScaleFactor)==true) // start move the contour
							labelState=MoveContour;
						else
						{
							selectedBoundingBoxCorner=m_LayerObjectList[selectedLayerIndex].selectBoundingBox(curFrameIndex,point,ScaleFactor,IsBoundarySelected);
							if(selectedBoundingBoxCorner==4) // the boundary of the bonding box is selected, moving
							{	
								labelState=MoveContour;
								moveStart = point;
							}
							else if(selectedBoundingBoxCorner==-1) // an empty region is selected
							{
								m_IsLoadingStrokeProperties = true;
								emit updatePropertiesForSelectedStroke(m_CurrentStrokeProperties);
								m_IsLoadingStrokeProperties = false;
								labelState=Idle;
								IsEditingCorrespondence=false;
						
								emit layerDeSelected();
							}
							else // the corners are selected, scaling
								labelState=ScaleContour;
						}
					}
					motionParaRowSelected();
					if(tabIndex==0)
						emit setRow(m_LayerObjectList[selectedLayerIndex].selectedRow());
					emit Update();
					break;
				case ModifySecond:
					if(IsEditingCorrespondence)
					{
						Correspondence& curCorrespondence=m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex);
						curCorrespondence.updateSecond(point);
						curCorrespondence.estimateMotion();
						emit motionTypeChanged(curCorrespondence.getMotionType());
							
						QImage patch1,patch2;
						DImage vx,vy;
						m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);

						emit motionSelected(patch1,patch2,vx,vy);
						labelState=Selected;
						emit pointSelected(m_PrePoint,point);
						emit Update();
					}
					break;
				case ColorPicker:

					emit setCurrentColor(m_CurrentStrokeProperties.color, point.x(), point.y());
					//m_CurrentStrokeProperties.color.setRgb(m_Image.pixel(point.x(),point.y()));

					m_IsLoadingStrokeProperties = true;
					emit updatePropertiesForSelectedStroke(m_CurrentStrokeProperties);
					m_IsLoadingStrokeProperties = false;
					break;

				case MoveSelectedStrokes:

					for (int i=0;i<m_LayerObjectList.size();i++)
					{
						m_LayerObjectList[i].copySelectedStrokes(curFrameIndex);
					}

					moveStart = point;
					lastX =point.x();
					lastY =point.y();
					break;

				case PushSelectedStrokes:

					for (int i=0;i<m_LayerObjectList.size();i++)
					{
						m_LayerObjectList[i].copySelectedStrokes(curFrameIndex);
					}

					lastX =point.x();
					lastY =point.y();
					break;
				case SelectMultipleStrokes:

					//If shift isn't pressed, then deselect everything..
					if ((isShiftPressed==false) && (isControlPressed==false))
					{
						deselectAllStrokes();
					}
						
					m_SelectPoints.clear();
					m_SelectingPoints = true;
					m_SelectPoints.push_back(point);
					updateSelectedStrokes(point,isControlPressed);
					emit Update();
					break;		
				case OverdrawStroke:

					
					for (int i=0;i<m_LayerObjectList.size();i++)
					{
						if (m_LayerObjectList[i].copySelectedStrokes(curFrameIndex))
							anySelected=true;
					}

					m_SelectPoints.clear();
					if (anySelected==true)
					{
						m_SelectingPoints = true;
						m_SelectPoints.push_back(point);
					}
					else
					{
						m_SelectingPoints = false;
					}

					//updateSelectedStrokes(point,isControlPressed);
					emit Update();
					break;	
				default:
					break;
			}
			break; // end of LeftClick
		//---------------------------------------------------------------------------
		// right lick, to generate popup menus
		//---------------------------------------------------------------------------
		case ImageViewer::RightClick:
			switch(labelState){
				case Idle:
					deselectAllStrokes();
					emit Update();
					break;
				case CreateContour:
					labelState=Selected;


					if(m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).size()<=2)
					{
						m_LayerObjectList.removeAt(selectedLayerIndex);
						selectedLayerIndex=0;
						labelState=Idle;
						//m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).setIsPolygon(false);
					}
					else
					{
	
						m_LayerObjectList[selectedLayerIndex].setProjectDir(m_ProjectDir);
						

					   objname=QInputDialog::getText(pMainWindow,"Annotation","Please name the object:",QLineEdit::Normal,m_LayerObjectList[selectedLayerIndex].layerName(),0,Qt::Dialog);
						if(!objname.isEmpty())
							m_LayerObjectList[selectedLayerIndex].setLayerName(objname);

						int count =1;
						for(int i=0;i < m_LayerObjectList.size()-1;i++)
						{
							if (selectedLayerIndex==i)
								continue;

							if (QString::compare(m_LayerObjectList[selectedLayerIndex].layerName(),m_LayerObjectList[i].layerName())==0)
							{
								count++;
								m_LayerObjectList[selectedLayerIndex].setLayerName(objname+QString::number(count));
								i=0;
							}
						}
						m_ActionHistory.push_back(CreateRegionAction);

						IsLayerMaskGenerated=false;
					}

					emit Update();

					break;
				case CursorOver:

					IsBoundarySelected = true;
					labelState=Selected;
					m_IsLoadingStrokeProperties = true;
					emit updatePropertiesForSelectedStroke(m_CurrentStrokeProperties);
					m_IsLoadingStrokeProperties = false;
					ProcessPopupMenuSelection(selectPopupMenu(false));
					emit Update();
					break;
				case Selected:
					if(IsEditingCorrespondence) // the procedure of editing correspondence by feature points
					{
						if(curFrameIndex==numFrames-1) // if last frame, not editing
						{
							QMessageBox::warning(pMainWindow,"Warning","You are viewing the last frame, no correspondence exists!");
							return;
						}
						Correspondence& curCorrespondence=m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex);
						if(curCorrespondence.selectPoint(point,ScaleFactor*4))
						{
							if(curCorrespondence.size()==0)
								return;
							curCorrespondence.removePoint();
							//curCorrespondence.invalidateFlowEstimate();
							curCorrespondence.estimateMotion();
							QImage patch1,patch2;
							DImage vx,vy;
							m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);
							emit motionTypeChanged(curCorrespondence.getMotionType());
							emit motionSelected(patch1,patch2,vx,vy);
							emit Update();
							return;
						}

					}
					if(m_LayerObjectList[selectedLayerIndex].minPointDistance(curFrameIndex,point,IsBoundarySelected)<MyPreference::Threshold_ContourDistance/ScaleFactor)
						IsPointSelected=true;
					else if(m_LayerObjectList[selectedLayerIndex].minLineSegDistance(curFrameIndex,point,IsBoundarySelected)<MyPreference::Threshold_ContourDistance/ScaleFactor ||
								m_LayerObjectList[selectedLayerIndex].selectBoundingBox(curFrameIndex,point,ScaleFactor,IsBoundarySelected)>0)
								IsPointSelected=false;
					else
					{
						labelState=Idle;
						deselectAllStrokes();
						emit Update();
						return;
					}
					ProcessPopupMenuSelection(selectPopupMenu(IsPointSelected));
					break;
				default:
					break;
			}
			break; // end of right click
		//---------------------------------------------------------------------------
		// mouse move, transitions
		//---------------------------------------------------------------------------
		case ImageViewer::Move:
			
			switch(labelState){
				case Idle:
					if((m_CurrentMode==SelectMode) &&(selectLayerObject(point,ScaleFactor)==true))
					{
						IsBoundarySelected = true;
						labelState=CursorOver;
						emit Update();
					}
					if((m_CurrentMode==SelectMode) &&(selectStroke(point,ScaleFactor)==true))
					{
						IsBoundarySelected = false;
						labelState=CursorOverStroke;
						emit Update();
					}
					break;
				case CursorOver:
				case CursorOverStroke:
					preSelectedLayerIndex=selectedLayerIndex;
					if((selectLayerObject(point,ScaleFactor)==true) || (selectStroke(point,ScaleFactor)==true))
					{
						if(selectedLayerIndex!=preSelectedLayerIndex)
							emit Update();
					}
					else
					{
						labelState=Idle;
						emit Update();
					}
					break;
				case Selected:
					if(IsEditingCorrespondence)
						emit pointSelected(point,point);
					break;
				case ModifySecond:
					if(IsEditingCorrespondence)
					{
						Correspondence& curCorrespondence=m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex);
						curCorrespondence.updateSecond(point);
						emit pointSelected(m_PrePoint,point);
						emit Update();
					}
					break;
				case CreateContour:
					m_LayerObjectList[selectedLayerIndex].updateLastPoint(curFrameIndex,point);
					emit Update();
					break;
				case CreateStroke:

					//point.setY(m_Image.height()-point.y());
					if (pressure > 0.05)
					{
						
						QColor newcol;
						if (m_CurrentStrokeProperties.useImageColor)
						{

							if ((point.x() > 0) && (point.y() > 0) && (point.x()< m_Image.width()) && (point.y() < m_Image.height()))
							{

								QRgb pix = m_Image.pixel(point.x(), point.y());
								m_RedSum += qRed(pix);
								m_GreenSum += qGreen(pix);
								m_BlueSum += qBlue(pix);
								m_ColorCount++;
							}
							
							newcol = QColor(m_RedSum/m_ColorCount, m_GreenSum/m_ColorCount,m_BlueSum/m_ColorCount);
						}
						else
						{
							newcol = m_CurrentStrokeProperties.color;
						}

						StrokeProperties strokeProp = m_CurrentStrokeProperties;

						int h,s,v;
						newcol.getHsv(&h,&s,&v);
                              
						int newH = min(max(0,h +  strokeProp.hueOffset),359);
						int newS = min(max(0,s +  strokeProp.saturationOffset),254);
						int newV = min(max(0,v +  strokeProp.valueOffset),254);
       
						newcol.setHsv(newH,newS,newV);

						strokeProp.color= newcol;

						m_LayerObjectList[selectedLayerIndex].addStrokePoint(curFrameIndex,point,strokeProp, ScaleFactor,pressure);
						//emit redrawStrokes();
						emit Update();
						//emit strokesChanged(curFrameIndex);
					}
					break;
				case SelectMultipleStrokes:
					if (m_SelectingPoints ==true)
					{
						m_SelectPoints.push_back(point);
						updateSelectedStrokes(point,isControlPressed);
						emit Update();
					}
					break;
				case OverdrawStroke:
					if (m_SelectingPoints ==true)
						m_SelectPoints.push_back(point);
					else
						m_OverdrawStrokeFromLayer = getNearestLayer(curFrameIndex,point);

					emit Update();
					break;
				case MoveSelectedStrokes:
				case PushSelectedStrokes:
					if ((lastX != -1)||(lastY != -1))
					{
						
						for (int i=0; i < m_LayerObjectList.size();i++)
						{
                            QPointF lastpt(lastX,lastY);
							if (labelState==PushSelectedStrokes)
                                m_LayerObjectList[i].pushSelectedStrokes(curFrameIndex, _strokeWidth,lastpt,point);
							else
								m_LayerObjectList[i].shiftSelectedStrokes(curFrameIndex, QPointF(point.x()-lastX,point.y()-lastY));
						}
			
						lastX =point.x();
						lastY =point.y();
						emit redrawStrokes();
						emit Update();
					}
						
					break;
				case StrokeEraser:
					//deprecated
					/*
					if ((lastX != -1)||(lastY != -1))
					{

						for (int i=0; i < m_LayerObjectList.size();i++)
						{
							if (m_LayerObjectList[i].boundingBox(curFrameIndex).contains(point.x(), point.y()) || 
							m_LayerObjectList[i].boundingBox(curFrameIndex).contains(lastX,lastY))
							{
                                            sprintf(_strbuf, "found %f %f in layer %i", point.x(), point.y(), i);
								m_LayerObjectList[i].eraseUserStrokes(curFrameIndex, QPointF(lastX,lastY),point, _strokeWidth);
							}
							//seedPainter->drawLine(QPointF(lastX,lastY),point);
						}
						
						lastX =point.x();
						lastY =point.y();
						//emit strokesChanged(curFrameIndex);
					}
					*/
					emit Update();
					break;
				case MovePoint:
					m_LayerObjectList[selectedLayerIndex].updatePoint(curFrameIndex,point,IsBoundarySelected);
					emit redrawStrokes();
					if (IsBoundarySelected)
						IsLayerMaskGenerated=false;
					if (IsBoundarySelected == false)
						emit strokesChanged(curFrameIndex);
					emit Update();
					break;
				case MoveContour:
					m_LayerObjectList[selectedLayerIndex].moveContour(curFrameIndex,point,IsBoundarySelected);
					if (IsBoundarySelected)
						IsLayerMaskGenerated=false;
					emit strokesChanged(curFrameIndex);
					emit redrawStrokes();
					//Todo...have to update this for the rest of the affected frames
					emit Update();
					break;
				case ScaleContour:
					m_LayerObjectList[selectedLayerIndex].scaleContour(curFrameIndex,point,selectedBoundingBoxCorner,IsBoundarySelected);
					if (IsBoundarySelected)
						IsLayerMaskGenerated=false;
					emit strokesChanged(curFrameIndex);
					emit Update();
					break;
			}
			break; // end of Move
		//---------------------------------------------------------------------------
		// double click
		// it is effective only when the state is CreateContour. double click 
		// means finishing creating the contour and the state transit to Selected
		//---------------------------------------------------------------------------
		case ImageViewer::DoubleClick: // double click
			if(labelState==CreateContour)
			{
				m_LayerObjectList[selectedLayerIndex].removeLastPoint(curFrameIndex);
				if(m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).size()==2)
						m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).setIsPolygon(false);
				labelState=Selected;
				emit Update();
  			   objname=QInputDialog::getText(pMainWindow,"Annotation","Please name the object:",QLineEdit::Normal,m_LayerObjectList[selectedLayerIndex].layerName(),0,Qt::Dialog);
				if(!objname.isEmpty())
					m_LayerObjectList[selectedLayerIndex].setLayerName(objname);
			}

			if (((labelState==Idle) || (labelState==CursorOver) || (labelState==Selected)) &&
				(m_CurrentMode==SelectMode) &&((selectLayerObjectFromInterior(point)==true)))
				{
					setupSelectedLayer();
					emit Update();
				}

			break; // end of double lick
		//---------------------------------------------------------------------------
		// release
		//---------------------------------------------------------------------------
		case ImageViewer::Release: // mouse released

			switch(labelState){
				//case CreateContour:
				case 4:
					
					if ((m_CurrentStrokeProperties.useImageColor) && (m_CurrentStrokeProperties.isFill))
					{
						
						QList<ContourLabel>& layerStrokes = m_LayerObjectList[selectedLayerIndex].getStrokes(curFrameIndex);
						//QColor newcol;
						emit setCurrentColorForFilledStroke(layerStrokes[layerStrokes.size()-1].getStroke(), m_Image);
					}

					m_LayerObjectList[selectedLayerIndex].finishedStroke(curFrameIndex,m_ShowColorStrokes,m_ShowGuideStrokes,m_ShowSeedStrokes, m_ShowRegularStrokes);
					labelState=Idle;
					emit Update();
					emit redrawStrokes();
					emit strokesChanged(curFrameIndex);
					break;
				case MovePoint:
					m_LayerObjectList[selectedLayerIndex].updatePoint(curFrameIndex,point,IsBoundarySelected);
					m_LayerObjectList[selectedLayerIndex].propagateUpdatePoint(curFrameIndex,IsBoundarySelected);
					//taken out by peter temporarily
					//propagateUpdatePoint();
					labelState=Selected;
					emit Update();
					break;
				case MoveContour:
					//m_LayerObjectList[selectedLayerIndex].printGlobalLabels();
					m_LayerObjectList[selectedLayerIndex].moveContour(curFrameIndex,point,IsBoundarySelected);
					m_LayerObjectList[selectedLayerIndex].propagateMoveContour(curFrameIndex,IsBoundarySelected,false,point-moveStart,-1);
					labelState=Selected;
					//m_LayerObjectList[selectedLayerIndex].printGlobalLabels();
					emit Update();
					break;
				case ScaleContour:
					m_LayerObjectList[selectedLayerIndex].scaleContour(curFrameIndex,point,selectedBoundingBoxCorner,IsBoundarySelected);
					m_LayerObjectList[selectedLayerIndex].propagateScaleContour(curFrameIndex);
					labelState=Selected;
					emit Update();
					break;
				case MoveSelectedStrokes:
				case PushSelectedStrokes:

					m_ChangeStrokeId++;
					m_ActionHistory.push_back(ChangeStrokeAction);
					for (int i=0; i < m_LayerObjectList.size();i++)
					{

						if (labelState==PushSelectedStrokes)
						{
                            QPointF lastpt (lastX,lastY);
                            m_LayerObjectList[i].pushSelectedStrokes(curFrameIndex, _strokeWidth*2,lastpt,point);
							m_LayerObjectList[i].propagatePushedPoints(curFrameIndex,m_ChangeStrokeId);
							//m_LayerObjectList[i].updateSelectedStrokeColor(curFrameIndex,frameList,_colorUpdate);
						}
						else
						{
							m_LayerObjectList[i].shiftSelectedStrokes(curFrameIndex, QPointF(point.x()-lastX,point.y()-lastY));
							m_LayerObjectList[i].propagateMoveContour(curFrameIndex,false,true,point-moveStart,m_ChangeStrokeId);
							//m_LayerObjectList[i].updateSelectedStrokeColor(curFrameIndex,frameList,_colorUpdate);
						}
						
					}
					emit redrawStrokes();
					emit Update();
					lastX = -1;
					lastY = -1;
					break;

				case StrokeEraser:

					lastX = -1;
					lastY = -1;
					emit Update();
					break;
				case SelectMultipleStrokes:
					m_SelectingPoints = false;
					m_SelectPoints.clear();
					emit Update();
					break;
				case OverdrawStroke:

					if (m_SelectPoints.size()==0)
						break;
					int nearestLayer = getNearestLayer(curFrameIndex,m_SelectPoints[0]);
					if (nearestLayer==-1)
						break;

					if (m_LayerObjectList[nearestLayer].applyOverdrawStroke(curFrameIndex,frameList, _colorUpdate, m_ChangeStrokeId+1, m_SelectPoints,ScaleFactor,0.75,isShiftPressed)==true)
					{
						m_ActionHistory.push_back(ChangeStrokeAction);
						m_ChangeStrokeId++;
					}

					m_SelectingPoints = false;
					m_SelectPoints.clear();
					emit redrawStrokes();
					emit Update();
					
					break;

			}
			break; // end of release
	}

  }
  catch(...)
  {
	  cout << "Exception found in processing the mouse event" << endl;
  }

}



//------------------------------------------------------------------------------------------------------
// function to display the contour labels
//------------------------------------------------------------------------------------------------------
void VideoLabeling::Paint(ImageViewer &imageViewer, QPainter &mPainter,QImage& saveimg)
{
	if(m_LayerObjectList.size()==0)
		return;

	QPoint temp;
	QPointF ScreenOffset;
	double ScaleFactor;
	temp=imageViewer.getPaintingOffset();
	ScreenOffset.setX(temp.x());
	ScreenOffset.setY(temp.y());
	ScaleFactor=imageViewer.getScaleFactor();	



	if (m_ShowStrokes)
		emit paintStrokes(imageViewer, mPainter,frameList, m_LayerObjectList,curFrameIndex);

	
	if ((labelState==SelectMultipleStrokes) && (m_SelectPoints.size()> 1))
	{
			QPen currPen = mPainter.pen();
			QBrush currBrush = mPainter.brush();
			mPainter.setPen(QPen(QColor(255,255,255,100),_strokeWidth, Qt::SolidLine, Qt::RoundCap,Qt::MiterJoin));
			mPainter.setBrush(QBrush(QColor(255,255,255,100), Qt::SolidPattern));
			
			for (int i=1;i < m_SelectPoints.size();i++)
			{
				mPainter.drawLine(m_SelectPoints[i-1]*ScaleFactor+ScreenOffset,m_SelectPoints[i]*ScaleFactor+ScreenOffset);
			}

			mPainter.setPen(currPen);
			mPainter.setBrush(currBrush);
	}

	if ((labelState==OverdrawStroke) && (m_SelectPoints.size()> 1))
	{
			QPen currPen = mPainter.pen();
			QBrush currBrush = mPainter.brush();
			mPainter.setPen(QPen(QColor(0,0,0,255),3, Qt::SolidLine, Qt::RoundCap,Qt::MiterJoin));
			mPainter.setBrush(QBrush(QColor(0,0,0,255), Qt::SolidPattern));
			
			for (int i=1;i < m_SelectPoints.size();i++)
			{
				mPainter.drawLine(m_SelectPoints[i-1]*ScaleFactor+ScreenOffset,m_SelectPoints[i]*ScaleFactor+ScreenOffset);
			}

			mPainter.setPen(currPen);
			mPainter.setBrush(currBrush);
	}

	// draw the contour according to the depth
	QMap<unsigned int, int> map;
	for (int i=0;i<m_LayerObjectList.size();i++)
		map.insert(m_LayerObjectList[i].depth(curFrameIndex)*255+i,i);
	QList<unsigned int> list;
	list=map.keys();

	for(int i=0;i<m_LayerObjectList.size();i++)
	{
		int index=map.value(list[m_LayerObjectList.size()-1-i]);

		/*
		if (m_ShowLayerBoundaries)
			m_LayerObjectList[index].DrawContour(curFrameIndex,rotoPainter,ScreenOffset,ScaleFactor);

		bool overDrawLayer = (labelState==OverdrawStroke) && (m_OverdrawStrokeFromLayer==index);
		if (m_ShowStrokeSpines)
				m_LayerObjectList[index].DrawStrokes(curFrameIndex,rotoPainter,ScreenOffset,ScaleFactor,overDrawLayer);
		*/	

		if(index!=selectedLayerIndex)
		{
			if (m_ShowLayerBoundaries)
				m_LayerObjectList[index].DrawContour(curFrameIndex,mPainter,ScreenOffset,ScaleFactor);

			bool overDrawLayer = (labelState==OverdrawStroke) && (m_OverdrawStrokeFromLayer==index);

			if (m_ShowStrokeSpines)
				m_LayerObjectList[index].DrawStrokes(curFrameIndex,mPainter,ScreenOffset,ScaleFactor,overDrawLayer);
		}
		else
		{
			bool IsActive=false,IsCreating=false,IsCursorOver=false;
			switch(labelState){
				case Idle:
					break;
				case CursorOver:
				case CursorOverStroke:
					IsCursorOver=true;
					break;
				case CreateContour:
					IsCreating=true;
					break;
				case Selected:
				case MovePoint:
				case ScaleContour:	
				case MoveContour:
					IsActive=true;
				case ModifySecond:
					IsActive=true;
					break;

				
			}
			if (m_ShowLayerBoundaries)
				m_LayerObjectList[index].DrawContour(curFrameIndex,mPainter,ScreenOffset,ScaleFactor,IsActive,IsCreating,IsCursorOver,IsBoundarySelected);
			
			bool overDrawLayer = (labelState==OverdrawStroke) && (m_OverdrawStrokeFromLayer==index);

			if (m_ShowStrokeSpines)
				m_LayerObjectList[index].DrawStrokes(curFrameIndex,mPainter,ScreenOffset,ScaleFactor,overDrawLayer, IsActive,IsCreating,IsCursorOver,IsBoundarySelected);
			
			if((labelState==Selected || labelState==ModifySecond) && IsEditingCorrespondence)
				m_LayerObjectList[index].DrawCorrespondence(curFrameIndex,mPainter,ScreenOffset,ScaleFactor);
		}
	}
	if((labelState==CursorOver) && (m_ShowLayerBoundaries))
		m_LayerObjectList[selectedLayerIndex].ShowText(curFrameIndex,mPainter,ScreenOffset,ScaleFactor);

	//saveimg.save(m_LayerObjectList[0].projectDir()+QString("roto-")+QString::number(curFrameIndex)+QString(".png"));


}

//------------------------------------------------------------------------------------------------------
// function to output the labeling to an image
//------------------------------------------------------------------------------------------------------
void VideoLabeling::OutputLabelFrame(int frameIndex,const QString& filename,int quality)
{
	QImage outputImage;
	if(outputImage.load(framePath+fileList[frameIndex])==false)
		return;
	QPainter painter(&outputImage);
	painter.setRenderHint(QPainter::Antialiasing);
	if(m_LayerObjectList.size()>0)
	{
		// draw the contour according to the depth
		QMap<unsigned int, int> map;
		for (int i=0;i<m_LayerObjectList.size();i++)
			map.insert(m_LayerObjectList[i].depth(frameIndex)*255+i,i);
		QList<unsigned int> list;
		list=map.keys();
		for(int i=0;i<m_LayerObjectList.size();i++)
		{
			int index=map.value(list[m_LayerObjectList.size()-1-i]);
                        QPointF zero(0,0);
                        m_LayerObjectList[index].DrawContour(frameIndex,painter,zero,1,false,false,true);
                        m_LayerObjectList[index].ShowText(frameIndex,painter,zero,1);
		}
	}
	outputImage.save(filename,0,quality);
}

//------------------------------------------------------------------------------------------------------
// function to output the whole labling
//------------------------------------------------------------------------------------------------------
void VideoLabeling::OutputAllLabeling(const QString &pathname)
{
	QString PathName(pathname);
	if(PathName.right(1)!="/")
		PathName+="/";
	IsMouseAllowed=false;
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	for(int i=0;i<numFrames;i++)
	{
		QString filename=fileList[i];
		filename.chop(4);
		OutputLabelFrame(i,PathName+filename+"_label.jpg",97);
	}
	IsMouseAllowed=true;
	QApplication::restoreOverrideCursor();
}

//------------------------------------------------------------------------------------------------------
// overrided function, findex ranges [1,numFrames], MATLAB convention
//------------------------------------------------------------------------------------------------------
void VideoLabeling::gotoFrame(int findex)
{
	// for now we don't strecth the frames; it can only be tracked
	//for(int i=0;i<m_LayerObjectList.size();i++)
	//	if(i==selectedLayerIndex && labelState!=Idle && labelState!=CursorOver)
	//		m_LayerObjectList[i].StretchToFrame(findex-1);

	// when the contour is modified, change frame is prohibited
	if(labelState==CreateContour || labelState==MovePoint || labelState==ScaleContour || labelState==MoveContour)
		return;

	if(labelState==Selected)
		if((m_LayerObjectList[selectedLayerIndex].numFrames() >curFrameIndex)&& (m_LayerObjectList[selectedLayerIndex].isPolygon(curFrameIndex)==false))
			m_LayerObjectList[selectedLayerIndex].StretchToFrame(findex-1);

	if(labelState!=Idle)
		if(m_LayerObjectList.size()>0)
			if(m_LayerObjectList[selectedLayerIndex].startFrame()>findex-1 || m_LayerObjectList[selectedLayerIndex].endFrame()<findex-1)
				labelState=Idle;

	// it's important that this function is called later since it will update the drawing
	VideoSequence::gotoFrame(findex);

	if(findex<numFrames)
		m_NextImage.load(framePath+fileList[findex]);
	else
		m_NextImage=m_Image;
	emit frameChanged(&m_Image,&m_NextImage);	

	ContourLabel::setImWidth(m_Image.width());
	ContourLabel::setImHeight(m_Image.height());

	if(IsEditingCorrespondence)
	{
		if(curFrameIndex<numFrames-1)
		{
			// set the dimension of the layer for correspondence to get ready for piecewise smooth flow interpolation
			m_LayerObjectList[selectedLayerIndex].setCorrespondenceFlowDimension(curFrameIndex);

			QImage patch1,patch2;
			DImage vx,vy;
			m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);
			emit motionTypeChanged(m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex).getMotionType());
			emit lambdaChanged(m_LayerObjectList[selectedLayerIndex].lambda(curFrameIndex));
			emit motionSelected(patch1,patch2,vx,vy);
		}
		
	}
	if(labelState==Selected)
	{
		tabIndex=m_LayerObjectList[selectedLayerIndex].tabIndex(curFrameIndex);
                //sprintf(_strbuf, "Tab: %d",tabIndex);
		emit setTab(tabIndex);
	}
	IsMaskPyramidComputed=false;
	// to control the display of the flow field

	//DEBUG: Took this out since it was slowing down 
	//if(tabIndex==0)
	//	motionParaRowSelected();
}


//-----------------------------------------------------------------------------------------------------
// function to select layer object according to the the interior of the region
//-----------------------------------------------------------------------------------------------------
bool VideoLabeling::selectLayerObjectFromInterior(const QPointF &point)
{
	// check whether the bounding box is selected
	for(int i=0;i<m_LayerObjectList.size();i++)
	{

		if(m_LayerObjectList[i].IsWithinLayer(curFrameIndex,point,true))
		{
			selectedLayerIndex=i;
			return true;
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------------------------
// function to select layer object according to the distance to the contour
//-----------------------------------------------------------------------------------------------------
bool VideoLabeling::selectLayerObject(const QPointF &point, double ScaleFactor)
{

	double MinDistance=MyPreference::DefaultContourDistance;
	int layerIndex=0;
	// first check whether the query point selects a point
	for(int i=0;i<m_LayerObjectList.size();i++)
	{
		//double distance=m_LayerObjectList[i].distanceToContour(curFrameIndex,point);
		double distance;
		if(m_LayerObjectList[i].isPolygon(curFrameIndex))
			distance=m_LayerObjectList[i].minPointDistance(curFrameIndex,point,true);
		else
			continue;
		if(distance<MinDistance)
		{
			MinDistance=distance;
			layerIndex=i;
		}
	}
	if(MinDistance<MyPreference::Threshold_PointDistance/sqrt(ScaleFactor))
	{
		selectedLayerIndex=layerIndex;
		return true;
	}
	// then check whether the query point selects a contour
	MinDistance=MyPreference::DefaultContourDistance;
	layerIndex=0;
	for(int i=0;i<m_LayerObjectList.size();i++)
	{
		double distance;
		if(m_LayerObjectList[i].isPolygon(curFrameIndex))
			distance=m_LayerObjectList[i].minLineSegDistance(curFrameIndex,point,true);
		else
			continue;
		if(distance<MinDistance)
		{
			MinDistance=distance;
			layerIndex=i;
		}
	}
	if(MinDistance<MyPreference::Threshold_LineDistance/sqrt(ScaleFactor))
	{
		selectedLayerIndex=layerIndex;
		return true;
	}
	// check whether the bounding box is selected
	for(int i=0;i<m_LayerObjectList.size();i++)
	{
		if(m_LayerObjectList[i].isPolygon(curFrameIndex))
			continue;
		if(m_LayerObjectList[i].IsCentroidSelected(curFrameIndex,point,ScaleFactor))
		{
			selectedLayerIndex=i;
			return true;
		}
		if(m_LayerObjectList[i].selectBoundingBox(curFrameIndex,point,ScaleFactor,true)>0)
		{
			selectedLayerIndex=i;
			return true;
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------------------------
// function to check whether a stroke is selected
//-----------------------------------------------------------------------------------------------------
bool VideoLabeling::selectStroke(const QPointF &point,double ScaleFactor)
{
	for(int i=0;i < m_LayerObjectList.size();i++)
	{
		if(m_LayerObjectList[i].IsStrokeSelected(curFrameIndex,point,ScaleFactor))
		{
			selectedLayerIndex = i;
                        //sprintf(_strbuf, "found stroke for layer %i",selectedLayerIndex);
			return true;
		}
	}
	return false;
}




//-----------------------------------------------------------------------------------------------------
// function to create a layer object
//-----------------------------------------------------------------------------------------------------
void VideoLabeling::createLayerObject(const QPointF &point)
{
	LayerObject newLayerObject;
	newLayerObject.create(numFrames,curFrameIndex,point,m_LayerObjectList.size());
	m_LayerObjectList.append(newLayerObject);
	selectedLayerIndex=m_LayerObjectList.size()-1;
}

//-----------------------------------------------------------------------------------------------------
// function to create a new stroke object
//-----------------------------------------------------------------------------------------------------
void VideoLabeling::startStroke(const QPointF &point, float pressure, double ScaleFactor)
{
	if (m_LayerObjectList.size()==0)
		return;

	StrokeProperties strokeProp = m_CurrentStrokeProperties;

	int minDepth = 1000;
	for(int i=0;i < m_LayerObjectList.size();i++)
	{
		if (m_LayerObjectList[i].IsWithinLayer(curFrameIndex,point,true) && m_LayerObjectList[i].depth(curFrameIndex) < minDepth)
		{
			minDepth = m_LayerObjectList[i].depth(curFrameIndex);
			selectedLayerIndex =i;
                        //sprintf(_strbuf, "associated stroke with layer %i at depth ", selectedLayerIndex, minDepth);
		}

	}
	if (m_CurrentStrokeProperties.useImageColor)
	{
		QRgb pix = m_Image.pixel(point.x(), point.y());
		strokeProp.color.setRgb(pix);
		m_RedSum = qRed(pix);
		m_GreenSum = qGreen(pix);
		m_BlueSum = qBlue(pix);
		m_ColorCount = 1;
	}

        //sprintf(_strbuf, "offsets %i %i %i", _hueOffset,_saturationOffset,_valueOffset);
        //sprintf(_strbuf, "noise %i %i %i", _hueNoise,_saturationNoise,_valueNoise);

	m_CurrentStrokeProperties.hueOffset = normal(_hueOffset,_hueNoise+0.001);
	m_CurrentStrokeProperties.saturationOffset = normal(_saturationOffset,_saturationNoise+0.001);
	m_CurrentStrokeProperties.valueOffset = normal(_valueOffset,_valueNoise+0.001);

        //sprintf(_strbuf, "m_StrokeNoise %i %i %i", m_StrokeNoiseX,m_StrokeNoiseY,m_StrokeNoiseZ);

	

	int h=0,s=0,v=0;
	strokeProp.color.getHsv(&h,&s,&v);

	int newH = min(max(0,h +  m_CurrentStrokeProperties.hueOffset),359);
	int newS = min(max(0,s +  m_CurrentStrokeProperties.saturationOffset),254);
	int newV = min(max(0,v +  m_CurrentStrokeProperties.valueOffset),254);
	strokeProp.color.setHsv(newH,newS,newV);


	m_LayerObjectList[selectedLayerIndex].createStroke(curFrameIndex,point,strokeProp,pressure,m_MaxStrokeId);
	m_MaxStrokeId++;

	m_ActionHistory.push_back(CreateStrokeAction);

	//tell the stroke painter to redraw this frame
	//emit strokesChanged(curFrameIndex);
	
}


//some link errors, these are the same as in Randn.h
double VideoLabeling::normal(const double &mean, const double &std)
{
  double pii=3.1415927;
  double r_max=RAND_MAX;
  double randVal = rand();
  double firstTerm = (randVal+1)/r_max;
  double secondTerm = 2*pii*randVal/r_max;

 // (std*sqrt(-2*log((rand()+1)/r_max))*sin(2*pii*rand()/r_max))+mean;
  return (std*sqrt(-2*log(firstTerm))*sin(secondTerm))+mean;
} 

double VideoLabeling::truncNormal(const double &mean, const double &std,const double &lowerBound,const double &upperBound)
{
	double n= normal(mean,std);
	if (n < lowerBound) 
		n =lowerBound;

	if (n > upperBound) 
		n = upperBound;

	return n;
}



bool VideoLabeling::selectPoint(const QPointF &point, double ScaleFactor,bool boundaryContour)
{


	// first search the points
	double minPointDistance=m_LayerObjectList[selectedLayerIndex].minPointDistance(curFrameIndex,point,boundaryContour);

	if(minPointDistance<MyPreference::Threshold_PointDistance/ScaleFactor)
		return true;

	double minLineDist = m_LayerObjectList[selectedLayerIndex].minLineSegDistance(curFrameIndex,point,boundaryContour);

	if(minLineDist>MyPreference::Threshold_LineDistance/ScaleFactor)
		return false;

        sprintf(_strbuf, "dist was %f, thresh was %f",minLineDist,MyPreference::Threshold_LineDistance/ScaleFactor);
	LOG_STR_COUT(_strbuf);

	m_LayerObjectList[selectedLayerIndex].insertPoint(curFrameIndex,point,boundaryContour);
	return true;
}

void VideoLabeling::Initialize(const QString&path, const QFileInfoList& filelist)
{
	clear();
	VideoSequence::Initialize(path,filelist);
}

//----------------------------------------------------------------------------------------
// function to display a popup menu and return which one is chosen based on
// whether a point is selected
// return value: 0--delete point if selected (when IsPointSelected==false, no 0
//						 -2--delete label
//						  1--delete cfontour is selected
//						  2--rename is selected
//						  3--change depth
//
//						  11--forward track one frame
//						  12--forward track five frames
//						  13--forward track ten frames
//
//						  21--backward track one frame
//						  22--backward track five frames
//						  23--backward track five frames
//
//						 100--track all the frames
//
//						 51--forward replicate one frame
//						 52--forward replicate five frames
//						 53--forward replicate ten frames
//						 57--forward replicate all frames
//
//						 61--backward replicate one frame
//						 62--backward replicate five frames
//						 63--backward replicate ten frames
//						 67--backward replicate all frames
//
//						 70--replicate all the frames
//
//						  40--cut the labeling beyond the point
//						  41--remove all the trackings
//
//						  30--enforce the frame to be a key frame
//						  31--remove the user labels
//
//						  81--cut all the forward frames
//						  82--cut all the backward frames
//
// starting from 100 is the process of motion labeling
//						 100-- output the layer mask
//						 101-- output the visible layer mask
//----------------------------------------------------------------------------------------
/*
int VideoLabeling::selectPopupMenu(bool IsPointSelected)
{
	QMenu popupMenu;
	QAction* outputLayerMask;
	QAction* outputVisibleLayerMask;
	QAction *changeDepth;
	if(IsLayerMaskGenerated==true)
	{
		changeDepth=popupMenu.addAction(tr("Change depth"));
		outputLayerMask=popupMenu.addAction(tr("Output layer mask"));
		outputVisibleLayerMask=popupMenu.addAction(tr("Output visible layer mask"));

	}
	QAction* selected=popupMenu.exec(QCursor::pos());
	if(selected==NULL)
		return -1;
	if(selected==changeDepth)
		return 3;
	if(selected==outputLayerMask)
		return 100;
	if(selected==outputVisibleLayerMask)
		return 101;
	return -1;	
}
*/



int VideoLabeling::selectPopupMenu(bool IsPointSelected)
{


	QMenu popupMenu;
	QAction *deletePoint,*deleteLabel,*deleteContour,*deleteContourForward,*deleteContourBackward,*rename,*changeDepth,*selected,*changeBoundaryBlurForRegion,*changeMedianBlurForRegion;
	QAction *fillRegionWithStrokes,*fillRegionWithStrokesLayers,*deleteGeneratedStrokesForRegion,*deleteUserStrokesForRegion,*deleteUserStrokesForAllRegion;
	QAction *replicateUserStrokesForRegion,*selectUserStrokesForRegion,*assignUserStrokesToRegion;
	QAction *deleteStrokeForward,*deleteStrokeBackward,*deleteStroke;
	QAction *forwardTrackOneFrame,*forwardTrackFiveFrames,*forwardTrackTenFrames,*forwardTrackAll;
	QAction *backwardTrackOneFrame,*backwardTrackFiveFrames,*backwardTrackTenFrames,*backwardTrackAll,*trackAll;
	QAction* forwardReplicateOneFrame,*forwardReplicateFiveFrames,*forwardReplicateTenFrames,*forwardReplicateAll;
	QAction* backwardReplicateOneFrame=NULL,*backwardReplicateFiveFrames=NULL,*backwardReplicateTenFrames=NULL,*backwardReplicateAll=NULL,*replicateAll=NULL;
	QAction *enforceKeyFrame=NULL,*removeUserLabels=NULL;
	QAction *cutLabeling=NULL,*removeAllTracking=NULL;
	// these are the actions for the boundingbox objects
	QAction *cutForward=NULL,*cutBackward=NULL;

	if(IsPointSelected)
	{
		deletePoint=popupMenu.addAction(tr("Delete &Point"));
		deleteLabel=popupMenu.addAction(tr("Delete &Label"));
	}

	
	if (IsBoundarySelected)
	{
		
		deleteContour=popupMenu.addAction(tr("Delete Region"));
		deleteContourForward=popupMenu.addAction(tr("Delete Region Forward"));
		deleteContourBackward=popupMenu.addAction(tr("Delete Region Backward"));

		rename=popupMenu.addAction(tr("&Rename Region..."));
		changeDepth=popupMenu.addAction(tr("&Change depth"));

		popupMenu.addSeparator();
		fillRegionWithStrokes=popupMenu.addAction(tr("Generate Stroke Layer"));
		fillRegionWithStrokesLayers=popupMenu.addAction(tr("Generate Stroke Layers From List"));

		//changeBoundaryBlurForRegion=popupMenu.addAction(tr("Change Boundary Blur For Region"));
		//changeMedianBlurForRegion=popupMenu.addAction(tr("Change Median Filter For Region"));
		deleteGeneratedStrokesForRegion=popupMenu.addAction(tr("Delete Generated Strokes For Region"));
		deleteUserStrokesForRegion=popupMenu.addAction(tr("Delete User Strokes For Region"));
		deleteUserStrokesForAllRegion=popupMenu.addAction(tr("Delete User Strokes For All Regions"));
		replicateUserStrokesForRegion=popupMenu.addAction(tr("Replicate User Strokes"));
		selectUserStrokesForRegion=popupMenu.addAction(tr("Select All User Strokes"));
		assignUserStrokesToRegion=popupMenu.addAction(tr("Assign Selected Strokes to Region"));
	}
	else
	{
		deleteStroke=popupMenu.addAction(tr("Delete Stroke"));
		deleteStrokeForward=popupMenu.addAction(tr("Delete Stroke Forward"));
		deleteStrokeBackward=popupMenu.addAction(tr("Delete Stroke Backward"));
		
	}

	if(curFrameIndex!=m_LayerObjectList[selectedLayerIndex].createdFrame())
	{
		popupMenu.addSeparator();
		enforceKeyFrame=popupMenu.addAction(tr("Enforce this frame to key frame"));
		removeUserLabels=popupMenu.addAction(tr("Remove all the user labels"));
	}
	popupMenu.addSeparator();

	//if(m_LayerObjectList[selectedLayerIndex].isPolygon(curFrameIndex))
	//{
		// the tracking subMenus
		bool IsForwardTracking=false;
		QMenu* forwardTrackingMenu;
		if(m_LayerObjectList[selectedLayerIndex].IsForwardTrackable(curFrameIndex))
		{
			forwardTrackingMenu=popupMenu.addMenu("Forward tracking");
			forwardTrackOneFrame=forwardTrackingMenu->addAction(tr("1 frame"));
			forwardTrackFiveFrames=forwardTrackingMenu->addAction(tr("5 frames"));
			forwardTrackTenFrames=forwardTrackingMenu->addAction(tr("10 frames"));
			forwardTrackAll=forwardTrackingMenu->addAction(tr("All frames"));
			IsForwardTracking=true;
		}
		bool IsBackwardTracking=false;
		QMenu* backwardTrackingMenu;
		if(m_LayerObjectList[selectedLayerIndex].IsBackwardTrackable(curFrameIndex))
		{
			backwardTrackingMenu=popupMenu.addMenu("Backward tracking");
			backwardTrackOneFrame=backwardTrackingMenu->addAction(tr("1 frame"));
			backwardTrackFiveFrames=backwardTrackingMenu->addAction(tr("5 frames"));
			backwardTrackTenFrames=backwardTrackingMenu->addAction(tr("10 frames"));
			backwardTrackAll=backwardTrackingMenu->addAction(tr("All frames"));
			IsBackwardTracking=true;
		}
		bool IsTrackAll=false;
		if(m_LayerObjectList[selectedLayerIndex].endFrame()<numFrames-1 || m_LayerObjectList[selectedLayerIndex].startFrame()>0)
		{
			trackAll=popupMenu.addAction(tr("Track all the frames"));
			IsTrackAll=true;
		}
		if(IsForwardTracking || IsBackwardTracking || IsTrackAll)
			popupMenu.addSeparator();

		// the replicating subMenus
		bool IsForwardReplicate=false;
		QMenu* forwardReplicateMenu;
		if(m_LayerObjectList[selectedLayerIndex].IsForwardTrackable(curFrameIndex))
		{
			forwardReplicateMenu=popupMenu.addMenu("Forward replicate");
			forwardReplicateOneFrame=forwardReplicateMenu->addAction(tr("1 frame"));
			forwardReplicateFiveFrames=forwardReplicateMenu->addAction(tr("5 frames"));
			forwardReplicateTenFrames=forwardReplicateMenu->addAction(tr("10 frames"));
			forwardReplicateAll=forwardReplicateMenu->addAction(tr("All frames"));
			IsForwardReplicate=true;
		}
		bool IsBackwardReplicate=false;
		QMenu* backwardReplicateMenu;
		if(m_LayerObjectList[selectedLayerIndex].IsBackwardTrackable(curFrameIndex))
		{
			backwardReplicateMenu=popupMenu.addMenu("Backward replicate");
			backwardReplicateOneFrame=backwardReplicateMenu->addAction(tr("1 frame"));
			backwardReplicateFiveFrames=backwardReplicateMenu->addAction(tr("5 frames"));
			backwardReplicateTenFrames=backwardReplicateMenu->addAction(tr("10 frames"));
			backwardReplicateAll=backwardReplicateMenu->addAction(tr("All frames"));
			IsBackwardReplicate=true;
		}
		bool IsReplicateAll=false;
		if(m_LayerObjectList[selectedLayerIndex].endFrame()<numFrames-1 || m_LayerObjectList[selectedLayerIndex].startFrame()>0)
		{
			replicateAll=popupMenu.addAction(tr("Replicate to the rest"));
			IsReplicateAll=true;
		}
		if(IsForwardReplicate || IsBackwardReplicate || IsReplicateAll)
			popupMenu.addSeparator();

		// actions to cut the labeling
		bool IsCutLabeling=false;
		if(m_LayerObjectList[selectedLayerIndex].createdFrame()!=curFrameIndex)
		{
			cutLabeling=popupMenu.addAction(tr("Cut labeling beyond this frame"));
			IsCutLabeling=true;
		}
		bool IsRemoveAllTracking=false;;
		if(m_LayerObjectList[selectedLayerIndex].endFrame()-m_LayerObjectList[selectedLayerIndex].startFrame()>0)
		{
			removeAllTracking=popupMenu.addAction(tr("Remove all trackings"));
			IsRemoveAllTracking=true;
		}

	//}
	
	//else
	//{
	//	cutForward=popupMenu.addAction(tr("Remove all the annotations before this frame"));
	//	cutBackward=popupMenu.addAction(tr("Remove all the annotations after this frame"));
	//}
	

	selected=popupMenu.exec(QCursor::pos());

	if(selected==NULL)
		return -1;

	if(IsPointSelected)
	{
		if(selected==deletePoint)
			return 0;
		if(selected==deleteLabel)
			return -2;
	}


	if(selected==enforceKeyFrame)
		return 30;
	if((IsBoundarySelected) && (selected==deleteContour))
		return 1;
	if((IsBoundarySelected) && (selected==rename))
		return 2;
	if((IsBoundarySelected) && (selected==changeDepth))
		return 3;
	if((IsBoundarySelected) && (selected==deleteContourForward))
		return 4;
	if((IsBoundarySelected) && (selected==deleteContourBackward))
		return 5;

	if((IsBoundarySelected==true)&& (selected==fillRegionWithStrokesLayers))
		return 199;
	if((IsBoundarySelected==true)&& (selected==fillRegionWithStrokes))
		return 200;

	if((IsBoundarySelected==true)&& (selected==deleteGeneratedStrokesForRegion))
		return 201;
	if((IsBoundarySelected==true)&& (selected==deleteUserStrokesForRegion))
		return 202;
	if((IsBoundarySelected==true)&& (selected==deleteUserStrokesForAllRegion))
		return 203;
	if((IsBoundarySelected==true)&& (selected==replicateUserStrokesForRegion))
		return 204;
	if((IsBoundarySelected==true)&& (selected==selectUserStrokesForRegion))
		return 205;
	if((IsBoundarySelected==true)&& (selected==assignUserStrokesToRegion))
		return 206;
	if ((IsBoundarySelected==false)&& (selected==deleteStroke))
		return 210;
	if ((IsBoundarySelected==false)&& (selected==deleteStrokeForward))
		return 211;
	if ((IsBoundarySelected==false)&& (selected==deleteStrokeBackward))
		return 212;
	if((IsBoundarySelected==true)&& (selected==changeBoundaryBlurForRegion))
		return 213;
	if((IsBoundarySelected==true)&& (selected==changeMedianBlurForRegion))
		return 214;
	if(m_LayerObjectList[selectedLayerIndex].isPolygon(curFrameIndex))
	{
		if(m_LayerObjectList[selectedLayerIndex].IsForwardTrackable(curFrameIndex))
		{
			if(selected==forwardTrackOneFrame)
				return 11;
			if(selected==forwardTrackFiveFrames)
				return 12;
			if(selected==forwardTrackTenFrames)
				return 13;
			if(selected==forwardTrackAll)
				return 17;

			if(selected==forwardReplicateOneFrame)
				return 51;
			if(selected==forwardReplicateFiveFrames)
				return 52;
			if(selected==forwardReplicateTenFrames)
				return 53;
			if(selected==forwardReplicateAll)
				return 57;

		}

		if(m_LayerObjectList[selectedLayerIndex].IsBackwardTrackable(curFrameIndex))
		{
			if(selected==backwardTrackOneFrame)
				return 21;
			if(selected==backwardTrackFiveFrames)
				return 22;
			if(selected==backwardTrackTenFrames)
				return 23;
			if(selected==backwardTrackAll)
				return 27;

			if(selected==backwardReplicateOneFrame)
				return 61;
			if(selected==backwardReplicateFiveFrames)
				return 62;
			if(selected==backwardReplicateTenFrames)
				return 63;
			if(selected==backwardReplicateAll)
				return 67;

		}





		if(selected==replicateAll)
			return 70;

		if(selected==trackAll)
			return 100;

		if(selected==cutLabeling)
			return 40;
		if(selected==removeAllTracking)
			return 41;

		if(selected==enforceKeyFrame)
			return 30;
		if(selected==removeUserLabels)
			return 31;


	}
	else
	{
		if(selected==cutForward)
			return 81;
		if(selected==cutBackward)
			return 82;
	}
	return -1;
}
/*
void VideoLabeling::ProcessPopupMenuSelection(int choice)
{
	QList<LayerDepth> layerDepthList;
	LayerDepth layerDepth;
	QString outputPathName;
	switch(choice){
		case 3: // change the depth
			for(int i=0;i<m_LayerObjectList.size();i++)
			{
				m_LayerObjectList[i].FormLayerDepth(curFrameIndex,layerDepth);
				layerDepthList.append(layerDepth);
			}
			emit changeLayerDepth(layerDepthList,selectedLayerIndex,curFrameIndex);
			m_LayerObjectList[selectedLayerIndex].extendDepth();
			break;
		case 100:
			outputPathName=QFileDialog::getExistingDirectory(pMainWindow,"Specify path for output layer mask",framePath);
			if(!outputPathName.isEmpty())
				outputLayerMask(outputPathName);
			break;
		case 101:
			outputPathName=QFileDialog::getExistingDirectory(pMainWindow,"Specify path for output visible layer mask",framePath);
			if(!outputPathName.isEmpty())
				outputVisibleLayerMask(outputPathName);
			break;
	}
}
*/


void VideoLabeling::ProcessPopupMenuSelection(int choice)
{
	QString objname;
	QList<LayerDepth> layerDepthList;
	LayerDepth layerDepth;
	int tempCurFrameIndex;
	//cout<<choice<<endl;
	switch(choice){
		case 0: // delete point
			m_LayerObjectList[selectedLayerIndex].removePoint(curFrameIndex,IsBoundarySelected);
			labelState=Selected;
			emit redrawStrokes();
			emit strokesChanged(curFrameIndex);
			emit Update();
			break;
		case -2:// delete label
			m_LayerObjectList[selectedLayerIndex].removeLabel(curFrameIndex);
			labelState=Selected;
			emit Update();
			break;
		case 1: // delete the contour

			if(QMessageBox::question(pMainWindow,"Warning!","Are you sure deleting this region?",QMessageBox::Yes,QMessageBox::No)==QMessageBox::No)
				return;

			IsLayerMaskGenerated=false;
			m_LayerObjectList.removeAt(selectedLayerIndex);
			selectedLayerIndex=0;
			labelState=Idle;
			emit Update();
			break;


		case 2: // rename the contour
			objname=QInputDialog::getText(pMainWindow,"Annotation","Rename the object:",QLineEdit::Normal,m_LayerObjectList[selectedLayerIndex].layerName(),0,Qt::Dialog);
			if(!objname.isEmpty())
				m_LayerObjectList[selectedLayerIndex].setLayerName(objname);
			break;
		case 3: // change the depth
			for(int i=0;i<m_LayerObjectList.size();i++)
			{
				m_LayerObjectList[i].FormLayerDepth(curFrameIndex,layerDepth);
				layerDepthList.append(layerDepth);
			}
			emit changeLayerDepth(layerDepthList,selectedLayerIndex,curFrameIndex);
			m_LayerObjectList[selectedLayerIndex].extendDepth();
                        IsLayerMaskGenerated=false;
			break;
		case 4: // delete the contour

			if(QMessageBox::question(pMainWindow,"Warning!","Are you sure deleting this region forward?",QMessageBox::Yes,QMessageBox::No)==QMessageBox::No)
				return;
			IsLayerMaskGenerated=false;
			m_LayerObjectList[selectedLayerIndex].removeFrames(curFrameIndex,true);
			emit Update();
			break;
		case 5: // delete the contour

			if(QMessageBox::question(pMainWindow,"Warning!","Are you sure deleting this region backward?",QMessageBox::Yes,QMessageBox::No)==QMessageBox::No)
				return;
			IsLayerMaskGenerated=false;
			m_LayerObjectList[selectedLayerIndex].removeFrames(curFrameIndex,false);
			emit Update();
			break;
		case 11:// forward track one frame
			TrackContour(true,1);
			break;
		case 12:// forward track five frames
			TrackContour(true,5);
			break;
		case 13:// forward track five frames
			TrackContour(true,10);
			break;
		case 17:// forward track all
			curFrameIndex=m_LayerObjectList[selectedLayerIndex].endFrame();
			gotoFrame(curFrameIndex+1);
			QApplication::processEvents();
			TrackContour(true,numFrames-1-curFrameIndex);
			m_LayerObjectList[selectedLayerIndex].printGlobalLabels();
			break;
		case 21://backward track one frame
			TrackContour(false,1);
			break;
		case 22://backward track five frames
			TrackContour(false,5);
			break;
		case 23://backward track five frames
			TrackContour(false,10);
			break;
		case 27: //backward track all
			curFrameIndex=m_LayerObjectList[selectedLayerIndex].startFrame();
			gotoFrame(curFrameIndex+1);
			QApplication::processEvents();
			TrackContour(false,curFrameIndex);
			break;
		case 100:// track all the frames;
			tempCurFrameIndex=curFrameIndex;
			if(m_LayerObjectList[selectedLayerIndex].endFrame()<numFrames-1)
			{
				curFrameIndex=m_LayerObjectList[selectedLayerIndex].endFrame();
				gotoFrame(curFrameIndex+1);
				QApplication::processEvents();
				TrackContour(true,numFrames-1-curFrameIndex);
			}
			if(m_LayerObjectList[selectedLayerIndex].startFrame()>0)
			{
				curFrameIndex=m_LayerObjectList[selectedLayerIndex].startFrame();
				gotoFrame(curFrameIndex+1);
				QApplication::processEvents();
				TrackContour(false,curFrameIndex);
			}
			break;
		case 51: // forward replicate one frame
			ReplicateContour(true,curFrameIndex+1);
			break;
		case 52: // forward replicate five frames
			ReplicateContour(true,curFrameIndex+5);
			break;
		case 53: // forward replicate ten frames
			ReplicateContour(true,curFrameIndex+10);
			break;
		case 57: // forward replicate all the frames
			ReplicateContour(true,numFrames-1);
			break;
		case 61: // backward replicate one frame
			ReplicateContour(false,curFrameIndex-1);
			break;
		case 62: // backward replicate five frames
			ReplicateContour(false,curFrameIndex-5);
			break;
		case 63: // backward replicate ten frames
			ReplicateContour(false,curFrameIndex-10);
			break;
		case 67: // backward replicate all the frames
			ReplicateContour(false,0);
			break;
		case 70: // replicate all the frames
			m_LayerObjectList[selectedLayerIndex].StretchToFrame(0);
			m_LayerObjectList[selectedLayerIndex].StretchToFrame(numFrames-1);
			break;
		case 40:// cut labeling
			m_LayerObjectList[selectedLayerIndex].cutLabeling(curFrameIndex);
			break;
		case 41:// remove all the trackings
			m_LayerObjectList[selectedLayerIndex].removeAllTracking();
			if(m_LayerObjectList[selectedLayerIndex].createdFrame()!=curFrameIndex)
			{
				labelState=Idle;
				emit Update();
			}
			break;
		case 30://enforce the current frame to be a key frame
			m_LayerObjectList[selectedLayerIndex].enforceKeyFrame(curFrameIndex,IsBoundarySelected);
			emit Update();
			break;
		case 31: // remove the user labels
			m_LayerObjectList[selectedLayerIndex].removeUserLabels(curFrameIndex);
			emit Update();
		case 81:
			m_LayerObjectList[selectedLayerIndex].cutForward(curFrameIndex);
			emit Update();
			break;
		case 82:
			m_LayerObjectList[selectedLayerIndex].cutBackward(curFrameIndex);
			emit Update();
			break;

		case 199: //fill region with strokes
			generateStrokeLayersFromList();
			break;
		case 200: //fill region with strokes
			generateStrokeLayer();
			break;
		case 201: //delete system strokes for current region
			deleteStrokeLayers();
			break;
		case 202: //delete user strokes for current region
			m_CurrDeleteId++;
			for(int j=0;j<numFrames;j++)
			{
				m_LayerObjectList[selectedLayerIndex].deleteUserStrokes(j,m_CurrDeleteId);
			}
			emit strokesChanged(curFrameIndex);
			emit Update();
			break;
		case 213:
			changeRegionBoundaryBlur();
			break;
		case 214:
			changeRegionMedianBlur();
			break;
		case 203: //delete user strokes for all regions
			m_CurrDeleteId++;
			for(int j=0;j<numFrames;j++)
			{
				for(int i=0;i<m_LayerObjectList.size();i++)
				{
					m_LayerObjectList[i].deleteUserStrokes(j,m_CurrDeleteId);
					
				}
			}
			emit strokesChanged(curFrameIndex);
			emit Update();
			break;
		case 204: //track user strokes for current region
			replicateSelectedStrokes();
			break;
		case 205: //track user strokes for current region
			selectAllRegionStrokes();
			break;
		case 206: //track user strokes for current region
			assignStrokesToRegion();
			break;
		case 210: 
			if (IsBoundarySelected == false)
			{
				int firstFrame = numFrames, lastFrame = 0;
				m_CurrDeleteId++;
				m_LayerObjectList[selectedLayerIndex].deleteCurrentStroke(curFrameIndex, m_CurrDeleteId,0,numFrames-1,&firstFrame,&lastFrame);

				for (int i=firstFrame;i <= lastFrame;i++)
					emit strokesChanged(i);
			}
			break;
		case 211:
			if (IsBoundarySelected == false)
			{
				int firstFrame = numFrames, lastFrame = 0;
				m_CurrDeleteId++;
				m_LayerObjectList[selectedLayerIndex].deleteCurrentStroke(curFrameIndex, m_CurrDeleteId,curFrameIndex,numFrames-1,&firstFrame,&lastFrame);

				for (int i=firstFrame;i <= lastFrame;i++)
					emit strokesChanged(i);
			}
			break;
		case 212:
			if (IsBoundarySelected == false)
			{
				int firstFrame = numFrames, lastFrame = 0;
				m_CurrDeleteId++;
				m_LayerObjectList[selectedLayerIndex].deleteCurrentStroke(curFrameIndex, m_CurrDeleteId,0,curFrameIndex+1,&firstFrame,&lastFrame);

				for (int i=firstFrame;i <= lastFrame;i++)
					emit strokesChanged(i);
			}


	}
}

void VideoLabeling::ReplicateContour(bool direction,int destFrame)
{
	IsLayerMaskGenerated=false;
	m_LayerObjectList[selectedLayerIndex].StretchToFrame(destFrame);
	if(direction)
		gotoFrame(m_LayerObjectList[selectedLayerIndex].endFrame()+1);
	else
		gotoFrame(m_LayerObjectList[selectedLayerIndex].startFrame()+1);
}




void VideoLabeling::replicateSelectedStrokes()
{


	float factorSum = _imageFlowFactor+_boundaryFlowFactor+_userFlowFactor+ _fixedFlowFactor;

	bool replicatedStrokes = false;


	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].copySelectedStrokes(curFrameIndex);

		m_LayerObjectList[i].saveChangedStrokes(curFrameIndex, m_ChangeStrokeId+1,false);

		if (m_FillFramesBackward >0)
		{
			if (m_LayerObjectList[i].ReplicateStrokesToFrame(curFrameIndex,curFrameIndex-m_FillFramesBackward,m_ChangeStrokeId+1,frameList,_colorUpdate,_imageFlowFactor/factorSum,_boundaryFlowFactor/factorSum,_userFlowFactor/factorSum, _fixedFlowFactor/factorSum,_fixedMovementAngle,_fixedMovementRate, _followPreviousStrokesFactor, _followGradientFactor))
			{
				replicatedStrokes=true;
				 sprintf(_strbuf, "going to frame %i, %i %i", max(curFrameIndex-m_FillFramesBackward, m_LayerObjectList[i].startFrame()),curFrameIndex-m_FillFramesBackward,m_LayerObjectList[i].startFrame());
				LOG_STR_COUT(_strbuf);
			}
		}
		if (m_FillFramesForward >0)
		{
			if (m_LayerObjectList[i].ReplicateStrokesToFrame(curFrameIndex,curFrameIndex+m_FillFramesForward,m_ChangeStrokeId+1,frameList,_colorUpdate,_imageFlowFactor/factorSum,_boundaryFlowFactor/factorSum,_userFlowFactor/factorSum, _fixedFlowFactor/factorSum,_fixedMovementAngle,_fixedMovementRate, _followPreviousStrokesFactor, _followGradientFactor))
			{
				replicatedStrokes=true;
                sprintf(_strbuf, "going to frame %i, %i %i", min(curFrameIndex+m_FillFramesForward, m_LayerObjectList[i].endFrame()),curFrameIndex+m_FillFramesForward,m_LayerObjectList[i].endFrame());
				LOG_STR_COUT(_strbuf);
			}
		}
	}

	if (replicatedStrokes)
	{
		m_ActionHistory.push_back(ChangeStrokeAction);
		m_ChangeStrokeId++;
	}


    sprintf(_strbuf, "replicating strokes finished");
	LOG_STR_COUT(_strbuf);

	for (int i=max(0,curFrameIndex-m_FillFramesBackward); i < min(numFrames,curFrameIndex+m_FillFramesForward);i++)
	emit strokesChanged(i);
}

void VideoLabeling::selectAllRegionStrokes()
{
	deselectAllStrokes();

	m_LayerObjectList[selectedLayerIndex].selectMultipleContours(curFrameIndex, QPointF(1,1), 10000, true);

	emit showUserStrokeOptions(true);
	emit Update();
}

void VideoLabeling::assignStrokesToRegion()
{


	QList<QList<ContourLabel> > selectedStrokes;
	for(int i=0;i < numFrames;i++)
	{
		QList<ContourLabel> list;
		selectedStrokes.push_back(list);
	}
	bool anySelected=false;
	for (int i=0;i < m_LayerObjectList.size();i++)
	{
		if (m_LayerObjectList[i].getSelectedStrokes(curFrameIndex,selectedStrokes)>0)
			anySelected=true;	
	}


	m_CurrDeleteId++;
	m_ActionHistory.push_back(DeleteStrokesAction);

	int firstFrame =numFrames , lastFrame = 0;
    sprintf(_strbuf, "deleting selected strokes");
	LOG_STR_COUT(_strbuf);
	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].deleteSelectedStrokes(curFrameIndex,m_CurrDeleteId,0,numFrames-1, &firstFrame, &lastFrame);
	}

	//m_ActionHistory.push_back(ChangeStrokeAction);
	//m_ChangeStrokeId++;

	m_LayerObjectList[selectedLayerIndex].addNewSelectedStrokes(selectedStrokes);

	emit showUserStrokeOptions(true);
	emit Update();

}


void VideoLabeling::fixSelectedStrokes()
{

	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].fixSelectedStrokes(curFrameIndex);
	}

	emit Update();
}




void VideoLabeling::setLayerDepth(const LayerDepth &layerDepth)
{
	m_LayerObjectList[selectedLayerIndex].SetLayerDepth(layerDepth);
	emit Update();
}

//-------------------------------------------------------------------------------------------------------------------------
// file IO
//-------------------------------------------------------------------------------------------------------------------------
bool VideoLabeling::writeVideoLabelMeXML(const QString &filename)
{
	QFile file(filename);
	if(file.open(QIODevice::WriteOnly)==false)
		return false;

	// compute the relative path
	QString pathName,subFolder;
	QVideoProcessing::separatePath_FileNames(framePath,pathName,subFolder);

	XmlWriter xw(&file);
	xw.setAutoNewLine(true);

	xw.writeOpenTag("annotation");
		xw.writeTaggedString("version","1.00");
		xw.writeTaggedString("videoType","frames");
		
		xw.writeTaggedString("folder",framePath);
		xw.writeTaggedString("NumFrames",QString::number(numFrames));
		xw.writeTaggedString("currentFrame",QString::number(curFrameIndex));
		xw.writeOpenTag("fileList");
			for(int i=0;i<numFrames;i++)
				xw.writeTaggedString("fileName",fileList[i]);
		xw.writeCloseTag("fileList");
		xw.writeOpenTag("source");
			xw.writeTaggedString("sourceImage","videos taken by Antonio Torrabla");
			xw.writeTaggedString("sourceAnnotation","Qt-based C++ video labeling tool (by Ce Liu, Aug 2007)");
		xw.writeCloseTag("source");
		for(int i=0;i<m_LayerObjectList.size();i++)
			m_LayerObjectList[i].writeXML(xw,i);
	xw.writeCloseTag("annotation");
	return true;
}

bool VideoLabeling::readVideoLabelMeXML(const QString& filename)
{
	// read XML into buffer
	 QDomDocument doc("mydocument");
	 QFile file(filename);
	 if (!file.open(QIODevice::ReadOnly))
		 return false;
	 if (!doc.setContent(&file)) {
		 file.close();
		 return false;
	 }
	 file.close();

    createDirectoryStructure(filename,false);

	 // clear the member variables
	clear();



	 // 
	 QDomElement root=doc.documentElement();
	 QDomNode n=root.firstChild();
	 numFrames=0;
	 while(!n.isNull())
	 {
		 QDomElement e=n.toElement();
		 // parse the element at the top level
		 if(e.tagName()=="folder")
			 framePath=e.text();
		 if(e.tagName()=="NumFrames")
			 numFrames=e.text().toInt();
		 if(e.tagName()=="currentFrame")
			 curFrameIndex=e.text().toInt();

		 // load all the image file names
		 if(e.tagName()=="fileList")
		 {
			 QDomNode filelist_n=e.firstChild();
			 while(!filelist_n.isNull())
			 {
				 QDomElement filelist_e=filelist_n.toElement();
				 if(filelist_e.tagName()=="fileName")
					 fileList.append(filelist_e.text());
				 filelist_n=filelist_n.nextSibling();
			 }
			 if(numFrames==0)
				 numFrames=fileList.size();
			 // check whether  the files are found
			 if(QVideoProcessing::FileListExist(framePath,fileList)==false)
			 {
				 QString curFolderPath,temp,subFolderName;
				 QVideoProcessing::separatePath_FileNames(filename,curFolderPath,temp);
				 if(framePath.right(1)=="/")
					 framePath.chop(1);
				 QVideoProcessing::separatePath_FileNames(framePath,temp,subFolderName);
				 if(curFolderPath.right(1)!="/")
					 curFolderPath+="/";
				 if(QVideoProcessing::FileListExist(curFolderPath+subFolderName,fileList)==false)
					 return false;
				 framePath=curFolderPath+subFolderName+"/";
			 }
			if(framePath.right(1)!="/")
				 framePath+="/";
		 }

		 if(e.tagName()=="source") // so far does not parse source
		 {
		 }

		 if(e.tagName()=="object") // parse the layer object
		 {
			 QDomNode obj_n=e.firstChild();
			 LayerObject layerObject;
			 while(!obj_n.isNull())
			 {
				 QDomElement obj_e=obj_n.toElement();
				 if(obj_e.tagName()=="name")
					 layerObject.setLayerName(obj_e.text());
				 if(obj_e.tagName()=="deleted")
				 {
				 }
				 if(obj_e.tagName()=="verified")
				 {
				 }
				 if(obj_e.tagName()=="date")
				 {
				 }
				 if(obj_e.tagName()=="id")
				 {
				 }
				 if(obj_e.tagName()=="numFrames")
					 layerObject.create(obj_e.text().toInt());
				 if(obj_e.tagName()=="startFrame")
					 layerObject.setStartFrame(obj_e.text().toInt());
				 if(obj_e.tagName()=="endFrame")
					 layerObject.setEndFrame(obj_e.text().toInt());
				 if(obj_e.tagName()=="createdFrame")
					 layerObject.setCreatedFrame(obj_e.text().toInt());
				 if(obj_e.tagName()=="labels")
				 {
					 // load the labels
					 QDomNode label_n=obj_e.firstChild();
					 while(!label_n.isNull())
					 {
						 int frameIndex;
						 QDomElement label_e=label_n.toElement();
						 if(label_e.tagName()=="frame")
						 {
							 QDomNode frame_n=label_e.firstChild();
							 while(!frame_n.isNull())
							 {
								 QDomElement frame_e=frame_n.toElement();

								 if(frame_e.tagName()=="index")
									 frameIndex=frame_e.text().toInt();
								 if(frame_e.tagName()=="depth")
									layerObject.setDepth(frameIndex,frame_e.text().toDouble());
								 if(frame_e.tagName()=="globalLabel")
									 layerObject.setGlobalLabel(frameIndex,frame_e.text().toInt());
								 if(frame_e.tagName()=="localLabel")
									 layerObject.setLocalLabel(frameIndex,frame_e.text().toInt());
								 if(frame_e.tagName()=="tracked")
									 layerObject.setTracked(frameIndex,frame_e.text().toInt());
								 if(frame_e.tagName()=="depthLabel")
									 layerObject.setDepthLabel(frameIndex,frame_e.text().toInt());
								 if(frame_e.tagName()=="polygon") // read the polygon
								 {
									 ContourLabel contourLabel;
									 QDomNode contour_n=frame_e.firstChild();
									 while(!contour_n.isNull())
									 {
										 QPointF point(0,0);
										 bool isLabeled=true;
										 QDomElement contour_e=contour_n.toElement();
										 // add one point
										 if(contour_e.tagName()=="pt")
										 {
											 QDomNode pt_n=contour_e.firstChild();
											 while(!pt_n.isNull())
											 {
												 QDomElement pt_e=pt_n.toElement();
												 if(pt_e.tagName()=="x")
													 point.setX(pt_e.text().toDouble());
												 if(pt_e.tagName()=="y")
													 point.setY(pt_e.text().toDouble());
												 if(pt_e.tagName()=="labeled")
													 isLabeled=pt_e.text().toInt();
												 pt_n=pt_n.nextSibling();
											 }
											 contourLabel.addPoint(point,isLabeled);
										 }
										contour_n=contour_n.nextSibling();
									 }
									layerObject.setContourLabel(frameIndex,contourLabel);
								 } // end of reading the polygon
								 if(frame_e.tagName()=="boundingBox")
								 {
									 ContourLabel contourLabel;
									 contourLabel.setIsPolygon(false);
									 QRectF boundingBox;
									 QDomNode contour_n=frame_e.firstChild();
									 while(!contour_n.isNull())
									 {
										 QPointF point(0,0);
										 bool isLabeled=true;
										 QDomElement contour_e=contour_n.toElement();
										 // add one point
										 if(contour_e.tagName()=="topLeft")
										 {
											 QDomNode pt_n=contour_e.firstChild();
											 while(!pt_n.isNull())
											 {
												 QDomElement pt_e=pt_n.toElement();
												 if(pt_e.tagName()=="x")
													 point.setX(pt_e.text().toDouble());
												 if(pt_e.tagName()=="y")
													 point.setY(pt_e.text().toDouble());
												 pt_n=pt_n.nextSibling();
											 }
											 boundingBox.setTopLeft(point);
										 }
										 if(contour_e.tagName()=="bottomRight")
										 {
											 QDomNode pt_n=contour_e.firstChild();
											 while(!pt_n.isNull())
											 {
												 QDomElement pt_e=pt_n.toElement();
												 if(pt_e.tagName()=="x")
													 point.setX(pt_e.text().toDouble());
												 if(pt_e.tagName()=="y")
													 point.setY(pt_e.text().toDouble());
												 pt_n=pt_n.nextSibling();
											 }
											 boundingBox.setBottomRight(point);
										 }
										contour_n=contour_n.nextSibling();
									 }
									contourLabel.setBoundingBox(boundingBox);
									layerObject.setContourLabel(frameIndex,contourLabel);
								 }
								 frame_n=frame_n.nextSibling();
							 }
						 }// end of reading the frames
						 label_n=label_n.nextSibling();
					 }// end of reading the labels
				 }
				 obj_n=obj_n.nextSibling();
			 } // end of parsing all the element in one layer

			 layerObject.extendDepth();
			 m_LayerObjectList.append(layerObject);
		 } // end of looping over layers

		 // end parsing the element at the top level
		 n=n.nextSibling();
	 }
	 int tempFrameIndex=curFrameIndex;
	 emit NumFramesChanged(numFrames);
	 curFrameIndex=tempFrameIndex;
	 gotoFrame(curFrameIndex+1);
	 IsVideoLoaded=true;
	 return true;
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// function to track contours
//	 Input arguments
//		direction-- the tracking direction, true: forward, false: backward
//	    nFrames-- the number of frames to track
//-----------------------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::TrackContour(bool direction,int nFrames)
{
	if((curFrameIndex==0 && direction==false) || (curFrameIndex==numFrames-1 && direction==true))
		return;
	if(curFrameIndex<m_LayerObjectList[selectedLayerIndex].endFrame() && curFrameIndex>m_LayerObjectList[selectedLayerIndex].startFrame())
		return;
	IsMouseAllowed=false;
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

	IsLayerMaskGenerated=false;

	// get the contour of the current object at the current frame
	QList<QPointF> contour=m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).PointList();
	// display the contour
	//for(int i=0;i<contour.size();i++)
	//	cout<<contour[i].x()<<" "<<contour[i].y()<<endl;

	BiImage frame1(m_Image),frame2;
	ImPyramid CurFramePyrd;
	QPointF offset1;
	PyrdPatchSet pyrdPatchSet1,pyrdPatchSet2;

	// build image pyramid and sample the patches
	offset1=ContourTracking::CreateCroppedPyramid(CurFramePyrd,frame1,contour,MyPreference::nPyrdLevels);
	ContourTracking::SubtractOffset(contour,offset1);
	pyrdPatchSet1.CreatePyrdPatchSet(CurFramePyrd,contour);
	ContourTracking::AddOffset(contour,offset1);

	// code for debugging
	//pyrdPatchSet1.dispPyrdWeights("Weight");

	// prepare for tracking
	int nPoints=contour.size();
	//QList<QPointF> trackedContour(QList<QPointF>::fromVector(QVector<QPointF>(nPoints)));
	QList<QPointF> trackedContour;
	for(int i=0;i < nPoints;i++)
	{
		trackedContour.push_back(QPointF(0,0));
	}

	QList<bool> IsVisibleList(QList<bool>::fromVector(QVector<bool>(nPoints,true)));
	QList<QPointF> prediction;


	prediction=contour;
	OccludingReasoning(IsVisibleList,prediction);
	//pyrdPatchSet1.dispPyrdWeights("Weight",6);
	//pyrdPatchSet1.dispPyrdPatches("Patch",6);
	//pyrdPatchSet1.dispPyrdFeatures("Feature",6);

	QList<QPointF> vHistory; // the list of velocity
	// start tracking
	for(int k=1;k<=nFrames;k++)
	{
		// check whether frame beyond the scope of the video needs to be loaded
		if((curFrameIndex==0 && direction==false) || (curFrameIndex==numFrames-1 && direction==true))
			break;

		// goto the next frame
		if(direction)
			curFrameIndex++;
		else
			curFrameIndex--;
		
		// load the next image
		if(m_Image.load(framePath+fileList[curFrameIndex])==false)
			return;
		frame2.imread(m_Image);

		// compute the visibility of the prediction
		OccludingReasoning(IsVisibleList,prediction);
		bool IsWholeOccluded=true;
		for(int i=0;i<IsVisibleList.size();i++)
			if(IsVisibleList[i]==true)
			{
				IsWholeOccluded=false;
				break;
			}
		/*
		if(IsWholeOccluded==true)
		{
			m_LayerObjectList[selectedLayerIndex].TrackToFrame(curFrameIndex,prediction);
			emit ImageChanged(m_Image);
			updateFrameControl();
			emit Update();
			QApplication::processEvents();
			break;
		}
		*/
		//for(int i=0;i<IsVisibleList.size();i++)
		//	cout<<(int)prediction[i].x()<<" "<<(int)prediction[i].y()<<" "<<IsVisibleList[i]<<endl;
		//cout<<endl;
		
		// the tracking procedure
		if(k==1)
			ContourTracking::TrackOneFrameCropped(trackedContour,pyrdPatchSet2,contour,prediction,IsVisibleList,pyrdPatchSet1,frame2,0.002,6,16); // extra margin for the first time calling
		else
			ContourTracking::TrackOneFrameCropped(trackedContour,pyrdPatchSet2,contour,prediction,IsVisibleList,pyrdPatchSet1,frame2,0.002,6,10); 
		
		// prepare for the next tracking and update the contour
		// meanwhile, compute the velocity and check whether the tracking is totally wrong
		QPointF velocity(0,0);
		for(int i=0;i<nPoints;i++)
		{
			prediction[i]=trackedContour[i]*2-contour[i];
			QPointF diff=trackedContour[i]-contour[i];
			velocity+=diff;
		}
		velocity/=nPoints;

		contour=trackedContour;
		//pyrdPatchSet1.copyData(pyrdPatchSet2);
		pyrdPatchSet1.updateData(pyrdPatchSet2,0.7);

		// update the layer object information
		m_LayerObjectList[selectedLayerIndex].TrackToFrame(curFrameIndex,contour);

		// update the controller and draw the tracked contour
		emit ImageChanged(m_Image);
		updateFrameControl();
		emit Update();
		QApplication::processEvents();

		// check wether the tracking is valid
		if(vHistory.size()==0)
		{
			double temp=sqrt(velocity.x()*velocity.x()+velocity.y()*velocity.y());
			if(temp>MyPreference::Threshold_MaxTrackingVelocity)  // it's unlikely the object moves beyond a certain velocity
				break;
		}
		else
		{
			QPointF acceleration;
			acceleration=velocity-vHistory.last();
			double temp=sqrt(acceleration.x()*acceleration.x()+acceleration.y()*acceleration.y());
			if(temp>MyPreference::Threshold_MaxTrackingAcceleration)
				break;
		}
		vHistory.append(velocity);
		m_LayerObjectList[selectedLayerIndex].setGlobalLabel(curFrameIndex,false);
	}
	IsMouseAllowed=true;
	QApplication::restoreOverrideCursor();
}




//-----------------------------------------------------------------------------------------------------------------------------------------
// function to derive whether the contour points are visible
//-----------------------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::OccludingReasoning(QList<bool> &IsVisibleList,const QList<QPointF>& contour)
{
        QVector<QList<QPointF> > OccludingContours;
	for(int i=0;i<m_LayerObjectList.size();i++)
	{
		// if this contour, continue;
		if(i==selectedLayerIndex)
			continue;
		// if the current frame is not in the life of the contour object
		if(m_LayerObjectList[i].endFrame()<curFrameIndex || m_LayerObjectList[i].startFrame()>curFrameIndex)
			continue;
		if(m_LayerObjectList[i].depth(curFrameIndex)<m_LayerObjectList[selectedLayerIndex].depth(curFrameIndex))
			OccludingContours.append(m_LayerObjectList[i].contourLabel(curFrameIndex).PointList());
	}
	ContourTracking::OcclusionReasoning(IsVisibleList,contour,OccludingContours,m_Image.width(),m_Image.height(),7);
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// function to derive whether a point is visible
//-----------------------------------------------------------------------------------------------------------------------------------------
bool VideoLabeling::OccludingReasoning(const QPointF& point)
{
	if(point.x()<0 || point.x()>m_Image.width()-1 || point.y()<0 || point.y()>m_Image.height()-1)
		return false;
        QVector<QList<QPointF> > OccludingContours;
	for(int i=0;i<m_LayerObjectList.size();i++)
	{
		// if this contour, continue;
		if(i==selectedLayerIndex)
			continue;
		// if the current frame is not in the life of the contour object
		if(m_LayerObjectList[i].endFrame()<curFrameIndex || m_LayerObjectList[i].startFrame()>curFrameIndex)
			continue;
		if(m_LayerObjectList[i].depth(curFrameIndex)<m_LayerObjectList[selectedLayerIndex].depth(curFrameIndex))
			OccludingContours.append(m_LayerObjectList[i].contourLabel(curFrameIndex).PointList());
	}
	if(OccludingContours.size()==0)
		return true;
	QList<bool> IsVisibleList;
	IsVisibleList.append(true);
	QList<QPointF> contour;
	contour.append(point);
	ContourTracking::OcclusionReasoning(IsVisibleList,contour,OccludingContours,m_Image.width(),m_Image.height(),7);
	return IsVisibleList[0];
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// function to propagate the point that has just been updated
//-----------------------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::propagateUpdatePoint()
{
	int lastLabelFrame,firstLabelFrame;
	bool IsLastFrameLabeled,IsFirstFrameLabeled;
	int pointIndex=m_LayerObjectList[selectedLayerIndex].SelectedPointIndex();
	lastLabelFrame=findLastLocalLabeledFrame(curFrameIndex,pointIndex,IsLastFrameLabeled);
	firstLabelFrame=findFirstLocalLabeledFrame(curFrameIndex,pointIndex,IsFirstFrameLabeled);
	// first only do goemetric propagation
	if(OccludingReasoning(m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).getPoint(pointIndex))==false)
	{
		m_LayerObjectList[selectedLayerIndex].propagateUpdatePoint(curFrameIndex,lastLabelFrame,IsLastFrameLabeled,firstLabelFrame,IsFirstFrameLabeled);
		return;
	}
	// forward tracking
	if(lastLabelFrame-curFrameIndex-1>0)
		TrackPoint(pointIndex,true,lastLabelFrame-curFrameIndex-1,lastLabelFrame);
	// backward tracking
	if(curFrameIndex-firstLabelFrame-1>0)
		TrackPoint(pointIndex,false,curFrameIndex-firstLabelFrame-1,firstLabelFrame);
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// function to track a particular point from curFrameIndex, forward (direction=true) or backward (direction=false), for
// nFrames.
//-----------------------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::TrackPoint(int pointIndex,bool direction, int nFrames,int destFrame)
{
	if((curFrameIndex==0 && direction==false) || (curFrameIndex==numFrames-1 && direction==true))
		return;
	IsMouseAllowed=false;
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

	// get the point of the current object at the current frame
	QPointF point=m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).getPoint(pointIndex);


	BiImage frame1(m_Image),frame2;
	ImPyramid CurFramePyrd;
	QPointF offset1;
	PyrdPatchSet pyrdPatchSet1,pyrdPatchSet2;

	// build image pyramid and sample the patches
	QList<QPointF> contour;
	contour.append(point);
	offset1=ContourTracking::CreateCroppedPyramid(CurFramePyrd,frame1,contour,4,20);
	ContourTracking::SubtractOffset(contour,offset1);
	QList<QPointF> tempContour(m_LayerObjectList[selectedLayerIndex].contourLabel(curFrameIndex).PointList());
	ContourTracking::SubtractOffset(tempContour,offset1);
	pyrdPatchSet1.CreatePyrdPatchSet(CurFramePyrd,point-offset1,pointIndex,tempContour);
	ContourTracking::AddOffset(contour,offset1);

	//pyrdPatchSet1.dispPyrdWeights("weight1_",1);
	//pyrdPatchSet1.dispPyrdPatches("patch1_",1);

	// prepare for tracking
	QPointF	prediction,tracked;
	int frameIndex=curFrameIndex;

	// start tracking
	double alpha;

	// for debug
	QPointF refPoint=m_LayerObjectList[selectedLayerIndex].contourLabel(10).getPoint(pointIndex);

	for(int k=1;k<=nFrames;k++)
	//for(int k=1;k<=1;k++)
	{
		// check whether frame beyond the scope of the video needs to be loaded
		if((curFrameIndex==0 && direction==false) || (curFrameIndex==numFrames-1 && direction==true))
			break;

		// goto the next frame
		if(direction)
			frameIndex++;
		else
			frameIndex--;
		// get the prediction
		prediction=m_LayerObjectList[selectedLayerIndex].contourLabel(frameIndex).getPoint(pointIndex);
		// load the next image
		if(frame2.imread(framePath+fileList[frameIndex])==false)
			return;

		// compute the visibility of the prediction
		if(OccludingReasoning(prediction)==false)
		{
			if(direction==true)
				m_LayerObjectList[selectedLayerIndex].propagateUpdatePointForward(frameIndex-1,destFrame);
			else
				m_LayerObjectList[selectedLayerIndex].propagateUpdatePointBackward(frameIndex+1,destFrame);
			break;
		}

		tracked=ContourTracking::TrackPointOneFrameCropped(pyrdPatchSet2,point,prediction,pointIndex,m_LayerObjectList[selectedLayerIndex].contourLabel(frameIndex).PointList(),pyrdPatchSet1,frame2,0.00001,5,25);
		//pyrdPatchSet2.dispPyrdWeights("weight2_",1);
		//pyrdPatchSet2.dispPyrdPatches("patch2_",1);

		// prepare for the next tracking and update the point
		if(nFrames>1)
			alpha=(double)(k-1)/(nFrames-1);
		else
			alpha=0;
		QPointF updated=tracked*(1-alpha)+prediction*alpha;
		m_LayerObjectList[selectedLayerIndex].contourLabel(frameIndex).updatePoint(updated,pointIndex,false);
		point=tracked;
		pyrdPatchSet1.updateData(pyrdPatchSet2,0.6);

	}
	IsMouseAllowed=true;
	QApplication::restoreOverrideCursor();
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// function to find the last labeled frame
// If the current point is visible, and it becomes invisible and visible again before reaching the last labeled frame, the last
// invisible frame is the last labeled frame
// If the current point is invisible, and it becomes visible before reaching the last labeled frame, the last invisible frame is 
// the last labeled frame
//-----------------------------------------------------------------------------------------------------------------------------------------
int VideoLabeling::findLastLocalLabeledFrame(int frameIndex,int pointIndex,bool &IsLastFrameLabeled)
{
	if(frameIndex==m_LayerObjectList[selectedLayerIndex].endFrame())
	{
		IsLastFrameLabeled=m_LayerObjectList[selectedLayerIndex].contourLabel(frameIndex).labeled(pointIndex);
		return frameIndex;
	}
	int i;
	IsLastFrameLabeled=false;
	bool IsVisible0=OccludingReasoning(m_LayerObjectList[selectedLayerIndex].contourLabel(frameIndex).getPoint(pointIndex));
	bool IsCurVisible;
	int flag=0;
	for(i=frameIndex+1;i<=m_LayerObjectList[selectedLayerIndex].endFrame();i++)
	{
		IsCurVisible=OccludingReasoning(m_LayerObjectList[selectedLayerIndex].contourLabel(i).getPoint(pointIndex));
		if(IsVisible0==false && IsCurVisible==true)
		{
			IsLastFrameLabeled=true;
			return i;
		}
		if(IsVisible0==true && IsCurVisible==true && flag==1)
		{
			IsLastFrameLabeled=true;
			return i;
		}
		if(IsVisible0==true && IsCurVisible==false && flag==0)
			flag=1;
		if(m_LayerObjectList[selectedLayerIndex].contourLabel(i).labeled(pointIndex))
		{
			IsLastFrameLabeled=true;
			return i;
		}
	}
	return i;
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// function to find the first labeled frame for a particular point pointIndex
//-----------------------------------------------------------------------------------------------------------------------------------------
int VideoLabeling::findFirstLocalLabeledFrame(int frameIndex,int pointIndex,bool &IsFirstFrameLabeled)
{
	if(frameIndex==m_LayerObjectList[selectedLayerIndex].startFrame())
	{
		IsFirstFrameLabeled=m_LayerObjectList[selectedLayerIndex].contourLabel(frameIndex).labeled(pointIndex);
		return frameIndex;
	}
	int i;
	IsFirstFrameLabeled=false;
	bool IsVisible0=OccludingReasoning(m_LayerObjectList[selectedLayerIndex].contourLabel(frameIndex).getPoint(pointIndex));
	bool IsCurVisible;
	int flag=0;
	for(i=frameIndex-1;i>=m_LayerObjectList[selectedLayerIndex].startFrame();i--)
	{
		IsCurVisible=OccludingReasoning(m_LayerObjectList[selectedLayerIndex].contourLabel(i).getPoint(pointIndex));
		if(IsVisible0==false && IsCurVisible==true)
		{
			IsFirstFrameLabeled=true;
			return i;
		}
		if(IsVisible0==true && IsCurVisible==true && flag==1)
		{
			IsFirstFrameLabeled=true;
			return i;
		}
		if(IsVisible0==true && IsCurVisible==false && flag==0)
			flag=1;
		if(m_LayerObjectList[selectedLayerIndex].contourLabel(i).labeled(pointIndex))
		{
			IsFirstFrameLabeled=true;
			return i;
		}
	}
	return i;
}

//-----------------------------------------------------------------------------------------------------------------------------------------
// functions for layer generation
//-----------------------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::GenerateLayers()
{



	if((IsVideoLoaded==false)||(frameList.size()==0)) // if no video loaded, of course no proceeding
		return;


	DImage	image1 = frameList[0];
	DImage	image2 = frameList[1];


	// create background layer
	if(IsBgLayerCreated==false)
	{
		LayerObject background;
		background.createBackground(numFrames,m_Image.width(),m_Image.height());
		m_LayerObjectList.append(background);
		IsBgLayerCreated=true;
		emit Update();
	}
	QProgressDialog process(pMainWindow);
	process.setLabel(new QLabel("Generating layer masks..."));
	process.setRange(1,numFrames);
	process.setAutoClose(false);
	process.setMinimumWidth(300);

	cout<<"Generate layers..."<<endl;
	for(int k=0;k<numFrames;k++) // loop over every frame
	{
		process.setValue(k+1);
		process.update();

		cout<<"Frame No."<<k<<"...";
		// sort the layers according to the depth
		QMap<unsigned int, int> map;
		for (int i=0;i<m_LayerObjectList.size();i++)
			map.insert(m_LayerObjectList[i].depth(k)*255+i,i);
		QList<unsigned int> list;
		list=map.keys();
		
		BinaryImage OMask(m_Image.width(),m_Image.height()),layerMask;
		OMask.setAllZeros();

		for(int i=0;i<m_LayerObjectList.size();i++)
		{
			int index=map.value(list[i]);
			if(!m_LayerObjectList[index].InRange(k))
				continue;
			// create the layer mask
			m_LayerObjectList[index].CreateMask(k,m_Image.width(),m_Image.height());
			// create the visible mask
			m_LayerObjectList[index].CreateVisibleMask(k,OMask);
			// obtain the layer mask and dilate it
			layerMask=m_LayerObjectList[index].Mask(k);
			layerMask.imdilate8();
			// add the dilated layer mask to the occludign mask
            OMask.pixelor(layerMask);
			m_LayerObjectList[index].setImageSize(m_Image.width(),m_Image.height());

			//generate layers for stroke images
			if (m_LayerObjectList[index].strokeImageExists(k)==false)
				m_LayerObjectList[index].CreateStrokeImage(k);



		}
		cout<<"done!"<<endl;
	}
	process.setValue(numFrames);
	IsLayerMaskGenerated=true;



	// emit the signal to indicate that it is ok to compute motion
	emit setMotionAnalysis(true);
}

void VideoLabeling::outputLayerMask(QString& pathname)
{
	if(pathname.right(1)!="/")
		pathname+="/";
	for(int i=0;i<numFrames;i++)
		m_LayerObjectList[selectedLayerIndex].Mask(i).imwrite(pathname+"layer_"+QString::number(selectedLayerIndex)+"_"+QString::number(i)+".bmp");
}

void VideoLabeling::outputVisibleLayerMask(QString& pathname)
{
	if(pathname.right(1)!="/")
		pathname+="/";
	for(int i=0;i<numFrames;i++)
		m_LayerObjectList[selectedLayerIndex].visibleMask(i).imwrite(pathname+"layer_"+QString::number(selectedLayerIndex)+"_"+QString::number(i)+".bmp");
}

void VideoLabeling::outputDepthMap(QString &pathname)
{
	if(pathname.right(1)!="/")
		pathname+="/";
	for(int i=0;i<numFrames;i++)
	{

		Image<unsigned char> dispImage(m_Image.width(),m_Image.height());
		dispImage.reset();
		// sort the layers according to the depth
		getDepthMap(i,dispImage);
		
		dispImage.imwrite(pathname+"depth_"+QString::number(i)+".jpg",ImageIO::standard,99);
	}
}

void VideoLabeling::getDepthMap(int frame, Image<unsigned char>& img)
{
	QMap<unsigned int, int> map;
	for (int k=0;k<m_LayerObjectList.size();k++)
		map.insert(m_LayerObjectList[k].depth(frame)*255+k,k);
	QList<unsigned int> list;
	list=map.keys();

	for(int k=0;k<m_LayerObjectList.size();k++)
	{
		int index=map.value(list[m_LayerObjectList.size()-1-k]);
		if(!m_LayerObjectList[index].InRange(frame))
			continue;
		QRect boundingBox=m_LayerObjectList[index].boundingBox(frame);
		for(int y=boundingBox.top();y<=boundingBox.bottom();y++)
			for(int x=boundingBox.left();x<=boundingBox.right();x++)
				if(m_LayerObjectList[index].Mask(frame).getPixel(x,y))
					img.data()[y*m_Image.width()+x]=255-m_LayerObjectList[index].depth(frame);
	}


}

//------------------------------------------------------------------------------------------------------------------------
// function to output the composite motion
//------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::outputMotion(QString& pathname)
{
	if(pathname.right(1)!="/")
		pathname+="/";

	QProgressDialog progress(pMainWindow);
	progress.setLabel(new QLabel("Composing and output motion"));
	progress.setRange(1,numFrames-1);
	progress.setAutoClose(false);
	progress.setMinimumWidth(300);
	progress.show();

	FILE *fp = fopen("flowfile.ff","wb");
	if (fp == NULL)
    {
      printf("Couldn't create flow file\n'");
      exit(1);
    }

	int frameSkip=0;
	int width = m_Image.width();
	int height = m_Image.height();
	fwrite(&numFrames,sizeof(unsigned int),1,fp);
	fwrite(&frameSkip,sizeof(unsigned int),1,fp);
	fwrite(&width,sizeof(unsigned int),1,fp);
	fwrite(&height,sizeof(unsigned int),1,fp);

	//double Max,Min;
	//Max=max(vx.maximum(),vy.maximum());
	//Min=min(vx.minimum(),vy.minimum());

	for(int i=0;i<numFrames-1;i++)
	{
		DImage vx(m_Image.width(),m_Image.height()),vy(m_Image.width(),m_Image.height());
		vx.reset();
		vy.reset();
		// sort the layers according to the depth
		QMap<unsigned int, int> map;
		for (int k=0;k<m_LayerObjectList.size();k++)
			map.insert(m_LayerObjectList[k].depth(i)*100+k,k);
		QList<unsigned int> list;
		list=map.keys();

		// code to see how ranking works
		//for(int k=0;k<m_LayerObjectList.size();k++)
		//{
		//	int index=map.value(list[m_LayerObjectList.size()-1-k]);
                //	sprintf(_strbuf, "No.%d: %d",k,index);
		//}
		BiImage refImage;
		refImage.imread(m_Image);

		for(int k=0;k<m_LayerObjectList.size();k++)
		{
			int index=map.value(list[m_LayerObjectList.size()-1-k]);
			if(!m_LayerObjectList[index].InRange(i))
				continue;
			//if(k==m_LayerObjectList.size()-1)
			//	k=k;
			m_LayerObjectList[index].ComposeMotion(i,vx,vy,refImage);
			//vx.imwrite("vx.bmp",ImageIO::normalized);
			//vy.imwrite("vy.bmp",ImageIO::normalized);
		}
		QString filename;
		filename=fileList[i];
		filename.chop(4);
		vx.writeImage(pathname+filename+"_vx.flw");
		vy.writeImage(pathname+filename+"_vy.flw");

		vx.imwrite(pathname+filename+"_vx.jpg",-10,10,99);
		vy.imwrite(pathname+filename+"_vy.jpg",-10,10,99);

		for (int x=0;x < vx.width();x++)
		for (int y=0;y < vx.height();y++)
		{
			float v = vx.val(x,y);
			fwrite(&v,sizeof(float),1,fp);
		}

		for (int x=0;x < vx.width();x++)
		for (int y=0;y < vx.height();y++)
		{
			float v = vy.val(x,y);
			fwrite(&v,sizeof(float),1,fp);
		}

		progress.setValue(i+1);
		QApplication::processEvents();

	}

	fclose(fp);
}

//------------------------------------------------------------------------------------------------------------------------
// function to output the composite motion
//------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::outputLayerMotion(QString& pathname)
{
	if(pathname.right(1)!="/")
		pathname+="/";

	QProgressDialog progress(pMainWindow);
	progress.setLabel(new QLabel("Composing and output layer motion"));
	progress.setRange(1,numFrames-1);
	progress.setAutoClose(false);
	progress.setMinimumWidth(300);
	progress.show();

	for(int i=0;i<numFrames-1;i++)
	{
		// sort the layers according to the depth
		QMap<unsigned int, int> map;
		for (int k=0;k<m_LayerObjectList.size();k++)
			map.insert(m_LayerObjectList[k].depth(i)*100+k,k);
		QList<unsigned int> list;
		list=map.keys();
		QString filename;
		filename=fileList[i];
		filename.chop(4);

		for(int k=0;k<m_LayerObjectList.size();k++)
		{
			int index=map.value(list[m_LayerObjectList.size()-1-k]);
			if(!m_LayerObjectList[index].InRange(i))
				continue;
			m_LayerObjectList[index].OutputLayerMotion(i,pathname+filename+"_"+m_LayerObjectList[index].layerName()+"_");
		}
		progress.setValue(i+1);
		QApplication::processEvents();
	}
}


//------------------------------------------------------------------------------------------------------------------------
// function for motion analysis, including both optical flow estimation and parametric motion estimation
//------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::MotionAnalysis()
{

	if(QMessageBox::question(pMainWindow,"Warning!","This may take a few hours. Would you like to continue?",QMessageBox::Yes,QMessageBox::No)==QMessageBox::No)
				return;
	//Delete any previous flow information calculated
	for(int i=0;i<m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].clearMotionAnalysis();
	}

	int total=0;
	DImage image1,image2;
	// loop over all the frames to estimat the computation time
	for(int i=0;i<numFrames-1;i++)
		for(int k=0;k<m_LayerObjectList.size();k++)
			m_LayerObjectList[k].MotionAnalysis(i,image1,image2,total);

	if(total==0) // there is no need to do motion analysis
		return;




	QProgressDialog progress(pMainWindow);
	progress.setLabel(new QLabel("Computing optical flow (sorry, but it is slow)..."));
	progress.setRange(1,total);
	progress.setAutoClose(false);
	progress.setMinimumWidth(360);
	progress.show();

	// loop over all the frames, for now only the forward flow field
	int sum=total;
	total=0;
	time_t start;
	time(&start);
	
	for(int i=0;i<numFrames-1;i++)
	{

		QString stf = framePath+fileList[i];
		cout<<"loading frame "<< stf.toAscii().data() <<endl;
		stf = framePath+fileList[i+1];
		cout<<"loading next frame "<< stf.toAscii().data() <<endl;

                sprintf(_strbuf, "framelist size %i", frameList.size());
		LOG_STR_COUT(_strbuf);

		// load the images
		if(!image1.imread(framePath+fileList[i]))
			cout<<"Error in loading frame "<<i<<endl;
		if(!image2.imread(framePath+fileList[i+1]))
			cout<<"Error in loading frame "<<i+1<<endl;

		image1 = frameList[i];
		image2 = frameList[i+1];


                sprintf(_strbuf, "\n\n-------------------------------------------------------------------");
		LOG_STR_COUT(_strbuf);
                sprintf(_strbuf, "- Frame %d (out of %d)",i+1,numFrames-1);
		LOG_STR_COUT(_strbuf);
                sprintf(_strbuf, "-------------------------------------------------------------------");
		LOG_STR_COUT(_strbuf);

		// loop over all the layers

		for(int k=0;k<m_LayerObjectList.size();k++)
		{
    sprintf(_strbuf, "\n\t------------------------------------------");
			LOG_STR_COUT(_strbuf);
    sprintf(_strbuf, "\t Layer %d (out of %d)",k,m_LayerObjectList.size());
			LOG_STR_COUT(_strbuf);
    sprintf(_strbuf, "\t------------------------------------------");
			LOG_STR_COUT(_strbuf);

			m_LayerObjectList[k].MotionAnalysis(i,image1,image2,total,&progress,&start,sum);
		}
	}
}

/*

//------------------------------------------------------------------------------------------------------------------------
// fill in hidden motions
//------------------------------------------------------------------------------------------------------------------------
void VideoLabeling::FillInMotion()
{

	for(int i=0;i<numFrames-1;i++)
	{

		QString stf = framePath+fileList[i];
		cout<<"loading frame "<< stf.toAscii().data() <<endl;
		stf = framePath+fileList[i+1];
		cout<<"loading next frame "<< stf.toAscii().data() <<endl;

                sprintf(_strbuf, "framelist size %i", frameList.size());

		// load the images
		if(!image1.imread(framePath+fileList[i]))
			cout<<"Error in loading frame "<<i<<endl;


		img= frameList[i];

                sprintf(_strbuf, "\n\n-------------------------------------------------------------------");
                sprintf(_strbuf, "- Frame %d (out of %d)",i+1,numFrames-1);
                sprintf(_strbuf, "-------------------------------------------------------------------");

		float vx, vy;

		for(int x=0;x < img.width();x++)
		for(int y=0;y < img.height();y++)
		{
			

			// loop over all the layers

			int layer = -1;
			for(int k=0;k<m_LayerObjectList.size();k++)
			{
				if (m_LayerObjectList[k].visibleMask(i).getPixel(x,y) == 1)
				{
					int leftFlow = layer.layerFlow(frameRel+startFrame).left();
					int topFlow = layer.layerFlow(frameRel+startFrame).top();
					layer = k;
					m_LayerObjectList[k].layerFlow(i).getFlow(x-leftFlow,y-topFlow,&vx,&vy);
				}
			}

			if (k< 0)
            sprintf(_strbuf, "Couldn't find a flow vector at %i %i",

			for(int k=0;k<m_LayerObjectList.size();k++)
			{
				if (m_LayerObjectList[k].visibleMask(i).getPixel(x,y) == 1)
				{
					int leftFlow = layer.layerFlow(frameRel+startFrame).left();
					int topFlow = layer.layerFlow(frameRel+startFrame).top();
					layer = k;
					m_LayerObjectList[k].layerFlow(i).getFlow(x-leftFlow,y-topFlow,&vx,&vy);
				}
			}

		}	
	}
}
*/


//------------------------------------------------------------------------------------------------------------------------
// functions for writing and reading the video labeling information
//------------------------------------------------------------------------------------------------------------------------
bool VideoLabeling::writeVideoLabeling(const QString &filename)
{
	QFile file(filename);
	if(file.open(QIODevice::WriteOnly)==false)
		return false;

    createDirectoryStructure(filename,false);


	// write the version number
	QVideoProcessing::writeString(file,m_Version);

	QVideoProcessing::writeString(file,framePath);
	file.write((char *)&numFrames,sizeof(int));
	file.write((char *)&curFrameIndex,sizeof(int));
	int numLayers=m_LayerObjectList.size();
	file.write((char *)&numLayers,sizeof(int));
	file.write((char *)&IsBgLayerCreated,sizeof(bool));
	file.write((char *)&IsLayerMaskGenerated,sizeof(bool));
	file.write((char *)&m_MaxStrokeId,sizeof(int));

	// output the filename of each frame
	for(int i=0;i<numFrames;i++)
		if(!QVideoProcessing::writeString(file,fileList[i]))
			return false;

    QDir projectDir(m_ProjectDir);
	projectDir.mkdir(QString("strokeLayerProperties"));

	/*
	 QDir slpdir( m_ProjectDir +QString("/strokeLayerProperties"));
	 if (slpdir.exists())
	 {
		 QFileInfoList list = slpdir.entryInfoList();

		 for (int i = 2; i < list.size(); ++i) {
			 QFileInfo fileInfo = list.at(i);
			 slpdir.remove(fileInfo.absoluteFilePath() );
		 }
	 }
	 */

	
	for(int i=0;i<numLayers;i++)
	{
		m_LayerObjectList[i].setProjectDir(m_ProjectDir);
		if(!m_LayerObjectList[i].writeLayerObject(file))
			return false;
	}

	return true;
}


bool VideoLabeling::copyVideoLabelingFiles(const QString &oldDir,const QString &newDir)
{

	 QDir newheaddir( newDir);

	 if (newheaddir.exists()==false)
		 return false;

	 newheaddir.mkdir(QString("undo"));
	 newheaddir.mkdir(QString("strokeLayers"));
	 newheaddir.mkdir(QString("output"));


	 QDir undodir( oldDir +QString("undo"));
	 QFileInfoList list = undodir.entryInfoList();

	 for (int i = 2; i < list.size(); ++i) {
		 QString absfileName = list.at(i).absoluteFilePath();
		 QString relfileName = list.at(i).fileName();
		 QFile file (absfileName);
		 file.copy(newDir+QString("undo/")+relfileName);
	 }

	 /*
	 QDir layerdir( oldDir +QString("strokeLayers"));
	 list = layerdir.entryInfoList();

	 for (int i = 2; i < list.size(); ++i) {
		 QString absfileName = list.at(i).absoluteFilePath();
		 QString relfileName = list.at(i).fileName();
		 QFile file (absfileName);
		 file.copy(newDir+QString("strokeLayers/")+relfileName);
	 }
	 */

	 for(int i=0;i < m_LayerObjectList.size();i++)
	 {
		for(int f=m_LayerObjectList[i].startFrame();f< m_LayerObjectList[i].endFrame();f++)
		{

			QMap<int,QVector<Stroke *> >& strokemap = m_LayerObjectList[i].getGeneratedStrokes(f);

			QMap<int,QVector<Stroke *> >::iterator itr = strokemap.begin();
			while (itr != strokemap.end()) 
			{

				QString oldlayerfname = oldDir+QString("strokeLayers/")+m_LayerObjectList[i].layerName()+QString("-frame-")+QString::number(f)+QString("-")+QString::number(itr.key())+QString(".layer");
				QString newlayerfname = newDir+QString("strokeLayers/")+m_LayerObjectList[i].layerName()+QString("-frame-")+QString::number(f)+QString("-")+QString::number(itr.key())+QString(".layer");

				QFile layerfile(oldlayerfname);

				layerfile.copy(newlayerfname);
				itr++;
			}
		}
	 }
	

		/*
        QDir strokeLayerDir(m_ProjectDir+QString("strokeLayers/"));

        QMap<int,QVector<Stroke *> >::iterator itr = m_GeneratedStrokeList[i].begin();
		while (itr != m_GeneratedStrokeList[i].end()) 
		{
			strokeLayerDir.remove(m_LayerName+QString("-frame-")+QString::number(i)+QString("-")+QString::number(itr.key())+QString(".layer"));
			++itr;
		}
		*/

	 QDir outputdir( oldDir +QString("output"));
	 list = outputdir.entryInfoList();

	 for (int i = 2; i < list.size(); ++i) {
		 QString absfileName = list.at(i).absoluteFilePath();
		 QString relfileName = list.at(i).fileName();
		 QFile file (absfileName);
		 file.copy(newDir+QString("output/")+relfileName);
	 }

	 /*
	 QDir slpdir( oldDir +QString("strokeLayerProperties"));
	 list = slpdir.entryInfoList();

	 for (int i = 2; i < list.size(); ++i) {
		 QString absfileName = list.at(i).absoluteFilePath();
		 QString relfileName = list.at(i).fileName();
		 QFile file (absfileName);
		 file.copy(newDir+QString("strokeLayerProperties/")+relfileName);
	 }
	 */

}


void VideoLabeling::createDirectoryStructure(const QString& filename,bool updateFramePath)
{

        int idx = filename.lastIndexOf(QChar('/'));
        if (updateFramePath)
        {
            framePath = filename;
            framePath.truncate(idx+1);
        }

        int idx2 = filename.lastIndexOf(QChar('.'));
        m_ProjectDir = filename;
        m_ProjectDir.truncate(idx2);
        m_ProjectDir += QString("_data/");

        QString fname = filename.mid(idx+1,idx2-idx-1)+QString("_data");
        QDir headProjectDir(framePath);
        headProjectDir.mkdir(fname);

		QDir projDir(m_ProjectDir);
		projDir.mkdir("StrokeLayerProperties");

        cout << "framePath: "<< framePath.toAscii().data()<< std::endl;
        cout << "m_ProjectDir: "<< m_ProjectDir.toAscii().data()<< std::endl;
        cout << "fname: "<< fname.toAscii().data()<< std::endl;

}

bool VideoLabeling::readVideoLabeling(const QString &filename)
{
	 QFile file(filename);
	 if (!file.open(QIODevice::ReadOnly))
		 return false;

	 QString tempVersion;
	 QVideoProcessing::readString(file,tempVersion);
	 // set the version
	 m_CurVersion=tempVersion.toDouble();

    sprintf(_strbuf, "Read in file %s",filename.toAscii().data() );
	LOG_STR_COUT(_strbuf);

	 // clear the member variables
	clear();
	 
	QVideoProcessing::readString(file,framePath);

    createDirectoryStructure(filename);


	 QDir undodir( m_ProjectDir +QString("/undo"));
	 QFileInfoList list = undodir.entryInfoList();

	 for (int i = 2; i < list.size(); ++i) {
		 QFileInfo fileInfo = list.at(i);
		 undodir.remove(fileInfo.absoluteFilePath() );
		// std::cout << qPrintable(QString("%1 %2").arg(fileInfo.size(), 10)
		//										 .arg(fileInfo.fileName()));
		// std::cout << std::endl;
	 }


	file.read((char *)&numFrames,sizeof(int));
	file.read((char *)&curFrameIndex,sizeof(int));
	int numLayers;
	file.read((char *)&numLayers,sizeof(int));
	file.read((char *)&IsBgLayerCreated,sizeof(bool));
	file.read((char *)&IsLayerMaskGenerated,sizeof(bool));
	file.read((char *)&m_MaxStrokeId,sizeof(int));

	// output the filename of each frame
	for(int i=0;i<numFrames;i++)
	{
		fileList.append(QString());
		if(!QVideoProcessing::readString(file,fileList[i]))
			return false;
		cout << "read " << fileList[i].toAscii().data() << std::endl;
	}
	
	m_GeneratedStrokesId=-1;
	for(int i=0;i<numLayers;i++)
	{
		m_LayerObjectList.append(LayerObject());
		if(!m_LayerObjectList[i].readLayerObject(file,numFrames,m_ProjectDir)) // numFrames is passed for sanity check
		{
			m_LayerObjectList.removeLast();
			return false;
		}

		m_GeneratedStrokesId = max(m_GeneratedStrokesId,m_LayerObjectList[i].maxGenerateStrokeId());
	}
	m_GeneratedStrokesId++;
	

	//cleanup strokeLayers directory


	
	QSet<QString> layerSet;

	
	 for(int i=0;i < m_LayerObjectList.size();i++)
	 {
		for(int f=m_LayerObjectList[i].startFrame();f< m_LayerObjectList[i].endFrame();f++)
		{
			QMap<int,QVector<Stroke *> >& strokemap = m_LayerObjectList[i].getGeneratedStrokes(f);
			QMap<int,QVector<Stroke *> >::iterator itr = strokemap.begin();
			while (itr != strokemap.end()) 
			{
				layerSet.insert(m_LayerObjectList[i].layerName()+QString("-frame-")+QString::number(f)+QString("-")+QString::number(itr.key())+QString(".layer"));
				itr++;
			}
		}
	 }

	 QDir layerdir( m_ProjectDir +QString("/strokeLayers"));
	 list = layerdir.entryInfoList();

	 for (int i = 2; i < list.size(); ++i) {
		 QFileInfo fileInfo = list.at(i);
		 if (layerSet.contains(fileInfo.fileName()))
			 continue;
		 layerdir.remove(fileInfo.absoluteFilePath() );
		 //std::cout << qPrintable(QString("Deleting %1").arg(fileInfo.fileName()));
		// std::cout << std::endl;
	 }





	framesRead=false;
	int tempFrameIndex=curFrameIndex;
	emit NumFramesChanged(numFrames);
	curFrameIndex=tempFrameIndex;
	emit setMotionAnalysis(IsLayerMaskGenerated);
	gotoFrame(curFrameIndex+1);
	IsVideoLoaded=true;
	m_CurVersion=m_Version.toDouble();


	return true;
}



void VideoLabeling::setStrokeWidth(int width)
{
	_strokeWidth=width;

}
	

//---------------------------------------------------------------------------------------
// function to reduce the range of the frames
//---------------------------------------------------------------------------------------
void VideoLabeling::reduceFrameRange(int startFrame, int endFrame)
{
        //sprintf(_strbuf, "Reduce frame range!");
	for(int i=endFrame+1;i<numFrames;i++)
	{
		fileList.removeLast();
		if(m_IsPyramidCreated)
			m_ImPyramidList.removeLast();
	}
	for(int i=0;i<startFrame;i++)
	{
		fileList.removeFirst();
		if(m_IsPyramidCreated)
			m_ImPyramidList.removeFirst();
	}

	for(int i=m_LayerObjectList.size()-1;i>=0;i--)
		if(m_LayerObjectList[i].reduceFrameRange(startFrame,endFrame))
			m_LayerObjectList.removeAt(i);

	numFrames=endFrame-startFrame+1;
	curFrameIndex-=startFrame;
	curFrameIndex=__min(__max(curFrameIndex,0),numFrames-1);
	
	int tempFrameIndex=curFrameIndex;
	 emit NumFramesChanged(numFrames);
	 curFrameIndex=tempFrameIndex;
	 gotoFrame(curFrameIndex+1);
}

//---------------------------------------------------------------------------------------
// function to handle the selected motion parameter row 
//---------------------------------------------------------------------------------------
void VideoLabeling::motionParaRowSelected(int row)
{
	if(labelState!=Selected)
		return;
	if(tabIndex!=0) // the current panel window has to be "optical flow"
		return;
	QImage patch1,patch2;
	m_LayerObjectList[selectedLayerIndex].selectedRow()=row;
	if(curFrameIndex<numFrames-1)
	{
		DImage vx,vy;
		m_NextImage.load(framePath+fileList[curFrameIndex+1]);
		if(m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,row,patch1,patch2,vx,vy,m_Image,m_NextImage)) 
			// if the flow field under the current parameter is computed, then the flow field viewer will display the motion
			emit motionSelected(patch1,patch2,vx,vy);
		else
			emit motionSelected(patch1,patch2);
	}
	else
	{
		m_LayerObjectList[selectedLayerIndex].getLayerPatch(curFrameIndex,patch1,m_Image);
		emit motionSelected(patch1);
	}
}

void VideoLabeling::motionParaRowSelected()
{
	if(labelState==Selected && tabIndex==0)
		motionParaRowSelected(m_LayerObjectList[selectedLayerIndex].selectedRow());
}

//---------------------------------------------------------------------------------------
// function to handle the tab change in the inspector 
// if the current tab is manual (2), then enable labeling correspondence
//---------------------------------------------------------------------------------------
void VideoLabeling::tabChanged(int index)
{
	tabIndex=index;
	if(labelState!=Selected)
		return;
	m_LayerObjectList[selectedLayerIndex].tabIndex(curFrameIndex)=index;
	if(index==2)
	{
		emit SearchingWindowSizeChanged(m_SearchSize,m_WinSize);
		if(labelState==Selected)
		{
			IsEditingCorrespondence=true;
			if(IsLayerMaskGenerated==false)
				GenerateLayers();
			else
			{
				QImage patch1,patch2;
				DImage vx,vy;
				if(curFrameIndex<numFrames-1)
				{
					m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);
					emit motionSelected(patch1,patch2,vx,vy);
				}
				else
				{
					m_LayerObjectList[selectedLayerIndex].getLayerPatch(curFrameIndex,patch1,m_Image);
					emit motionSelected(patch1);
				}
				emit Update();				
			}
		}
	}
	else
	{
		IsEditingCorrespondence=false;
		motionParaRowSelected();
		emit Update();
	}
}


void VideoLabeling::buildPyramid()
{
	if(m_ImPyramidList.size()>0)
		m_ImPyramidList.clear();
	m_ImPyramidList=QList<ImPyramid>::fromVector(QVector<ImPyramid>(numFrames));

	QProgressDialog progress(pMainWindow);
	progress.setLabel(new QLabel("Generate pyramid for the video sequence"));
	progress.setRange(1,numFrames);
	progress.setAutoClose(false);
	progress.setMinimumWidth(300);
	progress.show();
	
	QImage tempImage;
	for(int i=0;i<numFrames;i++)
	{
		tempImage.load(framePath+fileList[i]);
		m_ImPyramidList[i].CreatePyramid(tempImage);
		progress.setValue(i);
	}
	m_IsPyramidCreated=true;
}

//---------------------------------------------------------------------------------------
// function to set the type of the parametric motion type for the selected layer
//---------------------------------------------------------------------------------------
void VideoLabeling::setParametricMotionType(int motionType)
{
	if(IsEditingCorrespondence && labelState==Selected)
	{
		m_LayerObjectList[selectedLayerIndex].setCorrespondenceFlowDimension(curFrameIndex);
		m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex).specifyMotionType(motionType);
		m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex).estimateMotion();

		QImage patch1,patch2;
		DImage vx,vy;
		m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);

		emit motionSelected(patch1,patch2,vx,vy);

		emit Update();
	}
}

//---------------------------------------------------------------------------------------
// function to set lambda for correspondence
//---------------------------------------------------------------------------------------
void VideoLabeling::setLambda(double lambda)
{
	if(IsEditingCorrespondence && labelState==Selected)
		m_LayerObjectList[selectedLayerIndex].lambda(curFrameIndex)=lambda;
}

//---------------------------------------------------------------------------------------
// function to clear the labeling
//---------------------------------------------------------------------------------------
void VideoLabeling::clearLabeling()
{
	if(labelState=Selected)
	{
		if(IsLayerMaskGenerated==false)
			return;
		m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex).clear();

		QImage patch1,patch2;
		DImage vx,vy;
		if(curFrameIndex<numFrames-1)
		{
			m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);
			emit motionSelected(patch1,patch2,vx,vy);
		}
		else
		{
			m_LayerObjectList[selectedLayerIndex].getLayerPatch(curFrameIndex,patch1,m_Image);
			emit motionSelected(patch1);
		}
		emit Update();
	}
}

void VideoLabeling::redoMatching()
{
	if(labelState==Selected && IsEditingCorrespondence && curFrameIndex<numFrames-1)
	{
		m_LayerObjectList[selectedLayerIndex].setCorrespondenceFlowDimension(curFrameIndex);
		Correspondence& curCorrespondence=m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex);
		curCorrespondence.invalidateFlowEstimate();
		curCorrespondence.estimateMotion();
		QImage patch1,patch2;
		DImage vx,vy;
		m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);
		emit motionSelected(patch1,patch2,vx,vy);
	}
}

void VideoLabeling::grabFeatures()
{
	if(labelState==Selected && IsEditingCorrespondence && curFrameIndex>0 &&  curFrameIndex<numFrames-1)
	{
		if(curFrameIndex<=m_LayerObjectList[selectedLayerIndex].startFrame() || curFrameIndex>=m_LayerObjectList[selectedLayerIndex].endFrame())
			return;

		m_LayerObjectList[selectedLayerIndex].setCorrespondenceFlowDimension(curFrameIndex);
		Correspondence& curCorrespondence=m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex);
		Correspondence& preCorrespondence=m_LayerObjectList[selectedLayerIndex].correspondence(curFrameIndex-1);
		if(preCorrespondence.size()==0)
			return;
		

		// add the certain points
		int nPointAdded=0;
		for(int i=0;i<preCorrespondence.size();i++)
		{
			if(preCorrespondence.IsUncertain(i)==true)
				continue;
			int x1,y1;
			x1=preCorrespondence.getPoint2(i).x();
			y1=preCorrespondence.getPoint2(i).y();
			if(x1<1 || y1<1 || x1>m_Image.width()-2 || y1>m_Image.height()-2)
				continue;

			// check whether this point is already in the correspondence
			bool IsAlreadyExisting=false;
			for(int j=0;j<curCorrespondence.size();j++)
				if(curCorrespondence.getPoint1(j)==QPointF(x1,y1))
				{
					IsAlreadyExisting=true;
					break;
				}
			if(IsAlreadyExisting)
				continue;

			double x2,y2;
			x2=x1;
			y2=y1;

			if(m_IsPyramidCreated==false) // build the pyramid of the whole sequence if it has not been built
			{
				buildPyramid();
				m_IsPyramidCreated=true;
			}
			// set the masks to get read for matching
			if(IsMaskPyramidComputed==false)
			{
				m_MaskPyramid.GenerateMask(m_LayerObjectList[selectedLayerIndex].visibleMask(curFrameIndex),6);
				ImageMatching::setMask(&m_MaskPyramid);
				IsMaskPyramidComputed=true;
			}
			ImageMatching::setQueueRatio(6);
			ImageMatching::MatchCoarse2Fine(x1,y1,x2,y2,m_ImPyramidList[curFrameIndex],m_ImPyramidList[curFrameIndex+1],m_SearchSize,m_WinSize,(double)m_WinSize/2);
			ImageMatching::LucasKanade(x1,y1,x2,y2,m_ImPyramidList[curFrameIndex],m_ImPyramidList[curFrameIndex+1],m_WinSize-2,(double)(m_WinSize-2)/2);
			curCorrespondence.addCorrespondence(QPointF(x1,y1),QPointF(x2,y2));
			nPointAdded++;
		}
		if(nPointAdded==0)
			return;
		curCorrespondence.clearMotoinType();
		curCorrespondence.estimateMotion();

		// add uncertaint points
		for(int i=0;i<preCorrespondence.size();i++)
		{
			if(preCorrespondence.IsUncertain(i)==false)
				continue;
			int x1,y1;
			x1=preCorrespondence.getPoint2(i).x();
			y1=preCorrespondence.getPoint2(i).y();
			if(x1<1 || y1<1 || x1>m_Image.width()-2 || y1>m_Image.height()-2)
				continue;

			// check whether this point is already in the correspondence
			bool IsAlreadyExisting=false;
			for(int j=0;j<curCorrespondence.size();j++)
				if(curCorrespondence.getPoint1(j)==QPointF(x1,y1))
				{
					IsAlreadyExisting=true;
					break;
				}
			if(IsAlreadyExisting)
				continue;

			double x2,y2;
			curCorrespondence.applyMotion(x2,y2,x1,y1);

			SortQueue<Point> sort(m_SearchSize*m_SearchSize);
			ImageMatching::setQueueRatio(m_SearchSize);
			
			ImageMatching::MatchCoarse2Fine(x1,y1,x2,y2,sort,m_ImPyramidList[curFrameIndex],m_ImPyramidList[curFrameIndex+1],m_SearchSize,m_WinSize,(double)m_WinSize/2);
			curCorrespondence.addCorrespondence(QPointF(x1,y1),sort);
		}

		curCorrespondence.invalidateFlowEstimate();
		curCorrespondence.estimateMotion();
		QImage patch1,patch2;
		DImage vx,vy;
		m_LayerObjectList[selectedLayerIndex].getSelectedMotion(curFrameIndex,patch1,patch2,vx,vy,m_Image,m_NextImage);
		emit motionSelected(patch1,patch2,vx,vy);
		emit motionTypeChanged(curCorrespondence.getMotionType());
		emit Update();
	}
}

void VideoLabeling::setStrokeProperties(StrokeProperties& sp)
{
	if (m_IsLoadingStrokeProperties == true)
		return;

        //sprintf(_strbuf, "setting stroke properties");
	if ((labelState==Selected)&&(IsBoundarySelected == false))
	{
		m_LayerObjectList[selectedLayerIndex].setStrokeProperties(curFrameIndex,sp);	
                //sprintf(_strbuf, "setting selected");
	}
	else
	{
		m_CurrentStrokeProperties = sp;		
                //sprintf(_strbuf, "setting current");
	}
	emit strokesChanged(curFrameIndex);
	emit Update();
}

void VideoLabeling::setStrokeProperties(QString prop, QString val)
{
	//If we're loading properties, then we don't want to update the property while we're doing it
	if (m_IsLoadingStrokeProperties == true)
		return;

	/*
        sprintf(_strbuf, "setting stroke properties");
	if ((labelState==Selected)&&(IsBoundarySelected == false))
	{
		m_LayerObjectList[selectedLayerIndex].setSelectedStrokeProperties(curFrameIndex,prop, val);	
		emit strokesChanged(curFrameIndex);
		emit Update();
                sprintf(_strbuf, "setting selected");
	}
	else
	{
		m_CurrentStrokeProperties.updateStrokeProperty(prop, val);
		//m_CurrentStrokeProperties = sp;		
                sprintf(_strbuf, "setting current");
	}
	*/

	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].setSelectedStrokeProperties(curFrameIndex,prop, val);	
	}
	m_CurrentStrokeProperties.updateStrokeProperty(prop,val);
	emit strokesChanged(curFrameIndex);
	emit Update();
}



void VideoLabeling::updateSelectedStrokes(QPointF point,bool deselecting)
{
	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].selectMultipleContours(curFrameIndex, point, _strokeWidth, !(deselecting));
	}

	emit Update();
}

void VideoLabeling::deleteSelectedStrokes()
{
	deleteSelectedStrokes(true,true);
}
void VideoLabeling::deleteSelectedStrokesForward()
{
	deleteSelectedStrokes(true,false);
}
void VideoLabeling::deleteSelectedStrokesBackward()
{
	deleteSelectedStrokes(false,true);
}

void VideoLabeling::deleteSelectedStrokes(bool forward, bool backward)
{

	int startDeleteFrame = 0;
	int endDeleteFrame = numFrames-1;

	if (forward==false)
		endDeleteFrame = curFrameIndex;
	if (backward == false)
		startDeleteFrame = curFrameIndex;

	m_CurrDeleteId++;

	m_ActionHistory.push_back(DeleteStrokesAction);

	int firstFrame =numFrames , lastFrame = 0;

    sprintf(_strbuf, "deleting selected strokes");
	LOG_STR_COUT(_strbuf);
	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].deleteSelectedStrokes(curFrameIndex,m_CurrDeleteId,startDeleteFrame,endDeleteFrame, &firstFrame, &lastFrame);
	}

	if ((labelState==Selected) && (IsBoundarySelected==false))
	{
		m_LayerObjectList[selectedLayerIndex].deleteCurrentStroke(curFrameIndex,m_CurrDeleteId,startDeleteFrame,endDeleteFrame, &firstFrame, &lastFrame);
	}

	emit Update();
	emit redrawStrokes();

	for (int i=firstFrame;i <= lastFrame;i++)
		emit strokesChanged(i);
}

void VideoLabeling::deselectAllStrokes()
{
		for (int i=0; i < m_LayerObjectList.size();i++)
		{
			m_LayerObjectList[i].deselectAllStrokes();
		}
}




void VideoLabeling::undoLastAction()
{



	if (m_ActionHistory.size() ==0)
	{
        sprintf(_strbuf, "No more actions to undo...");
		LOG_STR_COUT(_strbuf);
		return;
	}
	int lastAction = (int) m_ActionHistory.last();
	m_ActionHistory.pop_back();

	if (lastAction == DeleteStrokesAction)
	{
        sprintf(_strbuf, "Undoing last delete");
		LOG_STR_COUT(_strbuf);


		for (int i=0; i < m_LayerObjectList.size();i++)
		{
			m_LayerObjectList[i].undoDeletedStrokes(m_CurrDeleteId);

		}
		if (m_CurrDeleteId > 0)
			m_CurrDeleteId--;
	}
	else if (lastAction == CreateStrokeAction)
	{

        sprintf(_strbuf, "Undoing last stroke creation");
		LOG_STR_COUT(_strbuf);

		m_MaxStrokeId--;

		for (int i=0; i < m_LayerObjectList.size();i++)
		{
			m_LayerObjectList[i].undoStroke(m_MaxStrokeId);
		}
	}
	else if (lastAction == ChangeStrokeAction)
	{
        sprintf(_strbuf, "Undoing last stroke change");
		LOG_STR_COUT(_strbuf);

		for (int i=0; i < m_LayerObjectList.size();i++)
		{
			m_LayerObjectList[i].undoChangedStrokes(m_ChangeStrokeId);
			//m_LayerObjectList[i].checkForDuplicateStokes();
		}
		m_ChangeStrokeId--;
	}

	else if (lastAction == GenerateStrokesAction)
	{
        sprintf(_strbuf, "Undoing last generated layer");
		LOG_STR_COUT(_strbuf);
		int layer = m_GeneratedStrokeLayerList.last();
		for (int i=0; i < m_LayerObjectList.size();i++)
		{
			m_LayerObjectList[i].undoGeneratedStrokes(m_GeneratedStrokesId);
		}
		m_GeneratedStrokesId--;
		m_GeneratedStrokeLayerList.pop_back();
	}
	else if (lastAction == CreateRegionAction)
	{
        sprintf(_strbuf, "Undoing last layer creation");
		LOG_STR_COUT(_strbuf);
		m_LayerObjectList.pop_back();
		if (selectedLayerIndex>=m_LayerObjectList.size())
		{
			selectedLayerIndex=0;
		}

	}
	else
	{
        sprintf(_strbuf, "Error: action doesn't match any accepted types");
		LOG_STR_COUT(_strbuf);
	}


	emit Update();
	emit redrawStrokes();
	emit strokesChanged(curFrameIndex);
}

void VideoLabeling::generateStrokeLayersFromList(QString filename)
{
	bool fromScript=!(filename.isEmpty());

	if(IsLayerMaskGenerated==false)
		GenerateLayers();

	if (filename.isEmpty())
	{
		if ((labelState==Selected)&&(IsBoundarySelected == true))
		{
			QDir projDir(m_ProjectDir);
			projDir.mkdir("StrokeLayerProperties");
			filename=QFileDialog::getOpenFileName(pMainWindow,"Load stroke layer parameter list",m_ProjectDir+QString("strokeLayerProperties"),tr("XML file (*.xml)"));
			if (filename.isEmpty())
				return;
		}
		else
		{
			QMessageBox::warning(pMainWindow,"Warning","Oi! You have to select a region to generate strokes for :)");
			return;
		}
	}

	QList<StrokeLayerProperties> slpList;

	QDomDocument doc("mydocument");
	QFile file(filename);
	if ((!file.open(QIODevice::ReadOnly))||(!doc.setContent(&file)))
	{
		QMessageBox::warning(pMainWindow,"Warning","Problem with reading in xml file");
		file.close();
		return;
	}
	file.close();

	QDomElement root=doc.documentElement();

	if (root.tagName()=="StrokeLayerPropertiesList")
	{
		QDomNode n=root.firstChild();
		while(!n.isNull())
		{
			QDomElement e=n.toElement();
			StrokeLayerProperties slp;
			slp.readStrokeLayerPropertiesFromXML(e);
			slpList.push_back(slp);
			n=n.nextSibling();
		}
	}
	for(int i=0;i< slpList.size();i++)
	{
		m_ActionHistory.push_back(GenerateStrokesAction);
		m_GeneratedStrokesId++;

		int layerIndex=selectedLayerIndex;

		//if we're running this from a script, then we're more careful about the layer names 
		//since the user hasn't specifically chosen the parameter list file and the region
		if (fromScript)
		{
			layerIndex=-1;
			for(int j=0;j < m_LayerObjectList.size();j++)
			{
				if (QString::compare(m_LayerObjectList[j].layerName(),slpList[i].layerName)==0)
				{
					layerIndex=j;
				}
			}
		}


		if (layerIndex>=0)
		{
			m_GeneratedStrokeLayerList.push_back(layerIndex);
			emit generateStrokes(framePath,m_LayerObjectList[layerIndex],m_LayerObjectList,frameList,fileList, curFrameIndex,m_GeneratedStrokesId,&(slpList[i]),fromScript);
		}
	}
	emit Update();
	emit redrawStrokes();
	

}

void VideoLabeling::generateStrokeLayer()
{

	if ((labelState==Selected)&&(IsBoundarySelected == true))
	{

	
		if(IsLayerMaskGenerated==false)
			GenerateLayers();


		m_ActionHistory.push_back(GenerateStrokesAction);
		m_GeneratedStrokesId++;
		m_GeneratedStrokeLayerList.push_back(selectedLayerIndex);
		emit generateStrokes(framePath,m_LayerObjectList[selectedLayerIndex],m_LayerObjectList,frameList, fileList,curFrameIndex,m_GeneratedStrokesId,0,false);
		//emit strokesChanged(curFrameIndex);
		emit Update();
		emit redrawStrokes();
	}
	else
	{
		QMessageBox::warning(pMainWindow,"Warning","Oi! You have to select a region to generate strokes for :)");
	}
}

void VideoLabeling::deleteStrokeLayers()
{
	if ((labelState==Selected)&&(IsBoundarySelected == true))
	{
		//m_LayerObjectList[selectedLayerIndex].deleteStrokeImages(curFrameIndex);
		m_LayerObjectList[selectedLayerIndex].deleteAllGeneratedStrokes();
		//emit strokesChanged(curFrameIndex);
		emit Update();
		emit redrawStrokes();
	}

}

void VideoLabeling::undoLastLayer()
{

	for(int i=m_ActionHistory.size()-1;i>0;i--)
	{
                sprintf(_strbuf, "undoing last generated layer");


		if (m_ActionHistory[i]== GenerateStrokesAction)
		{
			m_ActionHistory.removeAt(i);

			int layer = m_GeneratedStrokeLayerList.last();
			m_LayerObjectList[layer].undoGeneratedStrokes(m_GeneratedStrokesId);
			m_GeneratedStrokesId--;
			m_GeneratedStrokeLayerList.pop_back();

			return;
		}
	}


	emit Update();
	emit redrawStrokes();
	emit strokesChanged(curFrameIndex);



}

void VideoLabeling::changeRegionBoundaryBlur()
{
	if ((labelState==Selected)&&(IsBoundarySelected == true))
	{
		m_LayerObjectList[selectedLayerIndex].setStrokeImageBoundaryBlur(m_CurrentBoundaryBlur);
		m_LayerObjectList[selectedLayerIndex].refreshAllStrokeImages();
		emit Update();
		emit redrawStrokes();
	}
}

void VideoLabeling::changeRegionMedianBlur()
{
	if ((labelState==Selected)&&(IsBoundarySelected == true))
	{
		m_LayerObjectList[selectedLayerIndex].setStrokeImageMedianBlur(m_CurrentMedianBlur);
		m_LayerObjectList[selectedLayerIndex].refreshAllStrokeImages();
		emit Update();
		emit redrawStrokes();
	}
}




int VideoLabeling::getNearestLayer(int curFrameIndex, QPointF point)
{
	float minDist = 99999;
	float nearestLayer=-1;
	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		float dist = m_LayerObjectList[i].minPointDistanceSelectedStrokes(curFrameIndex,point);

		if (dist < minDist)
		{
			nearestLayer=i;
			minDist=dist;
		}
	}

	return nearestLayer;
}


void VideoLabeling::showStrokeTypes()
{

	for (int i=0; i < m_LayerObjectList.size();i++)
	{
		m_LayerObjectList[i].showStrokeTypes(m_ShowColorStrokes,m_ShowGuideStrokes,m_ShowSeedStrokes,m_ShowRegularStrokes);
	}
	emit Update();
	emit redrawStrokes();
}


void VideoLabeling::renderPainting(int requestStartFrame,int requestEndFrame,float renderScale,QString outputDir)
{
	emit renderStrokes(requestStartFrame,requestEndFrame,renderScale,m_LayerObjectList,frameList,outputDir);
}
