#include "Image.h"
#include "emboss.h"
#include "assert.h"
// ------------------ MATERIALS AND LIGHTING PARAMETERS ----------

float lightx = -2, lighty = 1, lightz = 3;
float ambient = 0.15;
float diffuse = 1, diffuseSpread = 1;
float highlight = 0.5, highlightSpread = 25;
float heightScale = .001;
int lightToDark = 0;

// ------------------ LIGHTING FUNCTIONS -------------------------

void surfaceNormal(ColorImage * heightField,
		   int x,int y, float & nx, float & ny, float &nz)
{
  //  fprintf(stderr, "surfaceNormal %d %d\n",x,y);

  float dhdx, dhdy;

 // assert(heightField->width() > 2 && heightField->height() > 2);

  if (x == 0)
    dhdx = heightField->val(x+1,y) - heightField->val(x,y);
  else if (x >= heightField->width()-1)
    dhdx = heightField->val(x,y) - heightField->val(x-1,y);
  else
    dhdx = heightField->val(x+1,y) - heightField->val(x-1,y);

  if (y == 0)
    dhdy = heightField->val(x,y+1) - heightField->val(x,y);
  else if (y >= heightField->height()-1)
    dhdy = heightField->val(x,y) - heightField->val(x,y-1);
  else
    dhdy = heightField->val(x,y+1) - heightField->val(x,y-1);

  nx = dhdy*heightScale;
  ny = dhdx*heightScale;
  nz = 1;

  float nmag = sqrtf(nx*nx+ny*ny+1);
  nx /= nmag;
  ny /= nmag;
  nz /= nmag;
} 

void emboss(ColorImage * sourceImage, ColorImage * heightField,
	    ColorImage * embossedPtg, int x,int y)
{
  // compute surface normal 
  float nx, ny, nz;

  surfaceNormal(heightField, x,y, nx, ny, nz);

  //  fprintf(stderr, "light %d %d\n",x,y);

  float fx = x/embossedPtg->width();
  float fy = y/embossedPtg->height();

  float lightvecx = lightx - fx, lightvecy = lighty - fy,
    lightvecz = lightz;

  float lightmag = sqrt(lightvecx*lightvecx + lightvecy*lightvecy + 
		    lightvecz*lightvecz);
  
  lightvecx /= lightmag;
  lightvecy /= lightmag;
  lightvecz /= lightmag;

  float rx = 2*nx - lightvecx;
  float ry = 2*ny - lightvecy;
  float rz = 2*nz - lightvecz;

  float rmag = sqrt(rx*rx + ry*ry + rz*rz);
  rx /= rmag;
  ry /= rmag;
  rz /= rmag;

  float diffuseFac = lightvecx*nx + lightvecy*ny + lightvecz*nz;
  if (diffuseFac < 0) diffuseFac =0;
  diffuseFac = diffuse*pow(diffuseFac,diffuseSpread);

  float highlightFac = highlight*pow(rz,highlightSpread);

  for(int d=0;d<3;d++)
    {
      float val = sourceImage->val(x,y,d);
      val = (ambient + diffuseFac) * val + 
	highlightFac*255.;
      if (val > 255.)
			val = 255.;
      //      val = min(val, sourceImage->maxVal());
      embossedPtg->setPixelValue((unsigned char)val,x,y,d);
    }    

//  fprintf(stderr, "n = [%f %f %f], diffuseFac = %f, highlightFac = %f\n",nx,ny,nz,diffuseFac, highlightFac);
///  embossedPtg->val(x,y,3) = 255;
}

void emboss(ColorImage * sourceImage, ColorImage * heightField,
	    ColorImage * embossedPtg)
{
  fprintf(stderr, "embossing ...\n");

    fprintf(stderr, "%d %d\n", embossedPtg->width(), embossedPtg->height());


	assert(embossedPtg->width() == sourceImage->width() && embossedPtg->height() == sourceImage->height());

  for(int x=0;x<embossedPtg->width();x++)
    for(int y=0;y<embossedPtg->height();y++)
	{
      emboss(sourceImage, heightField, embossedPtg, x,y);
		
	}

  fprintf(stderr, "done embossing\n");
 
}

void solveForAmbient(int imageWidth,int imageHeight)
{
     float nx=0, ny=0, nz=1;

  //  fprintf(stderr, "light %d %d\n",x,y);

  float fx = .5;
  float fy = .5;

  float lightvecx = lightx - fx, lightvecy = lighty - fy,
    lightvecz = lightz;

  float lightmag = sqrt(lightvecx*lightvecx + lightvecy*lightvecy + 
		    lightvecz*lightvecz);
  
  lightvecx /= lightmag;
  lightvecy /= lightmag;
  lightvecz /= lightmag;

  float rx = 2*nx - lightvecx;
  float ry = 2*ny - lightvecy;
  float rz = 2*nz - lightvecz;

  float rmag = sqrt(rx*rx + ry*ry + rz*rz);
  rx /= rmag;
  ry /= rmag;
  rz /= rmag;

  float diffuseFac = lightvecx*nx + lightvecy*ny + lightvecz*nz;
  if (diffuseFac < 0) diffuseFac =0;
  diffuseFac = diffuse*pow(diffuseFac,diffuseSpread);

  float highlightFac = highlight*pow(rz,highlightSpread);

  ambient = 1 - diffuseFac - 2*highlightFac;

}




