#pragma comment(lib, "C:\\glew\\lib\\glew32.lib")
#include <GL/glew.h>
#include <QtGui>
#include <QtOpenGL>
#include <stdio.h>
#include <assert.h>
#include <iostream>
#include <math.h>
#include "StrokePainter.h"
#include "MyPreference.h"gener
#include "Randn.h"
#include "ImPyramid.h"
#include "Point.h"
#include <QTime>
#include <QDate>
#include <QDateTime>
#include <QProgressDialog>

GLhandleARB f,f2,p;

StrokePainter::StrokePainter(QWidget *parent, int width, int height, int frames)
	: QGLWidget(parent)
{
	_w = width;
	_h = height;
	_frames = frames;

	_textureNames.resize(3000); 

	_fboscaleFactor = 0;
	_fbo = 0;
	_fboVar = 0;

	_renderScale=2;

	_etfCreated.resize(frames);
	_etfAngle.resize(frames);
	_etfMag.resize(frames);

	inp.startFrame = -1;
	inp.endFrame = -1;

	_magMax=10000;

	_kdTreeK = 5;

	_renderFrames = false;
	_redraw = true;
	_oldBlend = false;
	_multiPassRender = true;
	_systemStrokeTransparency=1;
	_userStrokeTransparency=1;
	_imageTransparency=1;
	_imageTex =0;
	_mainTex = 0;
	_secondaryTex = 0;

     _readPixelsTiming=false;

	_startHeavySample = 1000;
	_endHeavySample = -1;

	_paperColor = QColor(255,255,255,255);
	_depthBlending=ADD_DEPTH;

	_renderWithLighting=true;

	//_startingCheck=0;

	initializeGL();
	resizeGL(width, height,1);

	_followFlowFactor=0;

	_renderWithCurrentTexture=false;

	_feedForwardFade=false;

	_drawSpines=false;


}


StrokePainter::StrokePainter(const StrokePainter &strokePainter)
{
        sprintf(_strbuf, "copying painter? shouldn't be here");
	LOG_STR_COUT(_strbuf);
}



//Cleanup all the strokes
StrokePainter::~StrokePainter()
{

	delete _fboVar;
	delete _fbo;

}  
void StrokePainter::setSize(int width, int height)
{
	_w = width;
	_h = height;

}


void StrokePainter::clear()
{
	//_startingCheck=0;
	delete _fboVar;
	delete _fbo;
	_fboscaleFactor = 0;
	_fbo = 0;
	_fboVar = 0;
	inp.startFrame = -1;
	inp.endFrame = -1;
	_imageTex =0;
	_mainTex = 0;
	_secondaryTex = 0;
}

void StrokePainter::initializeGL()
{
	glEnable( GL_TEXTURE_2D );
	glShadeModel(GL_FLAT);
	//glBlendFuncSeparateEXT(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ZERO);
	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE) ; 
}

void StrokePainter::paintGL()
{

}

void StrokePainter::resizeGL(int width, int height, double scaleFactor)
{
	glViewport(0,0,(int)(((double) width)*scaleFactor), (int)(((double) height)*scaleFactor));

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, width, 0, height,-1,1);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


void StrokePainter::Paint(ImageViewer &imageViewer, QPainter &mPainter)
{

}



void StrokePainter::gotoFrame(int frame)
{
	
}

/*

Paint all the strokes to the given paint object

If the strokes haven't changed since last time, we can use the saved pixmap

*/

void StrokePainter::paintStrokes(ImageViewer& imageViewer,QPainter& painter,QList<QImage>& imgList, QList<LayerObject>& layerObjectList,int curFrameIndex)
{

	if ((_errCode = glGetError()) != GL_NO_ERROR) {
                sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}

    if (curFrameIndex > imgList.size()-1)
    {
        sprintf(_strbuf, "problem: frame %i is bigger than image list size %i\n", curFrameIndex,imgList.size());
		LOG_STR_COUT(_strbuf);
        return;
    }

	//if (++ ==1)
	if (_fbo ==0)
	{
		initializePainter((imgList[curFrameIndex]));
		//_startingCheck++;
	}
	if (_renderFrames)
	{
		_renderFrames = false;


		renderStrokes(_interfaceSlp.requestedStartFrame, _interfaceSlp.requestedEndFrame,_renderScale, layerObjectList,imgList,QString("output"));
	}
	try
	{
		paintFrameStrokes(imageViewer,painter, layerObjectList,curFrameIndex);
	}
	catch(...) 
	{
                sprintf(_strbuf, "ERROR: Exception raised in stroke painting...");
		LOG_STR_COUT(_strbuf);
	}

}


void StrokePainter::renderStrokes(int requestedStartFrame, int requestedEndFrame, float renderScale,QList<LayerObject>& layerObjectList,  QList<QImage>& imgList, QString outputDir)
{


	try{
		if (_fbo ==0)
		{

			if (imgList.size() <= requestedStartFrame)
			{
				QMessageBox::warning(this,"Warning","The # of requested frames for rendering don't match the # of loaded frames");
				return;
			}
			initializePainter((imgList[requestedStartFrame]));
		}
		QGLFramebufferObject *fbo = new QGLFramebufferObject(_w*renderScale,_h*renderScale,QGLFramebufferObject::CombinedDepthStencil);
		//QGLFramebufferObject *fbo = new QGLFramebufferObject(20000,20000,QGLFramebufferObject::CombinedDepthStencil);
		

		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		LOG_STR ("Rendering");
		LOG (renderScale);
		LOG (_w);
		LOG (_h);
		LOG (requestedStartFrame);
		LOG (requestedEndFrame);

		int endFrame = min(requestedEndFrame,_frames-1);
		int startFrame = max(requestedStartFrame,1);

		LOG (startFrame);
		LOG (endFrame);

		if ((imgList.size()==0) || (imgList.size() < endFrame))
			return;

		bool doubleExposure=false;

	
		QImage currImage;
		QImage lastImage;

		//for(int i=1;i < 20;i++)
		for(int i=startFrame;i <= endFrame;i++)
		{
			if (_imageTex > 0)
				glDeleteTextures( 1, &_imageTex );

			_imageTex = loadStrokeImageTexture(imgList[i]);


			if (_imageTex==0)
			{
				deleteFrameTextures();
				return;
			}

			
			paintFrameStrokesFinal(fbo,layerObjectList,i,renderScale,outputDir,doubleExposure,lastImage,currImage);

			lastImage=currImage;

		}
		delete fbo;
	}
	catch(...) 
	{
        sprintf(_strbuf, "ERROR: Exception raised in stroke rendering..." );
		LOG_STR_COUT(_strbuf);
	}

}


void StrokePainter::paintFrameStrokesFinal(QGLFramebufferObject *fbo, QList<LayerObject>& layerObjectList,int curFrameIndex,float renderScale,QString outputDir, bool doubleExposure,QImage &lastImg,QImage &currImg)
{

	glEnable(GL_BLEND);

	if (fbo->bind()==false)
	{
		LOG_STR_COUT("couldn't bind fbo");
		return;
	}

	QImage blank(_w*renderScale,_h*renderScale,QImage::Format_ARGB32_Premultiplied);
	GLuint tex = loadStrokeImageTexture(blank);
	if (tex ==0)
	{
		deleteFrameTextures();
		LOG_STR_COUT("ERROR: problem getting blank texture");
		return;
	}

	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, tex, 0);

	char buffer[1024];
    sprintf(buffer, "writing out frame %i" ,curFrameIndex );
	LOG_STR_COUT(buffer);

	initializeGL();
	resizeGL(_w,_h,renderScale);
	glLoadIdentity();

	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}

	
	if (_oldBlend==true)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	else
		glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ONE);
	

	

	//sort the depths
	QMap<unsigned int, int> map;
	for (int i=0;i<layerObjectList.size();i++)
		map.insert(layerObjectList[i].depth(curFrameIndex)*255+i,i);
	QList<unsigned int> list;
	list=map.keys();

    QList<QList<ContourLabel> > userStrokes;
	Stroke *str;


    QList<QList<Stroke *> > systemStrokes;

    sprintf(_strbuf, "num layers %i\n", layerObjectList.size());
	LOG_STR_COUT(_strbuf);

	int totalStrokes=0;
	int totalLayers=0;
	
 
	//loop over all the layer objects to get the strokes
	for(int i=0;i<layerObjectList.size();i++)
	{		

		QList<ContourLabel> layerUserStrokes;
		QList<Stroke *> layerSystemStrokes;
		userStrokes.append(layerUserStrokes);
		systemStrokes.append(layerSystemStrokes);
		int index=map.value(list[layerObjectList.size()-1-i]);

		int strokeLayers=0;


        QMap<int,QVector<Stroke *> >& strokemap = layerObjectList[index].getGeneratedStrokes(curFrameIndex);

        QMap<int,QVector<Stroke *> >::iterator itr = strokemap.begin();
		while (itr != strokemap.end()) 
		{
			
			QString layerfname = layerObjectList[index].projectDir()+QString("strokeLayers/")+layerObjectList[index].layerName()+QString("-frame-")+QString::number(curFrameIndex)+QString("-")+QString::number(itr.key())+QString(".layer");

			QFile layerfile(layerfname);

			if(layerfile.open(QIODevice::ReadOnly)==false)
			{
				itr++;
				sprintf(_strbuf, "problem opening layer file");
				LOG_STR_COUT(_strbuf);

				continue;
			}

			strokeLayers++;
		
			//QVector<Stroke *> strokeVect;

			int numLayerStroke;
			layerfile.read((char *)&numLayerStroke,sizeof(int));

			totalStrokes+=numLayerStroke;

			for(int j=0;j < numLayerStroke;j++)
			{
				Stroke * str = new Stroke();
				if (str->readStroke(layerfile)==false)
				{
					j=numLayerStroke;
					sprintf(_strbuf, "problem reading strokes from file");
					LOG_STR_COUT(_strbuf);
					continue;
				}
				/*
				QVector<Point>& pts=str->getControlPoints();
				float distx=pts[0].x()-pts[pts.size()-1].x();
				float disty=pts[0].y()-pts[pts.size()-1].y();
				if (sqrt(distx*distx+disty*disty)>50)
					continue;
					*/
	
				if ((str->properties.texture == -1) && str->properties.textureFilename.length() > 0)
				{
					if (_renderWithCurrentTexture)
						str->properties.texture = loadTexture(_interfaceSlp.strokeProperties.textureFilename);	
					else
						str->properties.texture = loadTexture(str->properties.textureFilename);	

				}

				systemStrokes[i].push_back(str);
			}
			++itr;
		}

		
		QList<ContourLabel>& layerStrokes = layerObjectList[index].getStrokes(curFrameIndex);

		for(int j=0; j < layerStrokes.size();j++)
		{
			if (layerStrokes[j].deleted()== false)
			{

				str = layerStrokes[j].getStroke();

				if ((str->properties.texture == -1) && str->properties.textureFilename.length() > 0)
					str->properties.texture = loadTexture(str->properties.textureFilename);	

				userStrokes[i].append((layerStrokes[j]));
				//systemStrokes.append((layerStrokes[j].getStroke()));
			}
		}

        sprintf(_strbuf, "%i layers for %s",strokeLayers,layerObjectList[index].layerName().toAscii().data() );
		LOG_STR_COUT(_strbuf);

		totalLayers +=strokeLayers;

	}

    sprintf(_strbuf, "totalLayers %i totalStrokes %i\n",totalLayers, totalStrokes);
	LOG_STR_COUT(_strbuf);

	//draw the user strokes ordered by when they were drawn

	//qStableSort(userStrokes.begin(), userStrokes.end(), ContourLabel::lessThan);

	QDir projectDir(layerObjectList[0].projectDir());
	projectDir.mkdir(outputDir+QString("/"));

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	drawPaperAndImage();

	if (_renderWithLighting==false)
	{
		for(int i=0;i<layerObjectList.size();i++)
		{
			//Draw system strokes
			for(int j=0;j<systemStrokes[i].size();j++)
				systemStrokes[i][j]->render();

			//draw the user strokes
			for(int j=0;j < userStrokes[i].size();j++)
				userStrokes[i][j].drawStroke(_userStrokeTransparency,true,_slp.fadeRate);

		}
		
		QImage img = fbo->toImage();

		img.save(layerObjectList[0].projectDir()+outputDir+QString("/final-nolight-")+QString::number(curFrameIndex)+QString(".png"));
	

		if ( _drawSpines)
		{
			QPainter painter(&img);
			painter.setRenderHint(QPainter::Antialiasing);
		
			//for(int i=0;i<layerObjectList.size();i++)
			//	layerObjectList[i].DrawContour(curFrameIndex,painter,QPointF(0,0),_renderScale,true,false,false,true);
			//}
		
			for(int i=0;i<layerObjectList.size();i++)
			{
				layerObjectList[i].selectMultipleContours(curFrameIndex, QPointF(1,1), 10000, true);
				layerObjectList[i].DrawStrokes(curFrameIndex,painter,QPointF(0,0),_renderScale);
				layerObjectList[i].deselectAllStrokes();
			}
	
			img.save(layerObjectList[0].projectDir()+outputDir+QString("/final-nolight-spines-")+QString::number(curFrameIndex)+QString(".png"));
	
		}
		

	}
	else
	{
		

		for(int l=0;l<layerObjectList.size();l++)
		{
			//Draw system strokes
			for(int i=0;i<systemStrokes[l].size();i++)
			{

				GLuint oldtex = systemStrokes[l][i]->properties.texture;
				systemStrokes[l][i]->properties.texture = _alphaTextureMap.value((int)oldtex);

				assert(systemStrokes[l][i]->properties.texture>0);

				systemStrokes[l][i]->render();
				systemStrokes[l][i]->properties.texture = oldtex;
			}


			//draw the user strokes with the alpha texture
			for(int i=0;i < userStrokes[l].size();i++)
			{
				str = userStrokes[l][i].getStroke();

				GLuint oldtex = str->properties.texture;
				str->properties.texture = _alphaTextureMap.value((int)oldtex);
				assert(str->properties.texture>0);

				userStrokes[l][i].drawStroke(_userStrokeTransparency,true,_slp.fadeRate);

				str->properties.texture = oldtex;
			}	
		}


		QImage colorImg =fbo->toImage();
		ColorImage clrimg(colorImg);

		colorImg.save(layerObjectList[0].projectDir()+outputDir+QString("/final-color-")+QString::number(curFrameIndex)+QString(".png"));
	
		fbo->bind();
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, tex, 0);

        glClearColor(0,0,0,1);
        glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
 

		int nstrokes = 0;

	

		for(int l=0;l<layerObjectList.size();l++)
		{
			nstrokes+=systemStrokes[l].size()+userStrokes[l].size();
		}

        int n = 0;

		//float addFactor = 0;


		for(int l=0;l<layerObjectList.size();l++)
		{
			//Draw system strokes
			for(int i=0;i<systemStrokes[l].size();i++)
			{
			
				float z = (n++/float(nstrokes));

				str = systemStrokes[l][i];

				str->z = z;

				GLubyte grey=0;
				
				switch(_depthBlending)
				{
					case ADD_DEPTH: grey = 255; break;
					case CONSTANT_DEPTH: grey = 255; break;
					case MULTIPLY_DEPTH: grey = z*255; break;
				}

				QColor oldColor = str->properties.color;

				str->properties.color = QColor(grey,grey,grey);
				str->render();

				str->properties.color = oldColor;
			}


			//draw the user strokes
			for(int i=0;i < userStrokes[l].size();i++)
			{
				float z = (n++/float(nstrokes));

				str = userStrokes[l][i].getStroke();
				str->z = z;

				GLubyte grey=0;
				
				switch(_depthBlending)
				{
					case ADD_DEPTH: grey = 255; break;
					case CONSTANT_DEPTH: grey = 255; break;
					case MULTIPLY_DEPTH: grey = z*255; break;
				}

				QColor oldColor = str->properties.color;
				str->properties.color = QColor(grey,grey,grey);
				userStrokes[l][i].drawStroke(_userStrokeTransparency,true,_slp.fadeRate);



				str->properties.color = oldColor;

			}	

		}
		
		QImage heightImg =fbo->toImage();

		ColorImage hgtimg(heightImg);

	
		ColorImage finalImg(colorImg.width(),colorImg.height(),3);
	
		emboss(&clrimg, &hgtimg,&finalImg);

		int outputFrameNum=curFrameIndex;

		if (doubleExposure)
			outputFrameNum=curFrameIndex*2;

		finalImg.imwrite(layerObjectList[0].projectDir()+outputDir+QString("/final-")+QString::number(outputFrameNum)+QString(".png"));

		if (doubleExposure)
		{
			
			QImage img(layerObjectList[0].projectDir()+outputDir+QString("/final-")+QString::number(outputFrameNum)+QString(".png"));
	
			currImg=img;

			QImage blend=lastImg;

			QPainter painter(&blend);
			painter.setOpacity(0.5);

			QRect rec(0,0,img.width(),img.height());
			painter.drawImage(rec,currImg);

			blend.save(layerObjectList[0].projectDir()+outputDir+QString("/final-")+QString::number(outputFrameNum-1)+QString(".png"));

		}
		
		if (_drawSpines)
		{

			QImage img(layerObjectList[0].projectDir()+outputDir+QString("/final-")+QString::number(outputFrameNum)+QString(".png"));
			QPainter painter(&img);
			painter.setRenderHint(QPainter::Antialiasing);
	


			for(int i=0;i<layerObjectList.size();i++)
			{
				layerObjectList[i].selectMultipleContours(curFrameIndex, QPointF(1,1), 10000, true);
				layerObjectList[i].DrawStrokes(curFrameIndex,painter,QPointF(0,0),_renderScale);
				layerObjectList[i].deselectAllStrokes();
			}

			img.save(layerObjectList[0].projectDir()+outputDir+QString("/final-spines-")+QString::number(curFrameIndex)+QString(".png"));
		}

	
	}

	glClearColor(0,0,0,0);

	glDeleteTextures( 1, &tex );

	for(int l=0;l<layerObjectList.size();l++)
	for(int i=0;i<systemStrokes[l].size();i++)
	{
		delete systemStrokes[l][i];
	}

}


void StrokePainter::initializePainter(const QImage& currImage)
{

	LOG_STR ("Initializing stroke painter");

	QImage saveimg = grabFrameBuffer(true);

	QString temptex("Brushes/soft.png");
	int t = loadTexture(temptex);	
	_interfaceSlp.strokeProperties.texture = t;
	_interfaceSlp.strokeProperties.textureFilename = temptex;

	emit strokePropertiesChanged(_interfaceSlp.strokeProperties);

    QString papername("Paper/defaultpaper.pgm");
    loadPaperTexture(papername);


	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
        sprintf(_strbuf, "glewInit Error: %s\n", glewGetErrorString(err));
		LOG_STR_COUT(_strbuf);
	}
	
	initializeGL();
	resizeGL(_w,_h,_fboscaleFactor);
	glLoadIdentity();

	_fbo = new QGLFramebufferObject(_w,_h);

    sprintf(_strbuf, "fixed fbo size %f %f", ((float)_w),((float)_h));
	LOG_STR_COUT(_strbuf);


	_imageTex = loadStrokeImageTexture(currImage);

	QImage blank(_w,_h,QImage::Format_ARGB32_Premultiplied);

	glDeleteTextures( 1, &_secondaryTex );
	_secondaryTex = loadStrokeImageTexture(blank);
	glDeleteTextures( 1, &_mainTex );
	_mainTex = loadStrokeImageTexture(blank);


}

void StrokePainter::paintFrameStrokes(ImageViewer& imageViewer,QPainter& painter, QList<LayerObject>& layerObjectList,int curFrameIndex)
{


	QPoint temp;
	QPointF ScreenOffset;
	double ScaleFactor;
	temp=imageViewer.getPaintingOffset();
	ScreenOffset.setX(temp.x());
	ScreenOffset.setY(temp.y());
	ScaleFactor=imageViewer.getScaleFactor();	

	if (ScaleFactor != _fboscaleFactor)
	{



		_fboscaleFactor = ScaleFactor;
		if (_fboVar != 0)
			delete _fboVar;
		_fboVar = new QGLFramebufferObject(_w*ScaleFactor,_h*ScaleFactor,QGLFramebufferObject::CombinedDepthStencil);


        sprintf(_strbuf, "new variable fbo size %f %f", ((float)_w)*ScaleFactor,((float)_h)*ScaleFactor);
		LOG_STR_COUT(_strbuf);
		
		QImage blank(_w*ScaleFactor,_h*ScaleFactor,QImage::Format_ARGB32_Premultiplied);

		glDeleteTextures( 1, &_secondaryTex );
		_secondaryTex = loadStrokeImageTexture(blank);
		glDeleteTextures( 1, &_mainTex );
		_mainTex = loadStrokeImageTexture(blank);

		_redraw = true;
	}
	
	if (_fboVar->bind()==false)
	{
		LOG_STR_COUT ("Error binding the frame buffer. Does your video card support frame buffer objects?");
		return;
	}
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _mainTex, 0);

	initializeGL();
	resizeGL(_w,_h,_fboscaleFactor);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	if (_oldBlend==true)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	else
		glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ONE);
	

	
	//if something's changed in the strokes since the last time
	//we rendered, we have to redraw
	if (_redraw == true)
	{

		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _secondaryTex, 0);

		drawPaperAndImage();


		_redraw = false;

		//sort the depths
		QMap<unsigned int, int> map;
		for (int i=0;i<layerObjectList.size();i++)
		{
			int depth = layerObjectList[i].depth(curFrameIndex)*255+i;		
			map.insert(depth,i);
		}
		QList<unsigned int> list;
		list=map.keys();


		QList<ContourLabel> allStrokes;

		Stroke *str;
	
	 
		//loop over all the layer objects ordered by depth and draw the stored strokes
		for(int i=0;i<layerObjectList.size();i++)
		{		

			int index=map.value(list[layerObjectList.size()-1-i]);
			//if (_startingCheck >=2)
			//{

			drawTexture(layerObjectList[index], curFrameIndex,true);

			QList<ContourLabel>& layerStrokes = layerObjectList[index].getStrokes(curFrameIndex);
		

		
			for(int j=0; j < layerStrokes.size();j++)
			{

				if (layerStrokes[j].isBeingDrawn()==false)
				{
					str = layerStrokes[j].getStroke();

					if ((str->properties.texture == -1) && str->properties.textureFilename.length() > 0)
						str->properties.texture = loadTexture(str->properties.textureFilename);	
					
					try{
						layerStrokes[j].drawStroke(_userStrokeTransparency,true,_slp.fadeRate);
					}
					catch(...)
					{
						LOG_STR_COUT("Problem drawing the stroke");
					}
				}
			}
		
				

			for(int j=0; j < layerStrokes.size()-1;j++)
			{
				if (layerStrokes[j].deleted()== false)
					allStrokes.append((layerStrokes[j]));
				
			}

			if ((layerStrokes.size() > 0) && (layerStrokes[layerStrokes.size()-1].isBeingDrawn()==false))
			{
				if (layerStrokes[layerStrokes.size()-1].deleted()== false)
					allStrokes.append((layerStrokes[layerStrokes.size()-1]));

			}
				
			//}
		}

		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		//draw the user strokes ordered by when they were drawn

		//glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ONE);
		qStableSort(allStrokes.begin(), allStrokes.end(), ContourLabel::lessThan);
		//qStableSort(allStrokes);

		if (_oldBlend==true)
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		else
			glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ONE);

		
		//glColorMask(1,1,1,1);							// Set Color Mask to TRUE, TRUE, TRUE, TRUE
		glStencilFunc(GL_EQUAL, 1,  1);						// We Draw Only Where The Stencil Is 1
		glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);	

		for(int j=0;j < allStrokes.size();j++)
		{
			str = allStrokes[j].getStroke();

			if ((str->properties.texture == -1) && str->properties.textureFilename.length() > 0)
				str->properties.texture = loadTexture(str->properties.textureFilename);	

		}	
	}

	glEnable(GL_TEXTURE_2D);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _mainTex, 0);


	//drawPaperAndImage();


	glColor4f(1,1,1,1);
	glBindTexture(GL_TEXTURE_2D,_secondaryTex);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);
		glTexCoord2f(0,1);
		glVertex3f(0,_h,0);
		glTexCoord2f(1,1);
		glVertex3f(_w,_h,0);
		glTexCoord2f(1,0);
		glVertex3f(_w,0,0);
	glEnd();	


	//loop over all the layer objects and find any strokes currently being drawn
	for(int i=0;i<layerObjectList.size();i++)
	{		
		QList<ContourLabel>& layerStrokes = layerObjectList[i].getStrokes(curFrameIndex);
		bool beingDrawn =false;

		if (layerStrokes.size() > 0) 
			beingDrawn = layerStrokes[layerStrokes.size()-1].isBeingDrawn();

		if ((beingDrawn==true)&& (layerStrokes[layerStrokes.size()-1].getStroke()->properties.isVisible))
		{
			float fadeTransparency = 1;

			if (layerStrokes[layerStrokes.size()-1].getStroke()->properties.isFill)
				fadeTransparency = 0.5;


			layerStrokes[layerStrokes.size()-1].getStroke()->properties.alpha *= _userStrokeTransparency*fadeTransparency;
			layerStrokes[layerStrokes.size()-1].getStroke()->render();
			layerStrokes[layerStrokes.size()-1].getStroke()->properties.alpha /= _userStrokeTransparency*fadeTransparency;
		}
	}



	QImage saveimg;
	bool haveImage = true;

	try {
		saveimg = _fboVar->toImage();
	}
	catch( char * str ) {
                sprintf(_strbuf, "Exception raised: %s",str );
		LOG_STR_COUT(_strbuf);
		haveImage = false;
	}

	if (haveImage)
	{
	
		QPoint pt2(ScreenOffset.x()+_w*ScaleFactor, ScreenOffset.y()+_h*ScaleFactor);
		QRect rec(ScreenOffset.toPoint(),pt2);
		painter.drawImage(rec,saveimg);
	}

}


void StrokePainter::drawPaperAndImage()
{

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glColor4f(((float)_paperColor.red())/255.,((float)_paperColor.green())/255.,((float)_paperColor.blue())/255.,1);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,_paperTexture);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);
		glTexCoord2f(0,_paperHeightRatio);
		glVertex3f(0,_h,0);
		glTexCoord2f(_paperWidthRatio,_paperHeightRatio);
		glVertex3f(_w,_h,0);
		glTexCoord2f(_paperWidthRatio,0);
		glVertex3f(_w,0,0);
	glEnd();

	if (_imageTex > 0)
	{
		glColor4f(1,1,1,_imageTransparency);
		glBindTexture(GL_TEXTURE_2D,_imageTex);
		glBegin(GL_QUADS);
			glTexCoord2f(0,1);
			glVertex3f(0,0,0);
			glTexCoord2f(0,0);
			glVertex3f(0,_h,0);
			glTexCoord2f(1,0);
			glVertex3f(_w,_h,0);
			glTexCoord2f(1,1);
			glVertex3f(_w,0,0);
		glEnd();	
	}
}



void  StrokePainter::drawTexture(LayerObject& layer, int curFrameIndex,bool useTransparency)
{
	

	int tex = layer.getStrokeImageTexture(curFrameIndex) ;
		
	int state = layer.getStrokeImageState(curFrameIndex) ;

	//Free the texture if needed
	if (state==FREE_TEXTURE)
	{

		GLuint texture = (GLuint) std::abs(tex);
		glDeleteTextures( 1, &texture );
		layer.setStrokeImageTexture(curFrameIndex,0);
		layer.setStrokeImageState(curFrameIndex,NO_IMAGE);
	}	

	if ((state==CREATE_TEXTURE))
	{
		if (tex >=1)
		{
			GLuint texture = (GLuint)std::abs(tex);
			glDeleteTextures( 1, &texture );
		}

		const QRect& rect = layer.boundingBox(curFrameIndex);

		QImage img = layer.getStrokeImage(curFrameIndex);

		applyMedianFilter(img,layer.getStrokeImageMedianBlur());
		

		tex = loadStrokeImageTexture(img);
		if (tex ==0)
		{
			LOG_STR_COUT("problem loading texture");
			return;
		}
	
		layer.setStrokeImageTexture(curFrameIndex,tex);
		layer.setStrokeImageState(curFrameIndex,TEXTURE_EXISTS);
		state = TEXTURE_EXISTS;
	}

	if (state==TEXTURE_EXISTS)
	{

		Stroke str;
		str.properties = _interfaceSlp.strokeProperties;
		str.properties.endRadius=1;
		str.properties.startRadius=1;
		str.add(0,0);
		str.add(100,100);
		str.render();	
		if (useTransparency==true)
			glColor4f(1,1,1,_systemStrokeTransparency);
		else
			glColor4f(1,1,1,1);

		const QRect& rect = layer.boundingBox(curFrameIndex);
		//glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, tex, 0);
		//_fbo->drawTexture(QRect(rect.left(),_h-rect.bottom(),rect.width(), rect.height()),tex);		


		double wid = rect.width();
		double hei = rect.height();
		double left = rect.left();
		double top = _h-rect.top();


		glBindTexture(GL_TEXTURE_2D,tex);
		glBegin(GL_QUADS);
			glTexCoord2f(0,0);
			glVertex2i(left,top);
			glTexCoord2f(0,1);
                        glVertex2i(left,top-rect.height());
			glTexCoord2f(1,1);
                        glVertex2i(left+wid,top-hei);
			glTexCoord2f(1,0);
                        glVertex2i(left+wid,top);
		glEnd();

	}
}

void StrokePainter::loadTextureForCurrentStroke(QString& filename)
{
	_interfaceSlp.strokeProperties.textureFilename = filename;
	int t = loadTexture(filename);	
	_interfaceSlp.strokeProperties.texture = t;
	emit strokePropertiesChanged(_interfaceSlp.strokeProperties);
}


int StrokePainter::loadTexture(QString &filename)
{


	if (filename=="Brushes/paintbrush09.png")
		filename=QString("Brushes/paintbrush09.pgm");



	//if (filename=="Brushes/paintbrush09.pgm")
	//	filename=QString("Brushes/hayes2.pgm");

	int idx = filename.indexOf(QString("Brushes/"));
	filename = filename.right(filename.size() - idx);


	//Check if we've seen this texture before and 
	for(int i=0;i < _textureNames.size();i++)
	{
		if (filename == _textureNames[i])
		{
			return i;
		}
	}
    sprintf(_strbuf, "Loading stroke texture %s",filename.toAscii().data() );
	LOG_STR_COUT(_strbuf);

	GLuint texture,alphatexture;
	// allocate a texture name
	glGenTextures( 1, &texture );
	glGenTextures( 1, &alphatexture );

    // open texture data 
    QImage* im = new QImage(filename);  
	if ( im == NULL )
	{
		LOG_STR_COUT("Problem reading image");
		return -1;
	} 

	FImage teximg(im->width(),im->height(),1);


	for(int x = 0 ; x < im->width() ; x++)
    for(int y = 0 ; y <  im->height() ; y++) 
	{
        QRgb color = im->pixel(x,y);
        float lum = 0.299 * qRed(color) + 0.587 * qGreen(color) + 0.114 * qBlue(color) ;  
		teximg.setPixelValue( lum/255.,x,y);
	}

	FImage texsmooth;
	teximg.GaussianSmoothing(texsmooth,8,30);

	texsmooth.Multiplywith(teximg);
	texsmooth.normalize(texsmooth);
	texsmooth.imwrite(QString("images/texalpha.jpg"));
	teximg.imwrite(QString("images/teximg.jpg"));

	QString format=filename.right(4);
	QString alphaname=filename;
	alphaname.chop(4);

	bool alphaFileExists=false;
    QImage* alphaim = new QImage(alphaname+QString("_alpha")+format);  
	if (( alphaim != NULL )&&(alphaim->width()==im->width()) &&(alphaim->height()==im->height()))
	{
		alphaFileExists=true;
	} 

	unsigned char *data = new unsigned char [ 4 * im->width() * im->height() ] ; 
	assert( data ) ;   

	unsigned char *alphadata = new unsigned char [4*im->width() * im->height() ] ; 
	assert( alphadata ) ;   


	// calculate the intensity for each pix in the rgb data and copy to the rgba data. 
	for(int y = 0 ; y < im->height() ; y++)
	for(int x = 0 ; x <  im->width() ; x++) 
	{
		//QRgb color = im->pixel(col,row);

		float lum= teximg.val(x,y);
		float alpha = texsmooth.val(x,y);

		alpha = 1/ (1+ exp(-1*((alpha-0.5)*10)));

		alpha = alpha*255.;
		if (alpha < 0  )
		  alpha = 0 ;

		if ( alpha > 255 ) 
		 alpha = 255 ; 

		if (alphaFileExists)
		{
			QRgb color = alphaim->pixel(x,y);
			float lum = 0.299 * qRed(color) + 0.587 * qGreen(color) + 0.114 * qBlue(color) ;  
			alpha=lum;
		}

		data [ 4*(  y*im->width() + x ) + 0 ] = (unsigned char )(lum*255.); 
		data [ 4*(  y*im->width() + x ) + 1 ] = (unsigned char )(lum*255.); 
		data [ 4*(  y*im->width() + x ) + 2 ] = (unsigned char )(lum*255.); 
		data [ 4*(  y*im->width() + x ) + 3 ] = (unsigned char )alpha;



		alphadata [ 4*( y*im->width() + x ) + 0 ] = 255;
		alphadata [ 4*( y*im->width() + x ) + 1 ] = 255;
		alphadata [ 4*( y*im->width() + x ) + 2 ] = 255; 
		alphadata [ 4*( y*im->width() + x ) + 3 ] = (unsigned char )alpha;

		if (filename.indexOf("png") > -1)
		{
			QRgb color = im->pixel(x,y);

            data [ 4*(  y*im->width() + x ) + 2 ] =  qRed(color);
			data [ 4*(  y*im->width() + x ) + 1 ] =  qGreen(color); 
            data [ 4*(  y*im->width() + x ) + 0 ] =  qBlue(color);

			if (alphaFileExists)
			{
				QRgb color = alphaim->pixel(x,y);
				float lum = 0.299 * qRed(color) + 0.587 * qGreen(color) + 0.114 * qBlue(color) ;  
				alpha=lum;
			}
			else
				alpha=qAlpha(color);

			data [ 4*(  y*im->width() + x ) + 3 ] =  alpha;



			alphadata [ 4*(  y*im->width() + x ) + 0 ] = (unsigned char )255;
			alphadata [ 4*(  y*im->width() + x ) + 1 ] = (unsigned char )255;
			alphadata [ 4*(  y*im->width() + x ) + 2 ] = (unsigned char )255; 
			alphadata [ 4*(  y*im->width() + x ) + 3 ] = (unsigned char )alpha;
		}

	}

	QList<int> widths;
	for(int x = 0 ; x <  im->width() ; x++) 
	
	{

		int left=im->width();
		for(int y = 0 ; y < im->height() ; y++)
		{

			if (((int)alphadata [ 4*( y*im->width() + x ) + 3 ]) > 10)
			{
				left=y;
				y = im->height();
			}
		}

		int right=0;
		for(int y = im->height()-1 ; y >-1 ; y--)
		{

			if (((int)alphadata [ 4*( y*im->width() + x ) + 3 ]) > 10)
			{
				right=y;
				y = -1;
			}
		}

		if (right-left > 0)
			widths.push_back(right-left);
	}
	qSort(widths);

	float strokeRadius=1;
	if (widths.size()!=0)
		strokeRadius = ((float) widths[widths.size()/2]) / im->height();

    sprintf(_strbuf, "median stroke radius %f", strokeRadius);
	LOG_STR_COUT(_strbuf);


	glBindTexture( GL_TEXTURE_2D, texture ) ; 

	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE ) ;  
	//glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_DECAL ) ;  

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, im->width(), im->height(),0, GL_BGRA, GL_UNSIGNED_BYTE, data );

	//gluBuild2DMipmaps( GL_TEXTURE_2D, 0, im->width(), im->height(), GL_RGBA, GL_UNSIGNED_BYTE, data );


	glBindTexture( GL_TEXTURE_2D, alphatexture ) ; 

	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE ) ;  
	//glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_DECAL ) ;  

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, im->width(), im->height(),0, GL_BGRA, GL_UNSIGNED_BYTE, alphadata );


	delete im;
	delete [] data ; 
	delete [] alphadata;

	
	if (texture < _textureNames.size())
	{
		_textureNames.insert((int)texture, filename);
		_alphaTextureMap.insert((int)texture,(int)alphatexture);
		_textureWidthMap.insert((int)texture,strokeRadius);
	}
		
	else
	{
	
		return -1;

	}


	return texture;
}



void StrokePainter::loadPaperTexture(QString &filename)
{

	GLuint texture;
	// allocate a texture name
	glGenTextures( 1, &texture );

    // open texture data 
    QImage* im = new QImage(filename);  
	if ( im == NULL )
	{
		LOG_STR_COUT("Problem reading image"); 
		return;
	} 

	//printf("paper texture size %i %i\n", im->width(),im->height());

    sprintf(_strbuf, "Loading paper texture %s",filename.toAscii().data() );
	LOG_STR_COUT(_strbuf);

	unsigned char *data = new unsigned char [ 4 * im->width() * im->height() ] ; 
	assert( data ) ;   


	_paperWidthRatio = ((float)_w)/im->width();
	_paperHeightRatio = ((float)_h)/im->height();


	// calculate the intensity for each pix in the rgb data and copy to the rgba data. 
	for(int row = 0 ; row < im->height() ; row++)
    for(int col = 0 ; col <  im->width() ; col++) 
	{
		QRgb color = im->pixel(col,row);

		data [ 4*( row*im->width() + col ) + 2 ] =  qRed(color); 
		data [ 4*( row*im->width() + col ) + 1 ] =  qGreen(color); 
		data [ 4*( row*im->width() + col ) + 0 ] =  qBlue(color); 
		data [ 4*( row*im->width() + col ) + 3 ] =  qAlpha(color);
	}

	glBindTexture( GL_TEXTURE_2D, texture ) ; 

	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE ) ;  
	//glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_DECAL ) ;  

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, im->width(), im->height(),0, GL_BGRA, GL_UNSIGNED_BYTE, data );


	//printf("new paper texture created  %i\n",  texture);

	delete im;
	delete [] data ; 

	_paperTexture = texture;

	return ;
}




GLuint StrokePainter::loadStrokeImageTexture(const QImage& im)
{


	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
		return 0;
	}
	
	GLuint texture;
	// allocate a texture name
	glGenTextures( 1, &texture );


	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
		return 0;
	}


	glBindTexture( GL_TEXTURE_2D, texture ) ; 

	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE ) ;  
	//glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_REPLACE ) ;  

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );


	unsigned char *data = new unsigned char [ 4 * im.width() * im.height() ] ; 
	assert( data ) ; 


	// calculate the intensity for each pix in the rgb data and copy to the rgba data. 
	for(int row = 0 ; row < im.height() ; row++)
    for(int col = 0 ; col <  im.width() ; col++)
	{
        QRgb color = im.pixel(col,row);

        data [ 4*( row*im.width() + col ) + 2 ] = qRed(color);
		data [ 4*( row*im.width() + col ) + 1 ] = qGreen(color); 
        data [ 4*( row*im.width() + col ) + 0 ] = qBlue(color);
		data [ 4*( row*im.width() + col ) + 3 ] = qAlpha(color);
	}


    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, im.width(), im.height(),0, GL_BGRA, GL_UNSIGNED_BYTE, data );

	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
		glDeleteTextures(1,&texture);
	    delete [] data;
	    return 0;
	}

	//printf("new stroke texture created  %i\n",  texture);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE ) ;  

	delete [] data;
	
	return texture;
}




void StrokePainter::strokeStartWidthChanged(int startWidth)
{
	_interfaceSlp.strokeProperties.startRadius = startWidth;
	emit strokePropertiesChanged(_interfaceSlp.strokeProperties);
}
void StrokePainter::strokeEndWidthChanged(int endWidth)
{
	_interfaceSlp.strokeProperties.endRadius = endWidth;
	emit strokePropertiesChanged(_interfaceSlp.strokeProperties);
}


void StrokePainter::useImageColor(bool useImageColor)
{	
	_interfaceSlp.strokeProperties.useImageColor = useImageColor;
	emit strokePropertiesChanged(_interfaceSlp.strokeProperties);
}
void StrokePainter::useColor(QColor col)
{
	_interfaceSlp.strokeProperties.color = col;
	emit strokePropertiesChanged(_interfaceSlp.strokeProperties);
}
void StrokePainter::alphaChanged(double alpha)
{
	_interfaceSlp.strokeProperties.alpha = alpha;
	emit strokePropertiesChanged(_interfaceSlp.strokeProperties);
}



void StrokePainter::usePaperColor(QColor col)
{
	_paperColor = col;
}


void StrokePainter::guideStroke(bool isGuide)
{
	printf("new isGuide %i\n", isGuide);
	_interfaceSlp.strokeProperties.isGuide = isGuide;
	int guide = isGuide;
	emit strokePropertiesChanged("isGuide", QString::number(guide));
}
void StrokePainter::colorStroke(bool isColor)
{
	printf("new isColor %i\n", isColor);
	_interfaceSlp.strokeProperties.isColor = isColor;
	int color = isColor;
	emit strokePropertiesChanged("isColor", QString::number(color));
}

void StrokePainter::visibleStroke(bool isVisible)
{
	printf("new isVisible %i\n", isVisible);
	_interfaceSlp.strokeProperties.isVisible = isVisible;
	int visible = isVisible;
	emit strokePropertiesChanged("isVisible", QString::number(visible));
}

void StrokePainter::fillStroke(bool isFill)
{
	_interfaceSlp.strokeProperties.isFill = isFill;
	int fill = isFill;
	emit strokePropertiesChanged("isFill", QString::number(fill));
}

void StrokePainter::seedStroke(bool isSeed)
{
	printf("new isSeed %i\n", isSeed);
	_interfaceSlp.strokeProperties.isSeed = isSeed;
	int seed = isSeed;
	emit strokePropertiesChanged("isSeed", QString::number(seed));
}


void StrokePainter::createETFImage(int frame, QList<QImage>& imgList,QStringList& fileList,QString& path)
{
	assert((frame >=0) && (frame<_frames));

	QString name=fileList[frame];
	QString etfname = name.left(name.lastIndexOf(QChar('.')));
	
    QImage qangle(path+QString("etf/"+etfname+QString("-angle")+QString(".bmp")));

	if (qangle.isNull())
	{
		qangle = QImage(path+QString("etf/etfangle-")+QString::number(frame)+QString(".bmp"));
	}

	if (qangle.isNull())
	{
		qangle = QImage(path+QString("etfangle-")+QString::number(frame)+QString(".bmp"));
	}

	QImage qmag(path+QString("etf/"+etfname+QString("-gradmag")+QString(".bmp")));
	
	if (qmag.isNull())
	{
		qmag = QImage(path+QString("etf/etfgradmag-")+QString::number(frame)+QString(".bmp"));
	}

	if (qmag.isNull())
	{
		qmag = QImage(path+QString("etfgradmag-")+QString::number(frame)+QString(".bmp"));
	}

	if (!qangle.isNull() && !qmag.isNull())
	{

		FImage angle(_w,_h,1);
		FImage etfMag(_w,_h,1);

		angle.imread(qangle);
		etfMag.imread(qmag);

		//angle.Multiplywith(2*PI);
		//angle.Add(-1*PI);

		BiImage angleBi(_w,_h,1);
		BiImage magBi(_w,_h,1);

		for(int y=0;y<_h;y++)
		for(int x=0;x<_w;x++)
		{
			unsigned char mag = etfMag.val(x,y)*255.;
			unsigned char ang = angle.val(x,y)*255.;

			magBi.setPixelValue(mag,x,y);
			angleBi.setPixelValue(ang,x,y);
		}

		_etfMag[frame] = magBi;
		_etfAngle[frame] = angleBi;

        sprintf(_strbuf, "found etf file for frame %i",frame);
		LOG_STR_COUT(_strbuf);


		_etfCreated[frame] = true;

		return;
	}
	FImage etfX(_w,_h,1);
	FImage etfY(_w,_h,1);

	FImage angle(_w,_h,1);
	FImage etfMag(_w,_h,1);

    sprintf(_strbuf, "creating etf for frame %i. Don't worry, it only happens once :)", frame);
	LOG_STR_COUT(_strbuf);

	FImage pregradmag(_w,_h);
	FImage imgSmooth(_w,_h);

	FImage img(imgList[frame]);
	img.desaturate();

	img.GaussianSmoothing(imgSmooth,1,4);
	
	imgSmooth.gradientmag(pregradmag);
	pregradmag.normalize(etfMag);

	//get the inital tangent vectors
	imgSmooth.dx(etfY);
	imgSmooth.dy(etfX);

	etfY.Multiplywith(-1);

	for(int y=0;y<_h;y++)
	for(int x=0;x<_w;x++)
	{
		Vec2f currDir(etfX.val(x,y), etfY.val(x,y));

		currDir.Normalize();
		if ((currDir.x() ==0) && (currDir.y() ==0) && (etfMag.val(x,y) != 0))
		{

			etfMag.setPixelValue(0,x,y);
		}
		etfX.setPixelValue(currDir.x(),x,y);
		etfY.setPixelValue(currDir.y(),x,y);

	}

	FImage initialEtfMag = etfMag;

	int boxFilterRadius = 3;
	int neighborhoodRadius=5;



	//etf iterations
	for(int i=0;i < 3;i++)
	{	


		FImage nextEtfX(_w,_h);
		FImage nextEtfY(_w,_h);
		FImage nextEtfMag(_w,_h);

		double newGradX=0, newGradY=0,currX=0, currY=0;

		double addX=0,addY=0,dotprod=0;

		for(int x=neighborhoodRadius;x<_w-neighborhoodRadius;x++)
		for(int y=neighborhoodRadius;y<_h-neighborhoodRadius;y++)
		{
			newGradX=0;
			newGradY=0;

			currX = etfX.val(x,y);
			currY = etfY.val(x,y);
			double currGrad = initialEtfMag.val(x,y);

			double magSum = 0;
			double weightSum = 0;
			float weight = 0;


			//loop over 
			for(int j=x-neighborhoodRadius;j<=x+neighborhoodRadius;j++)
			for(int k=y-neighborhoodRadius;k<=y+neighborhoodRadius;k++)
			{
				float dist = (x-j)*(x-j) + (y-k)*(y-k);
		
				if (sqrt(dist) >= boxFilterRadius)
					continue;

				addX = etfX.val(j,k);
				addY = etfY.val(j,k);

				dotprod = currX*addX + currY*addY;

				weight = 0.5*(1+tanh(1*(initialEtfMag.val(j,k)-currGrad)))*abs(dotprod);

				addX*=weight;
				addY*=weight;

				if (dotprod <=0)
				{
					addX*=-1;
					addY*=-1;
				}

				newGradX += addX;
				newGradY += addY;

				weightSum += weight;
				magSum+=etfMag.val(j,k)*weight;
			}

			if (magSum/weightSum> 1)
			{
            sprintf(_strbuf, "over 1 %f %f", magSum, weightSum);
				LOG_STR_COUT(_strbuf);

				magSum = weightSum;
			}

			if (weightSum>0.1)
				nextEtfMag.setPixelValue(magSum/weightSum,x,y);
			else
				nextEtfMag.setPixelValue(0,x,y);

			float mag = sqrt(newGradX*newGradX + newGradY*newGradY);
			newGradX /= mag; 
			newGradY /= mag; 

			if (mag!= 0)
			{
				nextEtfX.setPixelValue(newGradX,x,y);
				nextEtfY.setPixelValue(newGradY,x,y);
			}
			else
			{
				nextEtfX.setPixelValue(0,x,y);
				nextEtfY.setPixelValue(0,x,y);
				nextEtfMag.setPixelValue(0,x,y);
			}
		}

		etfX = nextEtfX;
		etfY = nextEtfY;
		etfMag = nextEtfMag;
	}


    QDir dir(path);
    dir.mkdir(QString("etf"));
    etfMag.imwrite(path+QString("etf/"+etfname+QString("-gradmag")+QString(".bmp")));

	FImage etfangle(_w,_h,1);
	FImage angleView(_w,_h);
	float ang =0;
	for(int y=boxFilterRadius;y<_h-boxFilterRadius;y++)
	for(int x=boxFilterRadius;x<_w-boxFilterRadius;x++)
	{


		if (etfX.val(x,y) != 0)
			ang = atan2(etfY.val(x,y),etfX.val(x,y));

		if (ang < 0)
			ang = ang+PI;

		etfangle.setPixelValue(ang,x,y);
		angleView.setPixelValue(abs(ang),x,y);
	}
	etfangle.Add(PI);
	etfangle.Multiplywith(1/(2*PI));


    etfangle.imwrite(path+QString("etf/"+etfname+QString("-angle")+QString(".bmp")));
    //angleView.imwrite(path+QString("viewetfangle-")+QString::number(frame)+QString(".bmp"));

	BiImage angleBi(_w,_h,1);
	BiImage magBi(_w,_h,1);

	for(int y=0;y<_h;y++)
	for(int x=0;x<_w;x++)
	{
		unsigned char mag = etfMag.val(x,y)*255.;
		unsigned char ang = etfangle.val(x,y)*255.;

		magBi.setPixelValue(mag,x,y);
		angleBi.setPixelValue(ang,x,y);
	}


	_etfAngle[frame] = angleBi;
	_etfMag[frame] = magBi;

	_etfCreated[frame] = true;
}




void StrokePainter::numFramesChanged(int numFrames)
{
	_frames = numFrames;
	_etfCreated.resize(numFrames);
	_etfAngle.resize(numFrames);
	_etfMag.resize(numFrames);
	_frameTextures.resize(_frames);

	for(int i=0; i < numFrames;i++)
	{
		_etfCreated[i] = false;
		_frameTextures[i] = 0;
		
		//_etfAngle[i] = FImage();
		//_etfMag[i] = FImage();

	}	

}

void StrokePainter::frameChanged(const QImage* m_Image,const QImage* m_NextImage)
{

	try{

		if (_fbo ==0)
			return;

		if (_imageTex > 0)
			glDeleteTextures( 1, &_imageTex );
		
		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		_imageTex = loadStrokeImageTexture(*m_Image);

		_redraw=true;

	}
	catch(...) 
	{
        sprintf(_strbuf, "ERROR: Exception raised in frame change" );
		LOG_STR_COUT(_strbuf);
	}
}


void StrokePainter::deleteFrameTextures()
{
	for(int i=0;i <=_frames;i++)
	{
		if (_frameTextures[i] > 0)
		{	
			GLuint tex = _frameTextures[i];
			glDeleteTextures( 1, &tex );
			_frameTextures[i]=0;
		}
	}
}


//signal from the video labeller to generate strokes for a given layer in this 
//we will generate strokes based on the current stroke properties, filling and motion
//parameters passed from the paint options widget
//the rendered strokes are stored in the layer object
void StrokePainter::generateStrokes(QString& path,LayerObject& layer,  QList<LayerObject>& allLayers, QList<QImage>& imgList,QStringList& fileList,int curFrameIndex,int generateId,StrokeLayerProperties *slp,bool fromScript)
{

	QString currtime =QDateTime::currentDateTime().toString();

    sprintf(_strbuf,"Starting stroke generation at %s",currtime.toAscii().data() );
    LOG_STR_COUT(_strbuf);
	
	if (_fbo==0)
	{
		initializePainter((imgList[curFrameIndex]));
	}

	//bind the fixed size fbo
	if (_fbo->bind() == false)
	{
		QMessageBox::warning(this,"Warning","Error binding the framebuffer object. Does your video card support frame buffer objects?");
		
        sprintf(_strbuf, "Error binding the framebuffer object." );
		LOG_STR_COUT(_strbuf);
		return;
	}
	


	//user the interface stroke layer properties if none have been provided
	if (slp==0)
	{
		_slp=_interfaceSlp;
		_slp.strokeProperties = _interfaceSlp.strokeProperties;
	}
	else
	{
		_slp=*slp;

		//check that we've loaded the stroke texture
		//if ((_slp.strokeProperties.texture == -1) && _slp.strokeProperties.textureFilename.length() > 0)
		_slp.strokeProperties.texture = loadTexture(_slp.strokeProperties.textureFilename);	
	}

	if (_slp.strokeProperties.texture<=0)
	{
		sprintf(_strbuf, "Problem with stroke texture %i" ,_slp.strokeProperties.texture);
		LOG_STR_COUT(_strbuf);
		return;
	}

	initializeGL();
	resizeGL(_w,_h,1);

	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}

	//if we're running this interactively, default to the requested frames from
	//the interface
	if (fromScript==false)
	{
		_slp.requestedEndFrame=_interfaceSlp.requestedEndFrame;
		_slp.requestedStartFrame=_interfaceSlp.requestedStartFrame;
	}

	int endFrame = min(_slp.requestedEndFrame,layer.endFrame()-1);
	int startFrame = max(_slp.requestedStartFrame,layer.startFrame()+1);

	_slp.startFrame = startFrame;
	_slp.endFrame = endFrame;
	_slp.layerName = layer.layerName();
	_slp.generationId=generateId;

    _slp.logLayerProperties();


	_readPixelsTiming=false;

	if (endFrame<=startFrame)
	{
		QMessageBox::warning(this,"Warning","Layer does not exist for enough frames to generate strokes");
		return;
	}

	if (_slp.maxStrokeLength<=_slp.minStrokeLength)
	{
		QMessageBox::warning(this,"Warning","Invalid parameters...stroke min length greater or equal than max length");
		return;
	}

	if (_slp.maxBrightness<_slp.minBrightness)
	{
		QMessageBox::warning(this,"Warning","Invalid parameters...min brightness greater than max brightness ");
		return;
	}
	if (_slp.maxFrames<_slp.minFrames)
	{
		QMessageBox::warning(this,"Warning","Invalid parameters...min frames greater than max frames ");
		return;
	}
	if (_slp.imageGradientFactor + _slp.boundaryFactor + _slp.userStrokesFactor + _slp.fixedOrientationFactor==0)
	{
		QMessageBox::warning(this,"Warning","Invalid parameters...check filling parameter weights");
		return;
	}
	if (_slp.imageFlowFactor+_slp.boundaryFlowFactor+_slp.userFlowFactor+_slp.fixedFlowFactor==0)
	{
		QMessageBox::warning(this,"Warning","Invalid parameters...check motion parameter weights");
		return;
	}

	if (_slp.endFrame-_slp.startFrame<_slp.minFrames)
	{
		QMessageBox::warning(this,"Warning","Invalid parameters...min frames less than the number of frames specified");
		return;
	}



    sprintf(_strbuf,"Generating Strokes From %i To %i", startFrame,endFrame);
	LOG_STR_COUT(_strbuf);

	emit generatingStrokes(true);

	bool saved=false;
	try 
	{
		bool haveInputs = false;


		if ((inp.startFrame != startFrame) || (inp.endFrame != endFrame) || (QString::compare(inp.layerName,layer.layerName())!=0))//|| ( _slp.optimizeStrokeWidths && (inp.edges.size()==0))
		//if ((recreateStroke) || (inp.startFrame != startFrame) || (inp.endFrame != endFrame) || (QString::compare(inp.layerName,layer.layerName())!=0))
		{
			inp.clear();
		}
		else 
		{
			if (inp.visited.size()<endFrame-startFrame)
			{
				inp.clear();
				sprintf(_strbuf,"Problem with the inputs...");
				LOG_STR_COUT(_strbuf);
			}
			else
			{
				for(int i=0;i <=endFrame-startFrame;i++)
					inp.visited[i].setAllZeros();

				haveInputs = true;
			}
		}

		inp.startFrame = startFrame;
		inp.endFrame = endFrame;
		inp.layerName = layer.layerName();

		float strokeRad = ((float)_slp.strokeProperties.startRadius + _slp.strokeProperties.endRadius)/2.0;
		float avgRadius = (((float)strokeRad)*_textureWidthMap.value((int)_slp.strokeProperties.texture));


        _gaussianMask = getGaussianFilter(min((double)avgRadius,15.0)*_slp.radiusFactor,&_gaussianMaskSize);


         QTime start = QTime::currentTime();

		 _slp.date=QDate::currentDate().toString();
		 _slp.startTime=start.toString(Qt::TextDate);



		//if we're starting a new input, then 

 		for(int i=0;i <_frameTextures.size();i++)
		{		
			//wipe out other frame textures to save memory
			if (((i < startFrame) || (i > endFrame)) &&(_frameTextures[i] > 0))
			{	
				GLuint tex = _frameTextures[i];
				glDeleteTextures( 1, &tex );
				_frameTextures[i]=0;
			}
		}

        sprintf(_strbuf,"Getting the inputs for the stroke generation...");
		LOG_STR_COUT(_strbuf);

		for(int i=startFrame;i <=endFrame;i++)
		{		

			//create a new blank texture if necessary
			if (_frameTextures[i] == 0)
			{	
				QImage blank(_w,_h,QImage::Format_ARGB32_Premultiplied);
				_frameTextures[i] = loadStrokeImageTexture(blank);
				if (_frameTextures[i] ==0)
				{
					deleteFrameTextures();
					return;
				}
			}
			//bind the texture for this frame and clear it
			glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[i], 0);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			if ((_errCode = glGetError()) != GL_NO_ERROR) {
				sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
				LOG_STR_COUT(_strbuf);

				deleteFrameTextures();
				return;
			}
		}

	
		if (!haveInputs)
		{
			if (getInputs(layer,imgList,fileList,path,startFrame,endFrame)==false)
			{
				inp.clear();
				return;
			}
				
		}

		bool anySeedStrokes=false;
		getUserStrokeInputs(allLayers,layer,imgList,path,startFrame,endFrame,&anySeedStrokes);
		
		if ((!anySeedStrokes)&&(_slp.fillUserSeed))
		{
			QMessageBox::warning(this,"Warning","You have selected to seed from user strokes but none are drawn in this frame range for this region");
			return;
		}
		
		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		_slp.getInputTime=start.elapsed();

	
		saved= fillLayerWithStrokes(layer,allLayers,  imgList,generateId);

		
		_slp.overallTime=start.elapsed();


        sprintf(_strbuf,"It took %i mseconds for getting the inputs.", _slp.getInputTime );
		LOG_STR_COUT(_strbuf);
        sprintf(_strbuf,"It took %i mseconds overall.",_slp.overallTime);
		LOG_STR_COUT(_strbuf);
	}
	catch(...) 
	{
		

        sprintf(_strbuf,"ERROR: Exception raised in stroke creation. Probably out of memory...clearing stored inputs. If this doesn't work, try saving and restarting" );
		LOG_STR_COUT(_strbuf);

		//with a lot of frames, we may have run out of memory...

		inp.clear();

		for (int i=0;i < _frames;i++)
		{
			if (_etfCreated[i]==true)
			{
				_etfMag[i] = BiImage();
				_etfAngle[i] = BiImage();
				_etfCreated[i]=false;
			}

			//create a new blank texture if necessary
			if (_frameTextures[i] > 0)
			{	
				GLuint tex = _frameTextures[i];
				glDeleteTextures( 1, &tex );
				_frameTextures[i]=0;

				
				if ((_errCode = glGetError()) != GL_NO_ERROR) {
					sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
					LOG_STR_COUT(_strbuf);
				}
			}

		}
	}

	emit generatingStrokes(false);

	_redraw = true;

	if (saved)
		layer.saveGeneratedStrokesParameters(generateId,_slp);

	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _mainTex, 0);
}


bool StrokePainter::getInputs(LayerObject& layer, QList<QImage>& imgList,QStringList& fileList,QString& path, int startFrame, int endFrame)
{
	
	QProgressDialog progress("Getting inputs (may take longer the first time)...", "Cancel", 0, endFrame-startFrame, this);


    sprintf(_strbuf, "Input directory %s", path.toAscii().data() );
	LOG_STR_COUT(_strbuf);
	for(int i=startFrame;i<= endFrame;i++)
	{

		progress.setValue(i-startFrame);

		qApp->processEvents();

        if (progress.wasCanceled())
		{
			LOG_STR_COUT("Stopped by user");
            return false;
		}

        sprintf(_strbuf, "getting inputs for frame %i", i);
		LOG_STR_COUT(_strbuf);

		if (_etfCreated[i] == false)
		{
			createETFImage(i,imgList,fileList,path);
		}

		QDir dir(path);
		dir.mkdir(QString("edge"));

		QString name=fileList[i];
		QString fname = name.left(name.lastIndexOf(QChar('.')));


		BiImage im(imgList[i]);

		inp.imgs.push_back(im);

		
	
		BiImage cannyedge;
	
		cannyedge.imread(path+QString("edge/"+fname+QString("-nonmax")+QString(".png")));

		if (cannyedge.width()==0)
		{
			im.justNonMaxSuppression(cannyedge, 0.5);
			cannyedge.imwrite(path+QString("edge/"+fname+QString("-nonmax")+QString(".png")));
		}

		inp.edges.push_back(cannyedge);
	

	
		BinaryImage visited(_w,_h);
		inp.visited.push_back(visited);



		//Check that we have all the necessary boundary and flow flow fields
		if (layer.boundaryImageCreated(i) == false)
		{
			layer.CreateBoundaryDistanceImage(i,_w,_h);
		}
	}

	if (layer.boundaryImageCreated(startFrame) == false)
		layer.CreateBoundaryDistanceImage(startFrame,_w,_h);
	if (layer.boundaryImageCreated(endFrame) == false)
		layer.CreateBoundaryDistanceImage(endFrame,_w,_h);

	return true;
}


void StrokePainter::getUserStrokeInputs( QList<LayerObject>& allLayers,LayerObject& layer, QList<QImage>& imgList,QString& path, int startFrame, int endFrame,bool *anySeedStrokesForAllFrames)
{

	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}

	inp.usrPts.resize(_frames);
	inp.usrDisp.resize(_frames);
	inp.usrFlowPts.resize(_frames);
	inp.usrFlowDisp.resize(_frames);
	inp.usrReverseFlowDisp.resize(_frames);

	inp.colorStrokeImgs.resize(_frames);
	inp.colorStrokeAlphaImgs.resize(_frames);
	inp.seedStrokeImgs.resize(_frames);

	*anySeedStrokesForAllFrames=false;
	for(int i=startFrame;i<= endFrame;i++)
	{

        sprintf(_strbuf, "getting user stroke inputs for frame %i", i);
		LOG_STR_COUT(_strbuf);

		QList<ContourLabel>& userStrokes =layer.getStrokes(i);

		QVector<Vec2f> userPts;
		QVector<Vec2f> userDisp;

		QVector<Vec2f> userFlowPts;
		QVector<Vec2f> userFlowDisp;
		QVector<Vec2f> userReverseFlowDisp;


		layer.createStrokeKDTree(i,true);
		//layer.createStrokeKDTree(i,false);


		int colorCount = 0;

        sprintf(_strbuf, "getting guide stroke inputs for frame %i", i);
		LOG_STR(_strbuf);

		
		for (int j=0; j < userStrokes.size();j++)
		{

			QVector<Point> *pointList = userStrokes[j].getStroke()->getLimitCurve();
		
			if ((userStrokes[j].IsGuideStroke() == false) || (userStrokes[j].deleted()==true))
				continue;

			//printf("size of point list %i\n",pointList->size() );
			for (int k=0;k < pointList->size();k++)
			{
				userPts.push_back(Vec2f(pointList->at(k).x(),_h-pointList->at(k).y()));
				Vec2f disp;
				if (k>0)
				{
					disp.set_x(pointList->at(k).x()-pointList->at(k-1).x());
					disp.set_y((_h-pointList->at(k).y())-(_h-pointList->at(k-1).y()));
				}
				else
				{
					disp.set_x(pointList->at(k+1).x()-pointList->at(k).x());
					disp.set_y((_h-pointList->at(k+1).y())-(_h-pointList->at(k).y()));
					disp *= (float)0.01;
				}
				//disp.Normalize();
				userDisp.push_back(disp);
			}


			QList<QPointF> pts =userStrokes[j].PointList();

			for(int k=0;k < pts.size();k++)
			{
				userFlowPts.push_back(Vec2f(pts[k].x(), pts[k].y()));
			}

			ContourLabel *nextStroke = userStrokes[j].getNext();

			
			if (nextStroke != 0)
			{
				nextStroke->getStroke()->render();

				QList<QPointF> nextPts = nextStroke->PointList();
				assert(pts.size() == nextPts.size());

				for(int k=0;k < pts.size();k++)
				{
		
					userFlowDisp.push_back(Vec2f(nextPts[k].x()-pts[k].x(), nextPts[k].y()-pts[k].y()));
				}		
			}

			ContourLabel *prevStroke = userStrokes[j].getPrev();
			if (prevStroke != 0)
			{	
				prevStroke->getStroke()->render();
				
				QList<QPointF> prevPts = prevStroke->PointList();
				assert(pts.size() == prevPts.size());

				for(int k=0;k < pts.size();k++)
				{
					userReverseFlowDisp.push_back(Vec2f(prevPts[k].x()-pts[k].x(), prevPts[k].y()-pts[k].y()));
				}		
			}
		}

		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		inp.usrPts[i-startFrame] = userPts;
		inp.usrDisp[i-startFrame] = userDisp;

		inp.usrFlowPts[i-startFrame] = userFlowPts;
		inp.usrFlowDisp[i-startFrame] = userFlowDisp;
		inp.usrReverseFlowDisp[i-startFrame] = userReverseFlowDisp;

        sprintf(_strbuf, "getting color stroke inputs for frame %i", i);
		LOG_STR(_strbuf);


		_fbo->bind();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		bool anyColorStrokes = false;

		glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ONE);
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER,0.1);


		Stroke *str;

		for (int j=0; j < userStrokes.size();j++)
		{
			ContourLabel& contour = userStrokes[j];
			bool del = userStrokes[j].deleted();

			if ((userStrokes[j].IsColorStroke() == false) || (userStrokes[j].deleted()==true))
				continue;

			anyColorStrokes = true;

			str = userStrokes[j].getStroke();
			if ((str->properties.texture == -1) && str->properties.textureFilename.length() > 0)
				str->properties.texture = loadTexture(str->properties.textureFilename);	

			str->render();
		}


		if (anyColorStrokes)
		{

		
			QImage img = _fbo->toImage();
			QImage alphaImg= img.alphaChannel();
		
			if(img.format()!=QImage::Format_RGB32)
			{
				QImage temp=img.convertToFormat(QImage::Format_RGB32);
				img=temp;
			}

			BiImage im(img);


			if(alphaImg.format()!=QImage::Format_RGB32)
			{
				QImage temp=alphaImg.convertToFormat(QImage::Format_RGB32);
				alphaImg=temp;
			}
			
			BiImage imAlpha(alphaImg);

			imAlpha.desaturate();

			inp.colorStrokeImgs[i-startFrame] =im;
			inp.colorStrokeAlphaImgs[i-startFrame] = imAlpha;

			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		}
		else
		{
			if (inp.colorStrokeImgs[i-startFrame].width()!=0)
			{
				inp.colorStrokeImgs[i-startFrame] = BiImage();
				inp.colorStrokeAlphaImgs[i-startFrame] = BiImage();
			}
		}



		if (_slp.fillUserSeed==true)
		{

			sprintf(_strbuf, "getting seed stroke inputs for frame %i", i);
			LOG_STR(_strbuf);

			bool anySeedStrokes=false;

			_fbo->bind();
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			for (int j=0; j < userStrokes.size();j++)
			{

				if ((userStrokes[j].IsSeedStroke() == false) || (userStrokes[j].deleted()==true))
					continue;

				anySeedStrokes = true;

				str = userStrokes[j].getStroke();
				if ((str->properties.texture == -1) && str->properties.textureFilename.length() > 0)
					str->properties.texture = loadTexture(str->properties.textureFilename);	
				str->render();
			}


			if (anySeedStrokes)
			{

				*anySeedStrokesForAllFrames=true;

				QImage img = _fbo->toImage().alphaChannel();
	
				QImage monoimg=img.convertToFormat(QImage::Format_Mono);
				//monoimg.save(path+QString("seedStrokeImg-")+QString::number(i)+QString(".png"));

				inp.seedStrokeImgs[i-startFrame] = monoimg;

			}
			else
			{
				if (inp.seedStrokeImgs[i-startFrame].width()!=0)
				{
					inp.seedStrokeImgs[i-startFrame] = QImage();
				}
			}
		}

		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		glDisable(GL_ALPHA_TEST);

	}
}


/*
bool StrokePainter::feedForwardFillLayerWithStrokes(LayerObject& layer,QList<LayerObject>& allLayers,QList<QImage>& imgList, int generateId)
{
    sprintf(_strbuf, "Starting stroke creation");
	LOG_STR_COUT(_strbuf);

    QTime startGen = QTime::currentTime();


    QVector<QVector<Stroke*> > strokeList(inp.endFrame-inp.startFrame+1);
	QVector<QVector<pair<int,pair<int,int> > > > pointQueue(inp.endFrame-inp.startFrame+1);

	_fbo->bind();
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[inp.startFrame], 0);

	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}

	long int pointQueueSize = 0;

	QVector<bool>firstAdded (inp.endFrame-inp.startFrame+1);

	FImage gradImg;
	FImage imgSmooth(_w,_h);

	FImage strokeGrad;
	FImage strokeSmooth;

    sprintf(_strbuf, "Seed Order: %s" ,qPrintable((_slp.seedOrder)) );
	LOG_STR_COUT(_strbuf);



	QProgressDialog seedProgress("Choosing stroke start points...", "Cancel", 0, inp.endFrame-inp.startFrame, this);


	
	//Add all the points we'll draw to the list...
	for(int i=inp.startFrame;i<=inp.endFrame;i++)
	//for(int i=inp.startFrame;i<=inp.startFrame+1;i++)
	//for(int i=inp.endFrame-1;i>=inp.endFrame-2;i--)
	//for(int i=(inp.startFrame+inp.endFrame)/2;i<=(inp.startFrame+inp.endFrame)/2;i++)
	{

		seedProgress.setValue(i-inp.startFrame);
		qApp->processEvents();

        if (seedProgress.wasCanceled())
		{
			LOG_STR_COUT("Stopped by user");
            return false;
		}

		if ((_slp.fillUserSeed)&& (inp.seedStrokeImgs[i-inp.startFrame].width()==0))
			continue;

		FImage *strokeImg = new FImage(layer.getStrokeImage(i));
		strokeImg->GaussianSmoothing(strokeSmooth,1,4);
		delete strokeImg;
		
		if ((_slp.seedOrder=="Image Canvas Edge Difference")&&(layer.getStrokeImage(i).width() > 0))
		{
			strokeSmooth.gradientmag(strokeGrad);
		}

		if ((_slp.seedOrder=="Edge") || (_slp.seedOrder=="Image Canvas Edge Difference"))
		{
			inp.imgs[i-inp.startFrame].GaussianSmoothing(imgSmooth,1,4);
			imgSmooth.gradientmag(gradImg);
		}

		int top = layer.boundingBox(i).top();
		int bottom = layer.boundingBox(i).bottom();
		int left = layer.boundingBox(i).left();
		int right = layer.boundingBox(i).right();


		for(int x=left+1;x<right-1;x++)
		for(int y=top+1;y<bottom-1;y++)
		{

			//if ((_slp.fillUserSeed)&& ((x < 0) || (y<0) || (x > inp.seedStrokeImgs[i-inp.startFrame].width()-1)|| (y > inp.seedStrokeImgs[i-inp.startFrame].height()-1))
			//continue;


			inp.visited[i-inp.startFrame].setPixel(x,y,false);
			

			int vis;
			if (_slp.startInOccludedRegions )	
				vis = layer.Mask(i).getPixel(x,y);
			else
				vis = layer.visibleMask(i).getPixel(x,y);

			if (vis==0)
				continue;
			
			if ((_slp.fillUserSeed)&&(qRed(inp.seedStrokeImgs[i-inp.startFrame].pixel(x,y))==0))
				continue;

			int pixAlpha = 0;

			bool choosepix = (rand()%5==1)&& (rand()%5 ==1) ;
			//bool choosepix = 1 ;
			float currRed=-1, currGreen=-1,currBlue=-1;
			float colDist=-1;

			if (layer.getStrokeImage(i).width() > 0)
			{
				
				QColor pixelCol(strokeSmooth.val(x-left,y-top));

			
				pixAlpha = qAlpha(layer.getStrokeImage(i).pixel(x-left,y-top));


				 if ((_slp.strokeProperties.useImageColor) && ((_slp.fillColorDifference)||(_slp.seedOrder=="Color Difference")))
				{

					getCombinedImageAndUserStrokeColor(i-inp.startFrame, x, y,_gaussianMask,_gaussianMaskSize,layer.visibleMask(i),&currRed, &currGreen, &currBlue);
					QColor currCol(currRed,currGreen,currBlue);
					colDist = getColorDist(pixelCol,currCol);
					//
					if (_misc4==1)
						colDist = getColorDifference(i-inp.startFrame, x,y, _gaussianMask,_gaussianMaskSize,pixelCol);
					
					if (_slp.fillColorDifference)
					{
						//if (( colDist> _slp.colorDifference) && (pixAlpha>50))
						if (( colDist> _slp.colorDifference))
							choosepix = (rand()%3==1)&& (rand()%3 ==1);
						else
							continue;
					}
				}
			}
			if ((_slp.fillUserSeed)||((i > _startHeavySample && i < _endHeavySample))) //|| (_slp.fillEmptyRegions))
				 choosepix = (rand()%2==1)&& (rand()%2 ==1) ;

			if (_slp.fillEmptyRegions==true)
			{
				if (pixAlpha<50)
					choosepix = 1;
				else
					choosepix = 0;
			}

			if (choosepix==0)
				continue;


			int brightness=0;
			if (currRed == -1)
			{
				getCombinedImageAndUserStrokeColor(i-inp.startFrame, x, y,_gaussianMask,_gaussianMaskSize,layer.visibleMask(i),&currRed, &currGreen, &currBlue);
			}
			brightness =currRed*0.3 + currGreen*0.59+ currBlue*0.11;

			if ((_slp.minBrightness != 0) || (_slp.maxBrightness != 255))
			{
				if ((brightness < _slp.minBrightness) || (brightness > _slp.maxBrightness) )
				{
					continue;
				}
			}
		



			int sortDist = rand();

			if (_slp.seedOrder=="Light to Dark")
				sortDist= -1*brightness;
			if (_slp.seedOrder=="Dark to Light")
				sortDist= brightness;
			if (_slp.seedOrder=="Edge")
			{
				sortDist= -1*gradImg.val(x,y);
			}
			if (_slp.seedOrder=="Image Canvas Edge Difference")
			{	
				sortDist= -1*abs(strokeGrad.val(x-left,y-top)-gradImg.val(x,y));
			}
			if ((_slp.seedOrder=="Color Difference") && (colDist>-1))
				sortDist = -1*colDist*1000.;

			pair<int,int> p1(x,y);
                        pair<int,pair<int,int> > p2(sortDist,p1);
			pointQueue[i-inp.startFrame].push_back(p2);

			firstAdded[i-inp.startFrame] = true;
			pointQueueSize++;
			
		}

		
		qSort(pointQueue[i-inp.startFrame]);
	}



	QVector<int> seedFrames;

	for (int i=0;i < firstAdded.size();i++)
	{
		if (firstAdded[i] ==true)
			seedFrames.push_back(i);
	}

	
    sprintf(_strbuf, "start queue size %i", pointQueueSize);
	LOG_STR_COUT(_strbuf);

	
	//int *strokeCount = new int[inp.endFrame-inp.startFrame+1];
	//for(int i=0;i<= inp.endFrame-inp.startFrame;i++)
	//	strokeCount[i] = 0;

	int count = 0;
	int success = 0;

	float strokeRad = ((float)_slp.strokeProperties.startRadius + _slp.strokeProperties.endRadius)/2.0;
	_slp.actualRadius=((strokeRad)*_textureWidthMap.value((int)_slp.strokeProperties.texture));
	_slp.gridSize = max((int)_slp.actualRadius,1);

	if (_slp.gridSize%1==1)
		_slp.gridSize--;

    sprintf(_strbuf, "gridsize %i", _slp.gridSize);
	LOG_STR_COUT(_strbuf);

	_imgDensity = new unsigned char[_slp.gridSize*_slp.gridSize*4];
	


    sprintf(_strbuf, "stroke density %f", _slp.strokeDensity);
	LOG_STR_COUT(_strbuf);


	QVector<Stroke*> createdStrokes(inp.endFrame-inp.startFrame+1);

	StrokeGenInfo sgi;

	QProgressDialog progress("Generating stroke layer...", "Stop now.", 0, _slp.maxStrokes, this);
	int highestFrameStrokeCount=0;


	int frame=0, col,outerCount = 0,startedStroke = 0;
	//Go over all the start points
	//while (pointQueueSize != 0)
	for(int f=inp.startFrame;f < inp.endFrame;f++)
	{
		int frame = f-inp.startFrame;

		sprintf(_strbuf, "Now at frame %i", frame);
		LOG_STR_COUT(_strbuf);


		if (frame >0)
		{
			for(int i=0;i < strokeList[frame-1].size();i++)
			{
				
				Stroke *newStroke = startStrokeFeedForward(0,0,strokeList[frame-1][i],f, layer,allLayers,createdStrokes,sgi,pointQueue,&pointQueueSize);

				
				if (newStroke!=0)
				{
					strokeList[frame].push_back(newStroke);
					//newStroke->nextStroke=0;

					assert(newStroke->properties.texture==_slp.strokeProperties.texture);
				}
				else
					assert(strokeList[frame-1][i]->nextStroke==0);

			}

		}
		

	

		while (pointQueue[frame].size() != 0)
		{

			if (pointQueueSize%1000==0)
			{
				if ((_slp.generationTimelimit >0) && (_slp.generationTimelimit*1000<startGen.elapsed()))
				{
					sprintf(_strbuf, "reached max generation time (%i s) so gave up", _slp.generationTimelimit);
					LOG_STR_COUT(_strbuf);
					pointQueue.clear();
					pointQueueSize=0;
					continue;
				}
				
				sprintf(_strbuf, "current queue size %i", pointQueueSize);
				LOG_STR_COUT(_strbuf);
			}

			pair<int,int> p = pointQueue[frame][0].second;
			int x = p.first;
			int y = p.second;
			pointQueue[frame].pop_front();
			pointQueueSize--;

			if (inp.visited[frame].getPixel(x,y) ==false)
			{

				//remove the point after processing
				inp.visited[frame].setPixel(x,y,true);

				bool densityCheck = true;
				if ((_slp.strokeDensity < 0.1) || (_slp.fillEmptyRegions))
				{

					if ((_errCode = glGetError()) != GL_NO_ERROR) {
						sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
						LOG_STR_COUT(_strbuf);
					}

					_fbo->bind();
					//printf("here\n");
					glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[frame+inp.startFrame], 0);

					if ((_errCode = glGetError()) != GL_NO_ERROR) {
						sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
						LOG_STR_COUT(_strbuf);
					}

					float nearbyStrokeDensity = 0;
					
					if (_readPixelsTiming)
						t.start();

					glReadPixels(x-_slp.gridSize/2,_h-(y+_slp.gridSize/2),_slp.gridSize,_slp.gridSize, GL_BGRA, GL_UNSIGNED_BYTE, (void*) _imgDensity);

					if (_readPixelsTiming)
						printf("glReadPixels took %i msec\n", t.elapsed());


					for(int i=3;i<_slp.gridSize*_slp.gridSize*4;i+=4)
							nearbyStrokeDensity+=(float)_imgDensity[i];

					nearbyStrokeDensity /= (double)_slp.gridSize*_slp.gridSize*255;

		
					if ((_errCode = glGetError()) != GL_NO_ERROR) {
						sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
						LOG_STR_COUT(_strbuf);
					}

					if ((nearbyStrokeDensity > _slp.strokeDensity) && (_slp.fillEmptyRegions==false))
					{			
						densityCheck = false;
					}

					if ((_slp.fillEmptyRegions) && (nearbyStrokeDensity!= 0))
						densityCheck = false;
				}

				//If the density for this pixel is less than the threshold, start a new stroke
				//if (nearbyStrokeDensity <= (double)_slp.strokeDensity/ ((double)gridSize*gridSize))
				if (densityCheck == true)
				{
					startedStroke++;



					Stroke *newStroke = startStrokeFeedForward(x,y,0,f, layer,allLayers,createdStrokes,sgi,pointQueue,&pointQueueSize);


					if (newStroke!=0)
					{
						assert(newStroke->nextStroke==0);
						assert(newStroke->age==1);
						assert(newStroke->dying==false);
						success++;

						strokeList[frame].push_back(newStroke);
				
						if (strokeList[frame].size()%200 ==5)
						{
							
							sprintf(_strbuf, "stroke count for frame %i %i",f, strokeList[frame].size());
							LOG_STR_COUT(_strbuf);
						}

						if (strokeList[frame].size()>highestFrameStrokeCount)
						{
							highestFrameStrokeCount=strokeList[frame].size();
							progress.setValue(highestFrameStrokeCount);
						}

						if ((strokeList[frame].size() > _slp.maxStrokes))
						{
							LOG_STR_COUT("found max strokes..");
							//pointQueueSize = 0;
							//for(int i=0;i < pointQueue.size();i++)
							//{
								pointQueue[frame].clear();
							//}
						}


					}

					
				}
			}
		}
	}


	sgi.print();
	sgi.clear();


	int numStrokes=0;
	for(int f=inp.startFrame;f < inp.endFrame;f++)
	{
		int frame = f-inp.startFrame;

		for (int i=0;i < strokeList[frame].size();i++)
		{
			Stroke *str = strokeList[frame][i];

			try
			{
				if ((strokeList[frame][i]->ageChecked==false) )
				{
					bool pastMinAge=ageFade(str,f);
					if (pastMinAge)
						numStrokes++;		
				}

				if ((strokeList[frame][i]->nextStroke!=0)&&(getColorDist(strokeList[frame][i]->properties.color,strokeList[frame][i]->nextStroke->properties.color)>0.5))
				{
					strokeList[frame][i]->nextStroke->properties.alpha=0;
					strokeList[frame][i]->properties.alpha=0;
				}


			}
			catch(...) 
			{
				sprintf(_strbuf, "ERROR: Exception raised in ageFade");
				LOG_STR_COUT(_strbuf);
			}
			
		}
	}


	//printf("finished with %i strokes\n",strokeCount);

	delete [] _imgDensity;
	//delete [] strokeCount;

	_slp.strokeGenTime=startGen.elapsed();

    QTime afterGen;
    afterGen.start();
	


	QImage densityImg = _fbo->toImage();
	densityImg.save(QString("images/strokedensity")+QString(".bmp"));

	_fbo->release();
	 
	
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE ) ; 

	QGLFramebufferObject *tmpfbo = new QGLFramebufferObject(_w,_h,QGLFramebufferObject::CombinedDepthStencil);
		
	tmpfbo->bind();


	float fboscale = _fboscaleFactor;
	_fboscaleFactor=1;

	resizeGL(_w,_h,_fboscaleFactor);

	QImage blank(_w*_fboscaleFactor,_h*_fboscaleFactor,QImage::Format_ARGB32_Premultiplied);
	GLuint finalTex = loadStrokeImageTexture(blank);
	if (finalTex==0)
	{
		deleteFrameTextures();	
		return false;
	}

	QImage fb = grabFrameBuffer(true);

	glEnable(GL_STENCIL_TEST);
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER,0.1);


	bool saved = false;

	_slp.actualStrokesPerFrame.clear();

	//go over all the frames and draw the new strokes with corrected colors
	for(int i=inp.startFrame;i <=inp.endFrame;i++)
	{
        sprintf(_strbuf, "Drawing frame %i with %i strokes",i,strokeList[i-inp.startFrame].size());
		LOG_STR_COUT(_strbuf);

		_slp.actualStrokesPerFrame.push_back(strokeList[i-inp.startFrame].size());
	
		if (strokeList[i-inp.startFrame].size()==0)
			continue;

		for(int j=0; j < strokeList[i-inp.startFrame].size();j++)
		{
		
			if (strokeList[i-inp.startFrame][j]->properties.isFill)
			{
				setCurrentColorForFilledStroke(strokeList[i-inp.startFrame][j],imgList[i]);
			}
		}
	

		tmpfbo->bind();
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, finalTex, 0);

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

		
		glBlendFuncSeparate(GL_ONE, GL_ZERO,GL_ONE, GL_ZERO);
		glStencilFunc(GL_ALWAYS, 1,  1);
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);

		const QRect& rect = layer.boundingBox(i);
		
		drawTexture(layer, i,false);

		
		if (_slp.addVisibleStrokesToLayer)
		{
              QList<Stroke *> userStrokes = layer.getVisibleStrokes(i);

				for (int j = userStrokes.size()-1;j >=0 ;j--)
				{
	
					Stroke * copyStroke= new Stroke();
					*copyStroke=*(userStrokes[j]);
					strokeList[i-inp.startFrame].push_front(copyStroke);
					
				}		
		}
		//char *version = (char*) glGetString(GL_VENDOR);
		//printf("version %s \n",version);
		
		for(int j=0; j < strokeList[i-inp.startFrame].size();j++)
		{
			multiPassStrokeRender(strokeList[i-inp.startFrame][j]);
			//delete strokeList[i-inp.startFrame][j];
			strokeList[i-inp.startFrame][j]->cleanup();
		}

		layer.saveGeneratedStrokes(i,strokeList[i-inp.startFrame],generateId);
		saved=true;

	
		for(int j=0; j < strokeList[i-inp.startFrame].size();j++)
		{
			//strokeList[i-inp.startFrame][j]->cleanup();
			Stroke *str = strokeList[i-inp.startFrame][j];
			//printf("size %i\n", str->getControlPoints().size());
			delete str;
			strokeList[i-inp.startFrame][j]=0;
		}
		
		QImage newStrokes = tmpfbo->toImage();

		int scaleX = ((double)rect.x())*_fboscaleFactor;
		int scaleY = ((double)rect.y())*_fboscaleFactor;
		int scaleWidth = ((double)rect.width())*_fboscaleFactor;
		int scaleHeight = ((double)rect.height())*_fboscaleFactor;


		QImage layerCutout = newStrokes.copy(scaleX, scaleY,scaleWidth,scaleHeight );
		//layerCutout.save(layer.projectDir()+layer.layerName()+QString("-cutoutstrokes-")+QString::number(i)+QString(".png"));

		//float currRed, currGreen, currBlue, placeholder;


		bool blankImage = true;
		for(int x=0;x < layerCutout.width();x++)
		for(int y=0;y < layerCutout.height();y++)
		{
			if (qAlpha(layerCutout.pixel(x,y)) > 0)
			{
				x=layerCutout.width();y=layerCutout.height();
				blankImage=false;
			}
		}
		if (blankImage==true)
		{
            sprintf(_strbuf, "maybe a problem...the new image was empty at frame %i",i);
			LOG_STR_COUT(_strbuf);
		}

		//free any texture for this layer if it exists
		GLuint tex = (GLuint) layer.getStrokeImageTexture(i);
		if (tex > 0)
		{
			glDeleteTextures( 1, &tex );
		}

		//writes out the new stroke image
		layer.setStrokeImage(i,layerCutout,generateId);

		GLuint newtex = loadStrokeImageTexture(layerCutout);

		if (newtex ==0)
		{
			layer.setStrokeImageState(i,CREATE_TEXTURE);

			LOG_STR_COUT("ERROR: problem getting texture image");
			continue;
		}
		layer.setStrokeImageTexture(i,newtex);
		layer.setStrokeImageState(i,TEXTURE_EXISTS);

		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		if (layerCutout.width() == 0)
		{
			LOG_STR_COUT("ERROR!!! Cutout size 0");
		}

	}


	
	glDisable(GL_STENCIL_TEST);
	glDisable(GL_ALPHA_TEST);

	glDeleteTextures( 1, &finalTex );
	
	_fboscaleFactor=fboscale;
	delete tmpfbo;

	LOG_STR_COUT("finished stroke creation");

	_fbo->bind();

	
	_slp.renderTime=afterGen.elapsed();



	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}
    sprintf(_strbuf, "It took %i mseconds for generating the strokes.", _slp.strokeGenTime );
	LOG_STR_COUT(_strbuf);
    sprintf(_strbuf, "It took %i mseconds for the final render.", _slp.renderTime);
	LOG_STR_COUT(_strbuf);

	_slp.distinctStrokes=numStrokes;

	sprintf(_strbuf, "Total # distinct strokes: %i", numStrokes);
	LOG_STR_COUT(_strbuf);

	return saved;
}

*/


bool StrokePainter::ageFade(Stroke *stroke,int frame)
{

	QVector<int> strokeAge(inp.endFrame-inp.startFrame+1);

	int age = _slp.fadeRate+1;

	QVector<Stroke *> createdStrokes(inp.endFrame-inp.startFrame+1);

	for(int i=0;i <= inp.endFrame-inp.startFrame;i++)
		createdStrokes[i]=0;
	
	

	Stroke *currentStroke=stroke;
	int f=frame-inp.startFrame;
	while (currentStroke!=0)
	{
		createdStrokes[f]=currentStroke;
		currentStroke->ageChecked=true;
		f++;
		currentStroke=currentStroke->nextStroke;
		if ((currentStroke!=0) && (currentStroke->properties.texture != _slp.strokeProperties.texture))
		{
			sprintf(_strbuf, "Problem with stroke");
			LOG_STR_COUT(_strbuf);
			break;
		}
	}

	if (_feedForwardFade)
		return true;

	for(int i=0;i <= inp.endFrame-inp.startFrame;i++)
	{
		if (createdStrokes[i] != 0)
		{
			age++;

			//We don't want to get rid of strokes at the boundaries
			if (i==0)
				age = _slp.minStrokeLength/4;
			else if (((i == inp.endFrame-inp.startFrame) && (age > _slp.minFrames/4)))
				age = 1000;
		
		}
		else
			age = 0;

		strokeAge[i] = age;
	}
	for(int i=inp.endFrame-inp.startFrame-1;i >=0;i--)
	{
		if ((strokeAge[i+1] > strokeAge[i])&& (strokeAge[i] != 0))
			strokeAge[i] = strokeAge[i+1];

	}


	bool killed=false;

	float fadeShift = 1;
	int fade = fadeShift;
	//if (inp.startFrame<=1)
	//	fade +=_slp.fadeRate;

	for(int i=0;i <= inp.endFrame-inp.startFrame;i++)
	{
		if (createdStrokes[i] != 0)
		{

            fade = min((float) fade+1,_slp.fadeRate+fadeShift);
	
			if ((strokeAge[i] < _slp.minFrames))
			{
				createdStrokes[i]->properties.alpha = 0;
				killed=true;
			}
			else
				createdStrokes[i]->properties.alpha = _slp.strokeProperties.alpha * ((float)fade/(float)(_slp.fadeRate+fadeShift));
		}
		else
			fade =fadeShift;
	
	}


	fade = fadeShift;
	//if (inp.endFrame>=_frames-1)
	//	fade +=_slp.fadeRate;

	for(int i=inp.endFrame-inp.startFrame;i >=0;i--)
	{
		if (createdStrokes[i] != 0)
		{
			
            fade = min((float)fade+1,_slp.fadeRate+fadeShift);
		
			float currentAlpha=createdStrokes[i]->properties.alpha;
			float backAlpha=_slp.strokeProperties.alpha * ((float)fade/(float)(_slp.fadeRate+fadeShift));

			float setAlpha= min(currentAlpha,backAlpha);
			createdStrokes[i]->properties.alpha = setAlpha;
										
		}
		else
			fade =fadeShift;
	}

	return !killed;

}



bool StrokePainter::fillLayerWithStrokes(LayerObject& layer,QList<LayerObject>& allLayers,QList<QImage>& imgList, int generateId)
{
    sprintf(_strbuf, "Starting stroke creation");
	LOG_STR_COUT(_strbuf);

    QTime startGen = QTime::currentTime();


    QVector<QVector<Stroke*> > strokeList(inp.endFrame-inp.startFrame+1);
	QVector<QVector<pair<int,pair<int,int> > > > pointQueue(inp.endFrame-inp.startFrame+1);

	_fbo->bind();
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[inp.startFrame], 0);

	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}

	long int pointQueueSize = 0;

	QVector<bool>firstAdded (inp.endFrame-inp.startFrame+1);

	FImage gradImg;
	FImage imgSmooth(_w,_h);

	FImage strokeGrad;
	FImage strokeSmooth;

    sprintf(_strbuf, "Seed Order: %s" ,qPrintable((_slp.seedOrder)) );
	LOG_STR_COUT(_strbuf);



	QProgressDialog seedProgress("Choosing stroke start points...", "Cancel", 0, inp.endFrame-inp.startFrame, this);


	
	//Add all the points we'll draw to the list...
	for(int i=inp.startFrame;i<=inp.endFrame;i++)
	//for(int i=inp.startFrame;i<=inp.startFrame+1;i++)
	//for(int i=inp.endFrame-1;i>=inp.endFrame-2;i--)
	//for(int i=(inp.startFrame+inp.endFrame)/2;i<=(inp.startFrame+inp.endFrame)/2;i++)
	{

		seedProgress.setValue(i-inp.startFrame);
		qApp->processEvents();

        if (seedProgress.wasCanceled())
		{
			LOG_STR_COUT("Stopped by user");
            return false;
		}

		if ((_slp.fillUserSeed)&& (inp.seedStrokeImgs[i-inp.startFrame].width()==0))
			continue;

		FImage *strokeImg = new FImage(layer.getStrokeImage(i));
		strokeImg->GaussianSmoothing(strokeSmooth,1,4);
		delete strokeImg;
		
		if ((_slp.seedOrder=="Image Canvas Edge Difference")&&(layer.getStrokeImage(i).width() > 0))
		{
			strokeSmooth.gradientmag(strokeGrad);
		}

		if ((_slp.seedOrder=="Edge") || (_slp.seedOrder=="Image Canvas Edge Difference"))
		{
			inp.imgs[i-inp.startFrame].GaussianSmoothing(imgSmooth,1,4);
			imgSmooth.gradientmag(gradImg);
		}

		int top = layer.boundingBox(i).top();
		int bottom = layer.boundingBox(i).bottom();
		int left = layer.boundingBox(i).left();
		int right = layer.boundingBox(i).right();


		for(int x=left+1;x<right-1;x++)
		for(int y=top+1;y<bottom-1;y++)
		{

			//if ((_slp.fillUserSeed)&& ((x < 0) || (y<0) || (x > inp.seedStrokeImgs[i-inp.startFrame].width()-1)|| (y > inp.seedStrokeImgs[i-inp.startFrame].height()-1))
			//continue;


			inp.visited[i-inp.startFrame].setPixel(x,y,false);
			

			int vis;
			if (_slp.startInOccludedRegions )	
				vis = layer.Mask(i).getPixel(x,y);
			else
				vis = layer.visibleMask(i).getPixel(x,y);

			if (vis==0)
				continue;
			
			if ((_slp.fillUserSeed)&&(qRed(inp.seedStrokeImgs[i-inp.startFrame].pixel(x,y))==0))
				continue;

			int pixAlpha = 0;

			bool choosepix = (rand()%10==1)&& (rand()%10 ==1) ;
			//bool choosepix=1;
			float currRed=-1, currGreen=-1,currBlue=-1;
			float colDist=-1;

			if (layer.getStrokeImage(i).width() > 0)
			{
				
				QColor pixelCol(strokeSmooth.val(x-left,y-top));

			
				pixAlpha = qAlpha(layer.getStrokeImage(i).pixel(x-left,y-top));


				 if ((_slp.strokeProperties.useImageColor) && ((_slp.fillColorDifference)||(_slp.seedOrder=="Color Difference")))
				{

					if (getCombinedImageAndUserStrokeColor(i-inp.startFrame, x, y,_gaussianMask,_gaussianMaskSize,layer,&currRed, &currGreen, &currBlue))
					{
						QColor currCol(currRed,currGreen,currBlue);
						colDist = getColorDist(pixelCol,currCol);
					}
					else
						colDist=0;
					
					
				
					if (_slp.fillColorDifference)
					{
						//if (( colDist> _slp.colorDifference) && (pixAlpha>50))
						if (( colDist> _slp.colorDifference))
							choosepix = (rand()%3==1)&& (rand()%3 ==1);
						else
							continue;
					}
				}
			}
			if ((_slp.fillUserSeed)||((i > _startHeavySample && i < _endHeavySample))) //|| (_slp.fillEmptyRegions))
				 choosepix = (rand()%2==1)&& (rand()%2 ==1) ;

			if (_slp.fillEmptyRegions==true)
			{
				if (pixAlpha<50)
					choosepix = 1;
				else
					choosepix = 0;
			}

			if (choosepix==0)
				continue;


			bool gotCol=false;
			int brightness=0;
			if (currRed == -1)
			{
				gotCol=getCombinedImageAndUserStrokeColor(i-inp.startFrame, x, y,_gaussianMask,_gaussianMaskSize,layer,&currRed, &currGreen, &currBlue);
			}
			brightness =currRed*0.3 + currGreen*0.59+ currBlue*0.11;

			if (gotCol && ((_slp.minBrightness != 0) || (_slp.maxBrightness != 255)))
			{
				if ((brightness < _slp.minBrightness) || (brightness > _slp.maxBrightness) )
				{
					continue;
				}
			}
		



			int sortDist = rand();

			if (_slp.seedOrder=="Light to Dark")
				sortDist= -1*brightness;
			if (_slp.seedOrder=="Dark to Light")
				sortDist= brightness;
			if (_slp.seedOrder=="Edge")
			{
				sortDist= -1*gradImg.val(x,y);
			}
			if (_slp.seedOrder=="Image Canvas Edge Difference")
			{	
				sortDist= -1*abs(strokeGrad.val(x-left,y-top)-gradImg.val(x,y));
			}
			if ((_slp.seedOrder=="Color Difference") && (colDist>-1))
				sortDist = -1*colDist*1000.;

			pair<int,int> p1(x,y);
                        pair<int,pair<int,int> > p2(sortDist,p1);
			pointQueue[i-inp.startFrame].push_back(p2);

			firstAdded[i-inp.startFrame] = true;
			pointQueueSize++;
			
		}

		
		qSort(pointQueue[i-inp.startFrame]);
	}



	QVector<int> seedFrames;

	for (int i=0;i < firstAdded.size();i++)
	{
		if (firstAdded[i] ==true)
			seedFrames.push_back(i);
	}

	
    sprintf(_strbuf, "start queue size %i", pointQueueSize);
	LOG_STR_COUT(_strbuf);

	
	int *strokeCount = new int[inp.endFrame-inp.startFrame+1];
	for(int i=0;i<= inp.endFrame-inp.startFrame;i++)
		strokeCount[i] = 0;

	int count = 0;
	int success = 0;

	float strokeRad = ((float)_slp.strokeProperties.startRadius + _slp.strokeProperties.endRadius)/2.0;
	_slp.actualRadius=((strokeRad)*_textureWidthMap.value((int)_slp.strokeProperties.texture));
	_slp.gridSize = max((int)_slp.actualRadius,1);

	if (_slp.gridSize%1==1)
		_slp.gridSize--;

    sprintf(_strbuf, "gridsize %i", _slp.gridSize);
	LOG_STR_COUT(_strbuf);

	_imgDensity = new unsigned char[_slp.gridSize*_slp.gridSize*4];
	


    sprintf(_strbuf, "stroke density %f", _slp.strokeDensity);
	LOG_STR_COUT(_strbuf);


	QVector<Stroke*> createdStrokes(inp.endFrame-inp.startFrame+1);

	StrokeGenInfo sgi;

	QProgressDialog progress("Generating stroke layer...", "Stop now.", 0, _slp.maxStrokes, this);
	int highestFrameStrokeCount=0;

	int numStrokes=0;

	int frame=0, col,outerCount = 0,startedStroke = 0;
	//Go over all the start points
	while (pointQueueSize != 0)
	{
		
		outerCount++;

		//check for events every few hundred strokes
		if(outerCount%200 ==0)
		{
			qApp->processEvents();

            if (progress.wasCanceled())
			{
				LOG_STR_COUT("Stopped by user");
                break;
			}
		}


		if (outerCount%10000 ==9999)
		{
			if ((startedStroke ==0) && (_slp.fillEmptyRegions==false))
			{
				LOG_STR_COUT("no starts in 10k tries, giving up");
				break;
			}
            sprintf(_strbuf, "outer success rate %f, queue size %i", (float) startedStroke / (float) outerCount,pointQueueSize);
			LOG_STR_COUT(_strbuf);

			outerCount = 0;
			startedStroke = 0;
		}
		
	
		if (inp.endFrame!=inp.startFrame)
		{
			 int frameidx = rand()%(seedFrames.size());
			 frame = seedFrames[frameidx];
		}
		else 
			frame = 0;

		if (pointQueue[frame].size()==0)
			continue;
	




		if (pointQueueSize%1000==0)
		{
			if ((_slp.generationTimelimit >0) && (_slp.generationTimelimit*1000<startGen.elapsed()))
			{
                sprintf(_strbuf, "reached max generation time (%i s) so gave up", _slp.generationTimelimit);
				LOG_STR_COUT(_strbuf);
				pointQueue.clear();
				pointQueueSize=0;
				continue;
			}
			
            sprintf(_strbuf, "current queue size %i", pointQueueSize);
			LOG_STR_COUT(_strbuf);
		}


		pair<int,int> p = pointQueue[frame][0].second;
		int x = p.first;
		int y = p.second;
		pointQueue[frame].pop_front();
		pointQueueSize--;

		if (inp.visited[frame].getPixel(x,y) ==false)
		{

			//remove the point after processing
			inp.visited[frame].setPixel(x,y,true);

			bool densityCheck = true;
			if ((_slp.strokeDensity < 0.1) || (_slp.fillEmptyRegions))
			{

				if ((_errCode = glGetError()) != GL_NO_ERROR) {
					sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
					LOG_STR_COUT(_strbuf);
				}

				_fbo->bind();
				//printf("here\n");
                glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[frame+inp.startFrame], 0);

				if ((_errCode = glGetError()) != GL_NO_ERROR) {
					sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
					LOG_STR_COUT(_strbuf);
				}

				float nearbyStrokeDensity = 0;
				
                if (_readPixelsTiming)
                    t.start();

                glReadPixels(x-_slp.gridSize/2,_h-(y+_slp.gridSize/2),_slp.gridSize,_slp.gridSize, GL_BGRA, GL_UNSIGNED_BYTE, (void*) _imgDensity);

                if (_readPixelsTiming)
                    printf("glReadPixels took %i msec\n", t.elapsed());


                for(int i=3;i<_slp.gridSize*_slp.gridSize*4;i+=4)
                        nearbyStrokeDensity+=(float)_imgDensity[i];

                nearbyStrokeDensity /= (double)_slp.gridSize*_slp.gridSize*255;

                //glReadPixels(x,_h-y,1,1, GL_BGRA, GL_FLOAT, (void*) _imgDensity);
                //nearbyStrokeDensity = _imgDensity[3];
			


	
				if ((_errCode = glGetError()) != GL_NO_ERROR) {
					sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
					LOG_STR_COUT(_strbuf);
				}

				if ((nearbyStrokeDensity > _slp.strokeDensity) && (_slp.fillEmptyRegions==false))
					densityCheck = false;

				if ((_slp.fillEmptyRegions) && (nearbyStrokeDensity!= 0))
					densityCheck = false;
			}

			//If the density for this pixel is less than the threshold, start a new stroke
			//if (nearbyStrokeDensity <= (double)_slp.strokeDensity/ ((double)gridSize*gridSize))
			if (densityCheck == true)
			{
				startedStroke++;


				startStroke(x,y,frame+inp.startFrame, layer,allLayers,createdStrokes,sgi,pointQueue,&pointQueueSize);


				bool createdStroke = false;


				for(int j=0;j <= inp.endFrame-inp.startFrame;j++)
				{
					//if we got a stroke then add it to the list
					if (createdStrokes[j] != 0)
					{
						createdStroke = true;

						strokeList[j].push_back(createdStrokes[j]);
				

						strokeCount[j] =strokeCount[j]+1;
	
						if (strokeCount[j]%200 ==5)
						{
							
							sprintf(_strbuf, "stroke count for frame %i %i",j+inp.startFrame, strokeCount[j]);
							LOG_STR_COUT(_strbuf);
						}

						if (strokeCount[j]>highestFrameStrokeCount)
						{
							highestFrameStrokeCount=strokeCount[j];
							progress.setValue(highestFrameStrokeCount);
						}

						if ((strokeCount[j] > _slp.maxStrokes))
						{
							LOG_STR_COUT("found max strokes..");
							pointQueueSize = 0;
							for(int i=0;i < pointQueue.size();i++)
							{
								pointQueue[i].clear();
							}
						}
					}
				}

				if (createdStroke == true)
				{
					success++;
					numStrokes++;
				}

				count++;

				if (count%1000 ==999)
				{

					sprintf(_strbuf, "success rate %f", (float) success / (float) count);
					LOG_STR_COUT(_strbuf);
					sgi.print();
					sgi.clear();

					if (success <= 3)
					{

						for(int i=0;i < pointQueue.size();i++)
						{
							pointQueue[i].clear();
						}
						break;
					}

					count =0;
					success = 0;
				}
			}
		}
	}

	sgi.print();
	sgi.clear();



	//printf("finished with %i strokes\n",strokeCount);

	delete [] _imgDensity;
	delete [] strokeCount;

	_slp.strokeGenTime=startGen.elapsed();

    QTime afterGen;
    afterGen.start();
	


	QImage densityImg = _fbo->toImage();
	densityImg.save(QString("images/strokedensity")+QString(".bmp"));

	_fbo->release();
	 
	
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE ) ; 

	QGLFramebufferObject *tmpfbo = new QGLFramebufferObject(_w,_h,QGLFramebufferObject::CombinedDepthStencil);
		
	tmpfbo->bind();


	float fboscale = _fboscaleFactor;
	_fboscaleFactor=1;

	resizeGL(_w,_h,_fboscaleFactor);

	QImage blank(_w*_fboscaleFactor,_h*_fboscaleFactor,QImage::Format_ARGB32_Premultiplied);
	GLuint finalTex = loadStrokeImageTexture(blank);
	if (finalTex==0)
	{
		deleteFrameTextures();	
		return false;
	}

	QImage fb = grabFrameBuffer(true);

	glEnable(GL_STENCIL_TEST);
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER,0.1);


	bool saved = false;

	_slp.actualStrokesPerFrame.clear();

	//go over all the frames and draw the new strokes with corrected colors
	for(int i=inp.startFrame;i <=inp.endFrame;i++)
	{
        sprintf(_strbuf, "Drawing frame %i with %i strokes",i,strokeList[i-inp.startFrame].size());
		LOG_STR_COUT(_strbuf);

		_slp.actualStrokesPerFrame.push_back(strokeList[i-inp.startFrame].size());
	
		if (strokeList[i-inp.startFrame].size()==0)
			continue;

		for(int j=0; j < strokeList[i-inp.startFrame].size();j++)
		{
		
			if (strokeList[i-inp.startFrame][j]->properties.isFill)
			{
				setCurrentColorForFilledStroke(strokeList[i-inp.startFrame][j],imgList[i]);
			}
		}
	

		tmpfbo->bind();
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, finalTex, 0);

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

		
		glBlendFuncSeparate(GL_ONE, GL_ZERO,GL_ONE, GL_ZERO);
		glStencilFunc(GL_ALWAYS, 1,  1);
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);

		const QRect& rect = layer.boundingBox(i);
		
		drawTexture(layer, i,false);

		
		if (_slp.addVisibleStrokesToLayer)
		{
              QList<Stroke *> userStrokes = layer.getVisibleStrokes(i);

				for (int j = userStrokes.size()-1;j >=0 ;j--)
				{
	
					Stroke * copyStroke= new Stroke();
					*copyStroke=*(userStrokes[j]);
					strokeList[i-inp.startFrame].push_front(copyStroke);
					
				}		
		}
		//char *version = (char*) glGetString(GL_VENDOR);
		//printf("version %s \n",version);
		
		for(int j=0; j < strokeList[i-inp.startFrame].size();j++)
		{
			multiPassStrokeRender(strokeList[i-inp.startFrame][j]);
			//delete strokeList[i-inp.startFrame][j];
			strokeList[i-inp.startFrame][j]->cleanup();
		}

		layer.saveGeneratedStrokes(i,strokeList[i-inp.startFrame],generateId);
		saved=true;

	
		for(int j=0; j < strokeList[i-inp.startFrame].size();j++)
		{
			//strokeList[i-inp.startFrame][j]->cleanup();
			Stroke *str = strokeList[i-inp.startFrame][j];
			//printf("size %i\n", str->getControlPoints().size());
			delete str;
			strokeList[i-inp.startFrame][j]=0;
		}
		
		QImage newStrokes = tmpfbo->toImage();

		int scaleX = ((double)rect.x())*_fboscaleFactor;
		int scaleY = ((double)rect.y())*_fboscaleFactor;
		int scaleWidth = ((double)rect.width())*_fboscaleFactor;
		int scaleHeight = ((double)rect.height())*_fboscaleFactor;


		QImage layerCutout = newStrokes.copy(scaleX, scaleY,scaleWidth,scaleHeight );
		//layerCutout.save(layer.projectDir()+layer.layerName()+QString("-cutoutstrokes-")+QString::number(i)+QString(".png"));

		//float currRed, currGreen, currBlue, placeholder;


		bool blankImage = true;
		for(int x=0;x < layerCutout.width();x++)
		for(int y=0;y < layerCutout.height();y++)
		{
			if (qAlpha(layerCutout.pixel(x,y)) > 0)
			{
				x=layerCutout.width();y=layerCutout.height();
				blankImage=false;
			}
		}
		if (blankImage==true)
		{
            sprintf(_strbuf, "maybe a problem...the new image was empty at frame %i",i);
			LOG_STR_COUT(_strbuf);
		}

		//free any texture for this layer if it exists
		GLuint tex = (GLuint) layer.getStrokeImageTexture(i);
		if (tex > 0)
		{
			glDeleteTextures( 1, &tex );
		}

		//writes out the new stroke image
		layer.setStrokeImage(i,layerCutout,generateId);

		GLuint newtex = loadStrokeImageTexture(layerCutout);

		if (newtex ==0)
		{
			layer.setStrokeImageState(i,CREATE_TEXTURE);

			LOG_STR_COUT("ERROR: problem getting texture image");
			continue;
		}
		layer.setStrokeImageTexture(i,newtex);
		layer.setStrokeImageState(i,TEXTURE_EXISTS);

		if ((_errCode = glGetError()) != GL_NO_ERROR) {
			sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
			LOG_STR_COUT(_strbuf);
		}

		if (layerCutout.width() == 0)
		{
			LOG_STR_COUT("ERROR!!! Cutout size 0");
		}

	}


	
	glDisable(GL_STENCIL_TEST);
	glDisable(GL_ALPHA_TEST);

	glDeleteTextures( 1, &finalTex );
	
	_fboscaleFactor=fboscale;
	delete tmpfbo;

	LOG_STR_COUT("finished stroke creation");

	_fbo->bind();

	
	_slp.renderTime=afterGen.elapsed();



	if ((_errCode = glGetError()) != GL_NO_ERROR) {
        sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}
    sprintf(_strbuf, "It took %i mseconds for generating the strokes.", _slp.strokeGenTime );
	LOG_STR_COUT(_strbuf);
    sprintf(_strbuf, "It took %i mseconds for the final render.", _slp.renderTime);
	LOG_STR_COUT(_strbuf);

	_slp.distinctStrokes=numStrokes;

	sprintf(_strbuf, "Total # distinct strokes: %i", numStrokes);
	LOG_STR_COUT(_strbuf);

	return saved;
}


void StrokePainter::multiPassStrokeRender(Stroke* s)
{

	if (_multiPassRender)
	{
		glBlendFuncSeparate(GL_ONE, GL_ZERO,GL_ONE, GL_ZERO);
		glColorMask(1,1,1,1);						
		glStencilFunc(GL_NOTEQUAL, 1, 1);  
	    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
		s->render();
	
		glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ONE);
		glStencilFunc(GL_EQUAL, 1,  1);						
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
		s->render();
	
		glColorMask(0,0,0,0);
		glStencilFunc(GL_ALWAYS, 1,  1);
		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
		s->render();
		glColorMask(1,1,1,1);

		/*if ((s->properties.alpha==1) && ((s->nextStroke==0)||(s->nextStroke->properties.alpha==0)))
		{
			printf("wierd\n");
		}*/
	}
	else
	{

		if (_oldBlend==true)
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		else
			glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,GL_ONE, GL_ONE);

		s->render();

	}
}





void StrokePainter::startStroke(int initx, int inity,int frame, LayerObject& layer,QList<LayerObject>& allLayers, QVector<Stroke*>& createdStrokes, StrokeGenInfo& sgi,QVector<QVector<pair<int,pair<int,int> > > > pointQueue,long *pointQueueSize)
{	
        bool debug = false;

	sgi.count++;

	if (debug)
	printf("starting stroke at %i %i %i\n", initx,inity,frame);	



	//initalized to no strokes
	for (int i=0;i < inp.endFrame-inp.startFrame+1;i++)
		createdStrokes[i] = 0;

	//float mainDir = 1;
	float directionSwitch = 1;
	float initDirectionSwitch = 1;
	if ((_slp.reverseDirection) && (rand()%2==1))
	{
		initDirectionSwitch =-1;
	}
	int frameNum = inp.endFrame-inp.startFrame+1;

	QVector<QColor> colorStart(frameNum);
	QVector<Vec3i> colorSum(frameNum);
	QVector<int> colorCount(frameNum);
    QVector<QVector<Vec2f> > strokePoints(frameNum);
    QVector<QVector<Vec2f> > strokeAngles(frameNum);
	QVector<Vec2f> strokeWidths(frameNum);
	QVector<int> strokePointCount(frameNum);

	for(int i=0;i < frameNum;i++)
	{
		strokePointCount[i] = 0;

		for(int j=0;j <= _slp.maxStrokeLength;j++)
		{
			strokePoints[i].push_back(Vec2f(-1,-1));
			strokeAngles[i].push_back(Vec2f(0,0));
			//strokeFlow[i].push_back(Vec2f(0,0));
		}

		strokeWidths[i]=Vec2f(_slp.strokeProperties.startRadius,_slp.strokeProperties.endRadius);

	}

	float strokeRad = ((float)_slp.strokeProperties.startRadius + _slp.strokeProperties.endRadius)/2.0;
	float avgRadius = ((strokeRad)*_textureWidthMap.value((int)_slp.strokeProperties.texture));

	int gridSize=avgRadius;

	//int radius = min(_slp.strokeProperties.startRadius, _slp.strokeProperties.endRadius);
	int radius = strokeRad;
	//int radius = avgRadius;

	double angle =1;

	float flowx = 0, flowy = 0;


	Vec2f oldDir(0,0);
	Vec2f firstDir(0,0);

	int currFrame;
	int currFrameRel;


	Vec3i strokeNoise;
	strokeNoise.set_x(normal(_slp.hueOffset,_slp.hueNoise+0.001));
	strokeNoise.set_y(normal(_slp.saturationOffset,_slp.saturationNoise+0.001));
	strokeNoise.set_z(normal(_slp.valueOffset,_slp.valueNoise+0.001));


	double *gaussianMask;
	int gaussianMaskSize;
	std::pair<double *,int> p;

	int shiftx=-1, shifty=-1;

	if ((_slp.shiftToEdges==true)&&(shiftStrokeToOrientationField(initx,inity,frame-inp.startFrame, avgRadius-1, layer, &shiftx,&shifty)==true))
	{
		initx=shiftx;
		inity=shifty;
		inp.visited[frame-inp.startFrame].setPixel(shiftx,shifty,true);
	}

	bool thrownOutside = false;


	int vis;
	if (_slp.startInOccludedRegions )	
		vis = layer.Mask(frame).getPixel(initx,inity);
	else
		vis = layer.visibleMask(frame).getPixel(initx,inity);

	if (vis==0)
	{

		return;
	}


	for(int frameDir=0;frameDir<2;frameDir++)
	{
		int destinationFrame;
		bool goingForward;

		float startx = initx;
		float starty = inity;

		if (frameDir==0)
		{
			goingForward = true;
			destinationFrame = min(inp.endFrame+1,frame+_slp.maxFrames/2);
			currFrame = frame;
			currFrameRel = frame-inp.startFrame;
		}
		else
		{
			goingForward = false;
			destinationFrame = max(inp.startFrame-1,frame-_slp.maxFrames/2);
			if (frame != inp.startFrame)
			{
				float fx, fy;
				bool possible = getFlowVector(startx,starty,frame-inp.startFrame,true,inp.startFrame,layer,allLayers,&fx, &fy);
				
				startx += fx;
				starty += fy;

				currFrame = frame-1;
				currFrameRel = frame-inp.startFrame-1;				
			}
			else	
				continue;
		}

		//Vec2f lastFrameDirTest(0,0);
		//lastFrameDirTest.set_x(cos(_etfAngle[frame].val(initx,inity)));
		//lastFrameDirTest.set_y(sin(_etfAngle[frame].val(initx,inity)));

		flowx =0;
		flowy =0;


		//draw the stroke for the current frame
		while (currFrame!=destinationFrame)
		{
			if (debug)
			printf("currFrame %i, currFrameRel %i, destination %i\n",currFrame,currFrameRel,destinationFrame);
			//assert(_etfCreated[currFrameRel] == true);


			int strokeLength = 1;

			int lastProcessedFrame = currFrameRel+1;
			if (goingForward==true)
				lastProcessedFrame = currFrameRel-1;


			int leftBoundBox = layer.boundingBox(currFrame).left();
			int rightBoundBox = layer.boundingBox(currFrame).right();
			int topBoundBox = layer.boundingBox(currFrame).top();
			int bottomBoundBox = layer.boundingBox(currFrame).bottom();

			//add the flow to the last starting point
			startx+= flowx;
			starty+= flowy;


			float strokeRad;
			if ((lastProcessedFrame==-1)||(lastProcessedFrame>=strokeWidths.size()))
				strokeRad=(_slp.strokeProperties.startRadius + _slp.strokeProperties.startRadius )/2.0;
			else
				strokeRad=(strokeWidths[lastProcessedFrame].x() + strokeWidths[lastProcessedFrame].y())/2.0;
			double actualRadius = strokeRad*_textureWidthMap.value((int)_slp.strokeProperties.texture);
		

	

			if ((startx<1) || (starty<1) || (startx>=_w-2) || (starty>=_h-2))
			{
				sgi.thrownOutside++;
				thrownOutside = true;
				if (debug)
					printf("outside, breaking\n");
				break;
			}
		
			bool possibleFlow = false;
			
			//get the flow vector if we're going forward
			if (goingForward==true)
				possibleFlow= getFlowVector(startx,starty,currFrameRel,false,inp.startFrame,layer,allLayers,&flowx, &flowy);
			//if we're going backward, don't try this for the first frame (nowhere to go...)
			else if (currFrame != inp.startFrame-1)
				possibleFlow = getFlowVector(startx,starty,currFrameRel,true,inp.startFrame,layer,allLayers,&flowx, &flowy);

			//if we didn't make it, try the other side
			if (possibleFlow == false)
			{
				sgi.outsideFlowImage++;
				currFrame=destinationFrame;

				if (debug)
					printf("outside flow image\n");
				continue;
			}




			if ((currFrame != frame) && ( strokePointCount[lastProcessedFrame] < _slp.minStrokeLength))
			{
				if (debug)
					printf("currframe %i, last count %i\n",currFrame,  strokePointCount[lastProcessedFrame]);
				currFrame=destinationFrame;
				sgi.lastFrameTooShort++;

				if (lastProcessedFrame == frame-inp.startFrame)
				{
					if (debug)
					printf("start frame %i was too small: %i\n",frame,strokePointCount[frame-inp.startFrame]);
					sgi.tooShort++;
					return;
				}

				continue;	
			}

			int gridSize;
			if (_slp.optimizeStrokeWidths)
			{
				float strokeRad;
				if ((lastProcessedFrame==-1)||(lastProcessedFrame>=strokeWidths.size()))
					strokeRad=(_slp.strokeProperties.startRadius + _slp.strokeProperties.startRadius )/2.0;
				else
					strokeRad=(strokeWidths[lastProcessedFrame].x() + strokeWidths[lastProcessedFrame].y())/2.0;
				double actualRadius = strokeRad*_textureWidthMap.value((int)_slp.strokeProperties.texture);
				gridSize = max((int)actualRadius,1);

				int avgRad=actualRadius*100;

				if (_gaussianMasks.contains(avgRad))
				{
					p= _gaussianMasks.value(avgRad);
					gaussianMask=p.first;
					gaussianMaskSize=p.second;
				}
				else
				{
					gaussianMask= getGaussianFilter(min((double)avgRadius,15.0)*_slp.radiusFactor,&gaussianMaskSize);

					p.first=gaussianMask;
					p.second=gaussianMaskSize;
					_gaussianMasks.insert(avgRad,p);
				}
			}
			else
			{
				gridSize=_slp.gridSize;
				gaussianMask = _gaussianMask;
				gaussianMaskSize=_gaussianMaskSize;
			}


		
			if (debug)
				printf("flow %f %f\n", flowx,flowy);

		
			if (_slp.colorDifference < 1)
			{
				float r,g,b;				
				bool gotCol=getCombinedImageAndUserStrokeColor(currFrameRel, startx, starty,gaussianMask,gaussianMaskSize,layer,&r, &g, &b);
				QColor currCol(r,g,b);
				colorStart[currFrameRel] = currCol;
				float colDist =0;
				
				if (currFrame != frame)
				{
					colDist = getColorDist(colorStart[lastProcessedFrame],currCol);

				}

				bool allowOccluded = (!_slp.startInOccludedRegions) && _slp.paintOccludedRegions;
		
				if ((currFrame != frame) && (colDist > _slp.colorDifference)&& (!allowOccluded))
				//if ((currFrame != frame) && () > _slp.colorDifference))
				{
					sgi.colorThresh++;
					currFrame=destinationFrame;
					if (debug)
						printf("color thresh\n");
					continue;

				}
			}
			

			//record the points we've visited
			strokePoints[currFrameRel][_slp.maxStrokeLength/2] = Vec2f(startx,starty);

			float x = startx;
			float y = starty;




			for(int dir=0;dir < 2;dir++)
			for(strokeLength=0; strokeLength < (_slp.maxStrokeLength/2);strokeLength++)
			{


				if (debug)
					printf("dir %i, strokelen %i\n", dir,strokeLength);

				bool firstPtForFrame = (strokeLength ==0) && (dir==0);

				//If this is the first part of the stroke, then reset the initial point to the center
				//and set the old direction
				if (strokeLength ==0)
				{

					x = startx;
					y = starty;

					if (dir==1)
					{
						directionSwitch = -1*initDirectionSwitch;
						oldDir.set_x(firstDir.x()*-1);
						oldDir.set_y(firstDir.y()*-1);
					}
					else
					{
						directionSwitch = initDirectionSwitch;
						oldDir.set_x(0);
						oldDir.set_y(0);
					}
		
				}

				int lastPointIndex;
				if (dir == 0)
					lastPointIndex= (_slp.maxStrokeLength/2)*(1-dir) - strokeLength-1;
				else
					lastPointIndex =(_slp.maxStrokeLength/2)*(dir) + strokeLength+1;
			
		
				//Vec2f optFlowDir = getOpticalFlowDirection(Vec2f(x,y),layer,allLayers,currFrameRel,lastPointIndex,goingForward,strokePoints);
				Vec2f optFlowDir(0,0);

				Vec2f lastFrameDir(0,0);
				if (frame != currFrame)
					lastFrameDir =  strokeAngles[lastProcessedFrame][lastPointIndex];
	

				//assertions
				//if this isn't our first point in this frame, we must have an old direction
				if ((firstPtForFrame==false)&& (oldDir.Len2()==0))
				{
	
					strokeLength = (_slp.maxStrokeLength/2) +1;
					continue;
				}

				//if this isn't the first frame we're drawing, then we must have an direction
				//from a previous frame. There's no point tracing without it since it will be truncated later
				if ((frame != currFrame)&& (lastFrameDir.Len2()==0))
				{
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				}


				float magnitude;
				Vec2f newDir = getDirection(oldDir,lastFrameDir, optFlowDir,x,y,currFrameRel,radius,dir,directionSwitch,layer,&magnitude);

	

				if (newDir.Dot2(oldDir)<0)
				{
					int test=1;
					Vec2f newDir = getDirection(oldDir,lastFrameDir, optFlowDir,x,y,currFrameRel,radius,dir,directionSwitch,layer,&magnitude);

				}

				if (magnitude >=_slp.magnitudeThreshold)
				{

					sgi.pastMagnitude++;

					//If this is the first point on the first frame, don't bother looking any farther.
					if ((firstPtForFrame) && (currFrame==frame))
					{
						return;
					}

					
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;

				}

				if (_slp.optimizeStrokes==false)
				{

					newDir.set_x(_slp.followGradientFactor*(magnitude)*newDir.x() + _slp.followPreviousStrokesFactor*lastFrameDir.x());
					newDir.set_y(_slp.followGradientFactor*(magnitude)*newDir.y() + _slp.followPreviousStrokesFactor*lastFrameDir.y());
					newDir.Normalize();

					truncateDirection(newDir,lastFrameDir,_slp.maxFlowAngle);
					truncateDirection(newDir,oldDir,_slp.maxStrokeAngle);
					if (magnitude == -1)
					{
						if (debug)
							printf("something strange happened while getting the new direction...\n");
						sgi.directionError++;
						strokeLength = _slp.maxStrokeLength/2 +1;
						continue;
					}
					else if (magnitude == -2)
					{
						if (debug)
							printf("intersected a guide stroke...\n");
						sgi.guideStrokeIntersection++;
						strokeLength = _slp.maxStrokeLength/2 +1;
						continue;
					}
				}
				//optimizing the direction
				else
				{
					Vec2f initDir(0,0);
				
					if (oldDir.Len2() != 0)
						initDir= oldDir;
					else if (lastFrameDir.Len() !=0)
						initDir= lastFrameDir;
					else if  ((dir==0) && (strokeLength==0) && (currFrame == frame))
					{
						initDir= newDir;
					}
					else
						LOG_STR_COUT("Problems with initial stroke direction. Should never be here.");

					//initDir= newDir;

					getNewOptimizedDirection(oldDir,lastFrameDir, x, y, currFrameRel,radius,dir,directionSwitch,colorStart[currFrameRel],layer,initDir,gaussianMask, gaussianMaskSize,  &magnitude,newDir);

					if (newDir.Dot2(oldDir)<0)
					{
						int test=1;
						getNewOptimizedDirection(oldDir,lastFrameDir, x, y, currFrameRel,radius,dir,directionSwitch,colorStart[currFrameRel],layer,initDir,gaussianMask, gaussianMaskSize,  &magnitude,newDir);

					}

					if (magnitude>= _magMax)
					{
						strokeLength = _slp.maxStrokeLength/2 +1;
						sgi.strokeOptimizationThreshold++;
						continue;
					}
				}

		
			

				if ((strokeLength ==0) && (dir==0))
					firstDir = newDir;

				x += radius*newDir.x();
				y += radius*newDir.y();
			
				oldDir = newDir;



				//If we're outside the image, add the point but then quit
				if ((x<2) || (y<2) || (x>=_w-2) || (y>=_h-2))
				{
					if (debug)
						printf("image %i %i\n",x,y);
					strokeLength = _slp.maxStrokeLength/2 +1;

					strokePoints[currFrameRel][lastPointIndex] = Vec2f(x,y);
					strokeAngles[currFrameRel][lastPointIndex] = newDir;
					strokePointCount[currFrameRel]++;

					continue;
				}

				else if ((x<leftBoundBox) || (x>=rightBoundBox)  || (y<topBoundBox) || (y >= bottomBoundBox)) 
				{

					if (debug)
						printf("outside box %f %f, bound %i %i %i %i\n",x,y,leftBoundBox,rightBoundBox,topBoundBox,bottomBoundBox);
					strokeLength = _slp.maxStrokeLength/2 +1;
					sgi.strokeOutsideBox++;
					continue;
				}


				//clip to the edge of the boundary to avoid overlap.
				float dist = layer.boundaryDistanceImage(currFrameRel+inp.startFrame).val(x-leftBoundBox,y-topBoundBox);

				if (dist < avgRadius/2)
				{
					sgi.boundaryClip++;
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				}


				int pix=0;
				if (_slp.paintOccludedRegions)	
					pix = layer.Mask(currFrame).getPixel(x,y);
				else
					pix = layer.visibleMask(currFrame).getPixel(x,y);

				if (pix == 0)
				{
					if (debug)
						printf("outside mask %i,%f %f, bound %i %i %i %i\n",currFrameRel+inp.startFrame, x,y,leftBoundBox,rightBoundBox,topBoundBox,bottomBoundBox);
					sgi.outsideMask++;
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				}


				if ((_slp.fillUserSeed)&&(_slp.clipToSeedRegions))
				{
					if (inp.seedStrokeImgs[currFrameRel].width()!=0)
					{

						int clip = qRed(inp.seedStrokeImgs[currFrameRel].pixel((int)x,(int)y));
						if (clip==0)
						{
							strokeLength = _slp.maxStrokeLength/2 +1;
							continue;
						}
					}
				}


				if (_slp.strokeDensity < 0.1)
				{

					glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[currFrame], 0);

        
					if ((_errCode = glGetError()) != GL_NO_ERROR) {
						sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
						LOG_STR_COUT(_strbuf);
					}

					float nearbyStrokeDensity = 0;
				
                    if (_readPixelsTiming)
                        t.start();

                    glReadPixels(x-_slp.gridSize/2,_h-(y+_slp.gridSize/2),_slp.gridSize,_slp.gridSize, GL_BGRA, GL_UNSIGNED_BYTE, (void*) _imgDensity);

                    if (_readPixelsTiming)
                       printf("glReadPixels took %i msec\n", t.elapsed());

					if ((_errCode = glGetError()) != GL_NO_ERROR) {
						sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
						LOG_STR_COUT(_strbuf);
					}

                    for(int i=3;i<_slp.gridSize*_slp.gridSize*4;i+=4)
                    	nearbyStrokeDensity+=(float)_imgDensity[i];

                    nearbyStrokeDensity /= (double)_slp.gridSize*_slp.gridSize*255.;

                	//check against density threshold
					if ((nearbyStrokeDensity > _slp.strokeDensity ) && (strokeLength < _slp.maxStrokeLength/2-1))
					{

	
						//if (strokeLength ==0)
						//	dir = 2;

						sgi.pastDensity++;
						strokeLength = _slp.maxStrokeLength/2 +1;
						continue;
					}
				}
			
				float r=-1,g=-1,b=-1;				


				//check against color threshold
				if (_slp.colorDifference < 1)
				{

					getCombinedImageAndUserStrokeColor(currFrameRel, x, y,gaussianMask,gaussianMaskSize,layer,&r, &g, &b);
					QColor currColor(r,g,b);
					float colDist=getColorDist(colorStart[currFrameRel],currColor);
					
					bool allowOccluded = (!_slp.startInOccludedRegions) && _slp.paintOccludedRegions;
					if ((colDist>_slp.colorDifference ) && (!allowOccluded))
					{
						strokeLength = _slp.maxStrokeLength/2 +1;
						sgi.colorThresh++;
						continue;
					}
				}

				if ((_slp.minBrightness != 0) || (_slp.maxBrightness != 255))
				{
					if (r == -1)
					{
						getCombinedImageAndUserStrokeColor(currFrameRel, x, y,gaussianMask,gaussianMaskSize,layer,&r, &g, &b);
					}

					int brightness =r*0.3 + g*0.59+ b*0.11;

					if ((brightness < _slp.minBrightness) || (brightness > _slp.maxBrightness) )
					{
							strokeLength = _slp.maxStrokeLength/2 +1;
							sgi.brightnessThresh++;
							continue;
					}
				}


				if (debug)
					printf("max %i len %i dir %i: index %i\n", _slp.maxStrokeLength, strokeLength,dir, lastPointIndex);


				strokePoints[currFrameRel][lastPointIndex] = Vec2f(x,y);
				strokeAngles[currFrameRel][lastPointIndex] = newDir;
				strokePointCount[currFrameRel]++;

			}//end strokeLength

			
			if (( _slp.optimizeStrokeWidths) && (strokePointCount[currFrameRel]>= _slp.minStrokeLength))
			{

				Vec2f width;

				if ((lastProcessedFrame==-1)||(lastProcessedFrame>=strokeWidths.size()))
					width= optimizeStrokeWidth(currFrameRel,layer,currFrame==frame,strokePoints[currFrameRel],_slp.strokeProperties.startRadius,_slp.strokeProperties.endRadius,colorStart[currFrameRel],currFrame== frame);
				else
					width= optimizeStrokeWidth(currFrameRel,layer,currFrame==frame,strokePoints[currFrameRel],strokeWidths[lastProcessedFrame].x(),strokeWidths[lastProcessedFrame].y(),colorStart[currFrameRel],currFrame== frame);
		
				
				strokeWidths[currFrameRel]=width;
			}
		

			//set the next frame
			if (goingForward==true)
			{
				currFrame++;
				currFrameRel++;
			}
			else
			{
				currFrame--;
				currFrameRel--;
			}
	
		}//end i
	}//end while

	if (debug)
		printf("starting checks\n");
	


	QColor newColor(1,1,1), oldColor(255,255,255),imgColor(255,255,255);
	bool oldColorSet = false;


	QVector<bool> usePoint(_slp.maxStrokeLength);
	for(int j=0;j <_slp.maxStrokeLength;j++)
		usePoint[j] = true;
	
	for(int j=0;j <_slp.maxStrokeLength;j++)
	for(int i=0;i < frameNum;i++)
	{
		if ((strokePointCount[i]>= _slp.minStrokeLength) && ((strokePoints[i][j].x() == -1) && (strokePoints[i][j].y() == -1)))
			usePoint[j] = false;
	}
	
	int seedFramestrokeSize = 0;
	for(int j=0;j < _slp.maxStrokeLength;j++)
	{
		if (((strokePoints[frame-inp.startFrame][j].x() == -1) && (strokePoints[frame-inp.startFrame][j].y() == -1) ) || (usePoint[j] == false))
			continue;
		seedFramestrokeSize++;
	}

	if (seedFramestrokeSize < _slp.minStrokeLength)
	{
		if (debug)
		printf("start frame %i was too small: %i\n",frame,strokePointCount[frame-inp.startFrame]);
		sgi.tooShort++;
		return;
	}


	QColor lastColor = colorStart[frame-inp.startFrame];

	for(int i=0;i < frameNum;i++)
	{

			Stroke *str = new Stroke();
			str->properties = _slp.strokeProperties;

			float divergenceSum =0;
			int divergenceCount =1;


			int leftBoundBox = layer.boundingBox(i+inp.startFrame).left();
			int rightBoundBox = layer.boundingBox(i+inp.startFrame).right();
			int topBoundBox = layer.boundingBox(i+inp.startFrame).top();
			int bottomBoundBox = layer.boundingBox(i+inp.startFrame).bottom();

	
			int strokeSize = 0;
			for(int j=0;j < _slp.maxStrokeLength;j++)
			{
				if (((strokePoints[i][j].x() == -1) && (strokePoints[i][j].y() == -1) ) || (usePoint[j] == false))
				{
					continue;
				}

				float x = strokePoints[i][j].x();
				float y = strokePoints[i][j].y();
				str->add(x,_h-y);
				strokeSize++;


				//record we've been here
				if ((i==0)&&(x>=0) && (x < _w) && (y>=0) && (y < _h)&& (j>0)  && (j < strokePoints[i].size()-1))
					inp.visited[i].setPixel(x,y,true);
				
			/*
			
				//if ((_slp.seedOrder=="Random") && (j>0)  && (j < strokePoints[i].size()-2))
				if ((j==0)  && (j == strokePoints[i].size()-1))
				{
		
					float nextx=strokePoints[i][j].x();
					float nexty=strokePoints[i][j].y();
					int vis;
					if (_slp.paintOccludedRegions == true)	
						vis = layer.Mask(i+inp.startFrame).getPixel(nextx,nexty);
					else
						vis = layer.visibleMask(i).getPixel(nextx,nexty);

					//if the points in the region, push it to the front of the queue to process next
					if ((vis) &&(nextx>=leftBoundBox) && (nextx <= rightBoundBox) && (nexty>=topBoundBox) && (nexty <=bottomBoundBox)&&
						inp.visited[i].getPixel(nextx,nexty)==false)
					{
						float magnitude;
						Vec2f zero(0,0);
						Vec2f newDir = getGreedyDirection(zero,zero,zero,nextx,nexty,i,1,0,1,layer,&magnitude);
						if  (magnitude < _slp.magnitudeThreshold)
						{

							pair<int,int> location(nextx,nexty);
							pair<int,pair<int,int>> seed(rand(),location);
							pointQueue[i].push_front(seed);

							*pointQueueSize=*pointQueueSize+1;
						}
					}
				}
			*/
			}

			if (debug)
			printf("stroke length: %i , min %i \n",strokeSize,_slp.minStrokeLength);
			//Stroke not long enough...
			if (strokeSize< _slp.minStrokeLength)
			{
				createdStrokes[i] = 0;
				delete str;
				continue;
			}


			//Now set the rendering properties back to normal...
			str->properties.alpha = _slp.strokeProperties.alpha;
			str->properties.startRadius=strokeWidths[i].x();
			str->properties.endRadius=strokeWidths[i].y();

		
			//QColor firstCol = colorStart[frame-inp.startFrame];


			//printf("%i %i %i\n", firstCol.red(), firstCol.green(), firstCol.blue());

			bool strokeVis=getNewStrokeColor(layer, i, newColor,imgColor, str,strokeNoise);

			
			//printf("last %i %i %i, new %i %i %i, vis %i \n", lastColor.red(), lastColor.green(), lastColor.blue(), newColor.red(), newColor.green(), newColor.blue(),strokeVis);
			
			if (strokeVis)
			{
				str->properties.color = newColor;
				lastColor=newColor;
			}
			else
				str->properties.color = lastColor;
	
			
			createdStrokes[i] = str;
	}
	//printf("\n" );


//
	blendStrokeColors(createdStrokes,frame-inp.startFrame);

	Stroke *firstStroke = createdStrokes[frame-inp.startFrame];

	if ((_slp.strokeProperties.useImageColor==false) &&((abs(_slp.strokeProperties.color.hue()-firstStroke->properties.color.hue())>100)||(abs(_slp.strokeProperties.color.value()-firstStroke->properties.color.value())>50)||(abs(_slp.strokeProperties.color.saturation()-firstStroke->properties.color.saturation())>50)))
	{
			sprintf(_strbuf, "color problem ");
			LOG_STR_COUT(_strbuf);
			for (int i=0;i < createdStrokes.size();i++)
			{
				delete createdStrokes[i];
				createdStrokes[i]=0;

			}
	}

	QVector<int> strokeAge(inp.endFrame-inp.startFrame+1);

	int age = _slp.fadeRate+1;
	for(int i=0;i <= inp.endFrame-inp.startFrame;i++)
	{
		if (createdStrokes[i] != 0)
		{
			age++;

			if (i==0)
			//We don't want to get rid of strokes at the boundaries
				age = _slp.minStrokeLength/4;
			else if (((i == inp.endFrame-inp.startFrame) && (age > _slp.minFrames/4)))
				age = 1000;
		
		}
		else
			age = 0;

		strokeAge[i] = age;
	}
	for(int i=inp.endFrame-inp.startFrame-1;i >=0;i--)
	{
		if ((strokeAge[i+1] > strokeAge[i])&& (strokeAge[i] != 0))
			strokeAge[i] = strokeAge[i+1];

	}


	float fadeShift = 1;
	int fade = fadeShift;
	if (inp.startFrame<=1)
		fade +=_slp.fadeRate;

	for(int i=0;i <= inp.endFrame-inp.startFrame;i++)
	{
		if (createdStrokes[i] != 0)
		{

            fade = min((float) fade+1,_slp.fadeRate+fadeShift);
	
			if ((strokeAge[i] < _slp.minFrames) )
				createdStrokes[i]->properties.alpha = 0;
			else
			{
				//float currentAlpha=createdStrokes[i]->properties.alpha;
				//float backAlpha=_slp.strokeProperties.alpha * ((float)fade/(float)(_slp.fadeRate+fadeShift));

				//float setAlpha= min(currentAlpha,backAlpha);
				//createdStrokes[i]->properties.alpha = setAlpha;

				createdStrokes[i]->properties.alpha = _slp.strokeProperties.alpha * ((float)fade/(float)(_slp.fadeRate+fadeShift));
		
			}
		}
		else
			fade =fadeShift;
	
	}


	fade = fadeShift;
	if (inp.endFrame>=_frames-1)
		fade +=_slp.fadeRate;

	for(int i=inp.endFrame-inp.startFrame;i >=0;i--)
	{
		if (createdStrokes[i] != 0)
		{
			
            fade = min((float)fade+1,_slp.fadeRate+fadeShift);
		
			createdStrokes[i]->properties.alpha = min(createdStrokes[i]->properties.alpha,_slp.strokeProperties.alpha * ((float)fade/(float)(_slp.fadeRate+fadeShift)));
			fade++;							
		}
		else
			fade =fadeShift;
	}

	int addedStrokes = 0;

	//if ( _slp.optimizeStrokeWidths)
	//{
	//	optimizeStrokeWidths(layer,createdStrokes,strokeNoise);
	//}

	//density render and cleanup
	for(int i=0;i <= inp.endFrame-inp.startFrame;i++)
	{
		if (createdStrokes[i] != 0)
		{


			//if the alpha is zero, don't bother keeping it...
			if (createdStrokes[i]->properties.alpha == 0)
			{
				if (debug) printf("alpha zero\n");
				delete createdStrokes[i];
				createdStrokes[i] = 0;
				continue;
			}

			addedStrokes++;

			if ((_slp.strokeDensity < 0.1) || (_slp.fillEmptyRegions ==true) || (_slp.optimizeStrokes ==true))
			{

                if (_fbo->bind()==false)
                {
                    LOG_STR_COUT("problem binding");
                    return;
                }

				glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[i+inp.startFrame], 0);		
		              


				QColor strokeColor = createdStrokes[i]->properties.color;

				//first, we render as a partly transparent white stroke 
				//for determining stroke density

				float densityWidthShift= _slp.densityWidthAdd + (((float)imgColor.value())/255. +1)*_slp.densityBrightnessFactor;
				
				//float currentBrightnessFactor = _slp.densityWidthAdd + (((float)imgColor.value()-_brightnessLowerThresh)/(_brightnessUpperThresh - _brightnessLowerThresh) + 1)*_slp.densityBrightnessFactor;

				//imgColor.value()

				createdStrokes[i]->properties.startRadius += densityWidthShift;
				createdStrokes[i]->properties.endRadius += densityWidthShift;
				createdStrokes[i]->forceRecompute();

				GLuint oldtex = createdStrokes[i]->properties.texture;
				createdStrokes[i]->properties.texture = _alphaTextureMap.value((int)oldtex);

				createdStrokes[i]->properties.color.setRgb(255,255,255,1);
				createdStrokes[i]->properties.alpha *=0.04;
				createdStrokes[i]->render();

				createdStrokes[i]->properties.texture = oldtex;

				createdStrokes[i]->properties.alpha /= 0.04;
				createdStrokes[i]->properties.startRadius -= densityWidthShift;
				createdStrokes[i]->properties.endRadius -= densityWidthShift;
				createdStrokes[i]->properties.color = strokeColor;
				createdStrokes[i]->forceRecompute();

			}
		}

		strokePoints[i].clear();
		strokeAngles[i].clear();
	}
	strokeWidths.clear();


	sgi.madeToEnd++;
	sgi.addedStrokes+=addedStrokes;


	return;
}


/*

Stroke* StrokePainter::startStrokeFeedForward(int initx, int inity, Stroke* startStroke,int frame, LayerObject& layer,QList<LayerObject>& allLayers, QVector<Stroke*>& createdStrokes, StrokeGenInfo& sgi,QVector<QVector<pair<int,pair<int,int> > > > pointQueue,long *pointQueueSize)
{	
    bool debug = false;

	sgi.count++;

	int currFrame= frame;
	int currFrameRel= frame-inp.startFrame;


	float flowx = 0, flowy = 0;

	float startx=initx;
	float starty=inity;

	if (startStroke!=0)
	{
		startx=startStroke->anchorPoint.x();
		starty=startStroke->anchorPoint.y();

		//get the flow vector if we're going forward
		bool possibleFlow = false;
		possibleFlow= getFlowVector(startx,starty,currFrameRel-1,false,inp.startFrame,layer,allLayers,&flowx, &flowy);
	
		//add the flow to the last starting point
		startx+= flowx;
		starty+= flowy;
	}

	if (debug)
	printf("starting stroke at %f %f %i\n", startx,starty,frame);	


	

	double angle =1;

	


	Vec2f oldDir(0,0);
	Vec2f firstDir(0,0);



	Vec3i strokeNoise;
	strokeNoise.set_x(normal(_slp.hueOffset,_slp.hueNoise+0.001));
	strokeNoise.set_y(normal(_slp.saturationOffset,_slp.saturationNoise+0.001));
	strokeNoise.set_z(normal(_slp.valueOffset,_slp.valueNoise+0.001));


	double *gaussianMask;
	int gaussianMaskSize;
	std::pair<double *,int> p;

	int shiftx=-1, shifty=-1;


	float strokeRad;
	//if (startStroke==0)
		strokeRad=(_slp.strokeProperties.startRadius + _slp.strokeProperties.startRadius )/2.0;
	//else
	//	strokeRad=(startStroke->strokeWidths.x() + startStroke->strokeWidths.y())/2.0;
	double actualRadius = strokeRad*_textureWidthMap.value((int)_slp.strokeProperties.texture);
	
	float avgRadius = actualRadius;


	if ((startStroke==0)&&(_slp.shiftToEdges==true)&&(shiftStrokeToOrientationField(startx,starty,frame-inp.startFrame, actualRadius-1, layer, &shiftx,&shifty)==true))
	{
		startx=shiftx;
		starty=shifty;
		inp.visited[frame-inp.startFrame].setPixel(shiftx,shifty,true);
	}

	bool thrownOutside = false;

	Stroke *newStroke = new Stroke();
	
	if (startStroke!=0)
	{
		startStroke->nextStroke=newStroke;

		if (_feedForwardFade)
		{
			if (startStroke->dying)
			{
				newStroke->dying=true;
				newStroke->age=startStroke->age-1;
				if (newStroke->age <=0)
				{
					
					startStroke->nextStroke=0;
					delete newStroke;
					return 0;
				}
			}
			else
				newStroke->age=startStroke->age+1;
		}
	}
	else
		newStroke->age=1;





	newStroke->strokePointCount = 0;

	for(int j=0;j <= _slp.maxStrokeLength;j++)
	{
		newStroke->strokePoints.push_back(Vec2f(-1,-1));
		newStroke->strokeAngles.push_back(Vec2f(0,0));
	}

	newStroke->strokeWidths=Vec2f(_slp.strokeProperties.startRadius,_slp.strokeProperties.endRadius);



	

	flowx =0;
	flowy =0;


	int strokeLength = 1;

	int leftBoundBox = layer.boundingBox(currFrame).left();
	int rightBoundBox = layer.boundingBox(currFrame).right();
	int topBoundBox = layer.boundingBox(currFrame).top();
	int bottomBoundBox = layer.boundingBox(currFrame).bottom();


	int gridSize=avgRadius;

	int radius = strokeRad;
	

	
	if ((startx<0) || (starty<0) || (startx>=_w-1) || (starty>=_h-1))
	{

		sgi.thrownOutside++;
		thrownOutside = true;
		if (debug)
			printf("outside, breaking\n");

		if (startStroke!=0)
			startStroke->nextStroke=0;
		delete newStroke;
		return 0;
	}




	if (_slp.optimizeStrokeWidths)
	{
		//float strokeRad;
		//if ((lastProcessedFrame==-1)||(lastProcessedFrame>=strokeWidths.size()))
		//	strokeRad=(_slp.strokeProperties.startRadius + _slp.strokeProperties.startRadius )/2.0;
		//else
		//	strokeRad=(strokeWidths[lastProcessedFrame].x() + strokeWidths[lastProcessedFrame].y())/2.0;
		double actualRadius = strokeRad*_textureWidthMap.value((int)_slp.strokeProperties.texture);
		gridSize = max((int)actualRadius,1);

		int avgRad=actualRadius*100;

		if (_gaussianMasks.contains(avgRad))
		{
			p= _gaussianMasks.value(avgRad);
			gaussianMask=p.first;
			gaussianMaskSize=p.second;
		}
		else
		{
			gaussianMask= getGaussianFilter(min((double)avgRadius,15.0)*_slp.radiusFactor,&gaussianMaskSize);

			p.first=gaussianMask;
			p.second=gaussianMaskSize;
			_gaussianMasks.insert(avgRad,p);
		}
	}
	else
	{
		gridSize=_slp.gridSize;
		gaussianMask = _gaussianMask;
		gaussianMaskSize=_gaussianMaskSize;
	}



	if (debug)
		printf("flow %f %f\n", flowx,flowy);

	float r,g,b;				
	getCombinedImageAndUserStrokeColor(currFrameRel, startx, starty,gaussianMask,gaussianMaskSize,layer.visibleMask(currFrameRel+inp.startFrame),&r, &g, &b);
	QColor currCol(r,g,b);
	QColor colorStart = currCol;


	if (_slp.colorDifference < 1)
	{

		float colDist =0;
		
		if (startStroke!=0)
		{
			colDist = getColorDist(startStroke->properties.color,currCol);

			if (_misc4==1)
				colDist = getColorDifference(currFrameRel, startx,starty, gaussianMask,gaussianMaskSize,startStroke->properties.color);
		}

		if ((startStroke!=0) && (colDist > _slp.colorDifference))
		{
			sgi.colorThresh++;
			if (debug)
				printf("color thresh\n");

			if ((!_feedForwardFade)||(startStroke==0))
			{
				if (startStroke!=0)
					startStroke->nextStroke=0;
				delete newStroke;
				return 0;	
			}
			else
			{
				newStroke->dying=true;
				newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
			}

		}
	}

	//record the points we've visited
	newStroke->strokePoints[_slp.maxStrokeLength/2]= Vec2f(startx,starty);

	float x = startx;
	float y = starty;


	newStroke->anchorPoint = Vec2f(x,y);


	for(int dir=0;dir < 2;dir++)
	for(strokeLength=0; strokeLength < (_slp.maxStrokeLength/2);strokeLength++)
	{

		if (dir >=2)
		{
			strokeLength == (_slp.maxStrokeLength/2);
			continue;
		}

		if (debug)
			printf("dir %i, strokelen %i\n", dir,strokeLength);

		bool firstPtForFrame = (strokeLength ==0) && (dir==0);
		float directionSwitch=1;

		//If this is the first part of the stroke, then reset the initial point to the center
		//and set the old direction
		if (strokeLength ==0)
		{

			x = startx;
			y = starty;

			
			if (dir==1)
			{
				directionSwitch = -1;
				oldDir.set_x(firstDir.x()*-1);
				oldDir.set_y(firstDir.y()*-1);
			}
			else
			{
		
				oldDir.set_x(0);
				oldDir.set_y(0);
			}

		}

		assert(dir<2);
		int lastPointIndex;
		if (dir == 0)
			lastPointIndex= (_slp.maxStrokeLength/2)*(1-dir) - strokeLength-1;
		else
			lastPointIndex =(_slp.maxStrokeLength/2)*(dir) + strokeLength+1;
	

		//Vec2f optFlowDir = getOpticalFlowDirection(Vec2f(x,y),layer,allLayers,currFrameRel,lastPointIndex,goingForward,strokePoints);
		Vec2f optFlowDir(0,0);

		
		Vec2f lastFrameDir(0,0);
		if (startStroke!=0)
			lastFrameDir =  startStroke->strokeAngles[lastPointIndex];


		//assertions
		//if this isn't our first point in this frame, we must have an old direction
		if ((firstPtForFrame==false)&& (oldDir.Len2()==0))
		{

			strokeLength = (_slp.maxStrokeLength/2) +1;
			continue;
		}

		//if this isn't the first frame we're drawing, then we must have an direction
		//from a previous frame. There's no point tracing without it since it will be truncated later
		
	
		if ((startStroke!=0)&& (lastFrameDir.Len2()==0))
		{
			strokeLength = _slp.maxStrokeLength/2 +1;
			continue;
		}
	


		float magnitude;
		Vec2f newDir = getDirection(oldDir,lastFrameDir, optFlowDir,x,y,currFrameRel,radius,dir,directionSwitch,layer,&magnitude);



		if (newDir.Dot2(oldDir)<0)
		{
			int test=1;
			Vec2f newDir = getDirection(oldDir,lastFrameDir, optFlowDir,x,y,currFrameRel,radius,dir,directionSwitch,layer,&magnitude);

		}

		if (magnitude >=_slp.magnitudeThreshold)
		{

			sgi.pastMagnitude++;

			//If this is the first point on the first frame, don't bother looking any farther.
			if ((firstPtForFrame) && (startStroke==0))
			{
				if ((_feedForwardFade) &&(startStroke!=0))
				{
					delete newStroke;
					return 0;
				}
				else
				{
					newStroke->dying=true;
					newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
				}
				continue;
			}

	
			
			strokeLength = _slp.maxStrokeLength/2 +1;
			continue;

		}

		if (_slp.optimizeStrokes==false)
		{

			newDir.set_x(_slp.followGradientFactor*(magnitude)*newDir.x() + _slp.followPreviousStrokesFactor*lastFrameDir.x());
			newDir.set_y(_slp.followGradientFactor*(magnitude)*newDir.y() + _slp.followPreviousStrokesFactor*lastFrameDir.y());
			newDir.Normalize();

			truncateDirection(newDir,lastFrameDir,_slp.maxFlowAngle);
			truncateDirection(newDir,oldDir,_slp.maxStrokeAngle);
			if (magnitude == -1)
			{
				if (debug)
					printf("something strange happened while getting the new direction...\n");
				sgi.directionError++;

				if ((_feedForwardFade) &&(startStroke!=0))
				{
					newStroke->dying=true;
					newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
				}
				else
				{
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				}
			}
			else if (magnitude == -2)
			{
				if (debug)
					printf("intersected a guide stroke...\n");
				sgi.guideStrokeIntersection++;
				if ((_feedForwardFade) &&(startStroke!=0))
				{
					newStroke->dying=true;
					newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
				}
				else
				{
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				}
			}
		}
		//optimizing the direction
		else
		{
			Vec2f initDir(0,0);
		
			if (oldDir.Len2() != 0)
				initDir= oldDir;
			else if (lastFrameDir.Len() !=0)
				initDir= lastFrameDir;
			else if  ((dir==0) && (strokeLength==0) && (currFrame == frame))
			{
				initDir= newDir;
			}
			else
				LOG_STR_COUT("Problems with initial stroke direction. Should never be here.");

			//initDir= newDir;

			getNewOptimizedDirection(oldDir,lastFrameDir, x, y, currFrameRel,radius,dir,directionSwitch,colorStart,layer,initDir,gaussianMask, gaussianMaskSize,  &magnitude,newDir);

			if (newDir.Dot2(oldDir)<0)
			{
				int test=1;
				getNewOptimizedDirection(oldDir,lastFrameDir, x, y, currFrameRel,radius,dir,directionSwitch,colorStart,layer,initDir,gaussianMask, gaussianMaskSize,  &magnitude,newDir);

			}

			if (magnitude>= _magMax)
			{
			
				sgi.strokeOptimizationThreshold++;
				if ((_feedForwardFade) &&(startStroke!=0))
				{
					newStroke->dying=true;
					newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
				}
				else
				{
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				}
			}
		}


	

		if ((strokeLength ==0) && (dir==0))
			firstDir = newDir;

		x += radius*newDir.x();
		y += radius*newDir.y();
	
		oldDir = newDir;



		//If we're outside the image, add the point but then quit
		if ((x<2) || (y<2) || (x>=_w-2) || (y>=_h-2))
		{
			if (debug)
				printf("image %i %i\n",x,y);
			strokeLength = _slp.maxStrokeLength/2 +1;


			newStroke->strokePoints[lastPointIndex] = Vec2f(x,y);
			newStroke->strokeAngles[lastPointIndex] = newDir;
			newStroke->strokePointCount++;

			continue;
		}

		else if ((x<leftBoundBox) || (x>=rightBoundBox)  || (y<topBoundBox) || (y >= bottomBoundBox)) 
		{

			if (debug)
				printf("outside box %f %f, bound %i %i %i %i\n",x,y,leftBoundBox,rightBoundBox,topBoundBox,bottomBoundBox);
			strokeLength = _slp.maxStrokeLength/2 +1;
			sgi.strokeOutsideBox++;
			continue;
		}


		//clip to the edge of the boundary to avoid overlap.
		float dist = layer.boundaryDistanceImage(currFrameRel+inp.startFrame).val(x-leftBoundBox,y-topBoundBox);

		if (dist < avgRadius/2)
		{
			sgi.boundaryClip++;
			
			//if ((_feedForwardFade) &&(startStroke!=0))
			//{
			//	newStroke->dying=true;
			//	newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
			//}
			//else
			//{
				strokeLength = _slp.maxStrokeLength/2 +1;
				continue;
			//}
		}


		int pix=0;
		if (_slp.paintOccludedRegions)	
			pix = layer.Mask(currFrame).getPixel(x,y);
		else
			pix = layer.visibleMask(currFrame).getPixel(x,y);

		if (pix == 0)
		{
			if (debug)
				printf("outside mask %i,%f %f, bound %i %i %i %i\n",currFrameRel+inp.startFrame, x,y,leftBoundBox,rightBoundBox,topBoundBox,bottomBoundBox);
			sgi.outsideMask++;
			//if ((_feedForwardFade) &&(startStroke!=0))
			//{
			//	newStroke->dying=true;
			//	newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
			//}
			//else
			//{
				strokeLength = _slp.maxStrokeLength/2 +1;
				continue;
			//}
		}


		if ((_slp.fillUserSeed)&&(_slp.clipToSeedRegions))
		{
			if (inp.seedStrokeImgs[currFrameRel].width()!=0)
			{

				int clip = qRed(inp.seedStrokeImgs[currFrameRel].pixel((int)x,(int)y));
				if (clip==0)
				{
					if ((_feedForwardFade) &&(startStroke!=0))
					{
						newStroke->dying=true;
						newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
					}
					else
					{
						strokeLength = _slp.maxStrokeLength/2 +1;
						continue;
					}
				}
			}
		}


		if (_slp.strokeDensity < 0.1)
		{

			glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[currFrame], 0);


			if ((_errCode = glGetError()) != GL_NO_ERROR) {
				sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
				LOG_STR_COUT(_strbuf);
			}

			float nearbyStrokeDensity = 0;
		
            if (_readPixelsTiming)
                t.start();

            glReadPixels(x-_slp.gridSize/2,_h-(y+_slp.gridSize/2),_slp.gridSize,_slp.gridSize, GL_BGRA, GL_UNSIGNED_BYTE, (void*) _imgDensity);

            if (_readPixelsTiming)
               printf("glReadPixels took %i msec\n", t.elapsed());

			if ((_errCode = glGetError()) != GL_NO_ERROR) {
				sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
				LOG_STR_COUT(_strbuf);
			}

            for(int i=3;i<_slp.gridSize*_slp.gridSize*4;i+=4)
            	nearbyStrokeDensity+=(float)_imgDensity[i];

            nearbyStrokeDensity /= (double)_slp.gridSize*_slp.gridSize*255.;

        	//check against density threshold
			if ((nearbyStrokeDensity > _slp.strokeDensity ) && (strokeLength < _slp.maxStrokeLength/2-1))
			{


				if (strokeLength ==0)
					dir = 2;

				sgi.pastDensity++;
				if ((_feedForwardFade) &&(startStroke!=0))
				{
					newStroke->dying=true;
					newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
				}
				else
				{
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				}
			}
		}
	
		float r=-1,g=-1,b=-1;				


		//check against color threshold
		if (_slp.colorDifference < 1)
		{

			getCombinedImageAndUserStrokeColor(currFrameRel, x, y,gaussianMask,gaussianMaskSize,layer.visibleMask(currFrameRel+inp.startFrame),&r, &g, &b);
			QColor currColor(r,g,b);
			float colDist=getColorDist(colorStart,currColor);
			if (_misc4==1)
				colDist = getColorDifference(currFrameRel, x,y, gaussianMask,gaussianMaskSize,colorStart);
			
			if ( colDist>_slp.colorDifference )
			{
				
				sgi.colorThresh++;
				//if ((_feedForwardFade) &&(startStroke!=0))
				//{
				//	newStroke->dying=true;
				//	newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
				//}
				//else
				//{
					strokeLength = _slp.maxStrokeLength/2 +1;
					continue;
				//}
			}
		}

		if ((_slp.minBrightness != 0) || (_slp.maxBrightness != 255))
		{
			if (r == -1)
			{
				getCombinedImageAndUserStrokeColor(currFrameRel, x, y,gaussianMask,gaussianMaskSize,layer.visibleMask(currFrameRel+inp.startFrame),&r, &g, &b);
			}

			int brightness =r*0.3 + g*0.59+ b*0.11;

			if ((brightness < _slp.minBrightness) || (brightness > _slp.maxBrightness) )
			{
					
					sgi.brightnessThresh++;
					if ((_feedForwardFade) &&(startStroke!=0))
					{
						newStroke->dying=true;
						newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
					}
					else
					{
						strokeLength = _slp.maxStrokeLength/2 +1;
						continue;
					}
			}
		}


		if (debug)
			printf("max %i len %i dir %i: index %i\n", _slp.maxStrokeLength, strokeLength,dir, lastPointIndex);

		newStroke->strokePoints[lastPointIndex] = Vec2f(x,y);
		newStroke->strokeAngles[lastPointIndex] = newDir;
		newStroke->strokePointCount++;


	}//end strokeLength

	
	if (( _slp.optimizeStrokeWidths) && (newStroke->strokePointCount>= _slp.minStrokeLength))
	{

		Vec2f width;

		if (startStroke==0)
			width= optimizeStrokeWidth(currFrameRel,layer,true,newStroke->strokePoints,_slp.strokeProperties.startRadius,_slp.strokeProperties.endRadius,colorStart,true);
		else
			width= optimizeStrokeWidth(currFrameRel,layer,true,newStroke->strokePoints,startStroke->strokeWidths.x(),startStroke->strokeWidths.y(),startStroke->properties.color,false);

		newStroke->strokeWidths=width;

	}
		


	if (debug)
		printf("starting checks\n");
	


	QColor newColor(1,1,1), oldColor(255,255,255),imgColor(255,255,255);
	bool oldColorSet = false;


	
	QVector<bool> usePoint(_slp.maxStrokeLength);
	for(int j=0;j <_slp.maxStrokeLength;j++)
		usePoint[j] = true;
	

	for(int j=0;j <_slp.maxStrokeLength;j++)
	{
		if ((newStroke->strokePointCount>= _slp.minStrokeLength) && ((newStroke->strokePoints[j].x() == -1) && (newStroke->strokePoints[j].y() == -1)))
			usePoint[j] = false;
	}
	
	
	int seedFramestrokeSize = 0;
	for(int j=0;j < _slp.maxStrokeLength;j++)
	{
		if (((newStroke->strokePoints[j].x() == -1) && (newStroke->strokePoints[j].y() == -1) ) || (usePoint[j] == false))
			continue;
		seedFramestrokeSize++;
	}

	if (seedFramestrokeSize < _slp.minStrokeLength)
	{
		if (debug)
		printf("start frame %i was too small: %i\n",frame,newStroke->strokePointCount);
		sgi.tooShort++;

		if ((!_feedForwardFade) || (startStroke==0)|| (seedFramestrokeSize<=1))
		{
			if (startStroke!=0)
				startStroke->nextStroke=0;
			delete newStroke;
			return 0;	
		}
		else
		{
			newStroke->dying=true;
			newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
		}

	}

	

	newStroke->properties = _slp.strokeProperties;

	float divergenceSum =0;
	int divergenceCount =1;


	int strokeSize = 0;
	for(int j=0;j < _slp.maxStrokeLength;j++)
	{
		if (((newStroke->strokePoints[j].x() == -1) && (newStroke->strokePoints[j].y() == -1) ) || (usePoint[j] == false))
		{
			continue;
		}

		float x = newStroke->strokePoints[j].x();
		float y = newStroke->strokePoints[j].y();
		newStroke->add(x,_h-y);
		strokeSize++;


		//record we've been here
		if ((x>=0) && (x < _w) && (y>=0) && (y < _h)&& (j>0)  && (j < newStroke->strokePoints.size()-1))
			inp.visited[frame-inp.startFrame].setPixel(x,y,true);
		
	
	}

	if (debug)
	printf("stroke length: %i , min %i \n",strokeSize,_slp.minStrokeLength);
	//Stroke not long enough...
	if (strokeSize< _slp.minStrokeLength)
	{
		if ((!_feedForwardFade) || (startStroke==0))
		{
			if (startStroke!=0)
				startStroke->nextStroke=0;
			delete newStroke;
			return 0;	
		}
		else
		{
			newStroke->dying=true;
			newStroke->age=min(_slp.fadeRate-1,startStroke->age-1);
		}

	}


	//Now set the rendering properties back to normal...
	newStroke->properties.alpha = _slp.strokeProperties.alpha;
	newStroke->properties.startRadius=newStroke->strokeWidths.x();
	newStroke->properties.endRadius=newStroke->strokeWidths.y();

	getNewStrokeColor(layer, frame-inp.startFrame, newColor,imgColor, newStroke,strokeNoise);



	if ((_slp.strokeProperties.useImageColor==false) &&((abs(_slp.strokeProperties.color.red()-newColor.red())>50)||(abs(_slp.strokeProperties.color.green()-newColor.green())>50)||(abs(_slp.strokeProperties.color.blue()-newColor.blue())>50)))
	{
		printf ("lastCol %i %i %i %i %i %i\n", newColor.red(), newColor.green(), newColor.blue(),_slp.strokeProperties.color.red(),_slp.strokeProperties.color.green(),_slp.strokeProperties.color.blue());
		LOG_STR_COUT("problem with color");
		getNewStrokeColor(layer, frame-inp.startFrame, newColor,imgColor, newStroke,strokeNoise);
	}
	newStroke->properties.color = newColor;

	if (startStroke!=0)
	{
		
		QColor lastCol=startStroke->properties.color;
		newColor.setRed(truncateColor( _slp.colorUpdate*((float)newColor.red()) + (1.0-_slp.colorUpdate)*((float)lastCol.red()))) ;
		newColor.setGreen( truncateColor(_slp.colorUpdate*((float)newColor.green()) + (1.0-_slp.colorUpdate)*((float)lastCol.green()))) ;
		newColor.setBlue(truncateColor( _slp.colorUpdate*((float)newColor.blue()) + (1.0-_slp.colorUpdate)*((float)lastCol.blue()))) ;

		newStroke->properties.color = newColor;

	}


	if ((_slp.strokeDensity < 0.1) || (_slp.fillEmptyRegions ==true) || (_slp.optimizeStrokes ==true))
	{

        if (_fbo->bind()==false)
        {
            LOG_STR_COUT("problem binding");
			if (startStroke!=0)
				startStroke->nextStroke=0;
			delete newStroke;
            return 0;
        }

		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[frame], 0);		
              


		QColor strokeColor = newStroke->properties.color;

		//first, we render as a partly transparent white stroke 
		//for determining stroke density

		float densityWidthShift= _slp.densityWidthAdd + (((float)imgColor.value())/255. +1)*_slp.densityBrightnessFactor;
		
		//float currentBrightnessFactor = _slp.densityWidthAdd + (((float)imgColor.value()-_brightnessLowerThresh)/(_brightnessUpperThresh - _brightnessLowerThresh) + 1)*_slp.densityBrightnessFactor;

		//imgColor.value()

		float fadeAlpha=1;
		if (_feedForwardFade)
		{	
			fadeAlpha= min(((float)(newStroke->age+1)/(float)(_slp.fadeRate+1)),1);
		}
		newStroke->properties.alpha *=fadeAlpha;

		newStroke->properties.startRadius += densityWidthShift;
		newStroke->properties.endRadius += densityWidthShift;
		newStroke->forceRecompute();

		GLuint oldtex = newStroke->properties.texture;
		newStroke->properties.texture = _alphaTextureMap.value((int)oldtex);

		newStroke->properties.color.setRgb(255,255,255,1);
		newStroke->properties.alpha *=0.04;
		newStroke->render();

		newStroke->properties.texture = oldtex;

		newStroke->properties.alpha /= 0.04;
		newStroke->properties.startRadius -= densityWidthShift;
		newStroke->properties.endRadius -= densityWidthShift;
		newStroke->properties.color = strokeColor;
		newStroke->forceRecompute();

	}


	sgi.madeToEnd++;
	sgi.addedStrokes++;


	return newStroke;
}

*/


void StrokePainter::blendStrokeColors(QVector<Stroke*>& createdStrokes, int seedFrameRel)
{


	assert(createdStrokes[seedFrameRel] != 0);

	QColor lastCol = createdStrokes[seedFrameRel]->properties.color;

	createdStrokes[seedFrameRel]->properties.initColor=lastCol;

	for(int i=seedFrameRel+1;i < createdStrokes.size();i++)
	{
		if (createdStrokes[i] == 0)
			break;

		QColor currCol = createdStrokes[i]->properties.color;
		QColor lastColorBkp = lastCol;

		int lr = lastCol.red(), lg=lastCol.green(),lb=lastCol.blue();

		float newRed = _slp.colorUpdate*((float)currCol.red()) + (1.0-_slp.colorUpdate)*((float)lastCol.red()) ;
		float newGreen = _slp.colorUpdate*((float)currCol.green()) + (1.0-_slp.colorUpdate)*((float)lastCol.green());
		float newBlue = _slp.colorUpdate*((float)currCol.blue()) + (1.0-_slp.colorUpdate)*((float)lastCol.blue());

		int tRed = truncateColor(newRed);
		int tGreen = truncateColor(newGreen);
		int tBlue= truncateColor(newBlue);
		lastCol.setRed(tRed) ;
		lastCol.setGreen(tGreen ) ;
		lastCol.setBlue(tBlue) ;

		createdStrokes[i]->properties.color = lastCol;

	}

	lastCol = createdStrokes[seedFrameRel]->properties.color;

	for(int i=seedFrameRel-1;i >= 0;i--)
	{
		if (createdStrokes[i] == 0)
			break;
		
		QColor currCol = createdStrokes[i]->properties.color;

		lastCol.setRed(truncateColor( _slp.colorUpdate*((float)currCol.red()) + (1.0-_slp.colorUpdate)*((float)lastCol.red()))) ;
		lastCol.setGreen( truncateColor(_slp.colorUpdate*((float)currCol.green()) + (1.0-_slp.colorUpdate)*((float)lastCol.green()))) ;
		lastCol.setBlue(truncateColor( _slp.colorUpdate*((float)currCol.blue()) + (1.0-_slp.colorUpdate)*((float)lastCol.blue()))) ;


		createdStrokes[i]->properties.color = lastCol;

	}	


}



bool StrokePainter::getCombinedImageAndUserStrokeColor(int currFrameRel,float x, float y,double* mask, int maskSize, LayerObject& layer, float* red,float* green,float* blue)
{
	float currRed, currGreen, currBlue, placeholder;
	float colorStrokeRed, colorStrokeGreen, colorStrokeBlue, colorStrokeAlpha;

	bool gotCol=false;
	if (_slp.startInOccludedRegions)
	{
		getImageColor(inp.imgs[currFrameRel], x, y,mask,maskSize,layer.Mask(currFrameRel+inp.startFrame),&currRed, &currGreen, &currBlue,&placeholder); 
		getImageColor(inp.colorStrokeImgs[currFrameRel],x, y,mask,maskSize,layer.Mask(currFrameRel+inp.startFrame),&colorStrokeRed, &colorStrokeGreen, &colorStrokeBlue,&placeholder); 
		getImageColor(inp.colorStrokeAlphaImgs[currFrameRel], x, y,mask,maskSize,layer.Mask(currFrameRel+inp.startFrame),&placeholder, &placeholder, &placeholder,&colorStrokeAlpha); 
		gotCol=true;
	}
	else
	{
		gotCol=getImageColor(inp.imgs[currFrameRel], x, y,mask,maskSize,layer.visibleMask(currFrameRel+inp.startFrame),&currRed, &currGreen, &currBlue,&placeholder); 
		getImageColor(inp.colorStrokeImgs[currFrameRel],x, y,mask,maskSize,layer.visibleMask(currFrameRel+inp.startFrame),&colorStrokeRed, &colorStrokeGreen, &colorStrokeBlue,&placeholder); 
		getImageColor(inp.colorStrokeAlphaImgs[currFrameRel], x, y,mask,maskSize,layer.visibleMask(currFrameRel+inp.startFrame),&placeholder, &placeholder, &placeholder,&colorStrokeAlpha); 
	}

	

	*red = colorStrokeRed*(colorStrokeAlpha/255.) + currRed*(1. - colorStrokeAlpha/255.);
	*green = colorStrokeGreen*(colorStrokeAlpha/255.) + currGreen*(1. - colorStrokeAlpha/255.);
	*blue = colorStrokeBlue*(colorStrokeAlpha/255.) + currBlue*(1. - colorStrokeAlpha/255.);

	return gotCol;
	/*
	if (!((*red >=0)&&(*green >=0)&&(*blue >=0)&&(*red <=255)&&(*green <=255)&&(*blue <=255)))
	{
		int test=1;
		*red = truncateColor(*red);
		*green = truncateColor(*green);
		*blue = truncateColor(*blue);
	}
	*/
}



/*

*/



bool StrokePainter::getNewStrokeColor(LayerObject& layer, int currFrameRel,QColor& color,QColor& imgColor,  Stroke *stroke, Vec3i& strokeNoise)
{

	float redSum = 0;
	float greenSum = 0;
	float blueSum =0;

	QColor curr, strokeColor;

	float currRed=0, currGreen=0, currBlue=0, placeholder=0;

	QVector<Point>& pnts = stroke->getControlPoints();


	float imgRed=0, imgGreen=0, imgBlue=0;
	//int lastRed=firstColor.red(), lastGreen=firstColor.green(), lastBlue=firstColor.blue();

	int lastRed=0, lastGreen=0, lastBlue=0;
	bool updatedLastCol=false;


	int pointCount=0;
	double* gaussianMask;
	int maskSize;

	std::pair<double *,int> p;

	if (_slp.strokeProperties.texture==0)
		return false;

	for (int i=0; i < pnts.size();i++)
	{
		
		bool addColor = true;

		//get the avg color from the control points of the stroke
		if (_slp.strokeProperties.useImageColor)
		{	

		
			if ((pnts[i].x()>=0) && (_h-pnts[i].y()>=0) && (pnts[i].x()<_w-1)&& (_h-pnts[i].y()<_h-1))
			{

			
				double weight= (float)(i)/((float)pnts.size()-1);
				double strokeRadius = (1.0-weight)*stroke->properties.startRadius +	weight*stroke->properties.endRadius ;
				double avgRadius = strokeRadius*_textureWidthMap.value((int)_slp.strokeProperties.texture);

				int avgRad=avgRadius*100;

				if (_gaussianMasks.contains(avgRad))
				{
					p= _gaussianMasks.value(avgRad);
					gaussianMask=p.first;
					maskSize=p.second;
				}
				else
				{
					gaussianMask= getGaussianFilter(min((double)avgRadius,15.0)*_slp.radiusFactor,&maskSize);

					p.first=gaussianMask;
					p.second=maskSize;
					_gaussianMasks.insert(avgRad,p);
				}

			
				//bool vis = (bool) layer.visibleMask(currFrameRel+inp.startFrame).getPixel(pnts[i].x(),_h-pnts[i].y());				

				
				//drawing a background so we want to use the previous color
				//if (!(_slp.startInOccludedRegions) && (_slp.paintOccludedRegions) && (!vis))
				//if (!vis)
				//{
						/*
					if (!updatedLastCol)
						addColor=false;
				
					else
					{
						imgRed +=(int)lastRed;
						imgGreen +=(int)lastGreen;
						imgBlue +=(int)lastBlue ;
					}
					*/
					
				//}
				//else
				//{
					//imgRed +=(int)currRed;
					//imgGreen +=(int)currGreen;
					//imgBlue +=(int)currBlue ;
				bool foundCol = getCombinedImageAndUserStrokeColor(currFrameRel,pnts[i].x(),_h-pnts[i].y(),gaussianMask,maskSize,layer,&currRed, &currGreen, &currBlue); 

				if (foundCol)
				{

					lastRed=currRed;
					lastGreen=currGreen;
					lastBlue=currBlue;

					updatedLastCol=true;
				}
				else
					addColor=false;

				//}
			}
			else
				addColor=false;
			
		}
		else
		{
			currRed = _slp.strokeProperties.color.red();
			currGreen = _slp.strokeProperties.color.green();
			currBlue = _slp.strokeProperties.color.blue();
		}

		if (addColor)
		{
			redSum += currRed;
			greenSum += currGreen;
			blueSum += currBlue;
			pointCount++;
		}

	}

	//if ((redSum>pointCount*255)||(greenSum>pointCount*255)||(blueSum>pointCount*255))
	//	return;
	if (pointCount ==0)
	{
	//	printf("point count was 0\n");
		return false;
	}

	if (redSum+greenSum+blueSum<=10)
	{
		printf("black stroke\n");
		return false;
	}

	color.setRed(truncateColor(((float)redSum)/(float)pointCount));
	color.setGreen(truncateColor(((float)greenSum)/(float)pointCount));
	color.setBlue(truncateColor(((float)blueSum)/(float)pointCount));

	//imgColor.setRed(truncateColor(((float)imgRed)/(float)pointCount));
	//imgColor.setGreen(truncateColor(((float)imgGreen)/(float)pointCount));
	//imgColor.setBlue(truncateColor(((float)imgBlue)/(float)pointCount));



	if ((strokeNoise.x()!=0)||(strokeNoise.y()!=0)||(strokeNoise.z()!=0))
	{
	
		int h,s,v;
		color.getHsv(&h,&s,&v);
		//printf("pre %i %i %i\n", h,s,v);
		int newH = max(0,h +  strokeNoise.x());
		int newS = max(0,s +  strokeNoise.y());
		int newV = max(0,v +  strokeNoise.z());
		if (newH >= 360)
			newH = 360;
		if (newS >= 255)
			newS = 255;
		if (newV >= 255)
			newV = 255;
		//printf("post %i %i %i\n",newH,newS,newV);
		color.setHsv(newH,newS,newV);
		//newColor.setHsv(h,s,v);

	}

	return true;
}


bool StrokePainter::getOpticalFlow(int frame, int x, int y,LayerObject& layer,bool reverse,bool useOccludingLayer,QList<LayerObject>& allLayers,  float *vx, float *vy)
{

	if (layer.visibleMask(frame).getPixel(x,y) == 0)
	{
		if (useOccludingLayer==false)
			return false;

		//check the 
		for(int i=0;i < allLayers.size();i++)
		{
			if (allLayers[i].visibleMask(frame).getPixel(x,y) == 1)
			{

				if (reverse==false)
				{
					if (allLayers[i].flowExists(frame) ==false)
						return false;
					int leftFlow = allLayers[i].layerFlow(frame).left();
					int rightFlow = allLayers[i].layerFlow(frame).right();
					int topFlow = allLayers[i].layerFlow(frame).top();
					int bottomFlow = allLayers[i].layerFlow(frame).bottom();

					if ((x < leftFlow) || (y< topFlow)|| (x>=rightFlow)|| (y>=bottomFlow))
					{
						//printf("outside flow image at %i %i, frame %i\n", x,y,frame);
						return false;
					}

					allLayers[i].layerFlow(frame).getFlow(x-leftFlow,y-topFlow,vx,vy);
					return true;
				}
				else
				{
					if (allLayers[i].reverseFlowExists(frame) ==false)
						return false;
					int leftFlow = allLayers[i].layerReverseFlow(frame).left();
					int rightFlow = allLayers[i].layerReverseFlow(frame).right();
					int topFlow = allLayers[i].layerReverseFlow(frame).top();
					int bottomFlow = allLayers[i].layerReverseFlow(frame).bottom();

					if ((x < leftFlow) || (y< topFlow)|| (x>=rightFlow)|| (y>=bottomFlow))
					{
						//printf("outside reverse flow image at %i %i, frame %i\n", x,y,frame);
						return false;
					}
					allLayers[i].layerReverseFlow(frame).getFlow(x-leftFlow,y-topFlow,vx,vy);
					return true;
				}
			}

		}
		return false;
	}
		

	if (reverse==false)
	{
		if (layer.flowExists(frame)==false)
			return false;

		int leftFlow = layer.layerFlow(frame).left();
		int rightFlow = layer.layerFlow(frame).right();
		int topFlow = layer.layerFlow(frame).top();
		int bottomFlow = layer.layerFlow(frame).bottom();

		if ((x < leftFlow) || (y< topFlow)|| (x>=rightFlow)|| (y>=bottomFlow))
		{
			//printf("outside flow image at %i %i, frame %i\n", x,y,frame);
			return false;
		}

		layer.layerFlow(frame).getFlow(x-leftFlow,y-topFlow,vx,vy);
	}
	else
	{
		if (layer.flowExists(frame)==false)
			return false;

		int leftFlow = layer.layerReverseFlow(frame).left();
		int rightFlow = layer.layerReverseFlow(frame).right();
		int topFlow = layer.layerReverseFlow(frame).top();
		int bottomFlow = layer.layerReverseFlow(frame).bottom();

		if ((x < leftFlow) || (y< topFlow)|| (x>=rightFlow)|| (y>=bottomFlow))
		{
			//printf("outside reverse flow image at %i %i, frame %i\n", x,y,frame);
			return false;
		}
		layer.layerReverseFlow(frame).getFlow(x-leftFlow,y-topFlow,vx,vy);
	}

	return true;
}



bool StrokePainter::getFlowVector(float startx, float starty, int frameRel,bool reverse, int startFrame, LayerObject& layer, QList<LayerObject>& allLayers, float* flowx, float* flowy)
{
	float imgMag = 0;
	float boundMag =10, userMag = 10, fixedMag = 1;
	float imgX=0, imgY=0, boundX=0, boundY=0, userX=0, userY=0,fixedX=0,fixedY=0;

	bool gotflow=false;
	if (_slp.imageFlowFactor != 0)
	{


		bool useOccludingFlow=_slp.useOccludingLayerFlow && (_slp.startInOccludedRegions);

		gotflow = getOpticalFlow(frameRel + inp.startFrame, startx, starty, layer, reverse,useOccludingFlow,allLayers, &imgX,&imgY);

		imgMag = _slp.magWeight*(((float)_etfMag[frameRel + inp.startFrame].val(startx,starty))/255.) + 0.0002;
		
	}
	//use the boundary as a default
	if (_slp.userFlowFactor != 0)
	{
		

		float minUserDist = 9999;
	
		Vec2f currPos(startx,starty);

		//get the sum of all distances to all the stroke points
		for (int j=0; j < inp.usrPts[frameRel].size();j++)
		{
			double distance = currPos.distanceTo(inp.usrPts[frameRel][j]);
			if (distance < minUserDist)
				minUserDist = distance;
		}
			
		userMag = _slp.magWeight*exp(-(1.0/10)*minUserDist);


		Vec2f tempUserFlow(0,0);
		double  weightSum = 0;
		
		//get the sum of all distances to all the stroke points for the distances
		for (int j=0; j < inp.usrFlowPts[frameRel].size();j++)
		{
			weightSum +=exp(-0.1*currPos.distanceTo(inp.usrFlowPts[frameRel][j]));
		}

		if (reverse == false)
		{

			//use the distances to compute a weighted average of the displacement
			for (int j=0; j < inp.usrFlowDisp[frameRel].size();j++)
			{
			
				double distance = currPos.distanceTo(inp.usrFlowPts[frameRel][j]);

				double weight = exp(-0.1*distance);
				Vec2f displacement = inp.usrFlowDisp[frameRel][j];
				//printf("%f %f\n", displacement.x(), displacement.y());
				displacement *= (float) (weight/weightSum);
				tempUserFlow += displacement;
			}
		}
		else
		{
			//use the distances to compute a weighted average of the displacement
			for (int j=0; j < inp.usrReverseFlowDisp[frameRel].size();j++)
			{
			
				double distance = currPos.distanceTo(inp.usrFlowPts[frameRel][j]);

				double weight = exp(-0.1*distance);
				Vec2f displacement = inp.usrReverseFlowDisp[frameRel][j];
				displacement *= (float) (weight/weightSum);
				tempUserFlow += displacement;
			}

		}


		//printf("%f %f\n", tempUserFlow.x(), tempUserFlow.y());
		userX = tempUserFlow.x();
		userY = tempUserFlow.y();
	}


	if ((_slp.boundaryFlowFactor != 0) || ((_slp.imageFlowFactor != 0) && (gotflow==false)))
	{



		int leftBoundBox = layer.boundingBox(frameRel+startFrame).left();
		int rightBoundBox = layer.boundingBox(frameRel+startFrame).right();
		int topBoundBox = layer.boundingBox(frameRel+startFrame).top();
		int bottomBoundBox = layer.boundingBox(frameRel+startFrame).bottom();

		if ((layer.boundaryImageCreated(frameRel+startFrame)==false)|| (startx <= leftBoundBox) || (starty<= topBoundBox) || (startx>= rightBoundBox) || (starty>= bottomBoundBox))
			return false;

		//float dist = layer.boundaryDistanceImage(frameRel+startFrame).val(startx-leftBoundBox,starty-topBoundBox);

		float dist = layer.boundaryDistanceImage(frameRel+startFrame).val(startx-leftBoundBox,starty-topBoundBox);

		boundMag = _slp.magWeight*exp(-(1.0/10)*dist);


		ANNkd_tree* kdtree = layer.getBoundaryKDTree(frameRel+startFrame);


		if (kdtree ==0)
			return false;

		int numPoints = _kdTreeK;
		if (layer.getContourPoints(frameRel+startFrame).size() <_kdTreeK)
			numPoints = layer.getContourPoints(frameRel+startFrame).size();

		if (numPoints ==0)
		{
			boundX =0;
			boundY =0;
		}
		else
		{

                        ANNidxArray		nnIdx;					// near neighbor indices
			ANNdistArray		dists;					// near neighbor distances
			nnIdx = new ANNidx[numPoints];						// allocate near neigh indices
			dists = new ANNdist[numPoints];						// allocate near neighbor dists

			ANNpoint queryPt;				// query point
			queryPt = annAllocPt(2);	

			queryPt[0] = startx;
			queryPt[1] = starty;

			//printf("looking for %f %f\n", x, y);

			float eps = 0;

			kdtree->annkSearch(						// search
					queryPt,						// query point
					numPoints,						// number of near neighbors
					nnIdx,							// nearest neighbors (returned)
					dists,							// distance (returned)
					eps);							// error bound

			annDeallocPt(queryPt);

			double  weightSum = 0;

                        const QList<QPointF>& curPts = layer.getContourPoints(frameRel+startFrame);


			//printf("%i\n", inp.usrDisp[currFrameRel].size());

			for(int j=0;j < numPoints;j++)
			{
				weightSum +=exp(-(1.0/_slp.distWeight)*sqrt(dists[j]));


			}

			boundX =0;
			boundY =0;

			for(int j=0;j < numPoints;j++)
			{
				double weight = exp(-(1.0/_slp.distWeight)*sqrt(dists[j]));
				
				if (reverse == false)
				{	
                                        const QList<QPointF>& nextPts = layer.getContourPoints(frameRel+startFrame+1);
                                        int pt = nnIdx[j];
                                        if ((pt < 0) || (pt > nextPts.size()) || (pt > curPts.size()) )
                                        {
                                            int test=1;
                                             continue;
                                        }
                                        boundX += (weight/weightSum)*(nextPts[pt].x() - curPts[pt].x());
                                        boundY += (weight/weightSum)*(nextPts[pt].y() - curPts[pt].y());
				}
				else
				{
                                        const QList<QPointF>& nextPts = layer.getContourPoints(frameRel+startFrame-1);
                                        int pt = nnIdx[j];
                                         if ((pt < 0) || (pt > nextPts.size()) || (pt > curPts.size()) )
                                        {
                                            int test=1;
                                             continue;
                                        }
                                        boundX += (weight/weightSum)*(nextPts[pt].x()-curPts[pt].x());
                                        boundY += (weight/weightSum)*(nextPts[pt].y()-curPts[pt].y());
				}
			}
		}


		if ((_slp.imageFlowFactor != 0) && (gotflow==false))
		{
			imgX = boundX;
			imgY = boundY;
			imgMag = boundMag;
		}
	}

	if (_slp.fixedFlowFactor != 0)
	{
		fixedX = cos(_slp.fixedMovementAngle)*_slp.fixedMovementRate;
		fixedY = -1*sin(_slp.fixedMovementAngle)*_slp.fixedMovementRate;
		if (reverse == false)
		{
			fixedX*=-1;
			fixedY*=-1;
		}
	}

	
	float magSum = _slp.imageFlowFactor*imgMag+_slp.boundaryFlowFactor*boundMag+_slp.userFlowFactor*userMag+_slp.fixedFlowFactor*fixedMag;

	if (magSum != 0)
	{
		//Weight the flow by the magnitude of the each of the vector fields at this point.
		*flowx = imgX*(_slp.imageFlowFactor*imgMag/magSum) + 
				 userX*(_slp.userFlowFactor*userMag/magSum) + 
				 boundX*(_slp.boundaryFlowFactor*boundMag/magSum)  + 
				 fixedX*(_slp.fixedFlowFactor*fixedMag/magSum);
		*flowy = imgY*(_slp.imageFlowFactor*imgMag/magSum) + 
				 userY*(_slp.userFlowFactor*userMag/magSum) + 
				 boundY*(_slp.boundaryFlowFactor*boundMag/magSum)  + 
				 fixedY*(_slp.fixedFlowFactor*fixedMag/magSum);

                 //printf("flowx %f %f\n", *flowx,*flowy);
                //*flowx= normal(*flowx,_slp.movementVariance);
                //*flowy= normal(*flowy,_slp.movementVariance);
	}
	else
	{
		*flowx = 0;
		*flowy = 0;
	}



	return true;
}






bool StrokePainter::getNewOptimizedDirection(Vec2f& oldDir,Vec2f& lastFrameDir, float x, float y, int currFrameRel,int radius,int dir,float directionSwitch,QColor& startColor,LayerObject& layer, Vec2f& startingDir, double* gaussianMask,int gaussianMaskSize,float *eval,Vec2f& optimizedDir)
{

	if ( _slp.optimizeStrokes==false)
		return false;


	if (_fbo->bind() == false)
		LOG_STR_COUT("error binding");

	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _frameTextures[currFrameRel+inp.startFrame], 0);		

	if (lastFrameDir.Len() != 0)
	{
		int test=1;
	}

	float initAngle=atan2(startingDir.y(),startingDir.x());

	//Vec2f currAngle(newDir);

	float angleThresholdRadian =(PI*90)/180.; 
	float initialDist,distLeft,distRight;

	float r,g,b;
	float dirX=radius*cos(initAngle);
	float dirY=radius*sin(initAngle);
	initialDist = evaluateControlPoint(layer,oldDir,lastFrameDir,x,y, x+dirX, y+dirY, currFrameRel,gaussianMask, gaussianMaskSize, radius,dir,directionSwitch,startColor);


	//if (initialDist < _slp.colorDifference/2.)
	//	return false;
	float curShift = (-10*PI)/180;
	float leftx=x+dirX*cos(curShift) - dirY*sin(curShift);
	float lefty=y+dirX*sin(curShift) + dirY*cos(curShift);
	distLeft = evaluateControlPoint(layer,oldDir,lastFrameDir,x,y, leftx, lefty, currFrameRel,gaussianMask, gaussianMaskSize, radius,dir,directionSwitch,startColor);

	curShift = (10*PI)/180;
	float rightx=x+dirX*cos(curShift) - dirY*sin(curShift);
	float righty=y+dirX*sin(curShift) + dirY*cos(curShift);
	distRight = evaluateControlPoint(layer,oldDir,lastFrameDir,x,y, rightx, righty, currFrameRel,gaussianMask, gaussianMaskSize,  radius,dir,directionSwitch,startColor);


	bool goingLeft=false;
	float oldDist,newDist;

	if (distLeft < distRight)
	{
		goingLeft=true;
		curShift =(-10*PI)/180.;
		oldDist = distLeft;
	}
	else
		oldDist = distRight;

	//Both sides are higher...
	if (oldDist >= initialDist-0.01)
	{
		*eval=initialDist;
		optimizedDir = startingDir;
		return true;	
	}

	float minDist =min(ceil(float(radius)/2.),2);
	float lastx=x+dirX;
	float lasty=y+dirY;
	float newx=0;
	float newy=0;
	

	//int maxIter = 20;
	//int i=0;
	
	while(true)
	{

		if (angleThresholdRadian < abs(curShift))
			break;
		else
		{
			if (goingLeft)
				curShift -= (5*PI)/180.;
			else
				curShift += (5*PI)/180.;
		}

		newx=x + dirX*cos(curShift) - dirY*sin(curShift);
		newy=y + dirX*sin(curShift) + dirY*cos(curShift);

		//not far enough yet
		if (sqrt((lastx-newx)*(lastx-newx) + (lasty-newy)*(lasty-newy)) < minDist)
			continue;

		//i++;

		newDist = evaluateControlPoint(layer,oldDir,lastFrameDir,x,y, newx, newy, currFrameRel,gaussianMask, gaussianMaskSize,  radius,dir,directionSwitch,startColor);

		if (newDist >= oldDist)
		{
			newx=lastx;
			newy=lasty;
			break;
		}
		oldDist=newDist;

		//if (i >= maxIter)
	//		break;

		lastx=newx;
		lasty=newy;
	}

	*eval=oldDist;
	optimizedDir.set_x(newx-x);
	optimizedDir.set_y(newy-y);
	optimizedDir.Normalize();

	return true;

}


float StrokePainter::evaluateControlPoint(LayerObject& layer,Vec2f& oldDir,Vec2f& lastFrameDir,float startx, float starty, float newx, float newy, int currFrameRel, double* gaussianMask, int gaussianMaskSize, float radius,int dir,int directionSwitch,QColor& startColor)
{
	float orientationComp = 0;
	float coherenceComp = 0;
	float colorComp = 0;
	float densityComp = 0;
	float rigidityComp = 0;
	float coherenceAngle = 0;
	float rigidityAngle = 0;
	float edgeComp=0;

	if ((newx<0)||(newx>=_w)||(newy<0)||(newy>=_h))
		return 999999;

	Vec2f currDir(newx-startx, newy-starty);
	currDir.Normalize();
	
	if (lastFrameDir.Len() != 0)
	{
        float dotProd = min((double)currDir.Dot2(lastFrameDir),1.);
		coherenceAngle = acos(dotProd)*(180./PI);
		coherenceComp = exp(coherenceAngle/MyPreference::coherenceAngleSigma) - 1;
	}

	Vec2f checkDir = oldDir;
	if (checkDir.Len2() ==0)
		checkDir = lastFrameDir;

	if (checkDir.Len() != 0)
	{
         float dotProd = min((double)currDir.Dot2(checkDir),1.);
		 rigidityAngle = acos(dotProd)*(180./PI);
		rigidityComp = exp(rigidityAngle/MyPreference::rigidityAngleSigma) - 1;
	}

    orientationComp = evaluateOrientationFieldComponent(layer,startx, starty,currFrameRel+inp.startFrame, currDir,radius);

	float r,g,b;
	//get color difference
	if ((newx<0)||(newx>_w)||(newy<0)||(newy>_h))
		colorComp=0;
	else
	{
		bool gotCol=getCombinedImageAndUserStrokeColor(currFrameRel, newx, newy,gaussianMask,gaussianMaskSize,layer,&r, &g, &b);
        QColor col(r,g,b);
		if (gotCol && (_slp.imageGradientFactor != 0))
		{	
			colorComp = getColorDist(startColor,col);
			
		}
		

		if ((_slp.colorDifference < 1)&&( colorComp>_slp.colorDifference ))
		{
			return _magMax;
		}

		if ((_slp.minBrightness != 0) || (_slp.maxBrightness != 255))
		{
			if ((col.value() < _slp.minBrightness) || (col.value() > _slp.maxBrightness) )
			{
				return _magMax;
			}
		}
	}



	if (_slp.imageGradientFactor != 0)
	{
		getEdgeness(inp.edges[currFrameRel], newx, newy,gaussianMask,gaussianMaskSize,&edgeComp);
	}


	float nearbyStrokeDensity = 0;
	
	if (_slp.densityFactor!=0)
	{
		glReadPixels(newx-_slp.gridSize/2,_h-(newy+_slp.gridSize/2),_slp.gridSize,_slp.gridSize, GL_BGRA, GL_UNSIGNED_BYTE, (void*) _imgDensity);

		for(int i=3;i<_slp.gridSize*_slp.gridSize*4;i+=4)
					nearbyStrokeDensity+=(float)_imgDensity[i];

		nearbyStrokeDensity /= (double)_slp.gridSize*_slp.gridSize*255;
	}

	densityComp = nearbyStrokeDensity;
		

	float pointEval;


	pointEval =     _slp.orientationFactor*orientationComp + 
					_slp.coherenceFactor*coherenceComp + 
					_slp.rigidityFactor*rigidityComp +
					_slp.colorFactor*colorComp + 
					_slp.densityFactor*densityComp +
					_slp.oswEdgeFactor*edgeComp;

	
	return pointEval;
}



float StrokePainter::evaluateOrientationFieldComponent(LayerObject& layer,float startx, float starty,int currFrame,Vec2f& dir, float radius)
{

	//if (_misc4==3)
	//	return evaluateOrientationFieldComponentNew(layer,startx, starty, currFrame,dir, radius);


	Vec2f currDir;

	Vec2f pos(startx,starty);

	float discrepancy = 0;
        float mag, angle,dist;
	int count=0;

    IntegerImage& boundDist = layer.boundaryDistanceImage(currFrame);
	IntegerImage& boundDistDx = layer.boundaryDistanceDxImage(currFrame);
	IntegerImage& boundDistDy = layer.boundaryDistanceDyImage(currFrame);
    int leftBoundBox = layer.boundingBox(currFrame).left();
    int rightBoundBox = layer.boundingBox(currFrame).right();
    int topBoundBox = layer.boundingBox(currFrame).top();
    int bottomBoundBox = layer.boundingBox(currFrame).bottom();

    //make sure this is normalized
    dir.Normalize();

    float factorSum = _slp.imageGradientFactor + _slp.boundaryFactor + _slp.userStrokesFactor + _slp.fixedOrientationFactor;


	for (int r=0;r< radius;r++)
	{
		if ((pos.x()<1) || (pos.x()>=_w-1) || (pos.y()<1) || (pos.y()>=_h-1))
			break;



        if (_slp.boundaryFactor !=0)
        {


            if ((pos.x()>=leftBoundBox) && (pos.x() <= rightBoundBox) && (pos.y()>=topBoundBox) && (pos.y() <=bottomBoundBox))
            {
                dist = boundDist.val(pos.x()-leftBoundBox,pos.y()-topBoundBox);
                mag = _slp.magWeight*exp(-(1.0/10)*dist)+0.5;
                angle = atan2(boundDistDx.val(pos.x()-leftBoundBox,pos.y()-topBoundBox),-1*boundDistDy.val(pos.x()-leftBoundBox,pos.y()-topBoundBox));
             }

            currDir.set_x(cos(angle));
            currDir.set_y(sin(angle));
            discrepancy += mag*(1-abs(dir.Dot2(currDir))) * (_slp.boundaryFactor/factorSum);
        }

        if (_slp.imageGradientFactor != 0)
        {
            angle = angleTransform(_etfAngle[currFrame].val((pos.x()),(pos.y())));
			
            mag = (((float)_etfMag[currFrame].val(pos.x(),pos.y()))/255.) + 0.0002;

            currDir.set_x(cos(angle));
            currDir.set_y(sin(angle));

            discrepancy += _slp.magWeight*mag*(1-abs(dir.Dot2(currDir))) * (_slp.imageGradientFactor/factorSum);

        }

        if (_slp.fixedOrientationFactor != 0)
        {
            currDir.set_x(cos(_slp.orientationAngle));
            currDir.set_y(-1*sin(_slp.orientationAngle));
            discrepancy += (1-abs(dir.Dot2(currDir)))*(_slp.fixedOrientationFactor/factorSum);
        }


		pos += dir;
		count++;
	}	


	if (_slp.userStrokesFactor != 0)
	{

		currDir = getUserStrokeDirection(layer,pos.x(),pos.y(),currFrame, dir,&dist);
		mag = _slp.magWeight*exp(-(1.0/10)*dist)+0.5;

		if (mag>=0)
		{
			discrepancy += radius*mag*(1-abs(dir.Dot2(currDir))) * (_slp.userStrokesFactor/factorSum);
		}	
	
	}


	if (count==0)
		return 0;
	else
		return discrepancy/((float)count);
}



/*
float StrokePainter::evaluateOrientationFieldComponentNew(LayerObject& layer,float startx, float starty,int currFrame,Vec2f& dir, float radius)
{




	Vec2f pos(startx,starty);

	float discrepancy = 0;
    float mag;
	int count=0;


    //make sure this is normalized
    dir.Normalize();


	for (int r=0;r< radius;r++)
	{
		if ((pos.x()<1) || (pos.x()>=_w-1) || (pos.y()<1) || (pos.y()>=_h-1))
			break;

		Vec2f zero(0,0);
		Vec2f currDir = getDirection(zero,zero,zero,pos.x(),pos.y(),currFrame-inp.startFrame,1,0,1,layer,&mag,false);

		discrepancy += mag*(1-abs(dir.Dot2(currDir)));


		pos += currDir;
		count++;
	}	

	if (count==0)
		return 0;
	else
		return discrepancy/((float)count);
}
*/


Vec2f StrokePainter::getUserStrokeDirection( LayerObject& layer,float x, float y,int currFrame,Vec2f& checkDir,float *mag)
{

	Vec2f userDir(0,0);

	ANNkd_tree* kdtree = layer.getGuideKDTree(currFrame);

	if (kdtree ==0)
	{
	
		*mag = -1;
	}
	else
	{
		ANNidxArray			nnIdx = new ANNidx[_kdTreeK];					// near neighbor indices
		ANNdistArray		dists = new ANNdist[_kdTreeK];					// near neighbor distances

		float minUserDist = 9999;
		
	
		ANNpoint queryPt;				// query point
		queryPt = annAllocPt(2);	

		queryPt[0] = x;
		queryPt[1] = y;


		float eps = 0;

		kdtree->annkSearch(						// search
				queryPt,						// query point
				_kdTreeK,						// number of near neighbors
				nnIdx,							// nearest neighbors (returned)
				dists,							// distance (returned)
				eps);							// error bound

		minUserDist = sqrt(dists[0]);
		annDeallocPt(queryPt);

		minUserDist = sqrt(dists[0]);

		double  weightSum = 0;
		for(int j=0;j < _kdTreeK;j++)
			weightSum +=exp(-(1.0/_slp.distWeight)*sqrt(dists[j]));

		for(int j=0;j < _kdTreeK;j++)
		{
			
			double weight = exp(-(1.0/_slp.distWeight)*sqrt(dists[j]));

			if (inp.usrDisp[currFrame-inp.startFrame].size() <= nnIdx[j])
			{
            sprintf(_strbuf, "wtf, problem in getting user strokes. size is %i, looking for %i\n",inp.usrDisp[currFrame-inp.startFrame].size(),nnIdx[j]);
				LOG_STR_COUT(_strbuf);

				*mag=-1;
				delete [] nnIdx;
				delete [] dists;	
				Vec2f zero(0,0);
				return zero;
			}

			Vec2f displacement = inp.usrDisp[currFrame-inp.startFrame][(nnIdx[j])];
			displacement.Normalize();

			//if ((displacement.Dot2(checkDir) < 0))
			//	displacement *=-1;

			displacement *= (float) (weight/weightSum);
			userDir += displacement;	

		}
		*mag = minUserDist;

		delete [] nnIdx;
		delete [] dists;	
	}
	userDir.Normalize();

	return userDir;

}




Vec2f StrokePainter::getDirection(Vec2f& oldDir,Vec2f& lastFrameDir, Vec2f& optFlowDir, float x, float y, int currFrameRel,int radius,int dir,float directionSwitch,LayerObject& layer, float *mag,bool invMag)
{

	Vec2f newDir;

	float imgMag = 0;

	float invimgMag = 0;


	float boundMag = 0, userMag = 0;
	float fixedMag = 1;

	float invboundMag = 0, invuserMag = 0, invfixedMag = 1.0/fixedMag;

	Vec2f tempBoundDir;
	Vec2f boundDir(0,0);
	Vec2f imgDir(0,0);
	Vec2f userDir(0,0);
	Vec2f fixedDir(0,0);
	float boundDist;

	Vec2f checkDir = oldDir;
	if (checkDir.Len2() ==0)
		checkDir = lastFrameDir;

	//float imgAngle, 

	if (_slp.imageGradientFactor != 0)
	{
		float m = (((float)_etfMag[currFrameRel + inp.startFrame].val(x,y))/255.) + 0.0002;
		imgMag = _slp.magWeight*m;

		invimgMag = 1.0/(m+0.01);

		float angle = angleTransform(_etfAngle[currFrameRel+ inp.startFrame].val(x,y));
		angle+=PI;
	
		float ang=((float)angle)/255.; 
		if (ang>=PI) 
			ang=ang-PI; 

		imgDir.set_x(cos(2*angle));
		imgDir.set_y(sin(2*angle));
		//imgDir= getGradientDirection(oldDir,lastFrameDir, x,y,currFrameRel,inp.startFrame,radius,dir,directionSwitch);

	}
	if (_slp.boundaryFactor != 0)
	{
		
		IntegerImage& boundDistImg = layer.boundaryDistanceImage(currFrameRel+inp.startFrame);
		IntegerImage& boundDistDx = layer.boundaryDistanceDxImage(currFrameRel+inp.startFrame);
		IntegerImage& boundDistDy = layer.boundaryDistanceDyImage(currFrameRel+inp.startFrame);

		int leftBoundBox = layer.boundingBox(currFrameRel+inp.startFrame).left();
		int rightBoundBox = layer.boundingBox(currFrameRel+inp.startFrame).right();
		int topBoundBox = layer.boundingBox(currFrameRel+inp.startFrame).top();
		int bottomBoundBox = layer.boundingBox(currFrameRel+inp.startFrame).bottom();
		if (!((x>=leftBoundBox) && (x <= rightBoundBox) && (y>=topBoundBox) && (y <=bottomBoundBox)))
		{
			boundMag=0;
			invboundMag=0;
			boundDir.set_x(0);
			boundDir.set_y(0);
		}
		else
		{

			boundDist = boundDistImg.val(x-leftBoundBox,y-topBoundBox);

			if ((x>=leftBoundBox) && (x <= rightBoundBox) && (y>=topBoundBox) && (y <=bottomBoundBox))
			{
				
				float angle = atan2(boundDistDx.val(x-leftBoundBox,y-topBoundBox),-1*boundDistDy.val(x-leftBoundBox,y-topBoundBox));
				angle += PI;
				if (angle>=PI)
					angle-=PI;

				boundDir.set_x(cos(2*angle));
				boundDir.set_y(sin(2*angle));
	

			}
			boundMag = _slp.magWeight*exp(-(1.0/10)*boundDist);
			invboundMag = boundDist;

		}	
	
	}

	if (_slp.userStrokesFactor != 0)
	{

		float minUserDist;
		userDir =  getUserStrokeDirection(layer,x,y,currFrameRel+inp.startFrame,checkDir,&minUserDist);

		if (minUserDist==-1)
		{
			userMag=0;
			invuserMag=0;
		}
		else
		{
			float angle=atan2(userDir.y(),userDir.x());
			angle += PI;
			if (angle>=PI)
				angle-=PI;
			userDir.set_x(cos(2*angle));
			userDir.set_y(sin(2*angle));


			userMag = _slp.magWeight*exp(-(1.0/10)*minUserDist);
			invuserMag = minUserDist;
		}

		
	}
	if (_slp.fixedOrientationFactor != 0)
	{
		float angle =_slp.orientationAngle;
		if (angle>=PI)
			angle-=PI;
		fixedDir.set_x(cos(2*angle));
		fixedDir.set_y(-1*sin(2*angle));


	}


	float factorSum = _slp.imageGradientFactor + _slp.boundaryFactor + _slp.userStrokesFactor + _slp.fixedOrientationFactor;
	
	if (invMag)
	{
		*mag = (_slp.imageGradientFactor*invimgMag)/factorSum + 
			   (_slp.boundaryFactor*invboundMag)/factorSum + 
			   (_slp.userStrokesFactor*invuserMag)/factorSum + 
			   (_slp.fixedOrientationFactor*invfixedMag)/factorSum;
	}
	else
	{
		*mag = (_slp.imageGradientFactor*imgMag)/factorSum + 
		   (_slp.boundaryFactor*boundMag)/factorSum + 
		   (_slp.userStrokesFactor*userMag)/factorSum + 
		   (_slp.fixedOrientationFactor)/factorSum;
	}


	//get the new direction as a weighted average of the possible directions
	newDir.set_x(_slp.imageGradientFactor*imgMag*imgDir.x() + 
				 _slp.boundaryFactor*boundMag*boundDir.x() + 
				 _slp.userStrokesFactor*userMag*userDir.x() +
				 _slp.fixedOrientationFactor*fixedMag*fixedDir.x());
	newDir.set_y(_slp.imageGradientFactor*imgMag*imgDir.y() + 
				 _slp.boundaryFactor*boundMag*boundDir.y() + 
				 _slp.userStrokesFactor*userMag*userDir.y()+
				 _slp.fixedOrientationFactor*fixedMag*fixedDir.y());


	if (newDir.Len() == 0)
	{
		*mag = -1;
		return newDir;
	}
	newDir.Normalize();



	float angle=atan2(newDir.y(),newDir.x());
	angle/=2;
	newDir.set_x(cos(angle));
	newDir.set_y(sin(angle));


	if (newDir.Dot2(checkDir)<0)
		newDir*=-1;
	
	return newDir;

}


Vec2f StrokePainter::getGreedyDirection(Vec2f& oldDir,Vec2f& lastFrameDir, Vec2f& optFlowDir, float x, float y, int currFrameRel,int radius,int dir,float directionSwitch,LayerObject& layer, float *mag)
{

	Vec2f newDir;

	float imgMag = 0;

	float invimgMag = 0;


	float boundMag = 0, userMag = 0;
	float fixedMag = 1;

	float invboundMag = 0, invuserMag = 0, invfixedMag = 1.0/fixedMag;

	Vec2f tempBoundDir;
	Vec2f boundDir(0,0);
	Vec2f imgDir(0,0);
	Vec2f userDir(0,0);
	Vec2f fixedDir(0,0);
	float boundDist;

	Vec2f checkDir = oldDir;
	if (checkDir.Len2() ==0)
		checkDir = lastFrameDir;


	if (_slp.imageGradientFactor != 0)
	{
		float m = (((float)_etfMag[currFrameRel + inp.startFrame].val(x,y))/255.) + 0.0002;
		imgMag = _slp.magWeight*m;

		invimgMag = 1.0/(m+0.01);

		imgDir= getGradientDirection(oldDir,lastFrameDir, x,y,currFrameRel,inp.startFrame,radius,dir,directionSwitch);

		if ((imgDir.Dot2(checkDir) < 0))
			imgDir *=-1;

		if (checkDir.Len2() ==0)
			checkDir=imgDir;


	}
	if (_slp.boundaryFactor != 0)
	{
		
		
		boundDir = getBoundaryDirection(x,y,currFrameRel,inp.startFrame,radius,dir,directionSwitch,layer,&boundDist);	

		if (boundDir.Len() == 0)
		{
			boundMag=0;
			invboundMag=0;
		}
		else
		{
			boundMag = _slp.magWeight*exp(-(1.0/10)*boundDist);
			invboundMag = boundDist;
		}


		if ((boundDir.Dot2(checkDir) < 0))
			boundDir *=-1;

		if (checkDir.Len2() ==0)
			checkDir=boundDir;
	
	}

	if (_slp.userStrokesFactor != 0)
	{

		float minUserDist;
		userDir =  getUserStrokeDirection(layer,x,y,currFrameRel+inp.startFrame,checkDir,&minUserDist);

		if (minUserDist==-1)
		{
			userMag=0;
			invuserMag=0;
		}
		else
		{
			userMag = _slp.magWeight*exp(-(1.0/10)*minUserDist);
			invuserMag = minUserDist;
		}


		if ((userDir.Dot2(checkDir) < 0))
			userDir *=-1;

		if (checkDir.Len2() ==0)
			checkDir=userDir;
		
	}
	if (_slp.fixedOrientationFactor != 0)
	{
		fixedDir.set_x(cos(_slp.orientationAngle)*directionSwitch);
		fixedDir.set_y(-1*sin(_slp.orientationAngle)*directionSwitch);

		if ((fixedDir.Dot2(checkDir) < 0))
		{
			fixedDir *=-1;
		}

	}


	float factorSum = _slp.imageGradientFactor + _slp.boundaryFactor + _slp.userStrokesFactor + _slp.fixedOrientationFactor;
	
	*mag = (_slp.imageGradientFactor*invimgMag)/factorSum + 
		   (_slp.boundaryFactor*invboundMag)/factorSum + 
		   (_slp.userStrokesFactor*invuserMag)/factorSum + 
		   (_slp.fixedOrientationFactor*invfixedMag)/factorSum;


	//get the new direction as a weighted average of the possible directions
	newDir.set_x(_slp.imageGradientFactor*imgMag*imgDir.x() + 
				 _slp.boundaryFactor*boundMag*boundDir.x() + 
				 _slp.userStrokesFactor*userMag*userDir.x() +
				 _slp.fixedOrientationFactor*fixedMag*fixedDir.x());
	newDir.set_y(_slp.imageGradientFactor*imgMag*imgDir.y() + 
				 _slp.boundaryFactor*boundMag*boundDir.y() + 
				 _slp.userStrokesFactor*userMag*userDir.y()+
				 _slp.fixedOrientationFactor*fixedMag*fixedDir.y());


	if (newDir.Len() == 0)
	{
		*mag = -1;
		return newDir;
	}
	newDir.Normalize();

	newDir.set_x(_slp.followGradientFactor*(*mag)*newDir.x() + _slp.followPreviousStrokesFactor*lastFrameDir.x());
	newDir.set_y(_slp.followGradientFactor*(*mag)*newDir.y() + _slp.followPreviousStrokesFactor*lastFrameDir.y());
	newDir.Normalize();

	return newDir;

}



void StrokePainter::truncateDirection(Vec2f& newDir, Vec2f& oldDir, float thresholdAngle)
{

	if (oldDir.Len2()!= 0)
	{
		float signedAngle = atan2(oldDir.Cross2(newDir),oldDir.Dot2(newDir));
		float angleMag = abs(signedAngle);
		float maxAngle = (thresholdAngle*PI)/180.0;
		if (angleMag > maxAngle)
		{	
			float signedMaxAngle = maxAngle;
			if (signedAngle/angleMag < 0)
				signedMaxAngle *= -1;
			
			//rotate the lastDir by maxAngle7
			float c = cos(signedMaxAngle);
			float s = sin(signedMaxAngle);

			//printf("signed maxAngle %f\n", signedMaxAngle);
		
			newDir.set_x(c*oldDir.x() - s*oldDir.y());
			newDir.set_y(s*oldDir.x() + c*oldDir.y());
			//printf("new thresh dir %f %f\n", newDir.x(), newDir.y());
		}
	}
}




Vec2f StrokePainter::getGradientDirection(Vec2f& oldDir, Vec2f& lastFrameDir, float startx, float starty, int relativeFrame,int startFrame, int radius,int dir,float directionSwitch)
{
	double angle =0;
	double lastAngle = 0;
	Vec2f gradDir(0,0);

	Vec2f prevDir(0,0);
	Vec2f currDir = oldDir;

	if (currDir.Len2() ==0)
		currDir = lastFrameDir;

	Vec2f pos(startx,starty);

	for (int r=0;r< radius;r++)
	{
		prevDir = currDir;

		if ((pos.x()<1) || (pos.x()>=_w-1) || (pos.y()<1) || (pos.y()>=_h-1))
			break;
		angle = angleTransform(_etfAngle[relativeFrame+startFrame].val((pos.x()),(pos.y())));

		currDir.set_x(cos(angle));
		currDir.set_y(sin(angle));

		///currDir *=directionSwitch;

		if ((prevDir.Dot2(currDir) < 0))
			currDir *=-1;

		//printf("gradxy %f %f \n", gradx,grady);
		//printf("dir %i switch %f\n", dir, directionSwitch);

		pos += currDir;
	}
		
	//printf("final grad %f %f\n", gradx, grady);
	gradDir.set_x(pos.x()-startx);
	gradDir.set_y(pos.y()-starty);

	if (gradDir.Len2() == 0)
		return oldDir;
	
	gradDir.Normalize();

	return gradDir;

}





Vec2f StrokePainter::getBoundaryDirection(float startx, float starty, int relativeFrame,int startFrame, int radius,int dir,float directionSwitch,LayerObject& layer, float *dist)
{
	double angle =0;
	Vec2f boundDir(0,0);

	float boundx = startx;
	float boundy = starty;

	IntegerImage& boundDist = layer.boundaryDistanceImage(relativeFrame+startFrame);
	IntegerImage& boundDistDx = layer.boundaryDistanceDxImage(relativeFrame+startFrame);
	IntegerImage& boundDistDy = layer.boundaryDistanceDyImage(relativeFrame+startFrame);

	int leftBoundBox = layer.boundingBox(relativeFrame+startFrame).left();
	int rightBoundBox = layer.boundingBox(relativeFrame+startFrame).right();
	int topBoundBox = layer.boundingBox(relativeFrame+startFrame).top();
	int bottomBoundBox = layer.boundingBox(relativeFrame+startFrame).bottom();
	if ((boundx>=leftBoundBox) && (boundx <= rightBoundBox) && (boundy>=topBoundBox) && (boundy <=bottomBoundBox))
		*dist = boundDist.val(boundx-leftBoundBox,boundy-topBoundBox);
	else
	{
		*dist=99999;
		boundDir.set_x(0);
		boundDir.set_y(0);
		return boundDir;
	}

	for (int r=0;r< radius;r++)
	{
		if ((boundx>=leftBoundBox) && (boundx <= rightBoundBox) && (boundy>=topBoundBox) && (boundy <=bottomBoundBox))
		{
			//double dx = /100.;
			//double dy = /100.;
			angle = atan2(boundDistDx.val(boundx-leftBoundBox,boundy-topBoundBox),-1*boundDistDy.val(boundx-leftBoundBox,boundy-topBoundBox));
		}
			

		boundx += cos(angle)*directionSwitch;
		boundy += sin(angle)*directionSwitch;
	}

	boundDir.set_x(boundx-startx);
	boundDir.set_y(boundy-starty);

	boundDir.Normalize();


	return boundDir;

}

/*
void StrokePainter::applyBoundaryBlur(QImage& img,const BinaryImage& bimg, const QRect& rect,double layerBoundaryBlur)
{


	if ((layerBoundaryBlur <= 0))
		return;

	int edgeBlend = layerBoundaryBlur;
	//layerBoundaryBlur = 5;

	int filterWidth = (int)layerBoundaryBlur*4;



	FImage fimgalpha(img.width(), img.height(),1);
	for(int x=0 ;x < img.width();x++)
	for(int y=0 ;y < img.height();y++)
	{
		//
		if (qAlpha(img.pixel(x,y)) > 125)
			fimgalpha.setPixelValue(((float)qAlpha(img.pixel(x,y)))/255.,x,y);
		else
			fimgalpha.setPixelValue(0,x,y);
	}

	//int borderSize = layerObjectList[index].getStrokeBorderSize();

	FImage vis,region;
	bimg.export2Image(vis);


	vis.crop(region, rect.left(), rect.top(), rect.width(), rect.height());

	fimgalpha.GaussianSmoothing(fimgalpha, edgeBlend,edgeBlend*4);

	//fimgalpha.imwrite(QString("images/fimgalpha2.bmp"));
	

	region.GaussianSmoothing(region, layerBoundaryBlur,filterWidth);

	float scaleX = (float)rect.width()/ (float)img.width();
	float scaleY = (float)rect.height()/ (float)img.height();


	printf("img %i %i, rect %i %i\n", img.width(), img.height(), rect.width(), rect.height());
	printf("boundary blur %f\n", layerBoundaryBlur);



	for(int x=0 ;x < img.width();x++)
	for(int y=0 ;y < img.height();y++)
	{

		float blurAlpha = region.val(((float)x) * scaleX ,((float)y) * scaleY );


		QRgb cl = img.pixel(x,y);
		//printf("%f, %i, %i, %f\n", fimgalpha.val(x,y),qAlpha(cl),(int)(fimgalpha.val(x,y)*255.*blurAlpha), ((float)qAlpha(cl))*blurAlpha);
		if (fimgalpha.val(x,y) < 0.2)
		{	
			blurAlpha = 0;
		}
		QRgb  newrgb = qRgba(qRed(cl),qGreen(cl),qBlue(cl),(int)((fimgalpha.val(x,y)*((float)qAlpha(cl))*blurAlpha)));
			//QRgb  newrgb = qRgba(qRed(cl),qGreen(cl),qBlue(cl),((float)qAlpha(cl))*blurAlpha);
		img.setPixel(x,y,newrgb);
		//}
		//else
		//{
			//QRgb cl = img.pixel(x,y);
			//QRgb  newrgb = qRgba(qRed(cl),qGreen(cl),qBlue(cl),fimgalpha.val(x,y)*255.*blurAlpha);
			//QRgb  newrgb = qRgba(qRed(cl),qGreen(cl),qBlue(cl),((float)qAlpha(cl))*blurAlpha);
			//img.setPixel(x,y,newrgb);
		//}

	}

	printf("finished updating img\n");
}
*/


void StrokePainter::applyMedianFilter(QImage& img,double layerMedianBlur)
{

	if ((layerMedianBlur <= 0))
		return;

	FImage fimgalpha(img);

	for(int x=0 ;x < img.width();x++)
	for(int y=0 ;y < img.height();y++)
	{
		QRgb cl = img.pixel(x,y);

		if (qAlpha(cl) < 2)
			continue;


		QList<int> pixR,pixG,pixB;
		for(int i=x-layerMedianBlur/2 ;i < x+layerMedianBlur/2;i++)
		for(int j=y-layerMedianBlur/2 ;j < y+layerMedianBlur/2;j++)
		{
			if ((i >=0) && (j >=0)  && (i < img.width()) && (j <img.height()) && qAlpha(cl) > 10)
			{
				pixR.push_back(fimgalpha.val(i,j,2)*255);
				pixG.push_back(fimgalpha.val(i,j,1)*255);
				pixB.push_back(fimgalpha.val(i,j,0)*255);
			}
		}
		if ((pixR.size() == 0) || (pixG.size() == 0) || (pixB.size() == 0))
			continue;

		qSort(pixR);
		qSort(pixG);
		qSort(pixB);

		QRgb  newrgb = qRgba(pixR[pixR.size()/2],pixG[pixG.size()/2],pixB[pixB.size()/2],qAlpha(cl));
		img.setPixel(x,y,newrgb);

	}

	printf("finished median filtering\n");
}

/*
void StrokePainter::applyMedianFilter(QImage& img,double layerMedianBlur)
{

	if ((layerMedianBlur <= 0))
		return;

	FImage fimgalpha(img);

	for(int x=layerMedianBlur/2+1 ;x < img.width()-layerMedianBlur/2;x++)
	{

		QList<int> pixR,pixG,pixB;
		for(int i=x-layerMedianBlur/2;i < x+layerMedianBlur/2;i++)
		for(int j=0 ;j < layerMedianBlur/2;j++)
		{
			//if ((j >=0)  && (i < img.width()) && (j <img.height()) && qAlpha(cl) > 10)
			if (qAlpha(img.pixel(i,j)) > 10)
			{
	
				pixR.push_back(fimgalpha.val(i,j,2)*255);
				pixG.push_back(fimgalpha.val(i,j,1)*255);
				pixB.push_back(fimgalpha.val(i,j,0)*255);
			}
		}

		for(int y=layerMedianBlur/2+1 ;y < img.height()-layerMedianBlur/2;y++)
		{
			//if (qAlpha(cl) < 2)
			//	continue;


			int j=x-layerMedianBlur/2-1;
			for(int i=x-layerMedianBlur/2;i < x+layerMedianBlur/2;i++)
			{
				if (qAlpha(img.pixel(i,j)) > 10)
				{
					pixR.pop_front();
					pixG.pop_front();
					pixB.pop_front();
				}
			}
			j=x+layerMedianBlur/2-1;
			for(int i=x-layerMedianBlur/2;i < x+layerMedianBlur/2;i++)
			{
				if (qAlpha(img.pixel(i,j)) > 10)
				{
					pixR.push_back(fimgalpha.val(i,j,2)*255);
					pixG.push_back(fimgalpha.val(i,j,1)*255);
					pixB.push_back(fimgalpha.val(i,j,0)*255);
				}
			}


			if ((pixR.size() == 0) || (pixG.size() == 0) || (pixB.size() == 0))
				continue;

			QList<int>  pixRSort(pixR);
			QList<int>  pixGSort(pixG);
			QList<int>  pixBSort(pixB);

			qSort(pixRSort);
			qSort(pixGSort);
			qSort(pixBSort);

			QRgb  newrgb = qRgba(pixRSort[pixRSort.size()/2],pixGSort[pixGSort.size()/2],pixBSort[pixBSort.size()/2],qAlpha(img.pixel(x,y)));
			img.setPixel(x,y,newrgb);

		}
	}

	printf("finished median filtering\n");
}

*/

void StrokePainter::strokesChanged(int curFrameIndex)
{
	if (curFrameIndex < _frames)
	{
		//printf("strokes changed for frame %i\n", curFrameIndex);
		_redraw = true;

	}
	
};

void StrokePainter::redraw()
{
	_redraw = true;
};



void StrokePainter::setCurrentColor(QColor& color, float pointx, float pointy)
{
    if ((_fbo!= 0) && (_fbo->bind()==false))
    {
        printf("problem binding framebuffer object\n");
        return;
    }
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	//glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _mainTex, 0);
	//glColor4f(1,1,1,1);
	//_fbo->drawTexture(QRect(QPoint(0,_h-1),QPoint(_w,0)), _mainTex);
        printf("main tex %i\n", _mainTex);

	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _mainTex, 0);
	glColor4f(1,1,1,1);
	_fbo->drawTexture(QRect(QPoint(0,_h-1),QPoint(_w,0)), _secondaryTex);

	unsigned char *imgcol = new unsigned char[4];
	glReadPixels(pointx*_fboscaleFactor,_h*_fboscaleFactor-pointy*_fboscaleFactor,1,1, GL_BGRA, GL_UNSIGNED_BYTE, (void*) imgcol);

	color.setRed(truncateColor(imgcol[2]));
	color.setGreen(truncateColor(imgcol[1]));
	color.setBlue(truncateColor(imgcol[0]));

	printf("selected color %i %i %i from point %f %f\n",imgcol[0], imgcol[1], imgcol[2],pointx, pointy);

	delete imgcol;

};




void StrokePainter::setCurrentColorForFilledStroke(Stroke *stroke,QImage& img)
{
	_fbo->bind();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	stroke->render();

	QVector<Point>& pts = stroke->getControlPoints();

	int left=10000, right=0, top=0,bottom=10000;

	for(int i=0;i < pts.size();i++)
	{
		if (pts[i].x() < left)
			left = pts[i].x();

		if (pts[i].x() > right)
			right = pts[i].x();

		if (pts[i].y() <bottom )
			bottom = pts[i].y();

		if (pts[i].y() > top)
			top = pts[i].y();
	}


	QRect bb(QPoint(left,bottom),QSize(right-left,top-bottom));

	
	float *current = new float[bb.width()*bb.height()];

	if (current==0)
		return;

	
	glReadPixels(left,bottom,bb.width(),bb.height(), GL_LUMINANCE, GL_FLOAT, (void*) current);


	if ((_errCode = glGetError()) != GL_NO_ERROR) {
                sprintf(_strbuf, "OpenGL Error: %s" ,gluErrorString(_errCode) );
		LOG_STR_COUT(_strbuf);
	}

	if (current==0)
	{
		return;
	}

	float rSum=0, gSum=0, bSum=0;
	float count=0;

	for(int i=0;i < bb.width();i++)
	for(int j=0;j < bb.height();j++)
	{

		if (current[j*((int)bb.width()) + i] != 0)
		{
			if ((i+bb.x() >=0) && (i+bb.x() <_w) && (_h-(j+bb.y()) >=0) && (_h-(j+bb.y()) <_h))
			{
				//printf("ij %i %i %i\n", i, j, (j+((int)bb.y())));
				QRgb pix = img.pixel(i+bb.x(),_h-(j+bb.y()));
				rSum+=qRed(pix);gSum+=qGreen(pix);bSum+=qBlue(pix);
				count++;
			}
		}
	}


	if (count < 5)
	{

		stroke->properties.alpha=0;
		//printf("ERROR: couldn't find the region	to get color from\n");
		return;
	}	
	
	//printf("area %i, count %f\n", area, count);
	delete current;


	QColor newcol(rSum/count, gSum/count,bSum/count);

	int h,s,v;
	newcol.getHsv(&h,&s,&v);

    int newH = max(0.0,h +  normal(_slp.hueOffset,_slp.hueNoise+0.001));
    int newS = max(0.0,s +  normal(_slp.saturationOffset,_slp.saturationNoise+0.001));
    int newV = max(0.0,v +  normal(_slp.valueOffset,_slp.valueNoise+0.001));
	if (newH >= 357)
		newH = 357;
	if (newS >= 254)
		newS = 254;
	if (newV >= 254)
		newV = 254;
	//printf("post %i %i %i\n",newH,newS,newV);
	stroke->properties.color.setHsv(newH,newS,newV);

}

/*
void StrokePainter::getImageColor(int frame, int startx, int starty,double* gaussfilter, int fsize, const BinaryImage& layerMask, float* red,float* green,float* blue )
{

	float r=0, g=0, b=0;
	float count = 0;

	int width = (fsize*2+1);

	//printf("sigmaBlur %f blursize %i\n",sigmaBlur, gBlurSize);

	float sum = 0, gaussian=0;

	bool check 	=((startx - fsize < 0) || (starty - fsize < 0) || (startx + fsize >= inp.imgs[frame].width()) || (starty + fsize >= inp.imgs[frame].height()));

	for(int x = startx - fsize;x <= startx + fsize;x++ )
	for(int y = starty - fsize;y <= starty + fsize;y++ )
	{
		if (check)
		{
			if ((x < 0) || (y < 0) || (x >= inp.imgs[frame].width()) || (y >= inp.imgs[frame].height()))
				continue;
		}

		if (layerMask.getPixel(x,y)==0)    
			continue;

		gaussian = gaussfilter[(x-(startx - fsize))*width + (y-(starty - fsize))];

		//printf("xy %i %i, gaussian %f\n", x,y,gaussian);
		r += gaussian*((float)inp.imgs[frame].val(x,y,2));
		g += gaussian*((float)inp.imgs[frame].val(x,y,1));
		b += gaussian*((float)inp.imgs[frame].val(x,y,0));

		sum +=gaussian;

		count++;
	}

	//printf("sum %f\n", sum);

	*red = r/sum;
	*green = g/sum;
	*blue = b/sum;
}
*/


bool StrokePainter::getImageColor(BiImage& img , int startx, int starty, double* gaussfilter, int fsize, const BinaryImage& layerMask, float* red,float* green,float* blue, float *grey )
{

	*red = 0;
	*green = 0;
	*blue = 0;
	*grey = 0;

	if (img.width() ==0)
	{
		return false;
	}

	float r=0, g=0, b=0,gr=0;
	float count = 0;

	//int fsize = _gaussianMaskSize;
	int width = (fsize*2+1);

	//printf("sigmaBlur %f blursize %i\n",sigmaBlur, gBlurSize);

	float sum = 0, gaussian=0;

	bool check 	=((startx - fsize < 0) || (starty - fsize < 0) || (startx + fsize >= img.width()) || (starty + fsize >= img.height()));

	for(int x = startx - fsize;x <= startx + fsize;x++ )
	for(int y = starty - fsize;y <= starty + fsize;y++ )
	{
		if (check)
		{
			if ((x < 0) || (y < 0) || (x >= img.width()) || (y >= img.height()))
				continue;
		}

		bool visible=layerMask.getPixel(x,y);
		if (!visible)    
			continue;

		gaussian = gaussfilter[(x-(startx - fsize))*width + (y-(starty - fsize))];

		if (img.nchannels()==3)
		{
			//printf("xy %i %i, gaussian %f\n", x,y,gaussian);
			r += gaussian*((float)img.val(x,y,2));
			g += gaussian*((float)img.val(x,y,1));
			b += gaussian*((float)img.val(x,y,0));
		}
		else
		{
			gr += gaussian*((float)img.val(x,y));
		}
		
		sum +=gaussian;

		count++;
	}

	if (sum<0.5)
	{
		int test=1;
		return false;
	}
	

	*red = r/sum;
	*green = g/sum;
	*blue = b/sum;
	*grey = gr/sum;

	return true;
}



float StrokePainter::getColorDifference(int currFrameRel, int startx, int starty, double* gaussfilter, int fsize,QColor& color)
{

	//printf ("test\n");
	float g=0;

	//int fsize = _gaussianMaskSize;
	int width = (fsize*2+1);

	float diffSum=0;
	float filterSum = 0;

	bool check 	=((startx - fsize < 0) || (starty - fsize < 0) || (startx + fsize >= inp.imgs[currFrameRel].width()) || (starty + fsize >= inp.imgs[currFrameRel].height()));

	bool userStrokes=false;
	if (inp.colorStrokeAlphaImgs[currFrameRel].width()!=0)
	{
		userStrokes=true;
	}
	float alpha, red, green, blue;
	for(int x = startx - fsize;x <= startx + fsize;x++ )
	for(int y = starty - fsize;y <= starty + fsize;y++ )
	{
		if (check)
		{
			if ((x < 0) || (y < 0) || (x >= inp.imgs[currFrameRel].width()) || (y >= inp.imgs[currFrameRel].height()))
				continue;
		}
		if (userStrokes)
		{
			alpha =inp.colorStrokeAlphaImgs[currFrameRel].val(x,y);

			red = ((float)inp.colorStrokeImgs[currFrameRel].val(x,y,2))*(alpha/255.) + ((float)inp.imgs[currFrameRel].val(x,y,2))*(1. - alpha/255.);
			green = ((float)inp.colorStrokeImgs[currFrameRel].val(x,y,1))*(alpha/255.) + ((float)inp.imgs[currFrameRel].val(x,y,1))*(1. - alpha/255.);
			blue = ((float)inp.colorStrokeImgs[currFrameRel].val(x,y,0))*(alpha/255.) + ((float)inp.imgs[currFrameRel].val(x,y,0))*(1. - alpha/255.);

		}
		else
		{
			red = inp.imgs[currFrameRel].val(x,y,2);
			green = inp.imgs[currFrameRel].val(x,y,1);
			blue = inp.imgs[currFrameRel].val(x,y,0);
		}

		g = gaussfilter[(x-(startx - fsize))*width + (y-(starty - fsize))];

		diffSum += g*sqrt((red-color.red())*(red-color.red())+(green-color.green())*(green-color.green())+(blue-color.blue())*(blue-color.blue()));
		filterSum +=g;
	}
	if (filterSum==0)
	{
		int test=1;
		return 0;
	}

	return (diffSum/filterSum)/(sqrt(3.)*255.);
}



/*
void StrokePainter::getEdgeness(BinaryImage& img , int startx, int starty, double* gaussfilter, int fsize, float* edgeness )
{

	*edgeness = 0;

	if (img.width() ==0)
	{
		return;
	}

	float e=0;
	float count = 0;

	//int fsize = _gaussianMaskSize;
	int width = (fsize*2+1);

	//printf("sigmaBlur %f blursize %i\n",sigmaBlur, gBlurSize);

	float sum = 0, gaussian=0;

	bool check 	=((startx - fsize < 0) || (starty - fsize < 0) || (startx + fsize >= img.width()) || (starty + fsize >= img.height()));

	for(int x = startx - fsize;x <= startx + fsize;x++ )
	for(int y = starty - fsize;y <= starty + fsize;y++ )
	{
		if (check)
		{
			if ((x < 0) || (y < 0) || (x >= img.width()) || (y >= img.height()))
				continue;
		}

		gaussian = gaussfilter[(x-(startx - fsize))*width + (y-(starty - fsize))];
		e += gaussian*((float)img.getPixel(x,y));
		//e += ((float)img.getPixel(x,y));

		//sum +=gaussian;

		count++;
	}
	
	
	*edgeness = e;
}
*/
void StrokePainter::getEdgeness(BiImage& img , int startx, int starty, double* gaussfilter, int fsize, float* edgeness )
{

	*edgeness = 0;

	if (img.width() ==0)
	{
		return;
	}

	float e=0;
	float count = 0;

	//int fsize = _gaussianMaskSize;
	int width = (fsize*2+1);

	//printf("sigmaBlur %f blursize %i\n",sigmaBlur, gBlurSize);

	float sum = 0, gaussian=0;

	bool check 	=((startx - fsize < 0) || (starty - fsize < 0) || (startx + fsize >= img.width()) || (starty + fsize >= img.height()));

	for(int x = startx - fsize;x <= startx + fsize;x++ )
	for(int y = starty - fsize;y <= starty + fsize;y++ )
	{
		if (check)
		{
			if ((x < 0) || (y < 0) || (x >= img.width()) || (y >= img.height()))
				continue;
		}

		gaussian = gaussfilter[(x-(startx - fsize))*width + (y-(starty - fsize))];
		e += gaussian*(((float)img.val(x,y))/255.);
		//e += ((float)img.getPixel(x,y));

		//sum +=gaussian;

		count++;
	}
	
	
	*edgeness = e;
}


/*
float StrokePainter::getWinnemollerDoGMagnitude(BiImage& img , int x, int y)
{

	float r1,r2,g1,g2,b1,b2,placeholder;
	
	getImageColor(img, x,y,_dogMask1,_dogMask1Size,&r1, &g1, &b1,&placeholder); 
	getImageColor(img, x,y,_dogMask2,_dogMask2Size,&r2, &g2, &b2,&placeholder); 

	//float dog2Weight=0.98
	float grey1 = (r1*0.3 + g1*0.59 + b1*0.11);
	float grey2 = (r2*0.3 + g2*0.59 + b2*0.11);

	float dog = grey1 - 0.98*grey2;

	if (dog >0)
		return 1;
	else
		return 1+ tanh(dog);

//	return dog;

}
*/

double *StrokePainter::getGaussianFilter(float sigma,int *filterSize)
{


	int fsize = 2*sigma;

	*filterSize = fsize;

	if (*filterSize==1)
	{
		int test=1;
	}

   	double *oneDMask=new double[fsize*2+1];

	double sum=0;

	for(int i=-fsize;i<=fsize;i++)
	{
		oneDMask[i+fsize]=(1/(sqrt(2*PI)*(sigma)))*exp(-1.0*((double)(i*i))/(2*sigma*sigma));
		sum+=oneDMask[i+fsize];
	}

	int width = (fsize*2+1);

	double *gaussianMask=new double[width*width];
	sum = 0;

	for(int i=-fsize;i<=fsize;i++)
	for(int j=-fsize;j<=fsize;j++)
	{
		gaussianMask[(i+fsize)*width + j+fsize] = oneDMask[i+fsize]*oneDMask[j+fsize];
		sum +=gaussianMask[(i+fsize)*width + j+fsize];
	}

	for(int i=-fsize;i<=fsize;i++)
	for(int j=-fsize;j<=fsize;j++)
	{
		gaussianMask[(i+fsize)*width + j+fsize] /= sum;
	}

	delete oneDMask;

	return gaussianMask;

}



bool StrokePainter::shiftStrokeToOrientationField(int initx,int inity,int currFrameRel, float strokeRadius,LayerObject& layer, int *shiftedx,int *shiftedy)
{

	int x=initx;
	int y=inity;
	float magnitude=1, nextmag=1;

	float curDistance = 0;

	int count = 0;

	bool reachedMax=false;
	
	//climb the gradient field check if we're too close to an edge
	while (true)
	{

		//check distance from current point to start point
		//if we've passed the stroke radius, no need to shift

		curDistance=sqrt((double)((x-initx)*(x-initx) + (y-inity)*(y-inity)));
		if (curDistance > strokeRadius)
			return false;

	
		if ((x<2) || (y<2) || (x>=_w-2) || (y>=_h-2))
		{
			return false;
		}

		if (count > 50)
		{
			int prob =1;
			//break;
		}
		else
			count++;

        Vec2f zero(0,0);
        Vec2f newDir = getDirection(zero,zero,zero,x,y,currFrameRel,1,0,1,layer,&magnitude);

		if ((count!=1)&&(nextmag != magnitude))
		{
			int prob=1;
		}
		//error getting dir...
		if (magnitude<0)
			return false;

		//check left rotate
		int nextx=((float)x)-newDir.y()+0.5;
		int nexty=((float)y)+newDir.x()+0.5;

        Vec2f nextDir = getDirection(zero,zero, zero,nextx,nexty,currFrameRel,1,0,1,layer,&nextmag);
		//error getting dir...
		if (nextmag<0)
			return false;

		if (nextmag < magnitude)
		{
			x=nextx;
			y=nexty;
			continue;
		}

		//check right rotate
		nextx=((float)x)+newDir.y()+0.5;
		nexty=((float)y)-newDir.x()+0.5;

        nextDir = getDirection(zero,zero, zero,nextx,nexty,currFrameRel,1,0,1,layer,&nextmag);
		
		//error getting dir...
		if (nextmag<0)
			return false;

		if (nextmag < magnitude)
		{
			x=nextx;
			y=nexty;
			continue;
		}
		//we must be an maximum point
		else
		{
			
			reachedMax=true;
			break;
		}
	}

	if (reachedMax==false)
		return false;
	
	if (nextmag> 50)
	{
		return false;
	}


	int maxx=x, maxy=y;


	x=initx;
	y=inity;
	count = 0;


	//otherwise, slide down the gradient field
	while (true)
	{
		//check distance from current point to start point
		//if we've passed the stroke radius, no need to shift

		curDistance=sqrt((double)((x-maxx)*(x-maxx) + (y-maxy)*(y-maxy)));
		if (curDistance > strokeRadius)
		{
			*shiftedx=x;
			*shiftedy=y;
			return true;
		}

		if (count > 50)
		{
			int prob =1;
			//break;
		}
		else
			count++;

		if ((x<2) || (y<2) || (x>=_w-2) || (y>=_h-2))
		{
			return false;
		}

        Vec2f zero(0,0);
        Vec2f newDir = getDirection(zero,zero, zero,x,y,currFrameRel,1,0,1,layer,&magnitude);

		if ((count!=1)&&(nextmag != magnitude))
		{
			int prob=1;
		}

		//error getting dir...
		if (magnitude<0)
			return false;

		//check left rotate
		int nextx=((float)x)-newDir.y()+0.5;
		int nexty=((float)y)+newDir.x()+0.5;

        Vec2f nextDir = getDirection(zero,zero, zero,nextx,nexty,currFrameRel,1,0,1,layer,&nextmag);
		//error getting dir...
		if (nextmag<0)
			return false;

		if (nextmag > magnitude)
		{
			x=nextx;
			y=nexty;
			continue;
		}

		//check right rotate
		nextx=((float)x)+newDir.y()+0.5;
		nexty=((float)y)-newDir.x()+0.5;

		nextDir = getDirection(zero,zero,  zero,nextx,nexty,currFrameRel,1,0,1,layer,&nextmag);
		
		//error getting dir...
		if (nextmag<0)
			return false;

		if (nextmag > magnitude)
		{
			x=nextx;
			y=nexty;
			continue;
		}
		//we must be an maximum point
		else
		{
			*shiftedx=x;
			*shiftedy=y;
			return true;
		}
	}


	return false;

}


float StrokePainter::getColorDist(QColor& oldCol, QColor& currCol)
{
	float colorDist = 0;

	/*
	int h1 = currCol.hue();
	int h2 = oldCol.hue();
	
	float hdisp = abs(currCol.hue()-oldCol.hue());
	if (hdisp > 180)
		hdisp = 360. -hdisp;

	float sdisp = (currCol.saturation()-oldCol.saturation());
	float vdisp = (currCol.value()-oldCol.value());
	colorDist = sqrt((hdisp*hdisp) + (sdisp*sdisp)+(vdisp*vdisp))/sqrt(255.*255.*2 + 180.*180.);
	*/

	float rdisp = (currCol.red()-oldCol.red());
	float gdisp = (currCol.green()-oldCol.green());
	float bdisp = (currCol.blue()-oldCol.blue());
	colorDist =  sqrt((rdisp*rdisp) + (gdisp*gdisp)+(bdisp*bdisp))/(sqrt(3.)*255.);

	return colorDist;

}



void StrokePainter::optimizeStrokeWidths(LayerObject& layer,QVector<Stroke*>& createdStrokes, Vec3i& strokeNoise)
{


	bool firstStroke=true;

	float lastStartWidth=_slp.strokeProperties.startRadius;
	float lastEndWidth=_slp.strokeProperties.endRadius;
	

	
	
	QColor newColor;
	QColor imgColor;

	for(int i=0;i <= inp.endFrame-inp.startFrame;i++)
	{
		if (createdStrokes[i] != 0)
		{
			QVector<Vec2f> controlPoints;
			
			QVector<Point>& strokePoints=createdStrokes[i]->getControlPoints();

			for(int j=0;j < strokePoints.size();j++)
			{
				controlPoints.push_back(Vec2f(strokePoints[j].x(), _h-strokePoints[j].y()));
			}
	


			double initVal=evaluateStrokeWidths(layer,i,lastStartWidth,lastEndWidth,lastStartWidth,lastEndWidth,controlPoints,createdStrokes[i]->properties.initColor, firstStroke);
				
		
			float minVal=100000;
			float minStart;
			float minEnd;
			for(float updateStart = -2;updateStart <=2;updateStart += 0.5)
			for(float updateEnd = -2;updateEnd <=2;updateEnd += 0.5)
			{

				if ((lastStartWidth+updateStart<1)|| (lastEndWidth+updateEnd<1))
					continue;

				double currVal=evaluateStrokeWidths(layer,i,lastStartWidth+updateStart,lastEndWidth+updateEnd,lastStartWidth,lastEndWidth,controlPoints,createdStrokes[i]->properties.initColor,firstStroke,minVal);
				
				if (currVal < minVal)
				{
					minStart=lastStartWidth+updateStart;
					minEnd=lastEndWidth+updateEnd;
					minVal=currVal;
				}
			}


			createdStrokes[i]->properties.startRadius=minStart;
			createdStrokes[i]->properties.endRadius=minEnd;
		
			lastStartWidth=minStart;
			lastEndWidth=minEnd;

			getNewStrokeColor(layer, i, newColor,imgColor, createdStrokes[i],strokeNoise);
			createdStrokes[i]->properties.color = newColor;


			firstStroke=false;

		}

	}
}



Vec2f StrokePainter::optimizeStrokeWidth(int currFrameRel,LayerObject& layer,bool first, QVector<Vec2f>& strokePoints, float previousStartWidth, float previousEndWidth,QColor& initColor, bool firstStroke)
{



	float lastStartWidth=previousStartWidth;
	float lastEndWidth=previousEndWidth;
	QVector<Vec2f> newStrokePts;
	int firstPt=-1;
	for (int i=0; i < strokePoints.size();i++)
	{
		if ((firstPt==-1)&&(strokePoints[i].x()!=-1))
			firstPt=i;

		if ((firstPt!=-1)&&(strokePoints[i].x()==-1))
		{
			break;
		}
		if (firstPt!=-1)
			newStrokePts.push_back(strokePoints[i]);

	}
	if (firstPt==-1)
	{
		sprintf(_strbuf, "Problem optimizing widths");
		LOG_STR_COUT(_strbuf);		
		return Vec2f(_slp.strokeProperties.startRadius,_slp.strokeProperties.endRadius);
	}


	double initVal=evaluateStrokeWidths(layer,currFrameRel,lastStartWidth,lastEndWidth,lastStartWidth,lastEndWidth,newStrokePts,initColor,firstStroke);
		
	float widthCheck=2;
	if (first)
		widthCheck=max(2,max(_slp.strokeProperties.startRadius,_slp.strokeProperties.endRadius));

	float minVal=9999999;
	float minStart;
	float minEnd;
	for(float updateStart = -widthCheck;updateStart <=widthCheck;updateStart += 0.5)
	for(float updateEnd = -widthCheck;updateEnd <=widthCheck;updateEnd += 0.5)
	{

		if ((lastStartWidth+updateStart<1)|| (lastEndWidth+updateEnd<1))
			continue;

		double currVal=evaluateStrokeWidths(layer,currFrameRel,lastStartWidth+updateStart,lastEndWidth+updateEnd,lastStartWidth,lastEndWidth,newStrokePts,initColor,firstStroke,minVal);
		
		if (currVal < minVal)
		{
			minStart=lastStartWidth+updateStart;
			minEnd=lastEndWidth+updateEnd;
			minVal=currVal;
		}
	}

	return Vec2f(minStart,minEnd);

}




double StrokePainter::evaluateStrokeWidths(LayerObject& layer,int currFrameRel,float currStartWidth,float currEndWidth,float lastStartWidth,float lastEndWidth,QVector<Vec2f>& strokePoints, QColor& initColor, bool firstFrame,float curMin)
{

	if ((currStartWidth<1)||(currEndWidth<1))
	{
		return 999999;
	}


	float regularizationTerm=0;

	float startWidthDiff=abs(_slp.strokeProperties.startRadius-currStartWidth)/_slp.strokeProperties.startRadius;
	float endWidthDiff=abs(_slp.strokeProperties.endRadius-currEndWidth)/_slp.strokeProperties.endRadius;

	if (_slp.oswRegularizationFactor!=0)
	{

		Vec2f user(_slp.strokeProperties.startRadius,_slp.strokeProperties.endRadius);
		Vec2f diff(_slp.strokeProperties.startRadius-currStartWidth,_slp.strokeProperties.endRadius-currEndWidth);

		regularizationTerm=diff.Len()/user.Len();
		if (regularizationTerm>_slp.relativeWidthShift*2)
			regularizationTerm=99999;


	}

	float curTotal = _slp.oswRegularizationFactor*regularizationTerm;
	if (curTotal> curMin)
		return curTotal;
	double coherenceTerm=0;


	if (_slp.oswCoherenceFactor!=0)
	{
	
		Vec2f last(lastStartWidth,lastEndWidth);
		Vec2f diff(lastStartWidth-currStartWidth,lastEndWidth-currEndWidth);

		coherenceTerm+=diff.Len2()/last.Len2();
	}
	curTotal+=_slp.oswCoherenceFactor*coherenceTerm;
	
	if ( curTotal> curMin)
		return curTotal;

	float textureWidth=_textureWidthMap.value((int)_slp.strokeProperties.texture);

	double* gaussianMask;
	int maskSize;
	float r,g,b,e;

	double colorTerm=0;

	std::pair<double *,int> p;

	QColor col;

	double edgeTerm=0;

	for (int i=0; i < strokePoints.size();i++)
	{
		double weight= (float)(i)/((float)strokePoints.size()-1);
		double strokeRadius = (1.0-weight)*currStartWidth +	weight*currEndWidth;
		double avgRadius = strokeRadius*textureWidth;

		int avgRad=avgRadius*100;

		if (_gaussianMasks.contains(avgRad))
		{
			p= _gaussianMasks.value(avgRad);
			gaussianMask=p.first;
			maskSize=p.second;
		}
		else
		{
			gaussianMask= getGaussianFilter(min((double)avgRadius,15.0)*_slp.radiusFactor,&maskSize);

			p.first=gaussianMask;
			p.second=maskSize;
			_gaussianMasks.insert(avgRad,p);
		}

		if ((_slp.oswEdgeFactor!=0)&& (inp.edges.size()>currFrameRel))
		{
			getEdgeness(inp.edges[currFrameRel], strokePoints[i].x(), strokePoints[i].y(),gaussianMask,maskSize,&e);

			edgeTerm += e*avgRadius;
		}

		if (_slp.oswColorFactor!=0)
		{
			float colDist;
			
			
			if (getCombinedImageAndUserStrokeColor(currFrameRel, strokePoints[i].x(), strokePoints[i].y(),gaussianMask,maskSize,layer,&r, &g, &b))
			{
				QColor col(r,g,b);
				colDist = getColorDist(initColor,col);
			}
			else
				colDist=0;
		
			colorTerm+= colDist*avgRadius;
		}
	}

	colorTerm /= (float) strokePoints.size();
	edgeTerm /= (float) strokePoints.size();

	if (edgeTerm!=0)
	{
		int test=1;
	}

	float growTerm=0.5*(1.0/currStartWidth + 1.0/currEndWidth);
	

	curTotal+=_slp.oswEdgeFactor*edgeTerm+ _slp.oswGrowFactor*growTerm+_slp.oswColorFactor*colorTerm ;

	return curTotal;
}


